@@ -76,7 +76,6 @@ char* strsav_string(CSOUND* csound, ENGINE_STATE* engineState, char* key) {
     if (retVal == NULL) {
         retVal = cs_hash_table_put_key(csound, engineState->stringPool, key);
     }
-
     return retVal;
 }
 
@@ -200,7 +199,6 @@ int argsRequired(char* argString)
         }
       }
     }
-
     return retVal;
 }
 
@@ -639,20 +637,20 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
         synterr(p, Str("%s invalid ksmps value"), err_msg);
       else if (UNLIKELY(FLOAT_COMPARE(sr,(double)kr *ksmps)))
         synterr(p, Str("%s inconsistent sr, kr, ksmps"), err_msg);
-      else if(ksmps > sr)
+      else if (ksmps > sr)
         synterr(p, Str("%s inconsistent sr, kr, ksmps \n"), err_msg);
     }
 
     csound->ksmps = ksmps;
+
     csound->nchnls = nchnls;
-    if(inchnls==0) csound->inchnls = nchnls;
+    if (inchnls==0) csound->inchnls = nchnls;
     else csound->inchnls = inchnls;
     csound->esr = sr;
     csound->ekr = kr;
-    if(_0dbfs < 0) csound->e0dbfs = DFLT_DBFS;
+    if (_0dbfs < 0) csound->e0dbfs = DFLT_DBFS;
     else csound->e0dbfs = _0dbfs;
 
-
     OPARMS  *O = csound->oparms;
     if (UNLIKELY(csound->e0dbfs <= FL(0.0))){
       csound->Warning(csound,
@@ -663,14 +661,15 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
 
     if (O->nchnls_override > 0)
       csound->nchnls = csound->inchnls = O->nchnls_override;
-    if(O->nchnls_i_override > 0) csound->inchnls = O->nchnls_i_override;
-    if(O->e0dbfs_override > 0) csound->e0dbfs = O->e0dbfs_override;
+    if (O->nchnls_i_override > 0) csound->inchnls = O->nchnls_i_override;
+    if (O->e0dbfs_override > 0) csound->e0dbfs = O->e0dbfs_override;
 
     if (UNLIKELY(O->odebug))
       csound->Message(csound, "esr = %7.1f, ekr = %7.1f, ksmps = %d, nchnls = %d "
                       "0dbfs = %.1f\n",
                       csound->esr, csound->ekr, csound->ksmps,
                       csound->nchnls, csound->e0dbfs);
+
     if (O->sr_override) {        /* if command-line overrides, apply now */
       MYFLT ensmps;
       csound->esr = (MYFLT) O->sr_override;
@@ -707,10 +706,11 @@ INSTRTXT *create_instrument0(CSOUND *csound, TREE *root,
     csound->onedkr = FL(1.0) / csound->ekr;
     csound->global_kcounter  = csound->kcounter;
 
-    if(csound->ksmps != DFLT_KSMPS){
+    if (csound->ksmps != DFLT_KSMPS) {
       reallocateVarPoolMemory(csound, engineState->varPool);
     }
     close_instrument(csound, engineState, ip);
+
     return ip;
 }
 
@@ -814,6 +814,16 @@ INSTRTXT *create_instrument(CSOUND *csound, TREE *root,
     ip->t.inlist = (ARGLST *) mmalloc(csound, sizeof(ARGLST));
     ip->t.inlist->count = 1;
 
+    /* create local ksmps variable */
+     CS_TYPE* rType = (CS_TYPE*)&CS_VAR_TYPE_R;
+     CS_VARIABLE *var = csoundCreateVariable(csound, csound->typePool,
+                                            rType, "ksmps", NULL);
+     csoundAddVariable(ip->varPool, var);
+     /* same for kr */
+     var = csoundCreateVariable(csound, csound->typePool,
+                                            rType, "kr", NULL);
+     csoundAddVariable(ip->varPool, var);
+
     /* Maybe should do this assignment at end when instr is setup?
      * Note: look into how "instr 4,5,6,8" is handled, i.e. if copies
      * are made or if they are all set to point to the same INSTRTXT
@@ -914,10 +924,13 @@ void free_instrtxt(CSOUND *csound, INSTRTXT *instrtxt)
           t = s;
         }
     // myflt_pool_free(csound, ip->varPool);
-     deleteVarPoolMemory(csound, ip->varPool);
+    /* VL: 19-12-13
+       an instrument varpool memory is allocated in the instrument block
+       so deallocating the pool is not really right */
+    // deleteVarPoolMemory(csound, ip->varPool);
      //mfree(csound, ip->varPool); /* need to delete the varPool memory */
      mfree(csound, ip);
-     if(csound->oparms->odebug)
+     if (csound->oparms->odebug)
        csound->Message(csound, Str("-- deleted instr from deadpool \n"));
 }
 
@@ -934,10 +947,10 @@ void add_to_deadpool(CSOUND *csound, INSTRTXT *instrtxt)
     /* check current items in deadpool to see if they need deleting */
     int i;
     for(i=0; i < csound->dead_instr_no; i++){
-      if(csound->dead_instr_pool[i] != NULL) {
+      if (csound->dead_instr_pool[i] != NULL) {
         INSDS *active = csound->dead_instr_pool[i]->instance;
         while (active != NULL) {
-          if(active->actflg) {
+          if (active->actflg) {
             // add_to_deadpool(csound,csound->dead_instr_pool[i]);
             break;
           }
@@ -945,7 +958,7 @@ void add_to_deadpool(CSOUND *csound, INSTRTXT *instrtxt)
         }
         /* no active instances */
         if (active == NULL) {
-        if(csound->oparms->odebug)
+        if (csound->oparms->odebug)
           csound->Message(csound, Str(" -- free instr def %p \n"),
                           csound->dead_instr_pool[i]);
         free_instrtxt(csound, csound->dead_instr_pool[i]);
@@ -969,10 +982,9 @@ void add_to_deadpool(CSOUND *csound, INSTRTXT *instrtxt)
       mrealloc(csound, csound->dead_instr_pool,
                ++csound->dead_instr_no * sizeof(INSTRTXT*));
     csound->dead_instr_pool[csound->dead_instr_no-1] = instrtxt;
-   if(csound->oparms->odebug)
+    if (csound->oparms->odebug)
     csound->Message(csound, Str(" -- added to deadpool slot %d \n"),
                     csound->dead_instr_no-1);
-
 }
 
 /**
@@ -994,9 +1006,10 @@ int named_instr_alloc(CSOUND *csound, char *s, INSTRTXT *ip,
       int i;
        inm->ip->isNew = 1;
       /* redefinition does not raise an error now, just a warning */
-      if(csound->oparms->odebug) csound->Warning(csound,
-                      Str("instr %ld redefined, replacing previous definition"),
-                      inm->instno);
+       if (csound->oparms->odebug)
+         csound->Warning(csound,
+                         Str("instr %ld redefined, replacing previous definition"),
+                         inm->instno);
       /* here we should move the old instrument definition into a deadpool
          which will be checked for active instances and freed when there are no
          further ones
@@ -1009,7 +1022,7 @@ int named_instr_alloc(CSOUND *csound, char *s, INSTRTXT *ip,
       }
       INSDS *active = engineState->instrtxtp[inm->instno]->instance;
       while (active != NULL) {
-        if(active->actflg) {
+        if (active->actflg) {
           add_to_deadpool(csound, engineState->instrtxtp[inm->instno]);
           break;
         }
@@ -1140,21 +1153,23 @@ void insert_instrtxt(CSOUND *csound, INSTRTXT *instrtxt,
     if (UNLIKELY(engineState->instrtxtp[instrNum] != NULL)) {
       instrtxt->isNew = 1;
       /* redefinition does not raise an error now, just a warning */
-      if(instrNum && csound->oparms->odebug) csound->Warning(csound,
-                      Str("instr %ld redefined, replacing previous definition"),
-                      instrNum);
+      if (instrNum && csound->oparms->odebug)
+        csound->Warning(csound,
+                        Str("instr %ld redefined, replacing previous definition"),
+                        instrNum);
       /* here we should move the old instrument definition into a deadpool
          which will be checked for active instances and freed when there are no
          further ones
       */
-      for(i=0; i < engineState->maxinsno; i++) {
+      for (i=0; i < engineState->maxinsno; i++) {
         /* check for duplicate numbers and do nothing */
-        if(i != instrNum &&
-           engineState->instrtxtp[i] == engineState->instrtxtp[instrNum]) goto end;
+        if (i != instrNum &&
+            engineState->instrtxtp[i] == engineState->instrtxtp[instrNum])
+          goto end;
       }
       INSDS *active = engineState->instrtxtp[instrNum]->instance;
       while (active != NULL && instrNum != 0) {
-        if(active->actflg) {
+        if (active->actflg) {
           add_to_deadpool(csound, engineState->instrtxtp[instrNum]);
           break;
         }
@@ -1245,27 +1260,30 @@ int engineState_merge(CSOUND *csound, ENGINE_STATE *engineState)
                         current_state->stringPool, engineState->stringPool);
 
     for (count = 0; count < engineState->constantsPool->count; count++) {
-     if(csound->oparms->odebug)
-      csound->Message(csound, Str(" merging constants %d) %f\n"),
-                      count, engineState->constantsPool->values[count]);
+      if (csound->oparms->odebug)
+        csound->Message(csound, Str(" merging constants %d) %f\n"),
+                        count, engineState->constantsPool->values[count]);
       myflt_pool_find_or_add(csound, current_state->constantsPool,
                              engineState->constantsPool->values[count]);
     }
     CS_VARIABLE* gVar = engineState->varPool->head;
-    while(gVar != NULL) {
+    while (gVar != NULL) {
       CS_VARIABLE* var;
-      if(csound->oparms->odebug)
-      csound->Message(csound, Str(" merging  %d) %s:%s\n"), count,
-                      gVar->varName, gVar->varType->varTypeName);
+      if (csound->oparms->odebug)
+        csound->Message(csound, Str(" merging  %d) %s:%s\n"), count,
+                        gVar->varName, gVar->varType->varTypeName);
       var = csoundFindVariableWithName(current_state->varPool, gVar->varName);
-      if(var == NULL){
-      var = csoundCreateVariable(csound, csound->typePool,
-                                 gVar->varType, gVar->varName, NULL);
-      csoundAddVariable(current_state->varPool, var);
-      /* memory has already been allocated, so we just point to it */
-      /* when disposing of the engineState global vars, we do not
-         delete the memBlock */
-      var->memBlock = gVar->memBlock;
+      if (var == NULL) {
+        ARRAY_VAR_INIT varInit;
+        varInit.dimensions = gVar->dimensions;
+        varInit.type = gVar->varType;
+        var = csoundCreateVariable(csound, csound->typePool,
+                                   gVar->varType, gVar->varName, &varInit);
+        csoundAddVariable(current_state->varPool, var);
+        /* memory has already been allocated, so we just point to it */
+        /* when disposing of the engineState global vars, we do not
+           delete the memBlock */
+        var->memBlock = gVar->memBlock;
       }
       gVar = gVar->next;
     }
@@ -1273,11 +1291,11 @@ int engineState_merge(CSOUND *csound, ENGINE_STATE *engineState)
     /* merge opcodinfo */
     insert_opcodes(csound, csound->opcodeInfo, current_state);
     insert_instrtxt(csound,engineState->instrtxtp[0],0,current_state);
-    for(i=1; i < end; i++){
+    for (i=1; i < end; i++){
       current = engineState->instrtxtp[i];
-      if(current != NULL){
-        if(current->insname == NULL) {
-          if(csound->oparms->odebug)
+      if (current != NULL){
+        if (current->insname == NULL) {
+          if (csound->oparms->odebug)
             csound->Message(csound, Str("merging instr %d \n"), i);
           /* a first attempt at this merge is to make it use
              insert_instrtxt again */
@@ -1285,7 +1303,7 @@ int engineState_merge(CSOUND *csound, ENGINE_STATE *engineState)
           insert_instrtxt(csound,current,i,current_state);
         }
         else {
-          if(csound->oparms->odebug)
+          if (csound->oparms->odebug)
             csound->Message(csound, Str("merging instr %s \n"), current->insname);
           /* allocate a named_instr string in the current engine */
           named_instr_alloc(csound,current->insname,current,-1L,current_state);
@@ -1298,26 +1316,27 @@ int engineState_merge(CSOUND *csound, ENGINE_STATE *engineState)
        in case of multiple instr numbers, so insprep() is called only once */
     current = (&(engineState->instxtanchor));//->nxtinstxt;
     while ((current = current->nxtinstxt) != NULL) {
-      if(csound->oparms->odebug) csound->Message(csound, "insprep %p \n", current);
+      if (csound->oparms->odebug)
+        csound->Message(csound, "insprep %p \n", current);
       insprep(csound, current, current_state);/* run insprep() to connect ARGS  */
       recalculateVarPoolMemory(csound,
                                current->varPool); /* recalculate var pool */
-  }
+    }
     /* now we need to patch up instr order */
     end = current_state->maxinsno;
     end = end < current_state->maxopcno ? current_state->maxopcno : end;
-    for(i=0; i < end; i++){
+    for (i=0; i < end; i++) {
       int j;
       current = current_state->instrtxtp[i];
-      if(current != NULL){
+      if (current != NULL) {
         if(csound->oparms->odebug)
           csound->Message(csound, "instr %d \n", i, current);
         current->nxtinstxt = NULL;
         j = i;
-        while(++j < end-1) {
-          if(current_state->instrtxtp[j] != NULL){
-          current->nxtinstxt = current_state->instrtxtp[j];
-          break;
+        while (++j < end-1) {
+          if (current_state->instrtxtp[j] != NULL) {
+            current->nxtinstxt = current_state->instrtxtp[j];
+            break;
           }
         }
       }
@@ -1568,8 +1587,8 @@ PUBLIC int csoundCompileTree(CSOUND *csound, TREE *root)
 
     /* lock to ensure thread-safety */
     csoundLockMutex(csound->API_lock);
-    if(csound->init_pass_threadlock) csoundLockMutex(csound->init_pass_threadlock);
-    if(engineState != &csound->engineState) {
+    if (csound->init_pass_threadlock) csoundLockMutex(csound->init_pass_threadlock);
+    if (engineState != &csound->engineState) {
       OPDS *ids = csound->ids;
       /* any compilation other than the first one */
       /* merge ENGINE_STATE */
@@ -1579,6 +1598,7 @@ PUBLIC int csoundCompileTree(CSOUND *csound, TREE *root)
       /* run global i-time code */
       init0(csound);
       csound->ids = ids;
+
     }
     else {
       /* first compilation */
@@ -1625,7 +1645,7 @@ PUBLIC int csoundCompileTree(CSOUND *csound, TREE *root)
 
     }
 
-    if(csound->init_pass_threadlock)
+    if (csound->init_pass_threadlock)
       csoundUnlockMutex(csound->init_pass_threadlock);
     /* notify API lock  */
     csoundUnlockMutex(csound->API_lock);
@@ -1639,14 +1659,14 @@ PUBLIC int csoundCompileTree(CSOUND *csound, TREE *root)
     On SUCCESS it returns a value passed to the
     'return' opcode in global space
 */
-PUBLIC MYFLT csoundEvalCode(CSOUND *csound, const char *str){
-
-  if(str && csoundCompileOrc(csound,str) == CSOUND_SUCCESS)
-   return csound->instr0->instance[0].retval;
+PUBLIC MYFLT csoundEvalCode(CSOUND *csound, const char *str)
+{
+    if (str && csoundCompileOrc(csound,str) == CSOUND_SUCCESS)
+      return csound->instr0->instance[0].retval;
 #ifdef NAN
-   else return NAN;
+    else return NAN;
 #else
-  else return 0;
+    else return 0;
 #endif
 }
 
@@ -1662,7 +1682,7 @@ PUBLIC int csoundCompileOrc(CSOUND *csound, const char *str)
     int retVal;
     TREE *root = csoundParseOrc(csound, str);
     if (LIKELY(root != NULL)) {
-    retVal = csoundCompileTree(csound, root);
+      retVal = csoundCompileTree(csound, root);
     }
     else
       return  CSOUND_ERROR;
@@ -1694,8 +1714,8 @@ static void insprep(CSOUND *csound, INSTRTXT *tp, ENGINE_STATE *engineState)
       TEXT *ttp = &optxt->t;
       ep = ttp->oentry;
 
-      if (strcmp(ep->opname, "endin") == 0        /*    (until ENDIN)     */
-            || strcmp(ep->opname, "endop") == 0) break;
+      if (strcmp(ep->opname, "endin") == 0      /*    (until ENDIN)     */
+          || strcmp(ep->opname, "endop") == 0) break;
       if (strcmp(ep->opname, "$label") == 0) {
         continue;
       }
@@ -1709,11 +1729,11 @@ static void insprep(CSOUND *csound, INSTRTXT *tp, ENGINE_STATE *engineState)
         argp = outlist->arg;                    /* get outarg indices */
         while (n--) {
           ARG* arg = createArg(csound, tp, *argp++, engineState);
-          if(ttp->outArgs == NULL) {
+          if (ttp->outArgs == NULL) {
             ttp->outArgs = arg;
           } else {
             ARG* current = ttp->outArgs;
-            while(current->next != NULL) {
+            while (current->next != NULL) {
               current = current->next;
             }
             current->next = arg;
@@ -1742,7 +1762,7 @@ static void insprep(CSOUND *csound, INSTRTXT *tp, ENGINE_STATE *engineState)
             arg = createArg(csound, tp, s, engineState);
           }
 
-          if(ttp->inArgs == NULL) {
+          if (ttp->inArgs == NULL) {
             ttp->inArgs = arg;
           } else {
             ARG* current = ttp->inArgs;
@@ -1779,6 +1799,7 @@ static void insprep(CSOUND *csound, INSTRTXT *tp, ENGINE_STATE *engineState)
                n++, inArgs = inArgs->next) {
             switch (inArgs->type) {
               case ARG_CONSTANT:
+
                 *fp1++ = engineState->constantsPool->values[inArgs->index];
                 break;
 
@@ -1860,6 +1881,7 @@ static ARG* createArg(CSOUND *csound, INSTRTXT* ip,
     if ((c >= '1' && c <= '9') || c == '.' || c == '-' || c == '+' ||
         (c == '0' && strcmp(s, "0dbfs") != 0)) {
       arg->type = ARG_CONSTANT;
+
       arg->index = myflt_pool_find_or_addc(csound, engineState->constantsPool, s);
     } else if (c == '"') {
       STRINGDAT *str = mcalloc(csound, sizeof(STRINGDAT));
@@ -1877,11 +1899,18 @@ static ARG* createArg(CSOUND *csound, INSTRTXT* ip,
       arg->type = ARG_PFIELD;
       arg->index = n;
     }
+    /* trap local ksmps and kr  */
+    else
+      if ((strcmp(s, "ksmps") == 0 && csoundFindVariableWithName(ip->varPool, s))
+          || (strcmp(s, "kr") == 0 && csoundFindVariableWithName(ip->varPool, s))) {
+        arg->type = ARG_LOCAL;
+        arg->argPtr = csoundFindVariableWithName(ip->varPool, s);
+        CS_VARIABLE *var = (CS_VARIABLE *)arg->argPtr;
+      }
     else if (c == 'g' || (c == '#' && *(s+1) == 'g') ||
              csoundFindVariableWithName(csound->engineState.varPool, s) != NULL) {
       // FIXME - figure out why string pool searched with gexist
       //|| string_pool_indexof(csound->engineState.stringPool, s) > 0) {
-
       arg->type = ARG_GLOBAL;
       arg->argPtr = csoundFindVariableWithName(engineState->varPool, s);
 
@@ -1889,8 +1918,7 @@ static ARG* createArg(CSOUND *csound, INSTRTXT* ip,
     else {
       arg->type = ARG_LOCAL;
       arg->argPtr = csoundFindVariableWithName(ip->varPool, s);
-
-      if(arg->argPtr == NULL) {
+      if (arg->argPtr == NULL) {
         csound->Message(csound, Str("Missing local arg: %s\n"), s);
       }
     }
@@ -1906,6 +1934,7 @@ char argtyp2(char *s)
     /* two situations: defined at header level: 0dbfs = 1.0
      *  and returned as a value:  idb = 0dbfs
      */
+
     if ((c >= '1' && c <= '9') || c == '.' || c == '-' || c == '+' ||
         (c == '0' && strcmp(s, "0dbfs") != 0))
       return('c');                              /* const */
@@ -1995,10 +2024,10 @@ void debugPrintCsound(CSOUND* csound)
                       count, current, current->nxtinstxt);
       csound->Message(csound, "Variables\n");
 
-      if(current->varPool != NULL) {
+      if (current->varPool != NULL) {
         CS_VARIABLE* var = current->varPool->head;
         int index = 0;
-        while(var != NULL) {
+        while (var != NULL) {
           if (var->varType == &CS_VAR_TYPE_ARRAY) {
             csound->Message(csound, "  %d) %s:[%s]\n", index++,
                             var->varName, var->subType->varTypeName);
