@@ -371,7 +371,7 @@ OENTRY opcodlst_1[] = {
   { "linsegb", S(LINSEG),0,  3,     "k",    "iin", lsgset_bkpt, klnseg, NULL  },
   { "linsegb.a", S(LINSEG),0,  5,     "a",    "iin", lsgset_bkpt, NULL, linseg  },
   { "linsegr",S(LINSEG),0,  3,      "k",    "iin",  lsgrset,klnsegr,NULL },
-  { "linsegr.a",S(LINSEG),0,  5,      "s",    "iin",  lsgrset,NULL,linsegr },
+  { "linsegr.a",S(LINSEG),0,  5,      "a",    "iin",  lsgrset,NULL,linsegr },
   { "expseg", S(EXXPSEG),0,  3,     "k",    "iin",  xsgset, kxpseg, NULL  },
   { "expseg.a", S(EXXPSEG),0,  5,     "a",    "iin",  xsgset, NULL, expseg  },
   { "expsegb", S(EXXPSEG),0,  3,     "k",    "iin",  xsgset_bkpt, kxpseg, NULL },
@@ -431,11 +431,15 @@ OENTRY opcodlst_1[] = {
   { "oscil1", S(OSCIL1), TR, 3,     "k",    "ikij", ko1set, kosc1          },
   { "oscil1i",S(OSCIL1), TR, 3,     "k",    "ikij", ko1set, kosc1i         },
   { "osciln", S(OSCILN), TR, 5,     "a",    "kiii", oscnset,NULL,   osciln },
-  { "oscil.a",S(OSC),0,    5,       "a",    "kkjo", oscset, NULL, osckk  },
+  { "oscil.a",S(OSC),0,    5,       "a",    "kkjo", oscset, NULL,   osckk  },
   { "oscil.kk",S(OSC),0,    7,      "s",    "kkjo", oscset, koscil, osckk  },
   { "oscil.ka",S(OSC),0,    5,      "a",    "kajo", oscset, NULL,   oscka  },
   { "oscil.ak",S(OSC),0,    5,      "a",    "akjo", oscset, NULL,   oscak  },
   { "oscil.aa",S(OSC),0,    5,      "a",    "aajo", oscset, NULL,   oscaa  },
+  { "oscil.kkA",S(OSC),0,   7,      "a",    "kki[]o", oscsetA, koscil, osckk },
+  { "oscil.kaA",S(OSC),0,   5,      "a",    "kai[]o", oscsetA, NULL, oscka },
+  { "oscil.akA",S(OSC),0,   5,      "a",    "aki[]o", oscsetA, NULL, oscak },
+  { "oscil.aaA",S(OSC),0,   5,      "a",    "aai[]o", oscsetA, NULL,oscaa  },
   /* Change these to
      { "oscil.kk", S(POSC),0, 7, "s", "kkjo", posc_set, kposc, posckk },
      { "oscil.ka", S(POSC),0, 5, "a", "kajo", posc_set, NULL,  poscka },
@@ -444,15 +448,25 @@ OENTRY opcodlst_1[] = {
      { "oscil3.kk",  S(POSC),0,  7, "s", "kkjo", posc_set, kposc3, posc3 },
   */
   { "oscili.a",S(OSC),0,   5,      "a",    "kkjo", oscset, NULL, osckki  },
-  { "oscili.kk",S(OSC),0,   3,      "k",    "kkjo", oscset, koscli, NULL  },
-  { "oscili.ka",S(OSC),0,   5,      "a",    "kajo", oscset, NULL,   osckai  },
-  { "oscili.ak",S(OSC),0,   5,      "a",    "akjo", oscset, NULL,   oscaki  },
-  { "oscili.aa",S(OSC),0,   5,      "a",    "aajo", oscset, NULL,   oscaai  },
-  { "oscil3.a",S(OSC),0,   5,      "a",      "kkjo", oscset, NULL, osckk3  },
-  { "oscil3.kk",S(OSC),0,   3,      "k",    "kkjo", oscset, koscl3, NULL  },
-  { "oscil3.ka",S(OSC),0,   5,      "a",    "kajo", oscset, NULL,   oscka3  },
-  { "oscil3.ak",S(OSC),0,   5,      "a",    "akjo", oscset, NULL,   oscak3  },
-  { "oscil3.aa",S(OSC),0,   5,      "a",    "aajo", oscset, NULL,   oscaa3  },
+  { "oscili.kk",S(OSC),0,   3,      "k",   "kkjo", oscset, koscli, NULL  },
+  { "oscili.ka",S(OSC),0,   5,      "a",   "kajo", oscset, NULL,   osckai  },
+  { "oscili.ak",S(OSC),0,   5,      "a",   "akjo", oscset, NULL,   oscaki  },
+  { "oscili.aa",S(OSC),0,   5,      "a",   "aajo", oscset, NULL,   oscaai  },
+  { "oscili.aA",S(OSC),0,   5,      "a",   "kki[]o", oscsetA, NULL, osckki  },
+  { "oscili.kkA",S(OSC),0,   3,      "k",  "kki[]o", oscsetA, koscli, NULL  },
+  { "oscili.kaA",S(OSC),0,   5,      "a",  "kai[]o", oscsetA, NULL,   osckai  },
+  { "oscili.akA",S(OSC),0,   5,      "a",  "aki[]o", oscsetA, NULL,   oscaki  },
+  { "oscili.aaA",S(OSC),0,   5,      "a",  "aai[]o", oscsetA, NULL,   oscaai  },
+  { "oscil3.a",S(OSC),0,   5,      "a",    "kkjo", oscset, NULL, osckk3  },
+  { "oscil3.kk",S(OSC),0,   3,      "k",   "kkjo", oscset, koscl3, NULL  },
+  { "oscil3.ka",S(OSC),0,   5,      "a",   "kajo", oscset, NULL,   oscka3  },
+  { "oscil3.ak",S(OSC),0,   5,      "a",   "akjo", oscset, NULL,   oscak3  },
+  { "oscil3.aa",S(OSC),0,   5,      "a",   "aajo", oscset, NULL,   oscaa3  },
+  { "oscil3.aA",S(OSC),0,   5,      "a",   "kki[]o", oscsetA, NULL, osckk3 },
+  { "oscil3.kkA",S(OSC),0,   3,      "k",  "kki[]o", oscsetA, koscl3, NULL },
+  { "oscil3.kaA",S(OSC),0,   5,      "a",  "kai[]o", oscsetA, NULL, oscka3 },
+  { "oscil3.akA",S(OSC),0,   5,      "a",  "aki[]o", oscsetA, NULL, oscak3 },
+  { "oscil3.aaA",S(OSC),0,   5,      "a",  "aai[]o", oscsetA, NULL, oscaa3 },
   /* end change */
   { "foscil", S(FOSC),TR,  5,      "a",  "xkxxkjo",foscset,NULL,   foscil  },
   { "foscili",S(FOSC),TR,  5,      "a",  "xkxxkjo",foscset,NULL,   foscili },
@@ -470,13 +484,13 @@ OENTRY opcodlst_1[] = {
   { "randi",  S(RANDI),0,   5,      "a",    "xxvoo", riset, NULL, randi   },
   { "randi.k",  S(RANDI),0,   3,      "k",    "xxvoo", riset, krandi, NULL  },
   { "port",   S(PORT),0,    3,      "k",    "kio",  porset, port            },
-  { "tone",   S(TONE),0,    5,      "a",    "ako",  tonset, NULL,   tone    },
-  { "tonex",  S(TONEX),0,   5,      "a",    "akoo", tonsetx, NULL,  tonex   },
-  { "atone",  S(TONE),0,    5,      "a",    "ako",  tonset, NULL,   atone   },
-  { "atonex", S(TONEX),0,   5,      "a",    "akoo", tonsetx, NULL,  atonex  },
-  { "reson",  S(RESON),0,   5,      "a",    "akkoo",rsnset, NULL,   reson   },
-  { "resonx", S(RESONX),0,  5,      "a",    "akkooo", rsnsetx, NULL, resonx },
-  { "areson", S(RESON),0,   5,      "a",    "akkoo",rsnset, NULL,   areson  },
+  { "tone.k", S(TONE),0,    5,      "a",    "ako",  tonset, NULL,   tone    },
+  { "tonex.k",S(TONEX),0,   5,      "a",    "akoo", tonsetx, NULL,  tonex   },
+  { "atone.k",  S(TONE),0,  5,      "a",    "ako",  tonset, NULL,   atone   },
+  { "atonex.k", S(TONEX),0, 5,      "a",    "akoo", tonsetx, NULL,  atonex  },
+  { "reson", S(RESON),   0, 5,      "a",    "axxoo", rsnset, NULL,  reson   },
+  { "resonx", S(RESONX),0,  5,      "a",    "axxooo", rsnsetx, NULL, resonx },
+  { "areson.kk", S(RESON),0,5,      "a",    "akkoo",rsnset, NULL,   areson  },
   { "lpread", S(LPREAD),0,  3,      "kkkk", "kSoo", lprdset_S,lpread          },
   { "lpread.i", S(LPREAD),0,  3,      "kkkk", "kioo", lprdset,lpread          },
   { "lpform", S(LPFORM),0,  3,      "kk", "k",     lpformantset,lpformant   },
@@ -801,13 +815,19 @@ OENTRY opcodlst_1[] = {
   { "schedkwhen.S", S(TRIGINSTR),0, 3,"",    "kkkSkz",
                                              triginset_S, ktriginstr_S, NULL },
   { "schedkwhennamed", S(TRIGINSTR),0, 3,"", "kkkkkz",triginset, ktriginstr, NULL },
-  { "schedkwhennamed.S", S(TRIGINSTR),0, 3,"", "kkkSkz",
-                                             triginset_S, ktriginstr_S, NULL },
+  { "schedkwhennamed.S", S(TRIGINSTR),0, 3,"",
+                                        "kkkSkz",triginset_S, ktriginstr_S, NULL },
   { "trigseq", S(TRIGSEQ),0, 3,     "",     "kkkkkz", trigseq_set, trigseq, NULL },
   { "event", S(LINEVENT),0,  2,     "",     "Skz",  NULL, eventOpcode, NULL   },
   { "event_i", S(LINEVENT),0,1,     "",     "Sim",  eventOpcodeI, NULL, NULL  },
-  { "event.S", S(LINEVENT),0,  2,     "",     "SSz",  NULL, eventOpcode_S, NULL   },
-  { "event_i.S", S(LINEVENT),0,1,     "",     "SSm",  eventOpcodeI_S, NULL, NULL  },
+  { "event.S", S(LINEVENT),0,  2,     "",    "SSz",  NULL, eventOpcode_S, NULL   },
+  { "event_i.S", S(LINEVENT),0,1,     "",    "SSm",  eventOpcodeI_S, NULL, NULL  },
+  { "instance", S(LINEVENT2),0,2,     "k",  "kkz",  NULL, instanceOpcode, NULL   },
+  { "instance.i", S(LINEVENT2),0,1,   "i",  "iiim",  instanceOpcode, NULL, NULL  },
+  { "instance.kS", S(LINEVENT2),0, 2, "k",  "SSz",  NULL, instanceOpcode_S, NULL },
+  { "instance.S", S(LINEVENT2),0, 1,  "i",  "Siim",  instanceOpcode_S, NULL, NULL},
+  { "kill", S(KILLOP),0,1,     "",     "i", kill_instance, NULL, NULL  },
+  { "kill.k", S(KILLOP),0,2,     "",     "k", NULL, kill_instance, NULL},
   { "lfo", S(LFO),0,         3,     "k",    "kko",  lfoset,   lfok,   NULL   },
   { "lfo.a", S(LFO),0,         5,     "a",    "kko",  lfoset,  NULL,   lfoa    },
   { "oscils",   S(OSCILS),0, 5,     "a", "iiio",
@@ -950,7 +970,7 @@ OENTRY opcodlst_1[] = {
     (SUBR)multitap_set,NULL,(SUBR)multitap_play},
   { "comb",   S(COMB),0,    5,  "a",  "akioo", (SUBR)cmbset,NULL, (SUBR)comb    },
   { "combinv",S(COMB),0,    5,  "a",  "akioo", (SUBR)cmbset,NULL, (SUBR)invcomb },
-  { "alpass", S(COMB),0,    5,  "a",  "akioo", (SUBR)cmbset,NULL, (SUBR)alpass  },
+  { "alpass", S(COMB),0,    5,  "a",  "axioo", (SUBR)cmbset,NULL, (SUBR)alpass  },
   {  "strset",   S(STRSET_OP),0,   1,  "",     "iS",
      (SUBR) strset_init, (SUBR) NULL, (SUBR) NULL                        },
   {  "strget",   S(STRGET_OP),0,   1,  "S",    "i",
@@ -1023,6 +1043,9 @@ OENTRY opcodlst_1[] = {
      (SUBR) strrindex_opcode, (SUBR) NULL, (SUBR) NULL                   },
   {  "strrindexk", S(STRINDEX_OP),0, 3, "k",   "SS",
      (SUBR) strrindex_opcode, (SUBR) strrindex_opcode, (SUBR) NULL       },
+#ifdef HAVE_CURL
+  {  "strfromurl", S(STRCPY_OP), 0, 1, "S", "S", (SUBR) str_from_url     },
+#endif
   {  "changed.S", S(STRINDEX_OP),0, 3, "k",   "S",
      (SUBR) str_changed, (SUBR) str_changed_k, (SUBR) NULL       },
   { "loop_lt",   0xfffb                                                  },
