@@ -25,11 +25,205 @@
 #include "csoundCore.h"     /*                              MEMFILES.C      */
 #include "soundio.h"
 #include "pvfileio.h"
+#include "convolve.h"
+#include "lpc.h"
 #include "pstream.h"
 #include "namedins.h"
 #include <sndfile.h>
 #include <string.h>
 
+static int Load_Het_File_(CSOUND *csound, const char *filnam,
+                          char **allocp, int32 *len)
+{
+    FILE *f;
+    int length = 1024;
+    int i = 0;
+    int cc;
+    int16 x;
+    char *all;
+    char buffer[10];
+    f = fopen(filnam, "r");
+    csoundNotifyFileOpened(csound, filnam, CSFTYPE_HETRO, 0, 0);
+    all = (char *)mmalloc(csound, (size_t) length);
+    for (i=0; i<6; i++) fgetc(f); /* Skip HETRO */
+    fgets(buffer, 10, f);         /* number of partials */
+    x = atoi(buffer);
+    memcpy(&all[0], &x, sizeof(int16));
+    /* Read data until end, pack as int16 */
+    for (i=sizeof(int16);;i+=sizeof(int16)) {
+      int p = 0;
+      while ((cc=getc(f))!=',' && cc!='\n' && p<15) {
+        if (cc == EOF) {
+          goto out;
+        }
+        buffer[p++] = cc;
+      }
+      buffer[p]='\0';
+      /* Expand as necessary */
+      if (i>=length-4) all = mrealloc(csound, all, length+=1024);
+      x = atoi(buffer);
+      memcpy(&all[i], &x, sizeof(int16));
+    }
+ out:
+    fclose(f);                                  /*   and close it      */
+    *len = i;
+    all = mrealloc(csound, all, i);
+    *allocp = all;
+    return 0;                                   /*   return 0 for OK   */
+}
+
+static MYFLT read_ieee(FILE* f, int *end)
+{
+    char buff[120];
+    double x;
+    char *p = fgets(buff, 120, f);
+
+    if (p==NULL || feof(f)) {
+      *end = 1;
+      return FL(0.0);
+    }
+    x = cs_strtod(buff, NULL);
+    return (MYFLT)x;
+    /* union { */
+    /*   double d; */
+    /*   struct {int  j,k;}  n; */
+    /*   int64_t  i; */
+    /* } x; */
+    /* int sign=1, ex; */
+    /* int64_t man; */
+    /* int64_t bit = 1; */
+    /* char buff[32]; */
+    /* char *p; */
+    /* bit <<= 62; */
+    /* p = fgets(buff, 32, f); */
+    /* printf("... %s", buff); */
+    /* if (p==NULL || feof(f)) { */
+    /*   printf("ending\n"); */
+    /*   *end = 1; */
+    /*   return FL(0.0); */
+    /* } */
+    /* if (strstr(p, "0x0p+0")) { */
+    /*   return FL(0.0); */
+    /* } */
+    /* if (buff[0]=='-') sign=-1; */
+    /* p = strchr(buff, '.')+1; */
+    /* sscanf(p, "%lxp%d", &man, &ex); */
+    /* x.i = man; */
+    /* if (man!=(int64_t)0) x.i |= bit; */
+    /* x.d = ldexp(x.d, ex-1); */
+    /* if (sign<0) x.d =-x.d; */
+    /* return (MYFLT)x.d; */
+}
+
+static int Load_CV_File_(CSOUND *csound, const char *filnam,
+                          char **allocp, int32 *len)
+{
+    FILE *f;
+    int length = 4096;
+    unsigned int i = 0;
+    int          j = 0;
+    MYFLT x;
+    char *all;
+    CVSTRUCT cvh = {0,0,0,0,0.0,0,0,0,0,{0}};
+    char buff[120];
+    char *p;
+
+    f = fopen(filnam, "r");
+    csoundNotifyFileOpened(csound, filnam, CSFTYPE_CVANAL, 0, 0);
+    all = (char *)mmalloc(csound, (size_t) length);
+    p = fgets(buff, 120, f); /* Skip CVANAL */
+    cvh.magic = CVMAGIC;
+    p = fgets(buff, 120, f);
+    cvh.headBsize = strtol(p, &p, 10);
+    cvh.dataBsize = strtol(p, &p, 10);
+    cvh.dataFormat = strtol(p, &p, 10);
+    cvh.samplingRate = (MYFLT)cs_strtod(p, &p);
+    cvh.src_chnls = strtol(p, &p, 10);
+    cvh.channel = strtol(p, &p, 10);
+    cvh.Hlen = strtol(p, &p, 10);
+    cvh.Format = strtol(p, &p, 10);
+    /* fscanf(f, "%d %d %d %g %d %d %d %d\n",  */
+    /*        &cvh.headBsize, &cvh.dataBsize, &cvh.dataFormat, */
+    /*        &cvh.samplingRate, &cvh.src_chnls, &cvh.channel, */
+    /*        &cvh.Hlen, &cvh.Format); */
+    cvh.headBsize = sizeof(int32)*8 + sizeof(MYFLT);
+    memcpy(&all[0], &cvh, sizeof(CVSTRUCT));
+
+    /* Read data until end, pack as MYFLTs */
+    for (i=sizeof(CVSTRUCT);;i+=sizeof(MYFLT)) {
+      /* Expand as necessary */
+      if (i>=length-sizeof(MYFLT)-4) {
+        //printf("expanding from %p[%d] to\n", all, length);
+        all = mrealloc(csound, all, length+=4096);
+        //printf("i=%d                     %p[%d]\n", i, all, length);
+      }      x = read_ieee(f, &j);
+      if (j) break;
+      memcpy(&all[i], &x, sizeof(MYFLT));
+    }
+    fclose(f);                                  /*   and close it      */
+    //printf("length=%d i=%d\n", length, i);
+    *len = i;
+    all = mrealloc(csound, all, i);
+    *allocp = all;
+    return 0;                                   /*   return 0 for OK   */
+}
+
+static int Load_LP_File_(CSOUND *csound, const char *filnam,
+                          char **allocp, int32 *len)
+{
+    FILE *f;
+    int length = 4096;
+    unsigned int i = 0;
+    int          j = 0;
+    MYFLT x;
+    char *all, *p;
+    LPHEADER lph = {0,0,0,0,0.0,0.0,0.0,{0}};
+    char buff[120];
+
+    f = fopen(filnam, "r");
+    csoundNotifyFileOpened(csound, filnam, CSFTYPE_LPC, 0, 0);
+    all = (char *)mmalloc(csound, (size_t) length);
+    for (i=0; i<6; i++) fgetc(f); /* Skip LPANAL */
+    fscanf(f, "%d %d %d %d\n",
+           &lph.headersize, &lph.lpmagic, &lph.npoles, &lph.nvals);
+    fgets(buff, 120, f);
+    lph.framrate = (MYFLT)cs_strtod(buff, &p);
+    lph.srate = (MYFLT)cs_strtod(p, &p);
+    lph.duration = (MYFLT)cs_strtod(p, &p);
+    /* lph.text[0] = (char)strtol(p, &p, 0); */
+    /* lph.text[1] = (char)strtol(p, &p, 0); */
+    /* lph.text[2] = (char)strtol(p, &p, 0); */
+    /* lph.text[3] = (char)strtol(p, &p, 0); */
+    /* printf("LPHeader %d %d %d %d\n%f %f %f\n", */
+    /*        lph.headersize, lph.lpmagic, lph.npoles, lph.nvals, */
+    /*        lph.framrate, lph.srate, lph.duration); */
+    /* fscanf(f, "%f %f %f %.2x %.2x %.2x %.2x\n", */
+    /*        &lph.framrate, &lph.srate, &lph.duration, */
+    /*        &lph.text[0], &lph.text[1], &lph.text[2], &lph.text[3]); */
+    // This needs surgery if in/out different MYFLT sizes *** FIX ME ***
+    lph.headersize = sizeof(int32)*4+sizeof(MYFLT)*3;
+    memcpy(&all[0], &lph, lph.headersize);
+
+    /* Read data until end, pack as MYFLTs */
+    for (i=lph.headersize;;i+=sizeof(MYFLT)) {
+      /* Expand as necessary */
+      if (i>=length-sizeof(MYFLT)-8) {
+        //printf("expanding from %p[%d] to\n", all, length);
+        all = mrealloc(csound, all, length+=4096);
+        //printf("i=%d                     %p[%d]\n", i, all, length);
+      }
+      x = read_ieee(f, &j);
+      if (j) break;
+      memcpy(&all[i], &x, sizeof(MYFLT));
+    }
+    fclose(f);                                  /*   and close it      */
+    printf("length=%d i=%u\n", length, i);
+    *len = i;
+    all = mrealloc(csound, all, i);
+    *allocp = all;
+    return 0;                                   /*   return 0 for OK   */
+}
+
 static int Load_File_(CSOUND *csound, const char *filnam,
                        char **allocp, int32 *len, int csFileType)
 {
@@ -38,6 +232,30 @@ static int Load_File_(CSOUND *csound, const char *filnam,
     f = fopen(filnam, "rb");
     if (UNLIKELY(f == NULL))                    /* if cannot open the file */
       return 1;                                 /*    return 1             */
+    if (csFileType==CSFTYPE_HETRO) {
+      char buff[8];
+      fgets(buff, 6, f);
+      if (strcmp(buff, "HETRO")==0) {
+        fclose(f);
+        return Load_Het_File_(csound, filnam, allocp, len);
+      }
+    }
+    else if (csFileType==CSFTYPE_CVANAL) {
+      char buff[8];
+      fgets(buff, 7, f);
+      if (strcmp(buff, "CVANAL")==0) {
+        fclose(f);
+        return Load_CV_File_(csound, filnam, allocp, len);
+      }
+    }
+    else if (csFileType==CSFTYPE_LPC) {
+      char buff[8];
+      fgets(buff, 7, f);
+      if (strcmp(buff, "LPANAL")==0) {
+        fclose(f);
+        return Load_LP_File_(csound, filnam, allocp, len);
+      }
+    }
     /* notify the host if it asked */
     csoundNotifyFileOpened(csound, filnam, csFileType, 0, 0);
     fseek(f, 0L, SEEK_END);                     /* then get its length     */
@@ -62,19 +280,20 @@ static int Load_File_(CSOUND *csound, const char *filnam,
 }
 
 /* Backwards-compatible wrapper for ldmemfile2().
-   Please use ldmemfile2() or ldmemfile2withCB() in all new code instead. */
+   Please use ldmemfile2() or ldmemfile2withCB() in all new code instead.
 MEMFIL *ldmemfile(CSOUND *csound, const char *filnam)
 {
     return ldmemfile2withCB(csound, filnam, CSFTYPE_UNKNOWN, NULL);
 }
-
+*/
 /* Takes an additional parameter specifying the type of the file being opened.
    The type constants are defined in the enumeration CSOUND_FILETYPES.
-   Use ldmemfile2() to load file without additional processing.  */
+   Use ldmemfile2() to load file without additional processing.
 MEMFIL *ldmemfile2(CSOUND *csound, const char *filnam, int csFileType)
 {
     return ldmemfile2withCB(csound, filnam, csFileType, NULL);
 }
+*/
 
 /* This version of ldmemfile2 allows you to specify a callback procedure
    to process the file's data after it is loaded.  This method ensures that
@@ -348,33 +567,27 @@ SNDMEMFILE *csoundLoadSoundFile(CSOUND *csound, const char *fileName, void *sfi)
     void          *fd;
     SNDMEMFILE    *p = NULL;
     SF_INFO       tmp;
-    unsigned char h;
 
 
     if (UNLIKELY(fileName == NULL || fileName[0] == '\0'))
       return NULL;
+
     /* check if file is already loaded */
-    h = name_hash_2(csound, fileName);
     if (csound->sndmemfiles != NULL) {
-      p = ((SNDMEMFILE**) csound->sndmemfiles)[(int) h];
-      while (p != NULL && sCmp(p->name, fileName) != 0)
-        p = p->nxt;
+      p = cs_hash_table_get(csound, csound->sndmemfiles, (char*)fileName);
     }
     else {
-      int i;
-      /* if no files loaded yet, allocate table */
-      csound->sndmemfiles = csound->Malloc(csound, sizeof(SNDMEMFILE*) * 256);
-      for (i = 0; i < 256; i++)
-        ((SNDMEMFILE**) csound->sndmemfiles)[i] = NULL;
+      csound->sndmemfiles = cs_hash_table_create(csound);
     }
+
     if (p != NULL) {
       /* if file was loaded earlier: */
       if (sfinfo != NULL) {
         memset(sfinfo, 0, sizeof(SF_INFO));
         sfinfo->frames = (sf_count_t) p->nFrames;
-         sfinfo->samplerate = ((int) p->sampleRate + 0.5);
-         sfinfo->channels = p->nChannels;
-         sfinfo->format = FORMAT2SF(p->sampleFormat) | TYPE2SF(p->fileType);
+        sfinfo->samplerate = ((int) p->sampleRate + 0.5);
+        sfinfo->channels = p->nChannels;
+        sfinfo->format = FORMAT2SF(p->sampleFormat) | TYPE2SF(p->fileType);
       }
       return p;
     }
@@ -412,8 +625,6 @@ SNDMEMFILE *csoundLoadSoundFile(CSOUND *csound, const char *fileName, void *sfi)
     p->loopEnd = 0.0;
     p->baseFreq = 1.0;
     p->scaleFac = 1.0;
-#if defined(HAVE_LIBSNDFILE) && HAVE_LIBSNDFILE >= 1013
-    /* sampler information requires libsndfile version 1.0.13 or later */
     {
       SF_INSTRUMENT lpd;
       if (sf_command(sf, SFC_GET_INSTRUMENT, &lpd, sizeof(SF_INSTRUMENT))
@@ -434,8 +645,6 @@ SNDMEMFILE *csoundLoadSoundFile(CSOUND *csound, const char *fileName, void *sfi)
         p->scaleFac = pow(10.0, (double) lpd.gain * 0.05);
       }
     }
-#endif      /* HAVE_LIBSNDFILE >= 1013 */
-    p->nxt = ((SNDMEMFILE**) csound->sndmemfiles)[(int) h];
     if ((size_t) sf_readf_float(sf, &(p->data[0]), (sf_count_t) p->nFrames)
         != p->nFrames) {
       csound->FileClose(csound, fd);
@@ -453,8 +662,10 @@ SNDMEMFILE *csoundLoadSoundFile(CSOUND *csound, const char *fileName, void *sfi)
                             p->fullName, (int) sfinfo->samplerate,
                             (int) sfinfo->channels,
                             (uint32) sfinfo->frames);
+
     /* link into database */
-    ((SNDMEMFILE**) csound->sndmemfiles)[(int) h] = p;
+    cs_hash_table_put(csound, csound->sndmemfiles, (char*)fileName, p);
+
     /* return with pointer to file structure */
     return p;
 }
