@@ -188,6 +188,7 @@ int     tablikt(CSOUND *, void *), ko1set(CSOUND *, void *);
 int     kosc1(CSOUND *, void *), kosc1i(CSOUND *, void *);
 int     oscnset(CSOUND *, void *), osciln(CSOUND *, void *);
 int     oscset(CSOUND *, void *), koscil(CSOUND *, void *);
+int     oscsetA(CSOUND *, void *);
 int     osckk(CSOUND *, void *), oscka(CSOUND *, void *);
 int     oscak(CSOUND *, void *), oscaa(CSOUND *, void *);
 int     koscli(CSOUND *, void *), osckki(CSOUND *, void *);
@@ -350,6 +351,8 @@ int     triginset_S(CSOUND *, void *), ktriginstr_S(CSOUND *, void *);
 int     trigseq_set(CSOUND *, void *), trigseq(CSOUND *, void *);
 int     eventOpcode(CSOUND *, void *), eventOpcodeI(CSOUND *, void *);
 int     eventOpcode_S(CSOUND *, void *), eventOpcodeI_S(CSOUND *, void *);
+int     instanceOpcode(CSOUND *, void *), instanceOpcode_S(CSOUND *, void *);
+int     kill_instance(CSOUND *csound, void *p);
 int     lfoset(CSOUND *, void *);
 int     lfok(CSOUND *, void *), lfoa(CSOUND *, void *);
 int     mute_inst(CSOUND *, void *);
