@@ -21,12 +21,13 @@
     02111-1307 USA
 */
 
-#include "lpc.h"        /*                                              UGENS5.H        */
+#include "lpc.h"        /*                               UGENS5.H        */
 
 typedef struct {
         OPDS    h;
         MYFLT   *kr, *ksig, *ihtim, *isig;
         double   c1, c2, yt1;
+        MYFLT  ihtim_old;
 } PORT;
 
 typedef struct {
@@ -60,7 +61,7 @@ typedef struct {
 
 typedef struct {
         OPDS    h;
-        MYFLT   *krmr, *krmo, *kerr, *kcps, *ktimpt, *ifilno, *inpoles, *ifrmrate;
+        MYFLT   *krmr, *krmo, *kerr, *kcps, *ktimpt, *ifilcod, *inpoles, *ifrmrate;
         int32   headlen, npoles, nvals, lastfram16, lastmsg;
         MYFLT   kcoefs[MAXPOLES*2], framrat16;
         int     storePoles ;
@@ -122,3 +123,25 @@ typedef struct {
         MYFLT   kcoefs[MAXPOLES*2], framrat16;
         int             storePoles ;
 } LPINTERPOL ;
+
+typedef struct {
+        OPDS    h;
+        MYFLT   *ans, *sig, *min, *max;
+} LIMIT;
+
+typedef PORT KPORT;
+typedef TONE KTONE;
+typedef RESON KRESON;
+
+int kporset(CSOUND*,PORT *p);
+int kport(CSOUND*,PORT *p);
+int ktonset(CSOUND*,TONE *p);
+int ktone(CSOUND*,TONE *p);
+int katone(CSOUND*,TONE *p);
+int krsnset(CSOUND*,RESON *p);
+int kreson(CSOUND*,RESON *p);
+int kareson(CSOUND*,RESON *p);
+int limitset(CSOUND*,LIMIT *p);
+int klimit(CSOUND*,LIMIT *p);
+int limit(CSOUND*,LIMIT *p);
+
