@@ -1,24 +1,24 @@
 /*
-    rtjack.c:
+  rtjack.c:
 
-    Copyright (C) 2005, 2006 Istvan Varga
+  Copyright (C) 2005, 2006 Istvan Varga
 
-    This file is part of Csound.
+  This file is part of Csound.
 
-    The Csound Library is free software; you can redistribute it
-    and/or modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+  The Csound Library is free software; you can redistribute it
+  and/or modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
 
-    Csound is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
+  Csound is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with Csound; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-    02111-1307 USA
+  You should have received a copy of the GNU Lesser General Public
+  License along with Csound; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+  02111-1307 USA
 */
 
 #include <jack/jack.h>
@@ -210,7 +210,7 @@ static CS_NOINLINE int rtJack_ListPorts(CSOUND *csound,
     int             i, nChn, maxChn, len, nPorts, retval = -1;
 
     portFlags = (isOutput ? (unsigned long) JackPortIsInput
-                          : (unsigned long) JackPortIsOutput);
+                 : (unsigned long) JackPortIsOutput);
     len = jack_port_name_size();
     prvPortName = (char*) malloc((size_t) len);
     if (UNLIKELY(prvPortName == (char*) NULL))
@@ -226,7 +226,7 @@ static CS_NOINLINE int rtJack_ListPorts(CSOUND *csound,
       goto err_return;
     retval = 0;
     csound->Message(csound, Str("The available JACK %s devices are:\n"),
-                            (isOutput ? Str("output") : Str("input")));
+                    (isOutput ? Str("output") : Str("input")));
     /* count the number of ports, and sort port names in alphabetical order */
     for (nPorts = 0; portNames[nPorts] != NULL; nPorts++)
       ;
@@ -265,7 +265,7 @@ static CS_NOINLINE int rtJack_ListPorts(CSOUND *csound,
           maxChn = chn_;
       }
       continue;
- nextPortName:
+    nextPortName:
       if (nChn == maxChn)
         rtJack_PrintPortName(csound, prvPortName, nChn);
       prvPortName[0] = (char) 0;
@@ -392,7 +392,7 @@ static void rtJack_RegisterPorts(RtJackGlobals *p)
       }
     }
 }
-
+static int listDevices(CSOUND *csound, CS_AUDIODEVICE *list, int isOutput);
 /* connect to JACK server, set up ports and ring buffers, */
 /* activate client, and connect ports if requested */
 
@@ -415,7 +415,7 @@ static void openJackStreams(RtJackGlobals *p)
     if (UNLIKELY(p->sampleRate != (int) jack_get_sample_rate(p->client))) {
       sprintf(&(buf[0]), Str("sample rate %d does not match "
                              "JACK sample rate %d"),
-                         p->sampleRate, (int) jack_get_sample_rate(p->client));
+              p->sampleRate, (int) jack_get_sample_rate(p->client));
       rtJack_Error(p->csound, -1, &(buf[0]));
     }
     if (UNLIKELY(p->bufSize < 8 || p->bufSize > 32768))
@@ -460,21 +460,25 @@ static void openJackStreams(RtJackGlobals *p)
       p->outPortBufs[0] = (jack_default_audio_sample_t*) NULL;
 
     /* register callback functions */
-    if (UNLIKELY(jack_set_sample_rate_callback(p->client, sampleRateCallback, (void*) p)
+    if (UNLIKELY(jack_set_sample_rate_callback(p->client,
+                                               sampleRateCallback, (void*) p)
                  != 0))
       rtJack_Error(csound, -1, Str("error setting sample rate callback"));
-    if (UNLIKELY(jack_set_buffer_size_callback(p->client, bufferSizeCallback, (void*) p)
+    if (UNLIKELY(jack_set_buffer_size_callback(p->client,
+                                               bufferSizeCallback, (void*) p)
                  != 0))
       rtJack_Error(csound, -1, Str("error setting buffer size callback"));
 #ifdef LINUX
-    if (UNLIKELY(jack_set_freewheel_callback(p->client, freeWheelCallback, (void*) p)
+    if (UNLIKELY(jack_set_freewheel_callback(p->client,
+                                             freeWheelCallback, (void*) p)
                  != 0))
       rtJack_Error(csound, -1, Str("error setting freewheel callback"));
 #endif
     if (UNLIKELY(jack_set_xrun_callback(p->client, xrunCallback, (void*) p) != 0))
       rtJack_Error(csound, -1, Str("error setting xrun callback"));
     jack_on_shutdown(p->client, shutDownCallback, (void*) p);
-    if (UNLIKELY(jack_set_process_callback(p->client, processCallback, (void*) p) != 0))
+    if (UNLIKELY(jack_set_process_callback(p->client,
+                                           processCallback, (void*) p) != 0))
       rtJack_Error(csound, -1, Str("error setting process callback"));
 
     /* activate client */
@@ -482,31 +486,65 @@ static void openJackStreams(RtJackGlobals *p)
       rtJack_Error(csound, -1, Str("error activating JACK client"));
 
     /* connect ports if requested */
-    if (p->inputEnabled && p->inDevName != NULL) {
-      char  *sp = strchr(p->inDevName, '\0');
-      for (i = 0; i < p->nChannels; i++) {
-        sprintf(sp, "%d", i + 1);
-        if (UNLIKELY(jack_connect(p->client, p->inDevName,
-                                  jack_port_name(p->inPorts[i])) != 0)) {
-          rtJack_ListPorts(csound, p->client, &(p->clientName[0]), 0);
-          rtJack_Error(csound, -1, Str("error connecting input ports"));
-        }
+    if (p->inputEnabled) {
+      char dev[128], *dev_final, *sp;
+      {
+        int i,n = listDevices(csound,NULL,0);
+        CS_AUDIODEVICE *devs = (CS_AUDIODEVICE *)
+                malloc(n*sizeof(CS_AUDIODEVICE));
+        listDevices(csound,devs,0);
+        for(i=0; i < n; i++)
+          csound->Message(csound, " %d: %s (%s)\n",
+                          i, devs[i].device_id, devs[i].device_name);
+        strncpy(dev, devs[0].device_name, 128);
+        free(devs);
       }
-      *sp = (char) 0;
-    }
-    if (p->outputEnabled && p->outDevName != NULL) {
-      char  *sp = strchr(p->outDevName, '\0');
-      for (i = 0; i < p->nChannels; i++) {
-        sprintf(sp, "%d", i + 1);
-        if (jack_connect(p->client, jack_port_name(p->outPorts[i]),
-                                    p->outDevName) != 0) {
-          rtJack_ListPorts(csound, p->client, &(p->clientName[0]), 1);
-          rtJack_Error(csound, -1, Str("error connecting output ports"));
-        }
-      }
-      *sp = (char) 0;
-    }
+      if(p->inDevName != NULL)
+          strncpy(dev, p->inDevName, 128);
+      if (dev) {
+        dev_final = dev;
+        sp = strchr(dev_final, '\0');
+        if(!isalpha(dev_final[0])) dev_final++;
 
+        for (i = 0; i < p->nChannels; i++) {
+          sprintf(sp, "%d", i + 1);
+          if (UNLIKELY(jack_connect(p->client, dev_final,
+                                    jack_port_name(p->inPorts[i])) != 0)) {
+              rtJack_Error(csound, -1, Str("error connecting input ports"));
+          }
+        }
+        *sp = (char) 0;
+      }
+
+    }
+    if (p->outputEnabled) {
+      char dev[128], *dev_final, *sp;
+      {
+          int i,n = listDevices(csound,NULL,1);
+          CS_AUDIODEVICE *devs = (CS_AUDIODEVICE *)
+                  malloc(n*sizeof(CS_AUDIODEVICE));
+          listDevices(csound,devs,1);
+          for(i=0; i < n; i++)
+            csound->Message(csound, " %d: %s (%s)\n",
+                            i, devs[i].device_id, devs[i].device_name);
+          strncpy(dev, devs[0].device_name, 128);
+          free(devs);
+      }
+      if(p->outDevName != NULL) strncpy(dev, p->outDevName, 128);
+      if (dev) {
+        dev_final = dev;
+        sp = strchr(dev_final, '\0');
+        if(!isalpha(dev_final[0])) dev_final++;
+        for (i = 0; i < p->nChannels; i++) {
+          sprintf(sp, "%d", i + 1);
+          if (jack_connect(p->client, jack_port_name(p->outPorts[i]),
+                           dev_final) != 0) {
+            rtJack_Error(csound, -1, Str("error connecting output ports"));
+          }
+        }
+        *sp = (char) 0;
+      }
+    }
     /* stream is now active */
     p->jackState = 0;
 }
@@ -547,19 +585,21 @@ static void rtJack_CopyDevParams(RtJackGlobals *p, char **devName,
       nBytes = strlen(parm->devName) + 4;
       if (UNLIKELY(nBytes > (size_t) jack_port_name_size()))
         rtJack_Error(csound, -1, Str("device name is too long"));
-      s = (char*) malloc(nBytes);
+      s = (char*) malloc(nBytes+1);
       if (UNLIKELY(s == NULL))
         rtJack_Error(csound, CSOUND_MEMORY, Str("memory allocation failure"));
       strcpy(s, parm->devName);
+
       *devName = s;
     }
     if (isOutput && p->inputEnabled) {
       /* full duplex audio I/O: check consistency of parameters */
-      if (UNLIKELY(p->nChannels != parm->nChannels || p->bufSize != parm->bufSamp_SW))
+      if (UNLIKELY(p->nChannels != parm->nChannels ||
+                   (unsigned int)p->bufSize != parm->bufSamp_SW))
         rtJack_Error(csound, -1,
                      Str("input and output parameters are not consistent"));
-      if (UNLIKELY(((parm->bufSamp_SW / csound->ksmps) * csound->ksmps)
-                   != parm->bufSamp_SW))
+      if (UNLIKELY((unsigned int)((parm->bufSamp_SW / csound->GetKsmps(csound)) *
+                                  csound->GetKsmps(csound)) != parm->bufSamp_SW))
         rtJack_Error(csound, -1,
                      Str("period size (-b) must be an integer multiple of ksmps"));
     }
@@ -580,20 +620,22 @@ static int recopen_(CSOUND *csound, const csRtAudioParams *parm)
     p = (RtJackGlobals*) csound->QueryGlobalVariable(csound, "_rtjackGlobals");
     if (p == NULL)
       return -1;
-    csound->rtRecord_userdata = (void*) p;
+    *(csound->GetRtRecordUserData(csound)) = (void*) p;
     rtJack_CopyDevParams(p, &(p->inDevName), parm, 0);
     p->inputEnabled = 1;
     /* allocate pointers to input ports */
     p->inPorts = (jack_port_t**)
-        calloc((size_t) p->nChannels, sizeof(jack_port_t*));
+      calloc((size_t) p->nChannels, sizeof(jack_port_t*));
     if (UNLIKELY(p->inPorts == NULL))
       rtJack_Error(p->csound, CSOUND_MEMORY, Str("memory allocation failure"));
     /* allocate pointers to input port buffers */
     p->inPortBufs = (jack_default_audio_sample_t**)
-        calloc((size_t) p->nChannels, sizeof(jack_default_audio_sample_t*));
+      calloc((size_t) p->nChannels, sizeof(jack_default_audio_sample_t*));
     if (UNLIKELY(p->inPortBufs == NULL))
       rtJack_Error(p->csound, CSOUND_MEMORY, Str("memory allocation failure"));
 
+
+
     return 0;
 }
 
@@ -606,17 +648,18 @@ static int playopen_(CSOUND *csound, const csRtAudioParams *parm)
     p = (RtJackGlobals*) csound->QueryGlobalVariable(csound, "_rtjackGlobals");
     if (p == NULL)
       return -1;
-    csound->rtPlay_userdata = (void*) p;
+    *(csound->GetRtPlayUserData(csound)) = (void*) p;
     rtJack_CopyDevParams(p, &(p->outDevName), parm, 1);
+
     p->outputEnabled = 1;
     /* allocate pointers to output ports */
     p->outPorts = (jack_port_t**)
-        calloc((size_t) p->nChannels, sizeof(jack_port_t*));
+      calloc((size_t) p->nChannels, sizeof(jack_port_t*));
     if (UNLIKELY(p->outPorts == NULL))
       rtJack_Error(p->csound, CSOUND_MEMORY, Str("memory allocation failure"));
     /* allocate pointers to output port buffers */
     p->outPortBufs = (jack_default_audio_sample_t**)
-        calloc((size_t) p->nChannels, sizeof(jack_default_audio_sample_t*));
+      calloc((size_t) p->nChannels, sizeof(jack_default_audio_sample_t*));
     if (UNLIKELY(p->outPortBufs == NULL))
       rtJack_Error(p->csound, CSOUND_MEMORY, Str("memory allocation failure"));
     /* activate client to start playback */
@@ -638,12 +681,12 @@ static int processCallback(jack_nframes_t nframes, void *arg)
     if (p->inputEnabled) {
       for (i = 0; i < p->nChannels; i++)
         p->inPortBufs[i] = (jack_default_audio_sample_t*)
-                                jack_port_get_buffer(p->inPorts[i], nframes);
+          jack_port_get_buffer(p->inPorts[i], nframes);
     }
     if (p->outputEnabled && p->outPortBufs[0] == NULL) {
       for (i = 0; i < p->nChannels; i++)
         p->outPortBufs[i] = (jack_default_audio_sample_t*)
-                                jack_port_get_buffer(p->outPorts[i], nframes);
+          jack_port_get_buffer(p->outPorts[i], nframes);
     }
     i = 0;
     do {
@@ -724,7 +767,7 @@ static int rtrecord_(CSOUND *csound, MYFLT *inbuf_, int bytes_)
     RtJackGlobals *p;
     int           i, j, k, nframes, bufpos, bufcnt;
 
-    p = (RtJackGlobals*) csound->rtPlay_userdata;
+    p = (RtJackGlobals*) *(csound->GetRtPlayUserData(csound));
     if (p->jackState != 0) {
       if (p->jackState < 0)
         openJackStreams(p);     /* open audio input */
@@ -760,7 +803,9 @@ static int rtrecord_(CSOUND *csound, MYFLT *inbuf_, int bytes_)
     }
     if (p->xrunFlag) {
       p->xrunFlag = 0;
-      if (csound->oparms->msglevel & 4)
+      OPARMS oparms;
+      csound->GetOParms(csound, &oparms);
+      if (oparms.msglevel & 4)
         csound->Warning(csound, Str("rtjack: xrun in real time audio"));
     }
 
@@ -774,7 +819,7 @@ static void rtplay_(CSOUND *csound, const MYFLT *outbuf_, int bytes_)
     RtJackGlobals *p;
     int           i, j, k, nframes;
 
-    p = (RtJackGlobals*) csound->rtPlay_userdata;
+    p = (RtJackGlobals*) *(csound->GetRtPlayUserData(csound));
     if (p == NULL)
       return;
     if (p->jackState != 0) {
@@ -794,7 +839,7 @@ static void rtplay_(CSOUND *csound, const MYFLT *outbuf_, int bytes_)
       /* copy audio data */
       for (k = 0; k < p->nChannels; k++)
         p->bufs[p->csndBufCnt]->outBufs[k][i] =
-            (jack_default_audio_sample_t) outbuf_[j++];
+          (jack_default_audio_sample_t) outbuf_[j++];
       if (++(p->csndBufPos) >= p->bufSize) {
         p->csndBufPos = 0;
         /* notify JACK callback that this buffer is now filled */
@@ -843,20 +888,20 @@ static CS_NOINLINE void rtclose_(CSOUND *csound)
     pp = (RtJackGlobals*) csound->QueryGlobalVariable(csound, "_rtjackGlobals");
     if (pp == NULL)
       return;
-    csound->rtRecord_userdata = NULL;
-    csound->rtPlay_userdata = NULL;
+    *(csound->GetRtPlayUserData(csound))  = NULL;
+    *(csound->GetRtRecordUserData(csound))  = NULL;
     memcpy(&p, pp, sizeof(RtJackGlobals));
     /* free globals */
     csound->DestroyGlobalVariable(csound, "_rtjackGlobals");
     if (p.client != (jack_client_t*) NULL) {
       /* deactivate client */
-      if (p.jackState != 2) {
-        if (p.jackState == 0)
-          csound->Sleep((size_t)
-                        ((int) ((double) (p.bufSize * p.nBuffers)
-                                * 1000.0 / (double) p.sampleRate + 0.999)));
-        jack_deactivate(p.client);
-      }
+      //if (p.jackState != 2) {
+      //if (p.jackState == 0)
+      //  csound->Sleep((size_t)
+      //                ((int) ((double) (p.bufSize * p.nBuffers)
+      //                        * 1000.0 / (double) p.sampleRate + 0.999)));
+      jack_deactivate(p.client);
+      //}
       csound->Sleep((size_t) 50);
       /* unregister and free all ports */
       if (p.inPorts != NULL) {
@@ -872,8 +917,9 @@ static CS_NOINLINE void rtclose_(CSOUND *csound)
         }
       }
       /* close connection */
-      if (p.jackState != 2)
+      if (p.jackState != 2) {
         jack_client_close(p.client);
+      }
     }
     /* free copy of input and output device name */
     if (p.inDevName != NULL)
@@ -903,15 +949,70 @@ static CS_NORETURN void rtJack_Error(CSOUND *csound,
     csound->LongJmp(csound, errCode);
 }
 
+int listDevices(CSOUND *csound, CS_AUDIODEVICE *list, int isOutput){
+
+    char            **portNames = (char**) NULL, port[64];
+    unsigned long   portFlags;
+    int             i, n, cnt=0;
+    jack_client_t *jackClient;
+    RtJackGlobals* p =
+      (RtJackGlobals*) csound->QueryGlobalVariableNoCheck(csound,
+                                                          "_rtjackGlobals");
+
+    if(p->listclient == NULL)
+      p->listclient = jack_client_open("list", JackNoStartServer, NULL);
+
+    jackClient  = p->listclient;
+
+    if(jackClient == NULL) return 0;
+    portFlags = (isOutput ? (unsigned long) JackPortIsInput
+                 : (unsigned long) JackPortIsOutput);
+
+    portNames = (char**) jack_get_ports(jackClient,
+                                        (char*) NULL,
+                                        JACK_DEFAULT_AUDIO_TYPE,
+                                        portFlags);
+    if(portNames == NULL) {
+      jack_client_close(jackClient);
+      p->listclient = NULL;
+      return 0;
+    }
+
+    memset(port, '\0', 64);
+    for(i=0; portNames[i] != NULL; i++) {
+      n = (int) strlen(portNames[i]);
+      do {
+        n--;
+      } while (n > 0 && isdigit(portNames[i][n]));
+      n++;
+      if(strncmp(portNames[i], port, n)==0) continue;
+      strncpy(port, portNames[i], n);
+      port[n] = '\0';
+      if (list != NULL) {
+        strncpy(list[cnt].device_name, port, 63);
+        snprintf(list[cnt].device_id, 63, "%s%s",
+                 isOutput ? "dac:" : "adc:",port);
+        list[cnt].max_nchnls = -1;
+        list[cnt].isOutput = isOutput;
+      }
+      cnt++;
+    }
+    jack_client_close(jackClient);
+    p->listclient = NULL;
+    return cnt;
+}
+
 /* module interface functions */
 
 PUBLIC int csoundModuleCreate(CSOUND *csound)
 {
     RtJackGlobals   *p;
     int             i, j;
+    OPARMS oparms;
+    csound->GetOParms(csound, &oparms);
 
     /* allocate and initialise globals */
-    if (csound->oparms->msglevel & 0x400)
+    if (oparms.msglevel & 0x400)
       csound->Message(csound, Str("JACK real-time audio module for Csound "
                                   "by Istvan Varga\n"));
     if (csound->CreateGlobalVariable(csound, "_rtjackGlobals",
@@ -923,7 +1024,7 @@ PUBLIC int csoundModuleCreate(CSOUND *csound)
                                                             "_rtjackGlobals");
     p->csound = csound;
     p->jackState = -1;
-    strcpy(&(p->clientName[0]), "csound5");
+    strcpy(&(p->clientName[0]), "csound6");
     strcpy(&(p->inputPortName[0]), "input");
     strcpy(&(p->outputPortName[0]), "output");
     p->sleepTime = 1000;        /* this is not actually used */
@@ -940,40 +1041,61 @@ PUBLIC int csoundModuleCreate(CSOUND *csound)
     i = jack_client_name_size();
     if (i > (MAX_NAME_LEN + 1))
       i = (MAX_NAME_LEN + 1);
-    csound->CreateConfigurationVariable(
-        csound, "jack_client", (void*) &(p->clientName[0]),
-        CSOUNDCFG_STRING, 0, NULL, &i,
-        "JACK client name (default: csound5)", NULL);
+    csound->CreateConfigurationVariable(csound, "jack_client",
+                                        (void*) &(p->clientName[0]),
+                                        CSOUNDCFG_STRING, 0, NULL, &i,
+                                        Str("JACK client name (default: csound6)"),
+                                        NULL);
     /*   input port name */
     i = jack_port_name_size() - 3;
     if (i > (MAX_NAME_LEN + 1))
       i = (MAX_NAME_LEN + 1);
-    csound->CreateConfigurationVariable(
-        csound, "jack_inportname", (void*) &(p->inputPortName[0]),
-        CSOUNDCFG_STRING, 0, NULL, &i,
-        "JACK input port name prefix (default: input)", NULL);
+    csound->CreateConfigurationVariable(csound, "jack_inportname",
+                                        (void*) &(p->inputPortName[0]),
+                                        CSOUNDCFG_STRING, 0, NULL, &i,
+                                        Str("JACK input port name prefix "
+                                            "(default: input)"), NULL);
     /*   output port name */
     i = jack_port_name_size() - 3;
     if (i > (MAX_NAME_LEN + 1))
       i = (MAX_NAME_LEN + 1);
-    csound->CreateConfigurationVariable(
-        csound, "jack_outportname", (void*) &(p->outputPortName[0]),
-        CSOUNDCFG_STRING, 0, NULL, &i,
-        "JACK output port name prefix (default: output)", NULL);
-    /* sleep time */
+    csound->CreateConfigurationVariable(csound, "jack_outportname",
+                                      (void*) &(p->outputPortName[0]),
+                                        CSOUNDCFG_STRING, 0, NULL, &i,
+                                        Str("JACK output port name prefix"
+                                            " (default: output)"), NULL);
+  /* sleep time */
     i = 250; j = 25000;         /* min/max value */
-        csound->CreateConfigurationVariable(
-        csound, "jack_sleep_time", (void*) &(p->sleepTime),
-        CSOUNDCFG_INTEGER, 0, &i, &j,
-        "Deprecated", NULL);
+    csound->CreateConfigurationVariable(csound, "jack_sleep_time",
+                                        (void*) &(p->sleepTime),
+                                        CSOUNDCFG_INTEGER, 0, &i, &j,
+                                        Str("Deprecated"), NULL);
     /* done */
+    p->listclient = NULL;
+
     return 0;
 }
 
+
+
+PUBLIC int csoundModuleDestroy(CSOUND *csound)
+{
+    RtJackGlobals* p =
+      (RtJackGlobals*) csound->QueryGlobalVariableNoCheck(csound,
+                                                          "_rtjackGlobals");
+    if(p && p->listclient) {
+      jack_client_close(p->listclient);
+      p->listclient = NULL;
+    }
+    return OK;
+}
+
+
+
 PUBLIC int csoundModuleInit(CSOUND *csound)
 {
     char    *drv;
-
+    csound->module_list_add(csound,"jack", "audio");
     drv = (char*) csound->QueryGlobalVariable(csound, "_RTAUDIO");
     if (drv == NULL)
       return 0;
@@ -981,13 +1103,15 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
           strcmp(drv, "JACK") == 0))
       return 0;
     csound->Message(csound, Str("rtaudio: JACK module enabled\n"));
-    /* register Csound interface functions */
-    csound->SetPlayopenCallback(csound, playopen_);
-    csound->SetRecopenCallback(csound, recopen_);
-    csound->SetRtplayCallback(csound, rtplay_);
-    csound->SetRtrecordCallback(csound, rtrecord_);
-    csound->SetRtcloseCallback(csound, rtclose_);
-
+    {
+      /* register Csound interface functions */
+      csound->SetPlayopenCallback(csound, playopen_);
+      csound->SetRecopenCallback(csound, recopen_);
+      csound->SetRtplayCallback(csound, rtplay_);
+      csound->SetRtrecordCallback(csound, rtrecord_);
+      csound->SetRtcloseCallback(csound, rtclose_);
+      csound->SetAudioDeviceListCallback(csound, listDevices);
+    }
     return 0;
 }
 
@@ -995,4 +1119,3 @@ PUBLIC int csoundModuleInfo(void)
 {
     return ((CS_APIVERSION << 16) + (CS_APISUBVER << 8) + (int) sizeof(MYFLT));
 }
-
