@@ -100,40 +100,66 @@ static int initPortAudio(CSOUND *csound)
 }
 
 /* list available input or output devices; returns the number of devices */
-
-static int listPortAudioDevices_blocking(CSOUND *csound,
-                                         int print_list, int play)
-{
-    PaDeviceInfo  *dev_info;
+int listDevices(CSOUND *csound, CS_AUDIODEVICE *list, int isOutput){
+     PaDeviceInfo  *dev_info;
     int           i, j, ndev;
+    char          tmp[256], *s;
+
+    if (initPortAudio(csound) != 0)
+      return 0;
+
+     if ((s = (char*) csound->QueryGlobalVariable(csound, "_RTAUDIO")) == NULL)
+      return 0;
 
     ndev = (int) Pa_GetDeviceCount();
     for (i = j = 0; i < ndev; i++) {
       dev_info = (PaDeviceInfo*) Pa_GetDeviceInfo((PaDeviceIndex) i);
-      if ((play && dev_info->maxOutputChannels > 0) ||
-          (!play && dev_info->maxInputChannels > 0))
+      if ((isOutput && dev_info->maxOutputChannels > 0) ||
+          (!isOutput && dev_info->maxInputChannels > 0))
         j++;
     }
-    if (!j) {
-      pa_PrintErrMsg(csound, Str("No %s devices are available\n"),
-                             (play ? Str("output") : Str("input")));
-      return 0;
-    }
-    if (!print_list)
-      return j;
-    csound->Message(csound, Str("PortAudio: available %s devices:\n"),
-                            (play ? Str("output") : Str("input")));
+    if (!j) return 0;
+    if(list!=NULL) {
     for (i = j = 0; i < ndev; i++) {
       dev_info = (PaDeviceInfo*) Pa_GetDeviceInfo((PaDeviceIndex) i);
-      if ((play && dev_info->maxOutputChannels > 0) ||
-          (!play && dev_info->maxInputChannels > 0)) {
-        csound->Message(csound, " %3d: %s\n", j, dev_info->name);
+      if ((isOutput && dev_info->maxOutputChannels > 0) ||
+          (!isOutput && dev_info->maxInputChannels > 0)) {
+        strncpy(list[j].device_name, dev_info->name, 63);
+        if (isOutput) {
+            sprintf(tmp, "dac%d", j);
+        } else {
+            sprintf(tmp, "adc%d", j);
+        }
+        strncpy(list[j].device_id, tmp, 63);
+        strncpy(list[j].rt_module, s, 63);
+        list[j].max_nchnls =
+          isOutput ?  dev_info->maxOutputChannels : dev_info->maxInputChannels;
+        list[j].isOutput = isOutput;
         j++;
       }
     }
+    }
     return j;
 }
 
+
+static int listPortAudioDevices_blocking(CSOUND *csound,
+                                         int print_list, int play)
+{
+    int i,n = listDevices(csound, NULL, play);
+    CS_AUDIODEVICE *devs =
+      (CS_AUDIODEVICE *) csound->Malloc(csound, n*sizeof(CS_AUDIODEVICE));
+    listDevices(csound, devs, play);
+    for(i=0; i < n; i++)
+      csound->Message(csound, " %3d: %s (%s)\n",
+                      i, devs[i].device_id, devs[i].device_name);
+    csound->Free(csound, devs);
+    return n;
+}
+
+
+
+
 /* select PortAudio device; returns the actual device number */
 
 static int selectPortAudioDevice(CSOUND *csound, int devNum, int play)
@@ -193,7 +219,6 @@ static int pa_SetStreamParameters(CSOUND *csound, PaStreamParameters *sp,
     if (dev < 0)
       return -1;
     sp->device = (PaDeviceIndex) dev;
-    if(!is_playback && (csound->nchnls != csound->inchnls)) parm->nChannels = csound->inchnls;
     sp->channelCount = (parm->nChannels < 2 ? 2 : parm->nChannels);
     sp->sampleFormat = (PaSampleFormat) paFloat32;
     sp->suggestedLatency = (PaTime) ((double) parm->bufSamp_HW
@@ -219,7 +244,7 @@ static int paBlockingReadWriteOpen(CSOUND *csound)
                                                              "_rtpaGlobals");
     if (pabs == NULL)
       return -1;
-    if (initPortAudio(csound) != 0)
+   if (initPortAudio(csound) != 0)
       goto err_return;
 
     if ((int) Pa_GetDeviceCount() <= 0) {
@@ -262,8 +287,8 @@ static int paBlockingReadWriteOpen(CSOUND *csound)
         pa_PrintErrMsg(csound, Str("Inconsistent full-duplex sample rates"));
         goto err_return;
       }
-      if (((pabs->inParm.bufSamp_SW / csound->ksmps) * csound->ksmps)
-          != pabs->inParm.bufSamp_SW)
+      if (UNLIKELY(((pabs->inParm.bufSamp_SW / csound->GetKsmps(csound)) *
+                    csound->GetKsmps(csound)) != pabs->inParm.bufSamp_SW))
         csound->MessageS(csound,
                          CSOUNDMSG_WARNING,
                          Str("WARNING: buffer size should be an integer "
@@ -306,7 +331,7 @@ static int paBlockingReadWriteOpen(CSOUND *csound)
                         (unsigned long) (pabs->mode & 2 ?
                                          pabs->outParm.bufSamp_SW
                                          : pabs->inParm.bufSamp_SW),
-                        (csound->dither_output ?
+                        (csound->GetDitherMode(csound) ?
                          (PaStreamFlags) paNoFlag : (PaStreamFlags) paDitherOff),
                         paBlockingReadWriteStreamCallback,
                         (void*) pabs);
@@ -535,6 +560,8 @@ static void rtclose_(CSOUND *csound)
     PA_BLOCKING_STREAM *pabs;
     pabs = (PA_BLOCKING_STREAM*) csound->QueryGlobalVariable(csound,
                                                              "_rtpaGlobals");
+
+    csound->Message(csound, Str("closing device\n"));
     if (pabs == NULL)
       return;
 
@@ -542,20 +569,20 @@ static void rtclose_(CSOUND *csound)
 
     if (pabs->paStream != NULL) {
       PaStream  *stream = pabs->paStream;
-      int       i;
-      pabs->paStream = NULL;
-      for (i = 0; i < 4; i++) {
+      unsigned int i;
+
+      for (i = 0; i < 4u; i++) {
 #if NO_FULLDUPLEX_PA_LOCK
         if (!pabs->noPaLock)
 #endif
           csound->NotifyThreadLock(pabs->paLock);
         csound->NotifyThreadLock(pabs->clientLock);
-//        Pa_Sleep(80);
+        //Pa_Sleep(80);
       }
 
       Pa_StopStream(stream);
       Pa_CloseStream(stream);
-//      Pa_Sleep(80);
+      //Pa_Sleep(80);
     }
 
     if (pabs->clientLock != NULL) {
@@ -577,7 +604,7 @@ static void rtclose_(CSOUND *csound)
       free(pabs->inputBuffer);
       pabs->inputBuffer = NULL;
     }
-
+    pabs->paStream = NULL;
     *(csound->GetRtRecordUserData(csound)) = NULL;
     *(csound->GetRtPlayUserData(csound)) = NULL;
     csound->DestroyGlobalVariable(csound, "_rtpaGlobals");
@@ -628,7 +655,7 @@ static int set_device_params(CSOUND *csound, DEVPARAMS *dev,
       err = (int) Pa_OpenStream(&(dev->handle), NULL, &streamParams,
                                 (double) parm->sampleRate,
                                 (unsigned long) parm->bufSamp_SW,
-                                (csound->dither_output?
+                                (csound->GetDitherMode(csound) ?
                                  paNoFlag:paDitherOff),
                                 NULL, NULL);
     }
@@ -747,7 +774,7 @@ static void rtplay_blocking(CSOUND *csound, const MYFLT *outbuf, int nbytes)
 static void rtclose_blocking(CSOUND *csound)
 {
     DEVPARAMS *dev;
-
+    csound->Message(csound, Str("closing device\n"));
     dev = (DEVPARAMS*) (*(csound->GetRtRecordUserData(csound)));
     if (dev != NULL) {
       *(csound->GetRtRecordUserData(csound)) = NULL;
@@ -777,8 +804,8 @@ static void rtclose_blocking(CSOUND *csound)
 PUBLIC int csoundModuleCreate(CSOUND *csound)
 {
     /* nothing to do, report success */
-   csound->Message(csound,
-     Str("PortAudio real-time audio module for Csound\n"));
+  //csound->Message(csound,
+  // Str("PortAudio real-time audio module for Csound\n"));
     return 0;
 }
 
@@ -786,7 +813,8 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
 {
     char    *s, drv[12];
     int     i;
-
+    csound->module_list_add(csound, "pa_bl", "audio");
+    csound->module_list_add(csound, "pa_cb", "audio");
     if ((s = (char*) csound->QueryGlobalVariable(csound, "_RTAUDIO")) == NULL)
       return 0;
     for (i = 0; s[i] != '\0' && i < 11; i++)
@@ -795,7 +823,7 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
     if (!(strcmp(drv, "PORTAUDIO") == 0 || strcmp(drv, "PA") == 0 ||
           strcmp(drv, "PA_BL") == 0 || strcmp(drv, "PA_CB") == 0))
       return 0;
-    csound->Message(csound, Str("rtaudio: PortAudio module enabled ... "));
+    csound->Message(csound, Str("rtaudio: PortAudio module enabled ... \n"));
     /* set function pointers */
 #ifdef LINUX
     if (strcmp(drv, "PA_CB") != 0)
@@ -809,6 +837,7 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
       csound->SetRtplayCallback(csound, rtplay_blocking);
       csound->SetRtrecordCallback(csound, rtrecord_blocking);
       csound->SetRtcloseCallback(csound, rtclose_blocking);
+      csound->SetAudioDeviceListCallback(csound, listDevices);
     }
     else {
       csound->Message(csound, Str("using callback interface\n"));
@@ -817,7 +846,10 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
       csound->SetRtplayCallback(csound, rtplay_);
       csound->SetRtrecordCallback(csound, rtrecord_);
       csound->SetRtcloseCallback(csound, rtclose_);
+      csound->SetAudioDeviceListCallback(csound, listDevices);
     }
+
+    csound->module_list_add(csound, s, "audio");
     return 0;
 }
 
