@@ -9,7 +9,6 @@
     and/or modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
     version 2.1 of the License, or (at your option) any later version.
-
     Csound is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -99,13 +98,17 @@ void PS_MakeGraph(CSOUND *csound, WINDAT *wdptr, const char *name)
     char      pathnam[1024];
     char      *t;
     time_t    lt;
+    OPARMS oparms;
+     csound->GetOParms(csound, &oparms);
+     IGN(wdptr);
+     IGN(name);
 
     if (csound->winEPS_globals != NULL)
       return;
     csound->winEPS_globals = csound->Calloc(csound, sizeof(winEPS_globals_t));
     pp = (winEPS_globals_t *) csound->winEPS_globals;
 
-    filenam = csound->oparms->outfilename;
+    filenam = oparms.outfilename;
     if (filenam == NULL)
       filenam = "test";     /* O.outfilename not set yet */
 
@@ -224,8 +227,8 @@ static void setAxisNumbers(MYFLT *min, MYFLT *max, char *cmin, char *cmax)
      *  Write characters
      */
 
-    sprintf(cmin, "%g", *min);
-    sprintf(cmax, "%g", *max);
+    CS_SPRINTF(cmin, "%g", *min);
+    CS_SPRINTF(cmax, "%g", *max);
 }
 
 static void PS_drawAxes(winEPS_globals_t *pp,
@@ -360,7 +363,7 @@ void PS_DrawGraph(CSOUND *csound, WINDAT *wdptr)
     //xmin = FL(0.0);
     /* xmax = FL(1.0) * wdptr->npts; */
     sprintf(cxmin, "%d", 0);
-    sprintf(cxmax, "%ld", wdptr->npts);
+    sprintf(cxmax, "%ld", (long)wdptr->npts);
 
     ymin = wdptr->min;
     ymax = wdptr->max;
