@@ -69,6 +69,8 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
     int           initFlags = 0;
     int           *fltkFlags;
     int           enableDisplays = 0;
+    OPARMS oparms;
+     csound->GetOParms(csound, &oparms);
 
     if (csound->QueryGlobalVariable(csound,
                                     "FLTK_Flags") == (void*) 0) {
@@ -80,7 +82,7 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
     }
     fltkFlags = getFLTKFlagsPtr(csound);
     if (((*fltkFlags) & 2) == 0 &&
-        !(csound->oparms->graphsoff || csound->oparms->postscript)) {
+        !(oparms.graphsoff || oparms.postscript)) {
 #ifdef LINUX
       Display *dpy = XOpenDisplay(NULL);
       if (dpy != NULL) {
@@ -97,9 +99,6 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
           csound->SetDrawGraphCallback(csound, DrawGraph_FLTK);
           csound->SetKillGraphCallback(csound, KillGraph_FLTK);
           csound->SetExitGraphCallback(csound, ExitGraph_FLTK);
-          csound->SetMakeXYinCallback(csound, MakeXYin_FLTK);
-          csound->SetReadXYinCallback(csound, ReadXYin_FLTK);
-          csound->SetKillXYinCallback(csound, KillXYin_FLTK);
            /* seemed to crash, but not anymore... */
           csound->RegisterResetCallback(csound, NULL, widget_reset);
           csound->Message(csound, "graph init \n");
@@ -118,7 +117,7 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
     if (!((*fltkFlags) & 129))
       for ( ; ep->opname != NULL; ep++) {
         if (csound->AppendOpcode(csound, ep->opname,
-                                 (int)ep->dsblksiz, (int)ep->thread,
+                                 (int)ep->dsblksiz, (int)ep->flags, (int)ep->thread,
                                  ep->outypes, ep->intypes,
                                  ep->iopadr, ep->kopadr, ep->aopadr) != 0) {
           csound->ErrorMsg(csound, Str("Error registering opcode '%s'"),
@@ -129,7 +128,8 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
     else if (!((*fltkFlags) & 128)) {
       for ( ; ep->opname != NULL; ep++) {
         if (csound->AppendOpcode(
-                csound, ep->opname, (int)ep->dsblksiz, (int)ep->thread,
+                                 csound, ep->opname, (int)ep->dsblksiz,
+                                 (int)ep->flags,(int)ep->thread,
                 ep->outypes, ep->intypes,
                 (((int)ep->thread & 1) ? dummyWidgetOpcode : (SUBR) 0),
                 (((int)ep->thread & 2) ? dummyWidgetOpcode : (SUBR) 0),
