@@ -24,7 +24,9 @@
 #include "csoundCore.h"                         /*      WINDOW.C        */
 #include "cwindow.h"                            /*  graph window mgr    */
 #include "winEPS.h"                             /* PostSCript routines  */
-#include "namedins.h"                           /*  dpwe 16may90        */
+                                                /*  dpwe 16may90        */
+
+extern OENTRY* find_opcode_new(CSOUND*, char*, char*, char*);
 
 extern void MakeAscii(CSOUND *, WINDAT *, const char *);
 extern void DrawAscii(CSOUND *, WINDAT *);
@@ -55,38 +57,22 @@ static int DummyFn3(CSOUND *csound)
 
 /* initial proportions */
 
-static void MkXYDummy(CSOUND *csound, XYINDAT *wdptr, MYFLT x, MYFLT y)
-{
-    IGN(csound);
-    wdptr->windid = 0;  /* xwin = MakeWindow(1);        */
-    wdptr->down = 0;    /* by def released after Make */
-
-    wdptr->m_x = 0;
-    wdptr->m_y = 0;
-    wdptr->x = x;
-    wdptr->y = y;
-}
-
-static void RdXYDummy(CSOUND *csound, XYINDAT *wdptr)
-{
-    IGN(csound);
-    IGN(wdptr);
-}
 
 /* called once on initialisation of program to */
 /*  choose between teletype or bitmap graphics */
 
 void dispinit(CSOUND *csound)
 {
-    OPARMS  *O = csound->oparms;
+      OPARMS  O;
+      csound->GetOParms(csound, &O);
 
-    if (O->displays && !(O->graphsoff || O->postscript)) {
+    if (O.displays && !(O.graphsoff || O.postscript)) {
       if (!csound->isGraphable_)
-      find_opcode(csound, "FLrun");       /* load FLTK for displays */
+      find_opcode_new(csound, "FLrun", NULL, NULL); /* load FLTK for displays */
       if (csound->isGraphable_)
         return;         /* provided by window driver: is this session able? */
     }
-    if (!O->displays) {
+    if (!O.displays) {
       csound->Message(csound, Str("displays suppressed\n"));
       csound->csoundMakeGraphCallback_ = DummyFn1;
       csound->csoundDrawGraphCallback_ = DummyFn2;
@@ -94,16 +80,13 @@ void dispinit(CSOUND *csound)
     }
     else {
       csound->Message(csound, Str("graphics %s, ascii substituted\n"),
-                      ((O->graphsoff || O->postscript) ?
+                      ((O.graphsoff || O.postscript) ?
                        Str("suppressed")
                        : Str("not supported on this terminal")));
       csound->csoundMakeGraphCallback_ = MakeAscii;
       csound->csoundDrawGraphCallback_ = DrawAscii;
       csound->csoundKillGraphCallback_ = KillAscii;
     }
-    csound->csoundMakeXYinCallback_ = MkXYDummy;
-    csound->csoundReadXYinCallback_ = RdXYDummy;
-    csound->csoundKillXYinCallback_ = RdXYDummy;
     csound->csoundExitGraphCallback_ = DummyFn3;
 }
 
@@ -115,10 +98,12 @@ void dispset(CSOUND *csound,            /* setup a new window       */
              int    waitflg,
              char   *label)
 {
+    OPARMS  O;
+    csound->GetOParms(csound, &O);
     char *s = caption;
     char *t = wdptr->caption;
     char *tlim = t + CAPSIZE - 1;
-    if (!csound->oparms->displays) return;    /* return if displays disabled */
+    if (!O.displays) return;    /* return if displays disabled */
     wdptr->fdata    = fdata;            /* init remainder of data structure   */
     wdptr->npts     = npts;
     while (*s != '\0' && t < tlim)
@@ -126,7 +111,7 @@ void dispset(CSOUND *csound,            /* setup a new window       */
     *t = '\0';
     if (!wdptr->windid) {   /* if no window defined for this str, create one */
       csound->csoundMakeGraphCallback_(csound, wdptr, label);
-      if (csound->oparms->postscript)
+      if (O.postscript)
         PS_MakeGraph(csound, wdptr, label); /* open PS file + write header    */
     }
 
@@ -141,10 +126,16 @@ void dispset(CSOUND *csound,            /* setup a new window       */
 
 int dispexit(CSOUND *csound)
 {
-    if (csound->oparms->postscript)
+    OPARMS  O;
+    csound->GetOParms(csound, &O);
+    if (O.postscript)
       PS_ExitGraph(csound);     /* Write trailer to PostScript file  */
     /* prompt for exit from last active window */
-    return csound->csoundExitGraphCallback_(csound);
+    int ret = -1;
+    if (csound->csoundExitGraphCallback_) {
+        ret = csound->csoundExitGraphCallback_(csound);
+    }
+    return ret;
 }
 
 void display(CSOUND *csound, WINDAT *wdptr)   /* prepare a MYFLT array, then  */
@@ -153,8 +144,10 @@ void display(CSOUND *csound, WINDAT *wdptr)   /* prepare a MYFLT array, then  */
     MYFLT   *fp, *fplim;
     MYFLT   max, min, absmax, fval;
     int     pol;
+    OPARMS  O;
+    csound->GetOParms(csound, &O);
 
-    if (!csound->oparms->displays)  return;   /* displays disabled? return */
+    if (!O.displays)  return;   /* displays disabled? return */
     fp = wdptr->fdata;
     fplim = fp + wdptr->npts;
     for (max = *fp++, min = max; fp < fplim; ) {  /* find max & min values */
@@ -183,6 +176,6 @@ void display(CSOUND *csound, WINDAT *wdptr)   /* prepare a MYFLT array, then  */
 
 
     /* Write postscript code */
-    if (csound->oparms->postscript)
+    if (O.postscript)
       PS_DrawGraph(csound, wdptr);
 }
