@@ -994,11 +994,7 @@ int cpspch(CSOUND *csound, EVAL *p)
 int cpsmidinn(CSOUND *csound, EVAL *p)
 {
     IGN(csound);
-    /* Convert Midi Note number to 8ve.decimal format */
-    MYFLT oct = (*p->a / FL(12.0)) + FL(MIDINOTE0);
-    /* Lookup in cpsoct table */
-    int32 loct = (int32)(oct * OCTRES);
-    *p->r = (MYFLT)CPSOCTL(loct);
+    *p->r = pow(FL(2.0), (*p->a - FL(69.0)) / FL(12.0)) * FL(440.0);
     return OK;
 }
 
@@ -2058,7 +2054,7 @@ int outRange_i(CSOUND *csound, OUTRANGE *p)
 {
    IGN(csound);
     p->narg = p->INOCOUNT-1;
-   
+
     return OK;
 }
 
@@ -2073,7 +2069,7 @@ int outRange(CSOUND *csound, OUTRANGE *p)
     int startChan = (int) *p->kstartChan -1;
     MYFLT *sp = CS_SPOUT + startChan;
     int narg = p->narg;
-    
+
 
     if (startChan < 0)
       return csound->PerfError(csound, p->h.insdshead,
