@@ -33,7 +33,7 @@ int ipow(CSOUND *csound, POW *p)        /*      Power for i-rate */
     MYFLT in = *p->in;
     MYFLT powerOf = *p->powerOf;
     if (UNLIKELY(in == FL(0.0) && powerOf == FL(0.0)))
-      return csound->PerfError(csound, Str("NaN in pow\n"));
+      return csound->PerfError(csound, p->h.insdshead, Str("NaN in pow\n"));
     else if (p->norm!=NULL && *p->norm != FL(0.0))
       *p->sr = POWER(in, powerOf) / *p->norm;
     else
@@ -43,24 +43,31 @@ int ipow(CSOUND *csound, POW *p)        /*      Power for i-rate */
 
 int apow(CSOUND *csound, POW *p)        /* Power routine for a-rate  */
 {
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *in = p->in, *out = p->sr;
     MYFLT powerOf = *p->powerOf;
     MYFLT norm = (p->norm!=NULL ? *p->norm : FL(1.0));
     if (norm==FL(0.0)) norm = FL(1.0);
+    memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (UNLIKELY(powerOf == FL(0.0))) {
       MYFLT yy = FL(1.0) / norm;
-      for (n = 0; n < nsmps; n++) {
+      for (n = offset; n < nsmps; n++) {
         MYFLT xx = in[n];
         if (UNLIKELY(xx == FL(0.0))) {
-          return csound->PerfError(csound, Str("NaN in pow\n"));
+          return csound->PerfError(csound, p->h.insdshead,Str("NaN in pow\n"));
         }
         else
           out[n] = yy;
       }
     }
     else {
-      for (n = 0; n < nsmps; n++)
+      for (n = offset; n < nsmps; n++)
         out[n] = POWER(in[n], powerOf) / norm;
     }
     return OK;
@@ -267,10 +274,17 @@ static MYFLT poissrand(CSOUND *csound, MYFLT lambda)
 int auniform(CSOUND *csound, PRAND *p)  /* Uniform distribution */
 {
     MYFLT   *out = p->out;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     double  scale = (double)*p->arg1 * (1.0 / 4294967295.03125);
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       out[n] = (MYFLT)((double)csoundRandMT(&(csound->randState_)) * scale);
     }
     return OK;
@@ -284,13 +298,21 @@ int ikuniform(CSOUND *csound, PRAND *p)
 
 int alinear(CSOUND *csound, PRAND *p)   /* Linear random functions      */
 {
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out = p->out;
     MYFLT arg1 = *p->arg1;
 
-    for (n = 0; n < nsmps; n++)
+    memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++)
       out[n] = linrand(csound, arg1);
     return OK;
+
 }
 
 int iklinear(CSOUND *csound, PRAND *p)
@@ -301,11 +323,18 @@ int iklinear(CSOUND *csound, PRAND *p)
 
 int atrian(CSOUND *csound, PRAND *p)    /* Triangle random functions  */
 {
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out = p->out;
     MYFLT arg1 = *p->arg1;
 
-    for (n = 0; n < nsmps; n++)
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++)
       out[n] = trirand(csound, arg1);
     return OK;
 }
@@ -331,7 +360,7 @@ int kexprndi(CSOUND *csound, PRANDI *p)
 {                                       /* rslt = (num1 + diff*phs) * amp */
     /* IV - Jul 11 2002 */
     *p->ar = (p->num1 + (MYFLT)p->phs * p->dfdmax) * *p->xamp;
-    p->phs += (int32)(*p->xcps * csound->kicvt); /* phs += inc           */
+    p->phs += (int32)(*p->xcps * CS_KICVT); /* phs += inc           */
     if (UNLIKELY(p->phs >= MAXLEN)) {         /* when phs overflows,  */
       p->phs &= PHMASK;                       /*      mod the phs     */
       p->num1 = p->num2;                      /*      & new num vals  */
@@ -350,21 +379,29 @@ int iexprndi(CSOUND *csound, PRANDI *p)
 int aexprndi(CSOUND *csound, PRANDI *p)
 {
     int32       phs = p->phs, inc;
-    int         n, nn = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       *ar, *ampp, *cpsp;
 
     cpsp = p->xcps;
     ampp = p->xamp;
     ar = p->ar;
-    inc = (int32)(*cpsp++ * csound->sicvt);
-    for (n=0;n<nn;n++) {
+    inc = (int32)(cpsp[0] * csound->sicvt);
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       /* IV - Jul 11 2002 */
-      ar[n] = (p->num1 + (MYFLT)phs * p->dfdmax) * *ampp;
       if (p->ampcod)
-        ampp++;
+        ar[n] = (p->num1 + (MYFLT)phs * p->dfdmax) * ampp[n];
+      else
+        ar[n] = (p->num1 + (MYFLT)phs * p->dfdmax) * ampp[0];
       phs += inc;                                /* phs += inc       */
       if (p->cpscod)
-        inc = (int32)(*cpsp++ * csound->sicvt);  /*   (nxt inc)      */
+        inc = (int32)(cpsp[n] * csound->sicvt);  /*   (nxt inc)      */
       if (UNLIKELY(phs >= MAXLEN)) {             /* when phs o'flows */
         phs &= PHMASK;
         p->num1 = p->num2;
@@ -378,11 +415,18 @@ int aexprndi(CSOUND *csound, PRANDI *p)
 
 int aexp(CSOUND *csound, PRAND *p)      /* Exponential random functions */
 {
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out = p->out;
     MYFLT arg1 = *p->arg1;
 
-    for (n = 0; n < nsmps; n++)
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++)
       out[n] = exprand(csound, arg1);
     return OK;
 }
@@ -395,11 +439,18 @@ int ikexp(CSOUND *csound, PRAND *p)
 
 int abiexp(CSOUND *csound, PRAND *p)    /* Bilateral exponential rand */
 {                                       /* functions */
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out = p->out;
     MYFLT arg1 = *p->arg1;
 
-    for (n = 0; n < nsmps; n++)
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++)
       out[n] = biexprand(csound, arg1);
     return OK;
 }
@@ -412,11 +463,18 @@ int ikbiexp(CSOUND *csound, PRAND *p)
 
 int agaus(CSOUND *csound, PRAND *p)     /* Gaussian random functions */
 {
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out = p->out;
     MYFLT arg1 = *p->arg1;
 
-    for (n = 0; n < nsmps; n++)
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++)
       out[n] = gaussrand(csound, arg1);
     return OK;
 }
@@ -429,11 +487,18 @@ int ikgaus(CSOUND *csound, PRAND *p)
 
 int acauchy(CSOUND *csound, PRAND *p)   /* Cauchy random functions */
 {
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out = p->out;
     MYFLT arg1 = *p->arg1;
 
-    for (n = 0; n < nsmps; n++)
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++)
       out[n] = cauchrand(csound, arg1);
     return OK;
 }
@@ -453,7 +518,7 @@ int kgaussi(CSOUND *csound, PRANDI *p)
 {                                       /* rslt = (num1 + diff*phs) * amp */
     /* IV - Jul 11 2002 */
     *p->ar = (p->num1 + (MYFLT)p->phs * p->dfdmax) * *p->xamp;
-    p->phs += (int32)(*p->xcps * csound->kicvt); /* phs += inc           */
+    p->phs += (int32)(*p->xcps * CS_KICVT); /* phs += inc           */
     if (UNLIKELY(p->phs >= MAXLEN)) {           /* when phs overflows,  */
       p->phs &= PHMASK;                         /*      mod the phs     */
       p->num1 = p->num2;                        /*      & new num vals  */
@@ -472,21 +537,29 @@ int igaussi(CSOUND *csound, PRANDI *p)
 int agaussi(CSOUND *csound, PRANDI *p)
 {
     int32       phs = p->phs, inc;
-    int         n, nn = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       *ar, *ampp, *cpsp;
 
     cpsp = p->xcps;
     ampp = p->xamp;
     ar = p->ar;
-    inc = (int32)(*cpsp++ * csound->sicvt);
-    for (n=0;n<nn;n++) {
+    inc = (int32)(*cpsp * csound->sicvt);
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       /* IV - Jul 11 2002 */
-      ar[n] = (p->num1 + (MYFLT)phs * p->dfdmax) * *ampp;
       if (p->ampcod)
-        ampp++;
-      phs += inc;                               /* phs += inc       */
+        ar[n] = (p->num1 + (MYFLT)phs * p->dfdmax) * ampp[n];
+      else
+        ar[n] = (p->num1 + (MYFLT)phs * p->dfdmax) * ampp[0];
+      phs += inc;                                /* phs += inc       */
       if (p->cpscod)
-        inc = (int32)(*cpsp++ * csound->sicvt);  /*   (nxt inc)      */
+        inc = (int32)(cpsp[n] * csound->sicvt);  /*   (nxt inc)      */
       if (UNLIKELY(phs >= MAXLEN)) {             /* when phs o'flows */
         phs &= PHMASK;
         p->num1 = p->num2;
@@ -506,11 +579,18 @@ int ikcauchy(CSOUND *csound, PRAND *p)
 
 int apcauchy(CSOUND *csound, PRAND *p)  /* +ve Cauchy random functions */
 {
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out = p->out;
     MYFLT arg1 = *p->arg1;
 
-    for (n = 0; n < nsmps; n++)
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++)
       out[n] = pcauchrand(csound, arg1);
     return OK;
 }
@@ -536,7 +616,7 @@ int kcauchyi(CSOUND *csound, PRANDI *p)
 {                                       /* rslt = (num1 + diff*phs) * amp */
     /* IV - Jul 11 2002 */
     *p->ar = (p->num1 + (MYFLT)p->phs * p->dfdmax) * *p->xamp;
-    p->phs += (int32)(*p->xcps * csound->kicvt); /* phs += inc           */
+    p->phs += (int32)(*p->xcps * CS_KICVT); /* phs += inc           */
     if (UNLIKELY(p->phs >= MAXLEN)) {         /* when phs overflows,  */
       p->phs &= PHMASK;                       /*      mod the phs     */
       p->num1 = p->num2;                      /*      & new num vals  */
@@ -555,21 +635,29 @@ int icauchyi(CSOUND *csound, PRANDI *p)
 int acauchyi(CSOUND *csound, PRANDI *p)
 {
     int32       phs = p->phs, inc;
-    int         n, nn = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       *ar, *ampp, *cpsp;
 
     cpsp = p->xcps;
     ampp = p->xamp;
     ar = p->ar;
-    inc = (int32)(*cpsp++ * csound->sicvt);
-    for (n=0;n<nn;n++) {
+    inc = (int32)(*cpsp * csound->sicvt);
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       /* IV - Jul 11 2002 */
-      ar[n] = (p->num1 + (MYFLT)phs * p->dfdmax) * *ampp;
       if (p->ampcod)
-        ampp++;
+        ar[n] = (p->num1 + (MYFLT)phs * p->dfdmax) * ampp[n];
+      else
+        ar[n] = (p->num1 + (MYFLT)phs * p->dfdmax) * ampp[0];
       phs += inc;                                /* phs += inc       */
       if (p->cpscod)
-        inc = (int32)(*cpsp++ * csound->sicvt);  /*   (nxt inc)      */
+        inc = (int32)(cpsp[n] * csound->sicvt);  /*   (nxt inc)      */
       if (UNLIKELY(phs >= MAXLEN)) {             /* when phs o'flows */
         phs &= PHMASK;
         p->num1 = p->num2;
@@ -583,12 +671,19 @@ int acauchyi(CSOUND *csound, PRANDI *p)
 
 int abeta(CSOUND *csound, PRAND *p)     /* Beta random functions   */
 {
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out = p->out;
     MYFLT arg1 = *p->arg1;
     MYFLT arg2 = *p->arg2;
     MYFLT arg3 = *p->arg3;
 
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
     for (n = 0; n < nsmps; n++)
       out[n] = betarand(csound, arg1, arg2, arg3);
     return OK;
@@ -602,12 +697,19 @@ int ikbeta(CSOUND *csound, PRAND *p)
 
 int aweib(CSOUND *csound, PRAND *p)     /* Weibull randon functions */
 {
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out = p->out;
     MYFLT arg1 = *p->arg1;
     MYFLT arg2 = *p->arg2;
 
-    for (n = 0; n < nsmps; n++)
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++)
       out[n] = weibrand(csound, arg1, arg2);
     return OK;
 }
@@ -620,11 +722,18 @@ int ikweib(CSOUND *csound, PRAND *p)
 
 int apoiss(CSOUND *csound, PRAND *p)    /*      Poisson random funcions */
 {
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *out = p->out;
     MYFLT arg1 = *p->arg1;
 
-    for (n = 0; n < nsmps; n++)
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++)
       out[n] = poissrand(csound, arg1);
     return OK;
 }
