@@ -32,11 +32,6 @@
 #include "csound.h"
 #include "fftlib.h"
 
-#define MYRECIPLN2  1.442695040888963407359924681001892137426 /* 1.0/log(2) */
-
-/* some useful conversions between a number and its power of 2 */
-
-/* #define LOG2(a) (MYRECIPLN2*log(a))       /* floating point logarithm base 2 */
 #define POW2(m) ((uint32) 1 << (m))       /* integer power of 2 for m<32 */
 
 /* fft's with M bigger than this bust primary cache */
@@ -3192,7 +3187,7 @@ static inline int ConvertFFTSize(CSOUND *csound, int N)
       case 0x08000000:  return 27;
       case 0x10000000:  return 28;
     }
-    csoundDie(csound, Str(" *** fftlib.c: internal error: "
+    csound->Warning(csound, Str(" *** fftlib.c: internal error: "
                           "invalid FFT size: %d"), N);
     return 0;
 }
@@ -3213,6 +3208,8 @@ static inline void getTablePointers(CSOUND *p, MYFLT **ct, int16 **bt,
 
 MYFLT csoundGetInverseComplexFFTScale(CSOUND *csound, int FFTsize)
 {
+    IGN(FFTsize);
+    IGN(csound);
     return FL(1.0);
 }
 
@@ -3223,6 +3220,8 @@ MYFLT csoundGetInverseComplexFFTScale(CSOUND *csound, int FFTsize)
 
 MYFLT csoundGetInverseRealFFTScale(CSOUND *csound, int FFTsize)
 {
+    IGN(FFTsize);
+    IGN(csound);
     return FL(1.0);
 }
 
@@ -3317,6 +3316,7 @@ void csoundRealFFTMult(CSOUND *csound, MYFLT *outbuf,
 {
     MYFLT re, im;
     int   i;
+    IGN(csound);
 
     if (scaleFac != FL(1.0)) {
       outbuf[0] = buf1[0] * buf2[0] * scaleFac;
