@@ -53,17 +53,34 @@ int midibset(CSOUND*, MIDIKMB*);
 
 /* IV - Oct 31 2002: modified to allow named instruments */
 
-int massign(CSOUND *csound, MASSIGN *p)
+int massign_p(CSOUND *csound, MASSIGN *p)
+{
+    int   chnl = (int)(*p->chnl + FL(0.5));
+    int   resetCtls;
+    int   retval = OK;
+
+    resetCtls = (*p->iresetctls == FL(0.0) ? 0 : 1);
+    if (--chnl >= 0)
+      retval = m_chinsno(csound, chnl, (int) *p->insno, resetCtls);
+    else {
+      for (chnl = 0; chnl < 16; chnl++) {
+        if (m_chinsno(csound, chnl, (int) *p->insno, resetCtls) != OK)
+          retval = NOTOK;
+      }
+    }
+    return retval;
+}
+
+int massign_S(CSOUND *csound, MASSIGNS *p)
 {
     int   chnl = (int)(*p->chnl + FL(0.5));
     int32  instno = 0L;
     int   resetCtls;
     int   retval = OK;
 
-    if (p->XSTRCODE || *(p->insno) >= FL(0.5)) {
-      if (UNLIKELY((instno = strarg2insno(csound, p->insno, p->XSTRCODE)) <= 0L))
+   if (UNLIKELY((instno = strarg2insno(csound, p->insno->data, 1)) <= 0L))
         return NOTOK;
-    }
+
     resetCtls = (*p->iresetctls == FL(0.0) ? 0 : 1);
     if (--chnl >= 0)
       retval = m_chinsno(csound, chnl, (int) instno, resetCtls);
@@ -76,6 +93,7 @@ int massign(CSOUND *csound, MASSIGN *p)
     return retval;
 }
 
+
 int ctrlinit(CSOUND *csound, CTLINIT *p)
 {
     int16 chnl = (int16)(*p->chnl - FL(0.5));
@@ -116,7 +134,7 @@ int cpstmid(CSOUND *csound, CPSTABLE *p)
     int basekeymidi;
     MYFLT basefreq, factor, interval;
 
-    if (UNLIKELY((ftp = csound->FTFind(csound, p->tablenum)) == NULL)) {
+    if (UNLIKELY((ftp = csound->FTnp2Find(csound, p->tablenum)) == NULL)) {
       return csound->InitError(csound, Str("cpstabm: invalid modulator table"));
     }
     func = ftp->ftable;
@@ -148,6 +166,7 @@ int veloc(CSOUND *csound, MIDIMAP *p)           /* valid only at I-time */
 
 int pchmidi(CSOUND *csound, MIDIKMB *p)
 {
+   IGN(csound);
     INSDS *lcurip = p->h.insdshead;
     double fract, oct, ioct;
     oct = lcurip->m_pitch / 12.0 + 3.0;
@@ -179,6 +198,7 @@ int pchmidib_i(CSOUND *csound, MIDIKMB *p)
 
 int octmidi(CSOUND *csound, MIDIKMB *p)
 {
+    IGN(csound);
     INSDS *lcurip = p->h.insdshead;
     *p->r = lcurip->m_pitch / FL(12.0) + FL(3.0);
     return OK;
@@ -186,6 +206,7 @@ int octmidi(CSOUND *csound, MIDIKMB *p)
 
 int octmidib(CSOUND *csound, MIDIKMB *p)
 {
+    IGN(csound);
     INSDS *lcurip = p->h.insdshead;
     *p->r = (lcurip->m_pitch + (pitchbend_value(lcurip->m_chnbp) *
                                 p->scale)) / FL(12.0) + FL(3.0);
@@ -254,7 +275,7 @@ int ampmidi(CSOUND *csound, MIDIAMP *p)   /* convert midi veloc to amplitude */
 
     amp = csound->curip->m_veloc / FL(128.0);     /* amp = normalised veloc */
     if ((fno = (int32)*p->ifn) > 0) {              /* if valid ftable,       */
-      if (UNLIKELY((ftp = csound->FTFind(csound, p->ifn)) == NULL))
+      if (UNLIKELY((ftp = csound->FTnp2Find(csound, p->ifn)) == NULL))
         return NOTOK;                             /*     use amp as index   */
       amp = *(ftp->ftable + (int32)(amp * ftp->flen));
     }
@@ -267,7 +288,7 @@ int ampmidi(CSOUND *csound, MIDIAMP *p)   /* convert midi veloc to amplitude */
 int midibset(CSOUND *csound, MIDIKMB *p)
 {
     MCHNBLK *chn;
-
+    IGN(csound);
     chn = p->h.insdshead->m_chnbp;
     if (*p->iscal > FL(0.0))
       p->scale = *p->iscal;
@@ -285,6 +306,7 @@ int midibset(CSOUND *csound, MIDIKMB *p)
 
 int aftset(CSOUND *csound, MIDIKMAP *p)
 {
+   IGN(csound);
     p->lo = *p->ilo;
     p->scale = (*p->ihi - p->lo) * dv127;
     return OK;
@@ -292,6 +314,7 @@ int aftset(CSOUND *csound, MIDIKMAP *p)
 
 int aftouch(CSOUND *csound, MIDIKMAP *p)
 {
+   IGN(csound);
     INSDS *lcurip = p->h.insdshead;
     *p->r = p->lo + MIDI_VALUE(lcurip->m_chnbp, aftouch) * p->scale;
     return OK;
@@ -322,6 +345,7 @@ int mctlset(CSOUND *csound, MIDICTL *p)
 
 int midictl(CSOUND *csound, MIDICTL *p)
 {
+    IGN(csound);
     INSDS *lcurip = p->h.insdshead;
     *p->r = MIDI_VALUE(lcurip->m_chnbp, ctl_val[p->ctlno]) * p->scale + p->lo;
     return OK;
@@ -352,6 +376,7 @@ int maftset(CSOUND *csound, MIDICTL *p)
 
 int midiaft(CSOUND *csound, MIDICTL *p)
 {
+    IGN(csound);
     INSDS *lcurip = p->h.insdshead;
     *p->r = MIDI_VALUE(lcurip->m_chnbp, polyaft[p->ctlno]) * p->scale + p->lo;
     return OK;
@@ -368,7 +393,7 @@ int midichn(CSOUND *csound, MIDICHN *p)
 
 /* pgmassign - assign MIDI program to instrument */
 
-int pgmassign(CSOUND *csound, PGMASSIGN *p)
+int pgmassign_(CSOUND *csound, PGMASSIGN *p, int instname)
 {
     int pgm, ins, chn;
 
@@ -376,7 +401,7 @@ int pgmassign(CSOUND *csound, PGMASSIGN *p)
     if (UNLIKELY(chn < 0 || chn > 16))
       return csound->InitError(csound, Str("illegal channel number"));
     /* IV - Oct 31 2002: allow named instruments */
-    if (p->XSTRCODE || *p->inst == SSTRCOD) {
+    if (instname || ISSTRCOD(*p->inst)) {
       MYFLT buf[128];
       csound->strarg2name(csound, (char*) buf, p->inst, "", 1);
       ins = (int)strarg2insno(csound, buf, 1);
@@ -412,6 +437,14 @@ int pgmassign(CSOUND *csound, PGMASSIGN *p)
     return OK;
 }
 
+int pgmassign_S(CSOUND *csound, PGMASSIGN *p){
+  return pgmassign_(csound,p,1);
+}
+
+int pgmassign(CSOUND *csound, PGMASSIGN *p){
+  return pgmassign_(csound,p,0);
+}
+
 int ichanctl(CSOUND *csound, CHANCTL *p)
 {
     int32  ctlno, chan = (int32)(*p->ichano - FL(1.0));
@@ -450,6 +483,7 @@ int chanctl(CSOUND *csound, CHANCTL *p)
 
 int ipchbend(CSOUND *csound, MIDIMAP *p)
 {
+    IGN(csound);
     *p->r = *p->ilo + (*p->ihi - *p->ilo) *
             pitchbend_value(p->h.insdshead->m_chnbp);
     return OK;
@@ -457,6 +491,7 @@ int ipchbend(CSOUND *csound, MIDIMAP *p)
 
 int kbndset(CSOUND *csound, MIDIKMAP *p)
 {
+   IGN(csound);
     p->lo = *p->ilo;
     p->scale = *p->ihi - *p->ilo;
     return OK;
@@ -464,6 +499,7 @@ int kbndset(CSOUND *csound, MIDIKMAP *p)
 
 int kpchbend(CSOUND *csound, MIDIKMAP *p)
 {
+   IGN(csound);
     INSDS *lcurip = p->h.insdshead;
     *p->r = p->lo + pitchbend_value(lcurip->m_chnbp) * p->scale;
     return OK;
