@@ -69,7 +69,7 @@ int release(CSOUND *csound, REL *p)
 int xtratim(CSOUND *csound, XTRADUR *p)
 {
     int *xtra = &(p->h.insdshead->xtratim);
-    int tim = (int)(*p->extradur * csound->ekr);
+    int tim = (int)(*p->extradur * p->h.insdshead->ekr);
     if (*xtra < tim)  /* gab-a5 revised */
       *xtra = tim;
     return OK;
@@ -77,7 +77,7 @@ int xtratim(CSOUND *csound, XTRADUR *p)
 
 int mclock_set(CSOUND *csound, MCLOCK *p)
 {
-    p->period= csound->ekr / *p->freq;
+    p->period= CS_EKR / *p->freq;
     p->clock_tics = p->period;
     p->beginning_flag = TRUE;
     return OK;
