@@ -168,8 +168,9 @@ static void fft_(CSOUND *csound, MYFLT *a, MYFLT *b,
     if (m <= kt+1)
       maxp = m + kt + 1;
     if (UNLIKELY(m+kt > 15)) {
-      csound->Die(csound, Str("\nerror - fft parameter n has "
+      csound->Warning(csound, Str("\nerror - fft parameter n has "
                               "more than 15 factors : %d"), n);
+      return;
     }
     if (kt!=0) {
       j = kt;
@@ -932,8 +933,10 @@ void csoundRealFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize)
       buf[FFTsize] = buf[1];
     }
     else {
-      if (UNLIKELY(FFTsize < 2 || (FFTsize & 1)))
-        csoundDie(csound, Str("csoundRealFFTnp2(): invalid FFT size"));
+      if (UNLIKELY(FFTsize < 2 || (FFTsize & 1))) {
+        csound->Warning(csound, Str("csoundRealFFTnp2(): invalid FFT size"));
+        return;
+      }
       buf[FFTsize] = buf[FFTsize + 1] = FL(0.0);
       fft_(csound, buf, &(buf[1]), 1, (FFTsize >> 1), 1, -2);
       reals_(csound, buf, &(buf[1]), (FFTsize >> 1), -2);
@@ -953,11 +956,30 @@ void csoundRealFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize)
  */
 void csoundInverseRealFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize)
 {
-    if (UNLIKELY(FFTsize < 2 || (FFTsize & 1)))
-      csoundDie(csound, Str("csoundInverseRealFFTnp2(): invalid FFT size"));
+  if (UNLIKELY(FFTsize < 2 || (FFTsize & 1))){
+      csound->Warning(csound, Str("csoundInverseRealFFTnp2(): invalid FFT size"));
+      return;
+  }
     buf[1] = buf[FFTsize + 1] = FL(0.0);
     reals_(csound, buf, &(buf[1]), (FFTsize >> 1), 2);
     fft_(csound, buf, &(buf[1]), 1, (FFTsize >> 1), 1, 2);
     buf[FFTsize] = buf[FFTsize + 1] = FL(0.0);
 }
 
+void csoundInverseComplexFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize)
+{
+  if (UNLIKELY(FFTsize < 2 || (FFTsize & 1))){
+      csound->Warning(csound, Str("csoundInverseRealFFTnp2(): invalid FFT size"));
+      return;
+  }
+    fft_(csound, buf, buf, 1, FFTsize, 1, 2);
+}
+
+void csoundComplexFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize)
+{
+      if (UNLIKELY(FFTsize < 2 || (FFTsize & 1))) {
+        csound->Warning(csound, Str("csoundRealFFTnp2(): invalid FFT size"));
+        return;
+      }
+      fft_(csound, buf, buf, 1, FFTsize, 1, -2);
+}
