@@ -69,7 +69,7 @@ int oscils_set(CSOUND *csound, OSCILS *p)
     int     iflg;
 
     iflg = (int) (*(p->iflg) + FL(0.5)) & 0x07; /* check flags */
-    if (iflg & 1) return OK;                    /* skip init, nothing to do */
+    if (UNLIKELY(iflg & 1)) return OK;          /* skip init, nothing to do */
     p->use_double = (iflg & 2 ? 1 : 0);         /* use doubles internally */
     init_sine_gen((double)*(p->iamp), (double)(*(p->icps) * csound->tpidsr),
                   (double)(*(p->iphs) * TWOPI_F),
@@ -86,16 +86,23 @@ int oscils_set(CSOUND *csound, OSCILS *p)
 
 int oscils(CSOUND *csound, OSCILS *p)
 {
-    int     n, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   *ar, x, c, v;
     double  xd, cd, vd;
 
     /* copy object data to local variables */
     ar = p->ar;
 
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->use_double) {            /* use doubles */
       xd = p->xd; cd = p->cd; vd = p->vd;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         ar[n] = (MYFLT) xd;
         vd += cd * xd;
         xd += vd;
@@ -104,7 +111,7 @@ int oscils(CSOUND *csound, OSCILS *p)
     }
     else {                          /* use floats */
       x = p->x; c = p->c; v = p->v;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         ar[n] = x;
         v += c * x;
         x += v;
@@ -118,14 +125,15 @@ int oscils(CSOUND *csound, OSCILS *p)
 
 int lphasor_set(CSOUND *csound, LPHASOR *p)
 {
-    if (*(p->istor) != FL(0.0)) return OK;               /* nothing to do */
+    IGN(csound);
+    if (UNLIKELY(*(p->istor) != FL(0.0))) return OK;       /* nothing to do */
 
     p->phs = (double)*(p->istrt);                          /* start phase */
     p->lps = (double)*(p->ilps);                           /* loop start */
     p->lpe = (double)*(p->ilpe);                           /* loop end */
-    p->loop_mode = (int) (*(p->imode) + FL(0.5)) & 0x03;    /* loop mode */
-    if (p->lpe <= p->lps) p->loop_mode = 0;                 /* disable loop */
-    p->dir = 1;                                             /* direction */
+    p->loop_mode = (int) (*(p->imode) + FL(0.5)) & 0x03;   /* loop mode */
+    if (p->lpe <= p->lps) p->loop_mode = 0;                /* disable loop */
+    p->dir = 1;                                            /* direction */
     return OK;
 }
 
@@ -133,7 +141,10 @@ int lphasor_set(CSOUND *csound, LPHASOR *p)
 
 int lphasor(CSOUND *csound, LPHASOR *p)
 {
-    int     n, nsmps=csound->ksmps, loop_mode, dir;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int loop_mode, dir;
     MYFLT   *ar, *xtrns;
     double  trns, phs, lps, lpe, lpt;
 
@@ -144,8 +155,13 @@ int lphasor(CSOUND *csound, LPHASOR *p)
     loop_mode = p->loop_mode;
     trns = (double)*xtrns;
 
-    for (n=0; n<nsmps; n++) {
-      if (XINARG1) trns = (double)*(xtrns++);
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
+      if (XINARG1) trns = (double)xtrns[n];
       ar[n] = (MYFLT) phs;
       phs += (p->dir ? trns : -trns);
       if (loop_mode) {
@@ -175,8 +191,9 @@ int lphasor(CSOUND *csound, LPHASOR *p)
 
 int tablexkt_set(CSOUND *csound, TABLEXKT *p)
 {
+   IGN(csound);
     p->wsize = (int)(*(p->iwsize) + 0.5);                  /* window size */
-    if (p->wsize < 3) {
+    if (UNLIKELY(p->wsize < 3)) {
       p->wsize = 2;
     }
     else {
@@ -199,7 +216,10 @@ int tablexkt_set(CSOUND *csound, TABLEXKT *p)
 
 int tablexkt(CSOUND *csound, TABLEXKT *p)
 {
-    int     i, n, nsmps=csound->ksmps, wsize, wsized2, wrap_ndx, warp;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int i, wsize, wsized2, wrap_ndx, warp;
     double  ndx, d, x, c, v, flen_d, onedpi_d, pidwarp_d;
     int32    ndx_i, flen;
     MYFLT   *ar, *xndx, ndx_f, a0, a1, a2, a3, v0, v1, v2, v3, *ftable;
@@ -209,7 +229,8 @@ int tablexkt(CSOUND *csound, TABLEXKT *p)
     /* window size */
     wsize = p->wsize;
     if (UNLIKELY((wsize < 2) || (wsize > 1024))) {
-      return csound->PerfError(csound, Str("tablexkt: not initialised"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("tablexkt: not initialised"));
     }
     wsized2 = wsize >> 1;
 
@@ -224,7 +245,7 @@ int tablexkt(CSOUND *csound, TABLEXKT *p)
     ar = p->ar;
     xndx = p->xndx;
     wrap_ndx = p->wrap_ndx;
-    if ((wsize > 4) && (*(p->kwarp) > FL(1.001))) {
+    if ((wsize > 4) && UNLIKELY((*(p->kwarp) > FL(1.001)))) {
       warp = 1;                     /* enable warp */
       onedwarp = FL(1.0) / *(p->kwarp);
       pidwarp_d = PI / (double)*(p->kwarp);
@@ -239,7 +260,12 @@ int tablexkt(CSOUND *csound, TABLEXKT *p)
     }
     onedpi_d = 1.0 / PI;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       ndx = (double)*xndx;
       if (XINARG1) xndx++;
       /* calculate table index */
@@ -259,10 +285,10 @@ int tablexkt(CSOUND *csound, TABLEXKT *p)
         while (ndx_i < 0) ndx_i += flen;
       }
       else {                        /* limit */
-        if (ndx_i < 0) {
+        if (UNLIKELY(ndx_i < 0)) {
           ndx_i = 0; ndx_f = FL(0.0);
         }
-        else if (ndx_i >= flen) {
+        else if (UNLIKELY(ndx_i >= flen)) {
           ndx_i = flen; ndx_f = FL(0.0);
         }
       }
@@ -383,4 +409,3 @@ int tablexkt(CSOUND *csound, TABLEXKT *p)
     }
     return OK;
 }
-
