@@ -29,7 +29,6 @@
 #include "csoundCore.h"
 #include "pstream.h"
 #include "pvfileio.h"
-#include "oload.h"
 
 #ifdef _DEBUG
 #include <assert.h>
@@ -52,12 +51,38 @@ int fsigs_equal(const PVSDAT *f1, const PVSDAT *f2)
 /* Pandora's box opcode, but we can make it at least plausible,
    by forbidding copy to different format. */
 
+int fassign_set(CSOUND *csound, FASSIGN *p)
+{
+     int32 N = p->fsrc->N;
+
+    p->fout->N =  N;
+    p->fout->overlap = p->fsrc->overlap;
+    p->fout->winsize = p->fsrc->winsize;
+    p->fout->wintype =p->fsrc->wintype;
+    p->fout->format = p->fout->format;
+    p->fout->sliding = p->fsrc->sliding;
+    /* sliding needs to be checked */
+    if (p->fsrc->sliding) {
+      p->fout->NB = p->fsrc->NB;
+      csound->AuxAlloc(csound, (N + 2) * sizeof(MYFLT) * csound->ksmps,
+                       &p->fout->frame);
+      return OK;
+    }
+    csound->AuxAlloc(csound, (N + 2) * sizeof(float), &p->fout->frame);
+    p->fout->framecount = 1;
+    //csound->Message(csound, Str("fsig = : init\n"));
+    return OK;
+}
+
+
 int fassign(CSOUND *csound, FASSIGN *p)
 {
     int32 framesize;
     float *fout,*fsrc;
-    if (UNLIKELY(!fsigs_equal(p->fout,p->fsrc)))
-      csound->Die(csound, Str("fsig = : formats are different.\n"));
+
+    // if (UNLIKELY(!fsigs_equal(p->fout,p->fsrc)))
+    //return csound->PerfError(csound,p->h.insdshead,
+    //                         Str("fsig = : formats are different.\n"));
     if (p->fsrc->sliding) {
       memcpy(p->fout->frame.auxp, p->fsrc->frame.auxp,
              sizeof(MYFLT)*(p->fsrc->N+2)*csound->ksmps);
@@ -67,8 +92,12 @@ int fassign(CSOUND *csound, FASSIGN *p)
     fsrc = (float *) p->fsrc->frame.auxp;
 
     framesize = p->fsrc->N + 2;
-    if (p->fout->framecount == p->fsrc->framecount) /* avoid duplicate copying*/
-      memcpy(fout, fsrc, framesize*sizeof(float));
+
+    if (p->fout->framecount == p->fsrc->framecount) {/* avoid duplicate copying*/
+    memcpy(fout, fsrc, framesize*sizeof(float));
+    p->fout->framecount++;
+    }
+
      return OK;
 }
 
@@ -79,27 +108,29 @@ int pvadsynset(CSOUND *csound, PVADS *p)
     /* get params from input fsig */
     /* we trust they are legit! */
     int i;
-    int32 N = p->fsig->N;
+    PVSDAT  *fs = p->fsig;
+    int32 N = fs->N;
     int32 noscs,n_oscs;
     int32 startbin,binoffset;
     MYFLT *p_x;
 
-    if (UNLIKELY(p->fsig->sliding))
-      csound->InitError(csound, Str("Sliding version not yet available"));
-    p->overlap = p->fsig->overlap;
+    if (UNLIKELY(fs->sliding))
+      return csound->InitError(csound, Str("Sliding version not yet available"));
+    p->overlap = fs->overlap;
     /* a moot question, whether window params are relevant for adsyn?*/
     /* but for now, we validate all params in fsig */
-    p->winsize = p->fsig->winsize;
-    p->wintype = p->fsig->wintype;
+    p->winsize = fs->winsize;
+    p->wintype = fs->wintype;
     p->fftsize = N;
     noscs   = N/2 + 1;                     /* max possible */
     n_oscs = (int32) *p->n_oscs;    /* user param*/
     if (UNLIKELY(n_oscs <=0))
-      csound->Die(csound, Str("pvadsyn: bad value for inoscs\n"));
+      csound->InitError(csound, Str("pvadsyn: bad value for inoscs\n"));
     /* remove this when I know how to do it... */
-    if (UNLIKELY(p->fsig->format != PVS_AMP_FREQ))
-      csound->Die(csound, Str("pvadsyn: format must be amp-freq (0).\n"));
-    p->format = p->fsig->format;
+    if (UNLIKELY(fs->format != PVS_AMP_FREQ))
+      return csound->InitError(csound,
+                               Str("pvadsyn: format must be amp-freq (0).\n"));
+    p->format = fs->format;
     /* interesting question: could noscs be krate variable?
      * answer: yes, but adds complexity to oscbank,
                as have to set/reset each osc when started and stopped */
@@ -108,13 +139,15 @@ int pvadsynset(CSOUND *csound, PVADS *p)
     startbin = (int32) *p->ibin;            /* default 0 */
     binoffset = (int32) *p->ibinoffset; /* default 1 */
     if (UNLIKELY(startbin < 0 || startbin > noscs))
-      csound->Die(csound, Str("pvsadsyn: ibin parameter out of range.\n"));
+      return csound->InitError(csound,
+                               Str("pvsadsyn: ibin parameter out of range.\n"));
     if (UNLIKELY(startbin + n_oscs > noscs))
-      csound->Die(csound, Str("pvsadsyn: ibin + inoscs too large.\n"));
+      return csound->InitError(csound,
+                               Str("pvsadsyn: ibin + inoscs too large.\n"));
     /* calc final max bin target */
     p->maxosc = startbin + (n_oscs * binoffset);
     if (UNLIKELY(p->maxosc > noscs))
-      csound->Die(csound, Str("pvsadsyn: "
+      return csound->InitError(csound, Str("pvsadsyn: "
                               "ibin + (inoscs * ibinoffset) too large."));
 
     p->outptr = 0;
@@ -170,8 +203,8 @@ static void adsyn_frame(CSOUND *csound, PVADS *p)
     amps      = (MYFLT *) p->amps.auxp;
     freqs     = (MYFLT *) p->freqs.auxp;
     lastamps  = (MYFLT *) p->lastamps.auxp;
-    startbin  = (int32)   *p->ibin;
-    binoffset = (int32)   *p->ibinoffset;
+    startbin  = (int32)  *p->ibin;
+    binoffset = (int32)  *p->ibinoffset;
     lastbin   = p->maxosc;
 
     /*update amps, freqs*/
@@ -217,14 +250,22 @@ static MYFLT adsyn_tick(CSOUND *csound, PVADS *p)
 
 int pvadsyn(CSOUND *csound, PVADS *p)
 {
-    int i;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
 
     MYFLT *aout = p->aout;
 
     if (UNLIKELY(p->outbuf.auxp==NULL)) {
-      csound->Die(csound, Str("pvsynth: Not initialised.\n"));
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsynth: Not initialised.\n"));
     }
-    for (i=0;i < csound->ksmps;i++)
+    if (UNLIKELY(offset)) memset(aout, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&aout[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset;i < nsmps;i++)
       aout[i] = adsyn_tick(csound, p);
     return OK;
 }
@@ -233,34 +274,36 @@ int pvadsyn(CSOUND *csound, PVADS *p)
 
 int pvscrosset(CSOUND *csound, PVSCROSS *p)
 {
-    int32 N = p->fsrc->N;
+    PVSDAT  *fsrc = p->fsrc;
+    PVSDAT  *fout = p->fout;
+    int32 N = fsrc->N;
     /* source fsig */
-    p->overlap = p->fsrc->overlap;
-    p->winsize = p->fsrc->winsize;
-    p->wintype = p->fsrc->wintype;
+    p->overlap = fsrc->overlap;
+    p->winsize = fsrc->winsize;
+    p->wintype = fsrc->wintype;
     p->fftsize = N;
-    p->format  = p->fsrc->format;
+    p->format  = fsrc->format;
 
     /* make sure fdest is same format */
-    if (UNLIKELY(!fsigs_equal(p->fsrc,p->fdest)))
-      csound->Die(csound, Str("pvscross: source and dest signals "
+    if (UNLIKELY(!fsigs_equal(fsrc,p->fdest)))
+      return csound->InitError(csound, Str("pvscross: source and dest signals "
                               "must have same format\n"));
-    p->fout->N =  N;
-    p->fout->overlap = p->overlap;
-    p->fout->winsize = p->winsize;
-    p->fout->wintype = p->wintype;
-    p->fout->format = p->format;
-    p->fout->format = p->fsrc->sliding;
-    if (p->fsrc->sliding) {
-      p->fout->NB = p->fsrc->NB;
-      csound->AuxAlloc(csound, (N + 2) * sizeof(MYFLT) * csound->ksmps,
-                       &p->fout->frame);
+    fout->N =  N;
+    fout->overlap = p->overlap;
+    fout->winsize = p->winsize;
+    fout->wintype = p->wintype;
+    fout->format = p->format;
+    //   fout->format = p->fsrc->sliding;
+    if (fsrc->sliding) {
+      fout->NB = fsrc->NB;
+      csound->AuxAlloc(csound, (N + 2) * sizeof(MYFLT) * CS_KSMPS,
+                       &fout->frame);
       return OK;
     }
     /* setup output signal */
     /* RWD MUST be 32bit */
-    csound->AuxAlloc(csound, (N + 2) * sizeof(float), &p->fout->frame);
-    p->fout->framecount = 1;
+    csound->AuxAlloc(csound, (N + 2) * sizeof(float), &fout->frame);
+    fout->framecount = 1;
     p->lastframe = 0;
     return OK;
 }
@@ -275,18 +318,24 @@ int pvscross(CSOUND *csound, PVSCROSS *p)
     float *fout = (float *) p->fout->frame.auxp;
 
     if (UNLIKELY(fout==NULL))
-      csound->Die(csound, Str("pvscross: not initialised\n"));
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvscross: not initialised\n"));
 
     /* make sure sigs have same format as output */
     if (UNLIKELY(!fsigs_equal(p->fout,p->fsrc)))
-      csound->Die(csound, Str("pvscross: mismatch in fsrc format\n"));
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvscross: mismatch in fsrc format\n"));
     if (UNLIKELY(!fsigs_equal(p->fout,p->fdest)))
-      csound->Die(csound, Str("pvscross: mismatch in fdest format\n"));
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvscross: mismatch in fdest format\n"));
     if (p->fsrc->sliding) {
       CMPLX *fout, *fsrc, *fdest;
-      int n, nsmps = csound->ksmps;
+      uint32_t offset = p->h.insdshead->ksmps_offset;
+      uint32_t early  = p->h.insdshead->ksmps_no_end;
+      uint32_t n, nsmps = CS_KSMPS;
       int NB = p->fsrc->NB;
-      for (n=0; n<nsmps; n++) {
+      nsmps -= early;
+      for (n=offset; n<nsmps; n++) {
         fsrc = (CMPLX *) p->fsrc->frame.auxp +n*NB;    /* RWD all must be 32bit */
         fdest = (CMPLX *) p->fdest->frame.auxp +n*NB;
         fout = (CMPLX *) p->fout->frame.auxp +n*NB;
@@ -315,13 +364,17 @@ int pvscross(CSOUND *csound, PVSCROSS *p)
 
 /******** PVSFREAD ************/
 
-int pvsfreadset(CSOUND *csound, PVSFREAD *p)
+static int pvsfreadset_(CSOUND *csound, PVSFREAD *p, int stringname)
 {
     PVOCEX_MEMFILE  pp;
     uint32   N;
     char            pvfilnam[MAXNAME];
 
-    csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.", p->XSTRCODE);
+    if(stringname) strncpy(pvfilnam, ((STRINGDAT*)p->ifilno)->data, MAXNAME-1);
+    else if (ISSTRCOD(*p->ifilno))
+      strncpy(pvfilnam, get_arg_string(csound, *p->ifilno), MAXNAME-1);
+    else csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.", 0);
+
     if (UNLIKELY(PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0)) {
       return csound->InitError(csound, Str("Failed to load PVOC-EX file"));
     }
@@ -336,23 +389,24 @@ int pvsfreadset(CSOUND *csound, PVSFREAD *p)
     p->arate   = csound->esr / (MYFLT) pp.overlap;
     p->membase = (float*) pp.data;
 
-    if (UNLIKELY(p->overlap < csound->ksmps || p->overlap < 10))
+    if (UNLIKELY(p->overlap < (int)CS_KSMPS || p->overlap < 10))
       csound->InitError(csound, Str("Sliding version not yet available"));
     if (UNLIKELY(p->nframes <= 0))
-      csound->Die(csound, Str("pvsfread: file is empty!\n"));
+      return csound->InitError(csound, Str("pvsfread: file is empty!\n"));
     /* special case if only one frame - it is an impulse response */
     if (UNLIKELY(p->nframes == 1))
-      csound->Die(csound, Str("pvsfread: file has only one frame "
+      return csound->InitError(csound, Str("pvsfread: file has only one frame "
                               "(= impulse response).\n"));
-    if (UNLIKELY(p->overlap < csound->ksmps))
-      csound->Die(csound, Str("pvsfread: analysis frame overlap "
+    if (UNLIKELY(p->overlap < (int)CS_KSMPS))
+      return csound->InitError(csound, Str("pvsfread: analysis frame overlap "
                               "must be >= ksmps\n"));
     p->blockalign = (p->fftsize+2) * p->chans;
     if (UNLIKELY((*p->ichan) >= p->chans))
-      csound->Die(csound, Str("pvsfread: ichan value exceeds "
+      return csound->InitError(csound, Str("pvsfread: ichan value exceeds "
                               "file channel count.\n"));
     if (UNLIKELY((int32) (*p->ichan) < 0))
-      csound->Die(csound, Str("pvsfread: ichan cannot be negative.\n"));
+      return csound->InitError(csound,
+                               Str("pvsfread: ichan cannot be negative.\n"));
 
     N = p->fftsize;
     /* setup output signal */
@@ -375,18 +429,29 @@ int pvsfreadset(CSOUND *csound, PVSFREAD *p)
     return OK;
 }
 
+
+int pvsfreadset(CSOUND *csound, PVSFREAD *p){
+  return pvsfreadset_(csound,p, 0);
+  }
+
+int pvsfreadset_S(CSOUND *csound, PVSFREAD *p){
+  return pvsfreadset_(csound,p, 1);
+  }
+
+
 int pvsfread(CSOUND *csound, PVSFREAD *p)
 {
     int i,j;
     MYFLT pos = *p->kpos;
     MYFLT framepos,frac;
-    int32 frame1pos,frame2pos,framesize,n_mcframes;
+    int32 frame1pos,/*frame2pos,*/framesize,n_mcframes;
 
     float *pmem,*pframe1,*pframe2;               /* RWD all MUST be 32bit */
     float *fout = (float *) p->fout->frame.auxp;
 
     if (UNLIKELY(fout==NULL))
-      csound->Die(csound, Str("pvsfread: not initialised.\n"));
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsfread: not initialised.\n"));
 
     pmem = (float *) p->membase;
     framesize = p->fftsize+2;
@@ -409,7 +474,7 @@ int pvsfread(CSOUND *csound, PVSFREAD *p)
         /* gotta interpolate */
         /* any optimizations possible here ?
            Avoid v samll frac values ? higher-order interp ? */
-        frame2pos = frame1pos+p->chans;
+        //frame2pos = frame1pos+p->chans;
         frac = framepos - (MYFLT) frame1pos;
         pframe1 = pmem + (frame1pos * p->blockalign) + p->chanoffset;
         pframe2 = pframe1 + p->blockalign;
@@ -426,7 +491,7 @@ int pvsfread(CSOUND *csound, PVSFREAD *p)
       p->fout->framecount++;
       p->lastframe = p->fout->framecount;
     }
-    p->ptr += csound->ksmps;
+    p->ptr += CS_KSMPS;
 
     return OK;
 }
@@ -434,10 +499,10 @@ int pvsfread(CSOUND *csound, PVSFREAD *p)
 /************* PVSMASKA ****************/
 int pvsmaskaset(CSOUND *csound, PVSMASKA *p)
 {
-    int i;
+    unsigned int i;
     MYFLT *ftable;
-    int32 N = p->fsrc->N;
-    int32 nbins = N/2 + 1;
+    uint32_t N = p->fsrc->N;
+    uint32_t nbins = N/2 + 1;
     /* source fsig */
     p->overlap = p->fsrc->overlap;
     p->winsize = p->fsrc->winsize;
@@ -445,7 +510,7 @@ int pvsmaskaset(CSOUND *csound, PVSMASKA *p)
     p->format  = p->fsrc->format;
     p->fftsize = N;
     if (UNLIKELY(!(p->format==PVS_AMP_FREQ) || (p->format==PVS_AMP_PHASE)))
-      csound->Die(csound, Str("pvsmaska: "
+      return csound->InitError(csound, Str("pvsmaska: "
                               "signal format must be amp-phase or amp-freq."));
     /* setup output signal */
     p->fout->N =  N;
@@ -455,7 +520,7 @@ int pvsmaskaset(CSOUND *csound, PVSMASKA *p)
     p->fout->format  = p->format;
     p->fout->sliding = p->fsrc->sliding;
     if (p->fsrc->sliding) {
-      csound->AuxAlloc(csound, (N + 2) * sizeof(MYFLT) * csound->ksmps,
+      csound->AuxAlloc(csound, (N + 2) * sizeof(MYFLT) * CS_KSMPS,
                        &p->fout->frame);
       p->fout->NB = p->fsrc->NB;
     }
@@ -466,13 +531,13 @@ int pvsmaskaset(CSOUND *csound, PVSMASKA *p)
         p->fout->framecount = 1;
         p->lastframe = 0;
       }
-    p->maskfunc = csound->FTFind(csound, p->ifn);
+    p->maskfunc = csound->FTnp2Find(csound, p->ifn);
     if (UNLIKELY(p->maskfunc==NULL))
       return NOTOK;
 
     /* require at least size of nbins */
     if (UNLIKELY(p->maskfunc->flen + 1 <         nbins))
-      csound->Die(csound, Str("pvsmaska: ftable too small.\n"));
+      return csound->InitError(csound, Str("pvsmaska: ftable too small.\n"));
 
     /* clip any negative values in table */
     ftable = p->maskfunc->ftable;
@@ -486,18 +551,19 @@ int pvsmaskaset(CSOUND *csound, PVSMASKA *p)
 int pvsmaska(CSOUND *csound, PVSMASKA *p)
 {
     int i;
-    int32 flen, nbins;
+    int32 /*flen, */nbins;
     MYFLT *ftable;
     float *fout,*fsrc;                      /* RWD MUST be 32bit */
     float margin, depth = (float)*p->kdepth;
 
-    flen = p->maskfunc->flen + 1;
+    //flen = p->maskfunc->flen + 1;
     ftable = p->maskfunc->ftable;
     fout = (float *) p->fout->frame.auxp;   /* RWD both MUST be 32bit */
     fsrc = (float *) p->fsrc->frame.auxp;
 
     if (UNLIKELY(fout==NULL))
-      csound->Die(csound, Str("pvsmaska: not initialised\n"));
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsmaska: not initialised\n"));
 
     if (depth < FL(0.0)) {
       /* need the warning: but krate linseg can give below-zeroes incorrectly */
@@ -526,9 +592,12 @@ int pvsmaska(CSOUND *csound, PVSMASKA *p)
     if (p->fsrc->sliding) {
       int NB = p->fsrc->NB;
       CMPLX *fout, *fsrc;
-      int n, nsmps=csound->ksmps;
+      uint32_t offset = p->h.insdshead->ksmps_offset;
+      uint32_t early  = p->h.insdshead->ksmps_no_end;
+      uint32_t n, nsmps = CS_KSMPS;
       MYFLT amp = FL(1.0);
-      for (n=0; n<nsmps; n++) {
+      nsmps -= early;
+      for (n=offset; n<nsmps; n++) {
         fout = (CMPLX *) p->fout->frame.auxp +n*NB;
         fsrc = (CMPLX *) p->fsrc->frame.auxp +n*NB;
         for (i=0; i<NB; i++) {
@@ -576,13 +645,13 @@ int pvsftwset(CSOUND *csound, PVSFTW *p)
     p->lastframe = 0;
 
     if (UNLIKELY(!(p->format==PVS_AMP_FREQ) || (p->format==PVS_AMP_PHASE)))
-      csound->Die(csound, Str("pvsftw: signal format must be "
+      return csound->InitError(csound, Str("pvsftw: signal format must be "
                               "amp-phase or amp-freq.\n"));
     if (UNLIKELY(*p->ifna < 1.0f))
-      csound->Die(csound, Str("pvsftw: bad value for ifna.\n"));
+      return csound->InitError(csound, Str("pvsftw: bad value for ifna.\n"));
     if (UNLIKELY(*p->ifnf < 0.0f))                /* 0 = notused */
-      csound->Die(csound, Str("pvsftw: bad value for ifnf.\n"));
-    p->outfna = csound->FTFind(csound, p->ifna);
+      return csound->InitError(csound, Str("pvsftw: bad value for ifnf.\n"));
+    p->outfna = csound->FTnp2Find(csound, p->ifna);
     if (UNLIKELY(p->outfna==NULL))
       return NOTOK;
     if (UNLIKELY(p->fsrc->sliding))
@@ -593,7 +662,7 @@ int pvsftwset(CSOUND *csound, PVSFTW *p)
     flena = p->outfna->flen + 1;
 
     if (UNLIKELY(flena < nbins))
-      csound->Die(csound, Str("pvsftw: amps ftable too small.\n"));
+      return csound->InitError(csound, Str("pvsftw: amps ftable too small.\n"));
 
     /* init tables */
     ftablea = p->outfna->ftable;
@@ -602,14 +671,15 @@ int pvsftwset(CSOUND *csound, PVSFTW *p)
 
     /* freq table? */
     if ((int32) *p->ifnf >= 1) {
-      p->outfnf = csound->FTFind(csound, p->ifnf);
+      p->outfnf = csound->FTnp2Find(csound, p->ifnf);
       if (UNLIKELY(p->outfnf==NULL))
         return NOTOK;
       ftablef = p->outfnf->ftable;
       if (ftablef) {
         flenf = p->outfnf->flen+1;
         if (UNLIKELY(flenf < nbins))
-          csound->Die(csound, Str("pvsftw: freqs ftable too small.\n"));
+          return csound->InitError(csound,
+                                   Str("pvsftw: freqs ftable too small.\n"));
         for (i=0;i < nbins;i++)
           ftablef[i] = fsrc[(i*2) + 1];
       }
@@ -629,13 +699,16 @@ int pvsftw(CSOUND *csound, PVSFTW *p)
     fsrc = (float *) p->fsrc->frame.auxp;
 
     if (UNLIKELY(fsrc==NULL))
-      csound->Die(csound, Str("pvsftw: not initialised\n"));
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsftw: not initialised\n"));
     if (UNLIKELY(ftablea==NULL))
-      csound->Die(csound, Str("pvsftw: no amps ftable!\n"));
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsftw: no amps ftable!\n"));
     if (p->outfnf) {
       ftablef = p->outfnf->ftable;
       if (UNLIKELY(ftablef==NULL))
-        csound->Die(csound, Str("pvsftw: no freqs ftable!\n"));
+        return csound->PerfError(csound,p->h.insdshead,
+                                 Str("pvsftw: no freqs ftable!\n"));
     }
     nbins = p->fftsize/2 + 1;
 
@@ -677,27 +750,27 @@ int pvsftrset(CSOUND *csound, PVSFTR *p)
     nbins        = p->fftsize/2 + 1;
 
     if (UNLIKELY(!(p->format==PVS_AMP_FREQ) || (p->format==PVS_AMP_PHASE)))
-      csound->Die(csound, Str("pvsftr: signal format must be "
+      return csound->InitError(csound, Str("pvsftr: signal format must be "
                               "amp-phase or amp-freq.\n"));
     /* ifn = 0 = notused */
     if (UNLIKELY(*p->ifna < FL(0.0)))
-      csound->Die(csound, Str("pvsftr: bad value for ifna.\n"));
+      return csound->InitError(csound, Str("pvsftr: bad value for ifna.\n"));
     if (UNLIKELY(*p->ifnf < FL(0.0)))
-      csound->Die(csound, Str("pvsftr: bad value for ifnf.\n"));
+      return csound->InitError(csound, Str("pvsftr: bad value for ifnf.\n"));
 
     /* if ifna=0; no amps to read; one assumes the user ias changing freqs only,
        otherwise, there is no change!
     */
     if ((int32) *p->ifna != 0) {
-      p->infna = csound->FTFind(csound, p->ifna);
+      p->infna = csound->FTnp2Find(csound, p->ifna);
       if (UNLIKELY(p->infna==NULL))
         return NOTOK;
       p->ftablea = p->infna->ftable;
       flena = p->infna->flen + 1;
       if (UNLIKELY(flena < nbins))
-        csound->Die(csound, Str("pvsftr: amps ftable too small.\n"));
+        return csound->InitError(csound, Str("pvsftr: amps ftable too small.\n"));
     }
-    if (UNLIKELY(p->overlap < csound->ksmps || p->overlap < 10))
+    if (UNLIKELY(p->overlap < (int)CS_KSMPS || p->overlap < 10))
       csound->InitError(csound, Str("Sliding version not yet available"));
     fdest = (float *) p->fdest->frame.auxp;             /* RWD MUST be 32bit */
 
@@ -708,13 +781,13 @@ int pvsftrset(CSOUND *csound, PVSFTR *p)
 
     /* freq table? */
     if ((int32) *p->ifnf >= 1) {
-      p->infnf = csound->FTFind(csound, p->ifnf);
+      p->infnf = csound->FTnp2Find(csound, p->ifnf);
       if (UNLIKELY(p->infnf==NULL))
         return NOTOK;
       p->ftablef = p->infnf->ftable;
       flenf = p->infnf->flen+1;
       if (UNLIKELY(flenf < nbins))
-        csound->Die(csound, Str("pvsftr: freqs ftable too small.\n"));
+        return csound->InitError(csound, Str("pvsftr: freqs ftable too small.\n"));
       for (i=0;i < nbins;i++)
         fdest[(i*2) + 1] = (float) p->ftablef[i];
 
@@ -731,7 +804,8 @@ int pvsftr(CSOUND *csound, PVSFTR *p)
 
     fdest = (float *) p->fdest->frame.auxp;
     if (UNLIKELY(fdest==NULL))
-      csound->Die(csound, Str("pvsftr: not initialised\n"));
+      return csound->PerfError(csound,p->h.insdshead,
+                               Str("pvsftr: not initialised\n"));
     nbins = p->fftsize/2 + 1;
 
     /* only write when a new frame is ready */
@@ -763,4 +837,3 @@ int pvsinfo(CSOUND *csound, PVSINFO *p)
     *p->iformat  = (MYFLT) p->fsrc->format;
     return OK;
 }
-
