@@ -422,7 +422,7 @@ int  pvoc_createfile(CSOUND *csound, const char *filename,
       csound->pvErrorCode = -3;
       return -1;
     }
-    if (UNLIKELY(format < PVOC_AMP_FREQ || format > PVOC_COMPLEX)) {
+    if (UNLIKELY(/*format < PVOC_AMP_FREQ ||*/ format > PVOC_COMPLEX)) {
       csound->pvErrorCode = -4;
       return -1;
     }
@@ -509,12 +509,14 @@ int  pvoc_createfile(CSOUND *csound, const char *filename,
 }
 
 int pvoc_openfile(CSOUND *csound,
-                  const char *filename, PVOCDATA *data, WAVEFORMATEX *fmt)
+                  const char *filename, void *data_, void *fmt_)
 {
     WAVEFORMATPVOCEX  wfpx;
     char              *pname;
     PVOCFILE          *p = NULL;
     int               fd;
+    PVOCDATA         *data = (PVOCDATA *) data_;
+    WAVEFORMATEX      *fmt = (WAVEFORMATEX *) fmt_;
 
     csound->pvErrorCode = -1;
     if (UNLIKELY(data == NULL || fmt == NULL)) {
@@ -683,7 +685,8 @@ static int pvoc_readheader(CSOUND *csound, PVOCFILE *p,
           return -1;
         }
         p->customWindow = mmalloc(csound, p->pvdata.dwWinlen * sizeof(float));
-        if (UNLIKELY(pvoc_readWindow(p, p->customWindow, p->pvdata.dwWinlen) != 0)) {
+        if (UNLIKELY(pvoc_readWindow(p,
+                                     p->customWindow, p->pvdata.dwWinlen) != 0)) {
           csound->pvErrorCode = -24;
           return -1;
         }
@@ -819,7 +822,8 @@ static int pvoc_updateheader(CSOUND *csound, int ofd)
       csound->pvErrorCode = -38;
       return 0;
     }
-    if (UNLIKELY(fseek(p->fp, (int32) (p->datachunkoffset - sizeof(uint32_t)), SEEK_SET)
+    if (UNLIKELY(fseek(p->fp,
+                       (int32) (p->datachunkoffset - sizeof(uint32_t)), SEEK_SET)
                  != 0)) {
       csound->pvErrorCode = -33;
       return 0;
@@ -1015,4 +1019,3 @@ int pvoc_framecount(CSOUND *csound, int ifd)
     }
     return p->nFrames;
 }
-
