@@ -37,7 +37,7 @@
 
 static int itblchk(CSOUND *csound, TABLE *p)
 {
-    if (UNLIKELY((p->ftp = csound->FTFind(csound, p->xfn)) == NULL))
+    if (UNLIKELY((p->ftp = csound->FTnp2Find(csound, p->xfn)) == NULL))
       return csound->InitError(csound, Str("table does not exist"));
 
     /* Although TABLE has an integer variable for the table number
@@ -70,6 +70,22 @@ static int itblchk(CSOUND *csound, TABLE *p)
     return OK;
 }
 
+int tblset(CSOUND *csound, TABLE *p)
+{
+    if (UNLIKELY(p->XINCODE != p->XOUTCODE)) {
+      const char  *opname = csound->GetOpcodeName(p);
+      const char  *msg = Str("%s: table index type inconsistent with output");
+      if (UNLIKELY(CS_KSMPS == 1))
+        csound->Warning(csound, msg, opname);
+      else {
+        return csound->InitError(csound, msg, opname);
+      }
+    }
+    p->h.iopadr = (SUBR) itblchk;
+    return itblchk(csound, p);
+}
+
+
 int pktable(CSOUND *,TABLE*);
 int pktabli(CSOUND *,TABLE*);
 int pktabl3(CSOUND *,TABLE*);
@@ -165,7 +181,8 @@ int pktable(CSOUND *csound, TABLE   *p)
     *p->rslt = *(ftp->ftable + indx);
     return OK;
  err1:
-    return csound->PerfError(csound, Str("ptable(krate): not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("ptable(krate): not initialised"));
 }
 
 /* ptablefn()  */
@@ -175,20 +192,27 @@ int ptablefn(CSOUND *csound, TABLE *p)
     FUNC        *ftp;
     MYFLT       *rslt, *pxndx, *tab;
     int32       indx, length;
-    int         n, nsmps=csound->ksmps;
+    uint32_t koffset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       ndx, xbmul, offset;
     int         wrap = p->wrap;
 
     ftp = p->ftp;
     if (UNLIKELY(ftp==NULL)) goto err1;            /* RWD fix */
     rslt = p->rslt;
+    if (koffset) memset(rslt, '\0', koffset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rslt[nsmps], '\0', early*sizeof(MYFLT));
+    }
     length = ftp->flen;
     pxndx = p->xndx;
     xbmul = (MYFLT)p->xbmul;
     offset = p->offset;
     //mask = ftp->lenmask;
     tab = ftp->ftable;
-    for (n=0; n<nsmps; n++) {
+    for (n=koffset; n<nsmps; n++) {
       /* Read in the next raw index and increment the pointer ready
        * for the next cycle.
        *
@@ -213,7 +237,8 @@ int ptablefn(CSOUND *csound, TABLE *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("table: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("table: not initialised"));
 }
 
 /* pktabli() */
@@ -262,7 +287,8 @@ int pktabli(CSOUND *csound, TABLE   *p)
     *p->rslt = v1 + (v2 - v1) * fract;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("ptablei(krate): not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("ptablei(krate): not initialised"));
 }
 
 
@@ -321,20 +347,28 @@ int pktabl3(CSOUND *csound, TABLE   *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("ptable3(krate): not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("ptable3(krate): not initialised"));
 }
 
 int ptabli(CSOUND *csound, TABLE   *p)
 {
     FUNC        *ftp;
     int32       indx, length;
-    int         n, nsmps=csound->ksmps;
+    uint32_t koffset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       *rslt, *pxndx, *tab;
     MYFLT       fract, v1, v2, ndx, xbmul, offset;
 
     ftp = p->ftp;
     if (UNLIKELY(ftp==NULL)) goto err1;
     rslt   = p->rslt;
+    if (koffset) memset(rslt, '\0', koffset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rslt[nsmps], '\0', early*sizeof(MYFLT));
+    }
     length = ftp->flen;
     pxndx  = p->xndx;
     xbmul  = (MYFLT)p->xbmul;
@@ -343,7 +377,7 @@ int ptabli(CSOUND *csound, TABLE   *p)
     tab    = ftp->ftable;
     /* As for ktabli() code to handle non wrap mode, and wrap mode.  */
     if (!p->wrap) {
-      for (n=0; n<nsmps; n++) {
+      for (n=koffset; n<nsmps; n++) {
         /* Read in the next raw index and increment the pointer ready
          * for the next cycle.
          * Then multiply the ndx by the denormalising factor and add in
@@ -371,7 +405,7 @@ int ptabli(CSOUND *csound, TABLE   *p)
       }
     }
     else {                      /* wrap mode */
-      for (n=0; n<nsmps; n++) {
+      for (n=koffset; n<nsmps; n++) {
         int j;
         /* Read in the next raw index and increment the pointer ready
          * for the next cycle.
@@ -395,14 +429,17 @@ int ptabli(CSOUND *csound, TABLE   *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("ptablei: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("ptablei: not initialised"));
 }
 
 int ptabl3(CSOUND *csound, TABLE *p)     /* Like ptabli but cubic interpolation */
 {
     FUNC        *ftp;
     int32       indx, length;
-    int         n, nsmps=csound->ksmps;
+    uint32_t    koffset = p->h.insdshead->ksmps_offset;
+    uint32_t    early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    n, nsmps = CS_KSMPS;
     MYFLT       *rslt, *pxndx, *tab;
     MYFLT       fract, v1, v2, ndx, xbmul, offset;
     int         wrap = p->wrap;
@@ -410,12 +447,17 @@ int ptabl3(CSOUND *csound, TABLE *p)     /* Like ptabli but cubic interpolation
     ftp = p->ftp;
     if (UNLIKELY(ftp==NULL)) goto err1;
     rslt = p->rslt;
+    if (koffset) memset(rslt, '\0', koffset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&rslt[nsmps], '\0', early*sizeof(MYFLT));
+    }
     length = ftp->flen;
     pxndx = p->xndx;
     xbmul = (MYFLT)p->xbmul;
     offset = p->offset;
     tab = ftp->ftable;
-    for (n=0; n<nsmps; n++) {
+    for (n=koffset; n<nsmps; n++) {
       /* Read in the next raw index and increment the pointer ready
        * for the next cycle.
        * Then multiply the ndx by the denormalising factor and add in
@@ -462,10 +504,51 @@ int ptabl3(CSOUND *csound, TABLE *p)     /* Like ptabli but cubic interpolation
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("ptable3: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("ptable3: not initialised"));
 }
 
-extern int itblchkw(CSOUND *, TABLEW*);
+int itblchkw(CSOUND *csound, TABLEW *p)
+{
+    /* Get pointer to the function table data structure of the table
+     * number specified in xfn. Return 0 if it cannot be found.
+     *
+     * csoundFTFind() generates an error message if the table cannot be
+     * found. This works OK at init time.  It also deactivates the
+     * instrument.
+     *
+     * It also checks for numbers < 0, and table 0 is never valid, so we
+     * do not need to check here for the table number being < 1.  */
+
+    if (UNLIKELY((p->ftp = csound->FTnp2Find(csound, p->xfn)) == NULL))
+      return NOTOK;
+    /* Although TABLEW has an integer variable for the table number
+     * (p->pfn) we do not need to * write it.  We know that the * k
+     * and a rate functions * which will follow will not * be
+     * expecting a changed * table number.
+     *
+     * p->pfn exists only for checking * table number changes for
+     * functions * which are expecting a k rate * table number.  */
+
+    /* Set denormalisation factor to 1 or table length, depending on
+     * the state of ixmode.  1L means a 32 bit 1.  */
+    /* JPff.............................^...not so; could be any length */
+    if (*p->ixmode)
+            p->xbmul = p->ftp->flen;
+    else    p->xbmul = 1L;
+    /* Multiply the ixoff value by the xbmul denormalisation
+     * factor and then check it is between 0 and the table length.  */
+      if (UNLIKELY((p->offset = p->xbmul * *p->ixoff) < FL(0.0)
+                   || p->offset > p->ftp->flen)) {
+      return csound->InitError(csound,
+                               Str("Table write offset %f < 0 or > tablelength"),
+                               p->offset);
+    }
+    p->iwgm   = (int)*p->iwgmode;
+    return OK;
+}
+
+
 int pktablew(CSOUND *, TABLEW*);
 int pitablew(CSOUND *csound, TABLEW *p)
 {
@@ -560,7 +643,9 @@ int ptablew(CSOUND *csound, TABLEW *p)
     int32        indx;   /* Used to read table. */
     int32        length; /* Length of table */
     int         liwgm;          /* Local copy of iwgm for speed */
-    int         n, nsmps = csound->ksmps;
+    uint32_t koffset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       ndx, xbmul, offset;
                                 /*-----------------------------------*/
     /* Assume that TABLEW has been set up correctly. */
@@ -574,7 +659,8 @@ int ptablew(CSOUND *csound, TABLEW *p)
     xbmul  = (MYFLT)p->xbmul;
     offset = p->offset;
                 /* Main loop - for the number of a samples in a k cycle. */
-    for (n=0; n<nsmps; n++) {
+    nsmps -= early;
+    for (n=koffset; n<nsmps; n++) {
       /* Read in the next raw index and increment the pointer ready for the
          next cycle.  Then multiply the ndx by the denormalising factor and
          add in the offset.  */
