@@ -24,7 +24,6 @@
 #include "csoundCore.h"         /*                              UGENS3.C    */
 #include "ugens3.h"
 #include <math.h>
-#include "oload.h"
 
 int foscset(CSOUND *csound, FOSC *p)
 {
@@ -48,7 +47,9 @@ int foscil(CSOUND *csound, FOSC *p)
     MYFLT   *ar, *ampp, *modp, cps, amp;
     MYFLT   xcar, xmod, *carp, car, fmod, cfreq, mod, ndx, *ftab;
     int32    mphs, cphs, minc, cinc, lobits;
-    int     n, nn=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   sicvt = csound->sicvt;
 
     ar = p->rslt;
@@ -65,9 +66,14 @@ int foscil(CSOUND *csound, FOSC *p)
     amp  = *ampp;
     xcar = *carp;
     xmod = *modp;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
 
     if (p->XINCODE) {
-      for (n=0;n<nn;n++) {
+      for (n=offset;n<nsmps;n++) {
         if (p->ampcod) amp = ampp[n];
         if (p->carcod) xcar = carp[n];
         if (p->modcod) xmod = modp[n];
@@ -92,7 +98,7 @@ int foscil(CSOUND *csound, FOSC *p)
       mod = cps * *modp;
       ndx = *p->kndx * mod;
       minc = (int32)(mod * sicvt);
-      for (n=0;n<nn;n++) {
+      for (n=offset;n<nsmps;n++) {
         mphs &= PHMASK;
         fmod = *(ftab + (mphs >>lobits)) * ndx;
         mphs += minc;
@@ -108,7 +114,8 @@ int foscil(CSOUND *csound, FOSC *p)
 
     return OK;
  err1:
-    return csound->PerfError(csound, Str("foscil: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("foscil: not initialised"));
 }
 
 int foscili(CSOUND *csound, FOSC *p)
@@ -117,7 +124,9 @@ int foscili(CSOUND *csound, FOSC *p)
     MYFLT  *ar, *ampp, amp, cps, fract, v1, car, fmod, cfreq, mod;
     MYFLT  *carp, *modp, xcar, xmod, ndx, *ftab;
     int32  mphs, cphs, minc, cinc, lobits;
-    int    n, nn=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT  sicvt = csound->sicvt;
     MYFLT  *ft;
 
@@ -135,8 +144,13 @@ int foscili(CSOUND *csound, FOSC *p)
     amp  = *ampp;
     xcar = *carp;
     xmod = *modp;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->XINCODE) {
-      for (n=0;n<nn;n++) {
+      for (n=offset;n<nsmps;n++) {
         if (p->ampcod)  amp = ampp[n];
         if (p->carcod)  xcar = carp[n];
         if (p->modcod)  xmod = modp[n];
@@ -166,7 +180,7 @@ int foscili(CSOUND *csound, FOSC *p)
       mod = cps * *modp;
       ndx = *p->kndx * mod;
       minc = (int32)(mod * sicvt);
-      for (n=0;n<nn;n++) {
+      for (n=offset;n<nsmps;n++) {
         mphs &= PHMASK;
         fract = PFRAC(mphs);
         ftab = ft + (mphs >>lobits);
@@ -188,7 +202,8 @@ int foscili(CSOUND *csound, FOSC *p)
 
     return OK;
  err1:
-    return csound->PerfError(csound, Str("foscili: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("foscili: not initialised"));
 }
 
 
@@ -219,10 +234,10 @@ int losset(CSOUND *csound, LOSC *p)
         p->end1 = (int32) (*p->iend1 * (MYFLT) (LOFACT));
         if (!p->beg1 && !p->end1) {
           /* default to looping the whole sample */
-          p->end1 = (p->mod1 ? maxphs : ((int32) ftp->flenfrms << LOBITS));
+          p->end1 = (p->mod1 ? (int32)maxphs : ((int32) ftp->flenfrms << LOBITS));
         }
         else if (UNLIKELY(p->beg1 < 0 ||
-                          p->end1 > maxphs ||
+                          p->end1 > (int32)maxphs ||
                           p->beg1 >= p->end1)) {
           csound->Message(csound, "beg: %d, end = %d, maxphs = %d\n",
                           p->beg1, p->end1, maxphs);
@@ -238,19 +253,20 @@ int losset(CSOUND *csound, LOSC *p)
         p->beg2 = (int32) (*p->ibeg2 * (MYFLT) (LOFACT));
         p->end2 = (int32) (*p->iend2 * (MYFLT) (LOFACT));
         if (UNLIKELY(p->mod2 < 0 || p->mod2 > 3 ||
-                     p->beg2 < 0 || p->end2 > maxphs || p->beg2 >= p->end2)) {
+                     p->beg2 < 0 || p->end2 > (int32)maxphs ||
+                     p->beg2 >= p->end2)) {
           goto lerr3;
         }
       }
       p->beg1 = (p->beg1 >= 0L ? p->beg1 : 0L);
-      p->end1 = (p->end1 < maxphs ? p->end1 : maxphs);
+      p->end1 = (p->end1 < (int32)maxphs ? p->end1 : (int32)maxphs);
       if (UNLIKELY(p->beg1 >= p->end1)) {
         p->mod1 = 0;
         p->beg1 = 0L;
         p->end1 = maxphs;
       }
       p->beg2 = (p->beg2 >= 0L ? p->beg2 : 0L);
-      p->end2 = (p->end2 < maxphs ? p->end2 : maxphs);
+      p->end2 = (p->end2 < (int32)maxphs ? p->end2 : (int32)maxphs);
       if (UNLIKELY(p->beg2 >= p->end2)) {
         p->mod2 = 0;
         p->beg2 = 0L;
@@ -361,18 +377,24 @@ static CS_NOINLINE void
     *arR = tmpR;
 }
 
+/* *********************** needs total rewrite **************** */
 int loscil(CSOUND *csound, LOSC *p)
 {
     FUNC    *ftp;
     MYFLT   *ar1, *ar2, *ftbl, *xamp;
     int32    phs, inc, beg, end;
-    int     nsmps = csound->ksmps, aamp;
+    uint32_t n = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nsmps = CS_KSMPS;
+    int      aamp;
+    MYFLT    xx;
 
     ftp = p->ftp;
     ftbl = ftp->ftable;
     if ((inc = (int32)(*p->kcps * p->cpscvt)) < 0)
       inc = -inc;
     xamp = p->xamp;
+    xx = *xamp;
     aamp = (p->XINCODE) ? 1 : 0;
     if (p->seg1) {                      /* if still segment 1  */
       beg = p->beg1;
@@ -386,8 +408,15 @@ int loscil(CSOUND *csound, LOSC *p)
     }
     phs = p->lphs;
     ar1 = p->ar1;
+    if (UNLIKELY(n)) memset(ar1, '\0', n*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar1[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->stereo) {
       ar2 = p->ar2;
+      if (UNLIKELY(n)) memset(ar2, '\0', n*sizeof(MYFLT));
+      if (UNLIKELY(early)) memset(&ar2[nsmps], '\0', early*sizeof(MYFLT));
       goto phsck2;
     }
  phschk:
@@ -395,53 +424,53 @@ int loscil(CSOUND *csound, LOSC *p)
       goto put0;
     switch (p->curmod) {
     case 0:
-      do {                                      /* NO LOOPING  */
-        loscil_linear_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* NO LOOPING  */
+        loscil_linear_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
         if ((phs += inc) >= end)
           goto nxtseg;
-      } while (--nsmps);
+      }
       break;
     case 1:
-      do {                                      /* NORMAL LOOPING */
-        loscil_linear_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* NORMAL LOOPING */
+        loscil_linear_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
         if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg;
           phs -= end - beg;
         }
-      } while (--nsmps);
+      }
       break;
     case 2:
     case2:
-      do {                                      /* BIDIR FORW, EVEN */
-        loscil_linear_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* BIDIR FORW, EVEN */
+        loscil_linear_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
         if ((phs += inc) >= end) {
           if (!(p->looping)) goto nxtseg;
           phs -= (phs - end) * 2;
           p->curmod = 3;
-          if (--nsmps) goto case3;
+          if (++n<nsmps) goto case3;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
     case 3:
     case3:
-      do {                                      /* BIDIR BACK, EVEN */
-        loscil_linear_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* BIDIR BACK, EVEN */
+        loscil_linear_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
         if (UNLIKELY((phs -= inc) < beg)) {
           phs += (beg - phs) * 2;
           p->curmod = 2;
-          if (--nsmps) goto case2;
+          if (++n<nsmps) goto case2;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
 
     nxtseg:
@@ -449,7 +478,7 @@ int loscil(CSOUND *csound, LOSC *p)
         p->seg1 = 0;
         if ((p->curmod = p->mod2) != 0)
           p->looping = 1;
-        if (--nsmps) {
+        if (++n>nsmps) {
           beg = p->beg2;
           end = p->end2;
           p->lphs = phs;
@@ -457,7 +486,7 @@ int loscil(CSOUND *csound, LOSC *p)
         }
         break;
       }
-      if (UNLIKELY(--nsmps)) goto phsout;
+      if (LIKELY(++n<nsmps)) goto phsout;
       break;
     }
     p->lphs = phs;
@@ -466,10 +495,7 @@ int loscil(CSOUND *csound, LOSC *p)
  phsout:
     p->lphs = phs;
  put0:
-    memset(ar1, 0, sizeof(MYFLT)*nsmps);
-    /* do { */
-    /*   *ar1++ = FL(0.0); */
-    /* } while (--nsmps); */
+    memset(&ar1[n], '\0', sizeof(MYFLT)*(nsmps-n));
     return OK;
 
  phsck2:
@@ -477,57 +503,57 @@ int loscil(CSOUND *csound, LOSC *p)
       goto put0s;                               /* for STEREO:  */
     switch (p->curmod) {
     case 0:
-      do {                                      /* NO LOOPING  */
-        loscil_linear_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* NO LOOPING  */
+        loscil_linear_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
         if (UNLIKELY((phs += inc) >= end))
           goto nxtseg2;
-      } while (--nsmps);
+      }
       break;
     case 1:
-      do {                                      /* NORMAL LOOPING */
-        loscil_linear_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* NORMAL LOOPING */
+        loscil_linear_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
         if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg2;
           phs -= end - beg;
         }
-      } while (--nsmps);
+      }
       break;
     case 2:
     case2s:
-      do {                                      /* BIDIR FORW, EVEN */
-        loscil_linear_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* BIDIR FORW, EVEN */
+        loscil_linear_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
         if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg2;
           phs -= (phs - end) * 2;
           p->curmod = 3;
-          if (--nsmps) goto case3s;
+          if (++n<nsmps) goto case3s;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
     case 3:
     case3s:
-      do {                                      /* BIDIR BACK, EVEN */
-        loscil_linear_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* BIDIR BACK, EVEN */
+       loscil_linear_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
         if (UNLIKELY((phs -= inc) < beg)) {
           phs += (beg - phs) * 2;
           p->curmod = 2;
-          if (--nsmps) goto case2s;
+          if (++n<nsmps) goto case2s;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
 
     nxtseg2:
@@ -535,7 +561,7 @@ int loscil(CSOUND *csound, LOSC *p)
         p->seg1 = 0;
         if ((p->curmod = p->mod2) != 0)
           p->looping = 1;
-        if (--nsmps) {
+        if (++n<nsmps) {
           beg = p->beg2;
           end = p->end2;
           p->lphs = phs;
@@ -543,7 +569,7 @@ int loscil(CSOUND *csound, LOSC *p)
         }
         break;
       }
-      if (UNLIKELY(--nsmps)) goto phsout2;
+      if (LIKELY(++n<nsmps)) goto phsout2;
       break;
     }
     p->lphs = phs;
@@ -552,8 +578,8 @@ int loscil(CSOUND *csound, LOSC *p)
  phsout2:
     p->lphs = phs;
  put0s:
-    memset(ar1, 0, sizeof(MYFLT)*nsmps);
-    memset(ar2, 0, sizeof(MYFLT)*nsmps);
+    memset(&ar1[n], '\0', sizeof(MYFLT)*(nsmps-n));
+    memset(&ar2[n], '\0', sizeof(MYFLT)*(nsmps-n));
     /* do { */
     /*   *ar1++ = FL(0.0); */
     /*   *ar2++ = FL(0.0); */
@@ -567,13 +593,18 @@ int loscil3(CSOUND *csound, LOSC *p)
     FUNC    *ftp;
     MYFLT   *ar1, *ar2, *ftbl, *xamp;
     int32    phs, inc, beg, end;
-    int     nsmps = csound->ksmps, aamp;
+    uint32_t n = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nsmps = CS_KSMPS;
+    int     aamp;
+    MYFLT   xx;
 
     ftp = p->ftp;
     ftbl = ftp->ftable;
     if ((inc = (int32)(*p->kcps * p->cpscvt)) < 0)
       inc = -inc;
     xamp = p->xamp;
+    xx = *xamp;
     aamp = (p->XINCODE) ? 1 : 0;
     if (p->seg1) {                      /* if still segment 1  */
       beg = p->beg1;
@@ -587,8 +618,15 @@ int loscil3(CSOUND *csound, LOSC *p)
     }
     phs = p->lphs;
     ar1 = p->ar1;
+    if (UNLIKELY(n)) memset(ar1, '\0', n*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar1[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->stereo) {
       ar2 = p->ar2;
+      if (UNLIKELY(n)) memset(ar1, '\0', n*sizeof(MYFLT));
+      if (UNLIKELY(early)) memset(&ar2[nsmps], '\0', early*sizeof(MYFLT));
       goto phsck2;
     }
  phschk:
@@ -596,53 +634,53 @@ int loscil3(CSOUND *csound, LOSC *p)
       goto put0;
     switch (p->curmod) {
     case 0:
-      do {                                      /* NO LOOPING  */
-        loscil_cubic_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* NO LOOPING  */
+        loscil_cubic_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
         if (UNLIKELY((phs += inc) >= end))
           goto nxtseg;
-      } while (--nsmps);
+      }
       break;
     case 1:
-      do {                                      /* NORMAL LOOPING */
-        loscil_cubic_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* NORMAL LOOPING */
+        loscil_cubic_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
         if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg;
           phs -= end - beg;
         }
-      } while (--nsmps);
+      }
       break;
     case 2:
     case2:
-      do {                                      /* BIDIR FORW, EVEN */
-        loscil_cubic_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* BIDIR FORW, EVEN */
+        loscil_cubic_interp_mono(&ar1[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
         if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg;
           phs -= (phs - end) * 2;
           p->curmod = 3;
-          if (--nsmps) goto case3;
+          if (++n<nsmps) goto case3;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
     case 3:
     case3:
-      do {                                      /* BIDIR BACK, EVEN */
-        loscil_cubic_interp_mono(ar1, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* BIDIR BACK, EVEN */
+        loscil_cubic_interp_mono(&ar1[n], ftbl, phs, ftp->flen);;
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
         if (UNLIKELY((phs -= inc) < beg)) {
           phs += (beg - phs) * 2;
           p->curmod = 2;
-          if (--nsmps) goto case2;
+          if (++n<nsmps) goto case2;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
 
     nxtseg:
@@ -658,7 +696,7 @@ int loscil3(CSOUND *csound, LOSC *p)
         }
         break;
       }
-      if (UNLIKELY(--nsmps)) goto phsout;
+      if (LIKELY(++n<nsmps)) goto phsout;
       break;
     }
     p->lphs = phs;
@@ -667,7 +705,7 @@ int loscil3(CSOUND *csound, LOSC *p)
  phsout:
     p->lphs = phs;
  put0:
-    memset(ar1, 0, sizeof(MYFLT)*nsmps);
+    memset(&ar1[n], 0, sizeof(MYFLT)*(nsmps-n));
     /* do { */
     /*   *ar1++ = FL(0.0); */
     /* } while (--nsmps); */
@@ -678,57 +716,57 @@ int loscil3(CSOUND *csound, LOSC *p)
       goto put0s;                               /* for STEREO:  */
     switch (p->curmod) {
     case 0:
-      do {                                      /* NO LOOPING  */
-        loscil_cubic_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* NO LOOPING  */
+        loscil_cubic_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
         if (UNLIKELY((phs += inc) >= end))
           goto nxtseg2;
-      } while (--nsmps);
+      }
       break;
     case 1:
-      do {                                      /* NORMAL LOOPING */
-        loscil_cubic_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* NORMAL LOOPING */
+        loscil_cubic_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
         if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg2;
           phs -= end - beg;
         }
-      } while (--nsmps);
+      }
       break;
     case 2:
     case2s:
-      do {                                      /* BIDIR FORW, EVEN */
-        loscil_cubic_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* BIDIR FORW, EVEN */
+        loscil_cubic_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
         if (UNLIKELY((phs += inc) >= end)) {
           if (!(p->looping)) goto nxtseg2;
           phs -= (phs - end) * 2;
           p->curmod = 3;
-          if (--nsmps) goto case3s;
+          if (++n<nsmps) goto case3s;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
     case 3:
     case3s:
-      do {                                      /* BIDIR BACK, EVEN */
-        loscil_cubic_interp_stereo(ar1, ar2, ftbl, phs, ftp->flen);
-        *ar1++ *= *xamp;
-        *ar2++ *= *xamp;
-        if (aamp)  xamp++;
+      for (; n<nsmps; n++) {                    /* BIDIR BACK, EVEN */
+        loscil_cubic_interp_stereo(&ar1[n], &ar2[n], ftbl, phs, ftp->flen);
+        if (aamp) xx = xamp[n];
+        ar1[n] *= xx;
+        ar2[n] *= xx;
         if (UNLIKELY((phs -= inc) < beg)) {
           phs += (beg - phs) * 2;
           p->curmod = 2;
-          if (--nsmps) goto case2s;
+          if (++n<nsmps) goto case2s;
           else break;
         }
-      } while (--nsmps);
+      }
       break;
 
     nxtseg2:
@@ -736,7 +774,7 @@ int loscil3(CSOUND *csound, LOSC *p)
         p->seg1 = 0;
         if ((p->curmod = p->mod2) != 0)
           p->looping = 1;
-        if (--nsmps) {
+        if (++n<nsmps) {
           beg = p->beg2;
           end = p->end2;
           p->lphs = phs;
@@ -744,7 +782,7 @@ int loscil3(CSOUND *csound, LOSC *p)
         }
         break;
       }
-      if (UNLIKELY(--nsmps)) goto phsout2;
+      if (LIKELY(++n<nsmps)) goto phsout2;
       break;
     }
     p->lphs = phs;
@@ -753,8 +791,8 @@ int loscil3(CSOUND *csound, LOSC *p)
  phsout2:
     p->lphs = phs;
  put0s:
-    memset(ar1, 0, sizeof(MYFLT)*nsmps);
-    memset(ar2, 0, sizeof(MYFLT)*nsmps);
+    memset(&ar1[n], '\0', sizeof(MYFLT)*(nsmps-n));
+    memset(&ar2[n], '\0', sizeof(MYFLT)*(nsmps-n));
    /*  do { */
    /*    *ar1++ = FL(0.0); */
    /*    *ar2++ = FL(0.0); */
@@ -766,7 +804,7 @@ int loscil3(CSOUND *csound, LOSC *p)
 #define ISINSIZ 32768L
 #define ADMASK  32767L
 
-int adset(CSOUND *csound, ADSYN *p)
+static int adset_(CSOUND *csound, ADSYN *p, int stringname)
 {
     int32    n;
     char    filnam[MAXNAME];
@@ -781,10 +819,16 @@ int adset(CSOUND *csound, ADSYN *p)
       for (n = 0; n < ISINSIZ; n++)
         *ip++ = (int16) (sin(TWOPI * n / ISINSIZ) * 32767.0);
     }
-    csound->strarg2name(csound, filnam, p->ifilcod, "adsyn.", p->XSTRCODE);
+    if(stringname) strncpy(filnam, ((STRINGDAT*)p->ifilcod)->data, MAXNAME-1);
+    else if (ISSTRCOD(*p->ifilcod))
+      strncpy(filnam, get_arg_string(csound, *p->ifilcod), MAXNAME-1);
+    else csound->strarg2name(csound, filnam, p->ifilcod, "adsyn.", 0);
+
+
     if ((mfp = p->mfp) == NULL || strcmp(mfp->filename,filnam) != 0) {
       /* readfile if reqd */
-      if (UNLIKELY((mfp = ldmemfile2(csound, filnam, CSFTYPE_HETRO)) == NULL)) {
+      if (UNLIKELY((mfp = ldmemfile2withCB(csound, filnam,
+                                           CSFTYPE_HETRO, NULL)) == NULL)) {
         csound->InitError(csound, Str("ADSYN cannot load %s"), filnam);
         return NOTOK;
       }
@@ -794,7 +838,7 @@ int adset(CSOUND *csound, ADSYN *p)
     adp = (int16 *) mfp->beginp;            /* align on file data */
     endata = (int16 *) mfp->endp;
     size = 1+(*adp == -1 ? MAXPTLS : *adp++); /* Old no header -> MAXPIL */
-    if (p->aux.auxp==NULL || p->aux.size < (int32)sizeof(PTLPTR)*size)
+    if (p->aux.auxp==NULL || p->aux.size < (uint32_t)sizeof(PTLPTR)*size)
       csound->AuxAlloc(csound, sizeof(PTLPTR)*size, &p->aux);
 
     ptlap = ptlfp = (PTLPTR*)p->aux.auxp;   /* find base ptl blk */
@@ -834,6 +878,14 @@ int adset(CSOUND *csound, ADSYN *p)
     return csound->InitError(csound, Str("partial count exceeds MAXPTLS"));
 }
 
+int adset(CSOUND *csound, ADSYN *p){
+  return adset_(csound,p,0);
+}
+
+int adset_S(CSOUND *csound, ADSYN *p){
+  return adset_(csound,p,1);
+}
+
 #define ADSYN_MAXLONG FL(2147483647.0)
 
 int adsyn(CSOUND *csound, ADSYN *p)
@@ -842,21 +894,24 @@ int adsyn(CSOUND *csound, ADSYN *p)
     DUPLE   *ap, *fp;
     int16   curtim, diff, ktogo;
     int32   phs, sinc, amp;
-    int     n, nsmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   *ar = p->rslt;
     MYFLT   ampscale, frqscale;
     int32   timkincr, nxtim;
 
     if (UNLIKELY(csound->isintab == NULL)) {      /* RWD fix */
-      return csound->PerfError(csound, Str("adsyn: not initialised"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("adsyn: not initialised"));
     }
     /* IV - Jul 11 2002 */
     ampscale = *p->kamod * csound->e0dbfs;      /* since 15-bit sine table */
     frqscale = *p->kfmod * ISINSIZ * csound->onedsr;
     /* 1024 * msecs of analysis */
-    timkincr = (int32)(*p->ksmod*FL(1024000.0)*csound->onedkr);
-    nsmps = csound->ksmps;
     memset(p->rslt,0,sizeof(MYFLT)*nsmps);
+    if (UNLIKELY(early)) nsmps -= early;
+    timkincr = (int32)(*p->ksmod*FL(1024000.0)*CS_ONEDKR);
     curtim = (int16)(p->mksecs >> 10);          /* cvt mksecs to msecs */
     curp = (PTLPTR*)p->aux.auxp;                /* now for each partial:    */
     while ((prvp = curp) && (curp = curp->nxtp) != NULL ) {
@@ -869,8 +924,8 @@ int adsyn(CSOUND *csound, ADSYN *p)
       if ((amp = curp->amp)) {            /* for non-zero amp   */
         sinc = (int32)(curp->frq * frqscale);
         phs = curp->phs;
-        nsmps = csound->ksmps;            /*   addin a sinusoid */
-        for (n=0; n<nsmps; n++) {
+        /*   addin a sinusoid */
+        for (n=offset; n<nsmps; n++) {
           ar[n] += (ampscale*(MYFLT)csound->isintab[phs]*(MYFLT)amp)/ADSYN_MAXLONG;
           phs += sinc;
           phs &= ADMASK;
@@ -898,4 +953,3 @@ int adsyn(CSOUND *csound, ADSYN *p)
     p->mksecs += timkincr;                  /* advance the time */
     return OK;
 }
-
