@@ -38,7 +38,7 @@ int vdelset(CSOUND *csound, VDEL *p)            /*  vdelay set-up   */
     uint32 n = (int32)(*p->imaxd * ESR)+1;
 
     if (!*p->istod) {
-      if (p->aux.auxp == NULL || (int32)(n * sizeof(MYFLT)) > p->aux.size)
+      if (p->aux.auxp == NULL || (uint32_t)(n * sizeof(MYFLT)) > p->aux.size)
         /* allocate space for delay buffer */
         csound->AuxAlloc(csound, n * sizeof(MYFLT), &p->aux);
       else {     /*    make sure buffer is empty       */
@@ -51,7 +51,10 @@ int vdelset(CSOUND *csound, VDEL *p)            /*  vdelay set-up   */
 
 int vdelay(CSOUND *csound, VDEL *p)               /*      vdelay  routine */
 {
-    int32  nn, nsmps = csound->ksmps, maxd, indx;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nn, nsmps = CS_KSMPS;
+    int32  maxd, indx;
     MYFLT *out = p->sr;     /* assign object data to local variables   */
     MYFLT *in = p->ain;
     MYFLT *del = p->adel;
@@ -60,9 +63,14 @@ int vdelay(CSOUND *csound, VDEL *p)               /*      vdelay  routine */
     if (UNLIKELY(buf==NULL)) goto err1;        /* RWD fix */
     maxd = (uint32) (1+*p->imaxd * ESR);
     indx = p->left;
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
 
     if (XINARG2) {          /*      if delay is a-rate      */
-      for (nn=0; nn<nsmps; nn++) {
+      for (nn=offset; nn<nsmps; nn++) {
         MYFLT  fv1, fv2;
         int32   v1, v2;
 
@@ -98,7 +106,7 @@ int vdelay(CSOUND *csound, VDEL *p)               /*      vdelay  routine */
     }
     else {                      /* and, if delay is k-rate */
       MYFLT fdel=*del;
-      for (nn=0; nn<nsmps; nn++) {
+      for (nn=offset; nn<nsmps; nn++) {
         MYFLT  fv1, fv2;
         int32   v1, v2;
 
@@ -129,12 +137,16 @@ int vdelay(CSOUND *csound, VDEL *p)               /*      vdelay  routine */
     p->left = indx;             /*      and keep track of where you are */
     return OK;
  err1:
-    return csound->PerfError(csound, Str("vdelay: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("vdelay: not initialised"));
 }
 
 int vdelay3(CSOUND *csound, VDEL *p)    /*  vdelay routine with cubic interp */
 {
-    int32  nn, nsmps = csound->ksmps, maxd, indx;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nn, nsmps = CS_KSMPS;
+    int32  maxd, indx;
     MYFLT *out = p->sr;  /* assign object data to local variables   */
     MYFLT *in = p->ain;
     MYFLT *del = p->adel;
@@ -143,11 +155,15 @@ int vdelay3(CSOUND *csound, VDEL *p)    /*  vdelay routine with cubic interp */
     if (UNLIKELY(buf==NULL)) goto err1;            /* RWD fix */
     maxd = (uint32) (*p->imaxd * ESR);
     if (UNLIKELY(maxd == 0)) maxd = 1;    /* Degenerate case */
-    nn = csound->ksmps;
     indx = p->left;
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
 
     if (XINARG2) {              /*      if delay is a-rate      */
-      for (nn=0; nn<nsmps; nn++) {
+      for (nn=offset; nn<nsmps; nn++) {
         MYFLT  fv1;
         int32   v0, v1, v2, v3;
 
@@ -201,10 +217,10 @@ int vdelay3(CSOUND *csound, VDEL *p)    /*  vdelay routine with cubic interp */
       }
 
       if (maxd<4) {
-        while (nn--) {
+        for (nn=offset; nn<nsmps; nn++) {
           /* Find next sample for interpolation      */
           v2 = (v1 == (int32)(maxd - 1UL) ? 0L : v1 + 1L);
-          *out++ = buf[v1] + fv1 * (buf[v2] - buf[v1]);
+          out[nn] = buf[v1] + fv1 * (buf[v2] - buf[v1]);
           if (UNLIKELY(++v1 >= (int32)maxd)) v1 -= (int32)maxd;
               if (UNLIKELY(++indx >= maxd)) indx -= maxd;
         }
@@ -214,7 +230,7 @@ int vdelay3(CSOUND *csound, VDEL *p)    /*  vdelay routine with cubic interp */
         z = fv1 * fv1; z--; z *= FL(0.1666666667);      /* IV Oct 2001 */
         y = fv1; y++; w = (y *= FL(0.5)); w--;
         x = FL(3.0) * z; y -= x; w -= z; x -= fv1;
-        for (nn=0; nn<nsmps; nn++) {
+        for (nn=offset; nn<nsmps; nn++) {
           buf[indx] = in[nn];
           /* Find next sample for interpolation      */
           v2 = (v1 == (int32)(maxd - 1UL) ? 0L : v1 + 1L);
@@ -231,7 +247,8 @@ int vdelay3(CSOUND *csound, VDEL *p)    /*  vdelay routine with cubic interp */
     p->left = indx;             /*      and keep track of where you are */
     return OK;
  err1:
-    return csound->PerfError(csound, Str("vdelay3: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("vdelay3: not initialised"));
 }
 
 /* vdelayx, vdelayxs, vdelayxq, vdelayxw, vdelayxws, vdelayxwq */
@@ -244,7 +261,7 @@ int vdelxset(CSOUND *csound, VDELX *p)      /*  vdelayx set-up (1 channel) */
     if (UNLIKELY(n == 0)) n = 1;          /* fix due to Troxler */
 
     if (!*p->istod) {
-      if (p->aux1.auxp == NULL || (int32)(n * sizeof(MYFLT)) > p->aux1.size)
+      if (p->aux1.auxp == NULL || (uint32_t)(n * sizeof(MYFLT)) > p->aux1.size)
         /* allocate space for delay buffer */
         csound->AuxAlloc(csound, n * sizeof(MYFLT), &p->aux1);
       else
@@ -264,12 +281,12 @@ int vdelxsset(CSOUND *csound, VDELXS *p)    /*  vdelayxs set-up (stereo) */
     if (UNLIKELY(n == 0)) n = 1;          /* fix due to Troxler */
 
     if (!*p->istod) {
-      if (p->aux1.auxp == NULL || (int32)(n * sizeof(MYFLT)) > p->aux1.size)
+      if (p->aux1.auxp == NULL || (uint32_t)(n * sizeof(MYFLT)) > p->aux1.size)
         /* allocate space for delay buffer */
         csound->AuxAlloc(csound, n * sizeof(MYFLT), &p->aux1);
       else
         memset(p->aux1.auxp, 0, n*sizeof(MYFLT));
-      if (p->aux2.auxp == NULL || (int32)(n * sizeof(MYFLT)) > p->aux2.size)
+      if (p->aux2.auxp == NULL || (uint32_t)(n * sizeof(MYFLT)) > p->aux2.size)
         csound->AuxAlloc(csound, n * sizeof(MYFLT), &p->aux2);
       else
         memset(p->aux2.auxp, 0, n*sizeof(MYFLT));
@@ -289,20 +306,20 @@ int vdelxqset(CSOUND *csound, VDELXQ *p) /* vdelayxq set-up (quad channels) */
     if (UNLIKELY(n == 0)) n = 1;          /* fix due to Troxler */
 
     if (!*p->istod) {
-      if (p->aux1.auxp == NULL || (int32)(n * sizeof(MYFLT)) > p->aux1.size)
+      if (p->aux1.auxp == NULL || (uint32_t)(n * sizeof(MYFLT)) > p->aux1.size)
         /* allocate space for delay buffer */
         csound->AuxAlloc(csound, n * sizeof(MYFLT), &p->aux1);
       else
         memset(p->aux1.auxp, 0, n*sizeof(MYFLT));
-      if (p->aux2.auxp == NULL || (int32)(n * sizeof(MYFLT)) > p->aux2.size)
+      if (p->aux2.auxp == NULL || (uint32_t)(n * sizeof(MYFLT)) > p->aux2.size)
         csound->AuxAlloc(csound, n * sizeof(MYFLT), &p->aux2);
       else
         memset(p->aux2.auxp, 0, n*sizeof(MYFLT));
-      if (p->aux3.auxp == NULL || (int32)(n * sizeof(MYFLT)) > p->aux3.size)
+      if (p->aux3.auxp == NULL || (uint32_t)(n * sizeof(MYFLT)) > p->aux3.size)
         csound->AuxAlloc(csound, n * sizeof(MYFLT), &p->aux3);
       else
         memset(p->aux3.auxp, 0, n*sizeof(MYFLT));
-      if (p->aux4.auxp == NULL || (int32)(n * sizeof(MYFLT)) > p->aux4.size)
+      if (p->aux4.auxp == NULL || (uint32_t)(n * sizeof(MYFLT)) > p->aux4.size)
         csound->AuxAlloc(csound, n * sizeof(MYFLT), &p->aux4);
       else
         memset(p->aux4.auxp, 0, n*sizeof(MYFLT));
@@ -317,7 +334,10 @@ int vdelxqset(CSOUND *csound, VDELXQ *p) /* vdelayxq set-up (quad channels) */
 
 int vdelayx(CSOUND *csound, VDELX *p)               /*      vdelayx routine  */
 {
-    int32  nn, nsmps = csound->ksmps, maxd, indx;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nn, nsmps = CS_KSMPS;
+    int32 indx, maxd;
     MYFLT *out1 = p->sr1;  /* assign object data to local variables   */
     MYFLT *in1 = p->ain1;
     MYFLT *del = p->adel;
@@ -332,8 +352,13 @@ int vdelayx(CSOUND *csound, VDELX *p)               /*      vdelayx routine  */
     indx = p->left;
     i2 = (wsize >> 1);
     d2x = (1.0 - pow ((double)wsize * 0.85172, -0.89624)) / (double)(i2 * i2);
+    if (UNLIKELY(offset)) memset(out1, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out1[nsmps], '\0', early*sizeof(MYFLT));
+    }
 
-    for (nn=0; nn<nsmps; nn++) {
+    for (nn=offset; nn<nsmps; nn++) {
       buf1[indx] = in1[nn];
       n1 = 0.0;
 
@@ -374,12 +399,16 @@ int vdelayx(CSOUND *csound, VDELX *p)               /*      vdelayx routine  */
     p->left = indx;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("vdelay: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("vdelay: not initialised"));
 }
 
 int vdelayxw(CSOUND *csound, VDELX *p)      /*      vdelayxw routine  */
 {
-    int32  nn, maxd, indx, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nn, nsmps = CS_KSMPS;
+    int32  maxd, indx;
     MYFLT *out1 = p->sr1;  /* assign object data to local variables   */
     MYFLT *in1 = p->ain1;
     MYFLT *del = p->adel;
@@ -395,7 +424,12 @@ int vdelayxw(CSOUND *csound, VDELX *p)      /*      vdelayxw routine  */
     i2 = (wsize >> 1);
     d2x = (1.0 - pow ((double)wsize * 0.85172, -0.89624)) / (double)(i2 * i2);
 
-    for (nn=0;nn<nsmps;nn++) {
+    if (UNLIKELY(offset)) memset(out1, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out1[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (nn=offset;nn<nsmps;nn++) {
       /* x1: fractional part of delay time */
       /* x2: sine of x1 (for interpolation) */
       /* xpos: integer part of delay time (buffer position to read from) */
@@ -407,7 +441,7 @@ int vdelayxw(CSOUND *csound, VDELX *p)      /*      vdelayxw routine  */
       x2 = sin (PI * x1) / PI;
       while (xpos >= maxd) xpos -= maxd;
 
-      if (x1 * (1.0 - x1) > 0.00000001) {
+      if (LIKELY(x1 * (1.0 - x1) > 0.00000001)) {
         n1 = (double)in1[nn] * x2;
         xpos += (1 - i2);
         while (xpos < 0) xpos += maxd;
@@ -434,7 +468,8 @@ int vdelayxw(CSOUND *csound, VDELX *p)      /*      vdelayxw routine  */
     p->left = indx;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("vdelay: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("vdelay: not initialised"));
 }
 
 int vdelayxs(CSOUND *csound, VDELXS *p)     /*      vdelayxs routine  */
@@ -450,7 +485,9 @@ int vdelayxs(CSOUND *csound, VDELXS *p)     /*      vdelayxs routine  */
     int   wsize = p->interp_size;
     double x1, x2, w, d, d2x, n1, n2;
     int32   i, i2, xpos;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (UNLIKELY((buf1 == NULL) || (buf2 == NULL))) goto err1; /* RWD fix */
     maxd = (int32)(*p->imaxd * csound->esr);
@@ -458,8 +495,17 @@ int vdelayxs(CSOUND *csound, VDELXS *p)     /*      vdelayxs routine  */
     indx = p->left;
     i2 = (wsize >> 1);
     d2x = (1.0 - pow ((double)wsize * 0.85172, -0.89624)) / (double)(i2 * i2);
+    if (UNLIKELY(offset)) {
+      memset(out1, '\0', offset*sizeof(MYFLT));
+      memset(out2, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out1[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out2[nsmps], '\0', early*sizeof(MYFLT));
+    }
 
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       buf1[indx] = in1[n]; buf2[indx] = in2[n];
       n1 = 0.0; n2 = 0.0;
 
@@ -500,12 +546,16 @@ int vdelayxs(CSOUND *csound, VDELXS *p)     /*      vdelayxs routine  */
     p->left = indx;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("vdelay: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("vdelay: not initialised"));
 }
 
 int vdelayxws(CSOUND *csound, VDELXS *p)    /*      vdelayxws routine  */
 {
-    int32  nn, maxd, indx;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32  maxd, indx;
     MYFLT *out1 = p->sr1;  /* assign object data to local variables   */
     MYFLT *out2 = p->sr2;
     MYFLT *in1 = p->ain1;
@@ -520,17 +570,25 @@ int vdelayxws(CSOUND *csound, VDELXS *p)    /*      vdelayxws routine  */
     if (UNLIKELY((buf1 == NULL) || (buf2 == NULL))) goto err1;     /* RWD fix */
     maxd = (int32)(*p->imaxd * csound->esr);
     if (UNLIKELY(maxd == 0)) maxd = 1;    /* Degenerate case */
-    nn = csound->ksmps;
     indx = p->left;
     i2 = (wsize >> 1);
     d2x = (1.0 - pow ((double)wsize * 0.85172, -0.89624)) / (double)(i2 * i2);
 
-    do {
+    if (UNLIKELY(offset)) {
+      memset(out1, '\0', offset*sizeof(MYFLT));
+      memset(out2, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out1[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out2[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       /* x1: fractional part of delay time */
       /* x2: sine of x1 (for interpolation) */
       /* xpos: integer part of delay time (buffer position to read from) */
 
-      x1 = (double)indx + ((double)*del++ * (double)csound->esr);
+      x1 = (double)indx + ((double)del[n] * (double)csound->esr);
       while (UNLIKELY(x1 < 0.0)) x1 += (double)maxd;
       xpos = (int32)x1;
       x1 -= (double)xpos;
@@ -538,7 +596,7 @@ int vdelayxws(CSOUND *csound, VDELXS *p)    /*      vdelayxws routine  */
       while (UNLIKELY(xpos >= maxd)) xpos -= maxd;
 
       if (x1 * (1.0 - x1) > 0.00000001) {
-        n1 = (double)*in1 * x2; n2 = (double)*in2 * x2;
+        n1 = (double)in1[n] * x2; n2 = (double)in2[n] * x2;
         xpos += (1 - i2);
         while (UNLIKELY(xpos < 0)) xpos += maxd;
         d = (double)(1 - i2) - x1;
@@ -554,24 +612,27 @@ int vdelayxws(CSOUND *csound, VDELXS *p)    /*      vdelayxws routine  */
       else {                                            /* integer sample */
         xpos = (int32)((double)xpos + x1 + 0.5);       /* position */
         if (UNLIKELY(xpos >= maxd)) xpos -= maxd;
-        buf1[xpos] += *in1; buf2[xpos] += *in2;
+        buf1[xpos] += in1[n]; buf2[xpos] += in2[n];
       }
 
-      *out1++ = buf1[indx]; buf1[indx] = FL(0.0);
-      *out2++ = buf2[indx]; buf2[indx] = FL(0.0);
+      out1[n] = buf1[indx]; buf1[indx] = FL(0.0);
+      out2[n] = buf2[indx]; buf2[indx] = FL(0.0);
       if (UNLIKELY(++indx == maxd)) indx = 0;
-      in1++; in2++;
-    } while (--nn);
+    }
 
     p->left = indx;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("vdelay: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("vdelay: not initialised"));
 }
 
 int vdelayxq(CSOUND *csound, VDELXQ *p)     /*      vdelayxq routine  */
 {
-    int32  nn, maxd, indx;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32  maxd, indx;
     MYFLT *out1 = p->sr1;  /* assign object data to local variables   */
     MYFLT *out2 = p->sr2;
     MYFLT *out3 = p->sr3;
@@ -593,14 +654,27 @@ int vdelayxq(CSOUND *csound, VDELXQ *p)     /*      vdelayxq routine  */
                  (buf3 == NULL) || (buf4 == NULL))) goto err1;
     maxd = (int32)(*p->imaxd * csound->esr);
     if (UNLIKELY(maxd == 0)) maxd = 1;    /* Degenerate case */
-    nn = csound->ksmps;
     indx = p->left;
     i2 = (wsize >> 1);
     d2x = (1.0 - pow ((double)wsize * 0.85172, -0.89624)) / (double)(i2 * i2);
 
-    do {
-      buf1[indx] = *in1++; buf2[indx] = *in2++;
-      buf3[indx] = *in3++; buf4[indx] = *in4++;
+    if (UNLIKELY(offset)) {
+      memset(out1, '\0', offset*sizeof(MYFLT));
+      memset(out2, '\0', offset*sizeof(MYFLT));
+      memset(out3, '\0', offset*sizeof(MYFLT));
+      memset(out4, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out1[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out2[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out3[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out4[nsmps], '\0', early*sizeof(MYFLT));
+    }
+
+    for (n=offset; n<nsmps; n++) {
+      buf1[indx] = in1[n]; buf2[indx] = in2[n];
+      buf3[indx] = in3[n]; buf4[indx] = in4[n];
       n1 = 0.0; n2 = 0.0; n3 = 0.0; n4 = 0.0;
 
       /* x1: fractional part of delay time */
@@ -614,7 +688,7 @@ int vdelayxq(CSOUND *csound, VDELXQ *p)     /*      vdelayxq routine  */
       x2 = sin (PI * x1) / PI;
       while (UNLIKELY(xpos >= maxd)) xpos -= maxd;
 
-      if (x1 * (1.0 - x1) > 0.00000001) {
+      if (LIKELY(x1 * (1.0 - x1) > 0.00000001)) {
         xpos += (1 - i2);
         while (UNLIKELY(xpos < 0)) xpos += maxd;
         d = (double)(1 - i2) - x1;
@@ -622,35 +696,38 @@ int vdelayxq(CSOUND *csound, VDELXQ *p)     /*      vdelayxq routine  */
           w = 1.0 - d*d*d2x; w *= (w / d++);
           n1 += (double)buf1[xpos] * w; n2 += (double)buf2[xpos] * w;
           n3 += (double)buf3[xpos] * w; n4 += (double)buf4[xpos] * w;
-          if (++xpos >= maxd) xpos -= maxd;
+          if (UNLIKELY(++xpos >= maxd)) xpos -= maxd;
           w = 1.0 - d*d*d2x; w *= (w / d++);
           n1 -= (double)buf1[xpos] * w; n2 -= (double)buf2[xpos] * w;
           n3 -= (double)buf3[xpos] * w; n4 -= (double)buf4[xpos] * w;
-          if (++xpos >= maxd) xpos -= maxd;
+          if (UNLIKELY(++xpos >= maxd)) xpos -= maxd;
         }
-        *out1 = (MYFLT) (n1 * x2); *out2 = (MYFLT) (n2 * x2);
-        *out3 = (MYFLT) (n3 * x2); *out4 = (MYFLT) (n4 * x2);
+        out1[n] = (MYFLT) (n1 * x2); out2[n] = (MYFLT) (n2 * x2);
+        out3[n] = (MYFLT) (n3 * x2); out4[n] = (MYFLT) (n4 * x2);
       }
       else {                                            /* integer sample */
         xpos = (int32)((double)xpos + x1 + 0.5);       /* position */
         if (UNLIKELY(xpos >= maxd)) xpos -= maxd;
-        *out1 = buf1[xpos]; *out2 = buf2[xpos];
-        *out3 = buf3[xpos]; *out4 = buf4[xpos];
+        out1[n] = buf1[xpos]; out2[n] = buf2[xpos];
+        out3[n] = buf3[xpos]; out4[n] = buf4[xpos];
       }
 
       if (UNLIKELY(++indx == maxd)) indx = 0;
-      out1++; out2++; out3++; out4++;
-    } while (--nn);
+    }
 
     p->left = indx;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("vdelay: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("vdelay: not initialised"));
 }
 
 int vdelayxwq(CSOUND *csound, VDELXQ *p)    /*      vdelayxwq routine  */
 {
-    int32  nn, maxd, indx;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32  maxd, indx;
     MYFLT *out1 = p->sr1;  /* assign object data to local variables   */
     MYFLT *out2 = p->sr2;
     MYFLT *out3 = p->sr3;
@@ -672,17 +749,30 @@ int vdelayxwq(CSOUND *csound, VDELXQ *p)    /*      vdelayxwq routine  */
                  (buf3 == NULL) || (buf4 == NULL))) goto err1;
     maxd = (int32)(*p->imaxd * csound->esr);
     if (UNLIKELY(maxd == 0)) maxd = 1;    /* Degenerate case */
-    nn = csound->ksmps;
     indx = p->left;
     i2 = (wsize >> 1);
     d2x = (1.0 - pow ((double)wsize * 0.85172, -0.89624)) / (double)(i2 * i2);
 
-    do {
+    if (UNLIKELY(offset)) {
+      memset(out1, '\0', offset*sizeof(MYFLT));
+      memset(out2, '\0', offset*sizeof(MYFLT));
+      memset(out3, '\0', offset*sizeof(MYFLT));
+      memset(out4, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out1[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out2[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out3[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out4[nsmps], '\0', early*sizeof(MYFLT));
+    }
+
+    for (n=offset; n<nsmps; n++) {
       /* x1: fractional part of delay time */
       /* x2: sine of x1 (for interpolation) */
       /* xpos: integer part of delay time (buffer position to read from) */
 
-      x1 = (double)indx + ((double)*del++ * (double)csound->esr);
+      x1 = (double)indx + ((double)del[n] * (double)csound->esr);
       while (UNLIKELY(x1 < 0.0)) x1 += (double)maxd;
       xpos = (int32)x1;
       x1 -= (double)xpos;
@@ -690,8 +780,8 @@ int vdelayxwq(CSOUND *csound, VDELXQ *p)    /*      vdelayxwq routine  */
       while (UNLIKELY(xpos >= maxd)) xpos -= maxd;
 
       if (x1 * (1.0 - x1) > 0.00000001) {
-        n1 = (double)*in1 * x2; n2 = (double)*in2 * x2;
-        n3 = (double)*in3 * x2; n4 = (double)*in4 * x2;
+        n1 = (double)in1[n] * x2; n2 = (double)in2[n] * x2;
+        n3 = (double)in3[n] * x2; n4 = (double)in4[n] * x2;
         xpos += (1 - i2);
         while (UNLIKELY(xpos < 0)) xpos += maxd;
         d = (double)(1 - i2) - x1;
@@ -709,39 +799,39 @@ int vdelayxwq(CSOUND *csound, VDELXQ *p)    /*      vdelayxwq routine  */
       else {                                            /* integer sample */
         xpos = (int32)((double)xpos + x1 + 0.5);       /* position */
         if (UNLIKELY(xpos >= maxd)) xpos -= maxd;
-        buf1[xpos] += *in1; buf2[xpos] += *in2;
-        buf3[xpos] += *in3; buf4[xpos] += *in4;
+        buf1[xpos] += in1[n]; buf2[xpos] += in2[n];
+        buf3[xpos] += in3[n]; buf4[xpos] += in4[n];
       }
 
-      *out1++ = buf1[indx]; buf1[indx] = FL(0.0);
-      *out2++ = buf2[indx]; buf2[indx] = FL(0.0);
-      *out3++ = buf3[indx]; buf3[indx] = FL(0.0);
-      *out4++ = buf4[indx]; buf4[indx] = FL(0.0);
+      out1[n] = buf1[indx]; buf1[indx] = FL(0.0);
+      out2[n] = buf2[indx]; buf2[indx] = FL(0.0);
+      out3[n] = buf3[indx]; buf3[indx] = FL(0.0);
+      out4[n] = buf4[indx]; buf4[indx] = FL(0.0);
       if (UNLIKELY(++indx == maxd)) indx = 0;
-      in1++; in2++; in3++; in4++;
-    } while (--nn);
+    }
 
     p->left = indx;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("vdelay: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("vdelay: not initialised"));
 }
 
 int multitap_set(CSOUND *csound, MDEL *p)
 {
-    int32 n;
+    uint32_t n, i;
     MYFLT max = FL(0.0);
 
     //if (UNLIKELY(p->INOCOUNT/2 == (MYFLT)p->INOCOUNT*FL(0.5)))
     /* Should this test just be p->INOCOUNT&1 ==  */
     if (UNLIKELY((p->INOCOUNT&1)==0))
-      csound->Die(csound, Str("Wrong input count in multitap\n"));
+      csound->InitError(csound, Str("Wrong input count in multitap\n"));
 
-    for (n = 0; n < p->INOCOUNT - 1; n += 2) {
-      if (max < *p->ndel[n]) max = *p->ndel[n];
+    for (i = 0; i < p->INOCOUNT - 1; i += 2) {
+      if (max < *p->ndel[i]) max = *p->ndel[i];
     }
 
-    n = (int32)(csound->esr * max * sizeof(MYFLT));
+    n = (uint32_t)(csound->esr * max * sizeof(MYFLT));
     if (p->aux.auxp == NULL ||    /* allocate space for delay buffer */
         n > p->aux.size)
       csound->AuxAlloc(csound, n, &p->aux);
@@ -756,34 +846,43 @@ int multitap_set(CSOUND *csound, MDEL *p)
 
 int multitap_play(CSOUND *csound, MDEL *p)
 {                               /* assign object data to local variables   */
-    int  i, n, nn = csound->ksmps, indx = p->left, delay;
+    int  indx = p->left, delay;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, n, nsmps = CS_KSMPS;
     MYFLT *out = p->sr, *in = p->ain;
     MYFLT *buf = (MYFLT *)p->aux.auxp;
     MYFLT max = (MYFLT)p->max;
 
     if (UNLIKELY(buf==NULL)) goto err1;           /* RWD fix */
-    for (i=0; i<nn; i++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       MYFLT v = FL(0.0);
-      buf[indx] = in[i];        /*      Write input     */
+      buf[indx] = in[n];        /*      Write input     */
 
       if (UNLIKELY(++indx == max)) indx = 0;   /*      Advance input pointer   */
-      for (n = 0; n < p->INOCOUNT - 1; n += 2) {
-        delay = indx - (int32)(csound->esr * *p->ndel[n]);
+      for (i = 0; i < p->INOCOUNT - 1; i += 2) {
+        delay = indx - (int32)(csound->esr * *p->ndel[i]);
         if (UNLIKELY(delay < 0))
           delay += (int32)max;
-        v += buf[delay] * *p->ndel[n+1]; /*      Write output    */
+        v += buf[delay] * *p->ndel[i+1]; /*      Write output    */
       }
-      out[i] = v;
+      out[n] = v;
     }
     p->left = indx;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("multitap: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("multitap: not initialised"));
 }
 
 /*      nreverb coded by Paris Smaragdis 1994 and Richard Karpen 1998 */
 
-#define LOG001  (-6.9078)       /* log(.001) */
+#define LOG001  (-6.9077552789821370521)       /* log(.001) */
 
 static const int smallprime[] = {
   2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61,
@@ -911,9 +1010,9 @@ int reverbx_set(CSOUND *csound, NREV2 *p)
       FUNC *ftCombs;
       p->numCombs = (int) *p->inumCombs;
       /* Get user-defined set of comb constants from table */
-      if (UNLIKELY((ftCombs = csound->FTFind(csound, p->ifnCombs)) == NULL))
+      if (UNLIKELY((ftCombs = csound->FTnp2Find(csound, p->ifnCombs)) == NULL))
         return NOTOK;
-      if (UNLIKELY(ftCombs->flen < p->numCombs * 2)) {
+      if (UNLIKELY(ftCombs->flen < (uint32_t)p->numCombs * 2)) {
         return csound->InitError(csound, Str("reverbx; Combs ftable must have "
                                              "%d time and %d gain values"),
                                  p->numCombs, p->numCombs);
@@ -943,9 +1042,9 @@ int reverbx_set(CSOUND *csound, NREV2 *p)
     else {    /* Have user-defined set of alpas constants */
       FUNC *ftAlpas;
       p->numAlpas = (int) *p->inumAlpas;
-      if (UNLIKELY((ftAlpas = csound->FTFind(csound, p->ifnAlpas)) == NULL))
+      if (UNLIKELY((ftAlpas = csound->FTnp2Find(csound, p->ifnAlpas)) == NULL))
         return NOTOK;
-      if (UNLIKELY(ftAlpas->flen < p->numAlpas * 2)) {
+      if (UNLIKELY(ftAlpas->flen < (unsigned int)p->numAlpas * 2)) {
         return csound->InitError(csound, Str("reverbx; Alpas ftable must have"
                                              " %d time and %d gain values"),
                                          p->numAlpas, p->numAlpas);
@@ -966,8 +1065,8 @@ int reverbx_set(CSOUND *csound, NREV2 *p)
 
     /* Init variables */
     if (*p->istor == FL(0.0) ||
-        p->temp.auxp == NULL || p->temp.size<csound->ksmps * sizeof(MYFLT)) {
-      csound->AuxAlloc(csound, csound->ksmps * sizeof(MYFLT), &p->temp);
+        p->temp.auxp == NULL || p->temp.size<CS_KSMPS * sizeof(MYFLT)) {
+      csound->AuxAlloc(csound, CS_KSMPS * sizeof(MYFLT), &p->temp);
 
       n = 0;
       for (i = 0; i < p->numCombs; i++) {
@@ -1044,7 +1143,10 @@ int reverbx_set(CSOUND *csound, NREV2 *p)
 
 int reverbx(CSOUND *csound, NREV2 *p)
 {
-    int32   i, n, nsmps = csound->ksmps;
+    int32   i;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   *in, *out = p->out, *buf, *end;
     MYFLT   gain, z;
     MYFLT   hdif = *p->hdif;
@@ -1057,6 +1159,7 @@ int reverbx(CSOUND *csound, NREV2 *p)
     in = p->in;
     memcpy(buf, in, nsmps*sizeof(MYFLT));
     memset(out, 0,  nsmps*sizeof(MYFLT));
+    if (UNLIKELY(early)) nsmps -= early;
     if (*p->time != p->prev_time || *p->hdif != p->prev_hdif) {
       if (UNLIKELY(hdif > FL(1.0))) {
         csound->Warning(csound, Str("High frequency diffusion>1\n"));
@@ -1092,7 +1195,7 @@ int reverbx(CSOUND *csound, NREV2 *p)
       gain = p->c_gain[i];
       in = (MYFLT*) p->temp.auxp;
       out = p->out;
-      for (n=0;n<nsmps;n++) {
+      for (n=offset;n<nsmps;n++) {
         out[n] += *buf;
         *buf += p->z[i] * p->g[i];
         p->z[i] = *buf;
@@ -1107,13 +1210,11 @@ int reverbx(CSOUND *csound, NREV2 *p)
     for (i = 0; i < numAlpas; i++) {
       in = (MYFLT*) p->temp.auxp;
       out = p->out;
-      memcpy(in, out, nsmps*sizeof(MYFLT));
+      memcpy(in+offset, out+offset, (nsmps-offset)*sizeof(MYFLT));
       buf = p->pabuf_cur[i];
       end = p->abuf_cur[i + 1];
       gain = p->a_gain[i];
-      in = (MYFLT*) p->temp.auxp;
-      out = p->out;
-      for (n=0;n<nsmps;n++) {
+      for (n=offset;n<nsmps;n++) {
         z = *buf;
         *buf = gain * z + in[n];
         out[n] = z - gain * *buf;
@@ -1125,6 +1226,7 @@ int reverbx(CSOUND *csound, NREV2 *p)
 
     return OK;
  err1:
-    return csound->PerfError(csound, Str("reverbx: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("reverbx: not initialised"));
 }
 
