@@ -19,7 +19,9 @@
  * a new pulse should start, and that pulseinc can be negative.
  */
 
-#include "csdl.h"
+//#include "csdl.h"
+#include "csoundCore.h"
+#include "interlocks.h"
 #include <math.h>
 #include <limits.h>
 
@@ -72,9 +74,9 @@ void vosim_event(CSOUND* csound, VOSIM *p)
                       Str("vosim: zero kfund. 'Infinite' length event generated."));
     }
     else {
-        p->timrem = (int32)(csound->esr / fundabs);
+        p->timrem = (int32)(CS_ESR / fundabs);
         if (UNLIKELY(p->timrem == 0)) {
-          p->timrem = csound->ksmps;
+          p->timrem = CS_KSMPS;
           p->pulstogo = 0;
           csound->Warning(csound,
                           Str("vosim: kfund (%f) > sr. Generating ksmps silence."),
@@ -116,7 +118,9 @@ void vosim_pulse(CSOUND* csound, VOSIM *p)
 
 int vosim(CSOUND* csound, VOSIM *p)
 {
-    int32 nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT *ar = p->ar;
     MYFLT *ftdata;
     int32  lobits;
@@ -126,7 +130,12 @@ int vosim(CSOUND* csound, VOSIM *p)
     ftdata = ftp->ftable;
     lobits = ftp->lobits;
 
-    while (nsmps > 0) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       /* new event? */
       if (p->timrem == 0)
         vosim_event(csound, p);
@@ -138,24 +147,25 @@ int vosim(CSOUND* csound, VOSIM *p)
       if (p->pulstogo > 0) {
         /* produce one sample */
         p->pulsephs &= PHMASK;
-        *ar++ = *(ftdata + (p->pulsephs >> lobits)) * p->pulseamp;
+        ar[n] = *(ftdata + (p->pulsephs >> lobits)) * p->pulseamp;
         --p->timrem;
-        --nsmps;
         p->pulsephs += p->pulseinc;
       }
       else {
         /* silence after last pulse in burst: */
         /* bypass regular synthesis and fill output with zeros */
-        while (p->timrem && nsmps) {
-          *ar++ = FL(0.0);
+        while (p->timrem && n<nsmps) {
+          ar[n] = FL(0.0);
           --p->timrem;
-          --nsmps;
+          n++;
         }
+        n--;
       }
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("vosim: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("vosim: not initialised"));
 }
 
 
@@ -164,9 +174,9 @@ int vosim(CSOUND* csound, VOSIM *p)
 
 #define S(x)    sizeof(x)
 
-static OENTRY localops[] = {
-  { "vosim", S(VOSIM), 5, "a", "kkkkkkio", (SUBR)vosimset, NULL, (SUBR)vosim }
+static OENTRY vosim_localops[] = {
+  { "vosim", S(VOSIM), TR, 5, "a", "kkkkkkio", (SUBR)vosimset, NULL, (SUBR)vosim }
 };
 
 
-LINKAGE
+LINKAGE_BUILTIN(vosim_localops)
