@@ -114,24 +114,35 @@ extern "C" {
   PUBLIC int csoundModuleInit(CSOUND *csound)
   {
       int status = csound->AppendOpcode(csound,
-                                        (char*)"kampmidid",
+                                        (char*)"ampmidid.k",
                                         sizeof(KAMPMIDID),
+                                        0,
                                         3,
                                         (char*)"k",
                                         (char*)"ki",
                                         (int(*)(CSOUND*,void*)) KAMPMIDID::init_,
                                         (int(*)(CSOUND*,void*)) KAMPMIDID::kontrol_,
                                         (int (*)(CSOUND*,void*)) 0);
-
       status |= csound->AppendOpcode(csound,
-                                     (char*)"iampmidid",
+                                     (char*)"ampmidid.i",
                                      sizeof(IAMPMIDID),
+                                        0,
                                      1,
                                      (char*)"i",
                                      (char*)"ii",
                                      (int (*)(CSOUND*,void*)) IAMPMIDID::init_,
                                      (int (*)(CSOUND*,void*)) 0,
                                      (int (*)(CSOUND*,void*)) 0);
+      status |= csound->AppendOpcode(csound,
+                                        (char*)"ampmidid",
+                                        0xffff,
+                                        0,
+                                        0,
+                                        0,
+                                        0,
+                                        0,
+                                        0,
+                                        0);
       return status;
   }
 
