@@ -20,7 +20,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
     02111-1307 USA
 */
-#include "csdl.h"
+#include "csoundCore.h"
+#include "interlocks.h"
 #include "bbcut.h"
 #include <math.h>
 
@@ -78,7 +79,7 @@ static int BBCutMonoInit(CSOUND *csound, BBCUTMONO *p)
 
     /* allocate space- need no more than a half bar at current
        tempo and barlength */
-    M = ((size_t)(csound->esr*(*p->barlength)/(*p->bps)))*sizeof(MYFLT);
+    M = ((size_t)(CS_ESR*(*p->barlength)/(*p->bps)))*sizeof(MYFLT);
     if (p->repeatbuffer.auxp == NULL || p->repeatbuffer.size<M) {
       csound->AuxAlloc(csound, M, &p->repeatbuffer);
     }
@@ -94,7 +95,7 @@ static int BBCutMonoInit(CSOUND *csound, BBCUTMONO *p)
     /* samp per unit= samp per bar/ subdiv */
     /* = samp per beat * beats per bar /subdiv */
     /* =(samp per sec / beats per sec)* (beats per bar/subdiv)  */
-    p->samplesperunit = roundoffint(((MYFLT)csound->esr*(FL(1.0)/(*p->bps)))*
+    p->samplesperunit = roundoffint(((MYFLT)CS_ESR*(FL(1.0)/(*p->bps)))*
                                     (*p->barlength/(MYFLT)p->Subdiv));
 
     /* enveloping */
@@ -110,13 +111,19 @@ static int BBCutMonoInit(CSOUND *csound, BBCUTMONO *p)
 
 static int BBCutMono(CSOUND *csound, BBCUTMONO *p)
 {
-    int i;
-    int nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     int oddmax,unitproj;
     int unitb,unitl,unitd;      /* temp for integer unitblock calculations */
     MYFLT envmult,out;          /* intermedaites for enveloping grains */
 
-    for (i=0;i<nsmps;i++) {
+    if (UNLIKELY(offset)) memset(p->aout, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->aout[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset;i<nsmps;i++) {
       if (UNLIKELY((p->unitsdone+FL(0.000001))>=p->totalunits)) {
         /* a new phrase of cuts */
         p->numbarsnow  = random_number(csound, 1, p->Phrasebars);
@@ -308,7 +315,7 @@ static int BBCutStereoInit(CSOUND *csound, BBCUTSTEREO * p)
 
     /* allocate space- need no more than a half bar at current tempo
        and barlength */
-    M = 2*((size_t)(csound->esr*(*p->barlength)/(*p->bps)))*sizeof(MYFLT);
+    M = 2*((size_t)(CS_ESR*(*p->barlength)/(*p->bps)))*sizeof(MYFLT);
     if (p->repeatbuffer.auxp == NULL || p->repeatbuffer.size<M) {
       /* multiply by 2 for stereo buffer */
       csound->AuxAlloc(csound, M, &p->repeatbuffer);
@@ -324,7 +331,7 @@ static int BBCutStereoInit(CSOUND *csound, BBCUTSTEREO * p)
     /* samp per unit= samp per bar/ subdiv */
     /* = samp per beat * beats per bar /subdiv */
     /* =(samp per sec / beats per sec)* (beats per bar/subdiv)  */
-    p->samplesperunit = roundoffint(((MYFLT)csound->esr/
+    p->samplesperunit = roundoffint(((MYFLT)CS_ESR/
                                      (*p->bps))*(*p->barlength/
                                                  (MYFLT)p->Subdiv));
 
@@ -340,14 +347,25 @@ static int BBCutStereoInit(CSOUND *csound, BBCUTSTEREO * p)
 /* only make floating point corrections for stutters with stutterspeed>1 */
 static int BBCutStereo(CSOUND *csound, BBCUTSTEREO *p)
 {
-    int i;
-    int nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     int oddmax,unitproj;
     int unitb,unitl,unitd;      /* temp for integer unitblock calculations */
     MYFLT envmult,out1,out2;/* intermediates for enveloping grains */
 
-    for (i=0;i<nsmps;i++) {
-      if (UNLIKELY((p->unitsdone+FL(0.000001))>=p->totalunits)) {/* a new phrase of cuts */
+    if (UNLIKELY(offset)) {
+      memset(p->aout1, '\0', offset*sizeof(MYFLT));
+      memset(p->aout2, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->aout1[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&p->aout2[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset;i<nsmps;i++) {
+      /* a new phrase of cuts */
+      if (UNLIKELY((p->unitsdone+FL(0.000001))>=p->totalunits)) {
         p->numbarsnow  = random_number(csound, 1, p->Phrasebars);
         p->totalunits  = p->numbarsnow*p->Subdiv;
 
@@ -514,9 +532,9 @@ static int BBCutStereo(CSOUND *csound, BBCUTSTEREO *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "bbcutm",S(BBCUTMONO), 5, "a","aiiiiipop",
+  { "bbcutm",S(BBCUTMONO), 0, 5, "a","aiiiiipop",
                                  (SUBR)BBCutMonoInit, NULL, (SUBR)BBCutMono  },
-{ "bbcuts",S(BBCUTSTEREO),5, "aa","aaiiiiipop",
+  { "bbcuts",S(BBCUTSTEREO), 0, 5, "aa","aaiiiiipop",
                                (SUBR)BBCutStereoInit, NULL, (SUBR)BBCutStereo}
 };
 
