@@ -90,14 +90,14 @@ int bowedbarset(CSOUND *csound, BOWEDBAR *p)
 
     if (*p->lowestFreq>=FL(0.0)) {      /* If no init skip */
       if (*p->lowestFreq!=FL(0.0))
-        p->length = (int32) (csound->esr / *p->lowestFreq + FL(1.0));
+        p->length = (int32) (CS_ESR / *p->lowestFreq + FL(1.0));
       else if (*p->frequency!=FL(0.0))
-        p->length = (int32) (csound->esr / *p->frequency + FL(1.0));
+        p->length = (int32) (CS_ESR / *p->frequency + FL(1.0));
       else {
         csound->Warning(csound,
                         Str("unknown lowest frequency for bowed bar -- "
                             "assuming 50Hz\n"));
-        p->length = (int32) (csound->esr / FL(50.0) + FL(1.0));
+        p->length = (int32) (CS_ESR / FL(50.0) + FL(1.0));
       }
     }
 
@@ -128,7 +128,9 @@ int bowedbarset(CSOUND *csound, BOWEDBAR *p)
 int bowedbar(CSOUND *csound, BOWEDBAR *p)
 {
     MYFLT       *ar = p->ar;
-    int         n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       amp = (*p->amp)*AMP_RSCALE; /* Normalise */
     int32 k;
     int i;
@@ -141,7 +143,7 @@ int bowedbar(CSOUND *csound, BOWEDBAR *p)
       p->freq = *p->frequency;
       if (p->freq > FL(1568.0)) p->freq = FL(1568.0);
 
-      p->length = (int)(csound->esr/p->freq);
+      p->length = (int)(CS_ESR/p->freq);
       p->nr_modes = NR_MODES;   /* reset for frequency shift */
       for (i = 0; i<NR_MODES; i++) {
         if ((int)(p->length/p->modes[i]) > 4)
@@ -184,7 +186,12 @@ int bowedbar(CSOUND *csound, BOWEDBAR *p)
     }
     maxVelocity = FL(0.03) + (FL(0.5) * amp);
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       MYFLT data = FL(0.0);
       MYFLT input = FL(0.0);
       if (integration_const == FL(0.0))
