@@ -28,6 +28,10 @@
 #include <sys/types.h>
 #include <signal.h>
 
+#if defined(__MACH__)
+#include <unistd.h>
+#endif
+
 static CS_NOINLINE CONTROL_GLOBALS *get_globals_(CSOUND *csound)
 {
     CONTROL_GLOBALS *p;
@@ -36,8 +40,10 @@ static CS_NOINLINE CONTROL_GLOBALS *get_globals_(CSOUND *csound)
     if (p != NULL)
       return p;
     if (csound->CreateGlobalVariable(csound, "controlGlobals_",
-                                     sizeof(CONTROL_GLOBALS)) != 0)
-      csound->Die(csound, Str("control: failed to allocate globals"));
+                                     sizeof(CONTROL_GLOBALS)) != 0){
+      csound->Warning(csound, Str("control: failed to allocate globals"));
+      return NULL;
+    }
     p = (CONTROL_GLOBALS*) csound->QueryGlobalVariable(csound,
                                                        "controlGlobals_");
     p->csound = csound;
@@ -179,7 +185,7 @@ static int control(CSOUND *csound, CNTRL *p)
     return OK;
 }
 
-static int ocontrol(CSOUND *csound, SCNTRL *p)
+static int ocontrol_(CSOUND *csound, SCNTRL *p, int istring)
 {
     CONTROL_GLOBALS *pp = get_globals(csound, &(p->p));
     int c = (int) *p->which;
@@ -207,7 +213,12 @@ static int ocontrol(CSOUND *csound, SCNTRL *p)
     case 4:
       {
         char buffer[100];
-        csound->strarg2name(csound, buffer, p->val, "Control ", p->XSTRCODE);
+        if(istring) {
+          csound->strarg2name(csound, buffer,
+                              ((STRINGDAT *)p->val)->data, "Control ",istring);
+        }
+        else
+         csound->strarg2name(csound, buffer, p->val, "Control ",istring);
         csound->Message(csound, Str("Slider %d set to %s\n"), slider, buffer);
         fprintf(pp->wish_cmd, "setlab %d \"%s\"\n", slider, buffer);
         break;
@@ -218,6 +229,14 @@ static int ocontrol(CSOUND *csound, SCNTRL *p)
     return OK;
 }
 
+static int ocontrol(CSOUND *csound, SCNTRL *p){
+  return ocontrol_(csound,p,0);
+}
+
+static int ocontrol_S(CSOUND *csound, SCNTRL *p){
+  return ocontrol_(csound,p,1);
+}
+
 static int button_set(CSOUND *csound, CNTRL *p)
 {
     CONTROL_GLOBALS *pp = get_globals(csound, &(p->p));
@@ -272,7 +291,7 @@ static int check(CSOUND *csound, CNTRL *p)
 
 /* **** Text Windows **** */
 
-static int textflash(CSOUND *csound, TXTWIN *p)
+static int textflash_(CSOUND *csound, TXTWIN *p, int istring)
 {
     CONTROL_GLOBALS *pp = get_globals(csound, &(p->p));
     int   wind = (int) MYFLT2LONG(*p->kcntl);
@@ -280,11 +299,15 @@ static int textflash(CSOUND *csound, TXTWIN *p)
 
     if (pp->wish_pid == 0)
       start_tcl_tk(pp);
-    if (p->XSTRCODE || *p->val == SSTRCOD) {
-      csound->strarg2name(csound, buffer, p->val, "", p->XSTRCODE);
+    if (istring) {
+      csound->strarg2name(csound, buffer, ((STRINGDAT *)p->val)->data, "", istring);
 /*    csound->Message(csound, "settext %d \"%s\"\n", wind, buffer); */
       fprintf(pp->wish_cmd, "settext %d \"%s\"\n", wind, buffer);
     }
+    else if(ISSTRCOD(*p->val)) {
+      csound->strarg2name(csound, buffer,
+                          csound->GetString(csound, *p->val), "", 1);
+    }
     else {
 /*    csound->Message(csound, "deltext %d\n", wind); */
       fprintf(pp->wish_cmd, "deltext %d\n", wind);
@@ -292,15 +315,25 @@ static int textflash(CSOUND *csound, TXTWIN *p)
     return OK;
 }
 
+static int textflash(CSOUND *csound, TXTWIN *p){
+  return textflash_(csound, p, 0);
+}
+
+static int textflash_S(CSOUND *csound, TXTWIN *p){
+  return textflash_(csound, p, 1);
+}
+
+
 #define S(x)    sizeof(x)
 
 static OENTRY control_localops[] = {
-{ "control",  S(CNTRL),  3, "k", "k",  (SUBR) cntrl_set, (SUBR) control, NULL },
-{ "setctrl",  S(SCNTRL), 1, "",  "iTi", (SUBR) ocontrol, NULL, NULL           },
-{ "button",   S(CNTRL),  3, "k", "k",  (SUBR) button_set, (SUBR) button, NULL },
-{ "checkbox", S(CNTRL),  3, "k", "k",   (SUBR) check_set, (SUBR) check, NULL  },
-{ "flashtxt", S(TXTWIN), 1, "",  "iT",  (SUBR) textflash, NULL, NULL          },
+  { "control",  S(CNTRL), 0, 3, "k", "k", (SUBR) cntrl_set, (SUBR) control, NULL },
+{ "setctrl",  S(SCNTRL), 0, 1, "",  "iii", (SUBR) ocontrol, NULL, NULL           },
+{ "setctrl.S",  S(SCNTRL), 0, 1, "",  "iSi", (SUBR) ocontrol_S, NULL, NULL       },
+{ "button",   S(CNTRL),  0, 3, "k", "k",  (SUBR) button_set, (SUBR) button, NULL },
+{ "checkbox", S(CNTRL),  0, 3, "k", "k",   (SUBR) check_set, (SUBR) check, NULL  },
+{ "flashtxt", S(TXTWIN), 0, 1, "",  "ii",  (SUBR) textflash, NULL, NULL          },
+{ "flashtxt.S", S(TXTWIN), 0, 1, "",  "iS",  (SUBR) textflash_S, NULL, NULL      },
 };
 
-LINKAGE1(control_localops)
-
+LINKAGE_BUILTIN(control_localops)
