@@ -24,6 +24,7 @@
 #include "stdopcod.h"
 #include "ptrigtbl.h"
 #include "fhtfun.h"
+#include "interlocks.h"
 #include <math.h>
 
 #define CH_THRESH       1.19209e-7
@@ -307,7 +308,7 @@ static void pfht(MYFLT *fz, int32 n)
 
 static int Xsynthset(CSOUND *csound, CON *p)
 {
-    int32 flen, bufsize;
+    uint32_t    flen, bufsize;
     MYFLT       *b;
     FUNC        *ftp;
     MYFLT       ovlp = *p->ovlp;
@@ -334,7 +335,7 @@ static int Xsynthset(CSOUND *csound, CON *p)
     p->in1 = b;            b += 2 * flen;
     p->in2 = b;            b += 2 * flen;
 
-    if ((ftp = csound->FTFind(csound, p->iwin)) != NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->iwin)) != NULL)
       p->win = ftp;
     else return NOTOK;
 
@@ -346,8 +347,11 @@ static int Xsynthset(CSOUND *csound, CON *p)
 static int Xsynth(CSOUND *csound, CON *p)
 {
     MYFLT               *s, *f, *out, *buf1, *buf2, *outbuf, rfn;
-    int32                n, size, samps, div;
-    int32                m;
+    int32                size, div;
+    int32                n, m;
+    uint32_t             offset = p->h.insdshead->ksmps_offset;
+    uint32_t             early  = p->h.insdshead->ksmps_no_end;
+    uint32_t             nn, nsmps = CS_KSMPS;
 
     s = p->as;
     f = p->af;
@@ -363,11 +367,16 @@ static int Xsynth(CSOUND *csound, CON *p)
 
     n = p->count;
     m = n % div;
-    for (samps = 0; samps < csound->ksmps; samps++) {
-      buf1[n] = s[samps];
-      buf2[n] = f[samps];
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (nn = offset; nn < nsmps; nn++) {
+      buf1[n] = s[nn];
+      buf2[n] = f[nn];
 
-      out[samps] = outbuf[n];
+      out[nn] = outbuf[n];
       n++; m++;
       if (n == size) n = 0;     /* Moved to here from inside loop */
       if (m == div) {           /* wrap */
@@ -429,7 +438,7 @@ static int Xsynth(CSOUND *csound, CON *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "cross2",  S(CON), 5, "a", "aaiiik",(SUBR)Xsynthset, NULL, (SUBR)Xsynth}
+{ "cross2",  S(CON), TR, 5, "a", "aaiiik",(SUBR)Xsynthset, NULL, (SUBR)Xsynth}
 };
 
 int cross2_init_(CSOUND *csound)
