@@ -4,6 +4,7 @@
 #include <list>
 #include <vector>
 
+/* ***************  does not deal with unaligned signals ************** */
 // Why not use the constants already defined?
 static MYFLT pi = std::atan(1.0) * MYFLT(4.0);
 
@@ -79,7 +80,8 @@ public:
   }
   MYFLT delayFrames(int delayFrames_)
   {
-    //std::cout << "DelayLine::delayFrames: delayFrames: " << delayFrames_ << std::endl;
+    //std::cout << "DelayLine::delayFrames: delayFrames: "
+      //        << delayFrames_ << std::endl;
     int readingFrame = writingFrame - delayFrames_;
     while (readingFrame < 0) {
       readingFrame += size_;
@@ -87,7 +89,8 @@ public:
     while (readingFrame >= size_) {
       readingFrame -= size_;
     }
-    // std::cout << "DelayLine::delayFrames: readingFrame: " << readingFrame << std::endl;
+    // std::cout << "DelayLine::delayFrames: readingFrame: "
+    //           << readingFrame << std::endl;
     return (*this)[(size_t) readingFrame];
   }
 };
@@ -123,8 +126,8 @@ public:
   int init(CSOUND *csound)
   {
     sampleRate = csound->GetSr(csound);
-    blockSize = csound->GetKsmps(csound);
-    blockRate = sampleRate / blockSize;
+    blockRate = opds.insdshead->ekr;
+    blockSize = opds.insdshead->ksmps;
     // Take care of default values.
     if (*jSpeedOfSound == MYFLT(-1.0)) {
       speedOfSound = MYFLT(340.29);
@@ -165,18 +168,24 @@ public:
     // because it must be initialized from a k-rate variable.
     if (!smoothingFilter) {
       smoothingFilter = new RCLowpassFilter();
-      smoothingFilter->initialize(sampleRate, smoothingFilterCutoff, targetPosition);
+      smoothingFilter->initialize(sampleRate,
+                                  smoothingFilterCutoff, targetPosition);
       log(csound, "Doppler::kontrol: sizeof(MYFLT):         %10d\n", sizeof(MYFLT));
       log(csound, "Doppler::kontrol: PI:                    %10.3f\n", pi);
       log(csound, "Doppler::kontrol: this:                  %10p\n", this);
       log(csound, "Doppler::kontrol: sampleRate:            %10.3f\n", sampleRate);
       log(csound, "Doppler::kontrol: blockSize:             %10.3f\n", blockSize);
       log(csound, "Doppler::kontrol: blockRate:             %10.3f\n", blockRate);
-      log(csound, "Doppler::kontrol: speedOfSound:          %10.3f\n", speedOfSound);
-      log(csound, "Doppler::kontrol: samplesPerDistance:    %10.3f\n", samplesPerDistance);
-      log(csound, "Doppler::kontrol: smoothingFilterCutoff: %10.3f\n", smoothingFilterCutoff);
-      log(csound, "Doppler::kontrol: kMicPosition:          %10.3f\n", *kMicPosition);
-      log(csound, "Doppler::kontrol: kSourcePosition:       %10.3f\n", *kSourcePosition);
+      log(csound, "Doppler::kontrol: speedOfSound:          %10.3f\n",
+          speedOfSound);
+      log(csound, "Doppler::kontrol: samplesPerDistance:    %10.3f\n",
+          samplesPerDistance);
+      log(csound, "Doppler::kontrol: smoothingFilterCutoff: %10.3f\n",
+          smoothingFilterCutoff);
+      log(csound, "Doppler::kontrol: kMicPosition:          %10.3f\n",
+          *kMicPosition);
+      log(csound, "Doppler::kontrol: kSourcePosition:       %10.3f\n",
+          *kSourcePosition);
     }
 
     for (size_t outputFrame = 0;
@@ -215,7 +224,7 @@ public:
                 audioBufferQueue->pop_front();
             }
             delete audioBufferQueue;
-            audioBufferQueue = 0;            
+            audioBufferQueue = 0;
         }
         if (sourcePositionQueue) {
             delete sourcePositionQueue;
@@ -240,6 +249,7 @@ extern "C"
       {
         (char*)"doppler",
         sizeof(Doppler),
+        0,
         3,
         (char*)"a",
         (char*)"akkjj",
@@ -256,6 +266,8 @@ extern "C"
         0,
         0,
         0,
+        0,
+        0,
       }
     };
 
@@ -270,7 +282,8 @@ extern "C"
     for(OENTRY *oentry = &oentries[0]; oentry->opname; oentry++)
       {
         status |= csound->AppendOpcode(csound, oentry->opname,
-                                       oentry->dsblksiz, oentry->thread,
+                                       oentry->dsblksiz, oentry->flags,
+                                       oentry->thread,
                                        oentry->outypes, oentry->intypes,
                                        (int (*)(CSOUND*,void*)) oentry->iopadr,
                                        (int (*)(CSOUND*,void*)) oentry->kopadr,
@@ -282,7 +295,7 @@ extern "C"
   PUBLIC int csoundModuleDestroy(CSOUND *csound)
   {
     //csound->Message(csound, "Deleting C++ objects from doppler...\n");
-    for (std::list<RCLowpassFilter *>::iterator it = smoothingFilterInstances.begin();
+    for (std::list<RCLowpassFilter *>::iterator it=smoothingFilterInstances.begin();
          it != smoothingFilterInstances.end();
          ++it) {
       delete *it;
@@ -297,4 +310,3 @@ extern "C"
     return 0;
   }
 }
-
