@@ -24,12 +24,15 @@
 #include <dlfcn.h>
 #include <dirent.h>
 
+#undef CS_KSMPS
+#define CS_KSMPS     (csound->GetKsmps(csound))
+
 #ifdef BETA
 #define DEBUG 1
 #endif
 #define DSSI4CS_MAX_NUM_EVENTS 128
 
-static const char   *version = "0.1alpha";
+//static const char   *version = "0.1alpha";
 
 /* TODO accomodate plugins which return control outputs */
 
@@ -38,7 +41,7 @@ static const char   *version = "0.1alpha";
 *****************************************************************************/
 void info(CSOUND * csound, DSSI4CS_PLUGIN * DSSIPlugin_)
 {
-    int     Ksmps = csound->ksmps;
+    int     Ksmps = csound->GetKsmps(csound);
     unsigned long PortCount;
     LADSPA_Descriptor *Descriptor;
     uint32 i;
@@ -118,7 +121,7 @@ DSSI4CS_PLUGIN *LocatePlugin(int PluginNumber, CSOUND * csound)
     if (!DSSIPlugin)
       return NULL;
     DSSIPlugin_ = DSSIPlugin;
-    if (PluginNumber > *DSSIPlugin->PluginCount) {
+    if (UNLIKELY(PluginNumber > *DSSIPlugin->PluginCount)) {
 #ifdef DEBUG
       csound->Message(csound,
                       "DSSI4CS: PluginNumber > *DSSIPlugin->PluginCount.\n");
@@ -132,7 +135,7 @@ DSSI4CS_PLUGIN *LocatePlugin(int PluginNumber, CSOUND * csound)
                       DSSIPlugin_->PluginNumber);
 #endif
 
-      if (!DSSIPlugin_->NextPlugin) {
+      if (UNLIKELY(!DSSIPlugin_->NextPlugin)) {
         return NULL;
       }
       DSSIPlugin_ = DSSIPlugin_->NextPlugin;
@@ -181,13 +184,13 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
 {
     /* TODO check if plugin has already been loaded and use same function */
     csound = p->h.insdshead->csound;
-    int     SampleRate = (int) MYFLT2LRND(csound->esr);
-    int     Ksmps = csound->ksmps;
-    int     i;
+    int     SampleRate = (int) MYFLT2LRND(csound->GetSr(csound));
+    int     Ksmps = csound->GetKsmps(csound);
+    unsigned long     i;
     LADSPA_Descriptor_Function pfDescriptorFunction;
     DSSI_Descriptor_Function pfDSSIDescriptorFunction;
     LADSPA_Descriptor *LDescriptor;
-    char    dssiFilename[0x100];
+    char    dssiFilename[MAXNAME];
     unsigned long PluginIndex;
     void   *PluginLibrary;
     unsigned long PortCount;
@@ -198,26 +201,21 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
     DSSI4CS_PLUGIN *DSSIPlugin =
         (DSSI4CS_PLUGIN *) csound->QueryGlobalVariable(csound, "$DSSI4CS");
 
-    if (!DSSIPlugin) {
-      csound->Message(csound, "=============================="
-                              "===============================\n");
-      csound->Message(csound, "dssi4cs version %s by Andres Cabrera\n",
-                      version);
-      csound->Message(csound,
-                      "Using code by Richard Furse from the LADSPA SDK.\n");
-      csound->Message(csound, "=============================="
-                              "===============================\n");
-    }
-    csound->strarg2name(csound, dssiFilename, p->iplugin, "dssiinit.",
-                                (int) csound->GetInputArgSMask(p));
+    if(csound->GetInputArgSMask(p))
+      strncpy(dssiFilename,((STRINGDAT *)p->iplugin)->data, MAXNAME-1);
+    else
+      csound->strarg2name(csound, dssiFilename, ISSTRCOD(*p->iplugin) ?
+                          csound->GetString(csound, *p->iplugin) :
+                          (char *) p->iplugin, "dssiinit.",
+                          (int) ISSTRCOD(*p->iplugin));
     PluginIndex = (unsigned long) *p->iindex;
     PluginLibrary = dlopenLADSPA(csound, dssiFilename, RTLD_NOW);
-    if (!PluginLibrary)
+    if (UNLIKELY(!PluginLibrary))
       return csound->InitError(csound, "DSSI4CS: Failed to load %s.",
                                        dssiFilename);
     if (!DSSIPlugin) {
-      if (csound->CreateGlobalVariable(csound, "$DSSI4CS",
-                                               sizeof(DSSI4CS_PLUGIN)) != 0)
+      if (UNLIKELY(csound->CreateGlobalVariable(csound, "$DSSI4CS",
+                                                sizeof(DSSI4CS_PLUGIN))) != 0)
         csound->Die(csound, "Error creating global variable '$DSSI4CS'");
       DSSIPlugin = (DSSI4CS_PLUGIN *) csound->QueryGlobalVariable(csound,
                                                                   "$DSSI4CS");
@@ -227,16 +225,16 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
       DSSIPlugin->PluginCount = (int *) csound->Malloc(csound, sizeof(int));
       *DSSIPlugin->PluginCount = 1;
       DSSIPlugin_ = DSSIPlugin;
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: Loading first instance.\n");
-#endif
+      if (p->iverbose != 0) {
+        csound->Message(csound, "DSSI4CS: Loading first instance.\n");
+      }
     }
     else {
       DSSIPlugin_ = LocatePlugin(*DSSIPlugin->PluginCount - 1, csound);
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: Located plugin: %i.\n",
-                      DSSIPlugin_->PluginNumber);
-#endif
+      if (p->iverbose != 0) {
+        csound->Message(csound, "DSSI4CS: Located plugin: %i.\n",
+                        DSSIPlugin_->PluginNumber);
+      }
       DSSIPlugin_->NextPlugin =
           (DSSI4CS_PLUGIN *) csound->Malloc(csound, sizeof(DSSI4CS_PLUGIN));
       DSSIPlugin_ = DSSIPlugin_->NextPlugin;
@@ -245,11 +243,11 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
       *DSSIPlugin_->PluginCount = (*DSSIPlugin_->PluginCount) + 1;
     }
     *p->iDSSIHandle = DSSIPlugin_->PluginNumber;
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: About to load descriptor function "
-                            "for plugin %i of %i.\n",
-                    DSSIPlugin_->PluginNumber, *DSSIPlugin_->PluginCount);
-#endif
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: About to load descriptor function "
+                      "for plugin %i of %i.\n",
+                      DSSIPlugin_->PluginNumber, *DSSIPlugin_->PluginCount);
+    }
     pfDSSIDescriptorFunction =
         (DSSI_Descriptor_Function) dlsym(PluginLibrary, "dssi_descriptor");
     if (pfDSSIDescriptorFunction) {
@@ -260,9 +258,9 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
       LDescriptor =
           (LADSPA_Descriptor *) DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin;
       DSSIPlugin_->Type = DSSI;
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: DSSI Plugin detected.\n");
-#endif
+      if (p->iverbose != 0) {
+        csound->Message(csound, "DSSI4CS: DSSI Plugin detected.\n");
+      }
     }
     else {
       pfDescriptorFunction =
@@ -275,11 +273,12 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
           (LADSPA_Descriptor *) pfDescriptorFunction(PluginIndex);
       LDescriptor = (LADSPA_Descriptor *) DSSIPlugin_->Descriptor;
       DSSIPlugin_->Type = LADSPA;
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: LADSPA Plugin Detected\n");
-#endif
+      if (p->iverbose != 0) {
+        csound->Message(csound, "DSSI4CS: LADSPA Plugin Detected\n");
+      }
     }
-    if ((!DSSIPlugin_->Descriptor) && (!DSSIPlugin_->DSSIDescriptor)) {
+    if (UNLIKELY((!DSSIPlugin_->Descriptor) &&
+                 (!DSSIPlugin_->DSSIDescriptor))) {
       const char *pcError = dlerror();
 
       /* TODO: cleanup if error */
@@ -302,38 +301,38 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
                                   dssiFilename);
       return NOTOK;
     }
-    if (!LDescriptor) {
+    if (UNLIKELY(!LDescriptor)) {
       csound->InitError(csound, "DSSI4CS: No plugin index %lu in %s",
                                 PluginIndex, dssiFilename);
       return NOTOK;
     }
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: About to instantiate plugin.\n");
-#endif
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: About to instantiate plugin.\n");
+    }
 
     if (DSSIPlugin_->Type == LADSPA) {
-      if (!
+      if (UNLIKELY(!
           (DSSIPlugin_->Handle =
            (LADSPA_Handle) DSSIPlugin_->Descriptor->instantiate(DSSIPlugin_->
                                                                 Descriptor,
-                                                                SampleRate)))
+                                                                SampleRate))))
         return csound->InitError(csound,
                                  "DSSI4CS: Could not instantiate plugin: %s",
                                  dssiFilename);
-      if (!DSSIPlugin_->Descriptor->run)
+      if (UNLIKELY(!DSSIPlugin_->Descriptor->run))
         return csound->InitError(csound, "DSSI4CS: No run() funtion in: %s",
                                          LDescriptor->Name);
       PortCount = DSSIPlugin_->Descriptor->PortCount;
     }
     else {
-      if (!
+      if (UNLIKELY(!
           (DSSIPlugin_->Handle =
            (LADSPA_Handle) DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->
-           instantiate(DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin, SampleRate)))
+           instantiate(DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin, SampleRate))))
         return csound->InitError(csound,
                                  "DSSI4CS: Could not instantiate plugin: %s",
                                  dssiFilename);
-      if (!DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->run)
+      if (UNLIKELY(!DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->run))
         return csound->InitError(csound, "DSSI4CS: No run() funtion in: %s",
                                          LDescriptor->Name);
       PortCount = DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->PortCount;
@@ -342,12 +341,12 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
                                              DSSI4CS_MAX_NUM_EVENTS
                                              * sizeof(snd_seq_event_t));
     }
-#ifdef DEBUG
-    if (DSSIPlugin_->Handle)
-      csound->Message(csound, "DSSI4CS: Plugin instantiated.\n");
-    else
-      csound->Message(csound, "DSSI4CS: Problem instantiating.\n");
-#endif
+    if (p->iverbose != 0) {
+      if (DSSIPlugin_->Handle)
+        csound->Message(csound, "DSSI4CS: Plugin instantiated.\n");
+      else
+        csound->Message(csound, "DSSI4CS: Problem instantiating.\n");
+    }
 
     for (i = 0; i < PortCount; i++) {
       PortDescriptor =
@@ -360,12 +359,12 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
       if (LADSPA_IS_PORT_AUDIO(PortDescriptor))
         ConnectedAudioPorts++;
     }
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: Found %lu control ports for: '%s'\n",
-                    ConnectedControlPorts, LDescriptor->Name);
-    csound->Message(csound, "DSSI4CS: Found %lu audio ports for: '%s'\n",
-                    ConnectedAudioPorts, LDescriptor->Name);
-#endif
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: Found %lu control ports for: '%s'\n",
+                      ConnectedControlPorts, LDescriptor->Name);
+      csound->Message(csound, "DSSI4CS: Found %lu audio ports for: '%s'\n",
+                      ConnectedAudioPorts, LDescriptor->Name);
+    }
 
     DSSIPlugin_->control =
         (LADSPA_Data **) csound->Calloc(csound, ConnectedControlPorts
@@ -373,9 +372,9 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
     DSSIPlugin_->audio =
         (LADSPA_Data **) csound->Calloc(csound, ConnectedAudioPorts
                                                 * sizeof(LADSPA_Data *));
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: Created port array.\n");
-#endif
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: Created port array.\n");
+    }
 
     ConnectedControlPorts = 0;
     ConnectedAudioPorts = 0;
@@ -384,27 +383,30 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
           (DSSIPlugin_->Type ==
            LADSPA ? DSSIPlugin_->Descriptor->PortDescriptors[i]
            : DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->PortDescriptors[i]);
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: Queried port descriptor.\n");
-#endif
+      if (p->iverbose != 0) {
+        csound->Message(csound, "DSSI4CS: Queried port descriptor.\n");
+      }
 
       if (LADSPA_IS_PORT_CONTROL(PortDescriptor)) {
         DSSIPlugin_->control[ConnectedControlPorts] =
             (LADSPA_Data *) csound->Calloc(csound, sizeof(LADSPA_Data));
-        if (DSSIPlugin_->Type == LADSPA)
+        if (DSSIPlugin_->Type == LADSPA) {
           DSSIPlugin_->Descriptor->connect_port(DSSIPlugin_->Handle,
                                                 i,
                                                 (LADSPA_Data *) DSSIPlugin_->
                                                 control[ConnectedControlPorts]);
-        else
+        }
+        else {
           DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->connect_port(
               DSSIPlugin_->Handle, i,
               (LADSPA_Data *) DSSIPlugin_->control[ConnectedControlPorts]);
-#ifdef DEBUG
-        csound->Message(csound,
-                        "DSSI4CS: Created control port %lu for Port %i.\n",
-                        ConnectedControlPorts, i);
-#endif
+        }
+        if (p->iverbose != 0) {
+          csound->Message(csound,
+                          "DSSI4CS: Created internal control port "
+                          "%lu for Port %i.\n",
+                          ConnectedControlPorts, i);
+        }
 
         ConnectedControlPorts++;
       }
@@ -420,31 +422,32 @@ int dssiinit(CSOUND * csound, DSSIINIT * p)
           DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin->connect_port(
               DSSIPlugin_->Handle, i,
               (LADSPA_Data *) DSSIPlugin_->audio[ConnectedAudioPorts]);
-#ifdef DEBUG
-        csound->Message(csound,
-                        "DSSI4CS: Created audio port %lu for Port %i.\n",
-                        ConnectedAudioPorts, i);
-#endif
+        if (p->iverbose != 0) {
+          csound->Message(csound,
+                          "DSSI4CS: Created internal audio port %lu for Port %i.\n",
+                          ConnectedAudioPorts, i);
+        }
 
         ConnectedAudioPorts++;
       }
 
     }
     /* All ports must be connected before calling run() */
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: Created %lu control ports for: '%s'\n",
-                    ConnectedControlPorts, LDescriptor->Name);
-    csound->Message(csound, "DSSI4CS: Created %lu audio ports for: '%s'\n",
-                    ConnectedAudioPorts, LDescriptor->Name);
-#endif
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: Created %lu control ports for: '%s'\n",
+                      ConnectedControlPorts, LDescriptor->Name);
+      csound->Message(csound, "DSSI4CS: Created %lu audio ports for: '%s'\n",
+                      ConnectedAudioPorts, LDescriptor->Name);
+    }
 
     DSSIPlugin_->Active = 0;
     DSSIPlugin_->EventCount = 0;
     if (*p->iverbose != 0)
       info(csound, DSSIPlugin_);
-#ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: Init Done.\n");
-#endif
+
+    if (p->iverbose != 0) {
+      csound->Message(csound, "DSSI4CS: Init Done.\n");
+    }
 
     return OK;
 }
@@ -507,8 +510,8 @@ int dssiactivate_init(CSOUND * csound, DSSIACTIVATE * p)
 
     p->DSSIPlugin_ = LocatePlugin(Number, csound);
     p->printflag = -999;
-    if ((!p->DSSIPlugin_) || (Number > *p->DSSIPlugin_->PluginCount) ||
-        (!p->DSSIPlugin_->Handle))
+    if (UNLIKELY((!p->DSSIPlugin_) || (Number > *p->DSSIPlugin_->PluginCount) ||
+                 (!p->DSSIPlugin_->Handle)))
       return csound->InitError(csound, "DSSI4CS: Invalid plugin: %i (MAX= %i).",
                                        Number, *p->DSSIPlugin_->PluginCount);
 #ifdef DEBUG
@@ -573,8 +576,9 @@ int dssiactivate(CSOUND * csound, DSSIACTIVATE * p)
     case -100:
       {
         if (p->printflag != -100) {
-          return csound->PerfError(csound, "DSSI4CS: dssiactivate "
-                                           "not properly initialised.");
+          return csound->PerfError(csound, p->h.insdshead,
+                                   "DSSI4CS: dssiactivate "
+                                   "not properly initialised.");
           p->printflag = -100;
         }
       }
@@ -593,6 +597,20 @@ int dssiaudio_init(CSOUND * csound, DSSIAUDIO * p)
 {
     /* TODO not realtime safe, try to make it so. */
     int     Number = *p->iDSSIhandle;
+    int     icnt = csound->GetInputArgCnt(p) - 1;
+    int     ocnt = csound->GetOutputArgCnt(p);
+
+    if (UNLIKELY(icnt > DSSI4CS_MAX_IN_CHANNELS))
+      csound->Die(csound,
+                  Str("DSSI4CS: number of audio input channels "
+                      "is greater than %d"),
+                  DSSI4CS_MAX_IN_CHANNELS);
+
+    if (UNLIKELY(ocnt > DSSI4CS_MAX_OUT_CHANNELS))
+      csound->Die(csound,
+                  Str("DSSI4CS: number of audio output channels "
+                      "is greater than %d"),
+                  DSSI4CS_MAX_OUT_CHANNELS);
 
 #ifdef DEBUG
     csound->Message(csound,
@@ -614,10 +632,10 @@ int dssiaudio_init(CSOUND * csound, DSSIAUDIO * p)
       Descriptor =
           (LADSPA_Descriptor *) p->DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin;
 
-    long    PortIndex = 0;
-    int     ConnectedInputPorts = 0;
-    int     ConnectedOutputPorts = 0;
-    int     ConnectedPorts = 0;
+    unsigned long PortIndex = 0;
+    int           ConnectedInputPorts = 0;
+    int           ConnectedOutputPorts = 0;
+    int           ConnectedPorts = 0;
     LADSPA_PortDescriptor PortDescriptor = 0;
 
     for (PortIndex = 0; PortIndex < Descriptor->PortCount; PortIndex++) {
@@ -637,10 +655,6 @@ int dssiaudio_init(CSOUND * csound, DSSIAUDIO * p)
     ConnectedOutputPorts = 0;
     ConnectedPorts = 0;
     for (PortIndex = 0; PortIndex < Descriptor->PortCount; PortIndex++) {
-#ifdef DEBUG
-      csound->Message(csound, "DSSI4CS: Port Index: %lu\n", PortIndex);
-#endif
-
       PortDescriptor = Descriptor->PortDescriptors[PortIndex];
       if (LADSPA_IS_PORT_INPUT(PortDescriptor)
           && LADSPA_IS_PORT_AUDIO(PortDescriptor)) {
@@ -679,7 +693,7 @@ int dssiaudio_init(CSOUND * csound, DSSIAUDIO * p)
 
     p->NumInputPorts = ConnectedInputPorts;
     p->NumOutputPorts = ConnectedOutputPorts;
-    if ((p->NumInputPorts) < (csound->GetInputArgCnt(p) - 1)) {
+    if ((p->NumInputPorts) < icnt) {
       if (p->NumInputPorts == 0)
         csound->Message(csound, "DSSI4CS: Plugin '%s' has %i audio input ports "
                                 "audio input discarded.\n",
@@ -689,7 +703,7 @@ int dssiaudio_init(CSOUND * csound, DSSIAUDIO * p)
                                          "has %i audio input ports.",
                                          Descriptor->Name, p->NumOutputPorts);
     }
-    if (p->NumOutputPorts < csound->GetOutputArgCnt(p))
+    if (p->NumOutputPorts < ocnt)
       return csound->InitError(csound, "DSSI4CS: Plugin '%s' "
                                        "has %i audio output ports.",
                                        Descriptor->Name, p->NumOutputPorts);
@@ -705,22 +719,22 @@ int dssiaudio(CSOUND * csound, DSSIAUDIO * p)
     else
       Descriptor =
           (LADSPA_Descriptor *) p->DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin;
-    int     i, j;
-    int     icnt = csound->GetInputArgCnt(p) - 1;
-    int     ocnt = csound->GetOutputArgCnt(p);
-    unsigned long Ksmps = (unsigned long) csound->ksmps;
+    unsigned int i, j;
+    unsigned int icnt = csound->GetInputArgCnt(p) - 1;
+    unsigned int ocnt = csound->GetOutputArgCnt(p);
+    unsigned long Ksmps = (unsigned long) csound->GetKsmps(csound);
 
     if (p->DSSIPlugin_->Active == 1) {
       for (j = 0; j < icnt; j++) {
         for (i = 0; i < Ksmps; i++)
           p->DSSIPlugin_->audio[p->InputPorts[j]][i] =
-              p->ain[j][i] * csound->dbfs_to_float;
+            p->ain[j][i] * (1.0/csound->Get0dBFS(csound));
       }
       Descriptor->run(p->DSSIPlugin_->Handle, Ksmps);
       for (j = 0; j < ocnt; j++) {
         for (i = 0; i < Ksmps; i++)
           p->aout[j][i] =
-              p->DSSIPlugin_->audio[p->OutputPorts[j]][i] * csound->e0dbfs;
+            p->DSSIPlugin_->audio[p->OutputPorts[j]][i] * csound->Get0dBFS(csound);
       }
     }
     else {
@@ -738,17 +752,19 @@ int dssiaudio(CSOUND * csound, DSSIAUDIO * p)
 int dssictls_kk(CSOUND * csound, DSSICTLS * p)
 {
     LADSPA_Data Value = *p->val;
-
-    if (!p->DSSIPlugin_)
-      return csound->PerfError(csound, "DSSI4CS: Invalid plugin handle.");
-    if (*p->ktrig == 1)
+    if (!p->DSSIPlugin_) {
+      return csound->PerfError(csound, p->h.insdshead,
+                               "DSSI4CS: Invalid plugin handle.");
+    }
+    if (*p->ktrig == 1) {
       *p->DSSIPlugin_->control[p->PortNumber] = Value * (p->HintSampleRate);
+    }
     return OK;
 }
 
 int dssictls_ak(CSOUND * csound, DSSICTLS * p)
 {
-    csound->PerfError(csound,
+    csound->PerfError(csound, p->h.insdshead,
                       "DSSI4CS: Audio Rate control ports not implemented yet.");
     return NOTOK;
 }
@@ -759,21 +775,24 @@ int dssictls_init(CSOUND * csound, DSSICTLS * p)
             Crash if audio port selected */
     const LADSPA_Descriptor *Descriptor;
     int     Number = *p->iDSSIhandle;
-    int     Sr = (int) MYFLT2LRND(csound->esr);
+    int     Sr = (int) MYFLT2LRND(csound->GetSr(csound));
     unsigned long PortIndex = *p->iport;
-    int     i;
+    unsigned int  i;
     unsigned long ControlPort = 0;
     unsigned long AudioPort = 0;
     unsigned long Port = 0;
 
     p->DSSIPlugin_ = LocatePlugin(Number, csound);
-    if (!p->DSSIPlugin_)
+    if (!p->DSSIPlugin_) {
       return csound->InitError(csound, "DSSI4CS: Invalid plugin handle.");
-    if (p->DSSIPlugin_->Type == LADSPA)
+    }
+    if (p->DSSIPlugin_->Type == LADSPA) {
       Descriptor = (LADSPA_Descriptor *) p->DSSIPlugin_->Descriptor;
-    else
+    }
+    else {
       Descriptor =
           (LADSPA_Descriptor *) p->DSSIPlugin_->DSSIDescriptor->LADSPA_Plugin;
+    }
     p->HintSampleRate =
         (LADSPA_IS_HINT_SAMPLE_RATE
          (Descriptor->PortRangeHints[PortIndex].HintDescriptor) ? Sr : 1);
@@ -794,37 +813,38 @@ int dssictls_init(CSOUND * csound, DSSICTLS * p)
       return csound->InitError(csound,
                                "DSSI4CS: Port %lu from '%s' is an output port.",
                                PortIndex, Descriptor->Name);
-    if (!LADSPA_IS_PORT_CONTROL(PortDescriptor))
-      for (i = 0; i < PortIndex; i++) {
-        PortDescriptor = Descriptor->PortDescriptors[i];
-        if (LADSPA_IS_PORT_CONTROL(PortDescriptor)) {
-          ControlPort++;
-          Port = ControlPort;
-          if (LADSPA_IS_PORT_AUDIO(PortDescriptor)) {
-            AudioPort++;
-            Port = AudioPort;
-          }
-        }
+    for (i = 0; i < PortIndex; i++) {
+      PortDescriptor = Descriptor->PortDescriptors[i];
+      if (LADSPA_IS_PORT_CONTROL(PortDescriptor)) {
+        ControlPort++;
+        Port = ControlPort;
       }
+      if (LADSPA_IS_PORT_AUDIO(PortDescriptor)) {
+        AudioPort++;
+        Port = AudioPort;
+      }
+    }
     p->PortNumber = Port;
 #ifdef DEBUG
-    csound->Message(csound, "DSSI4CS: Connected Plugport %lu to output %lu.\n",
-                    PortIndex, ControlPort);
+    csound->Message(csound, "DSSI4CS: Port %lu using internal port %lu.\n",
+                    PortIndex, p->PortNumber);
     csound->Message(csound, "DSSI4CS: ArgMask: %lu.\n",
                     csound->GetInputArgAMask(p));
 #endif
 
-    if ((int) csound->GetInputArgAMask(p) & 4)
-      p->h.opadr = (SUBR) dssictls_ak;  /* "iiak" */
-    else
-      p->h.opadr = (SUBR) dssictls_kk;  /* "iikk" */
+//    if ((int) csound->GetInputArgAMask(p) & 4) {
+//      p->h.opadr = (SUBR) dssictls_ak;  /* "iiak" */
+//    }
+//    else {
+//      p->h.opadr = (SUBR) dssictls_kk;  /* "iikk" */
+//    }
 
     return OK;
 }
 
 int dssictls_dummy(CSOUND * csound, DSSICTLS * p)
 {
-    csound->PerfError(csound,
+    csound->PerfError(csound, p->h.insdshead,
                       Str("DSSI4CS: Not initialised or wrong argument types."));
     return NOTOK;
 }
@@ -952,13 +972,14 @@ LADSPAPluginSearch(CSOUND *csound,
     if (!pcLADSPAPath) {
       csound->Message(csound,
                       "DSSI4CS: LADSPA_PATH environment variable not set.\n");
+      pcLADSPAPath = "/usr/lib/ladspa/";
     }
     if (!pcDSSIPath) {
       csound->Message(csound,
                       "DSSI4CS: DSSI_PATH environment variable not set.\n");
     }
-    if ((!pcLADSPAPath) && (!pcLADSPAPath))
-      return;
+    // if ((!pcLADSPAPath) && (!pcLADSPAPath))
+    //  return;
     if (pcDSSIPath) {
       pcLADSPAPath = strcat((char *) pcLADSPAPath, ":");
       pcLADSPAPath = strcat((char *) pcLADSPAPath, pcDSSIPath);
@@ -1046,31 +1067,31 @@ int dssilist(CSOUND * csound, DSSILIST * p)
     return OK;
 }
 
-static OENTRY localops[] = {
-    {"dssiinit", sizeof(DSSIINIT), 1, "i", "Tip", (SUBR) dssiinit, NULL, NULL }
+static OENTRY dssi_localops[] = {
+  {"dssiinit", sizeof(DSSIINIT), 0, 1, "i", "Tip", (SUBR) dssiinit, NULL, NULL }
     ,
-    {"dssiactivate", sizeof(DSSIACTIVATE), 3, "", "ik",
+    {"dssiactivate", sizeof(DSSIACTIVATE), 0, 3, "", "ik",
      (SUBR) dssiactivate_init, (SUBR) dssiactivate, NULL }
     ,
-    {"dssiaudio", sizeof(DSSIAUDIO), 5, "mmmm", "iy", (SUBR) dssiaudio_init,
-     NULL, (SUBR) dssiaudio }
+    {"dssiaudio", sizeof(DSSIAUDIO), 0, 5, "mmmmmmmmm", "iMMMMMMMMM",
+                                            (SUBR) dssiaudio_init,
+                                            NULL, (SUBR) dssiaudio }
     ,
-    {"dssictls", sizeof(DSSICTLS), 7, "", "iixk", (SUBR) dssictls_init,
-     (SUBR) dssictls_dummy, (SUBR) dssictls_dummy }
+    {"dssictls", sizeof(DSSICTLS), 0, 3, "", "iikk", (SUBR) dssictls_init,
+     (SUBR) dssictls_kk, NULL }
     ,
-    {"dssilist", sizeof(DSSILIST), 1, "", "", (SUBR) dssilist, NULL, NULL }
+    {"dssilist", sizeof(DSSILIST), 0, 1, "", "", (SUBR) dssilist, NULL, NULL }
 #if 0
     ,
-    {"dssisynth", sizeof(DSSISYNTH), 5, "aa",  "i", (SUBR) dssisynth_init,
+    {"dssisynth", sizeof(DSSISYNTH), 0, 5, "aa",  "i", (SUBR) dssisynth_init,
      NULL, (SUBR) dssisynth}
     ,
-    {"dssinote", sizeof(DSSINOTE), 3, "",  "kikkk", (SUBR) dssinote_init,
+    {"dssinote", sizeof(DSSINOTE), 0, 3, "",  "kikkk", (SUBR) dssinote_init,
      NULL, (SUBR) dssinote}
     ,
-    {"dssievent", sizeof(DSSINOTEON), 3, "",  "kikk", (SUBR) dssievent_init,
+    {"dssievent", sizeof(DSSINOTEON), 0, 3, "",  "kikk", (SUBR) dssievent_init,
      NULL, (SUBR) dssievent}
 #endif
 };
 
-LINKAGE
-
+LINKAGE_BUILTIN(dssi_localops)
