@@ -23,9 +23,7 @@
 
 // #include "csdl.h"
 #include "csoundCore.h"
-#ifdef PARCS
 #include "interlocks.h"
-#endif
 
 typedef struct _equ {
   OPDS h;
@@ -40,7 +38,7 @@ typedef struct _equ {
 static int equ_init(CSOUND *csound, equ *p)
 {
     if (*p->ini==0) {
-      double sr = (double)csound->GetSr(csound);
+      double sr = (double)CS_ESR;
       p->z1 = p->z2 = 0.0;
       p->frv = *p->fr; p->bwv = *p->bw;
       p->d = cos(2*PI*p->frv/sr);
@@ -54,10 +52,12 @@ static int equ_process(CSOUND *csound, equ *p)
 {
     double z1 = p->z1, z2 = p->z2,c,d,w,a,y;
     MYFLT  *in= p->sig,*out=p->out,g;
-    int i, ksmps = csound->GetKsmps(csound);
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    int i, ksmps = CS_KSMPS;
 
     if (*p->bw != p->bwv || *p->fr != p->frv){
-      double sr = (double)csound->GetSr(csound);
+      double sr = (double)CS_ESR;
       p->frv = *p->fr; p->bwv = *p->bw;
       p->d = cos(2*PI*p->frv/sr);
       p->c = tan(PI*p->bwv/sr);
@@ -67,7 +67,12 @@ static int equ_process(CSOUND *csound, equ *p)
     a = (1.0-c)/(1.0+c);
     g = *p->g;
 
-    for(i=0; i < ksmps; i++){
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      ksmps -= early;
+      memset(&out[ksmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=0; i < ksmps; i++){
       w = (double)(in[i]) + d*(1.0 + a)*z1 - a*z2;
       y = w*a - d*(1.0 + a)*z1 + z2;
       z2 = z1;
@@ -82,8 +87,8 @@ static int equ_process(CSOUND *csound, equ *p)
 }
 
 static OENTRY eqfil_localops[] = {
-  {"eqfil", sizeof(equ), 5,
+  {"eqfil", sizeof(equ), 0, 5,
    "a", "akkko", (SUBR)equ_init, NULL, (SUBR)equ_process},
 };
 
-LINKAGE1(eqfil_localops)
+LINKAGE_BUILTIN(eqfil_localops)
