@@ -26,7 +26,8 @@
     by Robin Whittle, see source OOps/ugrw1.c
 */
 
-#include "csdl.h"
+#include "csoundCore.h"
+#include "interlocks.h"
 #include <math.h>
 
 const int MAX_PFACTOR = 16;
@@ -246,12 +247,12 @@ int tablefilter (CSOUND *csound, TABFILT *p)
     /* Check the state of the two table number variables.
      * Error message if any are < 1 and no further action.     */
     if (UNLIKELY((*p->dft < 1) || (*p->sft < 1))) {
-      return csound->PerfError(csound,
+      return csound->PerfError(csound, p->h.insdshead,
                                Str("Farey: Table no. < 1 dft=%.2f  sft=%.2f"),
                                *p->dft, *p->sft);
     }
     if (UNLIKELY((*p->ftype < 1))) {
-      return csound->PerfError(csound,
+      return csound->PerfError(csound, p->h.insdshead,
                                Str("Farey: Filter type < 1 dft=%.2f  sft=%.2f"),
                                *p->ftype);
     }
@@ -266,7 +267,7 @@ int tablefilter (CSOUND *csound, TABFILT *p)
        */
       if (UNLIKELY((p->funcd = csound->FTFindP(csound, p->dft)) == NULL)) {
         return
-          csound->PerfError(csound,
+          csound->PerfError(csound, p->h.insdshead,
                             Str("Farey: Destination dft table %.2f not found."),
                             *p->dft);
       }
@@ -277,7 +278,7 @@ int tablefilter (CSOUND *csound, TABFILT *p)
     /* Source  */
     if (p->psft != (int)*p->sft) {
       if (UNLIKELY((p->funcs = csound->FTFindP(csound, p->sft)) == NULL)) {
-        return csound->PerfError(csound,
+        return csound->PerfError(csound, p->h.insdshead,
                                  Str("Farey: Source sft table %.2f not found."),
                                  *p->sft);
       }
@@ -301,7 +302,7 @@ int tableifilter (CSOUND *csound, TABFILT *p)
                                *p->dft, *p->sft);
     }
     if (UNLIKELY((*p->ftype < 1))) {
-      return csound->PerfError(csound,
+      return csound->PerfError(csound, p->h.insdshead,
                                Str("Farey: Filter type < 1"),
                                *p->ftype);
     }
@@ -312,7 +313,7 @@ int tableifilter (CSOUND *csound, TABFILT *p)
     if (p->pdft != (int)*p->dft) {
       /* Get pointer to the function table data structure.
        * csoundFTFindP() for perf time. csoundFTFind() for init time. */
-      if (UNLIKELY((p->funcd = csound->FTFind(csound, p->dft)) == NULL)) {
+      if (UNLIKELY((p->funcd = csound->FTnp2Find(csound, p->dft)) == NULL)) {
         return
           csound->InitError(csound,
                             Str("Farey: Destination dft table %.2f not found."),
@@ -324,7 +325,7 @@ int tableifilter (CSOUND *csound, TABFILT *p)
     }
     /* Source  */
     if (p->psft != (int)*p->sft) {
-      if (UNLIKELY((p->funcs = csound->FTFind(csound, p->sft)) == NULL)) {
+      if (UNLIKELY((p->funcs = csound->FTnp2Find(csound, p->sft)) == NULL)) {
         return csound->InitError(csound,
                                  Str("Farey: Source sft table %.2f not found."),
                                  *p->sft);
@@ -355,16 +356,13 @@ static int dotablefilter (CSOUND *csound, TABFILT *p)
     MYFLT *pdest, *ps;
     MYFLT threshold;
     int32 ftype;
-    MYFLT remember1st = FL(0.0);
     MYFLT previous = FL(0.0);
-    MYFLT oneminuslast = FL(0.0);
-    MYFLT wraparoundioi = FL(0.0);
-    int32 sourcelength;
+    //int32 sourcelength;
 
     ftype = (int32) *p->ftype;
     threshold = Digest (*p->threshold);
     loopcount = p->funcd->flen;
-    sourcelength = loopcount;
+    //sourcelength = loopcount;
 
     /* Now get the base addresses and length masks of the tables. */
     based  = p->funcd->ftable;
@@ -443,7 +441,7 @@ int tableshuffleset(CSOUND *csound, TABSHUFFLE *p)
 int tableshuffle (CSOUND * csound, TABSHUFFLE *p) {
 
     if (UNLIKELY(*p->sft < 1)) {
-      return csound->PerfError(csound,
+      return csound->PerfError(csound, p->h.insdshead,
                                Str("Table no. < 1 sft=%.2f"),
                                *p->sft);
     }
@@ -451,7 +449,7 @@ int tableshuffle (CSOUND * csound, TABSHUFFLE *p) {
     /* Source  */
     if (p->psft != (int)*p->sft) {
       if (UNLIKELY((p->funcs = csound->FTFindP(csound, p->sft)) == NULL)) {
-        return csound->PerfError(csound,
+        return csound->PerfError(csound, p->h.insdshead,
                                  Str("Source sft table %.2f not found."),
                                  *p->sft);
       }
@@ -464,7 +462,7 @@ int tableshuffle (CSOUND * csound, TABSHUFFLE *p) {
 int tableishuffle (CSOUND *csound, TABSHUFFLE *p) {
 
     if (UNLIKELY(*p->sft < 1)) {
-      return csound->PerfError(csound,
+      return csound->PerfError(csound, p->h.insdshead,
                                Str("Table no. < 1 sft=%.2f"),
                                *p->sft);
     }
@@ -472,7 +470,7 @@ int tableishuffle (CSOUND *csound, TABSHUFFLE *p) {
 
     /* Source  */
     if (p->psft != (int)*p->sft) {
-      if (UNLIKELY((p->funcs = csound->FTFind(csound, p->sft)) == NULL)) {
+      if (UNLIKELY((p->funcs = csound->FTnp2Find(csound, p->sft)) == NULL)) {
         return csound->InitError(csound,
                                  Str("Source sft table %.2f not found."),
                                  *p->sft);
@@ -492,19 +490,19 @@ static int dotableshuffle (CSOUND *csound, TABSHUFFLE *p)
 {
     time_t now;
     unsigned int seed = (unsigned int) time (&now);
-    srand (seed);
 
     MYFLT *bases;       /* Base address of the source table.*/
+    MYFLT *temp;
     int32 sourcelength;
     int32 i = 0;
 
+    srand (seed);
     sourcelength = p->funcs->flen;
 
     /* Now get the base address of the table. */
     bases = p->funcs->ftable;
 
-    MYFLT* temp; 
-    temp = (MYFLT*) calloc (sourcelength, sizeof(MYFLT)); 
+    temp = (MYFLT*) calloc (sourcelength, sizeof(MYFLT));
     memset (temp, 0, sizeof(MYFLT) * sourcelength);
 
     for (i = 0; i < sourcelength; i++) {
@@ -534,7 +532,7 @@ int fareylen (CSOUND *csound, FAREYLEN *p)
 int EulerPhi (int n)
 {
     int i = 0;
-    int pcount;
+    //int pcount;
     MYFLT result;
     PFACTOR p[MAX_PFACTOR];
     memset(p, 0, sizeof(PFACTOR)*MAX_PFACTOR);
@@ -543,7 +541,7 @@ int EulerPhi (int n)
       return 1;
     if (n == 0)
       return 0;
-    pcount = PrimeFactors (n, p);
+    (void)PrimeFactors (n, p);
 
     result = (MYFLT)n;
     for (i = 0; i < MAX_PFACTOR; i++) {
@@ -600,6 +598,7 @@ int PrimeFactors (int n, PFACTOR p[])
             ++pcount; ++j;
           }
       }
+    return j;
 }
 
 /* ----------------------------------------------- *
@@ -733,14 +732,16 @@ void float_to_cfrac (double r, int n, int a[], int p[], int q[])
 
 #define S sizeof
 
-static OENTRY localops[] = {
-    {"tablefilteri", S (TABFILT), 1, "i", "iiii", (SUBR) tableifilter, NULL, NULL},
-    {"tablefilter", S (TABFILT), 2, "k", "kkkk",
-                         (SUBR) tablefilterset, (SUBR) tablefilter, NULL},
-    {"fareyleni", S (FAREYLEN), 1, "i", "i", (SUBR) fareylen, NULL, NULL},
-    {"fareylen", S (FAREYLEN), 2, "k", "k", NULL, (SUBR) fareylen, NULL},
-    {"tableshufflei", S (TABSHUFFLE), 1, "", "i", (SUBR) tableshuffle, NULL, NULL},
-    {"tableshuffle", S (TABSHUFFLE), 2, "", "k", (SUBR) tableshuffleset, (SUBR) tableshuffle, NULL}
+static OENTRY fareyseq_localops[] = {
+    {"tablefilteri", S(TABFILT),TB, 1, "i", "iiii", (SUBR) tableifilter,NULL,NULL},
+    {"tablefilter", S(TABFILT), TB, 2, "k", "kkkk",
+                                (SUBR) tablefilterset, (SUBR) tablefilter, NULL},
+    {"fareyleni", S(FAREYLEN), TR, 1, "i", "i", (SUBR) fareylen, NULL, NULL},
+    {"fareylen", S(FAREYLEN), TR, 2, "k", "k", NULL, (SUBR) fareylen, NULL},
+    {"tableshufflei", S(TABSHUFFLE), TB, 1, "", "i",
+                                      (SUBR) tableshuffle, NULL, NULL},
+    {"tableshuffle", S(TABSHUFFLE), TB, 2, "", "k",
+                      (SUBR) tableshuffleset, (SUBR) tableshuffle, NULL},
 };
 
-LINKAGE
+LINKAGE_BUILTIN(fareyseq_localops)
