@@ -21,7 +21,7 @@
     02111-1307 USA
 */
 
-#include "csdl.h"               /* Flanger by Maldonado, with coding
+#include "stdopcod.h"               /* Flanger by Maldonado, with coding
                                    enhancements by JPff -- July 1998 */
 #include <math.h>
 #include "flanger.h"
@@ -29,7 +29,7 @@
 static int flanger_set (CSOUND *csound, FLANGER *p)
 {
         /*---------------- delay  -----------------------*/
-    p->maxdelay = (uint32)(*p->maxd  * csound->esr);
+    p->maxdelay = (uint32)(*p->maxd  * CS_ESR);
     csound->AuxAlloc(csound, p->maxdelay * sizeof(MYFLT), &p->aux);
     p->left = 0;
     p->yt1 = FL(0.0);
@@ -48,16 +48,23 @@ static int flanger(CSOUND *csound, FLANGER *p)
     MYFLT *freq_del = p->xdel;
     MYFLT feedback =  *p->kfeedback;
     MYFLT fv1;
-    int32  v2;
-    int32  v1;
+    int32 v2;
+    int32 v1;
     MYFLT yt1= p->yt1;
 
-    int n,nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
                 /*---------------- delay -----------------------*/
       buf[indx] = in[n] + (yt1 * feedback);
-      fv1 = indx - (*freq_del++ * csound->esr); /* Make sure inside the buffer*/
+      fv1 = indx - (*freq_del++ * CS_ESR); /* Make sure inside the buffer*/
       while (fv1 < 0)
         fv1 += maxdelay;
       while (fv1 >= maxdelay) fv1 -= maxdelay; /* Is this necessary? JPff */
@@ -77,7 +84,7 @@ static int flanger(CSOUND *csound, FLANGER *p)
 static int wguide1set (CSOUND *csound, WGUIDE1 *p)
 {
         /*---------------- delay -----------------------*/
-    p->maxd = (uint32) (MAXDELAY * csound->esr);
+    p->maxd = (uint32) (MAXDELAY * CS_ESR);
     csound->AuxAlloc(csound, p->maxd * sizeof(MYFLT), &p->aux);
     p->left = 0;
         /*---------------- filter -----------------------*/
@@ -95,14 +102,16 @@ static int wguide1(CSOUND *csound, WGUIDE1 *p)
     MYFLT *out = p->ar;  /* assign object data to local variables   */
     MYFLT *in = p->asig;
     MYFLT *buf = (MYFLT *)p->aux.auxp;
-    MYFLT *freq_del =  p->xdel; /*(1 / *p->xdel)  * csound->esr; */
+    MYFLT *freq_del =  p->xdel; /*(1 / *p->xdel)  * CS_ESR; */
     MYFLT feedback =  *p->kfeedback;
     MYFLT  fv1, fv2, out_delay,bufv1 ;
     unsigned int maxdM1 = p->maxd-1;
     int32   v1;
     /*---------------- filter -----------------------*/
     MYFLT c1, c2, yt1 = p->yt1;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     /*---------------- delay -----------------------*/
     indx = p->left;
@@ -116,14 +125,19 @@ static int wguide1(CSOUND *csound, WGUIDE1 *p)
     }
     c1= p->c1;
     c2= p->c2;
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->xdelcod) { /* delay changes at audio-rate */
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         /*---------------- delay -----------------------*/
         MYFLT fd = *freq_del++;
         buf[indx] = in[n] + (yt1 * feedback);
-        if (fd<FL(1.0)/MAXDELAY) /* Avoid silly values jpff */
+        if (UNLIKELY(fd<FL(1.0)/MAXDELAY)) /* Avoid silly values jpff */
           fd = FL(1.0)/MAXDELAY;
-        fv1 = indx - (csound->esr / fd); /* Make sure inside the buffer */
+        fv1 = indx - (CS_ESR / fd); /* Make sure inside the buffer */
         while (fv1 < 0) {
           fv1 = fv1 + (MYFLT)p->maxd;
         }
@@ -131,19 +145,19 @@ static int wguide1(CSOUND *csound, WGUIDE1 *p)
                   fv1 + 1 : 0;  /* Find next smpl for interpolation */
         bufv1 = buf[v1=(int32)fv1];
         out_delay = bufv1 + (fv1 - v1) * ( buf[(int32)fv2] - bufv1);
-        if (++indx == p->maxd) indx = 0; /* Advance current pointer */
+        if (UNLIKELY(++indx == p->maxd)) indx = 0; /* Advance current pointer */
         /*---------------- filter -----------------------*/
         out[n] = yt1 = c1 * out_delay + c2 * yt1;
       }
     }
     else {
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         /*---------------- delay -----------------------*/
         MYFLT fd = *freq_del;
         buf[indx] = in[n] + (yt1 * feedback);
-        if (fd<FL(1.0)/MAXDELAY) /* Avoid silly values jpff */
+        if (UNLIKELY(fd<FL(1.0)/MAXDELAY)) /* Avoid silly values jpff */
           fd = FL(1.0)/MAXDELAY;
-        fv1 = indx - (csound->esr / fd); /* Make sure inside the buffer */
+        fv1 = indx - (CS_ESR / fd); /* Make sure inside the buffer */
         while (fv1 < 0) {
           fv1 = fv1 + (MYFLT)p->maxd;
         }
@@ -151,7 +165,7 @@ static int wguide1(CSOUND *csound, WGUIDE1 *p)
               fv1 + 1 : 0;  /* Find next smpl for interpolation */
         bufv1 = buf[v1=(int32)fv1];
         out_delay = bufv1 + (fv1 - v1) * ( buf[(int32)fv2] - bufv1);
-        if (++indx == p->maxd) indx = 0;     /* Advance current pointer */
+        if (UNLIKELY(++indx == p->maxd)) indx = 0;     /* Advance current pointer */
         /*---------------- filter -----------------------*/
         out[n] = yt1 = c1 * out_delay + c2 * yt1;
       }
@@ -164,7 +178,7 @@ static int wguide1(CSOUND *csound, WGUIDE1 *p)
 static int wguide2set (CSOUND *csound, WGUIDE2 *p)
 {
         /*---------------- delay1 -----------------------*/
-    p->maxd = (uint32) (MAXDELAY * csound->esr);
+    p->maxd = (uint32) (MAXDELAY * CS_ESR);
     csound->AuxAlloc(csound, p->maxd * sizeof(MYFLT), &p->aux1);
     p->left1 = 0;
         /*---------------- delay2 -----------------------*/
@@ -194,26 +208,28 @@ static int wguide2(CSOUND *csound, WGUIDE2 *p)
 {
     MYFLT *out = p->ar;
     MYFLT *in = p->asig;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT out1,out2, old_out = p->old_out;
     unsigned int maxdM1 = p->maxd-1;
 
     /*---------------- delay1 -----------------------*/
-    uint32  indx1;
-    MYFLT *buf1 = (MYFLT *)p->aux1.auxp;
-    MYFLT *freq_del1 = p->xdel1; /*(1 / *p->xdel1)  * csound->esr; */
-    MYFLT feedback1 =  *p->kfeedback1;
+    uint32 indx1;
+    MYFLT  *buf1 = (MYFLT *)p->aux1.auxp;
+    MYFLT  *freq_del1 = p->xdel1; /*(1 / *p->xdel1)  * CS_ESR; */
+    MYFLT  feedback1 =  *p->kfeedback1;
     MYFLT  fv1_1, fv2_1, out_delay1 ;
-    int32   v1_1;
+    int32  v1_1;
         /*---------------- filter1 -----------------------*/
     MYFLT c1_1, c2_1, yt1_1;
         /*---------------- delay2 -----------------------*/
-    uint32  indx2;
-    MYFLT *buf2 = (MYFLT *)p->aux2.auxp;
-    MYFLT *freq_del2 = p->xdel2; /*(1 / *p->xdel2)  * csound->esr;*/
-    MYFLT feedback2 =  *p->kfeedback2;
+    uint32 indx2;
+    MYFLT  *buf2 = (MYFLT *)p->aux2.auxp;
+    MYFLT  *freq_del2 = p->xdel2; /*(1 / *p->xdel2)  * CS_ESR;*/
+    MYFLT  feedback2 =  *p->kfeedback2;
     MYFLT  fv1_2, fv2_2, out_delay2 ;
-    int32   v1_2;
+    int32  v1_2;
         /*---------------- filter2 -----------------------*/
     MYFLT c1_2, c2_2, yt1_2;
         /*-----------------------------------------------*/
@@ -241,18 +257,23 @@ static int wguide2(CSOUND *csound, WGUIDE2 *p)
     yt1_1= p->yt1_1;
     yt1_2= p->yt1_2;
 
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->xdel1cod) { /* delays change at audio-rate */
-      for (n=0;n<nsmps;n++) {
+      for (n=offset;n<nsmps;n++) {
         MYFLT fd1 = *freq_del1++;
         MYFLT fd2 = *freq_del2++;
         buf1[indx1] = buf2[indx2] =
           in[n] + old_out * (feedback1 + feedback2);
-        if (fd1<FL(1.0)/MAXDELAY) /* Avoid silly values jpff */
+        if (UNLIKELY(fd1<FL(1.0)/MAXDELAY)) /* Avoid silly values jpff */
           fd1 = FL(1.0)/MAXDELAY;
-        if (fd2<FL(1.0)/MAXDELAY) /* Avoid silly values jpff */
+        if (UNLIKELY(fd2<FL(1.0)/MAXDELAY)) /* Avoid silly values jpff */
           fd2 = FL(1.0)/MAXDELAY;
-        fv1_1 = indx1 - (csound->esr / fd1); /*Make sure inside the buffer */
-        fv1_2 = indx2 - (csound->esr / fd2); /* Make sure inside the buffer  */
+        fv1_1 = indx1 - (CS_ESR / fd1); /* Make sure inside the buffer */
+        fv1_2 = indx2 - (CS_ESR / fd2); /* Make sure inside the buffer */
         while (fv1_1 < 0)    fv1_1 += p->maxd;
         while (fv1_2 < 0)    fv1_2 += p->maxd;
         fv2_1 = (fv1_1<maxdM1)?
@@ -263,25 +284,25 @@ static int wguide2(CSOUND *csound, WGUIDE2 *p)
         v1_2 = (int32)fv1_2;
         out_delay1 = buf1[v1_1] + (fv1_1-v1_1)*(buf1[(int32)fv2_1]-buf1[v1_1]);
         out_delay2 = buf2[v1_2] + (fv1_2-v1_2)*(buf2[(int32)fv2_2]-buf2[v1_2]);
-        if (++indx1 == p->maxd) indx1 = 0;        /* Advance current pointer */
-        if (++indx2 == p->maxd) indx2 = 0;        /* Advance current pointer */
+        if (UNLIKELY(++indx1 == p->maxd)) indx1 = 0; /* Advance current pointer */
+        if (UNLIKELY(++indx2 == p->maxd)) indx2 = 0; /* Advance current pointer */
         out1 = yt1_1 = c1_1 * out_delay1 + c2_1 * yt1_1;
         out2 = yt1_2 = c1_2 * out_delay2 + c2_2 * yt1_2;
         out[n] = old_out = out1 + out2;
       }
     }
     else {
-      for (n=0; n<nsmps;n++) {
+      for (n=offset; n<nsmps;n++) {
         MYFLT fd1 = *freq_del1;
         MYFLT fd2 = *freq_del2;
         buf1[indx1] = buf2[indx2] =
           in[n] + old_out * (feedback1 + feedback2);
-        if (fd1<FL(1.0)/MAXDELAY) /* Avoid silly values jpff */
+        if (UNLIKELY(fd1<FL(1.0)/MAXDELAY))/* Avoid silly values jpff */
           fd1 = FL(1.0)/MAXDELAY;
-        if (fd2<FL(1.0)/MAXDELAY) /* Avoid silly values jpff */
+        if (UNLIKELY(fd2<FL(1.0)/MAXDELAY)) /* Avoid silly values jpff */
           fd2 = FL(1.0)/MAXDELAY;
-        fv1_1 = indx1 - (csound->esr / fd1); /* Make sure inside the buffer */
-        fv1_2 = indx2 - (csound->esr / fd2); /* Make sure inside the buffer */
+        fv1_1 = indx1 - (CS_ESR / fd1); /* Make sure inside the buffer */
+        fv1_2 = indx2 - (CS_ESR / fd2); /* Make sure inside the buffer */
         while (fv1_1 < 0)    fv1_1 += p->maxd;
         while (fv1_2 < 0)    fv1_2 += p->maxd;
         fv2_1 = (fv1_1<maxdM1)?
@@ -292,8 +313,8 @@ static int wguide2(CSOUND *csound, WGUIDE2 *p)
         v1_2 = (int32)fv1_2;
         out_delay1 = buf1[v1_1] + (fv1_1 - v1_1)*(buf1[(int32)fv2_1]-buf1[v1_1]);
         out_delay2 = buf2[v1_2] + (fv1_2 - v1_2)*(buf2[(int32)fv2_2]-buf2[v1_2]);
-        if (++indx1 == p->maxd) indx1 = 0;        /* Advance current pointer */
-        if (++indx2 == p->maxd) indx2 = 0;        /* Advance current pointer */
+        if (UNLIKELY(++indx1 == p->maxd)) indx1 = 0; /* Advance current pointer */
+        if (UNLIKELY(++indx2 == p->maxd)) indx2 = 0; /* Advance current pointer */
         out1 = yt1_1 = c1_1 * out_delay1 + c2_1 * yt1_1;
         out2 = yt1_2 = c1_2 * out_delay2 + c2_2 * yt1_2;
         out[n] = old_out = out1 + out2;
@@ -310,9 +331,9 @@ static int wguide2(CSOUND *csound, WGUIDE2 *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "flanger", S(FLANGER), 5, "a", "aakv", (SUBR)flanger_set, NULL, (SUBR)flanger },
-{ "wguide1", S(WGUIDE1), 5, "a", "axkk",(SUBR) wguide1set, NULL, (SUBR)wguide1  },
-{ "wguide2", S(WGUIDE2), 5, "a", "axxkkkk",(SUBR)wguide2set, NULL, (SUBR)wguide2 }
+{ "flanger", S(FLANGER), 0, 5, "a", "aakv", (SUBR)flanger_set, NULL, (SUBR)flanger },
+{ "wguide1", S(WGUIDE1), 0, 5, "a", "axkk",(SUBR) wguide1set, NULL, (SUBR)wguide1  },
+{ "wguide2", S(WGUIDE2), 0, 5, "a", "axxkkkk",(SUBR)wguide2set, NULL, (SUBR)wguide2 }
 };
 
 int flanger_init_(CSOUND *csound)
