@@ -39,6 +39,7 @@
 #include "OpcodeBase.hpp"
 
 #include <map>
+#include <cmath>
 #ifdef USE_OPENMP
 #include <omp.h>
 #endif
@@ -48,9 +49,11 @@
 /**
  * This may help avoid problems with the order of static initializations.
  */
-static std::map<CSOUND *, std::vector<fluid_synth_t *> > &getFluidSynthsForCsoundInstances()
+static std::map<CSOUND *, std::vector<fluid_synth_t *> >
+                &getFluidSynthsForCsoundInstances()
 {
-    static std::map<CSOUND *, std::vector<fluid_synth_t *> > fluidSynthsForCsoundInstances;
+    static std::map<CSOUND *, std::vector<fluid_synth_t *> >
+                fluidSynthsForCsoundInstances;
     return fluidSynthsForCsoundInstances;
 }
 
@@ -134,7 +137,8 @@ public:
             fluidSettings = new_fluid_settings();
             if (fluidSettings != NULL) {
                 fluid_settings_setnum(fluidSettings,
-                        (char *)"synth.sample-rate", (double) csound->esr);
+                        (char *)"synth.sample-rate",
+                                      (double) csound->GetSr(csound));
                 fluid_settings_setint(fluidSettings,
                         (char *)"synth.midi-channels", channelCount);
                 fluid_settings_setint(fluidSettings,
@@ -146,7 +150,8 @@ public:
                 if (fluidSettings) {
                     delete_fluid_settings(fluidSettings);
                 }
-                result = csound->InitError(csound, Str("error allocating fluid engine\n"));
+                result = csound->InitError(csound,
+                                           Str("error allocating fluid engine\n"));
             } else {
                 //csound_global_mutex_lock();
                 fluid_synth_set_chorus_on(fluidSynth, chorusEnabled);
@@ -154,7 +159,7 @@ public:
                 //csound_global_mutex_unlock();
                 log(csound, "Created fluidEngine 0x%p with sampling rate = %f, "
                     "chorus %s, reverb %s, channels %d, voices %d.\n",
-                    fluidSynth, (double) csound->esr,
+                    fluidSynth, (double) csound->GetSr(csound),
                     chorusEnabled ? "on" : "off",
                     reverbEnabled ? "on" : "off",
                     channelCount,
@@ -189,11 +194,18 @@ public:
             soundFontId = -1;
             toa(iFluidSynth, fluidSynth);
             listPresets = (int) *iListPresets;
-            filename = csound->strarg2name(csound,
+
+            if(csound->GetInputArgSMask(this))
+              filename = csound->Strdup(csound, ((STRINGDAT *)iFilename)->data);
+             else
+             filename = csound->strarg2name(csound,
                     (char*) NULL,
-                    iFilename,
+                   (std::isnan(*iFilename) ?
+                    csound->GetString(csound, *iFilename) :
+                    (char *) iFilename),
                     (char *)"fluid.sf2.",
-                    (int) csound->GetInputArgSMask(this));
+                    std::isnan(*iFilename));
+
             filepath = csound->FindInputFile(csound, filename, "SFDIR;SSDIR");
             if (filepath && fluid_is_soundfont(filepath)) {
                 log(csound, "Loading SoundFont : %s.\n", filepath);
@@ -205,7 +217,8 @@ public:
             }
             *iInstrumentNumber = (MYFLT) soundFontId;
             if (soundFontId < 0) {
-                csound->InitError(csound, Str("fluid: unable to load %s"), filename);
+                csound->InitError(csound,
+                                  Str("fluid: unable to load %s"), filename);
             }
             csound->NotifyFileOpened(csound, filepath, CSFTYPE_SOUNDFONT, 0, 0);
             if (soundFontId < 0) {
@@ -215,8 +228,11 @@ public:
                     fluid_synth_get_sfont_by_id(fluidSynth, soundFontId);
                 fluid_preset_t fluidPreset;
                 fluidSoundfont->iteration_start(fluidSoundfont);
-                if (csound->oparms->msglevel & 0x7)
-                    while (fluidSoundfont->iteration_next(fluidSoundfont, &fluidPreset)) {
+                OPARMS oparms;
+                 csound->GetOParms(csound, &oparms);
+                if (oparms.msglevel & 0x7)
+                    while (fluidSoundfont->iteration_next(fluidSoundfont,
+                                                          &fluidPreset)) {
                         log(csound,
                             "SoundFont: %3d  Bank: %3d  Preset: %3d  %s\n",
                             soundFontId,
@@ -378,21 +394,32 @@ public:
 #pragma omp critical (critical_section_fluid_out)
         {
             toa(iFluidSynth, fluidSynth);
-            ksmps = csound->GetKsmps(csound);
+            ksmps = opds.insdshead->ksmps;
         }
         return OK;
     }
     int audio(CSOUND *csound) {
 #pragma omp critical (critical_section_fluid_out)
         {
-            for (frame = 0; frame < ksmps; frame++) {
-                leftSample = 0.0f;
-                rightSample = 0.0f;
-                fluid_synth_write_float(fluidSynth, 1, &leftSample, 0, 1,
-                        &rightSample, 0, 1);
-                aLeftOut[frame] = leftSample /* * csound->e0dbfs */;
-                aRightOut[frame] = rightSample /* * csound->e0dbfs */;
-            }
+          uint32_t offset = opds.insdshead->ksmps_offset;
+          uint32_t early  = opds.insdshead->ksmps_no_end;
+          if (UNLIKELY(offset)) {
+            memset(aLeftOut, '\0', offset*sizeof(MYFLT));
+            memset(aRightOut, '\0', offset*sizeof(MYFLT));
+             }
+          if (UNLIKELY(early)) {
+            ksmps -= early;
+            memset(&aLeftOut[ksmps], '\0', early*sizeof(MYFLT));
+            memset(&aRightOut[ksmps], '\0', early*sizeof(MYFLT));
+          }
+         for (frame = offset; frame < ksmps; frame++) {
+            leftSample = 0.0f;
+            rightSample = 0.0f;
+            fluid_synth_write_float(fluidSynth, 1, &leftSample, 0, 1,
+                                    &rightSample, 0, 1);
+            aLeftOut[frame] = leftSample /* * csound->e0dbfs */;
+            aRightOut[frame] = rightSample /* * csound->e0dbfs */;
+          }
         }
         return OK;
     }
@@ -412,27 +439,39 @@ public:
     int init(CSOUND *csound) {
 #pragma omp critical (critical_section_fluid_all_out)
         {
-            ksmps = csound->GetKsmps(csound);
+            ksmps = opds.insdshead->ksmps;
         }
         return OK;
     }
     int audio(CSOUND *csound) {
 #pragma omp critical (critical_section_fluid_all_out)
         {
-            std::vector<fluid_synth_t *> &fluidSynths = getFluidSynthsForCsoundInstances()[csound];
-            for (frame = 0; frame < ksmps; frame++) {
-                aLeftOut[frame] = FL(0.0);
-                aRightOut[frame] = FL(0.0);
-                for (size_t i = 0, n = fluidSynths.size(); i < n; i++) {
-                    fluid_synth_t *fluidSynth = fluidSynths[i];
-                    leftSample = FL(0.0);
-                    rightSample = FL(0.0);  
-                    fluid_synth_write_float(fluidSynth, 1, &leftSample, 0, 1,
-                            &rightSample, 0, 1);
-                    aLeftOut[frame] += (MYFLT) leftSample /* * csound->e0dbfs */;
-                    aRightOut[frame] += (MYFLT) rightSample /* * csound->e0dbfs */;
-                }
+          uint32_t offset = opds.insdshead->ksmps_offset;
+          uint32_t early  = opds.insdshead->ksmps_no_end;
+          if (UNLIKELY(offset)) {
+            memset(aLeftOut, '\0', offset*sizeof(MYFLT));
+            memset(aRightOut, '\0', offset*sizeof(MYFLT));
+          }
+          if (UNLIKELY(early)) {
+            ksmps -= early;
+            memset(&aLeftOut[ksmps], '\0', early*sizeof(MYFLT));
+            memset(&aRightOut[ksmps], '\0', early*sizeof(MYFLT));
+          }
+          std::vector<fluid_synth_t *> &fluidSynths =
+            getFluidSynthsForCsoundInstances()[csound];
+          for (frame = offset; frame < ksmps; frame++) {
+            aLeftOut[frame] = FL(0.0);
+            aRightOut[frame] = FL(0.0);
+            for (size_t i = 0, n = fluidSynths.size(); i < n; i++) {
+              fluid_synth_t *fluidSynth = fluidSynths[i];
+              leftSample = FL(0.0);
+              rightSample = FL(0.0);
+              fluid_synth_write_float(fluidSynth, 1, &leftSample, 0, 1,
+                                      &rightSample, 0, 1);
+              aLeftOut[frame] += (MYFLT) leftSample /* * csound->e0dbfs */;
+              aRightOut[frame] += (MYFLT) rightSample /* * csound->e0dbfs */;
             }
+          }
         }
         return OK;
     }
@@ -466,7 +505,9 @@ public:
             priorMidiChannel = -1;
             priorMidiData1 = -1;
             priorMidiData2 = -1;
-            printMsgs = ((csound->oparms->msglevel & 7) == 7 ? 1 : 0);
+            OPARMS oparms;
+            csound->GetOParms(csound, &oparms);
+            printMsgs = ((oparms.msglevel & 7) == 7 ? 1 : 0);
         }
         return OK;
     }
@@ -478,7 +519,7 @@ public:
             midiData1 = (int) *kMidiData1;
             midiData2 = (int) *kMidiData2;
             int result =  -1;
-  
+
             if (midiData2 != priorMidiData2 ||
                     midiData1 != priorMidiData1 ||
                     midiChannel != priorMidiChannel ||
@@ -486,53 +527,60 @@ public:
                 switch (midiStatus) {
                 case (int) 0x80:
 noteOff:
-                    result = fluid_synth_noteoff(fluidSynth, midiChannel, midiData1);
+                    result = fluid_synth_noteoff(fluidSynth,
+                                                 midiChannel, midiData1);
                     if (printMsgs)
-                        csound->Message(csound,
-                                "result: %d \n Note off: c:%3d k:%3d\n",
-                                result,
-                                midiChannel,
-                                midiData1);
+                      csound->Message(csound,
+                                      Str("result: %d \n Note off: c:%3d k:%3d\n"),
+                                      result,
+                                      midiChannel,
+                                      midiData1);
                     break;
                 case (int) 0x90:
                     if (!midiData2) {
-                        goto noteOff;
+                      goto noteOff;
                     }
                     result = fluid_synth_noteon(fluidSynth, midiChannel,
-                            midiData1, midiData2);
+                                                midiData1, midiData2);
                     if (printMsgs)
-                        log(csound, "result: %d \nNote on: c:%3d k:%3d v:%3d\n",result,
-                            midiChannel, midiData1, midiData2);
+                      log(csound,
+                          "result: %d \nNote on: c:%3d k:%3d v:%3d\n",result,
+                          midiChannel, midiData1, midiData2);
                     break;
                 case (int) 0xA0:
-                    if (printMsgs)
-                        log(csound, "Key pressure (not handled): "
-                            "c:%3d k:%3d v:%3d\n",
-                            midiChannel, midiData1, midiData2);
+                  if (printMsgs)
+                    log(csound, "Key pressure (not handled): "
+                        "c:%3d k:%3d v:%3d\n",
+                        midiChannel, midiData1, midiData2);
                     break;
                 case (int) 0xB0:
-                    result = fluid_synth_cc(fluidSynth, midiChannel, midiData1, midiData2);
+                    result = fluid_synth_cc(fluidSynth, midiChannel,
+                                            midiData1, midiData2);
                     if (printMsgs)
-                        log(csound, "Result: %d Control change: c:%3d c:%3d v:%3d\n",result,
-                            midiChannel, midiData1, midiData2);
+                      log(csound,
+                          "Result: %d Control change: c:%3d c:%3d v:%3d\n",result,
+                          midiChannel, midiData1, midiData2);
                     break;
                 case (int) 0xC0:
-                    result = fluid_synth_program_change(fluidSynth, midiChannel, midiData1);
-                    if (printMsgs)
-                        log(csound, "Result: %d Program change: c:%3d p:%3d\n",result,
-                            midiChannel, midiData1);
-                    break;
+                  result = fluid_synth_program_change(fluidSynth,
+                                                      midiChannel, midiData1);
+                  if (printMsgs)
+                    log(csound,
+                        "Result: %d Program change: c:%3d p:%3d\n",result,
+                        midiChannel, midiData1);
+                  break;
                 case (int) 0xD0:
-                    if (printMsgs)
-                        log(csound, "After touch (not handled): c:%3d v:%3d\n",
-                            midiChannel, midiData1);
-                    break;
+                  if (printMsgs)
+                    log(csound, "After touch (not handled): c:%3d v:%3d\n",
+                        midiChannel, midiData1);
+                  break;
                 case (int) 0xE0: {
-                    int pbVal = midiData1 + (midiData2 << 7);
-                    fluid_synth_pitch_bend(fluidSynth, midiChannel, pbVal);
-                    if (printMsgs)
-                        log(csound, "Result: %d, Pitch bend:     c:%d b:%d\n", result,
-                            midiChannel, pbVal);
+                  int pbVal = midiData1 + (midiData2 << 7);
+                  fluid_synth_pitch_bend(fluidSynth, midiChannel, pbVal);
+                  if (printMsgs)
+                    log(csound,
+                        "Result: %d, Pitch bend:     c:%d b:%d\n", result,
+                        midiChannel, pbVal);
                 }
                 break;
                 case (int) 0xF0:
@@ -548,7 +596,7 @@ noteOff:
                 priorMidiData2 = midiData2;
             }
         }
-        
+
         return OK;
     }
 };
@@ -578,7 +626,8 @@ public:
                                 "either 0, 1, 4, or 7.\n"));
                 result = NOTOK;
             } else {
-                fluid_synth_set_interp_method(fluidSynth, channel, interpolationMethod);
+                fluid_synth_set_interp_method(fluidSynth, channel,
+                                              interpolationMethod);
             }
         }
         return result;
@@ -589,6 +638,7 @@ static OENTRY localops[] = {
     {
         (char *)"fluidEngine",
         sizeof(FluidEngine),
+        0,
         1,
         (char *)"i",
         (char *)"ppoo",
@@ -599,6 +649,7 @@ static OENTRY localops[] = {
     {
         (char *)"fluidLoad",
         sizeof(FluidLoad),
+        0,
         1,
         (char *)"i",
         (char *)"Tio",
@@ -609,6 +660,7 @@ static OENTRY localops[] = {
     {
         (char *)"fluidProgramSelect",
         sizeof(FluidProgramSelect),
+        0,
         1,
         (char *)"",
         (char *)"iiiii",
@@ -619,6 +671,7 @@ static OENTRY localops[] = {
     {
         (char *)"fluidCCi",
         sizeof(FluidCCI),
+        0,
         1,
         (char *)"",
         (char *)"iiii",
@@ -629,6 +682,7 @@ static OENTRY localops[] = {
     {
         (char *)"fluidCCk",
         sizeof(FluidCCK),
+        0,
         3,
         (char *)"",
         (char *)"ikkk",
@@ -639,6 +693,7 @@ static OENTRY localops[] = {
     {
         (char *)"fluidNote",
         sizeof(FluidNote),
+        0,
         3,
         (char *)"",
         (char *)"iiii",
@@ -649,6 +704,7 @@ static OENTRY localops[] = {
     {
         (char *)"fluidOut",
         sizeof(FluidOut),
+        0,
         5,
         (char *)"aa",
         (char *)"i",
@@ -659,6 +715,7 @@ static OENTRY localops[] = {
     {
         (char *)"fluidAllOut",
         sizeof(FluidAllOut),
+        0,
         5,
         (char *)"aa",
         (char *)"",
@@ -669,6 +726,7 @@ static OENTRY localops[] = {
     {
         (char *)"fluidControl",
         sizeof(FluidControl),
+        0,
         3,
         (char *)"",
         (char *)"ikkkk",
@@ -679,6 +737,7 @@ static OENTRY localops[] = {
     {
         (char *)"fluidSetInterpMethod",
         sizeof(FluidSetInterpMethod),
+        0,
         1,
         (char *)"",
         (char *)"iii",
@@ -692,6 +751,7 @@ static OENTRY localops[] = {
         0,
         0,
         0,
+        0,
         (SUBR) 0,
         (SUBR) 0,
         (SUBR) 0
@@ -715,6 +775,7 @@ PUBLIC int csoundModuleInit(CSOUND *csound)
         err |= csound->AppendOpcode(csound,
                 ep->opname,
                 ep->dsblksiz,
+                ep->flags,
                 ep->thread,
                 ep->outypes,
                 ep->intypes,
@@ -734,18 +795,16 @@ PUBLIC int csoundModuleDestroy(CSOUND *csound)
 {
 #pragma omp critical (critical_section_fluidopcodes)
     {
-        for (std::map<CSOUND *, std::vector<fluid_synth_t *> >::iterator it = getFluidSynthsForCsoundInstances().begin();
-        it != getFluidSynthsForCsoundInstances().end();
-        ++it) {
-            std::vector<fluid_synth_t *> &fluidSynths = it->second;
-            for (size_t i = 0, n = fluidSynths.size(); i < n; i++) {
-                fluid_synth_t *fluidSynth = fluidSynths[i];
-                fluid_settings_t *fluidSettings = fluid_synth_get_settings(fluidSynth);
-                delete_fluid_synth(fluidSynth);
-                delete_fluid_settings(fluidSettings);
-            }
-                        fluidSynths.clear();
-        }
+      std::vector<fluid_synth_t *> &fluidSynths =
+        getFluidSynthsForCsoundInstances()[csound];
+
+      for (size_t i = 0, n = fluidSynths.size(); i < n; i++) {
+        fluid_synth_t *fluidSynth = fluidSynths[i];
+        fluid_settings_t *fluidSettings = fluid_synth_get_settings(fluidSynth);
+        delete_fluid_synth(fluidSynth);
+        delete_fluid_settings(fluidSettings);
+      }
+      fluidSynths.clear();
     }
     return 0;
 }
@@ -754,4 +813,3 @@ PUBLIC int csoundModuleInfo(void)
 {
     return ((CS_APIVERSION << 16) + (CS_APISUBVER << 8) + (int) sizeof(MYFLT));
 }
-
