@@ -60,11 +60,11 @@ static int hvs1_set(CSOUND *csound, HVS1 *p)
 {
     FUNC        *ftp;
 
-    if ((ftp = csound->FTFind(csound, p->iOutTab)) != NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->iOutTab)) != NULL)
       p->outTable = ftp->ftable;
-    if ((ftp = csound->FTFind(csound, p->iPositionsTab)) != NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->iPositionsTab)) != NULL)
       p->posTable = ftp->ftable;
-    if ((ftp = csound->FTFind(csound, p->iSnapTab)) != NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->iSnapTab)) != NULL)
       p->snapTable = ftp->ftable;
     if (*p->inumPointsX < 2 )
       return csound->InitError(csound, Str("hvs1: a line segment must be "
@@ -73,7 +73,7 @@ static int hvs1_set(CSOUND *csound, HVS1 *p)
     if (*p->iConfigTab == 0)
       p->iconfFlag = 0;
     else {
-      if ((ftp = csound->FTFind(csound, p->iConfigTab)) != NULL)
+      if ((ftp = csound->FTnp2Find(csound, p->iConfigTab)) != NULL)
         p->outTable = ftp->ftable;
       p->iconfFlag = 1;
     }
@@ -142,11 +142,11 @@ static int hvs2_set(CSOUND *csound, HVS2 *p)
 {
     FUNC        *ftp;
 
-    if ((ftp = csound->FTFind(csound, p->iOutTab)) != NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->iOutTab)) != NULL)
       p->outTable = ftp->ftable;
-    if ((ftp = csound->FTFind(csound, p->iPositionsTab)) != NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->iPositionsTab)) != NULL)
       p->posTable = ftp->ftable;
-    if ((ftp = csound->FTFind(csound, p->iSnapTab)) != NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->iSnapTab)) != NULL)
       p->snapTable = ftp->ftable;
     if (*p->inumlinesX < 2 || *p->inumlinesY < 2)
       return csound->InitError(csound, Str("hvs2: a square area must be "
@@ -155,7 +155,7 @@ static int hvs2_set(CSOUND *csound, HVS2 *p)
     if (*p->iConfigTab == 0)
       p->iconfFlag = 0;
     else {
-      if ((ftp = csound->FTFind(csound, p->iConfigTab)) != NULL) {
+      if ((ftp = csound->FTnp2Find(csound, p->iConfigTab)) != NULL) {
         p->outTable = ftp->ftable;
       }
       p->iconfFlag = 1;
@@ -238,11 +238,11 @@ static int hvs3_set(CSOUND *csound, HVS3 *p)
 {
     FUNC        *ftp;
 
-    if ((ftp = csound->FTFind(csound, p->iOutTab)) != NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->iOutTab)) != NULL)
       p->outTable = ftp->ftable;
-    if ((ftp = csound->FTFind(csound, p->iPositionsTab)) != NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->iPositionsTab)) != NULL)
       p->posTable = ftp->ftable;
-    if ((ftp = csound->FTFind(csound, p->iSnapTab)) != NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->iSnapTab)) != NULL)
       p->snapTable = ftp->ftable;
     if (*p->inumlinesX < 2 || *p->inumlinesY < 2)
       return csound->InitError(csound, Str("hvs3: a square area must be "
@@ -252,7 +252,7 @@ static int hvs3_set(CSOUND *csound, HVS3 *p)
     if (*p->iConfigTab == 0)
       p->iconfFlag = 0;
     else {
-      if ((ftp = csound->FTFind(csound, p->iConfigTab)) != NULL)
+      if ((ftp = csound->FTnp2Find(csound, p->iConfigTab)) != NULL)
         p->outTable = ftp->ftable;
       p->iconfFlag = 1;
     }
@@ -393,7 +393,7 @@ static int vphaseseg_set(CSOUND *csound, VPSEG *p)
       p->vector = ftp->ftable;
       p->elements = (int) *p->ielements;
     }
-    if ( p->elements > ftp->flen )
+    if ( p->elements > (int)ftp->flen )
       return csound->InitError(csound,
                                Str("vphaseseg: invalid num. of elements"));
     /* vector = p->vector; */
@@ -483,13 +483,13 @@ static int vphaseseg(CSOUND *csound, VPSEG *p)
 #define S(x)    sizeof(x)
 
 OENTRY hvs_localops[] = {
-  { "hvs1",  S(HVS1), TB|3,  "",  "kiiiiio",
+  { "hvs1",  S(HVS1), TB, 3,  "",  "kiiiiio",
     (SUBR)hvs1_set, (SUBR)hvs1, (SUBR)NULL },
-  { "hvs2",  S(HVS2), TB|3,  "",  "kkiiiiiio",
+  { "hvs2",  S(HVS2), TB, 3,  "",  "kkiiiiiio",
     (SUBR)hvs2_set, (SUBR)hvs2, (SUBR)NULL },
-  { "hvs3",  S(HVS3), TB|3,  "",  "kkkiiiiiiio",
+  { "hvs3",  S(HVS3), TB, 3,  "",  "kkkiiiiiiio",
     (SUBR)hvs3_set, (SUBR)hvs3, (SUBR)NULL },
-  { "vphaseseg", S(VPSEG), TB|3,  "",  "kiim",
+  { "vphaseseg", S(VPSEG), TB, 3,  "",  "kiim",
     (SUBR)vphaseseg_set, (SUBR)vphaseseg }
 };
 
