@@ -36,7 +36,7 @@ typedef struct {
 static int  mtable1_set(CSOUND *csound, MTABLE1 *p)      /* mtab by G.Maldonado */
 {
     FUNC *ftp;
-    if ((ftp = csound->FTFind(csound, p->xfn)) == NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL)
       return csound->InitError(csound, Str("vtable1: incorrect table number"));
     p->ftable = ftp->ftable;
     p->nargs = p->INOCOUNT-1;
@@ -52,7 +52,8 @@ static int  mtable1_k(CSOUND *csound, MTABLE1 *p)
     if (p->pfn != (long)*p->xfn) {
       FUNC *ftp;
       if ( (ftp = csound->FTFindP(csound, p->xfn) ) == NULL)
-        return csound->PerfError(csound, Str("vtable1: incorrect table number"));
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("vtable1: incorrect table number"));
       p->pfn = (long)*p->xfn;
       p->ftable = ftp->ftable;
     }
@@ -92,9 +93,10 @@ PUBLIC int Sched(CSOUND *csound, MYFLT  *args[], int numargs) {
       unquote(name, csound->currevent->strarg);
       evt.strarg = name;
       evt.p[1] = SSTRCOD;
+      evt.scnt = 1;
     }
     else {
-      evt.strarg = NULL;
+      evt.strarg = NULL; evt.scnt = 0;
       evt.p[1] = *args[1];
     }
     evt.opcod = (char) *args[0];
@@ -106,7 +108,9 @@ PUBLIC int Sched(CSOUND *csound, MYFLT  *args[], int numargs) {
     if(starttime < FL(0.0)) {
       starttime = FL(0.0);
     }
-    starttime += (double) csound->global_kcounter / (double)csound->global_ekr;
+    starttime += (double) csound->GetKcounter(csound) /
+                 (double) csound->GetKr(csound);
+    /*starttime += (double) csound->global_kcounter / (double)csound->global_ekr;*/
    /* Copy all arguments to the new event */
     for (i = 0; i < numargs; i++)
       evt.p[i] = *args[i];
@@ -140,7 +144,7 @@ static int schedk_i(CSOUND *csound, SCHEDK *p)
     O->RTevents = 1;     /* Make sure kperf() looks for RT events */
     /*   O->ksensing = 1; */
     /*   O->OrcEvts  = 1; */    /* - of the appropriate type */
-    if (csound->kcounter > 0 && *p->trigger != FL(0.0) && p->h.insdshead->p3 == 0)
+    if (CS_KCNT > 0 && *p->trigger != FL(0.0) && p->h.insdshead->p3 == 0)
       schedk(csound,p);
     return OK;
 }
@@ -181,7 +185,7 @@ static int schedInTime_set(CSOUND *csound, SCHEDINTIME *p)
      p->argums[0]=&p->args2[0];
 
 
-     if (csound->kcounter > 0 && *p->trigger != 0 && p->h.insdshead->p3 == 0) {
+     if (CS_KCNT > 0 && *p->trigger != 0 && p->h.insdshead->p3 == 0) {
        if (*p->argums[3] >= 0) *p->argums[3] = -1; /* start a held note */
        *p->argums[1] += p->frac;
        Sched(csound, p->argums, p->nargs);
@@ -221,7 +225,7 @@ static int schedInTime(CSOUND *csound, SCHEDINTIME *p)
 /* -------------------------------------------------------------------- */
 
 /* These opocdes were not implemented because the functionality of      */
-/* CopyTabElements has already been included in vcopy and vcopy_i       */ 
+/* CopyTabElements has already been included in vcopy and vcopy_i       */
 
 typedef struct { /* GAB 11/Jan/2001 */
      OPDS   h;
@@ -236,9 +240,10 @@ static int copyTabElems_set(CSOUND *csound, COPYTABELEMS *p)
 {
     FUNC *ftp;
     int nelems = (int) *p->inumElems;
-    if ((ftp = csound->FTFind(csound, p->idestTab)) == NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->idestTab)) == NULL)
       return csound->InitError(csound,
-                               Str("copyTabElems: incorrect destination table number"));
+                               Str("copyTabElems: incorrect destination "
+                                   "table number"));
 
     p->dLen = ftp->flen;
     if (nelems > p->dLen)
@@ -247,7 +252,7 @@ static int copyTabElems_set(CSOUND *csound, COPYTABELEMS *p)
                                    "or number of elements to copy too big"));
 
     p->dTable = ftp->ftable;
-    if ((ftp = csound->FTFind(csound, p->isourceTab)) == NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->isourceTab)) == NULL)
       return csound->InitError(csound,
                                Str("copyTabElems: incorrect source table number"));
 
@@ -270,9 +275,11 @@ static int copyTabElems(CSOUND *csound, COPYTABELEMS *p)
       int j, sNdx = (int) *p->ksourceIndex * nelems,
              dNdx = (int) *p->kdestIndex * nelems;
       if (sNdx + nelems > p->sLen)
-        return csound->PerfError(csound, Str("copyTabElems: source table too short"));
+        return
+          csound->PerfError(csound, p->h.insdshead,
+                            Str("copyTabElems: source table too short"));
       if (dNdx + nelems > p->dLen)
-        return csound->PerfError(csound,
+        return csound->PerfError(csound, p->h.insdshead,
                                  Str("copyTabElems: destination table too short"));
 
       for (j = 0; j< nelems; j++)
@@ -291,7 +298,7 @@ static int copyTabElemsi(CSOUND *csound, COPYTABELEMS_I *p)
     FUNC *ftp;
     int nelems = (int) *p->inumElems, dLen, sLen;
     MYFLT *sTable, *dTable;
-    if ((ftp = csound->FTFind(csound, p->idestTab)) == NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->idestTab)) == NULL)
       return csound->InitError(csound,
                                Str("copyTabElems: incorrect destination "
                                    "table number"));
@@ -301,7 +308,7 @@ static int copyTabElemsi(CSOUND *csound, COPYTABELEMS_I *p)
                                Str("copyTabElems: destination table too short "
                                    "or number of elements to copy too big"));
     dTable = ftp->ftable;
-    if ((ftp = csound->FTFind(csound, p->isourceTab)) == NULL)
+    if ((ftp = csound->FTnp2Find(csound, p->isourceTab)) == NULL)
       return csound->InitError(csound,
                                Str("copyTabElems: incorrect source table number"));
     sLen = ftp->flen;
@@ -315,9 +322,11 @@ static int copyTabElemsi(CSOUND *csound, COPYTABELEMS_I *p)
       int j, sNdx = (int) *p->isourceIndex * nelems,
         dNdx = (int) *p->idestIndex * nelems;
       if (sNdx + nelems > sLen)
-        return csound->PerfError(csound, Str("copyTabElems: source table too short"));
+        return
+          csound->PerfError(csound, p->h.insdshead,
+                            Str("copyTabElems: source table too short"));
       if (dNdx + nelems > dLen)
-        return csound->PerfError(csound,
+        return csound->PerfError(csound, p->h.insdshead,
                                  Str("copyTabElems: destination table too short"));
       for (j = 0; j< nelems; j++) {
         dTable[dNdx+j] = sTable[sNdx+j];
@@ -345,91 +354,41 @@ static int inRange_i(CSOUND *csound, INRANGE *p)
 /*p->numChans = (PortaudioNumOfInPorts == -1) ? nchnls : PortaudioNumOfInPorts; */
     if (!csound->oparms->sfread)
       return csound->InitError(csound, Str("inrg: audio input is not enabled"));
-    p->numChans = csound->nchnls;
+    p->numChans = csound->GetNchnls(csound);
     return OK;
 }
 
 static int inRange(CSOUND *csound, INRANGE *p)
 {
-    int j, nsmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t j, nsmps = CS_KSMPS;
+    int i;
     MYFLT *ara[VARGMAX];
     int startChan = (int) *p->kstartChan -1;
     MYFLT *sp = csound->spin + startChan;
     int narg = p->narg, numchans = p->numChans;
 
     if (startChan < 0)
-      return csound->PerfError(csound,
+      return csound->PerfError(csound, p->h.insdshead,
                                Str("inrg: channel number cannot be < 1 "
                                    "(1 is the first channel)"));
 
-    for (j = 0; j < narg; j++)
-      ara[j] = p->argums[j];
-
-    nsmps = csound->ksmps;
-    do  {
-      int i;
+    if (UNLIKELY(early)) nsmps -= early;
+    for (i = 0; i < narg; i++) {
+      ara[i] = p->argums[i];
+      if (UNLIKELY(offset)) memset(ara[i], '\0', offset*sizeof(MYFLT));
+      if (UNLIKELY(early)) memset(&ara[i][nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (j=offset; j<nsmps; j++)  {
       for (i=0; i<narg; i++)
-        *ara[i]++ = sp[i];
+        ara[i][j] = sp[i];
       sp += numchans;
-    } while (--nsmps);
-    return OK;
-
-}
-/* -------------------------------------------------------------------- */
-
-typedef struct {
-        OPDS    h;
-        MYFLT   *kstartChan, *argums[VARGMAX];
-        int narg;
-} OUTRANGE;
-
-static int outRange_i(CSOUND *csound, OUTRANGE *p)
-{
-    p->narg = p->INOCOUNT-1;
-    return OK;
-}
-
-
-static int outRange(CSOUND *csound, OUTRANGE *p)
-{
-    int j, n, nsmps;
-    int ksmps = csound->ksmps, nchnls = csound->nchnls;
-    MYFLT *ara[VARGMAX];
-    int startChan = (int) *p->kstartChan -1;
-    MYFLT *sp = csound->spout + startChan;
-    int narg = p->narg;
-    if (startChan < 0)
-      return csound->PerfError(csound,
-                               Str("outrg: channel number cannot be < 1 "
-                                   "(1 is the first channel)"));
-
-    for (j = 0; j < narg; j++)
-      ara[j] = p->argums[j];
-
-    nsmps = ksmps;
-    if (!csound->spoutactive) {
-      memset(sp, 0, ksmps * nchnls * sizeof(MYFLT));
-      for (n=0; n<nsmps; n++) {
-        int i;
-        MYFLT *sptemp = sp;
-        for (i=0; i < narg; i++)
-          sptemp[i] = *ara[i]++;
-        sp += nchnls;
-      }
-      csound->spoutactive = 1;
-    }
-    else {
-      for (n=0; n<nsmps; n++) {
-        int i;
-        MYFLT *sptemp = sp;
-        for (i=0; i < narg; i++)
-          sptemp[i] += *ara[i]++;
-        sp += nchnls;
-      }
     }
     return OK;
+
 }
-/* -------------------------------------------------------------------- */
+
 #include "Opcodes/uggab.h"
 
 static int lposc_set(CSOUND *csound, LPOSC *p)
@@ -442,7 +401,7 @@ static int lposc_set(CSOUND *csound, LPOSC *p)
        csound->Message(csound,
                        Str("lposc: no sample rate stored in function;"
                            " assuming=sr\n"));
-       p->fsr=csound->esr;
+       p->fsr=CS_ESR;
     }
     p->ftp    = ftp;
     p->tablen = ftp->flen;
@@ -460,78 +419,16 @@ static int lposc_set(CSOUND *csound, LPOSC *p)
     return OK;
 }
 
-#if 0
-
-/* These opcodes left out while GEN22 is implemented */
-static int lposcint(CSOUND *csound, LPOSC *p)
-{
-    double  *phs= &p->phs;
-    double  si= *p->freq * (p->fsr/ csound->esr);
-
-    MYFLT   *out = p->out;
-    short   *ft = (short *) p->ftp->ftable, *curr_samp;
-    MYFLT   fract;
-    long     n = csound->ksmps;
-    long     loop, end, looplength; /* = p->looplength ; */
-
-    if ((loop = (long) *p->kloop) < 0) loop=0;// gab
-    else if (loop > p->tablen-3) loop = p->tablen-3;
-    if ((end = (long) *p->kend) > p->tablen-1 ) end = p->tablen - 1;
-    else if (end <= 2) end = 2;
-    if (end < loop+2) end = loop + 2;
-
-    looplength = end - loop;
-
-    do {
-      curr_samp= ft + (long)*phs;
-      fract= (MYFLT)(*phs - (long)*phs);
-      *out++ = *p->amp * (*curr_samp +(*(curr_samp+1)-*curr_samp)*fract);
-      *phs += si;
-      while (*phs  >= end) *phs -= looplength;
-      while (*phs  < loop) *phs += looplength;
-    } while (--n);
-    return OK;
-}
-
-static int lposcinta(CSOUND *csound, LPOSC *p)
-{
-
-    double  *phs= &p->phs;
-    double  si= *p->freq * (p->fsr/csound->esr);
-    MYFLT    *out = p->out,  *amp=p->amp;
-    short   *ft = (short *) p->ftp->ftable, *curr_samp;
-    MYFLT   fract;
-    long     n, nsmps = csound->ksmps;
-    long     loop, end, looplength; /* = p->looplength ; */
-
-    if ((loop = (long) *p->kloop) < 0) loop=0;// gab
-    else if (loop > p->tablen-3) loop = p->tablen-3;
-    if ((end = (long) *p->kend) > p->tablen-1 ) end = p->tablen - 1;
-    else if (end <= 2) end = 2;
-    if (end < loop+2) end = loop + 2;
-
-    looplength = end - loop;
-    for (n=0; n<nsmps; n++) {
-      curr_samp= ft + (long)*phs;
-      fract= (MYFLT)(*phs - (long)*phs);
-      out[n] = amp[n] * (*curr_samp +(*(curr_samp+1)-*curr_samp)*fract);
-      *phs += si;
-      while (*phs  >= end) *phs -= looplength;
-      while (*phs  < loop) *phs += looplength;
-    }
-    return OK;
-}
-#endif
-
-
 static int lposca(CSOUND *csound, LPOSC *p)
 {
     double  *phs= &p->phs;
-    double  si= *p->freq * (p->fsr/csound->esr);
+    double  si= *p->freq * (p->fsr/CS_ESR);
     MYFLT   *out = p->out,  *amp=p->amp;
     MYFLT   *ft =  p->ftp->ftable, *curr_samp;
     MYFLT   fract;
-    int32   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32   loop, end, looplength /* = p->looplength */ ;
 
     if ((loop = (long) *p->kloop) < 0) loop=0;/* gab */
@@ -540,7 +437,12 @@ static int lposca(CSOUND *csound, LPOSC *p)
     else if (end <= 2) end = 2;
     if (end < loop+2) end = loop + 2;
     looplength = end - loop;
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       curr_samp= ft + (long)*phs;
       fract= (MYFLT)(*phs - (long)*phs);
       out[n] = amp[n] * (*curr_samp +(*(curr_samp+1)-*curr_samp)*fract);
@@ -553,107 +455,6 @@ static int lposca(CSOUND *csound, LPOSC *p)
 
 /* -------------------------------------------------------------------- */
 
-/* These opcodes left out while GEN22 is implemented */
-
-#if 0
-typedef struct {
-     OPDS    h;
-     MYFLT   *out1, *out2, *amp, *freq, *kloop, *kend, *ift, *iphs;
-     long    tablen;
-     MYFLT   fsr;
-     short *ft;
-     double  phs, fsrUPsr /* , looplength; */
-     long    phs_int;
-} LPOSCINT_ST;
-
-static int lposcint_stereo_set(CSOUND *csound, LPOSCINT_ST *p)
-{
-    FUNC *ftp;
-    double  loop, end, looplength, fsr;
-    if ((ftp = csound->FTnp2Find(csound, p->ift)) == NULL)
-      return csound->InitError(csound, Str("invalid function"));
-    if (!(fsr = ftp->gen01args.sample_rate)){
-      csound->Message(csound,
-                      Str("lposcil: no sample rate stored in function assuming=sr\n"));
-      p->fsr=csound->esr;
-    }
-    p->fsrUPsr = fsr/csound->esr;
-    p->ft    = (short *) ftp->ftable;
-    p->tablen = ftp->flen/2;
-    /* changed from
-       p->phs    = *p->iphs * p->tablen; */
-    if ((loop = (long) *p->kloop) < 0) loop=0;// gab
-    else if (loop > p->tablen-3) loop = p->tablen-3;
-    if ((end = (long) *p->kend) > p->tablen-1 ) end = p->tablen - 1;
-    else if (end <= 2) end = 2;
-    if (end < loop+2) end = loop + 2;
-    looplength = end - loop;
-
-    if (*p->iphs >= 0)
-      p->phs_int = (long) (p->phs = *p->iphs);
-
-    while (p->phs >= end)
-      p->phs_int = (long) (p->phs -= looplength);
-    return OK;
-}
-
-static int lposcinta_stereo(CSOUND *csound, LPOSCINT_ST *p) // stereo lposcinta
-{
-    double  *phs= &p->phs,   si= *p->freq * p->fsrUPsr;
-    MYFLT    *out1 = p->out1, *out2 = p->out2, *amp=p->amp;
-    short   *ft =  p->ft;
-    long    n = csound->ksmps;
-    long     loop, end, looplength;
-    if ((loop = (long) *p->kloop) < 0) loop=0;// gab
-    else if (loop > p->tablen-3) loop = p->tablen-3;
-    if ((end = (long) *p->kend) > p->tablen-1 ) end = p->tablen - 1;
-    else if (end <= 2) end = 2;
-    if (end < loop+2) end = loop + 2;
-    looplength = end - loop;
-    do {
-      double fract;
-      short *curr_samp1 = ft + (long) *phs * 2;
-      short *curr_samp2 = curr_samp1 +1;
-      fract= *phs - (long) *phs;
-      *out1++ = *amp   * (MYFLT) (*curr_samp1 +(*(curr_samp1+2)-*curr_samp1)*fract);
-      *out2++ = *amp++ * (MYFLT) (*curr_samp2 +(*(curr_samp2+2)-*curr_samp2)*fract);
-      *phs += si;
-      while (*phs  >= end) *phs -= looplength;
-      while (*phs  < loop) *phs += looplength;
-    } while (--n);
-    return OK;
-}
-
-static int lposcinta_stereo_no_trasp(CSOUND *csound, LPOSCINT_ST *p)
-                            /* trasposition is allowed only */
-{                           /* in integer values (twice, three times etc.) */
-                            /* so it is faster */
-    long *phs = &p->phs_int, si = (long) *p->freq;
-    MYFLT    *out1 = p->out1, *out2 = p->out2, *amp=p->amp;
-    short   *ft =  p->ft;
-    long    n = csound->ksmps;
-    long     loop, end, looplength /* = p->looplength ; */
-      if ((loop = (long) *p->kloop) < 0) loop=0;/* gab */
-      else if (loop > p->tablen-3) loop = p->tablen-3;
-    if ((end = (long) *p->kend) > p->tablen-1 ) end = p->tablen - 1;
-    else if (end <= 2) end = 2;
-    if (end < loop+2) end = loop + 2;
-    looplength = end - loop;
-
-    do {
-      short *curr_samp1 = ft + *phs * 2;
-      *out1++ = *amp   * (MYFLT) *curr_samp1 ;
-      *out2++ = *amp++ * (MYFLT) *(curr_samp1+1) ;
-      *phs += si;
-      while (*phs  >= end) *phs -= looplength;
-      while (*phs  < loop) *phs += looplength;
-    } while (--n);
-    return OK;
-}
-#endif
-
-/* -------------------------------------------------------------------- */
-
 typedef struct  {
         OPDS    h;
         MYFLT   *out1, *out2, *amp, *freq, *kloop, *kend, *ift, *iphs;
@@ -673,9 +474,9 @@ static int lposc_stereo_set(CSOUND *csound, LPOSC_ST *p)
     if (!(fsr = ftp->gen01args.sample_rate)) {
       csound->Message(csound, Str("lposcil: no sample rate stored in function;"
                                   " assuming=sr\n"));
-      p->fsr=csound->esr;
+      p->fsr=CS_ESR;
     }
-    p->fsrUPsr = fsr/csound->esr;
+    p->fsrUPsr = fsr/CS_ESR;
     p->ft     = ftp->ftable;
     p->tablen = ftp->flen/2;
     /* changed from
@@ -700,7 +501,9 @@ static int lposca_stereo(CSOUND *csound, LPOSC_ST *p) /* stereo lposcinta */
     double  *phs= &p->phs,   si= *p->freq * p->fsrUPsr;
     MYFLT   *out1 = p->out1, *out2 = p->out2, *amp=p->amp;
     MYFLT   *ft =  p->ft;
-    int32    n = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32    loop, end, looplength /* = p->looplength */ ;
     if ((loop = (long) *p->kloop) < 0) loop=0;/* gab */
     else if (loop > p->tablen-3) loop = p->tablen-3;
@@ -708,17 +511,26 @@ static int lposca_stereo(CSOUND *csound, LPOSC_ST *p) /* stereo lposcinta */
     else if (end <= 2) end = 2;
     if (end < loop+2) end = loop + 2;
     looplength = end - loop;
-    do {
+    if (UNLIKELY(offset)) {
+      memset(out1, '\0', offset*sizeof(MYFLT));
+      memset(out2, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out1[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out2[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       double fract;
       MYFLT *curr_samp1 = ft + (long) *phs * 2;
       MYFLT *curr_samp2 = curr_samp1 +1;
       fract= *phs - (long) *phs;
-      *out1++ = *amp   * (MYFLT) (*curr_samp1 +(*(curr_samp1+2)-*curr_samp1)*fract);
-      *out2++ = *amp++ * (MYFLT) (*curr_samp2 +(*(curr_samp2+2)-*curr_samp2)*fract);
+      out1[n] = amp[n] * (MYFLT)(*curr_samp1 +(*(curr_samp1+2)-*curr_samp1)*fract);
+      out2[n] = amp[n] * (MYFLT)(*curr_samp2 +(*(curr_samp2+2)-*curr_samp2)*fract);
       *phs += si;
       while (*phs  >= end) *phs -= looplength;
       while (*phs  < loop) *phs += looplength;
-    } while (--n);
+    }
     return OK;
 }
 
@@ -728,7 +540,9 @@ static int lposca_stereo_no_trasp(CSOUND *csound, LPOSC_ST *p)
     long    *phs = &p->phs_int, si = (long) *p->freq;
     MYFLT   *out1 = p->out1, *out2 = p->out2, *amp=p->amp;
     MYFLT   *ft =  p->ft;
-    long    n = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     long    loop, end, looplength /* = p->looplength */ ;
     if ((loop = (long) *p->kloop) < 0) loop=0;/* gab */
     else if (loop > p->tablen-3) loop = p->tablen-3;
@@ -737,21 +551,29 @@ static int lposca_stereo_no_trasp(CSOUND *csound, LPOSC_ST *p)
     if (end < loop+2) end = loop + 2;
     looplength = end - loop;
 
-    do {
+    if (UNLIKELY(offset)) {
+      memset(out1, '\0', offset*sizeof(MYFLT));
+      memset(out2, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out1[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&out2[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       MYFLT *curr_samp1 = ft + *phs * 2;
-      *out1++ = *amp   * (MYFLT) *curr_samp1 ;
-      *out2++ = *amp++ * (MYFLT) *(curr_samp1+1) ;
+      out1[n] = amp[n] * (MYFLT) *curr_samp1 ;
+      out2[n] = amp[n] * (MYFLT) *(curr_samp1+1) ;
       *phs += si;
       while (*phs  >= end) *phs -= looplength;
       while (*phs  < loop) *phs += looplength;
-    } while (--n);
+    }
     return OK;
 }
 
 
 /* -------------------------------------------------------------------- */
 
-/* #undef oneUp31Bit \/\* to avoid warnings */
 #include "vectorial.h"
 
 typedef struct  {       /* gab d5*/
@@ -788,12 +610,12 @@ static int dashow (CSOUND *csound, DSH *p)
     MYFLT range = *p->kband_max - *p->kband_min;
     if (range != FL(0.0))
       *p->rmod = (*p->kfreq_max - *p->kfreq_min) / (*p->kband_max - *p->kband_min);
-    else 
+    else
       *p->rmod = FL(0.0);
     *p->rcar = (*p->kfreq_max - (*p->kband_max * *p->rmod));
 
-    if (*p->rmod <= FL(0.0)) *p->rmod = (MYFLT) fabs (*p->rmod);
-    if (*p->rcar <= FL(0.0)) *p->rcar = (MYFLT) fabs (*p->rcar);
+    if (*p->rmod <= FL(0.0)) *p->rmod = FABS (*p->rmod);
+    if (*p->rcar <= FL(0.0)) *p->rcar = FABS (*p->rcar);
     return OK;
 }
 #endif
@@ -804,47 +626,39 @@ static int dashow (CSOUND *csound, DSH *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-  { "vtable1k",       S(MTABLE1),         TR|3,  "",  "kz",          (SUBR)mtable1_set,      (SUBR)mtable1_k,        (SUBR) NULL },
-  /*      { "schedk",         S(SCHEDK),          3,  "",  "kkz",         (SUBR)schedk_i,         (SUBR) schedk,          (SUBR) NULL }, */
-  /*      { "schedInTime",    S(SCHEDINTIME),     3,  "",  "kz",          (SUBR)schedInTime_set,  (SUBR)schedInTime ,     (SUBR) NULL }, */
-  /*      { "copyTabElems",   S(COPYTABELEMS),    3,  "",  "kikiki",      (SUBR)copyTabElems_set, (SUBR)copyTabElems,     (SUBR)NULL  }, */
-  /*      { "copyTabElemsi",  S(COPYTABELEMS_I),  1,  "",  "iiiii",       (SUBR)copyTabElemsi,    (SUBR)NULL,             (SUBR)NULL  }, */
-  /*      { "Trandom",        S(TRANGERAND),              2,  "k", "kkk",                 NULL,                                   (SUBR)trRangeRand }, */
-  { "trandom",        S(TRANGERAND),              2,  "k", "kkk",                 NULL,                                   (SUBR)trRangeRand },
-  /*      { "lposcinta",      S(LPOSC),           5,      "a", "akkkio",      (SUBR)lposc_set,            NULL,                                   (SUBR)lposcinta}, */
-  /*      { "lposcintsa",     S(LPOSCINT_ST),     5,  "aa","akkkio",              (SUBR)lposcint_stereo_set,NULL,                                 (SUBR)lposcinta_stereo}, */
-  /*      { "lposcintsa2",    S(LPOSCINT_ST),     5,  "aa","akkkio",              (SUBR)lposcint_stereo_set,NULL,                                 (SUBR)lposcinta_stereo_no_trasp}, */
-  { "lposcila", S(LPOSC),      TR|5, "a", "akkkio", (SUBR)lposc_set, NULL, (SUBR)lposca},
-  { "lposcilsa", S(LPOSC_ST),  TR|5, "aa","akkkio", (SUBR)lposc_stereo_set, NULL, (SUBR)lposca_stereo},
-  { "lposcilsa2", S(LPOSC_ST), TR|5, "aa","akkkio", (SUBR)lposc_stereo_set, NULL, (SUBR)lposca_stereo_no_trasp},
-  /* { "dashow.i", S(DSH), 1,  "ii","iiii", (SUBR)dashow     }, */
-  /* { "dashow.k", S(DSH), 2,  "kk","kkkk", NULL, (SUBR)dashow   }, */
-  { "inrg", S(INRANGE), 5, "", "ky", (SUBR)inRange_i, (SUBR)NULL, (SUBR)inRange },
-  { "outrg", S(OUTRANGE), 5, "", "ky", (SUBR)outRange_i, (SUBR)NULL, (SUBR)outRange}
+  { "vtable1k",       S(MTABLE1),         TR, 3,  "",  "kz",
+                  (SUBR)mtable1_set,      (SUBR)mtable1_k,        (SUBR) NULL },
+  /*      { "schedk",         S(SCHEDK),     0,     3,  "",  "kkz",
+                  (SUBR)schedk_i,         (SUBR) schedk,          (SUBR) NULL }, */
+  /*      { "schedInTime",    S(SCHEDINTIME), 0,    3,  "",  "kz",
+                  (SUBR)schedInTime_set,  (SUBR)schedInTime ,     (SUBR) NULL }, */
+  /*      { "copyTabElems",   S(COPYTABELEMS),0,    3,  "",  "kikiki",
+                  (SUBR)copyTabElems_set, (SUBR)copyTabElems,     (SUBR)NULL  }, */
+  /*      { "copyTabElemsi",  S(COPYTABELEMS_I), 0, 1,  "",  "iiiii",
+                  (SUBR)copyTabElemsi,    (SUBR)NULL,             (SUBR)NULL  }, */
+  /*      { "Trandom",        S(TRANGERAND),        0,      2,  "k", "kkk",
+                    NULL,                                   (SUBR)trRangeRand }, */
+  { "trandom",        S(TRANGERAND),          0,    2,  "k", "kkk",
+                    NULL,                                   (SUBR)trRangeRand },
+  /*      { "lposcinta",      S(LPOSC),        0,   5,      "a", "akkkio",
+                                      (SUBR)lposc_set,  NULL,  (SUBR)lposcinta}, */
+  /*      { "lposcintsa",     S(LPOSCINT_ST),  0,   5,  "aa","akkkio",
+                        (SUBR)lposcint_stereo_set,NULL, (SUBR)lposcinta_stereo}, */
+  /*      { "lposcintsa2",    S(LPOSCINT_ST),   0,  5,  "aa","akkkio",
+              (SUBR)lposcint_stereo_set,NULL,  (SUBR)lposcinta_stereo_no_trasp}, */
+  { "lposcila", S(LPOSC),      TR, 5, "a", "akkkio",
+                                           (SUBR)lposc_set, NULL, (SUBR)lposca},
+  { "lposcilsa", S(LPOSC_ST),  TR, 5, "aa","akkkio",
+                             (SUBR)lposc_stereo_set, NULL, (SUBR)lposca_stereo},
+  { "lposcilsa2", S(LPOSC_ST), TR, 5, "aa","akkkio",
+                    (SUBR)lposc_stereo_set, NULL, (SUBR)lposca_stereo_no_trasp},
+  /* { "dashow.i", S(DSH), 0,1,  "ii","iiii", (SUBR)dashow     }, */
+  /* { "dashow.k", S(DSH), 0,2,  "kk","kkkk", NULL, (SUBR)dashow   }, */
+  { "inrg", S(INRANGE), 0,5, "", "ky", (SUBR)inRange_i, (SUBR)NULL, (SUBR)inRange }
+
+
 };
 
-
-
-typedef struct NEWGABOPC_GLOBALS_ {
-        int butta;
-
-} NEWGABOPC_GLOBALS;
-
-/*
-PUBLIC int csoundModuleInfo(void)
-{
-    return ((CS_APIVERSION << 16) + (CS_APISUBVER << 8) + (int) sizeof(MYFLT));
-}
-
-
-PUBLIC int csoundModuleCreate(CSOUND *csound)
-{
-    (void)csound;
-    return 0;
-}
-*/
-
-
 int newgabopc_init_(CSOUND *csound) {
    return csound->AppendOpcodes(csound, &(localops[0]),
                                 (int) (sizeof(localops) / sizeof(OENTRY)));
