@@ -21,7 +21,7 @@
 
 typedef struct {
         OPDS    h;
-        MYFLT   *out, *xindex, *xinterpoint, *xtabndx1, *xtabndx2, 
+        MYFLT   *out, *xindex, *xinterpoint, *xtabndx1, *xtabndx2,
                 *argums[VARGMAX];
         MYFLT   *table[VARGMAX];
         int     length;
@@ -39,7 +39,7 @@ static int tabmorph_set (CSOUND *csound, TABMORPH *p) /*Gab 13-March-2005 */
     numOfTabs = p->numOfTabs =((p->INCOUNT-4)); /* count segs & alloc if nec */
     argp = p->argums;
     for (j=0; j< numOfTabs; j++) {
-      if (UNLIKELY((ftp = csound->FTFind(csound, *argp++)) == NULL))
+      if (UNLIKELY((ftp = csound->FTnp2Find(csound, *argp++)) == NULL))
         return csound->InitError(csound, Str("tabmorph: invalid table number"));
       if (UNLIKELY(ftp->flen != flength && flength  != 0))
         return
@@ -140,15 +140,22 @@ static int tabmorphi(CSOUND *csound, TABMORPH *p) /* interpolation */
 
 static int atabmorphia(CSOUND *csound, TABMORPH *p) /* all arguments at a-rate */
 {
-    int    n, nsmps = csound->ksmps, tablen = p->length;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int      tablen = p->length;
     MYFLT *out = p->out;
     MYFLT *index = p->xindex;
     MYFLT *interpoint = p->xinterpoint;
     MYFLT *tabndx1 = p->xtabndx1;
     MYFLT *tabndx2 = p->xtabndx2;
 
-
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       MYFLT index_frac,tabndx1frac, tabndx2frac;
       MYFLT tab1val1a,tab1val2a, tab2val1a, tab2val2a, val1, val2;
       MYFLT val1a, val2a, val1b, val2b, tab1val1b,tab1val2b, tab2val1b, tab2val2b;
@@ -159,7 +166,6 @@ static int atabmorphia(CSOUND *csound, TABMORPH *p) /* all arguments at a-rate *
       index_frac = ndx - index_int;
       index_int %= tablen;
 
-
       tabndx1int = (int) tabndx1[n];
       tabndx1frac = tabndx1[n] - tabndx1int;
       tabndx1int %= p->numOfTabs;
@@ -200,7 +206,10 @@ static int atabmorphia(CSOUND *csound, TABMORPH *p) /* all arguments at a-rate *
  /* all args k-rate except out and table index */
 static int atabmorphi(CSOUND *csound, TABMORPH *p)
 {
-    int    n, nsmps = csound->ksmps, tablen = p->length;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int      tablen = p->length;
 
     MYFLT *out = p->out;
     MYFLT *index;
@@ -223,7 +232,12 @@ static int atabmorphi(CSOUND *csound, TABMORPH *p)
     interpoint = *p->xinterpoint;
     interpoint -= (int) interpoint; /* to limit to zero to 1 range */
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       MYFLT index_frac, tab1val1a,tab1val2a, tab2val1a, tab2val2a, val1, val2;
       MYFLT val1a, val2a, val1b, val2b, tab1val1b,tab1val2b, tab2val1b, tab2val2b;
       long index_int;
@@ -264,13 +278,13 @@ static int atabmorphi(CSOUND *csound, TABMORPH *p)
 
 OENTRY tabmoroph_localops[] = {
 
-{ "tabmorph",  S(TABMORPH), TR|3,  "k", "kkkkm",
+{ "tabmorph",  S(TABMORPH), TR, 3,  "k", "kkkkm",
                (SUBR) tabmorph_set, (SUBR) tabmorph, NULL},
-{ "tabmorphi", S(TABMORPH), TR|3,  "k", "kkkkm",
+{ "tabmorphi", S(TABMORPH), TR, 3,  "k", "kkkkm",
                (SUBR) tabmorph_set, (SUBR) tabmorphi, NULL},
-{ "tabmorpha", S(TABMORPH), TR|5,  "a", "aaaam",
+{ "tabmorpha", S(TABMORPH), TR, 5,  "a", "aaaam",
                (SUBR) tabmorph_set,  NULL, (SUBR) atabmorphia},
-{ "tabmorphak",S(TABMORPH), TR|5,  "a", "akkkm",
+{ "tabmorphak",S(TABMORPH), TR, 5,  "a", "akkkm",
                (SUBR) tabmorph_set,  NULL, (SUBR) atabmorphi }
 
 };
@@ -280,4 +294,3 @@ int tabmorph_init_(CSOUND *csound) {
       csound->AppendOpcodes(csound, &(tabmoroph_localops[0]),
                             (int) (sizeof(tabmoroph_localops) / sizeof(OENTRY)));
 }
-
