@@ -20,7 +20,7 @@
     Optional arguments to some opcodes and other fixes by Andres Cabrera
     and Istvan Varga.
 */
-//#include "csdl.h"
+//#include "stdopcod.h"
 #include "csoundCore.h"
 #include "interlocks.h"
 #include "vectorial.h"
@@ -32,7 +32,7 @@ static int mtable_i(CSOUND *csound,MTABLEI *p)
     int j, nargs;
     MYFLT *table, xbmul = FL(0.0), **out = p->outargs;
     if ((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL) {
-      return csound->InitError(csound, Str("mtablei: incorrect table number"));
+      return csound->InitError(csound, Str("vtablei: incorrect table number"));
     }
     table = ftp->ftable;
     nargs = p->INOCOUNT-4;
@@ -62,7 +62,7 @@ static int mtable_set(CSOUND *csound,MTABLE *p) /*  mtab by G.Maldonado */
 {
     FUNC *ftp;
     if ((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL) {
-      return csound->InitError(csound, Str("mtable: incorrect table number"));
+      return csound->InitError(csound, Str("vtable: incorrect table number"));
     }
     p->ftable = ftp->ftable;
     p->nargs = p->INOCOUNT-4;
@@ -82,7 +82,8 @@ static int mtable_k(CSOUND *csound,MTABLE *p)
     if (p->pfn != (long)*p->xfn) {
       FUNC *ftp;
       if ( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL) {
-        return csound->PerfError(csound, Str("mtable: incorrect table number"));
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("vtablek: incorrect table number"));
       }
       p->pfn = (long)*p->xfn;
       p->ftable = ftp->ftable;
@@ -123,7 +124,10 @@ static int mtable_k(CSOUND *csound,MTABLE *p)
 static int mtable_a(CSOUND *csound,MTABLE *p)
 {
     int j, nargs = p->nargs;
-    int nsmps = csound->ksmps, ixmode = (int) *p->ixmode, k;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+     uint32_t k, nsmps = CS_KSMPS;
+    int ixmode = (int) *p->ixmode;
     MYFLT **out = p->outargs;
     MYFLT *table;
     MYFLT *xndx = p->xndx, xbmul;
@@ -132,7 +136,8 @@ static int mtable_a(CSOUND *csound,MTABLE *p)
     if (p->pfn != (long)*p->xfn) {
       FUNC *ftp;
       if ( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL) {
-        return csound->PerfError(csound, Str("mtable: incorrect table number"));
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("vtablea: incorrect table number"));
       }
       p->pfn = (long)*p->xfn;
       p->ftable = ftp->ftable;
@@ -143,12 +148,20 @@ static int mtable_a(CSOUND *csound,MTABLE *p)
     table = p->ftable;
     len = p->len;
     xbmul = p->xbmul;
+    if (UNLIKELY(offset))
+      for (j=0; j < nargs; j++)
+        memset(out[j], '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      for (j=0; j < nargs; j++)
+        memset(&out[j][nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (*p->kinterp) {
       MYFLT fndx;
       long indx;
       MYFLT fract;
       long indxp1;
-      for (k=0; k<nsmps; k++) {
+      for (k=offset; k<nsmps; k++) {
         MYFLT   v1, v2 ;
         fndx = (ixmode) ? *xndx++ * xbmul : *xndx++;
         if (fndx >= len)
@@ -166,7 +179,7 @@ static int mtable_a(CSOUND *csound,MTABLE *p)
       }
     }
     else {
-      for (k=0; k<nsmps; k++) {
+      for (k=offset; k<nsmps; k++) {
         long indx = (ixmode) ? ((long)(*xndx++ * xbmul)%len) * nargs :
                                ((long) *xndx++ %len) * nargs;
         for (j=0; j < nargs; j++) {
@@ -184,7 +197,7 @@ static int mtab_i(CSOUND *csound,MTABI *p)
     long indx;
     MYFLT *table, **out = p->outargs;
     if ((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL) {
-      return csound->InitError(csound, Str("mtabi: incorrect table number"));
+      return csound->InitError(csound, Str("vtabi: incorrect table number"));
     }
     table = ftp->ftable;
     nargs = p->INOCOUNT-2;
@@ -199,7 +212,7 @@ static int mtab_set(CSOUND *csound,MTAB *p)     /* mtab by G.Maldonado */
 {
     FUNC *ftp;
     if ((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL) {
-      return csound->InitError(csound, Str("mtable: incorrect table number"));
+      return csound->InitError(csound, Str("vtab: incorrect table number"));
     }
     p->ftable = ftp->ftable;
     p->nargs = p->INOCOUNT-2;
@@ -226,14 +239,24 @@ static int mtab_k(CSOUND *csound,MTAB *p)
 static int mtab_a(CSOUND *csound,MTAB *p)
 {
     int j, nargs = p->nargs;
-    int nsmps = csound->ksmps, k;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t k, nsmps = CS_KSMPS;
     MYFLT **out = p->outargs;
     MYFLT *table;
     MYFLT *xndx = p->xndx;
     long len;
     table = p->ftable;
     len = p->len;
-    for (k=0;k<nsmps;k++) {
+    if (UNLIKELY(offset))
+      for (j=0; j < nargs; j++)
+        memset(out[j], '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      for (j=0; j < nargs; j++)
+        memset(&out[j][nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (k=offset;k<nsmps;k++) {
       long indx = ((long) *xndx++ %len) * nargs;
       for (j=0; j < nargs; j++) {
         out[j][k] =  table[indx + j];
@@ -251,7 +274,7 @@ static int mtablew_i(CSOUND *csound,MTABLEIW *p)
     long indx;
     MYFLT *table, xbmul = FL(0.0), **in = p->inargs;
     if ((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL) {
-      return csound->InitError(csound, Str("mtablewi: incorrect table number"));
+      return csound->InitError(csound, Str("vtablewi: incorrect table number"));
     }
     table = ftp->ftable;
     nargs = p->INOCOUNT-3;
@@ -267,7 +290,7 @@ static int mtablew_set(CSOUND *csound,MTABLEW *p)   /* mtabw by G.Maldonado */
 {
     FUNC *ftp;
     if ((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL) {
-      return csound->InitError(csound, Str("mtabw: incorrect table number"));
+      return csound->InitError(csound, Str("vtablew: incorrect table number"));
     }
     p->ftable = ftp->ftable;
     p->nargs = p->INOCOUNT-3;
@@ -287,7 +310,8 @@ static int mtablew_k(CSOUND *csound,MTABLEW *p)
     if (p->pfn != (long)*p->xfn) {
       FUNC *ftp;
       if ( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL) {
-        return csound->PerfError(csound, Str("mtabw: incorrect table number"));
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("vtablewk: incorrect table number"));
       }
       p->pfn = (long)*p->xfn;
       p->ftable = ftp->ftable;
@@ -307,7 +331,10 @@ static int mtablew_k(CSOUND *csound,MTABLEW *p)
 static int mtablew_a(CSOUND *csound,MTABLEW *p)
 {
     int j, nargs = p->nargs;
-    int nsmps = csound->ksmps, ixmode = (int) *p->ixmode, k=0;
+    int ixmode = (int) *p->ixmode;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t k, nsmps = CS_KSMPS;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
     MYFLT **in = p->inargs;
     MYFLT *table;
     MYFLT *xndx = p->xndx, xbmul;
@@ -316,7 +343,8 @@ static int mtablew_a(CSOUND *csound,MTABLEW *p)
     if (p->pfn != (long)*p->xfn) {
       FUNC *ftp;
       if ( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL) {
-        return csound->PerfError(csound, Str("mtabw: incorrect table number"));
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("vtablewa: incorrect table number"));
       }
       p->pfn = (long)*p->xfn;
       p->ftable = ftp->ftable;
@@ -327,14 +355,14 @@ static int mtablew_a(CSOUND *csound,MTABLEW *p)
     table = p->ftable;
     len = p->len;
     xbmul = p->xbmul;
-    do {
+    if (UNLIKELY(early)) nsmps -= early;
+    for (k=offset; k<nsmps; k++) {
       long indx = (ixmode) ? ((long)(*xndx++ * xbmul)%len) * nargs :
                              ((long) *xndx++ %len) * nargs;
       for (j=0; j < nargs; j++) {
         table[indx + j] = in[j][k];
       }
-      k++;
-    } while(--nsmps);
+    }
     return OK;
 }
 
@@ -347,7 +375,7 @@ static int mtabw_i(CSOUND *csound, MTABIW *p)
     long indx;
     MYFLT *table, **in = p->inargs;
     if ((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL) {
-      return csound->InitError(csound, Str("mtabwi: incorrect table number"));
+      return csound->InitError(csound, Str("vtabwi: incorrect table number"));
     }
     table = ftp->ftable;
     nargs = p->INOCOUNT-2;
@@ -361,7 +389,7 @@ static int mtabw_set(CSOUND *csound,MTABW *p)   /* mtabw by G.Maldonado */
 {
     FUNC *ftp;
     if (UNLIKELY((ftp = csound->FTnp2Find(csound, p->xfn)) == NULL)) {
-      return csound->InitError(csound, Str("mtabw: incorrect table number"));
+      return csound->InitError(csound, Str("vtablew: incorrect table number"));
     }
     p->ftable = ftp->ftable;
     p->nargs = p->INOCOUNT-2;
@@ -379,7 +407,8 @@ static int mtabw_k(CSOUND *csound,MTABW *p)
     if (p->pfn != (long)*p->xfn) {
       FUNC *ftp;
       if ( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL) {
-        return csound->PerfError(csound, Str("mtablew: incorrect table number"));
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("vtablewk: incorrect table number"));
       }
       p->pfn = (long)*p->xfn;
       p->ftable = ftp->ftable;
@@ -396,7 +425,9 @@ static int mtabw_k(CSOUND *csound,MTABW *p)
 static int mtabw_a(CSOUND *csound,MTABW *p)
 {
     int j, nargs = p->nargs;
-    int nsmps = csound->ksmps, k=0;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t k, nsmps = CS_KSMPS;
     MYFLT **in = p->inargs;
     MYFLT *table;
     MYFLT *xndx = p->xndx;
@@ -405,7 +436,8 @@ static int mtabw_a(CSOUND *csound,MTABW *p)
     if (p->pfn != (long)*p->xfn) {
       FUNC *ftp;
       if ( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL) {
-        return csound->PerfError(csound, Str("mtabw: incorrect table number"));
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("vtabwa: incorrect table number"));
       }
       p->pfn = (long)*p->xfn;
       p->ftable = ftp->ftable;
@@ -413,13 +445,13 @@ static int mtabw_a(CSOUND *csound,MTABW *p)
     }
     table = p->ftable;
     len = p->len;
-    do {
+    if (UNLIKELY(early)) nsmps -= early;
+    for (k=offset; k<nsmps; k++) {
       long indx = ((long) *xndx++ %len) * nargs;
       for (j=0; j < nargs; j++) {
         table[indx + j] = in[j][k];
       }
-      k++;
-    } while(--nsmps);
+    }
     return OK;
 }
 
@@ -1694,7 +1726,7 @@ static int vlimit_set(CSOUND *csound,VLIMIT *p)
       p->vector = ftp->ftable;
       p->elements = (int) *p->ielements;
     }
-    if (UNLIKELY(p->elements > ftp->flen )) {
+    if (UNLIKELY(p->elements > (int)ftp->flen )) {
       return csound->InitError(csound, Str("vectorop: invalid num of elements"));
     }
     return OK;
@@ -1717,12 +1749,12 @@ static int vport_set(CSOUND *csound,VPORT *p)
 {
     FUNC        *ftp;
     int elements;
-    MYFLT *vector, *yt1,*vecInit  = NULL;
+    MYFLT /* *vector,*/ *yt1,*vecInit  = NULL;
 
     if (LIKELY((ftp = csound->FTnp2Find(csound,p->ifn)) != NULL)) {
-      vector = (p->vector = ftp->ftable);
+      p->vector = ftp->ftable;
       elements = (p->elements = (int) *p->ielements);
-      if (UNLIKELY(elements > ftp->flen) )
+      if (UNLIKELY(elements > (int)ftp->flen) )
         return csound->InitError(csound,
                                  Str("vport: invalid table length or "
                                      "num of elements"));
@@ -1731,7 +1763,7 @@ static int vport_set(CSOUND *csound,VPORT *p)
     if (LIKELY(*p->ifnInit)) {
       if (LIKELY((ftp = csound->FTnp2Find(csound,p->ifnInit)) != NULL)) {
         vecInit = ftp->ftable;
-        if (UNLIKELY(elements > ftp->flen) )
+        if (UNLIKELY(elements > (int)ftp->flen) )
           return csound->InitError(csound, Str("vport: invalid init table length"
                                                " or num of elements"));
       }
@@ -1759,7 +1791,7 @@ static int vport(CSOUND *csound,VPORT *p)
     int elements = p->elements;
     MYFLT *vector = p->vector, *yt1 = p->yt1, c1, c2;
     if (p->prvhtim != *p->khtim) {
-      p->c2 = (MYFLT)pow(0.5, (double)csound->onedkr / *p->khtim);
+      p->c2 = (MYFLT)pow(0.5, (double)CS_ONEDKR / *p->khtim);
       p->c1 = FL(1.0) - p->c2;
       p->prvhtim = *p->khtim;
     }
@@ -1775,7 +1807,7 @@ static int vport(CSOUND *csound,VPORT *p)
 /*-------------------------------*/
 static int vwrap(CSOUND *csound,VLIMIT *p)
 {
-    int elements = p->elements; 
+    int elements = p->elements;
     MYFLT *vector = p->vector;
     MYFLT min = *p->kmin, max = *p->kmax;
 
@@ -1886,12 +1918,12 @@ static int vrandh_set(CSOUND *csound,VRANDH *p)
         p->offset = (int) *p->idstoffset;
       }
       else return csound->InitError(csound, Str("vrandh: Invalid table."));
-      if (UNLIKELY(*p->idstoffset >= ftp->flen))
+      if (UNLIKELY(*p->idstoffset >= (int)ftp->flen))
         return csound->InitError(csound,
                                  Str("vrandh: idstoffset is greater than"
                                      " table length."));
       p->vector = ftp->ftable + p->offset;
-      if (UNLIKELY(p->elements + p->offset > ftp->flen)) {
+      if (UNLIKELY(p->elements + p->offset > (int)ftp->flen)) {
         csound->Warning(csound,
                         Str("randh: Table length exceeded, "
                             "last elements discarded."));
@@ -1930,7 +1962,7 @@ static int vrandh(CSOUND *csound,VRANDH *p)
       *vector++ = (*num1++ * value) + *p->ioffset;
     } while (--elements);
 
-    p->phs += (int32)(*p->kcps * csound->kicvt);
+    p->phs += (int32)(*p->kcps * CS_KICVT);
     if (p->phs >= MAXLEN) {
       p->phs &= PHMASK;
       elements = p->elements;
@@ -1985,12 +2017,12 @@ static int vrandi_set(CSOUND *csound,VRANDI *p)
         p->offset = (int) *p->idstoffset;
       }
       else return csound->InitError(csound, Str("vrandi: Invalid table."));
-      if (UNLIKELY(p->offset >= ftp->flen))
+      if (UNLIKELY(p->offset >= (int)ftp->flen))
         return csound->InitError(csound,
                                  Str("vrandi: idstoffset is greater than"
                                      "table length."));
       p->vector = ftp->ftable + p->offset;
-      if (UNLIKELY(p->elements > ftp->flen)) {
+      if (UNLIKELY(p->elements > (int)ftp->flen)) {
         csound->Warning(csound,
                         Str("vrandi: Table length exceeded, "
                             "last elements discarded."));
@@ -2035,7 +2067,7 @@ static int vrandi(CSOUND *csound,VRANDI *p)
         *p->ioffset;
     } while (--elements);
 
-    p->phs += (int32)(*p->kcps * csound->kicvt);
+    p->phs += (int32)(*p->kcps * CS_KICVT);
     if (p->phs >= MAXLEN) {
       p->phs &= PHMASK;
       elements = p->elements;
@@ -2072,32 +2104,32 @@ static int vecdly_set(CSOUND *csound, VECDEL *p)
     if (LIKELY((ftp = csound->FTnp2Find(csound,p->ifnOut)) != NULL)) {
       p->outvec = ftp->ftable;
       elements = (p->elements = (int) *p->ielements);
-      if (UNLIKELY( elements > ftp->flen ))
+      if (UNLIKELY( elements > (int)ftp->flen ))
         return csound->InitError(csound,
                                  Str("vecdelay: invalid num of elements"));
     }
     else return csound->InitError(csound, Str("vecdly: invalid output table"));
     if (LIKELY((ftp = csound->FTnp2Find(csound,p->ifnIn)) != NULL)) {
       p->invec = ftp->ftable;
-      if (UNLIKELY(elements > ftp->flen))
+      if (UNLIKELY(elements > (int)ftp->flen))
         return csound->InitError(csound,
                                  Str("vecdelay: invalid num of elements"));
     }
     else return csound->InitError(csound, Str("vecdly: invalid input table"));
     if (LIKELY((ftp = csound->FTnp2Find(csound,p->ifnDel)) != NULL)) {
       p->dlyvec = ftp->ftable;
-      if (UNLIKELY( elements > ftp->flen ))
+      if (UNLIKELY( elements > (int)ftp->flen ))
         return csound->InitError(csound,
                                  Str("vecdelay: invalid num of elements"));
     }
     else return csound->InitError(csound, Str("vecdly: invalid delay table"));
 
-    n = (p->maxd = (int32) (*p->imaxd * csound->ekr));
+    n = (p->maxd = (int32) (*p->imaxd * CS_EKR));
     if (n == 0) n = (p->maxd = 1);
 
     if (!*p->istod) {
       if (p->aux.auxp == NULL ||
-          (int)(elements * sizeof(MYFLT *)
+          (unsigned int)(elements * sizeof(MYFLT *)
                 + n * elements * sizeof(MYFLT)
                 + elements * sizeof(int32)) > p->aux.size) {
         csound->AuxAlloc(csound, elements * sizeof(MYFLT *)
@@ -2139,7 +2171,7 @@ static int vecdly(CSOUND *csound,VECDEL *p)
     }
     do {
       (*buf)[*indx] = *inVec++;
-      fv1 = *indx - *dlyVec++ * csound->ekr;
+      fv1 = *indx - *dlyVec++ * CS_EKR;
       while (fv1 < FL(0.0))     fv1 += (MYFLT)maxd;
       while (fv1 >= (MYFLT)maxd) fv1 -= (MYFLT)maxd;
       if (fv1 < maxd - 1) fv2 = fv1 + 1;
@@ -2176,7 +2208,7 @@ static int vseg_set(CSOUND *csound,VSEG *p)
       p->vector = ftp->ftable;
       p->elements = (int) *p->ielements;
     }
-    if (UNLIKELY( p->elements > ftp->flen ))
+    if (UNLIKELY( p->elements > (int)ftp->flen ))
       return csound->InitError(csound,
                                Str("vlinseg/vexpseg: invalid num. of elements"));
 
@@ -2199,7 +2231,7 @@ static int vseg_set(CSOUND *csound,VSEG *p)
       if (UNLIKELY((nxtfunc = csound->FTnp2Find(csound,*argp++)) == NULL))
         return NOTOK;
       if (dur > FL(0.0)) {
-        segp->d = dur * csound->ekr;
+        segp->d = dur * CS_EKR;
         segp->function =  curfunc;
         segp->nxtfunction = nxtfunc;
         segp->cnt = (int32) MYFLT2LRND(segp->d);
@@ -2296,7 +2328,7 @@ static int vphaseseg_set(CSOUND *csound,VPSEG *p)
       p->vector = ftp->ftable;
       p->elements = (int) *p->ielements;
     }
-    if ( p->elements > ftp->flen )
+    if ( p->elements > (int)ftp->flen )
       return csound->InitError(csound,
                                Str("vphaseseg: invalid num. of elements"));
     vector = p->vector;
@@ -2317,7 +2349,7 @@ static int vphaseseg_set(CSOUND *csound,VPSEG *p)
       if ((nxtfunc = csound->FTnp2Find(csound,*argp++)) == NULL) return NOTOK;
       if (dur > FL(0.0)) {
         durtot+=dur;
-        segp->d = dur; /* * csound->ekr; */
+        segp->d = dur; /* * CS_EKR; */
         segp->function = curfunc;
         segp->nxtfunction = nxtfunc;
         /* segp->cnt = (int32) (segp->d + .5);  */
@@ -2383,11 +2415,11 @@ static int vphaseseg(CSOUND *csound,VPSEG *p)
 static int kdel_set(CSOUND *csound,KDEL *p)
 {
     uint32 n;
-    n = (p->maxd = (int32) (*p->imaxd * csound->ekr));
+    n = (p->maxd = (int32) (*p->imaxd * CS_EKR));
     if (n == 0) n = (p->maxd = 1);
 
     if (!*p->istod) {
-      if (p->aux.auxp == NULL || (int)(n*sizeof(MYFLT)) > p->aux.size)
+      if (p->aux.auxp == NULL || (unsigned int)(n*sizeof(MYFLT)) > p->aux.size)
         csound->AuxAlloc(csound, n * sizeof(MYFLT), &p->aux);
       else {
         memset(p->aux.auxp, 0, sizeof(MYFLT)*n);
@@ -2408,7 +2440,7 @@ static int kdelay(CSOUND *csound,KDEL *p)
 
     indx = p->left;
     buf[indx] = *p->kin;
-    fv1 = indx - *p->kdel * csound->ekr;
+    fv1 = indx - *p->kdel * CS_EKR;
     while (fv1 < FL(0.0))       fv1 += (MYFLT)maxd;
     while (fv1 >= (MYFLT)maxd) fv1 -= (MYFLT)maxd;
     if (*p->interp) { /*  no interpolation */
@@ -2435,13 +2467,13 @@ static int ca_set(CSOUND *csound,CELLA *p)
     if (LIKELY((ftp = csound->FTnp2Find(csound,p->ioutFunc)) != NULL)) {
       p->outVec = ftp->ftable;
       elements = (p->elements = (int) *p->ielements);
-      if (UNLIKELY( elements > ftp->flen ))
+      if (UNLIKELY( elements > (int)ftp->flen ))
         return csound->InitError(csound, Str("cella: invalid num of elements"));
     }
     else return csound->InitError(csound, Str("cella: invalid output table"));
     if (LIKELY((ftp = csound->FTnp2Find(csound,p->initStateFunc)) != NULL)) {
       initVec = (p->initVec = ftp->ftable);
-      if (UNLIKELY(elements > ftp->flen ))
+      if (UNLIKELY(elements > (int)ftp->flen ))
         return csound->InitError(csound, Str("cella: invalid num of elements"));
     }
     else return csound->InitError(csound,
@@ -2514,53 +2546,73 @@ static int ca(CSOUND *csound,CELLA *p)
 #define S(x)    sizeof(x)
 
 OENTRY vectorial_localops[] = {
-  { "vtablei", S(MTABLEI),   TR|1, "",   "iiiim", (SUBR)mtable_i,  NULL },
-  { "vtablek", S(MTABLE),    TR|3, "",   "kkkiz", (SUBR)mtable_set, (SUBR)mtable_k, NULL },
-  { "vtablea", S(MTABLE),    TR|5, "",   "akkiy", (SUBR)mtable_set, NULL, (SUBR)mtable_a },
-  { "vtablewi", S(MTABLEIW), TB|1, "",   "iiim", (SUBR)mtablew_i,  NULL },
-  { "vtablewk", S(MTABLEW),  TB|3, "",   "kkiz", (SUBR)mtablew_set, (SUBR)mtablew_k, NULL },
-  { "vtablewa", S(MTABLEW),  TB|5, "",   "akiy", (SUBR)mtablew_set, NULL, (SUBR)mtablew_a },
-  { "vtabi", S(MTABI),       TR|1, "",   "iim", (SUBR)mtab_i,  NULL },
-  { "vtabk", S(MTAB),        TR|3, "",   "kiz", (SUBR)mtab_set, (SUBR)mtab_k, NULL },
-  { "vtaba", S(MTAB),        TR|5, "",  "aiy", (SUBR)mtab_set, NULL, (SUBR)mtab_a },
-  { "vtabwi", S(MTABIW),     TB|1, "",  "iim", (SUBR)mtabw_i,  NULL },
-  { "vtabwk", S(MTABW),      TB|3, "",  "kiz", (SUBR)mtabw_set, (SUBR)mtabw_k, NULL },
-  { "vtabwa", S(MTABW),      TB|5, "",  "aiy", (SUBR)mtabw_set, NULL, (SUBR)mtabw_a },
+  { "vtablei", S(MTABLEI),   TR, 1, "",   "iiiim", (SUBR)mtable_i,  NULL },
+  { "vtablek", S(MTABLE),    TR, 3, "",   "kkkiz",
+                                  (SUBR)mtable_set, (SUBR)mtable_k, NULL },
+  { "vtablea", S(MTABLE),    TR, 5, "",   "akkiy",
+                                  (SUBR)mtable_set, NULL, (SUBR)mtable_a },
+  { "vtablewi", S(MTABLEIW), TB, 1, "",   "iiim", (SUBR)mtablew_i,  NULL },
+  { "vtablewk", S(MTABLEW),  TB, 3, "",   "kkiz",
+                                (SUBR)mtablew_set, (SUBR)mtablew_k, NULL },
+  { "vtablewa", S(MTABLEW),  TB, 5, "",   "akiy",
+                                (SUBR)mtablew_set, NULL, (SUBR)mtablew_a },
+  { "vtabi", S(MTABI),       TR, 1, "",   "iim", (SUBR)mtab_i,  NULL },
+  { "vtabk", S(MTAB),        TR, 3, "",   "kiz",
+                                      (SUBR)mtab_set, (SUBR)mtab_k, NULL },
+  { "vtaba", S(MTAB),        TR, 5, "",  "aiy",
+                                      (SUBR)mtab_set, NULL, (SUBR)mtab_a },
+  { "vtabwi", S(MTABIW),     TB, 1, "",  "iim", (SUBR)mtabw_i,  NULL },
+  { "vtabwk", S(MTABW),      TB, 3, "",  "kiz",
+                                       (SUBR)mtabw_set, (SUBR)mtabw_k, NULL },
+  { "vtabwa", S(MTABW),      TB, 5, "",  "aiy",
+                                       (SUBR)mtabw_set, NULL, (SUBR)mtabw_a },
 
-  { "vadd",   S(VECTOROP),   TB|3, "",  "ikkOO", (SUBR)vectorOp_set, (SUBR) vaddk },
-  { "vadd_i", S(VECTOROPI),  TB|1, "",  "iiio", (SUBR) vadd_i, NULL, NULL        },
-  { "vmult",  S(VECTOROP),   TB|3, "",  "ikkOO", (SUBR)vectorOp_set, (SUBR) vmultk},
-  { "vmult_i", S(VECTOROPI), TB|1, "",  "iiio", (SUBR) vmult_i, NULL, NULL       },
-  { "vpow",   S(VECTOROP),   TB|3, "",  "ikkOO", (SUBR)vectorOp_set, (SUBR) vpowk },
-  { "vpow_i", S(VECTOROPI),  TB|1, "",  "iiio", (SUBR) vpow_i, NULL, NULL        },
-  { "vexp",   S(VECTOROP),   TB|3, "",  "ikkOO", (SUBR)vectorOp_set, (SUBR) vexpk },
-  { "vexp_i", S(VECTOROPI),  TB|1, "",  "iiio", (SUBR) vexp_i, NULL, NULL        },
-  { "vaddv",  S(VECTORSOP),  TB|3, "",  "iikOOO", (SUBR)vectorsOp_set, (SUBR) vaddvk },
-  { "vaddv_i",  S(VECTORSOPI), TB|1, "",  "iiioo", (SUBR)vaddv_i, NULL, NULL        },
-  { "vsubv",  S(VECTORSOP),  TB|3, "",  "iikOOO", (SUBR)vectorsOp_set, (SUBR) vsubvk },
-  { "vsubv_i",  S(VECTORSOPI),  TB|1, "",  "iiioo", (SUBR)vsubv_i, NULL, NULL        },
-  { "vmultv", S(VECTORSOP),  TB|3, "",  "iikOOO", (SUBR)vectorsOp_set, (SUBR) vmultvk},
-  { "vmultv_i", S(VECTORSOPI),  TB|1, "",  "iiioo", (SUBR)vmultv_i, NULL, NULL       },
-  { "vdivv",  S(VECTORSOP), TB| 3, "",  "iikOOO", (SUBR)vectorsOp_set, (SUBR) vdivvk },
-  { "vdivv_i",  S(VECTORSOPI),  TB|1, "",  "iiioo", (SUBR)vdivv_i, NULL, NULL        },
-  { "vpowv",  S(VECTORSOP),  TB|3, "",  "iikOOO", (SUBR)vectorsOp_set, (SUBR) vpowvk },
-  { "vpowv_i",  S(VECTORSOPI),  TB|1, "",  "iiioo", (SUBR)vpowv_i, NULL, NULL      },
-  { "vexpv",  S(VECTORSOP),  TB|3, "",  "iikOOO", (SUBR)vectorsOp_set, (SUBR) vexpvk },
-  { "vexpv_i",  S(VECTORSOPI),  TB|1, "",  "iiioo", (SUBR)vexpv_i, NULL, NULL      },
-  { "vcopy",  S(VECTORSOP),  TB|3, "",  "iikOOO", (SUBR)vectorsOp_set, (SUBR) vcopy },
-  { "vcopy_i", S(VECTORSOP), TB|1, "",  "iiioo", (SUBR) vcopy_i, NULL, NULL   },
-  { "vmap",   S(VECTORSOPI), TB|1, "",  "iiioo", (SUBR)vmap_i, NULL, NULL     },
-  { "vlimit", S(VLIMIT),  TR|3, "",  "ikki",(SUBR)vlimit_set, (SUBR)vlimit    },
-  { "vwrap",  S(VLIMIT),  TB|3, "",  "ikki",(SUBR)vlimit_set, (SUBR) vwrap    },
-  { "vmirror", S(VLIMIT),    3, "",  "ikki",(SUBR)vlimit_set, (SUBR)vmirror   },
-  { "vlinseg", S(VSEG),   TB|3, "",  "iin", (SUBR)vseg_set,   (SUBR)vlinseg   },
-  { "vexpseg", S(VSEG),      3, "",  "iin", (SUBR)vseg_set,   (SUBR)vexpseg   },
-  { "vrandh", S(VRANDH),  TB|3, "",  "ikkiovoo",(SUBR)vrandh_set, (SUBR) vrandh},
-  { "vrandi", S(VRANDI),  TB|3, "",  "ikkiovoo",(SUBR)vrandi_set, (SUBR)vrandi },
-  { "vport",  S(VPORT),   TB|3, "",  "ikio",(SUBR)vport_set,  (SUBR)vport     },
-  { "vecdelay", S(VECDEL),   3, "",  "iiiiio",(SUBR)vecdly_set, (SUBR)vecdly  },
-  { "vdelayk", S(KDEL),      3, "k", "kkioo",(SUBR)kdel_set,  (SUBR)kdelay    },
-  { "vcella", S(CELLA),      TR|3, "",  "kkiiiiip",(SUBR)ca_set, (SUBR)ca     }
+  { "vadd",   S(VECTOROP),   TB, 3, "",  "ikkOO",
+                                           (SUBR)vectorOp_set, (SUBR) vaddk },
+  { "vadd_i", S(VECTOROPI),  TB, 1, "",  "iiio",  (SUBR) vadd_i, NULL, NULL },
+  { "vmult",  S(VECTOROP),   TB, 3, "",  "ikkOO",
+                                    (SUBR)vectorOp_set, (SUBR) vmultk},
+  { "vmult_i", S(VECTOROPI), TB, 1, "",  "iiio", (SUBR) vmult_i, NULL, NULL },
+  { "vpow",   S(VECTOROP),   TB, 3, "",  "ikkOO",
+                                           (SUBR)vectorOp_set, (SUBR) vpowk },
+  { "vpow_i", S(VECTOROPI),  TB, 1, "",  "iiio", (SUBR) vpow_i, NULL, NULL  },
+  { "vexp",   S(VECTOROP),   TB, 3, "",  "ikkOO",
+                                           (SUBR)vectorOp_set, (SUBR) vexpk },
+  { "vexp_i", S(VECTOROPI),  TB, 1, "",  "iiio", (SUBR) vexp_i, NULL, NULL  },
+  { "vaddv",  S(VECTORSOP),  TB, 3, "",  "iikOOO",
+                                         (SUBR)vectorsOp_set, (SUBR) vaddvk },
+  { "vaddv_i",  S(VECTORSOPI), TB, 1, "",  "iiioo", (SUBR)vaddv_i, NULL, NULL },
+  { "vsubv",  S(VECTORSOP),  TB, 3, "",  "iikOOO",
+                                         (SUBR)vectorsOp_set, (SUBR) vsubvk },
+  { "vsubv_i",  S(VECTORSOPI),  TB, 1, "",  "iiioo",
+                                           (SUBR)vsubv_i, NULL, NULL        },
+  { "vmultv", S(VECTORSOP),  TB, 3, "",  "iikOOO",
+                                         (SUBR)vectorsOp_set, (SUBR) vmultvk},
+  { "vmultv_i", S(VECTORSOPI),  TB, 1, "",  "iiioo", (SUBR)vmultv_i, NULL, NULL },
+  { "vdivv",  S(VECTORSOP), TB,  3, "",  "iikOOO",
+                                         (SUBR)vectorsOp_set, (SUBR) vdivvk },
+  { "vdivv_i",  S(VECTORSOPI),  TB, 1, "",  "iiioo", (SUBR)vdivv_i, NULL, NULL },
+  { "vpowv",  S(VECTORSOP),  TB, 3, "",  "iikOOO",
+                                         (SUBR)vectorsOp_set, (SUBR) vpowvk },
+  { "vpowv_i",  S(VECTORSOPI),  TB, 1, "",  "iiioo", (SUBR)vpowv_i, NULL, NULL },
+  { "vexpv",  S(VECTORSOP),  TB, 3, "",  "iikOOO",
+                                         (SUBR)vectorsOp_set, (SUBR) vexpvk },
+  { "vexpv_i",  S(VECTORSOPI),  TB, 1, "",  "iiioo", (SUBR)vexpv_i, NULL, NULL },
+  { "vcopy",  S(VECTORSOP),  TB, 3, "",  "iikOOO",
+                                          (SUBR)vectorsOp_set, (SUBR) vcopy },
+  { "vcopy_i", S(VECTORSOP), TB, 1, "",  "iiioo", (SUBR) vcopy_i, NULL, NULL},
+  { "vmap",   S(VECTORSOPI), TB, 1, "",  "iiioo", (SUBR)vmap_i, NULL, NULL  },
+  { "vlimit", S(VLIMIT),  TR, 3, "",  "ikki",(SUBR)vlimit_set, (SUBR)vlimit },
+  { "vwrap",  S(VLIMIT),  TB, 3, "",  "ikki",(SUBR)vlimit_set, (SUBR) vwrap },
+  { "vmirror", S(VLIMIT),  0,   3, "",  "ikki",(SUBR)vlimit_set, (SUBR)vmirror },
+  { "vlinseg", S(VSEG),   TB, 3, "",  "iin", (SUBR)vseg_set,   (SUBR)vlinseg },
+  { "vexpseg", S(VSEG),    0,   3, "",  "iin", (SUBR)vseg_set, (SUBR)vexpseg },
+  { "vrandh", S(VRANDH),  TB, 3, "",  "ikkiovoo",(SUBR)vrandh_set, (SUBR)vrandh},
+  { "vrandi", S(VRANDI),  TB, 3, "",  "ikkiovoo",(SUBR)vrandi_set, (SUBR)vrandi },
+  { "vport",  S(VPORT),   TB, 3, "",  "ikio",(SUBR)vport_set,  (SUBR)vport   },
+  { "vecdelay", S(VECDEL),0,    3, "",  "iiiiio",(SUBR)vecdly_set, (SUBR)vecdly },
+  { "vdelayk", S(KDEL),    0,   3, "k", "kkioo",(SUBR)kdel_set,  (SUBR)kdelay },
+  { "vcella", S(CELLA),      TR, 3, "",  "kkiiiiip",(SUBR)ca_set, (SUBR)ca    }
 };
 
 int gab_vectorial_init_(CSOUND *csound)
@@ -2569,5 +2621,3 @@ int gab_vectorial_init_(CSOUND *csound)
       csound->AppendOpcodes(csound, &(vectorial_localops[0]),
                             (int) (sizeof(vectorial_localops) / sizeof(OENTRY)));
 }
-
-
