@@ -182,7 +182,10 @@ static int kgendy(CSOUND *csound, GENDY *p)
 
 static int agendy(CSOUND *csound, GENDY *p)
 {
-    int     knum, n, nn = csound->ksmps;
+    int     knum;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   *out, *memamp, *memdur, minfreq, maxfreq, dist;
     out  = p->out;
     knum = (int)*p->knum;
@@ -190,7 +193,12 @@ static int agendy(CSOUND *csound, GENDY *p)
     memdur  = p->memdur.auxp;
     minfreq = *p->minfreq;
     maxfreq = *p->maxfreq;
-    for (n=0; n<nn; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+     if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+   for (n=offset; n<nsmps; n++) {
       if (p->phase >= FL(1.0)) {
         int index = p->index;
         p->phase -= FL(1.0);
@@ -309,7 +317,10 @@ static int kgendyx(CSOUND *csound, GENDYX *p)
 
 static int agendyx(CSOUND *csound, GENDYX *p)
 {
-    int     knum, n, nn = csound->ksmps;
+    int     knum;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT   *out, *memamp, *memdur, minfreq, maxfreq, dist, curve;
     out  = p->out;
     knum = (int)*p->knum;
@@ -317,7 +328,12 @@ static int agendyx(CSOUND *csound, GENDYX *p)
     memdur  = p->memdur.auxp;
     minfreq = *p->minfreq;
     maxfreq = *p->maxfreq;
-    for (n=0; n<nn; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       if (p->phase >= FL(1.0)) {
         int index = p->index;
         p->phase -= FL(1.0);
@@ -432,7 +448,7 @@ static int kgendyc(CSOUND *csound, GENDYC *p)
       memdur[index] = p->dur;
       fphase = (minfreq + (maxfreq - minfreq) * p->dur) * knum;
       fphase = (fphase > FL(0.001) ? fphase : FL(0.001));
-      p->phase = (int32)(csound->esr / fphase);
+      p->phase = (int32)(csound->GetSr(csound) / fphase);
       if (p->phase < 2) p->phase = 2;
       p->curve = FL(2.0) * (next_midpnt - p->midpnt - p->phase * p->slope);
       p->curve = p->curve / (p->phase * p->phase + p->phase);
@@ -447,7 +463,10 @@ static int kgendyc(CSOUND *csound, GENDYC *p)
 
 static int agendyc(CSOUND *csound, GENDYC *p)
 {
-    int     knum, remain = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    int     knum;
+    int     remain = CS_KSMPS-offset-early;
     MYFLT   *out, *memamp, *memdur, minfreq, maxfreq, dist;
     out  = p->out;
     knum = (int)*p->knum;
@@ -455,8 +474,12 @@ static int agendyc(CSOUND *csound, GENDYC *p)
     memdur  = p->memdur.auxp;
     minfreq = *p->minfreq;
     maxfreq = *p->maxfreq;
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      memset(&out[remain+offset], '\0', early*sizeof(MYFLT));
+    }
     do {
-      int nsmps, n;
+      uint32_t n, nsmps = CS_KSMPS;
       if (p->phase <= 0) {
         int     index = p->index;
         MYFLT   fphase, next_midpnt;
@@ -488,7 +511,7 @@ static int agendyc(CSOUND *csound, GENDYC *p)
         memdur[index] = p->dur;
         fphase = (minfreq + (maxfreq - minfreq) * p->dur) * knum;
         fphase = (fphase > FL(0.001) ? fphase : FL(0.001));
-        p->phase = (int32)(csound->esr / fphase);
+        p->phase = (int32)(csound->GetSr(csound) / fphase);
         if (p->phase < 2) p->phase = 2;
         p->curve = FL(2.0) * (next_midpnt - p->midpnt - p->phase * p->slope);
         p->curve = p->curve / (p->phase * p->phase + p->phase);
@@ -496,7 +519,7 @@ static int agendyc(CSOUND *csound, GENDYC *p)
       nsmps = (remain < p->phase ? remain : p->phase);
       remain   -= nsmps;
       p->phase -= nsmps;
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         *(out++) = *p->kamp * p->midpnt;
         p->slope  += p->curve;
         p->midpnt += p->slope;
@@ -509,18 +532,18 @@ static OENTRY gendy_localops[] = {
   { "gendy",    0xffff                                   },
   { "gendyx",   0xffff                                   },
   { "gendyc",   0xffff                                   },
-  { "gendy.k",  sizeof(GENDY),  3, "k", "kkkkkkkkkoO",
+  { "gendy.k",  sizeof(GENDY),  0,3, "k", "kkkkkkkkkoO",
     (SUBR)gendyset,  (SUBR)kgendy,  (SUBR)NULL           },
-  { "gendy.a",  sizeof(GENDY),  5, "a", "kkkkkkkkkoO",
+  { "gendy.a",  sizeof(GENDY),  0,5, "a", "kkkkkkkkkoO",
     (SUBR)gendyset,  (SUBR)NULL,    (SUBR)agendy         },
-  { "gendyx.k", sizeof(GENDYX), 3, "k", "kkkkkkkkkkkoO",
+  { "gendyx.k", sizeof(GENDYX), 0,3, "k", "kkkkkkkkkkkoO",
     (SUBR)gendyxset, (SUBR)kgendyx, (SUBR)NULL           },
-  { "gendyx.a", sizeof(GENDYX), 5, "a", "kkkkkkkkkkkoO",
+  { "gendyx.a", sizeof(GENDYX), 0,5, "a", "kkkkkkkkkkkoO",
     (SUBR)gendyxset, (SUBR)NULL,    (SUBR)agendyx        },
-  { "gendyc.k", sizeof(GENDYC), 3, "k", "kkkkkkkkkoO",
+  { "gendyc.k", sizeof(GENDYC), 0,3, "k", "kkkkkkkkkoO",
     (SUBR)gendycset, (SUBR)kgendyc, (SUBR)NULL           },
-  { "gendyc.a", sizeof(GENDYC), 5, "a", "kkkkkkkkkoO",
+  { "gendyc.a", sizeof(GENDYC), 0,5, "a", "kkkkkkkkkoO",
     (SUBR)gendycset, (SUBR)NULL,    (SUBR)agendyc        }
 };
 
-LINKAGE1(gendy_localops)
+LINKAGE_BUILTIN(gendy_localops)
