@@ -27,7 +27,7 @@
 
 /* Some speed hacks added by John Fitch */
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include "grain.h"
 
 static inline MYFLT Unirand(CSOUND *csound, MYFLT a)
@@ -58,14 +58,14 @@ static int agsset(CSOUND *csound, PGRA *p)  /*      Granular U.G. set-up    */
     else
       p->pr = FL(0.0);
 
-    bufsize = sizeof(MYFLT) * (2L * (size_t) (csound->esr * *p->imkglen)
-                               + (3L * csound->ksmps));
+    bufsize = sizeof(MYFLT) * (2L * (size_t) (CS_ESR * *p->imkglen)
+                               + (3L * CS_KSMPS));
 
-    if (p->aux.auxp == NULL || bufsize > p->aux.size)
+    if (p->aux.auxp == NULL || (unsigned int)bufsize > p->aux.size)
       csound->AuxAlloc(csound, bufsize, &p->aux);
     else memset(p->aux.auxp, '\0', bufsize); /* Clear any old data */
     d  = p->x = (MYFLT *)p->aux.auxp;
-    d +=  (int)(csound->esr * *p->imkglen) + csound->ksmps;
+    d +=  (int)(CS_ESR * *p->imkglen) + CS_KSMPS;
     p->y = d;
 
     p->ampadv = (XINARG1) ? 1 : 0;
@@ -80,16 +80,19 @@ static int ags(CSOUND *csound, PGRA *p) /*  Granular U.G. a-rate main routine */
     MYFLT       *buf, *out, *rem, *gtbl, *etbl;
     MYFLT       *xdns, *xamp, *xlfr, *temp, amp;
     int32       isc, isc2, inc, inc2, lb, lb2;
-    int32       n, i, bufsize;
+    int32       n, bufsize;
     int32       ekglen;
-    int         nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     MYFLT       kglen = *p->kglen;
     MYFLT       gcount = p->gcount;
 
                                 /* Pick up common values to locals for speed */
     if (UNLIKELY(p->aux.auxp==NULL)) goto err1;
     if (UNLIKELY(kglen<=FL(0.0)))
-      return csound->PerfError(csound, Str("grain: grain length zero"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("grain: grain length zero"));
     gtp  = p->gftp;
     gtbl = gtp->ftable;
 
@@ -105,16 +108,20 @@ static int ags(CSOUND *csound, PGRA *p) /*  Granular U.G. a-rate main routine */
 
     if (kglen > *p->imkglen) kglen = *p->imkglen;
 
-    ekglen  = (int32)(csound->esr * kglen);   /* Useful constant */
+    ekglen  = (int32)(CS_ESR * kglen);   /* Useful constant */
     inc2    = (int32)(csound->sicvt / kglen); /* Constant for each cycle */
-    bufsize = csound->ksmps + ekglen;
+    bufsize = CS_KSMPS + ekglen;
     xdns    = p->xdns;
     xamp    = p->xamp;
     xlfr    = p->xlfr;
 
     memset(buf, '\0', bufsize*sizeof(MYFLT));
-
-    for (i = 0 ; i < nsmps ; i++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i = offset ; i < nsmps ; i++) {
       if (gcount >= FL(1.0)) { /* I wonder..... */
         gcount = FL(0.0);
         amp = *xamp + Unirand(csound, *p->kabnd);
@@ -141,21 +148,22 @@ static int ags(CSOUND *csound, PGRA *p) /*  Granular U.G. a-rate main routine */
     n = bufsize;
     temp = rem;
     do {
-      *temp = *buf++ + *(temp + csound->ksmps);
+      *temp = *buf++ + *(temp + CS_KSMPS);
       temp++;
     } while (--n);
 
-    memcpy(out, rem, csound->ksmps*sizeof(MYFLT));
+    memcpy(&out[offset], rem, (nsmps-offset)*sizeof(MYFLT));
     p->gcount = gcount;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("grain: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("grain: not initialised"));
 }
 
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "grain", S(PGRA),  TR|5,   "a",    "xxxkkkiiio", (SUBR)agsset, NULL, (SUBR)ags }
+{ "grain", S(PGRA),  TR, 5,   "a",    "xxxkkkiiio", (SUBR)agsset, NULL, (SUBR)ags }
 };
 
 int grain_init_(CSOUND *csound)
