@@ -75,13 +75,16 @@
   row (l->r t->b) so a pixel value in imageData is located at: (w*y+x)*3.
 */
 
+/* #undef CS_KSMPS */
+/* #define CS_KSMPS     (csound->GetKsmps(csound)) */
+
 static Image * __doOpenImage(char * filename, CSOUND *csound)
 {
 #ifdef USE_LIBPNG
+#define HS (8)
     FILE *fp;
     void *fd;
-    const int hs = 8;
-    unsigned char header[8];
+    unsigned char header[HS];
     png_structp png_ptr;
     png_infop info_ptr;
     /* png_infop end_ptr; */
@@ -91,7 +94,7 @@ static Image * __doOpenImage(char * filename, CSOUND *csound)
     int color_type;
     unsigned char *image_data;
     png_bytepp row_pointers;
-    int i;
+    unsigned int i;
 
     Image *img;
 
@@ -103,11 +106,11 @@ static Image * __doOpenImage(char * filename, CSOUND *csound)
       return NULL;
     }
 
-    if (UNLIKELY(hs!=fread(header, 1, hs, fp)))
+    if (UNLIKELY(HS!=fread(header, 1, HS, fp)))
       csound->InitError(csound,
                         Str("imageload: file %s is not in PNG format.\n"),
                         filename);
-    is_png = !png_sig_cmp(header, 0, hs);
+    is_png = !png_sig_cmp(header, 0, HS);
 
     if (UNLIKELY(!is_png)) {
       csound->InitError(csound,
@@ -140,7 +143,7 @@ static Image * __doOpenImage(char * filename, CSOUND *csound)
     /* } */
 
     png_init_io(png_ptr, fp);
-    png_set_sig_bytes(png_ptr, hs);
+    png_set_sig_bytes(png_ptr, HS);
 
     png_read_info(png_ptr, info_ptr);
     {
@@ -415,7 +418,7 @@ static int imageload (CSOUND *csound, IMGLOAD * p)
       (Image **) csound->ReAlloc(csound, pimages->images,
                                  sizeof(Image *) * pimages->cnt);
 
-    strncpy(filename, (char*) p->ifilnam, 254);
+    strncpy(filename, (char*) (p->ifilnam->data), 254);
 
     img = __doOpenImage(filename, csound);
 
@@ -490,7 +493,9 @@ static int imagegetpixel_a (CSOUND *csound, IMGGETPIXEL * p)
     MYFLT *tx = p->kx;
     MYFLT *ty = p->ky;
 
-    int i, n=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     int w, h, x, y, pixel;
 
     pimages = (Images *) csound->QueryGlobalVariable(csound,
@@ -499,12 +504,24 @@ static int imagegetpixel_a (CSOUND *csound, IMGGETPIXEL * p)
     w = img->w;
     h = img->h;
 
-    for (i = 0; i < n; i++) {
+
+    if (UNLIKELY(offset)) {
+      memset(r, '\0', offset*sizeof(MYFLT));
+      memset(g, '\0', offset*sizeof(MYFLT));
+      memset(b, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&g[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&b[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i = 0; i < nsmps; i++) {
 
       x = tx[i]*w;
       y = ty[i]*h;
 
-      if (x >= 0 && x < w && y >= 0 && y < h ) {
+      if ( x >= 0 && x < w && y >= 0 && y < h ) {
         pixel = (w*y+x)*3;
         r[i] = img->imageData[pixel]/FL(255.0);
         g[i] = img->imageData[pixel+1]/FL(255.0);
@@ -532,7 +549,9 @@ static int imagesetpixel_a (CSOUND *csound, IMGSETPIXEL * p)
     MYFLT *tx = p->kx;
     MYFLT *ty = p->ky;
 
-    int i, n=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     int h,w,x, y, pixel;
 
     pimages = (Images *) csound->QueryGlobalVariable(csound,
@@ -542,7 +561,8 @@ static int imagesetpixel_a (CSOUND *csound, IMGSETPIXEL * p)
     w = img->w;
     h = img->h;
 
-    for (i = 0; i < n; i++) {
+    if (UNLIKELY(early)) nsmps -= early;
+    for (i = offset; i < nsmps; i++) {
 
       x = tx[i]*w;
       y = ty[i]*h;
@@ -590,7 +610,7 @@ static int imagesave (CSOUND *csound, IMGSAVE * p)
     Image *img;
     char filename[255];
 
-    strcpy(filename, (char*) p->ifilnam);
+    strcpy(filename, (char*) (p->ifilnam->data));
 
     pimages = (Images *) csound->QueryGlobalVariable(csound,
                                                      "imageOpcodes.images");
@@ -616,17 +636,16 @@ static int imagefree (CSOUND *csound, IMGSAVE * p)
 #define S(x)    sizeof(x)
 
 static OENTRY image_localops[] = {
-  { "imageload",  S(IMGLOAD),  1, "i", "S",   (SUBR)imageload, NULL, NULL   },
-  { "imagecreate",S(IMGCREATE),1, "i", "ii",  (SUBR)imagecreate, NULL, NULL },
-  { "imagesize",  S(IMGSIZE),  1, "ii", "i",  (SUBR)imagesize, NULL, NULL   },
-  { "imagegetpixel",  S(IMGGETPIXEL),  7, "sss", "ixx",
+  { "imageload",  S(IMGLOAD),  0, 1, "i", "S",   (SUBR)imageload, NULL, NULL   },
+  { "imagecreate",S(IMGCREATE),0, 1, "i", "ii",  (SUBR)imagecreate, NULL, NULL },
+  { "imagesize",  S(IMGSIZE),  0, 1, "ii", "i",  (SUBR)imagesize, NULL, NULL   },
+  { "imagegetpixel",  S(IMGGETPIXEL),  0, 7, "sss", "ixx",
     (SUBR)imagegetpixel, (SUBR)imagegetpixel, (SUBR)imagegetpixel_a   },
-  { "imagesetpixel",  S(IMGSETPIXEL),  7, "", "ixxxxx",
+  { "imagesetpixel",  S(IMGSETPIXEL),  0, 7, "", "ixxxxx",
     (SUBR)imagesetpixel, (SUBR)imagesetpixel, (SUBR)imagesetpixel_a   },
-  { "imagesave",  S(IMGSAVE),  1, "", "iS",   (SUBR)imagesave, NULL, NULL   },
-  { "imagefree",  S(IMGFREE),  1, "", "i",    (SUBR)imagefree, NULL, NULL   },
+  { "imagesave",  S(IMGSAVE),  0, 1, "", "iS",   (SUBR)imagesave, NULL, NULL   },
+  { "imagefree",  S(IMGFREE),  0, 1, "", "i",    (SUBR)imagefree, NULL, NULL   },
 };
 
 
-LINKAGE1(image_localops)
-
+LINKAGE_BUILTIN(image_localops)
