@@ -802,8 +802,8 @@ static int midiRead_(CSOUND *csound,
 
 struct JackoInit : public OpcodeBase<JackoInit>
 {
-  MYFLT *ServerName;
-  MYFLT *SclientName;
+  STRINGDAT *ServerName;
+  STRINGDAT *SclientName;
   const char *serverName;
   const char *clientName;
   JackoState *jackoState;
@@ -811,14 +811,14 @@ struct JackoInit : public OpcodeBase<JackoInit>
   {
     serverName = csound->strarg2name(csound,
                                      (char *) 0,
-                                     ServerName,
+                                     ServerName->data,
                                      (char *)"default",
-                                     (int) csound->GetInputArgSMask(this));
+                                     (int) 1);
     clientName = csound->strarg2name(csound,
                                      (char *) 0,
-                                     SclientName,
+                                     SclientName->data,
                                      (char *)"csound",
-                                     (int) csound->GetInputArgSMask(this));
+                                     (int) 1);
     jackoState = new JackoState(csound, serverName, clientName);
     return OK;
   }
@@ -920,8 +920,8 @@ struct JackoAudioInConnect : public OpcodeBase<JackoAudioInConnect>
 {
   // Out.
   // Ins.
-  MYFLT *SexternalPortName;
-  MYFLT *ScsoundPortName;
+  STRINGDAT *SexternalPortName;
+  STRINGDAT *ScsoundPortName;
   // State.
   const char *csoundPortName;
   char csoundFullPortName[0x100];
@@ -937,15 +937,15 @@ struct JackoAudioInConnect : public OpcodeBase<JackoAudioInConnect>
       clientName = jack_get_client_name(jackoState->jackClient);
       csoundPortName = csound->strarg2name(csound,
                                            (char *)0,
-                                           ScsoundPortName,
+                                           ScsoundPortName->data,
                                            (char *)"",
-                                           (int) csound->GetInputArgSMask(this));
+                                           (int) 1);
       std::sprintf(csoundFullPortName, "%s:%s", clientName, csoundPortName);
       externalPortName = csound->strarg2name(csound,
                                              (char *) 0,
-                                             SexternalPortName,
+                                             SexternalPortName->data,
                                              (char *)"csound",
-                                             (int) csound->GetInputArgSMask(this));
+                                             (int) 1);
       csoundPort = jack_port_by_name(jackoState->jackClient, csoundFullPortName);
       if (!csoundPort) {
         csoundPort = jack_port_register(jackoState->jackClient,
@@ -990,7 +990,7 @@ struct JackoAudioIn : public OpcodeBase<JackoAudioIn>
   // Out.
   MYFLT *asignal;
   // Ins.
-  MYFLT *ScsoundPortName;
+  STRINGDAT *ScsoundPortName;
   // State.
   const char *csoundPortName;
   JackoState *jackoState;
@@ -1003,9 +1003,9 @@ struct JackoAudioIn : public OpcodeBase<JackoAudioIn>
       csoundFramesPerTick = jackoState->csoundFramesPerTick;
       csoundPortName = csound->strarg2name(csound,
                                            (char *)0,
-                                           ScsoundPortName,
+                                           ScsoundPortName->data,
                                            (char *)"",
-                                           (int) csound->GetInputArgSMask(this));
+                                           (int) 1);
       csoundPort = jackoState->audioInPorts[csoundPortName];
       return result;
   }
@@ -1025,8 +1025,8 @@ struct JackoAudioOutConnect : public OpcodeBase<JackoAudioOutConnect>
 {
   // No outs.
   // Ins.
-  MYFLT *ScsoundPortName;
-  MYFLT *SexternalPortName;
+  STRINGDAT *ScsoundPortName;
+  STRINGDAT *SexternalPortName;
   // State.
   const char *csoundPortName;
   char csoundFullPortName[0x100];
@@ -1039,20 +1039,20 @@ struct JackoAudioOutConnect : public OpcodeBase<JackoAudioOutConnect>
   int init(CSOUND *csound)
   {
       int result = OK;
-      frames = csound->GetKsmps(csound);
+      frames = opds.insdshead->ksmps;
       jackoState = getJackoState(csound);
       clientName = jack_get_client_name(jackoState->jackClient);
       csoundPortName = csound->strarg2name(csound,
                                            (char *)0,
-                                           ScsoundPortName,
+                                           ScsoundPortName->data,
                                            (char *)"",
-                                           (int) csound->GetInputArgSMask(this));
+                                           (int) 1);
       std::sprintf(csoundFullPortName, "%s:%s", clientName, csoundPortName);
       externalPortName = csound->strarg2name(csound,
                                              (char *) 0,
-                                             SexternalPortName,
+                                             SexternalPortName->data,
                                              (char *)"csound",
-                                             (int) csound->GetInputArgSMask(this));
+                                             (int) 1);
       csoundPort = jack_port_by_name(jackoState->jackClient, csoundFullPortName);
       if (!csoundPort) {
         csoundPort = jack_port_register(jackoState->jackClient,
@@ -1097,7 +1097,7 @@ struct JackoAudioOut : public OpcodeBase<JackoAudioOut>
 {
   // No outs.
   // Ins.
-  MYFLT *ScsoundPortName;
+  STRINGDAT *ScsoundPortName;
   MYFLT *asignal;
   // State.
   const char *csoundPortName;
@@ -1111,9 +1111,9 @@ struct JackoAudioOut : public OpcodeBase<JackoAudioOut>
       csoundFramesPerTick = jackoState->csoundFramesPerTick;
       csoundPortName = csound->strarg2name(csound,
                                            (char *)0,
-                                           ScsoundPortName,
+                                           ScsoundPortName->data,
                                            (char *)"",
-                                           (int) csound->GetInputArgSMask(this));
+                                           (int) 1);
       csoundPort = jackoState->audioOutPorts[csoundPortName];
       return result;
   }
@@ -1133,8 +1133,8 @@ struct JackoMidiInConnect : public OpcodeBase<JackoMidiInConnect>
 {
   // No outs.
   // Ins.
-  MYFLT *SexternalPortName;
-  MYFLT *ScsoundPortName;
+  STRINGDAT *SexternalPortName;
+  STRINGDAT *ScsoundPortName;
   // State.
   const char *csoundPortName;
   char csoundFullPortName[0x100];
@@ -1147,20 +1147,20 @@ struct JackoMidiInConnect : public OpcodeBase<JackoMidiInConnect>
   int init(CSOUND *csound)
   {
       int result = OK;
-      frames = csound->GetKsmps(csound);
+      frames = opds.insdshead->ksmps;
       jackoState = getJackoState(csound);
       clientName = jack_get_client_name(jackoState->jackClient);
       csoundPortName = csound->strarg2name(csound,
                                            (char *)0,
-                                           ScsoundPortName,
+                                           ScsoundPortName->data,
                                            (char *)"",
-                                           (int) csound->GetInputArgSMask(this));
+                                           (int) 1);
       std::sprintf(csoundFullPortName, "%s:%s", clientName, csoundPortName);
       externalPortName = csound->strarg2name(csound,
                                              (char *) 0,
-                                             SexternalPortName,
+                                             SexternalPortName->data,
                                              (char *)"csound",
-                                             (int) csound->GetInputArgSMask(this));
+                                             (int) 1);
       csoundPort = jack_port_by_name(jackoState->jackClient, csoundFullPortName);
       if (!csoundPort) {
         csoundPort = jack_port_register(jackoState->jackClient, csoundPortName,
@@ -1204,8 +1204,8 @@ struct JackoMidiOutConnect : public OpcodeBase<JackoMidiOutConnect>
 {
   // No outs.
   // Ins.
-  MYFLT *ScsoundPortName;
-  MYFLT *SexternalPortName;
+  STRINGDAT *ScsoundPortName;
+  STRINGDAT *SexternalPortName;
   // State.
   const char *csoundPortName;
   char csoundFullPortName[0x100];
@@ -1218,20 +1218,20 @@ struct JackoMidiOutConnect : public OpcodeBase<JackoMidiOutConnect>
   int init(CSOUND *csound)
   {
       int result = OK;
-      frames = csound->GetKsmps(csound);
+      frames = opds.insdshead->ksmps;
       jackoState = getJackoState(csound);
       clientName = jack_get_client_name(jackoState->jackClient);
       csoundPortName = csound->strarg2name(csound,
                                            (char *)0,
-                                           ScsoundPortName,
+                                           ScsoundPortName->data,
                                            (char *)"",
-                                           (int) csound->GetInputArgSMask(this));
+                                           (int) 1);
       std::sprintf(csoundFullPortName, "%s:%s", clientName, csoundPortName);
       externalPortName = csound->strarg2name(csound,
                                              (char *) 0,
-                                             SexternalPortName,
+                                             SexternalPortName->data,
                                              (char *)"csound",
-                                             (int) csound->GetInputArgSMask(this));
+                                             (int) 1);
       csoundPort = jack_port_by_name(jackoState->jackClient, csoundFullPortName);
       if (!csoundPort) {
         csoundPort = jack_port_register(jackoState->jackClient,
@@ -1275,7 +1275,7 @@ struct JackoMidiOut : public OpcodeBase<JackoMidiOut>
 {
   // No outs.
   // Ins.
-  MYFLT *ScsoundPortName;
+  STRINGDAT *ScsoundPortName;
   MYFLT *kstatus;
   MYFLT *kchannel;
   MYFLT *kdata1;
@@ -1301,9 +1301,9 @@ struct JackoMidiOut : public OpcodeBase<JackoMidiOut>
       csoundFramesPerTick = jackoState->csoundFramesPerTick;
       csoundPortName = csound->strarg2name(csound,
                                            (char *)0,
-                                           ScsoundPortName,
+                                           ScsoundPortName->data,
                                            (char *)"",
-                                           (int) csound->GetInputArgSMask(this));
+                                           (int) 1);
       csoundPort = jackoState->midiOutPorts[csoundPortName];
       priorstatus = -1;
       priorchannel = -1;
@@ -1351,7 +1351,7 @@ struct JackoNoteOut : public OpcodeNoteoffBase<JackoNoteOut>
 {
   // No outs.
   // Ins.
-  MYFLT *ScsoundPortName;
+  STRINGDAT *ScsoundPortName;
   MYFLT *ichannel;
   MYFLT *ikey;
   MYFLT *ivelocity;
@@ -1372,9 +1372,9 @@ struct JackoNoteOut : public OpcodeNoteoffBase<JackoNoteOut>
       csoundFramesPerTick = jackoState->csoundFramesPerTick;
       csoundPortName = csound->strarg2name(csound,
                                            (char *)0,
-                                           ScsoundPortName,
+                                           ScsoundPortName->data,
                                            (char *)"",
-                                           (int) csound->GetInputArgSMask(this));
+                                           (int) 1);
       csoundPort = jackoState->midiOutPorts[csoundPortName];
       status = 144;
       channel = (char) *ichannel;
@@ -1465,6 +1465,7 @@ extern "C"
     {
       (char *)"JackoInit",
       sizeof(JackoInit),
+      0,
       1,
       (char *)"",
       (char *)"SS",
@@ -1475,6 +1476,7 @@ extern "C"
     {
       (char *)"JackoInfo",
       sizeof(JackoInfo),
+      0,
       1,
       (char *)"",
       (char *)"",
@@ -1485,6 +1487,7 @@ extern "C"
     {
       (char *)"JackoFreewheel",
       sizeof(JackoFreewheel),
+      0,
       1,
       (char *)"",
       (char *)"i",
@@ -1495,6 +1498,7 @@ extern "C"
     {
       (char *)"JackoOn",
       sizeof(JackoOn),
+      0,
       1,
       (char *)"",
       (char *)"j",
@@ -1505,6 +1509,7 @@ extern "C"
     {
       (char *)"JackoAudioInConnect",
       sizeof(JackoAudioInConnect),
+      0,
       1,
       (char *)"",
       (char *)"SS",
@@ -1515,6 +1520,7 @@ extern "C"
     {
       (char *)"JackoAudioIn",
       sizeof(JackoAudioIn),
+      0,
       5,
       (char *)"a",
       (char *)"S",
@@ -1525,6 +1531,7 @@ extern "C"
     {
       (char *)"JackoAudioOutConnect",
       sizeof(JackoAudioOutConnect),
+      0,
       1,
       (char *)"",
       (char *)"SS",
@@ -1535,6 +1542,7 @@ extern "C"
     {
       (char *)"JackoAudioOut",
       sizeof(JackoAudioOut),
+      0,
       5,
       (char *)"",
       (char *)"Sa",
@@ -1545,6 +1553,7 @@ extern "C"
     {
       (char *)"JackoMidiInConnect",
       sizeof(JackoMidiInConnect),
+      0,
       1,
       (char *)"",
       (char *)"SS",
@@ -1555,6 +1564,7 @@ extern "C"
     {
       (char *)"JackoMidiOutConnect",
       sizeof(JackoMidiOutConnect),
+      0,
       1,
       (char *)"",
       (char *)"SS",
@@ -1565,6 +1575,7 @@ extern "C"
     {
       (char *)"JackoMidiOut",
       sizeof(JackoMidiOut),
+      0,
       3,
       (char *)"",
       (char *)"SkkkO",
@@ -1575,6 +1586,7 @@ extern "C"
     {
       (char *)"JackoNoteOut",
       sizeof(JackoNoteOut),
+      0,
       3,
       (char *)"",
       (char *)"Siii",
@@ -1585,6 +1597,7 @@ extern "C"
     {
       (char *)"JackoTransport",
       sizeof(JackoTransport),
+      0,
       3,
       (char *)"",
       (char *)"kO", // O defaults to 0.
@@ -1592,7 +1605,7 @@ extern "C"
       (SUBR)&JackoTransport::kontrol_,
       0
     },
-    { 0, 0, 0, 0, 0, (SUBR) 0, (SUBR) 0, (SUBR) 0 }
+    { 0, 0, 0, 0, 0, 0, (SUBR) 0, (SUBR) 0, (SUBR) 0 }
   };
 
 
@@ -1609,6 +1622,7 @@ extern "C"
       err |= csound->AppendOpcode(csound,
                                   ep->opname,
                                   ep->dsblksiz,
+                                  ep->flags,
                                   ep->thread,
                                   ep->outypes,
                                   ep->intypes,
@@ -1625,7 +1639,8 @@ extern "C"
     int result = OK;
 #pragma omp critical
     {
-      std::map<CSOUND *, JackoState *>::iterator it = jackoStatesForCsoundInstances.find(csound);
+      std::map<CSOUND *, JackoState *>::iterator it =
+        jackoStatesForCsoundInstances.find(csound);
       if (it != jackoStatesForCsoundInstances.end()) {
         //delete it->second;
         jackoStatesForCsoundInstances.erase(it);
