@@ -23,7 +23,7 @@
 
 /* sliders and other MIDI opcodes by Gabriel Maldonado */
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include "midiops.h"
 #include "midiops3.h"
 #include <math.h>
@@ -65,7 +65,7 @@
                   j);                                             \
           return csound->InitError(csound, sbuf);                 \
         }                                                         \
-        if (*sld->ifn > 0)   *ftp++ = csound->FTFind(csound, sld->ifn); \
+        if (*sld->ifn > 0)   *ftp++ = csound->FTnp2Find(csound, sld->ifn); \
         else                 *ftp++ = NULL;                       \
         value =  (*(sld++)->initvalue - *min) / (*max++ - *min);  \
         min++;                                                    \
@@ -146,7 +146,7 @@ static int slider64(CSOUND *csound, SLIDER64 *p)
       return csound->InitError(csound, Str("illegal channel"));   \
     }                                                             \
     {                                                             \
-      MYFLT value;                                                \
+      MYFLT value = FL(0.0);                                                \
       int j = 0;                                                  \
       SLDf *sld = p->s;                                           \
       unsigned char *slnum = p->slnum;                            \
@@ -168,7 +168,7 @@ static int slider64(CSOUND *csound, SLIDER64 *p)
                   Str("illegal initvalue at position n.%d"), j);  \
           return csound->InitError(csound, sbuf);                 \
         }                                                         \
-        if (*sld->ifn > 0)   *ftp++ = csound->FTFind(csound, sld->ifn); \
+        if (*sld->ifn > 0)   *ftp++ = csound->FTnp2Find(csound, sld->ifn); \
         else                 *ftp++ = NULL;                       \
         value =  (*sld->initvalue - *min) / (*max++ - *min);      \
         min++;;                                                   \
@@ -178,7 +178,7 @@ static int slider64(CSOUND *csound, SLIDER64 *p)
         *yt1++ = FL(0.0);                                         \
         b = (MYFLT)(2.0 - cos((double)(*(sld++)->ihp              \
                                        * csound->tpidsr           \
-                                       * csound->ksmps)));        \
+                                       * CS_KSMPS)));        \
         *c2 = (MYFLT)(b - sqrt((double)(b * b - FL(1.0))));       \
         *c1++ = FL(1.0) - *c2++;                                  \
       }                                                           \
@@ -275,7 +275,7 @@ if (UNLIKELY(chan  > 15))  {                                      \
         }                                                         \
         value = chanblock[slnum] * oneTOf7bit;                    \
         if (*sld->ifn > 0)  {                                     \
-          ftp = csound->FTFind(csound, sld->ifn);                 \
+          ftp = csound->FTnp2Find(csound, sld->ifn);                 \
           value = *( ftp->ftable + (int32)(value * ftp->flen));    \
                                 /* no interpolation */            \
         }                                                         \
@@ -347,7 +347,7 @@ if (UNLIKELY(chan  > 15))  {                                           \
                   Str("illegal initvalue at position n.%d"), j);       \
           return csound->InitError(csound, sbuf);                      \
         }                                                              \
-        if (*sld->ifn > 0)   *ftp++ = csound->FTFind(csound, sld->ifn); \
+        if (*sld->ifn > 0)   *ftp++ = csound->FTnp2Find(csound, sld->ifn); \
         else                 *ftp++ = NULL;                            \
         intvalue = (int) (((*(sld++)->initvalue - *min) / (*max++ - *min)) \
                           * f14bit+FL(0.5));                           \
@@ -361,7 +361,7 @@ if (UNLIKELY(chan  > 15))  {                                           \
 
 #define SLIDER14(p, n)                                                 \
 {                                                                      \
-    MYFLT value;                                                       \
+    MYFLT value = FL(0.0);                                                       \
     int j = 0;                                                         \
     FUNC **ftp = p->ftp-1;                                             \
     MYFLT *chanblock = (MYFLT *) csound->m_chnbp[p->slchan]->ctl_val;  \
@@ -440,7 +440,7 @@ if (UNLIKELY(chan  > 15))  {                                           \
         value = (MYFLT)((chanblock[slnum_msb]  * 128                   \
                          + chanblock[slnum_lsb]) * oneTOf14bit);       \
         if (*sld->ifn > 0) {    /* linear interpolation routine */     \
-          FUNC *ftp= csound->FTFind(csound, sld->ifn);                 \
+          FUNC *ftp= csound->FTnp2Find(csound, sld->ifn);                 \
           MYFLT phase = value * ftp->flen;                             \
           MYFLT *base = ftp->ftable + (int32)(phase);                  \
           value = *base + (*(base + 1) - *base) * (phase - (int32) phase); \
@@ -472,35 +472,35 @@ static OENTRY localops[] = {
 { "slider32", 0xffff,                                                   },
 { "slider64", 0xffff,                                                   },
 { "slider8", 0xffff,                                                    },
-{ "slider8.k", S(SLIDER8), 3, "kkkkkkkk",  "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
+{ "slider8.k", S(SLIDER8), 0, 3, "kkkkkkkk",  "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                               "iiiiiiii", (SUBR)slider_i8, (SUBR)slider8, NULL },
-{ "slider8f", S(SLIDER8f), 3, "kkkkkkkk","iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
+{ "slider8f", S(SLIDER8f), 0, 3, "kkkkkkkk","iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiii",
                                         (SUBR)slider_i8f, (SUBR)slider8f, NULL },
-{ "slider8.i", S(SLIDER8), 1, "iiiiiiii", "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii",
+{ "slider8.i", S(SLIDER8), 0, 1, "iiiiiiii", "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii",
                                           (SUBR)islider8, NULL, NULL },
-{ "slider16.k", S(SLIDER16), 3, "kkkkkkkkkkkkkkkk",
+{ "slider16.k", S(SLIDER16), 0, 3, "kkkkkkkkkkkkkkkk",
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiii",
                                         (SUBR)slider_i16, (SUBR)slider16, NULL },
-{ "slider16f", S(SLIDER16f), 3, "kkkkkkkkkkkkkkkk",
+{ "slider16f", S(SLIDER16f), 0, 3, "kkkkkkkkkkkkkkkk",
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiii",
                                         (SUBR)slider_i16f, (SUBR)slider16f, NULL },
-{ "slider16.i", S(SLIDER16), 1, "iiiiiiiiiiiiiiii",
+{ "slider16.i", S(SLIDER16), 0, 1, "iiiiiiiiiiiiiiii",
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiii",
                                         (SUBR)islider16, NULL, NULL       },
-{ "slider32.k", S(SLIDER32),  3, "kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk",
+{ "slider32.k", S(SLIDER32),  0, 3, "kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk",
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiii",
                                         (SUBR)slider_i32, (SUBR)slider32, NULL  },
-{ "slider32f", S(SLIDER32f), 3, "kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk",
+{ "slider32f", S(SLIDER32f), 0, 3, "kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk",
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
@@ -508,13 +508,13 @@ static OENTRY localops[] = {
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiii",
                                         (SUBR)slider_i32f, (SUBR)slider32f, NULL },
-{ "slider32.i", S(SLIDER32), 1, "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii",
+{ "slider32.i", S(SLIDER32), 0, 1, "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii",
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiii",
                                         (SUBR)islider32, NULL, NULL  },
-{ "slider64.k", S(SLIDER64), 3, "kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk"
+{ "slider64.k", S(SLIDER64), 0, 3, "kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk"
                               "kkkkkkkkkkkkkkkkkkk",
                                         "iiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiii"
@@ -530,7 +530,7 @@ static OENTRY localops[] = {
                                         "iiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiii",
                                         (SUBR)slider_i64, (SUBR)slider64, NULL  },
-{ "slider64f", S(SLIDER64f), 3, "kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk"
+{ "slider64f", S(SLIDER64f), 0, 3, "kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk"
                                 "kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk",
                                         "iiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiii"
@@ -549,7 +549,7 @@ static OENTRY localops[] = {
                                         "iiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiii",
                                         (SUBR)slider_i64f, (SUBR)slider64f, NULL },
-{ "slider64.i", S(SLIDER64), 1, "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
+{ "slider64.i", S(SLIDER64), 0, 1, "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                 "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii",
                                         "iiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiii"
@@ -563,12 +563,12 @@ static OENTRY localops[] = {
                                         "iiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiii",
                                         (SUBR)islider64, NULL, NULL  },
-{ "s16b14.k", S(SLIDER16BIT14), 3, "kkkkkkkkkkkkkkkk",
+{ "s16b14.k", S(SLIDER16BIT14), 0, 3, "kkkkkkkkkkkkkkkk",
                                    "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                    "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                    "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii",
                                  (SUBR)slider_i16bit14, (SUBR)slider16bit14, NULL},
-{ "s32b14.k", S(SLIDER32BIT14), 3, "kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk",
+{ "s32b14.k", S(SLIDER32BIT14), 0, 3, "kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk",
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
@@ -576,14 +576,14 @@ static OENTRY localops[] = {
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii",
                                  (SUBR)slider_i32bit14, (SUBR)slider32bit14, NULL},
-{ "s16b14.i", S(ISLIDER16BIT14), 1, "iiiiiiiiiiiiiiii",
+{ "s16b14.i", S(ISLIDER16BIT14), 0, 1, "iiiiiiiiiiiiiiii",
                                         "iiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiii",
                                         (SUBR)islider16bit14, NULL, NULL  },
-{ "s32b14.i", S(ISLIDER32BIT14), 1, "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii",
+{ "s32b14.i", S(ISLIDER32BIT14), 0, 1, "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii",
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
                                         "iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii"
