@@ -139,7 +139,7 @@ static MYFLT Samp_tick(Wave *p)
 
     temp = (int32) temp_time;    /*  Integer part of time address    */
     temp1 = temp + 1;
-    if (UNLIKELY(temp1==p->wave->flen)) temp1 = 0; /* Wrap!! */
+    if (UNLIKELY(temp1==(int)p->wave->flen)) temp1 = 0; /* Wrap!! */
     /*  fractional part of time address */
     alpha = temp_time - (MYFLT)temp;
     lastOutput = p->wave->ftable[temp];  /* Do linear interpolation */
@@ -163,13 +163,13 @@ int Moog1set(CSOUND *csound, MOOG1 *p)
     make_FormSwep(&p->filters[0]);
     make_FormSwep(&p->filters[1]);
 
-    if (LIKELY((ftp = csound->FTFind(csound, p->iatt)) != NULL))
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->iatt)) != NULL))
       p->attk.wave = ftp; /* mandpluk */
     else return NOTOK;
-    if (LIKELY((ftp = csound->FTFind(csound, p->ifn )) != NULL))
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ifn )) != NULL))
       p->loop.wave = ftp; /* impuls20 */
     else return NOTOK;
-    if (LIKELY((ftp = csound->FTFind(csound, p->ivfn)) != NULL))
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->ivfn)) != NULL))
       p->vibr.wave = ftp; /* sinewave */
     else return NOTOK;
     p->attk.time = p->attk.phase = FL(0.0);
@@ -186,7 +186,9 @@ int Moog1(CSOUND *csound, MOOG1 *p)
 {
     MYFLT       amp = *p->amp * AMP_RSCALE; /* Normalised */
     MYFLT       *ar = p->ar;
-    int32        n, nsmps = csound->ksmps;
+    uint32_t    offset = p->h.insdshead->ksmps_offset;
+    uint32_t    early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    n, nsmps = CS_KSMPS;
     MYFLT       temp;
     MYFLT       vib = *p->vibAmt;
 
@@ -215,7 +217,12 @@ int Moog1(CSOUND *csound, MOOG1 *p)
     }
     p->vibr.rate = *p->vibf * p->vibr.wave->flen * csound->onedsr;
 
-    for (n = 0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n<nsmps; n++) {
       MYFLT     temp;
       MYFLT     output;
       int32      itemp;
