@@ -190,8 +190,8 @@ static MYFLT noise_tick(CSOUND *csound)
 static int cabasaset(CSOUND *csound, CABASA *p)
 {
     p->sndLevel = FL(0.0);
-    p->kloop = (int)(p->h.insdshead->offtim * csound->ekr)
-               - (int)(csound->ekr * *p->dettack);
+    p->kloop = (int)(p->h.insdshead->offtim * CS_EKR)
+               - (int)(CS_EKR * *p->dettack);
     p->outputs0 = FL(0.0);
     p->outputs1 = FL(0.0);
     p->shake_maxSave = FL(0.0);
@@ -213,7 +213,9 @@ static int cabasaset(CSOUND *csound, CABASA *p)
 static int cabasa(CSOUND *csound, CABASA *p)
 {
     MYFLT *ar = p->ar;
-    int   n,nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT data;
                                 /* Use locals for speed */
     MYFLT shakeEnergy = p->shakeEnergy;
@@ -245,7 +247,7 @@ static int cabasa(CSOUND *csound, CABASA *p)
 
     if (*p->shake_max != FL(0.0)) {
       shakeEnergy = p->shakeEnergy +=
-        csound->ksmps * *p->shake_max * MAX_SHAKE * FL(0.1);
+        CS_KSMPS * *p->shake_max * MAX_SHAKE * FL(0.1);
       if (shakeEnergy > MAX_SHAKE) shakeEnergy = MAX_SHAKE;
     }
 
@@ -254,7 +256,12 @@ static int cabasa(CSOUND *csound, CABASA *p)
       shakeEnergy = FL(0.0);
     }
 
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
 /*        if (shakeEnergy > MIN_ENERGY) { */
       shakeEnergy *= systemDecay;               /* Exponential system decay */
       if (my_random(csound, 1024) < p->num_objects) {
@@ -285,8 +292,8 @@ static int cabasa(CSOUND *csound, CABASA *p)
 static int sekereset(CSOUND *csound, SEKERE *p)
 {
     p->sndLevel = FL(0.0);
-    p->kloop = (int)(p->h.insdshead->offtim * csound->ekr)
-               - (int)(csound->ekr * *p->dettack);
+    p->kloop = (int)(p->h.insdshead->offtim * CS_EKR)
+               - (int)(CS_EKR * *p->dettack);
     p->outputs0 = FL(0.0);
     p->outputs1 = FL(0.0);
     p->finalZ2 = FL(0.0);
@@ -312,7 +319,9 @@ static int sekereset(CSOUND *csound, SEKERE *p)
 static int sekere(CSOUND *csound, SEKERE *p)
 {
     MYFLT *ar = p->ar;
-    int   n,nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT data;
                                 /* Use locals for speed */
     MYFLT shakeEnergy = p->shakeEnergy;
@@ -344,7 +353,7 @@ static int sekere(CSOUND *csound, SEKERE *p)
 
     if (*p->shake_max != FL(0.0)) {
       shakeEnergy = p->shakeEnergy +=
-        csound->ksmps * *p->shake_max * MAX_SHAKE * FL(0.1);
+        CS_KSMPS * *p->shake_max * MAX_SHAKE * FL(0.1);
       if (shakeEnergy > MAX_SHAKE) shakeEnergy = MAX_SHAKE;
     }
 
@@ -353,7 +362,12 @@ static int sekere(CSOUND *csound, SEKERE *p)
       shakeEnergy = FL(0.0);
     }
 
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
 /*        if (shakeEnergy > MIN_ENERGY) { */
       shakeEnergy *= systemDecay;           /* Exponential system decay */
       if (my_random(csound, 1024) < p->num_objects) {
@@ -377,6 +391,7 @@ static int sekere(CSOUND *csound, SEKERE *p)
 /*          ar[n] = 0.0f; */
 /*        } */
     }
+    printf("%d/%d:\n", offset, early);
     p->shakeEnergy = shakeEnergy;
     p->sndLevel = sndLevel;
     p->outputs0 = outputs0;
@@ -387,8 +402,8 @@ static int sekere(CSOUND *csound, SEKERE *p)
 static int sandset(CSOUND *csound, SEKERE *p)
 {
     p->sndLevel = FL(0.0);
-    p->kloop = (int)(p->h.insdshead->offtim * csound->ekr)
-               - (int)(csound->ekr * *p->dettack);
+    p->kloop = (int)(p->h.insdshead->offtim * CS_EKR)
+               - (int)(CS_EKR * *p->dettack);
     p->outputs0 = FL(0.0);
     p->outputs1 = FL(0.0);
     p->finalZ2 = FL(0.0);
@@ -415,8 +430,8 @@ static int sandset(CSOUND *csound, SEKERE *p)
 static int stixset(CSOUND *csound, SEKERE *p)
 {
     p->sndLevel = FL(0.0);
-    p->kloop = (int)(p->h.insdshead->offtim * csound->ekr)
-               - (int)(csound->ekr * *p->dettack);
+    p->kloop = (int)(p->h.insdshead->offtim * CS_EKR)
+               - (int)(CS_EKR * *p->dettack);
     p->outputs0 = FL(0.0);
     p->outputs1 = FL(0.0);
     p->finalZ2 = FL(0.0);
@@ -442,8 +457,8 @@ static int stixset(CSOUND *csound, SEKERE *p)
 static int crunchset(CSOUND *csound, CABASA *p)
 {
     p->sndLevel = FL(0.0);
-    p->kloop = (int)(p->h.insdshead->offtim * csound->ekr)
-               - (int)(csound->ekr * *p->dettack);
+    p->kloop = (int)(p->h.insdshead->offtim * CS_EKR)
+               - (int)(CS_EKR * *p->dettack);
     p->outputs0 = FL(0.0);
     p->outputs1 = FL(0.0);
     p->shake_maxSave = FL(0.0);
@@ -469,8 +484,8 @@ static int guiroset(CSOUND *csound, GUIRO *p)
     MYFLT temp;
 
     p->sndLevel = FL(0.0);
-    p->kloop = (int)(p->h.insdshead->offtim * csound->ekr)
-               - (int)(csound->ekr * *p->dettack);
+    p->kloop = (int)(p->h.insdshead->offtim * CS_EKR)
+               - (int)(CS_EKR * *p->dettack);
 
     p->outputs00    = FL(0.0);
     p->outputs01    = FL(0.0);
@@ -512,7 +527,9 @@ static int guiroset(CSOUND *csound, GUIRO *p)
 static int guiro(CSOUND *csound, GUIRO *p)
 {
     MYFLT *ar = p->ar;
-    int  n,nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT lastOutput;
 
     if (*p->num_teeth != FL(0.0) &&
@@ -572,7 +589,12 @@ static int guiro(CSOUND *csound, GUIRO *p)
       MYFLT gains0       = p->gains0;
       MYFLT gains1       = p->gains1;
       MYFLT amp          = *p->amp;
-      for (n=0;n<nsmps;n++) {
+      if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+      if (UNLIKELY(early)) {
+        nsmps -= early;
+        memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+      }
+      for (n=offset;n<nsmps;n++) {
         if (ratchetPos > 0) {
           ratchet -= (ratchetDelta + (FL(0.002)*totalEnergy));
           if (ratchet < FL(0.0)) {
@@ -628,8 +650,8 @@ static int tambourset(CSOUND *csound, TAMBOURINE *p)
     MYFLT temp;
 
     p->sndLevel = FL(0.0);
-    p->kloop = (int)(p->h.insdshead->offtim * csound->ekr)
-               - (int)(csound->ekr * *p->dettack);
+    p->kloop = (int)(p->h.insdshead->offtim * CS_EKR)
+               - (int)(CS_EKR * *p->dettack);
 
     p->outputs00       = FL(0.0);
     p->outputs01       = FL(0.0);
@@ -673,7 +695,9 @@ static int tambourset(CSOUND *csound, TAMBOURINE *p)
 static int tambourine(CSOUND *csound, TAMBOURINE *p)
 {
     MYFLT *ar = p->ar;
-    int  n,nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT data;
     MYFLT temp_rand;
     MYFLT lastOutput;
@@ -718,7 +742,12 @@ static int tambourine(CSOUND *csound, TAMBOURINE *p)
       MYFLT sndLevel = p->sndLevel;
       MYFLT soundDecay = p->soundDecay;
       MYFLT inputs0, inputs1, inputs2;
-      for (n=0;n<nsmps;n++) {
+      if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+      if (UNLIKELY(early)) {
+        nsmps -= early;
+        memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+      }
+      for (n=offset;n<nsmps;n++) {
         shakeEnergy *= systemDecay; /* Exponential system decay */
         if (my_random(csound, 1024) < p->num_objects) {
           sndLevel += p->gain * shakeEnergy;
@@ -766,8 +795,8 @@ static int bambooset(CSOUND *csound, BAMBOO *p)
     MYFLT temp;
 
     p->sndLevel = FL(0.0);
-    p->kloop = (int)(p->h.insdshead->offtim * csound->ekr)
-               - (int)(csound->ekr * *p->dettack);
+    p->kloop = (int)(p->h.insdshead->offtim * CS_EKR)
+               - (int)(CS_EKR * *p->dettack);
 
     p->outputs00       = FL(0.0);
     p->outputs01       = FL(0.0);
@@ -806,7 +835,9 @@ static int bambooset(CSOUND *csound, BAMBOO *p)
 static int bamboo(CSOUND *csound, BAMBOO *p)
 {
     MYFLT *ar = p->ar;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT data;
     MYFLT temp_rand;
     MYFLT lastOutput;
@@ -850,7 +881,12 @@ static int bamboo(CSOUND *csound, BAMBOO *p)
       MYFLT sndLevel    = p->sndLevel;
       MYFLT soundDecay  = p->soundDecay;
       MYFLT inputs0, inputs1, inputs2;
-      for (n=0;n<nsmps;n++) {
+      if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+      if (UNLIKELY(early)) {
+        nsmps -= early;
+        memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+      }
+      for (n=offset;n<nsmps;n++) {
         shakeEnergy *= systemDecay; /* Exponential system decay */
         if (my_random(csound, 1024) < p->num_objects) {
           sndLevel += shakeEnergy;
@@ -901,8 +937,8 @@ static int wuterset(CSOUND *csound, WUTER *p)
     MYFLT temp;
 
     p->sndLevel = FL(0.0);
-    p->kloop = (int)(p->h.insdshead->offtim * csound->ekr)
-             - (int)(csound->ekr * *p->dettack);
+    p->kloop = (int)(p->h.insdshead->offtim * CS_EKR)
+             - (int)(CS_EKR * *p->dettack);
 
     p->outputs00       = FL(0.0);
     p->outputs01       = FL(0.0);
@@ -942,7 +978,9 @@ static int wuterset(CSOUND *csound, WUTER *p)
 static int wuter(CSOUND *csound, WUTER *p)
 {
     MYFLT *ar = p->ar;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT data;
     MYFLT lastOutput;
 
@@ -987,7 +1025,12 @@ static int wuter(CSOUND *csound, WUTER *p)
       MYFLT soundDecay = p->soundDecay;
       MYFLT inputs0, inputs1, inputs2;
 
-      for (n=0;n<nsmps;n++) {
+      if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+      if (UNLIKELY(early)) {
+        nsmps -= early;
+        memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+      }
+      for (n=offset;n<nsmps;n++) {
 
         shakeEnergy *= systemDecay;               /* Exponential system decay */
         if (my_random(csound, 32767) < num_objects) {
@@ -1073,8 +1116,8 @@ static int sleighset(CSOUND *csound, SLEIGHBELLS *p)
     MYFLT temp;
 
     p->sndLevel = FL(0.0);
-    p->kloop = (int)(p->h.insdshead->offtim * csound->ekr)
-               - (int)(csound->ekr * *p->dettack);
+    p->kloop = (int)(p->h.insdshead->offtim * CS_EKR)
+               - (int)(CS_EKR * *p->dettack);
 
     p->outputs00 = FL(0.0);
     p->outputs01 = FL(0.0);
@@ -1123,7 +1166,9 @@ static int sleighset(CSOUND *csound, SLEIGHBELLS *p)
 static int sleighbells(CSOUND *csound, SLEIGHBELLS *p)
 {
     MYFLT *ar = p->ar;
-    int  n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT data;
     MYFLT temp_rand;
     MYFLT lastOutput;
@@ -1167,7 +1212,12 @@ static int sleighbells(CSOUND *csound, SLEIGHBELLS *p)
       MYFLT sndLevel = p->sndLevel;
       MYFLT soundDecay = p->soundDecay;
       MYFLT inputs0, inputs1, inputs2, inputs3, inputs4;
-      for (n=0;n<nsmps;n++) {
+      if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+      if (UNLIKELY(early)) {
+        nsmps -= early;
+        memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+      }
+      for (n=offset;n<nsmps;n++) {
         shakeEnergy *= systemDecay; /* Exponential system decay */
         if (my_random(csound, 1024) < p->num_objects) {
           sndLevel += p->gain * shakeEnergy;
@@ -1234,18 +1284,19 @@ static int sleighbells(CSOUND *csound, SLEIGHBELLS *p)
 #define S(x)    sizeof(x)
 
 static OENTRY phisem_localops[] = {
-{ "cabasa",  S(CABASA),  5, "a", "iiooo",    (SUBR)cabasaset, NULL, (SUBR)cabasa},
-{ "crunch",  S(CABASA),  5, "a", "iiooo",    (SUBR)crunchset, NULL, (SUBR)cabasa},
-{ "sekere",  S(SEKERE),  5, "a", "iiooo",    (SUBR)sekereset, NULL, (SUBR)sekere},
-{ "sandpaper", S(SEKERE),5, "a", "iiooo",    (SUBR)sandset, NULL,   (SUBR)sekere},
-{ "stix", S(SEKERE),     5, "a", "iiooo",    (SUBR)stixset, NULL,   (SUBR)sekere},
-{ "guiro", S(GUIRO),     5, "a", "kiooooo",  (SUBR)guiroset, NULL,  (SUBR)guiro },
-{ "tambourine", S(TAMBOURINE),5,"a", "kioooooo",
+{ "cabasa",  S(CABASA),  0, 5, "a", "iiooo",  (SUBR)cabasaset, NULL, (SUBR)cabasa},
+{ "crunch",  S(CABASA),  0, 5, "a", "iiooo",  (SUBR)crunchset, NULL, (SUBR)cabasa},
+{ "sekere",  S(SEKERE),  0, 5, "a", "iiooo",  (SUBR)sekereset, NULL, (SUBR)sekere},
+{ "sandpaper", S(SEKERE),0, 5, "a", "iiooo",  (SUBR)sandset, NULL,   (SUBR)sekere},
+{ "stix", S(SEKERE),     0, 5, "a", "iiooo",  (SUBR)stixset, NULL,   (SUBR)sekere},
+{ "guiro", S(GUIRO),     0, 5, "a", "kiooooo",(SUBR)guiroset, NULL,  (SUBR)guiro },
+{ "tambourine", S(TAMBOURINE),0, 5,"a", "kioooooo",
                                         (SUBR)tambourset, NULL, (SUBR)tambourine},
-{ "bamboo", S(BAMBOO),   5, "a", "kioooooo", (SUBR)bambooset, NULL, (SUBR)bamboo },
-{ "dripwater", S(WUTER), 5, "a", "kioooooo", (SUBR)wuterset, NULL, (SUBR)wuter },
-{ "sleighbells", S(SLEIGHBELLS), 5, "a","kioooooo",
+{ "bamboo", S(BAMBOO),   0, 5, "a", "kioooooo",
+                                        (SUBR)bambooset, NULL, (SUBR)bamboo },
+{ "dripwater", S(WUTER), 0, 5, "a", "kioooooo", (SUBR)wuterset, NULL, (SUBR)wuter },
+{ "sleighbells", S(SLEIGHBELLS), 0, 5, "a","kioooooo",
                                        (SUBR)sleighset, NULL, (SUBR)sleighbells }
 };
 
-LINKAGE1(phisem_localops)
+LINKAGE_BUILTIN(phisem_localops)
