@@ -65,6 +65,12 @@ typedef struct {
         void    *clk;
 } CLKRD;
 
+typedef struct {
+        OPDS    h;
+        MYFLT   *val;
+        MYFLT   *index;
+} SCRATCHPAD;
+
 typedef struct {
         OPDS    h;
         MYFLT   *ins;
@@ -76,6 +82,7 @@ typedef struct {
         MYFLT   *cnt;
         MYFLT   *ins;
         MYFLT   *opt;
+        MYFLT   *norel;
 } INSTCNT;
 
 typedef struct {
@@ -89,7 +96,7 @@ typedef struct {
     FUNC    *ftp;
     FUNC    *freqtp;
     FUNC    *amptp;
-    int     count;
+    unsigned int     count;
     int     inerr;
     AUXCH   lphs;
 } ADSYNT;
@@ -169,7 +176,7 @@ typedef struct {
         OPDS    h;
         MYFLT   *ar;
         MYFLT   *amp, *freq, *offset;
-        int     next;
+        unsigned int     next;
 } IMPULSE;
 
 typedef struct {
@@ -268,12 +275,16 @@ int clockoff(CSOUND *, CLOCK *p);
 int clockon(CSOUND *, CLOCK *p);
 int clockread(CSOUND *, CLKRD *p);
 int clockset(CSOUND *, CLOCK *p);
+int scratchread(CSOUND *, SCRATCHPAD *p);
+int scratchwrite(CSOUND *, SCRATCHPAD *p);
 int cpuperc(CSOUND *, CPU_PERC *p);
+int cpuperc_S(CSOUND *, CPU_PERC *p);
 int hsboscil(CSOUND *, HSBOSC *p);
 int hsboscset(CSOUND *, HSBOSC *p);
 int impulse(CSOUND *, IMPULSE *p);
 int impulse_set(CSOUND *, IMPULSE *p);
 int instcount(CSOUND *, INSTCNT *p);
+int instcount_S(CSOUND *, INSTCNT *p);
 int totalcount(CSOUND *, INSTCNT *p);
 int kphsorbnk(CSOUND *, PHSORBNK *p);
 int ktrnseg(CSOUND *, TRANSEG *p);
@@ -285,6 +296,8 @@ int maca(CSOUND *, SUM *p);
 int macset(CSOUND *, SUM *p);
 int maxalloc(CSOUND *, CPU_PERC *p);
 int mute_inst(CSOUND *, MUTE *p);
+int maxalloc_S(CSOUND *, CPU_PERC *p);
+int mute_inst_S(CSOUND *, MUTE *p);
 int pfun(CSOUND *, PFUN *p);
 int pfunk_init(CSOUND *, PFUNK *p);
 int pfunk(CSOUND *, PFUNK *p);
