@@ -63,7 +63,7 @@ PLUS a number of track processing opcodes.
 
 */
 
-#include "csdl.h"
+#include "pvs_ops.h"
 #include "pstream.h"
 
 typedef struct _psyn {
@@ -143,13 +143,16 @@ static int psynth_process(CSOUND *csound, _PSYN *p)
     double  a, f, frac, incra, incrph, factor;
     MYFLT   scale = *p->scal, pitch = *p->pitch;
     int     ndx, size = p->func->flen;
-    int     i, j, k, n, m, id;
+    int     i, j, k, m, id;
     int     notcontin = 0;
     int     contin = 0;
     int     tracks = p->tracks, maxtracks = (int) *p->maxtracks;
     MYFLT   *tab = p->func->ftable, *out = p->out;
     float   *fin = (float *) p->fin->frame.auxp;
-    int     ksmps = csound->ksmps, pos = p->pos;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int      pos = p->pos;
     double   *amps = (double *) p->amps.auxp, *freqs = (double *) p->freqs.auxp;
     double   *phases = (double *) p->phases.auxp;
     MYFLT    *outsum = (MYFLT *) p->sum.auxp;
@@ -161,7 +164,12 @@ static int psynth_process(CSOUND *csound, _PSYN *p)
 
     maxtracks = p->numbins > maxtracks ? maxtracks : p->numbins;
 
-    for (n = 0; n < ksmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       out[n] = outsum[pos];
       pos++;
       if (pos == hopsize) {
@@ -301,13 +309,16 @@ static int psynth2_process(CSOUND *csound, _PSYN2 *p)
     double   a, frac, incra, incrph, factor, lotwopi, cnt;
     MYFLT   scale = *p->scal;
     int     ndx, size = p->func->flen;
-    int     i, j, k, n, m, id;
+    int     i, j, k, m, id;
     int     notcontin = 0;
     int     contin = 0;
     int     tracks = p->tracks, maxtracks = (int) *p->maxtracks;
     MYFLT   *tab = p->func->ftable, *out = p->out;
     float   *fin = (float *) p->fin->frame.auxp;
-    int     ksmps = csound->ksmps, pos = p->pos;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int      pos = p->pos;
     double   *amps = (double *) p->amps.auxp, *freqs = (double *) p->freqs.auxp;
     double   *phases = (double *) p->phases.auxp;
     MYFLT   *outsum = (MYFLT *) p->sum.auxp;
@@ -320,7 +331,12 @@ static int psynth2_process(CSOUND *csound, _PSYN2 *p)
     facsqr = p->facsqr;
     maxtracks = p->numbins > maxtracks ? maxtracks : p->numbins;
 
-    for (n = 0; n < ksmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       out[n] = outsum[pos];
       pos++;
       if (pos == hopsize) {
@@ -426,13 +442,16 @@ static int psynth3_process(CSOUND *csound, _PSYN *p)
     double   a, frac, incra, incrph, factor, lotwopi, cnt;
     MYFLT   scale = *p->scal, pitch = *p->pitch;
     int     ndx, size = p->func->flen;
-    int     i, j, k, n, m, id;
+    int     i, j, k, m, id;
     int     notcontin = 0;
     int     contin = 0;
     int     tracks = p->tracks, maxtracks = (int) *p->maxtracks;
     MYFLT   *tab = p->func->ftable, *out = p->out;
     float   *fin = (float *) p->fin->frame.auxp;
-    int     ksmps = csound->ksmps, pos = p->pos;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int      pos = p->pos;
     double   *amps = (double *) p->amps.auxp, *freqs = (double *) p->freqs.auxp;
     double   *phases = (double *) p->phases.auxp;
     MYFLT    *outsum = (MYFLT *) p->sum.auxp;
@@ -445,7 +464,12 @@ static int psynth3_process(CSOUND *csound, _PSYN *p)
     facsqr = p->facsqr;
     maxtracks = p->numbins > maxtracks ? maxtracks : p->numbins;
 
-    for (n = 0; n < ksmps; n++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       out[n] = outsum[pos];
       pos++;
       if (pos == hopsize) {
@@ -587,7 +611,7 @@ static int trscale_process(CSOUND *csound, _PTRANS *p)
 {
     MYFLT   scale = *p->kpar;
     MYFLT   gain = (p->kgain != NULL ? *p->kgain : FL(1.0));
-    MYFLT   nyq = csound->esr * FL(0.5);
+    MYFLT   nyq = CS_ESR * FL(0.5);
     float   *framein = (float *) p->fin->frame.auxp;
     float   *frameout = (float *) p->fout->frame.auxp;
     int     i = 0, id = (int) framein[3], end = p->numbins * 4;
@@ -616,7 +640,7 @@ static int trshift_process(CSOUND *csound, _PTRANS *p)
 {
     MYFLT   shift = *p->kpar;
     MYFLT   gain = (p->kgain != NULL ? *p->kgain : FL(1.0));
-    MYFLT   nyq = csound->esr * FL(0.5);
+    MYFLT   nyq = CS_ESR * FL(0.5);
     float   *framein = (float *) p->fin->frame.auxp;
     float   *frameout = (float *) p->fout->frame.auxp;
     int     i = 0, id = (int) framein[3], end = p->numbins * 4;
@@ -679,7 +703,7 @@ static int trlowest_init(CSOUND *csound, _PLOW *p)
 static int trlowest_process(CSOUND *csound, _PLOW *p)
 {
     MYFLT   scale = *p->kpar;
-    MYFLT   nyq = csound->esr * FL(0.5);
+    MYFLT   nyq = CS_ESR * FL(0.5);
     float   lowest = (float) nyq, outamp = 0.0f, outph = 0.0f, outid = -1.0f;
     float   *framein = (float *) p->fin->frame.auxp;
     float   *frameout = (float *) p->fout->frame.auxp;
@@ -963,7 +987,7 @@ static int trfil_init(CSOUND *csound, _PSFIL *p)
 static int trfil_process(CSOUND *csound, _PSFIL *p)
 {
     MYFLT   amnt = *p->kpar, gain = FL(1.0);
-    MYFLT   nyq = csound->esr * FL(0.5);
+    MYFLT   nyq = CS_ESR * FL(0.5);
     MYFLT   *fil = p->tab->ftable;
     float   *framein = (float *) p->fin->frame.auxp;
     float   *frameout = (float *) p->fout->frame.auxp;
@@ -981,8 +1005,8 @@ static int trfil_process(CSOUND *csound, _PSFIL *p)
         fr = framein[i + 1];
         if (fr > nyq)
           fr = nyq;
-        if (fr < 0)
-          fr = -fr;
+        //if (fr < 0)
+        fr = FABS(fr);
         pos = fr * len / nyq;
         posi = (int) pos;
         frac = pos - posi;
@@ -1155,8 +1179,8 @@ static int binit_process(CSOUND *csound, _PSBIN *p)
     float   *framein = (float *) p->fsig2->frame.auxp;
     int     i = 0, n = 0, id = (int) framein[3], end = p->numbins * 4;
     int     maxi = -1;
-    MYFLT   bw = csound->esr / (MYFLT)N, boundup, boundown;
-    MYFLT   nyq = csound->esr * FL(0.5), centre;
+    MYFLT   bw = CS_ESR / (MYFLT)N, boundup, boundown;
+    MYFLT   nyq = CS_ESR * FL(0.5), centre;
 
     if (p->lastframe < p->fsig2->framecount) {
 
@@ -1197,40 +1221,40 @@ static int binit_process(CSOUND *csound, _PSBIN *p)
 
 static OENTRY localops[] =
   {
-    {"tradsyn", sizeof(_PSYN), 5, "a", "fkkki", (SUBR) psynth_init, NULL,
+    {"tradsyn", sizeof(_PSYN),0,  5, "a", "fkkki", (SUBR) psynth_init, NULL,
      (SUBR) psynth_process}
     ,
-    {"sinsyn", sizeof(_PSYN2), TR|5, "a", "fkki", (SUBR) psynth2_init, NULL,
+    {"sinsyn", sizeof(_PSYN2), TR, 5, "a", "fkki", (SUBR) psynth2_init, NULL,
      (SUBR) psynth2_process}
     ,
-    {"resyn", sizeof(_PSYN), TR|5, "a", "fkkki", (SUBR) psynth_init, NULL,
+    {"resyn", sizeof(_PSYN), TR, 5, "a", "fkkki", (SUBR) psynth_init, NULL,
      (SUBR) psynth3_process}
     ,
-    {"trscale", sizeof(_PTRANS), 3, "f", "fz", (SUBR) trans_init,
+    {"trscale", sizeof(_PTRANS),0,  3, "f", "fz", (SUBR) trans_init,
      (SUBR) trscale_process}
     ,
-    {"trshift", sizeof(_PTRANS), 3, "f", "fz", (SUBR) trans_init,
+    {"trshift", sizeof(_PTRANS),0,  3, "f", "fz", (SUBR) trans_init,
      (SUBR) trshift_process}
     ,
-    {"trsplit", sizeof(_PSPLIT), 3, "ff", "fz", (SUBR) trsplit_init,
+    {"trsplit", sizeof(_PSPLIT),0,  3, "ff", "fz", (SUBR) trsplit_init,
      (SUBR) trsplit_process}
     ,
-    {"trmix", sizeof(_PSMIX), 3, "f", "ff", (SUBR) trmix_init,
+    {"trmix", sizeof(_PSMIX),0,  3, "f", "ff", (SUBR) trmix_init,
      (SUBR) trmix_process}
     ,
-    {"trlowest", sizeof(_PLOW), 3, "fkk", "fk", (SUBR) trlowest_init,
+    {"trlowest", sizeof(_PLOW),0,  3, "fkk", "fk", (SUBR) trlowest_init,
      (SUBR) trlowest_process}
     ,
-    {"trhighest", sizeof(_PLOW), 3, "fkk", "fk", (SUBR) trlowest_init,
+    {"trhighest", sizeof(_PLOW),0,  3, "fkk", "fk", (SUBR) trlowest_init,
      (SUBR) trhighest_process}
     ,
-    {"trfilter", sizeof(_PSFIL), 3, "f", "fki", (SUBR) trfil_init,
+    {"trfilter", sizeof(_PSFIL),0,  3, "f", "fki", (SUBR) trfil_init,
      (SUBR) trfil_process}
     ,
-    {"trcross", sizeof(_PSCROSS), 3, "f", "ffkz", (SUBR) trcross_init,
+    {"trcross", sizeof(_PSCROSS),0,  3, "f", "ffkz", (SUBR) trcross_init,
      (SUBR) trcross_process}
     ,
-    {"binit", sizeof(_PSBIN), 3, "f", "fi", (SUBR) binit_init,
+    {"binit", sizeof(_PSBIN),0,  3, "f", "fi", (SUBR) binit_init,
      (SUBR) binit_process}
   };
 
