@@ -30,13 +30,13 @@
 #include <math.h>
 
 #define WLN   1         /* time window is WLN*2*ksmps long */
-#define OPWLEN (2*WLN*csound->ksmps)    /* manifest used for final time wdw */
+#define OPWLEN (2*WLN*CS_KSMPS)    /* manifest used for final time wdw */
 
 /************************************************************/
 /*************PVBUFREAD**************************************/
 /************************************************************/
 
-int pvbufreadset(CSOUND *csound, PVBUFREAD *p)
+int pvbufreadset_(CSOUND *csound, PVBUFREAD *p, int stringname)
 {
     char     pvfilnam[MAXNAME];
     PVOCEX_MEMFILE  pp;
@@ -58,7 +58,13 @@ int pvbufreadset(CSOUND *csound, PVBUFREAD *p)
       p->fftBuf = fltp;   /* fltp += PVFFTSIZE; */  /* Not needed */
     }
 
-    csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.", p->XSTRCODE);
+    if (stringname==0){
+      if (ISSTRCOD(*p->ifilno))
+        strncpy(pvfilnam,get_arg_string(csound, *p->ifilno), MAXNAME-1);
+      else csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.",0);
+    }
+    else strncpy(pvfilnam, ((STRINGDAT *)p->ifilno)->data, MAXNAME-1);
+
     if (UNLIKELY(csound->PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0))
       return csound->InitError(csound, Str("PVBUFREAD cannot load %s"),
                                        pvfilnam);
@@ -67,9 +73,9 @@ int pvbufreadset(CSOUND *csound, PVBUFREAD *p)
     frInc    = pp.overlap;
     chans    = pp.chans;
     p->asr   = pp.srate;
-    if (p->asr != csound->esr) {                /* & chk the data */
+    if (p->asr != CS_ESR) {                /* & chk the data */
       csound->Warning(csound, Str("%s's srate = %8.0f, orch's srate = %8.0f"),
-                              pvfilnam, p->asr, csound->esr);
+                              pvfilnam, p->asr, CS_ESR);
     }
     if (p->frSiz > PVFRAMSIZE) {
       return csound->InitError(csound,
@@ -87,8 +93,8 @@ int pvbufreadset(CSOUND *csound, PVBUFREAD *p)
     }
     p->frPtr = (float*) pp.data;
     p->maxFr = pp.nframes - 1;
-    p->frPktim = (MYFLT) csound->ksmps / (MYFLT) frInc;
-    p->frPrtim = csound->esr / (MYFLT) frInc;
+    p->frPktim = (MYFLT) CS_KSMPS / (MYFLT) frInc;
+    p->frPrtim = CS_ESR / (MYFLT) frInc;
     p->prFlg = 1;       /* true */
     /* amplitude scale for PVOC */
  /* p->scale = (MYFLT) pp.fftsize * ((MYFLT) pp.fftsize / (MYFLT) pp.winsize);
@@ -99,13 +105,21 @@ int pvbufreadset(CSOUND *csound, PVBUFREAD *p)
     if (UNLIKELY((OPWLEN / 2 + 1) > PVWINLEN )) {
       return csound->InitError(csound, Str("ksmps of %d needs wdw of %d, "
                                            "max is %d for pv %s"),
-                                       csound->ksmps, (int) (OPWLEN / 2 + 1),
+                                       CS_KSMPS, (int) (OPWLEN / 2 + 1),
                                        (int) PVWINLEN, pvfilnam);
     }
 
     return OK;
 }
 
+int pvbufreadset(CSOUND *csound, PVBUFREAD *p){
+  return pvbufreadset_(csound,p,0);
+}
+
+int pvbufreadset_S(CSOUND *csound, PVBUFREAD *p){
+  return pvbufreadset_(csound,p,1);
+}
+
 int pvbufread(CSOUND *csound, PVBUFREAD *p)
 {
     MYFLT  frIndx;
@@ -126,17 +140,18 @@ int pvbufread(CSOUND *csound, PVBUFREAD *p)
 
     return OK;
  err1:
-    return csound->PerfError(csound, Str("pvbufread: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvbufread: not initialised"));
  err2:
-    return csound->PerfError(csound, Str("PVOC timpnt < 0"));
+    return csound->PerfError(csound, p->h.insdshead, Str("PVOC timpnt < 0"));
 }
 
 /************************************************************/
 /*************PVINTERP**************************************/
 /************************************************************/
-int pvinterpset(CSOUND *csound, PVINTERP *p)
+int pvinterpset_(CSOUND *csound, PVINTERP *p, int stringname)
 {
-    int      i;
+    unsigned int      i;
     char     pvfilnam[MAXNAME];
     PVOCEX_MEMFILE  pp;
     int      frInc, chans; /* THESE SHOULD BE SAVED IN PVOC STRUCT */
@@ -160,7 +175,12 @@ int pvinterpset(CSOUND *csound, PVINTERP *p)
       p->window = fltp;
     }
 
-    csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.", p->XSTRCODE);
+    if (stringname==0){
+      if (ISSTRCOD(*p->ifilno))
+        strncpy(pvfilnam,get_arg_string(csound, *p->ifilno), MAXNAME-1);
+      else csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.",0);
+    }
+    else strncpy(pvfilnam, ((STRINGDAT *)p->ifilno)->data, MAXNAME-1);;
     if (UNLIKELY(csound->PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0))
       return csound->InitError(csound, Str("PVINTERP cannot load %s"),
                                        pvfilnam);
@@ -169,9 +189,9 @@ int pvinterpset(CSOUND *csound, PVINTERP *p)
     frInc    = pp.overlap;
     chans    = pp.chans;
     p->asr   = pp.srate;
-    if (p->asr != csound->esr) {                /* & chk the data */
+    if (p->asr != CS_ESR) {                /* & chk the data */
       csound->Warning(csound, Str("%s's srate = %8.0f, orch's srate = %8.0f"),
-                              pvfilnam, p->asr, csound->esr);
+                              pvfilnam, p->asr, CS_ESR);
     }
     if (UNLIKELY(p->frSiz != p->pvbufread->frSiz)) {
       return csound->InitError(csound,
@@ -188,9 +208,9 @@ int pvinterpset(CSOUND *csound, PVINTERP *p)
     p->baseFr = 0;  /* point to first data frame */
     p->maxFr = pp.nframes - 1;
     /* highest possible frame index */
-    p->frPktim = (MYFLT) csound->ksmps / (MYFLT) frInc;
+    p->frPktim = (MYFLT) CS_KSMPS / (MYFLT) frInc;
     /* factor by which to mult expand phase diffs (ratio of samp spacings) */
-    p->frPrtim = csound->esr / (MYFLT) frInc;
+    p->frPrtim = CS_ESR / (MYFLT) frInc;
     /* factor by which to mulitply 'real' time index to get frame index */
     /* amplitude scale for PVOC */
  /* p->scale = (MYFLT) pp.fftsize * ((MYFLT) pp.fftsize / (MYFLT) pp.winsize);
@@ -209,7 +229,7 @@ int pvinterpset(CSOUND *csound, PVINTERP *p)
     if (UNLIKELY((OPWLEN / 2 + 1) > PVWINLEN)) {
       return csound->InitError(csound, Str("ksmps of %d needs wdw of %d, "
                                            "max is %d for pv %s"),
-                                       csound->ksmps, (OPWLEN / 2 + 1),
+                                       CS_KSMPS, (OPWLEN / 2 + 1),
                                        PVWINLEN, pvfilnam);
     }
     for (i = 0; i < OPWLEN / 2 + 1; ++i)    /* time window is OPWLEN long */
@@ -223,6 +243,15 @@ int pvinterpset(CSOUND *csound, PVINTERP *p)
     return OK;
 }
 
+int pvinterpset(CSOUND *csound, PVINTERP *p){
+  return pvinterpset_(csound,p,0);
+}
+
+int pvinterpset_S(CSOUND *csound, PVINTERP *p){
+  return pvinterpset_(csound,p,1);
+}
+
+
 int pvinterp(CSOUND *csound, PVINTERP *p)
 {
     MYFLT   *ar = p->rslt;
@@ -244,7 +273,8 @@ int pvinterp(CSOUND *csound, PVINTERP *p)
     if (UNLIKELY(outlen>PVFFTSIZE))  /* Maximum transposition down is one octave */
                            /* ..so we won't run into buf2Size problems */
       goto err2;
-    if (UNLIKELY(outlen<2*csound->ksmps)) goto err3;   /* minimum post-squeeze windowlength */
+    if (UNLIKELY(outlen<(int)(2*CS_KSMPS)))
+      goto err3;   /* minimum post-squeeze windowlength */
     buf2Size = OPWLEN;     /* always window to same length after DS */
     if (UNLIKELY((frIndx = *p->ktimpnt * p->frPrtim) < 0)) goto err4;
     if (frIndx > (MYFLT)p->maxFr) { /* not past last one */
@@ -268,7 +298,7 @@ int pvinterp(CSOUND *csound, PVINTERP *p)
       buf[j] = (buf[j] + ((q->buf[j] - buf[j]) * *p->kfreqinterp));
     }
 /*******************************************************************/
-    FrqToPhase(buf, asize, pex * (MYFLT) csound->ksmps, p->asr,
+    FrqToPhase(buf, asize, pex * (MYFLT) CS_KSMPS, p->asr,
                (MYFLT) (0.5 * ((pex / p->lastPex) - 1)));
     /* accumulate phase and wrap to range -PI to PI */
     RewrapPhase(buf, asize, p->lastPhase);
@@ -284,32 +314,35 @@ int pvinterp(CSOUND *csound, PVINTERP *p)
              sizeof(MYFLT) * buf2Size);
     ApplyHalfWin(buf2, p->window, buf2Size);
 
-    addToCircBuf(buf2, p->outBuf, p->opBpos, csound->ksmps, circBufSize);
-    writeClrFromCircBuf(p->outBuf, ar, p->opBpos, csound->ksmps, circBufSize);
-    p->opBpos += csound->ksmps;
+    addToCircBuf(buf2, p->outBuf, p->opBpos, CS_KSMPS, circBufSize);
+    writeClrFromCircBuf(p->outBuf, ar, p->opBpos, CS_KSMPS, circBufSize);
+    p->opBpos += CS_KSMPS;
     if (p->opBpos > circBufSize)
       p->opBpos -= circBufSize;
-    addToCircBuf(buf2 + csound->ksmps, p->outBuf, p->opBpos,
-                 buf2Size - csound->ksmps, circBufSize);
+    addToCircBuf(buf2 + CS_KSMPS, p->outBuf, p->opBpos,
+                 buf2Size - CS_KSMPS, circBufSize);
     p->lastPex = pex;        /* needs to know last pitchexp to update phase */
 
     return OK;
  err1:
-    return csound->PerfError(csound, Str("pvinterp: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvinterp: not initialised"));
  err2:
-    return csound->PerfError(csound, Str("PVOC transpose too low"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("PVOC transpose too low"));
  err3:
-    return csound->PerfError(csound, Str("PVOC transpose too high"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("PVOC transpose too high"));
  err4:
-    return csound->PerfError(csound, Str("PVOC timpnt < 0"));
+    return csound->PerfError(csound, p->h.insdshead, Str("PVOC timpnt < 0"));
 }
 
 /************************************************************/
 /************* PVCROSS **************************************/
 /************************************************************/
-int pvcrossset(CSOUND *csound, PVCROSS *p)
+int pvcrossset_(CSOUND *csound, PVCROSS *p, int stringname)
 {
-    int      i;
+    uint32_t i;
     char     pvfilnam[MAXNAME];
     PVOCEX_MEMFILE  pp;
     int      frInc, chans; /* THESE SHOULD BE SAVED IN PVOC STRUCT */
@@ -332,7 +365,13 @@ int pvcrossset(CSOUND *csound, PVCROSS *p)
       p->outBuf = fltp;      fltp += PVFFTSIZE;
       p->window = fltp;
     }
-    csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.", p->XSTRCODE);
+    if(stringname==0){
+      if (ISSTRCOD(*p->ifilno))
+        strncpy(pvfilnam,get_arg_string(csound, *p->ifilno), MAXNAME-1);
+      else csound->strarg2name(csound, pvfilnam, p->ifilno, "pvoc.",0);
+    }
+    else strncpy(pvfilnam, ((STRINGDAT *)p->ifilno)->data, MAXNAME-1);
+
     if (UNLIKELY(csound->PVOCEX_LoadFile(csound, pvfilnam, &pp) != 0))
       return csound->InitError(csound, Str("PVCROSS cannot load %s"), pvfilnam);
 
@@ -340,9 +379,9 @@ int pvcrossset(CSOUND *csound, PVCROSS *p)
     frInc    = pp.overlap;
     chans    = pp.chans;
     p->asr   = pp.srate;
-    if (p->asr != csound->esr) {                /* & chk the data */
+    if (p->asr != CS_ESR) {                /* & chk the data */
       csound->Warning(csound, Str("%s's srate = %8.0f, orch's srate = %8.0f"),
-                              pvfilnam, p->asr, csound->esr);
+                              pvfilnam, p->asr, CS_ESR);
     }
     if (UNLIKELY(p->frSiz != p->pvbufread->frSiz)) {
       return csound->InitError(csound,
@@ -359,9 +398,9 @@ int pvcrossset(CSOUND *csound, PVCROSS *p)
     p->baseFr = 0;  /* point to first data frame */
     p->maxFr = pp.nframes - 1;
     /* highest possible frame index */
-    p->frPktim = (MYFLT) csound->ksmps / (MYFLT) frInc;
+    p->frPktim = (MYFLT) CS_KSMPS / (MYFLT) frInc;
     /* factor by which to mult expand phase diffs (ratio of samp spacings) */
-    p->frPrtim = csound->esr / (MYFLT) frInc;
+    p->frPrtim = CS_ESR / (MYFLT) frInc;
     /* factor by which to mulitply 'real' time index to get frame index */
     /* amplitude scale for PVOC */
  /* p->scale = (MYFLT) pp.fftsize * ((MYFLT) pp.fftsize / (MYFLT) pp.winsize);
@@ -378,7 +417,7 @@ int pvcrossset(CSOUND *csound, PVCROSS *p)
     if (UNLIKELY((OPWLEN / 2 + 1) > PVWINLEN )) {
       return csound->InitError(csound, Str("ksmps of %d needs wdw of %d, "
                                            "max is %d for pv %s"),
-                                       csound->ksmps, (OPWLEN / 2 + 1),
+                                       CS_KSMPS, (OPWLEN / 2 + 1),
                                        PVWINLEN, pvfilnam);
     }
     for (i = 0; i < OPWLEN / 2 + 1; ++i)    /* time window is OPWLEN long */
@@ -393,6 +432,14 @@ int pvcrossset(CSOUND *csound, PVCROSS *p)
     return OK;
 }
 
+int pvcrossset(CSOUND *csound, PVCROSS *p){
+  return pvcrossset_(csound,p,0);
+}
+
+int pvcrossset_S(CSOUND *csound, PVCROSS *p) {
+return pvcrossset_(csound,p,1);
+}
+
 int pvcross(CSOUND *csound, PVCROSS *p)
 {
     MYFLT   *ar = p->rslt;
@@ -414,10 +461,10 @@ int pvcross(CSOUND *csound, PVCROSS *p)
     pex = *p->kfmod;
     outlen = (int) (((MYFLT) size) / pex);
     /* use outlen to check window/krate/transpose combinations */
-    if (UNLIKELY(outlen>PVFFTSIZE))   /* Maximum transposition down is one octave */
-                            /* ..so we won't run into buf2Size problems */
+    if (UNLIKELY(outlen>PVFFTSIZE)) /* Maximum transposition down is one octave */
+                                    /* ..so we won't run into buf2Size problems */
       goto err2;
-    if (UNLIKELY(outlen<2*csound->ksmps))   /* minimum post-squeeze windowlength */
+    if (UNLIKELY(outlen<(int)(2*CS_KSMPS))) /* minimum post-squeeze windowlength */
       goto err3;
     buf2Size = OPWLEN;     /* always window to same length after DS */
     if ((frIndx = *p->ktimpnt * p->frPrtim) < 0) goto err4;
@@ -438,7 +485,7 @@ int pvcross(CSOUND *csound, PVCROSS *p)
       buf[i] = ((buf[i] * ampscale2) + (q->buf[i] * ampscale1)) * scaleFac;
 /***************************************************/
 
-    FrqToPhase(buf, asize, pex * (MYFLT) csound->ksmps, p->asr,
+    FrqToPhase(buf, asize, pex * (MYFLT) CS_KSMPS, p->asr,
                (MYFLT) (0.5 * ((pex / p->lastPex) - 1)));
     /* accumulate phase and wrap to range -PI to PI */
     RewrapPhase(buf, asize, p->lastPhase);
@@ -451,7 +498,7 @@ int pvcross(CSOUND *csound, PVCROSS *p)
         csound->Message(csound, Str("PVOC debug: one frame gets through\n"));
 #endif
       if (specwp > 0)
-        PreWarpSpec(p->pp, buf, asize, pex, (MYFLT *)p->memenv.auxp);
+        PreWarpSpec(buf, asize, pex, (MYFLT *)p->memenv.auxp);
 
       Polar2Real_PVOC(csound, buf, (int) size);
 
@@ -469,23 +516,26 @@ int pvcross(CSOUND *csound, PVCROSS *p)
       memset(buf2, 0, buf2Size*sizeof(MYFLT));
     }
 
-    addToCircBuf(buf2, p->outBuf, p->opBpos, csound->ksmps, circBufSize);
-    writeClrFromCircBuf(p->outBuf, ar, p->opBpos, csound->ksmps, circBufSize);
-    p->opBpos += csound->ksmps;
+    addToCircBuf(buf2, p->outBuf, p->opBpos, CS_KSMPS, circBufSize);
+    writeClrFromCircBuf(p->outBuf, ar, p->opBpos, CS_KSMPS, circBufSize);
+    p->opBpos += CS_KSMPS;
     if (p->opBpos > circBufSize)
       p->opBpos -= circBufSize;
-    addToCircBuf(buf2 + csound->ksmps, p->outBuf, p->opBpos,
-                 buf2Size - csound->ksmps, circBufSize);
+    addToCircBuf(buf2 + CS_KSMPS, p->outBuf, p->opBpos,
+                 buf2Size - CS_KSMPS, circBufSize);
     p->lastPex = pex;       /* needs to know last pitchexp to update phase */
 
     return OK;
  err1:
-    return csound->PerfError(csound, Str("pvcross: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvcross: not initialised"));
  err2:
-    return csound->PerfError(csound, Str("PVOC transpose too low"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("PVOC transpose too low"));
  err3:
-    return csound->PerfError(csound, Str("PVOC transpose too high"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("PVOC transpose too high"));
  err4:
-    return csound->PerfError(csound, Str("PVOC timpnt < 0"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("PVOC timpnt < 0"));
 }
-
