@@ -22,7 +22,7 @@
     02111-1307 USA
 */
 
-#include "csdl.h"
+#include "pvs_ops.h"
 #include "pstream.h"
 
 typedef struct {
@@ -47,9 +47,9 @@ static int pvsbandinit(CSOUND *csound, PVSBAND *p)
 
     if (p->fin->sliding) {
       if (p->fout->frame.auxp==NULL ||
-          csound->ksmps*(N+2)*sizeof(MYFLT) > (unsigned int)p->fout->frame.size)
-        csound->AuxAlloc(csound, csound->ksmps*(N+2)*sizeof(MYFLT),&p->fout->frame);
-      else memset(p->fout->frame.auxp, 0, csound->ksmps*(N+2)*sizeof(MYFLT));
+          CS_KSMPS*(N+2)*sizeof(MYFLT) > (unsigned int)p->fout->frame.size)
+        csound->AuxAlloc(csound, CS_KSMPS*(N+2)*sizeof(MYFLT),&p->fout->frame);
+      else memset(p->fout->frame.auxp, 0, CS_KSMPS*(N+2)*sizeof(MYFLT));
     }
     else
       {
@@ -89,10 +89,13 @@ static int pvsband(CSOUND *csound, PVSBAND *p)
     if (higbnd<lowbnd) higbnd = lowbnd;
     if (higcut<higbnd) higcut = higbnd;
     if (p->fin->sliding) {
-      int n, nsmps = csound->ksmps;
+      uint32_t offset = p->h.insdshead->ksmps_offset;
+      uint32_t early  = p->h.insdshead->ksmps_no_end;
+      uint32_t n, nsmps = CS_KSMPS;
       int NB  = p->fout->NB;
 
-      for (n=0; n<nsmps; n++) {
+      if (UNLIKELY(early)) nsmps -= early;
+      for (n=offset; n<nsmps; n++) {
         int change = 0;
         CMPLX *fin = (CMPLX *) p->fin->frame.auxp + n*NB;
         CMPLX *fout = (CMPLX *) p->fout->frame.auxp + n*NB;
@@ -171,7 +174,8 @@ static int pvsband(CSOUND *csound, PVSBAND *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("pvsband: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvsband: not initialised"));
 }
 
 static int pvsbrej(CSOUND *csound, PVSBAND *p)
@@ -193,10 +197,13 @@ static int pvsbrej(CSOUND *csound, PVSBAND *p)
     if (higbnd<lowbnd) higbnd = lowbnd;
     if (higcut<higbnd) higcut = higbnd;
     if (p->fin->sliding) {
-      int n, nsmps = csound->ksmps;
+      uint32_t offset = p->h.insdshead->ksmps_offset;
+      uint32_t early  = p->h.insdshead->ksmps_no_end;
+      uint32_t n, nsmps = CS_KSMPS;
       int NB  = p->fout->NB;
 
-      for (n=0; n<nsmps; n++) {
+      if (UNLIKELY(early)) nsmps -= early;
+      for (n=offset; n<nsmps; n++) {
         int change = 0;
         CMPLX *fin = (CMPLX *) p->fin->frame.auxp + n*NB;
         CMPLX *fout = (CMPLX *) p->fout->frame.auxp + n*NB;
@@ -273,12 +280,15 @@ static int pvsbrej(CSOUND *csound, PVSBAND *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("pvsband: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvsband: not initialised"));
 }
 
 static OENTRY localops[] = {
-  {"pvsbandp", sizeof(PVSBAND), 3, "f", "fxxxxO", (SUBR) pvsbandinit, (SUBR) pvsband, (SUBR) NULL },
-  {"pvsbandr", sizeof(PVSBAND), 3, "f", "fxxxxO", (SUBR) pvsbandinit, (SUBR) pvsbrej, (SUBR) NULL }
+  {"pvsbandp", sizeof(PVSBAND), 0, 3, "f", "fxxxxO",
+                    (SUBR) pvsbandinit, (SUBR) pvsband, (SUBR) NULL },
+  {"pvsbandr", sizeof(PVSBAND), 0, 3, "f", "fxxxxO",
+                    (SUBR) pvsbandinit, (SUBR) pvsbrej, (SUBR) NULL }
 };
 
 int pvsband_init_(CSOUND *csound)
@@ -286,5 +296,3 @@ int pvsband_init_(CSOUND *csound)
   return csound->AppendOpcodes(csound, &(localops[0]),
                                (int) (sizeof(localops) / sizeof(OENTRY)));
 }
-
-
