@@ -19,7 +19,10 @@
     02111-1307 USA
 */
 
-#include "csdl.h"
+// #include "csdl.h"
+#include "csoundCore.h"
+#include "interlocks.h"
+
 #include "pstream.h"
 
 typedef struct {
@@ -60,11 +63,11 @@ static int pvsbufferset(CSOUND *csound, PVSBUFFER *p)
     p->handle->header.wintype = p->fin->wintype;
     p->handle->header.format  = p->fin->format;
     p->handle->header.framecount = p->fin->framecount;
-    p->nframes = p->handle->frames = (*p->len) * csound->esr/hop;
+    p->nframes = p->handle->frames = (*p->len) * CS_ESR/hop;
     if (p->buffer.auxp == NULL ||
         p->buffer.size < sizeof(float) * (N + 2) * p->nframes)
       csound->AuxAlloc(csound, (N + 2) * sizeof(float) * p->nframes, &p->buffer);
-    /*else*/
+    else
       memset(p->buffer.auxp, 0, (N + 2) * sizeof(float) * p->nframes);
 
     p->handle->header.frame.auxp = p->buffer.auxp;
@@ -82,7 +85,8 @@ static int pvsbufferset(CSOUND *csound, PVSBUFFER *p)
      phandle = (FSIG_HANDLE **) csound->QueryGlobalVariable(csound,varname);
      /*csound->Message(csound, "%p -> %p \n", p->handle, phandle); */
     if (phandle == NULL)
-      csound->InitError(csound, Str("error... could not create global var for handle\n"));
+      csound->InitError(csound,
+                        Str("error... could not create global var for handle\n"));
     else
       *phandle = p->handle;
      }
@@ -107,7 +111,7 @@ static int pvsbufferproc(CSOUND *csound, PVSBUFFER *p)
         fout[i+1] = fin[i+1];
       }
       p->handle->header.framecount = p->lastframe = p->fin->framecount;
-      p->pos = p->cframes/(csound->esr/p->fin->overlap);
+      p->pos = p->cframes/(CS_ESR/p->fin->overlap);
       p->cframes++;
       if (p->cframes == p->nframes)p->cframes = 0;
     }
@@ -127,20 +131,22 @@ typedef struct {
   MYFLT *clear;
   MYFLT iclear, optr;
   FSIG_HANDLE *handle;
-  int scnt;
+  unsigned int scnt;
 } PVSBUFFERREAD;
 
 static int pvsbufreadset(CSOUND *csound, PVSBUFFERREAD *p)
 {
     int N;
-    FSIG_HANDLE *handle, **phandle;
+    FSIG_HANDLE *handle=NULL, **phandle;
     char varname[32];
 
     sprintf(varname, "::buffer%d", (int)(*p->hptr));
     /* csound->Message(csound, "%s:\n", varname); */
     phandle = (FSIG_HANDLE **) csound->QueryGlobalVariable(csound,varname);
     if (phandle == NULL)
-      csound->InitError(csound, Str("error... could not read handle from global variable\n"));
+      csound->InitError(csound,
+                        Str("error... could not read handle from "
+                            "global variable\n"));
     else
       handle = *phandle;
     p->optr = *p->hptr;
@@ -175,11 +181,11 @@ static int pvsbufreadset(CSOUND *csound, PVSBUFFERREAD *p)
  static int pvsbufreadproc(CSOUND *csound, PVSBUFFERREAD *p){
 
     unsigned int posi, frames;
-    MYFLT pos, sr = csound->esr;
+    MYFLT pos, sr = CS_ESR, frac;
     FSIG_HANDLE *handle =  p->handle, **phandle;
-    MYFLT frac;
     float *fout, *buffer;
-    int strt = *p->strt, end = *p->end, overlap, i, N;
+    int strt = *p->strt, end = *p->end, i, N;
+    unsigned int overlap;
     p->iclear = *p->clear;
 
    if (*p->hptr != p->optr) {
@@ -187,7 +193,9 @@ static int pvsbufreadset(CSOUND *csound, PVSBUFFERREAD *p)
      sprintf(varname, "::buffer%d", (int)(*p->hptr));
      phandle = (FSIG_HANDLE **) csound->QueryGlobalVariable(csound,varname);
      if (phandle == NULL)
-       csound->PerfError(csound, Str("error... could not read handle from global variable\n"));
+       csound->PerfError(csound, p->h.insdshead,
+                         Str("error... could not read handle "
+                             "from global variable\n"));
      else
        handle = *phandle;
    }
@@ -198,7 +206,7 @@ static int pvsbufreadset(CSOUND *csound, PVSBUFFERREAD *p)
      buffer = handle->data;
    N = p->fout->N;
    overlap = p->fout->overlap;
-   
+
    if (p->scnt >= overlap){
      float *frame1, *frame2;
      strt /= (sr/N);
@@ -207,13 +215,13 @@ static int pvsbufreadset(CSOUND *csound, PVSBUFFERREAD *p)
      end = (int)(end <= strt ? N/2 + 2 : end > N/2 + 2 ? N/2 + 2 : end);
      frames = handle->frames-1;
      pos = *p->ktime*(sr/overlap);
-     
+
      if (p->iclear) memset(fout, 0, sizeof(float)*(N+2));
      while (pos >= frames) pos -= frames;
      while (pos < 0) pos += frames;
      posi = (int) pos;
      if (N == handle->header.N &&
-         overlap == handle->header.overlap){
+         overlap == (unsigned int)handle->header.overlap){
        frame1 = buffer + (N + 2) * posi;
        frame2 = buffer + (N + 2)*(posi != frames-1 ? posi+1 : 0);
        frac = pos - posi;
@@ -231,30 +239,32 @@ static int pvsbufreadset(CSOUND *csound, PVSBUFFERREAD *p)
      p->scnt -= overlap;
      p->fout->framecount++;
    }
-   p->scnt += csound->ksmps;
+   p->scnt += CS_KSMPS;
 
    return OK;
  err1:
-    return csound->PerfError(csound, Str("Invalid buffer handle"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("Invalid buffer handle"));
   }
 
 
 static int pvsbufreadproc2(CSOUND *csound, PVSBUFFERREAD *p)
 {
     unsigned int posi, frames;
-    MYFLT pos, sr = csound->esr;
+    MYFLT pos, sr = CS_ESR;
     FSIG_HANDLE *handle =  p->handle, **phandle;
-    MYFLT frac, *tab1, *tab2, *tab;
-    FUNC *ftab;
-    float *fout, *buffer;
-    int overlap, i, N;
+    MYFLT    frac, *tab1, *tab2, *tab;
+    FUNC     *ftab;
+    float    *fout, *buffer;
+    uint32_t overlap, i;
+    int      N;
 
     if (*p->hptr != p->optr){
       char varname[32];
       sprintf(varname, "::buffer%d", (int)(*p->hptr));
       phandle = (FSIG_HANDLE **) csound->QueryGlobalVariable(csound,varname);
       if (phandle == NULL)
-        csound->PerfError(csound,
+        csound->PerfError(csound, p->h.insdshead,
                           Str("error... could not read handle from "
                               "global variable\n"));
       else
@@ -266,44 +276,47 @@ static int pvsbufreadproc2(CSOUND *csound, PVSBUFFERREAD *p)
     buffer = handle->data;
     N = p->fout->N;
     overlap = p->fout->overlap;
-   
-    
+
+
     if (p->scnt >= overlap) {
       float *frame1, *frame2;
       frames = handle->frames-1;
-      ftab = csound->FTFind(csound, p->strt);
-      if (ftab->flen < N/2+1) 
-        csound->PerfError(csound, "table length too small: needed %d, got %d\n",
+      ftab = csound->FTnp2Find(csound, p->strt);
+      if (UNLIKELY((int)ftab->flen < N/2+1))
+        csound->PerfError(csound, p->h.insdshead,
+                          Str("table length too small: needed %d, got %d\n"),
                           N/2+1, ftab->flen);
       tab = tab1 = ftab->ftable;
-      ftab = csound->FTFind(csound, p->end);
-      if (ftab->flen < N/2+1) 
-         csound->PerfError(csound, "table length too small: needed %d, got %d\n",
-                                   N/2+1, ftab->flen);
+      ftab = csound->FTnp2Find(csound, p->end);
+      if (UNLIKELY((int)ftab->flen < N/2+1))
+        csound->PerfError(csound, p->h.insdshead,
+                          Str("table length too small: needed %d, got %d\n"),
+                          N/2+1, ftab->flen);
       tab2 = ftab->ftable;
-      for (i=0; i < N+2; i++){
+      for (i=0; i < (unsigned int)N+2; i++){
         pos = (*p->ktime - tab[i])*(sr/overlap);
         while(pos >= frames) pos -= frames;
         while(pos < 0) pos += frames;
         posi = (int) pos;
         if (N == handle->header.N &&
-           overlap == handle->header.overlap){
+            overlap == (unsigned int)handle->header.overlap) {
            frame1 = buffer + (N + 2) * posi;
            frame2 = buffer + (N + 2)*(posi != frames-1 ? posi+1 : 0);
            frac = pos - posi;
            fout[i] = frame1[i] + frac*(frame2[i] - frame1[i]);
         } else
-	  fout[i] = 0.0f;
+          fout[i] = 0.0f;
         if (tab == tab1) tab = tab2;
-	  else tab = tab1;
+          else tab = tab1;
       }
       p->scnt -= overlap;
       p->fout->framecount++;
     }
-    p->scnt += csound->ksmps;
+    p->scnt += CS_KSMPS;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("Invalid buffer handle"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("Invalid buffer handle"));
   }
 
 
@@ -311,10 +324,15 @@ static int pvsbufreadproc2(CSOUND *csound, PVSBUFFERREAD *p)
 
 #define S(x)    sizeof(x)
 
-static OENTRY localops[] = {
-  {"pvsbuffer", S(PVSBUFFER), 3, "ik", "fi", (SUBR)pvsbufferset, (SUBR)pvsbufferproc, NULL},
-  {"pvsbufread", S(PVSBUFFERREAD), 3, "f", "kkOOo", (SUBR)pvsbufreadset, (SUBR)pvsbufreadproc, NULL},
-  {"pvsbufread2", S(PVSBUFFERREAD), 3, "f", "kkkk", (SUBR)pvsbufreadset, (SUBR)pvsbufreadproc2, NULL}
+/* static */
+static OENTRY pvsbuffer_localops[] = {
+  {"pvsbuffer", S(PVSBUFFER), 0, 3, "ik", "fi",
+   (SUBR)pvsbufferset, (SUBR)pvsbufferproc, NULL},
+  {"pvsbufread", S(PVSBUFFERREAD), 0, 3, "f", "kkOOo",
+   (SUBR)pvsbufreadset, (SUBR)pvsbufreadproc, NULL},
+  {"pvsbufread2", S(PVSBUFFERREAD), 0, 3, "f", "kkkk",
+   (SUBR)pvsbufreadset, (SUBR)pvsbufreadproc2, NULL}
 };
 
-LINKAGE
+LINKAGE_BUILTIN(pvsbuffer_localops)
+/* LINKAGE */
