@@ -22,7 +22,7 @@
     02111-1307 USA
 */
 
-#include "csdl.h"
+#include "csoundCore.h"
 #include "pstream.h"
 
 typedef struct {
@@ -31,7 +31,7 @@ typedef struct {
     PVSDAT *fin;
     MYFLT  *kmrate;
     MYFLT  *kfrate;
-    int    lastframe;
+    unsigned int lastframe;
 } PVSGENDY;
 
 
@@ -40,13 +40,13 @@ static int pvsgendyinit(CSOUND *csound, PVSGENDY *p)
     int     N = p->fin->N;
 
     if (UNLIKELY(p->fin == p->fout))
-      csound->Warning(csound, "Unsafe to have same fsig as in and out");
+      csound->Warning(csound, Str("Unsafe to have same fsig as in and out"));
 
     if (p->fin->sliding) {
       if (p->fout->frame.auxp==NULL ||
-          csound->ksmps*(N+2)*sizeof(MYFLT) > (unsigned int)p->fout->frame.size)
-        csound->AuxAlloc(csound, csound->ksmps*(N+2)*sizeof(MYFLT),&p->fout->frame);
-      else memset(p->fout->frame.auxp, 0, csound->ksmps*(N+2)*sizeof(MYFLT));
+          CS_KSMPS*(N+2)*sizeof(MYFLT) > (unsigned int)p->fout->frame.size)
+        csound->AuxAlloc(csound, CS_KSMPS*(N+2)*sizeof(MYFLT),&p->fout->frame);
+      else memset(p->fout->frame.auxp, 0, CS_KSMPS*(N+2)*sizeof(MYFLT));
     }
     else
       {
@@ -72,24 +72,29 @@ static int pvsgendy(CSOUND *csound, PVSGENDY *p)
     int     i, N = p->fin->N;
     MYFLT   mrate = *p->kmrate;
     MYFLT   frate = *p->kfrate;
-    float   *fin = (float *) p->fin->frame.auxp;
-    float   *fout = (float *) p->fout->frame.auxp;
+    float   *finf = (float *) p->fin->frame.auxp;
+    float   *foutf = (float *) p->fout->frame.auxp;
 
-    if (UNLIKELY(fout == NULL)) goto err1;
+    if (UNLIKELY(foutf == NULL)) goto err1;
 
     if (p->fin->sliding) {
-      int n, nsmps = csound->ksmps;
+      uint32_t offset = p->h.insdshead->ksmps_offset;
+      uint32_t early  = p->h.insdshead->ksmps_no_end;
+      uint32_t n, nsmps = CS_KSMPS;
       int NB  = p->fout->NB;
-
-      for (n=0; n<nsmps; n++) {
-        int change = 0;
+      for (n=0; n<offset; n+=2) foutf[n] = foutf[n+1] = FL(0.0);
+      for (n=nsmps-early; n<nsmps; n+=2) foutf[n] = foutf[n+1] = FL(0.0);
+      nsmps -= early;
+      for (n=offset; n<nsmps; n++) {
+        //int change = 0;
         CMPLX *fin = (CMPLX *) p->fin->frame.auxp + n*NB;
         CMPLX *fout = (CMPLX *) p->fout->frame.auxp + n*NB;
         for (i = 0; i < NB-1; i++) {
           MYFLT x = (MYFLT)(rand()-RAND_MAX/2)/(MYFLT)RAND_MAX;
           //          printf("%f\n", x);
           fout[i].re = fin[i].re + mrate * x;
-          fout[i].im = fin[i].im + frate * (MYFLT)(rand()-RAND_MAX/2)/(MYFLT)RAND_MAX/(MYFLT)(i+1);
+          fout[i].im = fin[i].im +
+            frate * (MYFLT)(rand()-RAND_MAX/2)/(MYFLT)RAND_MAX/(MYFLT)(i+1);
         }
       }
       return OK;
@@ -97,22 +102,23 @@ static int pvsgendy(CSOUND *csound, PVSGENDY *p)
     if (p->lastframe < p->fin->framecount) {
       for (i = 0; i < N; i += 2) {
         MYFLT x = frate * (MYFLT)(rand()-RAND_MAX/2)/(MYFLT)RAND_MAX/(MYFLT)(i+1);
-        fout[i+1] = fin[i+1] + x;
-        fout[i] = fin[i] + mrate * (MYFLT)(rand()-RAND_MAX/2)/(MYFLT)RAND_MAX;
+        foutf[i+1] = finf[i+1] + x;
+        foutf[i] = finf[i] + mrate * (MYFLT)(rand()-RAND_MAX/2)/(MYFLT)RAND_MAX;
       }
       p->fout->framecount = p->lastframe = p->fin->framecount;
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("pvsgendy: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvsgendy: not initialised"));
 }
 
 static OENTRY pvsgendy_localops[] = {
-  { "pvsgendy", sizeof(PVSGENDY), 3, "f", "fkk",
+  { "pvsgendy", sizeof(PVSGENDY), 0, 3, "f", "fkk",
                 (SUBR) pvsgendyinit, (SUBR) pvsgendy, (SUBR) NULL }
 };
 
-LINKAGE1(pvsgendy_localops)
+LINKAGE_BUILTIN(pvsgendy_localops)
 
 
 
