@@ -3,8 +3,13 @@ static int pyexec_krate(CSOUND *csound, PYEXEC *p)
 {
     char      source[1024];
     PyObject  *result;
+    int *py_initialize_done;
+    if ((py_initialize_done =
+         csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+        *py_initialize_done == 0)
+      return NOTOK;
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = exec_file_in_given_context(csound, source, 0);
     if (result == NULL) {
@@ -19,20 +24,30 @@ static int pyexeci_irate(CSOUND *csound, PYEXEC *p)
 {
     char      source[1024];
     PyObject  *result;
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = exec_file_in_given_context(csound, source, 0);
     if (result == NULL) {
       return pyErrMsg(p, "python exception");
     }
     Py_DECREF(result);
-    return OK;
+  return OK;
 }
 #endif
 
 static int pylexec_irate(CSOUND *csound, PYEXEC *p)
 {
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     create_private_namespace_if_needed(&p->h);
     return OK;
 }
@@ -41,8 +56,13 @@ static int pylexec_krate(CSOUND *csound, PYEXEC *p)
 {
     char      source[1024];
     PyObject  *result;
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = exec_file_in_given_context(csound, source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -56,10 +76,15 @@ static int pylexeci_irate(CSOUND *csound, PYEXEC *p)
 {
     char      source[1024];
     PyObject  *result;
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
 
     create_private_namespace_if_needed(&p->h);
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = exec_file_in_given_context(csound, source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -73,11 +98,16 @@ static int pyexect_krate(CSOUND *csound, PYEXECT *p)
 {
     char      source[1024];
     PyObject  *result;
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
 
     if (!*p->trigger)
       return OK;
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = exec_file_in_given_context(csound, source, 0);
     if (result == NULL) {
@@ -89,6 +119,11 @@ static int pyexect_krate(CSOUND *csound, PYEXECT *p)
 
 static int pylexect_irate(CSOUND *csound, PYEXECT *p)
 {
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     create_private_namespace_if_needed(&p->h);
     return OK;
 }
@@ -97,11 +132,16 @@ static int pylexect_krate(CSOUND *csound, PYEXECT *p)
 {
     char      source[1024];
     PyObject  *result;
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
 
     if (!*p->trigger)
       return OK;
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = exec_file_in_given_context(csound, source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -115,8 +155,13 @@ static int pyrun_krate(CSOUND *csound, PYRUN *p)
 {
     char      source[40960];
     PyObject  *result;
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = run_statement_in_given_context(source, 0);
     if (result == NULL) {
@@ -130,8 +175,13 @@ static int pyruni_irate(CSOUND *csound, PYRUN *p)
 {
     char      source[40960];
     PyObject  *result;
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = run_statement_in_given_context(source, 0);
     if (result == NULL) {
@@ -143,6 +193,11 @@ static int pyruni_irate(CSOUND *csound, PYRUN *p)
 
 static int pylrun_irate(CSOUND *csound, PYRUN *p)
 {
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     create_private_namespace_if_needed(&p->h);
     return OK;
 }
@@ -151,8 +206,13 @@ static int pylrun_krate(CSOUND *csound, PYRUN *p)
 {
     char      source[40960];
     PyObject  *result;
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = run_statement_in_given_context(source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -166,10 +226,14 @@ static int pylruni_irate(CSOUND *csound, PYRUN *p)
 {
     char      source[40960];
     PyObject  *result;
-
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     create_private_namespace_if_needed(&p->h);
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = run_statement_in_given_context(source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -183,11 +247,15 @@ static int pyrunt_krate(CSOUND *csound, PYRUNT *p)
 {
     char      source[40960];
     PyObject  *result;
-
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     if (!*p->trigger)
       return OK;
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = run_statement_in_given_context(source, 0);
     if (result == NULL) {
@@ -199,6 +267,11 @@ static int pyrunt_krate(CSOUND *csound, PYRUNT *p)
 
 static int pylrunt_irate(CSOUND *csound, PYRUNT *p)
 {
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     create_private_namespace_if_needed(&p->h);
     return OK;
 }
@@ -207,11 +280,15 @@ static int pylrunt_krate(CSOUND *csound, PYRUNT *p)
 {
     char      source[40960];
     PyObject  *result;
-
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     if (!*p->trigger)
       return OK;
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = run_statement_in_given_context(source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -225,8 +302,12 @@ static int pyeval_krate(CSOUND *csound, PYEVAL *p)
 {
     char      source[1024];
     PyObject  *result;
-
-    strcpy(source, (char*) p->string);
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
+    strcpy(source, (char*) p->string->data);
 
     result = eval_string_in_given_context(source, 0);
     if (result == NULL) {
@@ -247,8 +328,12 @@ static int pyevali_irate(CSOUND *csound, PYEVAL *p)
 {
     char      source[1024];
     PyObject  *result;
-
-    strcpy(source, (char*) p->string);
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
+    strcpy(source, (char*) p->string->data);
 
     result = eval_string_in_given_context(source, 0);
     if (result == NULL) {
@@ -267,6 +352,11 @@ static int pyevali_irate(CSOUND *csound, PYEVAL *p)
 
 static int pyleval_irate(CSOUND *csound, PYEVAL *p)
 {
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     create_private_namespace_if_needed(&p->h);
     return OK;
 }
@@ -275,8 +365,12 @@ static int pyleval_krate(CSOUND *csound, PYEVAL *p)
 {
     char      source[1024];
     PyObject  *result;
-
-    strcpy(source, (char*) p->string);
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
+    strcpy(source, (char*) p->string->data);
 
     result = eval_string_in_given_context(source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -296,10 +390,14 @@ static int pylevali_irate(CSOUND *csound, PYEVAL *p)
 {
     char      source[1024];
     PyObject  *result;
-
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     create_private_namespace_if_needed(&p->h);
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = eval_string_in_given_context(source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -319,13 +417,17 @@ static int pyevalt_krate(CSOUND *csound, PYEVALT *p)
 {
     char      source[1024];
     PyObject  *result;
-
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     if (!*p->trigger) {
       *p->result = p->oresult;
       return OK;
     }
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = eval_string_in_given_context(source, 0);
     if (result == NULL) {
@@ -344,6 +446,11 @@ static int pyevalt_krate(CSOUND *csound, PYEVALT *p)
 
 static int pylevalt_irate(CSOUND *csound, PYEVALT *p)
 {
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     create_private_namespace_if_needed(&p->h);
     return OK;
 }
@@ -352,13 +459,17 @@ static int pylevalt_krate(CSOUND *csound, PYEVALT *p)
 {
     char      source[1024];
     PyObject  *result;
-
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     if (!*p->trigger) {
       *p->result = p->oresult;
       return OK;
     }
 
-    strcpy(source, (char*) p->string);
+    strcpy(source, (char*) p->string->data);
 
     result = eval_string_in_given_context(source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -379,8 +490,12 @@ static int pyassign_krate(CSOUND *csound, PYASSIGN *p)
 {
     char      source[1024];
     PyObject  *result;
-
-    sprintf(source, "%s = %f", (char*) p->string, *p->value);
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
+    sprintf(source, "%s = %f", (char*) p->string->data, *p->value);
 
     result = run_statement_in_given_context(source, 0);
     if (result == NULL) {
@@ -395,8 +510,12 @@ static int pyassigni_irate(CSOUND *csound, PYASSIGN *p)
 {
     char      source[1024];
     PyObject  *result;
-
-    sprintf(source, "%s = %f", (char*) p->string, *p->value);
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
+    sprintf(source, "%s = %f", (char*) p->string->data, *p->value);
 
     result = run_statement_in_given_context(source, 0);
     if (result == NULL) {
@@ -409,6 +528,11 @@ static int pyassigni_irate(CSOUND *csound, PYASSIGN *p)
 
 static int pylassign_irate(CSOUND *csound, PYASSIGN *p)
 {
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     create_private_namespace_if_needed(&p->h);
     return OK;
 }
@@ -417,8 +541,12 @@ static int pylassign_krate(CSOUND *csound, PYASSIGN *p)
 {
     char      source[1024];
     PyObject  *result;
-
-    sprintf(source, "%s = %f", (char*) p->string, *p->value);
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
+    sprintf(source, "%s = %f", (char*) p->string->data, *p->value);
 
     result = run_statement_in_given_context(source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -432,10 +560,14 @@ static int pylassigni_irate(CSOUND *csound, PYASSIGN *p)
 {
     char      source[1024];
     PyObject  *result;
-
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     create_private_namespace_if_needed(&p->h);
 
-    sprintf(source, "%s = %f", (char*) p->string, *p->value);
+    sprintf(source, "%s = %f", (char*) p->string->data, *p->value);
 
     result = run_statement_in_given_context(source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -449,11 +581,15 @@ static int pyassignt_krate(CSOUND *csound, PYASSIGNT *p)
 {
     char      source[1024];
     PyObject  *result;
-
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     if (!*p->trigger)
       return OK;
 
-    sprintf(source, "%s = %f", (char*) p->string, *p->value);
+    sprintf(source, "%s = %f", (char*) p->string->data, *p->value);
 
     result = run_statement_in_given_context(source, 0);
     if (result == NULL) {
@@ -465,6 +601,11 @@ static int pyassignt_krate(CSOUND *csound, PYASSIGNT *p)
 
 static int pylassignt_irate(CSOUND *csound, PYASSIGNT *p)
 {
+    int *py_initialize_done;
+    if((py_initialize_done =
+        csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+       *py_initialize_done == 0)
+      return NOTOK;
     create_private_namespace_if_needed(&p->h);
     return OK;
 }
@@ -473,11 +614,15 @@ static int pylassignt_krate(CSOUND *csound, PYASSIGNT *p)
 {
     char      source[1024];
     PyObject  *result;
-
+    int *py_initialize_done;
+    if ((py_initialize_done =
+         csound->QueryGlobalVariable(csound,"PY_INITIALIZE")) == NULL ||
+        *py_initialize_done == 0)
+      return NOTOK;
     if (!*p->trigger)
       return OK;
 
-    sprintf(source, "%s = %f", (char*) p->string, *p->value);
+    sprintf(source, "%s = %f", (char*) p->string->data, *p->value);
 
     result = run_statement_in_given_context(source, GETPYLOCAL(p->h.insdshead));
     if (result == NULL) {
@@ -486,4 +631,3 @@ static int pylassignt_krate(CSOUND *csound, PYASSIGNT *p)
     Py_DECREF(result);
     return OK;
 }
-
