@@ -1,13 +1,14 @@
 /*****************************************************
- 
-			CSOUND SERIAL PORT OPCODES
-			  ma++ ingalls, 2011/9/4
+251
+
+                        CSOUND SERIAL PORT OPCODES
+                          ma++ ingalls, 2011/9/4
                      modified for WIndows John ffitch
  * based on "Arduino-serial"
  * Copyright (c) 2006, Tod E. Kurt, tod@todbot.com
  * http://todbot.com/blog/
 
-    Copyright (C) 2011 matt ingalls 
+    Copyright (C) 2011 matt ingalls
     based on "Arduino-serial", Copyright (c) 2006, Tod E. Kurt, tod@todbot.com
     http://todbot.com/blog/ and licenced LGPL to csound
 
@@ -28,8 +29,8 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
     02111-1307 USA
 */
- 
-#include <stdlib.h> 
+
+#include <stdlib.h>
 #include <stdint.h>   /* Standard types */
 #include <string.h>   /* String function definitions */
 
@@ -53,44 +54,44 @@
 /*****************************************************
 
 open a port.  baudRate defaults to 9600
-	iPort  serialBegin         SPortName [, baudRate ]
- 
+        iPort  serialBegin         SPortName [, baudRate ]
+
 close a port
-			serialEnd           iPort
+                        serialEnd           iPort
 
 write byte(s) to the port, at i or k rate
-			serialWrite_i       iPort, iByte
-			serialWrite_i       iPort, kByte
-			serialWrite_i       iPort, Sbytes
-			serialWrite         iPort, iByte
-			serialWrite         iPort, kByte
-			serialWrite         iPort, Sbytes
+                        serialWrite_i       iPort, iByte
+                        serialWrite_i       iPort, kByte
+                        serialWrite_i       iPort, Sbytes
+                        serialWrite         iPort, iByte
+                        serialWrite         iPort, kByte
+                        serialWrite         iPort, Sbytes
 
 read the next byte from the input buffer
 returned value will be in the range of 0-255
-	kByte	serialRead          iPort
-    
+        kByte   serialRead          iPort
+
 print to screen any bytes (up to 32k) in input buffer
 note that these bytes will be cleared from the buffer.
 use this opcode mainly for debugging messages.
-if you want to mix debugging and other communication 
+if you want to mix debugging and other communication
 messages over the same port, you will need to manually
 parse the data with the serialRead opcode.
-			serialPrint			iPort
- 
+                        serialPrint                     iPort
+
 clear the input buffer
-			serialFlush         iPort
+                        serialFlush         iPort
+
 
- 
 TODO: (might need some kind of threaded buffer-read?)
- 
+
  kNum   serialAvailable    iPort
  returns number of bytes available to read
- 
+
  kByte   serialPeekByte      iPort
  returns the next byte in the input buffer
  does not remove the byte from the buffer
- 
+
 *****************************************************/
 
 #ifdef WIN32
@@ -107,17 +108,19 @@ static HANDLE get_port(CSOUND *csound, int port)
     q = (SERIAL_GLOBALS*) csound->QueryGlobalVariable(csound,
                                                       "serialGlobals_");
     if (q == NULL) {
-      csound->PerfError(csound, Str("No ports available"));
+      csound->ErrorMsg(csound, Str("No ports available"));
       return NULL;
     }
-    hport = (HANDLE)q->handles[port];  
+    hport = (HANDLE)q->handles[port];
     return hport;
 }
 #endif
 
 typedef struct {
     OPDS  h;
-    MYFLT *returnedPort, *portName, *baudRate;
+  MYFLT *returnedPort;
+    STRINGDAT *portName;
+    MYFLT *baudRate;
 } SERIALBEGIN;
 int serialBegin(CSOUND *csound, SERIALBEGIN *p);
 
@@ -177,16 +180,16 @@ int serialport_init(CSOUND *csound, const char* serialport, int baud)
     int fd;
     speed_t brate;
 
-    csound = NULL;              /* Not used */
+    //csound = NULL;              /* Not used */
     fprintf(stderr,"init_serialport: opening port %s @ %d bps\n",
             serialport,baud);
-	
+
     fd = open(serialport, O_RDWR | O_NOCTTY | O_NDELAY);
     if (fd == -1)  {
       perror("init_serialport: Unable to open port ");
       return -1;
     }
-    
+
     if (tcgetattr(fd, &toptions) < 0) {
       perror("init_serialport: Couldn't get term attributes");
       return -1;
@@ -208,7 +211,7 @@ int serialport_init(CSOUND *csound, const char* serialport, int baud)
     }
     cfsetispeed(&toptions, brate);
     cfsetospeed(&toptions, brate);
-	
+
     // 8N1
     toptions.c_cflag &= ~PARENB;
     toptions.c_cflag &= ~CSTOPB;
@@ -216,22 +219,22 @@ int serialport_init(CSOUND *csound, const char* serialport, int baud)
     toptions.c_cflag |= CS8;
     // no flow control
     toptions.c_cflag &= ~CRTSCTS;
-	
+
     toptions.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
     toptions.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl
-	
+
     toptions.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // make raw
     toptions.c_oflag &= ~OPOST; // make raw
-	
+
     // see: http://unixwiz.net/techtips/termios-vmin-vtime.html
     toptions.c_cc[VMIN]  = 0;
     toptions.c_cc[VTIME] = 20;
-    
+
     if ( tcsetattr(fd, TCSANOW, &toptions) < 0) {
       perror("init_serialport: Couldn't set term attributes");
       return -1;
     }
-	
+
     return fd;
 }
 #else
@@ -247,8 +250,10 @@ int serialport_init(CSOUND *csound, const char* serialport, int baud)
                                                       "serialGlobals_");
     if (q == NULL) {
       if (csound->CreateGlobalVariable(csound, "serialGlobals_",
-                                       sizeof(SERIAL_GLOBALS)) != 0)
-        csound->Die(csound, Str("serial: failed to allocate globals"));
+                                       sizeof(SERIAL_GLOBALS)) != 0){
+        csound->ErrorMsg(csound, Str("serial: failed to allocate globals"));
+        return 0;
+      }
       q = (SERIAL_GLOBALS*) csound->QueryGlobalVariable(csound,
                                                       "serialGlobals_");
       q->csound = csound;
@@ -260,7 +265,7 @@ int serialport_init(CSOUND *csound, const char* serialport, int baud)
     /*                     (LPCSTR)wport, 256); */
     /* hSerial = CreateFile(serialport, GENERIC_READ | GENERIC_WRITE, 0,  */
     /*                      0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0); */
-    hSerial = CreateFileA(serialport, GENERIC_READ | GENERIC_WRITE, 0, 
+    hSerial = CreateFileA(serialport, GENERIC_READ | GENERIC_WRITE, 0,
                          NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
  //Check if the connection was successfull
     if (hSerial==INVALID_HANDLE_VALUE) {
@@ -326,7 +331,7 @@ int serialport_init(CSOUND *csound, const char* serialport, int baud)
 int serialBegin(CSOUND *csound, SERIALBEGIN *p)
 {
     *p->returnedPort =
-      (MYFLT)serialport_init(csound, (char *)p->portName, *p->baudRate);
+      (MYFLT)serialport_init(csound, (char *)p->portName->data, *p->baudRate);
     return OK;
 }
 
@@ -336,7 +341,7 @@ int serialEnd(CSOUND *csound, SERIALEND *p)
     SERIAL_GLOBALS *q;
     q = (SERIAL_GLOBALS*) csound->QueryGlobalVariable(csound,
                                                       "serialGlobals_");
-    if (q = NULL) 
+    if (q = NULL)
       return csound->PerfError(csound, Str("Nothing to close"));
     CloseHandle((HANDLE)q->handles[(int)p->port]);
     q->handles[(int)*p->port] = NULL;
@@ -350,19 +355,9 @@ int serialWrite(CSOUND *csound, SERIALWRITE *p)
 {
 #ifdef WIN32
     HANDLE port = get_port(csound, (int)*p->port);
-    if (port==NULL) return NOTOK;  
-#endif  
-    if (p->XSTRCODE & 2) {
-#ifndef WIN32
-      if (UNLIKELY(write((int)*p->port, p->toWrite, strlen((char *)p->toWrite))<0))
-        return NOTOK;
-#else
-      int nbytes;
-      WriteFile(port,p->toWrite, strlen((char *)p->toWrite),
-                (PDWORD)&nbytes, NULL);
+    if (port==NULL) return NOTOK;
 #endif
-    }
-    else {
+    {
       unsigned char b = *p->toWrite;
 #ifndef WIN32
       if (UNLIKELY(write((int)*p->port, &b, 1)<0))
@@ -375,22 +370,42 @@ int serialWrite(CSOUND *csound, SERIALWRITE *p)
     return OK;
 }
 
+int serialWrite_S(CSOUND *csound, SERIALWRITE *p)
+{
+#ifdef WIN32
+    HANDLE port = get_port(csound, (int)*p->port);
+    if (port==NULL) return NOTOK;
+#endif
+#ifndef WIN32
+    if (UNLIKELY(write((int)*p->port,
+                       ((STRINGDAT*)p->toWrite)->data,
+                       ((STRINGDAT*)p->toWrite)->size))<0)
+        return NOTOK;
+#else
+      int nbytes;
+      WriteFile(port,p->toWrite, strlen((char *)p->toWrite),
+                (PDWORD)&nbytes, NULL);
+#endif
+    return OK;
+}
+
+
 int serialRead(CSOUND *csound, SERIALREAD *p)
 {
     unsigned char b = 0;
     ssize_t bytes;
 #ifdef WIN32
     HANDLE port = get_port(csound, (int)*p->port);
-    if (port==NULL) return NOTOK;  
+    if (port==NULL) return NOTOK;
     ReadFile(port, &b, 1, (PDWORD)&bytes, NULL);
 #else
     bytes = read((int)*p->port, &b, 1);
 #endif
     if (bytes > 0)
       *p->rChar = b;
-    else 
+    else
       *p->rChar = -1;
-    
+
     return OK;
 }
 
@@ -400,14 +415,14 @@ int serialPrint(CSOUND *csound, SERIALPRINT *p)
     ssize_t bytes;
 #ifdef WIN32
     HANDLE port = get_port(csound, (int)*p->port);
-    if (port==NULL) return NOTOK;  
+    if (port==NULL) return NOTOK;
     ReadFile(port, str, 32768, (PDWORD)&bytes, NULL);
 #else
     bytes  = read((int)*p->port, str, 32768);
 #endif
     if (bytes > 0) {
       str[bytes] = 0; // terminate
-      csound->MessageS(csound, CSOUNDMSG_ORCH, "%s", str);
+      csound->MessageS(csound, CSOUNDMSG_ORCH, str);
     }
     return OK;
 }
@@ -436,24 +451,28 @@ int serialPeekByte(CSOUND *csound, SERIALPEEK *p)
 #define S(x)    sizeof(x)
 
 static OENTRY serial_localops[] = {
-    { (char *)"serialBegin", S(SERIALBEGIN), 1, (char *)"i", (char *)"So",
+    { (char *)"serialBegin", S(SERIALBEGIN), 0, 1, (char *)"i", (char *)"So",
       (SUBR)serialBegin, (SUBR)NULL, (SUBR)NULL   },
-    { (char *)"serialEnd", S(SERIALEND), 2, (char *)"", (char *)"i",
+    { (char *)"serialEnd", S(SERIALEND), 0, 2, (char *)"", (char *)"i",
       (SUBR)NULL, (SUBR)serialEnd, (SUBR)NULL   },
-    { (char *)"serialWrite_i", S(SERIALWRITE), 1, (char *)"", (char *)"iT",
+    { (char *)"serialWrite_i", S(SERIALWRITE), 0, 1, (char *)"", (char *)"ii",
       (SUBR)serialWrite, (SUBR)NULL, (SUBR)NULL   },
-    { (char *)"serialWrite", S(SERIALWRITE), 2, (char *)"", (char *)"iU",
+       { (char *)"serialWrite_i.S", S(SERIALWRITE), 0, 1, (char *)"", (char *)"iS",
+      (SUBR)serialWrite_S, (SUBR)NULL, (SUBR)NULL   },
+    { (char *)"serialWrite", S(SERIALWRITE), WR, 2, (char *)"", (char *)"ik",
       (SUBR)NULL, (SUBR)serialWrite, (SUBR)NULL   },
-    { (char *)"serialRead", S(SERIALREAD), 2, (char *)"k", (char *)"i",
+    { (char *)"serialWrite.S", S(SERIALWRITE), WR, 2, (char *)"", (char *)"iS",
+      (SUBR)NULL, (SUBR)serialWrite_S, (SUBR)NULL   },
+    { (char *)"serialRead", S(SERIALREAD), 0, 2, (char *)"k", (char *)"i",
       (SUBR)NULL, (SUBR)serialRead, (SUBR)NULL   },
-    { (char *)"serialPrint", S(SERIALPRINT), 2, (char *)"", (char *)"i",
+    { (char *)"serialPrint", S(SERIALPRINT), WR,2, (char *)"", (char *)"i",
       (SUBR)NULL, (SUBR)serialPrint, (SUBR)NULL   },
-    { (char *)"serialFlush", S(SERIALFLUSH), 2, (char *)"", (char *)"i",
+    { (char *)"serialFlush", S(SERIALFLUSH), 0, 2, (char *)"", (char *)"i",
       (SUBR)NULL, (SUBR)serialFlush, (SUBR)NULL   },
-    /* { (char *)"serialAvailable", S(SERIALAVAIL), 2, (char *)"k", (char *)"i", */
-    /*   (SUBR)NULL, (SUBR)serialAvailable, (SUBR)NULL   }, */
-    /* { (char *)"serialPeekByte", S(SERIALPEEK), 2, (char *)"k", (char *)"i", */
-    /*   (SUBR)NULL, (SUBR)serialPeekByte, (SUBR)NULL   } */
+/* { (char *)"serialAvailable", S(SERIALAVAIL), 0, 2, (char *)"k", (char *)"i", */
+/*   (SUBR)NULL, (SUBR)serialAvailable, (SUBR)NULL   }, */
+/* { (char *)"serialPeekByte", S(SERIALPEEK),0,  2, (char *)"k", (char *)"i", */
+/*   (SUBR)NULL, (SUBR)serialPeekByte, (SUBR)NULL   } */
 };
 
-LINKAGE1(serial_localops)
+LINKAGE_BUILTIN(serial_localops)
