@@ -60,7 +60,8 @@ kpitch - pitch control (transposition ratio)
 
 FLOOPER2
 
-asig flooper2 kamp, kpitch, kloopstart, kloopend, kcrossfade, ifn [, istart, imode, ifenv]
+asig flooper2 kamp, kpitch, kloopstart, kloopend, kcrossfade,
+              ifn [, istart, imode, ifenv]
 
 Function-table crossfading looper with variable loop parameters and
 different looping modes.
@@ -69,7 +70,8 @@ INIT
 
 ifn - sound source function table. Non-power-of-two and deferred allocation
 tables are allowed.
-istart - playback starting point in secs, only applicable to loop modes 0 & 2 [def:0]
+istart - playback starting point in secs, only applicable to loop modes 0 & 2
+         [def:0]
 imode - loop modes: 0 forward, 1 backward, 2 back-and-forth [def: 0]
 ifenv - if non-zero, crossfade envelope shape table number. 0, the default, sets
 the crossfade to linear.
@@ -116,7 +118,7 @@ kgain - signal gain
 
 */
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include "pstream.h"
 
 typedef struct _sndloop {
@@ -211,8 +213,8 @@ typedef struct _pvsmorph {
 
 static int sndloop_init(CSOUND *csound, sndloop *p)
 {
-    p->durs = (int32) (*(p->dur)*csound->esr); /* dur in samps */
-    p->cfds = (int32) (*(p->cfd)*csound->esr); /* fade in samps */
+    p->durs = (int32) (*(p->dur)*CS_ESR); /* dur in samps */
+    p->cfds = (int32) (*(p->cfd)*CS_ESR); /* fade in samps */
     if (UNLIKELY(p->durs < p->cfds))
       return
         csound->InitError(csound, Str("crossfade cannot be longer than loop\n"));
@@ -229,7 +231,10 @@ static int sndloop_init(CSOUND *csound, sndloop *p)
 
 static int sndloop_process(CSOUND *csound, sndloop *p)
 {
-    int i, on = (int) *(p->on), recon, n = csound->ksmps;
+    int on = (int) *(p->on), recon;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     int32 durs = p->durs, cfds = p->cfds, wp = p->wp;
     double rp = p->rp;
     MYFLT a = p->a, inc = p->inc;
@@ -239,7 +244,12 @@ static int sndloop_process(CSOUND *csound, sndloop *p)
     if (on) recon = p->rst; /* restart recording if switched on again */
     else recon = 0;  /* else do not record */
 
-    for (i=0; i < n; i++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset; i < nsmps; i++) {
       if (recon) { /* if the recording is ON */
         /* fade in portion */
         if (wp < cfds) {
@@ -287,9 +297,9 @@ static int sndloop_process(CSOUND *csound, sndloop *p)
 static int flooper_init(CSOUND *csound, flooper *p)
 {
     MYFLT *tab, *buffer, a = FL(0.0), inc;
-    int32 cfds = (int32) (*(p->cfd)*csound->esr);     /* fade in samps  */
-    int32 starts = (int32) (*(p->start)*csound->esr); /* start in samps */
-    int32 durs = (int32)  (*(p->dur)*csound->esr);    /* dur in samps   */
+    int32 cfds = (int32) (*(p->cfd)*CS_ESR);     /* fade in samps  */
+    int32 starts = (int32) (*(p->start)*CS_ESR); /* start in samps */
+    int32 durs = (int32)  (*(p->dur)*CS_ESR);    /* dur in samps   */
     int32 len, i;
 
     if (UNLIKELY(cfds > durs))
@@ -342,7 +352,9 @@ static int flooper_init(CSOUND *csound, flooper *p)
 
 static int flooper_process(CSOUND *csound, flooper *p)
 {
-    int i, n = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     int32 end = p->strts+p->durs, durs = p->durs;
     MYFLT *out = p->out, *buffer = p->buffer.auxp;
     MYFLT amp = *(p->amp), pitch = *(p->pitch);
@@ -351,7 +363,12 @@ static int flooper_process(CSOUND *csound, flooper *p)
     MYFLT  frac;
     int tndx, loop_off = p->loop_off;
 
-    for (i=0; i < n; i++) {
+    if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset; i < nsmps; i++) {
       tndx = (int) ndx;
       frac = ndx - tndx;
       /* this is the start portion of the sound */
@@ -362,8 +379,8 @@ static int flooper_process(CSOUND *csound, flooper *p)
       /* this is the loop section */
       else {
         if (loop_off) {
-          ndx -= end;
-          tndx -= end;
+          while(ndx >= end) ndx -= end;
+          tndx = (int) ndx;
           /* wrap-around, if reading backwards */
           while (tndx < 0) tndx += durs;
         }
@@ -388,13 +405,13 @@ static int flooper2_init(CSOUND *csound, flooper2 *p)
     if (UNLIKELY(p->sfunc==NULL)) {
       return csound->InitError(csound,Str("function table not found\n"));
     }
-    if (*p->ifn2 != 0) p->efunc = csound->FTFind(csound, p->ifn2);
+    if (*p->ifn2 != 0) p->efunc = csound->FTnp2Find(csound, p->ifn2);
     else p->efunc = NULL;
 
     if (*p->iskip == 0){
       p->mode = (int) *p->imode;
       if (p->mode == 0 || p->mode == 2){
-        if ((p->ndx[0] = *p->start*csound->GetSr(csound)) < 0)
+        if ((p->ndx[0] = *p->start*CS_ESR) < 0)
           p->ndx[0] = 0;
         if (p->ndx[0] >= p->sfunc->flen)
           p->ndx[0] = (double) p->sfunc->flen - 1.0;
@@ -409,18 +426,33 @@ static int flooper2_init(CSOUND *csound, flooper2 *p)
 
 static int flooper2_process(CSOUND *csound, flooper2 *p)
 {
-    int i, n = csound->ksmps;
-    MYFLT *out = p->out, sr = csound->GetSr(csound);
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
+    MYFLT *out = p->out, sr = CS_ESR;
     MYFLT amp = *(p->amp), pitch = *(p->pitch);
-    MYFLT *tab = p->sfunc->ftable;
+    MYFLT *tab;
     double *ndx = p->ndx;
     MYFLT frac0, frac1, *etab;
     int loop_end = p->lend, loop_start = p->lstart,
-      crossfade = p->cfade, len = p->sfunc->flen;
+      crossfade = p->cfade, len;
     MYFLT count = p->count,fadein, fadeout;
     int *firsttime = &p->firsttime, elen, mode=p->mode,
         init = p->init, ijump = *p->ijump;
     uint32 tndx0, tndx1;
+    FUNC *func;
+    func = csound->FTnp2Find(csound, p->ifn);
+
+    if(p->sfunc != func) {
+    p->sfunc = func;
+    if (UNLIKELY(func == NULL))
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("table %d invalid\n"), (int) *p->ifn);
+    if (p->ndx[0] >= p->sfunc->flen)
+       p->ndx[0] = (double) p->sfunc->flen - 1.0;
+    }
+    tab = p->sfunc->ftable;
+    len = p->sfunc->flen;
 
     if (p->efunc != NULL) {
       etab = p->efunc->ftable;
@@ -433,9 +465,15 @@ static int flooper2_process(CSOUND *csound, flooper2 *p)
 
     /* loop parameters & check */
     if (pitch < FL(0.0)) pitch = FL(0.0);
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
 
     if (*firsttime) {
       int loopsize;
+      /* offset non zero only if firsttime */
+      if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
       loop_start = (int) (*p->loop_start*sr);
       loop_end =   (int) (*p->loop_end*sr);
       p->lstart = loop_start = loop_start < 0 ? 0 : loop_start;
@@ -473,7 +511,7 @@ static int flooper2_process(CSOUND *csound, flooper2 *p)
       }
     }
 
-    for (i=0; i < n; i++) {
+    for (i=offset; i < nsmps; i++) {
       if (mode == 1){ /* backwards */
         tndx0 = (int) ndx[0];
         frac0 = ndx[0] - tndx0;
@@ -675,7 +713,7 @@ static int flooper3_init(CSOUND *csound, flooper3 *p)
     if (*p->iskip == 0){
       p->mode = (int) *p->imode;
       if (p->mode == 0 || p->mode == 2){
-        if ((p->ndx[0] = *p->start*csound->GetSr(csound)) < 0)
+        if ((p->ndx[0] = *p->start*CS_ESR) < 0)
           p->ndx[0] = 0;
         if (p->ndx[0] >= p->sfunc->flen)
           p->ndx[0] = p->sfunc->flen - 1.0;
@@ -691,8 +729,11 @@ static int flooper3_init(CSOUND *csound, flooper3 *p)
 
 static int flooper3_process(CSOUND *csound, flooper3 *p)
 {
-    int i, n = csound->ksmps, lobits = p->lobits,si,ei;
-    MYFLT *out = p->out, sr = csound->GetSr(csound);
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
+    int lobits = p->lobits,si,ei;
+    MYFLT *out = p->out, sr = CS_ESR;
     MYFLT amp = *(p->amp), pitch = *(p->pitch);
     MYFLT *tab = p->sfunc->ftable, cvt;
     int32 *ndx = p->ndx, lomask = p->lomask, pos;
@@ -703,10 +744,14 @@ static int flooper3_process(CSOUND *csound, flooper3 *p)
     int *firsttime = &p->firsttime, elen, init = p->init;
     uint32 tndx0, tndx1;
 
-
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&out[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (pitch < FL(0.0)) pitch = FL(0.0);
     if (*firsttime) {
       int loopsize;
+      if (UNLIKELY(offset)) memset(out, '\0', offset*sizeof(MYFLT));
       loop_start = MYFLT2LRND(*p->loop_start*sr);
       loop_end =   MYFLT2LRND (*p->loop_end*sr);
       p->lstart = loop_start = (loop_start < 0 ? 0 : loop_start);
@@ -744,7 +789,7 @@ static int flooper3_process(CSOUND *csound, flooper3 *p)
     si = MYFLT2LRND(pitch*(lomask));
     ei = MYFLT2LRND(pitch*(lomask));
 
-    for (i=0; i < n; i++) {
+    for (i=offset; i < nsmps; i++) {
       if (mode == 0){
         tndx0 = ndx[0]>>lobits;
         frac0 = (ndx[0] & lomask)*lodiv;
@@ -905,11 +950,13 @@ static int flooper3_process(CSOUND *csound, flooper3 *p)
             loop_start = MYFLT2LRND(*p->loop_start*sr);
             loop_end =   MYFLT2LRND(*p->loop_end*sr);
             p->lstart = loop_start = (loop_start < 0 ? 0 : loop_start);
-            p->lend = loop_end =   (loop_end > len ? len :
-                                    (loop_end < loop_start ? loop_start : loop_end));
+            p->lend = loop_end =
+              (loop_end > len ? len :
+              (loop_end < loop_start ? loop_start : loop_end));
             loopsize = (loop_end - loop_start);
             crossfade =  MYFLT2LRND(*p->crossfade*sr);
-            p->cfade = crossfade = crossfade > loopsize/2 ? loopsize/2-1 : crossfade;
+            p->cfade = crossfade =
+              crossfade > loopsize/2 ? loopsize/2-1 : crossfade;
             cvt = (MYFLT)elen/p->cfade;
           }
         }
@@ -972,7 +1019,8 @@ static int pvsarp_process(CSOUND *csound, pvsarp *p)
 
     return OK;
  err1:
-    return csound->PerfError(csound,Str("pvsarp: not initialised\n"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvsarp: not initialised\n"));
 }
 
 static int pvsvoc_init(CSOUND *csound, pvsvoc *p)
@@ -1010,6 +1058,9 @@ static int pvsvoc_init(CSOUND *csound, pvsvoc *p)
    return OK;
 }
 
+void csoundComplexFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize);
+void csoundInverseComplexFFTnp2(CSOUND *csound, MYFLT *buf, int FFTsize);
+
 static int pvsvoc_process(CSOUND *csound, pvsvoc *p)
 {
     int32 i,N = p->fout->N;
@@ -1027,40 +1078,48 @@ static int pvsvoc_process(CSOUND *csound, pvsvoc *p)
     if (UNLIKELY(fout==NULL)) goto err1;
 
     if (p->lastframe < p->fin->framecount) {
-      for(j=0; j < 2; j++) {
+      int tmp = N/2;
+      tmp = tmp + tmp%2;
+      for (j=0; j < 2; j++) {
         MYFLT a;
         maxe = 0.f;
         maxa = 0.f;
-      for(i=0; i < N; i+=2) {
-        a  = (j ? fin[i] : (fexc[i] = ffr[i]));
-        maxa = maxa < a ? a : maxa;
-        if (a <= 0) a = 1e-20;
-        fenv[i/2] = log(a);
-      }
-      if (coefs < 1) coefs = 80;
-        for(i=0; i < N; i+=2){
+        for (i=0; i < N; i+=2) {
+          a  = (j ? fin[i] : (fexc[i] = ffr[i]));
+          maxa = maxa < a ? a : maxa;
+          if (a <= 0) a = 1e-20;
+          fenv[i/2] = log(a);
+        }
+        if (coefs < 1) coefs = 80;
+        for (i=0; i < N; i+=2){
           ceps[i] = fenv[i/2];
           ceps[i+1] = 0.0;
         }
-        csound->InverseComplexFFT(csound, ceps, N/2);
-        for(i=coefs; i < N-coefs; i++) ceps[i] = 0.0;
-        csound->ComplexFFT(csound, ceps, N/2);
-        for(i=0; i < N; i+=2) {
+        if (!(N & (N - 1)))
+          csound->InverseComplexFFT(csound, ceps, N/2);
+        else
+          csoundInverseComplexFFTnp2(csound, ceps, tmp);
+        for (i=coefs; i < N-coefs; i++) ceps[i] = 0.0;
+        if (!(N & (N - 1)))
+          csound->ComplexFFT(csound, ceps, N/2);
+        else
+          csoundComplexFFTnp2(csound, ceps, tmp);
+        for (i=0; i < N; i+=2) {
           fenv[i/2] = exp(ceps[i]);
-           maxe = maxe < fenv[i/2] ? fenv[i/2] : maxe;
+          maxe = maxe < fenv[i/2] ? fenv[i/2] : maxe;
         }
         if (maxe)
-        for(i=0; i<N; i+=2){
-          if (j) fenv[i/2] *= maxa/maxe;
-          if (fenv[i/2] && !j) {
-             fenv[i/2] /= maxe;
-             fexc[i] /= fenv[i/2];
-          }
+          for (i=0; i<N; i+=2){
+            if (j) fenv[i/2] *= maxa/maxe;
+            if (fenv[i/2] && !j) {
+              fenv[i/2] /= maxe;
+              fexc[i] /= fenv[i/2];
+            }
           }
       }
 
       kdepth = kdepth >= 0 ? (kdepth <= 1 ? kdepth : FL(1.0)): FL(0.0);
-      for(i=0;i < N+2;i+=2) {
+      for (i=0;i < N+2;i+=2) {
         fout[i] = fenv[i/2]*(fexc[i]*kdepth + fin[i]*(FL(1.0)-kdepth))*gain;
         fout[i+1] = ffr[i+1]*(kdepth) + fin[i+1]*(FL(1.0)-kdepth);
       }
@@ -1069,7 +1128,8 @@ static int pvsvoc_process(CSOUND *csound, pvsvoc *p)
 
     return OK;
  err1:
-    return csound->PerfError(csound,Str("pvsvoc: not initialised\n"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvsvoc: not initialised\n"));
 }
 
 static int pvsmorph_init(CSOUND *csound, pvsmorph *p)
@@ -1120,23 +1180,24 @@ static int pvsmorph_process(CSOUND *csound, pvsmorph *p)
 
     return OK;
  err1:
-    return csound->PerfError(csound,Str("pvsmorph: not initialised\n"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("pvsmorph: not initialised\n"));
 }
 
 static OENTRY localops[] = {
-  {"sndloop", sizeof(sndloop), 5,
+  {"sndloop", sizeof(sndloop),0, 5,
    "ak", "akkii", (SUBR)sndloop_init, NULL, (SUBR)sndloop_process},
-  {"flooper", sizeof(flooper), TR|5,
+  {"flooper", sizeof(flooper), TR, 5,
    "a", "kkiiii", (SUBR)flooper_init, NULL, (SUBR)flooper_process},
-  {"pvsarp", sizeof(pvsarp), 3,
+  {"pvsarp", sizeof(pvsarp), 0,3,
    "f", "fkkk", (SUBR)pvsarp_init, (SUBR)pvsarp_process},
-  {"pvsvoc", sizeof(pvsvoc), 3,
+  {"pvsvoc", sizeof(pvsvoc), 0,3,
    "f", "ffkkO", (SUBR)pvsvoc_init, (SUBR)pvsvoc_process},
-  {"flooper2", sizeof(flooper2), TR|5,
+  {"flooper2", sizeof(flooper2), TR, 5,
    "a", "kkkkkiooooO", (SUBR)flooper2_init, NULL, (SUBR)flooper2_process},
-  /* {"flooper3", sizeof(flooper3), TR|5,
+  /* {"flooper3", sizeof(flooper3), TR, 5,
      "a", "kkkkkioooo", (SUBR)flooper3_init, NULL, (SUBR)flooper3_process},*/
- {"pvsmorph", sizeof(pvsvoc), 3,
+ {"pvsmorph", sizeof(pvsvoc), 0,3,
    "f", "ffkk", (SUBR)pvsmorph_init, (SUBR)pvsmorph_process}
 };
 
