@@ -29,7 +29,7 @@
 /*from soundfiles.                                            */
 /**************************************************************/
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include "sndwarp.h"
 
 #define unirand(x) ((MYFLT) (x->Rand31(&(x->randSeed1)) - 1) / FL(2147483645.0))
@@ -53,17 +53,19 @@ static int sndwarpgetset(CSOUND *csound, SNDWARP *p)
     }
     p->exp = (WARPSECTION *)auxp;
 
-    if (UNLIKELY((ftpSamp = csound->FTFind(csound, p->isampfun)) == NULL)) return NOTOK;
+    if (UNLIKELY((ftpSamp = csound->FTnp2Find(csound, p->isampfun)) == NULL))
+      return NOTOK;
     p->ftpSamp  = ftpSamp;
     p->sampflen = ftpSamp->flen;
 
-    if (UNLIKELY((ftpWind = csound->FTFind(csound, p->ifn)) == NULL)) return NOTOK;
+    if (UNLIKELY((ftpWind = csound->FTnp2Find(csound, p->ifn)) == NULL))
+      return NOTOK;
     p->ftpWind = ftpWind;
     p->flen    = ftpWind->flen;
 
     p->maxFr   = -1 + ftpSamp->flen;
     p->prFlg   = 1;    /* true */
-    p->begin   = (int)(*p->ibegin * csound->esr);
+    p->begin   = (int)(*p->ibegin * CS_ESR);
 
     exp        = p->exp;
     iwsize = *p->iwsize;
@@ -91,7 +93,9 @@ static int sndwarpgetset(CSOUND *csound, SNDWARP *p)
 
 static int sndwarp(CSOUND *csound, SNDWARP *p)
 {
-    int         n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       frm_0,frm_1;
     int32       base, longphase;
     MYFLT       frac, frIndx;
@@ -118,18 +122,27 @@ static int sndwarp(CSOUND *csound, SNDWARP *p)
     ftpSamp = p->ftpSamp;
 
     for (i=0; i<overlap; i++) {
-/*       nsmps = csound->ksmps; */
+/*       nsmps = CS_KSMPS; */
 /*       r1 = p->r1; */
 /*       if (p->OUTOCOUNT >1)  r2 = p->r2; */
       resample = p->xresample;
       timewarpby = p->xtimewarp;
       amp = p->xamp;
 
-      for (n=0; n<nsmps;n++) {
+      if (UNLIKELY(offset)) {
+        memset(r1, '\0', offset*sizeof(MYFLT));
+        if (p->OUTOCOUNT >1) memset(r2, '\0', offset*sizeof(MYFLT));
+      }
+     if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r1[nsmps], '\0', early*sizeof(MYFLT));
+      if (p->OUTOCOUNT >1) memset(&r2[nsmps], '\0', early*sizeof(MYFLT));
+     }
+     for (n=offset; n<nsmps;n++) {
         if (exp[i].cnt < exp[i].wsize) goto skipover;
 
         if (*p->itimemode!=0)
-          exp[i].offset=(csound->esr * *timewarpby)+p->begin;
+          exp[i].offset=(CS_ESR * *timewarpby)+p->begin;
         else
           exp[i].offset += (MYFLT)exp[i].wsize/(*timewarpby);
 
@@ -180,7 +193,8 @@ static int sndwarp(CSOUND *csound, SNDWARP *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("sndwarp: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("sndwarp: not initialised"));
 }
 
 /****************************************************************/
@@ -210,19 +224,19 @@ static int sndwarpstgetset(CSOUND *csound, SNDWARPST *p)
     }
     p->exp = (WARPSECTION *)auxp;
 
-    if (UNLIKELY((ftpSamp = csound->FTFind(csound, p->isampfun)) == NULL))
+    if (UNLIKELY((ftpSamp = csound->FTnp2Find(csound, p->isampfun)) == NULL))
       return NOTOK;
     p->ftpSamp = ftpSamp;
     p->sampflen=ftpSamp->flen;
 
-    if (UNLIKELY((ftpWind = csound->FTFind(csound, p->ifn)) == NULL))
+    if (UNLIKELY((ftpWind = csound->FTnp2Find(csound, p->ifn)) == NULL))
       return NOTOK;
     p->ftpWind = ftpWind;
     p->flen=ftpWind->flen;
 
     p->maxFr  = -1L + (int32)(ftpSamp->flen*FL(0.5));
     p->prFlg = 1;    /* true */
-    p->begin = (int)(*p->ibegin * csound->esr);
+    p->begin = (int)(*p->ibegin * CS_ESR);
     iwsize = *p->iwsize;
     exp = p->exp;
     for (i=0; i< nsections; i++) {
@@ -254,7 +268,9 @@ static int sndwarpstset(CSOUND *csound, SNDWARPST *p)
 
 static int sndwarpst(CSOUND *csound, SNDWARPST *p)
 {
-    int         n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     MYFLT       frm10,frm11, frm20, frm21;
     int32        base, longphase;
     MYFLT       frac, frIndx;
@@ -280,17 +296,17 @@ static int sndwarpst(CSOUND *csound, SNDWARPST *p)
     exp = p->exp;
     ftpWind = p->ftpWind;
     ftpSamp = p->ftpSamp;
-
+    if (UNLIKELY(early)) nsmps -= early;
     for (i=0; i<*p->ioverlap; i++) {
       resample = p->xresample;
       timewarpby = p->xtimewarp;
       amp = p->xamp;
 
-      for (n=0; n<nsmps;n++) {
+      for (n=offset; n<nsmps;n++) {
         if (exp[i].cnt < exp[i].wsize) goto skipover;
 
         if (*p->itimemode!=0)
-          exp[i].offset=(csound->esr * *timewarpby)+p->begin;
+          exp[i].offset=(CS_ESR * *timewarpby)+p->begin;
         else
           exp[i].offset += (MYFLT)exp[i].wsize/(*timewarpby);
 
@@ -349,15 +365,16 @@ static int sndwarpst(CSOUND *csound, SNDWARPST *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("sndwarpst: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("sndwarpst: not initialised"));
 }
 
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-  { "sndwarp", S(SNDWARP), TR|5, "mm", "xxxiiiiiii",
+  { "sndwarp", S(SNDWARP), TR, 5, "mm", "xxxiiiiiii",
     (SUBR)sndwarpgetset, NULL, (SUBR)sndwarp},
-  { "sndwarpst", S(SNDWARPST), TR|5, "mmmm","xxxiiiiiii",
+  { "sndwarpst", S(SNDWARPST), TR, 5, "mmmm","xxxiiiiiii",
     (SUBR)sndwarpstset,NULL,(SUBR)sndwarpst}
 };
 
