@@ -21,11 +21,10 @@
     02111-1307 USA
 */
 
-#include "csdl.h"
+#include "csoundCore.h"
 #include <sys/types.h>
 #ifdef WIN32
 #include <winsock2.h>
-#include <ws2tcpip.h>
 #else
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -40,7 +39,9 @@ extern  int     inet_aton(const char *cp, struct in_addr *inp);
 
 typedef struct {
     OPDS    h;
-    MYFLT   *asig, *ipaddress, *port, *buffersize;
+  MYFLT   *asig;
+  STRINGDAT *ipaddress;
+  MYFLT *port, *buffersize;
     MYFLT   *format;
     AUXCH   aux;
     int     sock;
@@ -51,7 +52,9 @@ typedef struct {
 
 typedef struct {
     OPDS    h;
-    MYFLT   *asigl, *asigr, *ipaddress, *port, *buffersize;
+  MYFLT   *asigl, *asigr;
+  STRINGDAT *ipaddress;
+MYFLT *port, *buffersize;
     MYFLT   *format;
     AUXCH   aux;
     int     sock;
@@ -76,7 +79,8 @@ static int init_send(CSOUND *csound, SOCKSEND *p)
     p->ff = (int)(*p->format);
     p->bsize = bsize = (int) *p->buffersize;
     /* if (UNLIKELY((sizeof(MYFLT) * bsize) > MTU)) { */
-    /*   return csound->InitError(csound, Str("The buffersize must be <= %d samples " */
+    /*   return csound->InitError(csound,
+                                  Str("The buffersize must be <= %d samples " */
     /*                                        "to fit in a udp-packet."), */
     /*                            (int) (MTU / sizeof(MYFLT))); */
     /* } */
@@ -90,16 +94,17 @@ static int init_send(CSOUND *csound, SOCKSEND *p)
     memset(&p->server_addr, 0, sizeof(p->server_addr));
     p->server_addr.sin_family = AF_INET;    /* it is an INET address */
 #ifdef WIN32
-    p->server_addr.sin_addr.S_un.S_addr = inet_addr((const char *) p->ipaddress);
+    p->server_addr.sin_addr.S_un.S_addr =
+      inet_addr((const char *) p->ipaddress->data);
 #else
-    inet_aton((const char *) p->ipaddress,
+    inet_aton((const char *) p->ipaddress->data,
               &p->server_addr.sin_addr);    /* the server IP address */
 #endif
     p->server_addr.sin_port = htons((int) *p->port);    /* the port */
 
     if (p->ff) bwidth = sizeof(int16);
     /* create a buffer to write the interleaved audio to  */
-    if (p->aux.auxp == NULL || (long) (bsize * bwidth) > p->aux.size)
+    if (p->aux.auxp == NULL || (uint32_t) (bsize * bwidth) > p->aux.size)
       /* allocate space for the buffer */
       csound->AuxAlloc(csound, (bsize * bwidth), &p->aux);
     else {
@@ -112,19 +117,23 @@ static int init_send(CSOUND *csound, SOCKSEND *p)
 static int send_send(CSOUND *csound, SOCKSEND *p)
 {
     const struct sockaddr *to = (const struct sockaddr *) (&p->server_addr);
-    int     i, wp, ksmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
+    int     wp;
     int     buffersize = p->bsize;
     MYFLT   *asig = p->asig;
     MYFLT   *out = (MYFLT *) p->aux.auxp;
     int16   *outs = (int16 *) p->aux.auxp;
     int     ff = p->ff;
 
-    for (i = 0, wp = p->wp; i < ksmps; i++, wp++) {
+    if (UNLIKELY(early)) nsmps -= early;
+    for (i = offset, wp = p->wp; i < nsmps; i++, wp++) {
       if (wp == buffersize) {
         /* send the package when we have a full buffer */
         if (UNLIKELY(sendto(p->sock, (void*)out, buffersize  * p->bwidth, 0, to,
                             sizeof(p->server_addr)) < 0)) {
-          return csound->PerfError(csound, Str("sendto failed"));
+          return csound->PerfError(csound, p->h.insdshead, Str("sendto failed"));
         }
         wp = 0;
       }
@@ -146,6 +155,42 @@ static int send_send(CSOUND *csound, SOCKSEND *p)
     return OK;
 }
 
+static int send_send_k(CSOUND *csound, SOCKSEND *p)
+{
+    const struct sockaddr *to = (const struct sockaddr *) (&p->server_addr);
+
+    int     buffersize = p->bsize;
+    MYFLT   *ksig = p->asig;
+    MYFLT   *out = (MYFLT *) p->aux.auxp;
+    int16   *outs = (int16 *) p->aux.auxp;
+    int     ff = p->ff;
+
+
+      if (p->wp == buffersize) {
+        /* send the package when we have a full buffer */
+        if (UNLIKELY(sendto(p->sock, (void*)out, buffersize  * p->bwidth, 0, to,
+                            sizeof(p->server_addr)) < 0)) {
+          return csound->PerfError(csound, p->h.insdshead, Str("sendto failed"));
+        }
+        p->wp = 0;
+      }
+      if (ff) { // Scale for 0dbfs and make LE
+        int16 val = (int16)((32768.0* (*ksig))/csound->e0dbfs);
+        union cheat {
+          char  benchar[2];
+          int16 bensht;
+        } ch;
+        ch.benchar[0] = 0xFF & val;
+        ch.benchar[1] = 0xFF & (val >> 8);
+        outs[p->wp] = ch.bensht;
+      }
+      else out[p->wp++] = *ksig;
+
+    return OK;
+}
+
+
+
 /* UDP version 2 channels */
 static int init_sendS(CSOUND *csound, SOCKSENDS *p)
 {
@@ -161,7 +206,8 @@ static int init_sendS(CSOUND *csound, SOCKSENDS *p)
     p->ff = (int)(*p->format);
     p->bsize = bsize = (int) *p->buffersize;
     /* if (UNLIKELY((sizeof(MYFLT) * bsize) > MTU)) { */
-    /*   return csound->InitError(csound, Str("The buffersize must be <= %d samples " */
+    /*   return csound->InitError(csound,
+                                  Str("The buffersize must be <= %d samples " */
     /*                                        "to fit in a udp-packet."), */
     /*                            (int) (MTU / sizeof(MYFLT))); */
     /* } */
@@ -175,16 +221,17 @@ static int init_sendS(CSOUND *csound, SOCKSENDS *p)
     memset(&p->server_addr, 0, sizeof(p->server_addr));
     p->server_addr.sin_family = AF_INET;    /* it is an INET address */
 #ifdef WIN32
-    p->server_addr.sin_addr.S_un.S_addr = inet_addr((const char *) p->ipaddress);
+    p->server_addr.sin_addr.S_un.S_addr =
+      inet_addr((const char *) p->ipaddress->data);
 #else
-    inet_aton((const char *) p->ipaddress,
+    inet_aton((const char *) p->ipaddress->data,
               &p->server_addr.sin_addr);    /* the server IP address */
 #endif
     p->server_addr.sin_port = htons((int) *p->port);    /* the port */
 
     if (p->ff) bwidth = sizeof(int16);
     /* create a buffer to write the interleaved audio to */
-    if (p->aux.auxp == NULL || (long) (bsize * bwidth) > p->aux.size)
+    if (p->aux.auxp == NULL || (uint32_t) (bsize * bwidth) > p->aux.size)
       /* allocate space for the buffer */
       csound->AuxAlloc(csound, (bsize * bwidth), &p->aux);
     else {
@@ -201,19 +248,22 @@ static int send_sendS(CSOUND *csound, SOCKSENDS *p)
     MYFLT   *asigr = p->asigr;
     MYFLT   *out = (MYFLT *) p->aux.auxp;
     int16   *outs = (int16 *) p->aux.auxp;
-    int     i;
+    int     wp;
     int     buffersize = p->bsize;
-    int     wp, ksmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     int     ff = p->ff;
 
+    if (UNLIKELY(early)) nsmps -= early;
     /* store the samples of the channels interleaved in the packet */
     /* (left, right) */
-    for (i = 0, wp = p->wp; i < ksmps; i++, wp += 2) {
+    for (i = offset, wp = p->wp; i < nsmps; i++, wp += 2) {
       if (wp == buffersize) {
         /* send the package when we have a full buffer */
         if (UNLIKELY(sendto(p->sock, (void*)out, buffersize * p->bwidth, 0, to,
                             sizeof(p->server_addr)) < 0)) {
-          return csound->PerfError(csound, Str("sendto failed"));
+          return csound->PerfError(csound, p->h.insdshead, Str("sendto failed"));
         }
         wp = 0;
       }
@@ -269,9 +319,10 @@ static int init_ssend(CSOUND *csound, SOCKSEND *p)
 
     /* the server IP address, in network byte order */
 #ifdef WIN32
-    p->server_addr.sin_addr.S_un.S_addr = inet_addr((const char *) p->ipaddress);
+    p->server_addr.sin_addr.S_un.S_addr =
+      inet_addr((const char *) p->ipaddress->data);
 #else
-    inet_aton((const char *) p->ipaddress, &(p->server_addr.sin_addr));
+    inet_aton((const char *) p->ipaddress->data, &(p->server_addr.sin_addr));
 #endif
 
     /* the port we are going to listen on, in network byte order */
@@ -292,12 +343,15 @@ again:
 
 static int send_ssend(CSOUND *csound, SOCKSEND *p)
 {
-    int     n = sizeof(MYFLT) * csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n = sizeof(MYFLT) * (CS_KSMPS-offset-early);
 
-    if (n != write(p->sock, p->asig, n)) {
-      csound->Message(csound, "Expected %d got %d\n",
-                      (int) (sizeof(MYFLT) * csound->ksmps), n);
-      return csound->PerfError(csound, Str("write to socket failed"));
+    if (n != write(p->sock, &p->asig[offset], n)) {
+      csound->Message(csound, Str("Expected %d got %d\n"),
+                      (int) (sizeof(MYFLT) * CS_KSMPS), n);
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("write to socket failed"));
     }
 
     return OK;
@@ -306,12 +360,14 @@ static int send_ssend(CSOUND *csound, SOCKSEND *p)
 #define S(x)    sizeof(x)
 
 static OENTRY socksend_localops[] = {
-  { "socksend", S(SOCKSEND), 5, "", "aSiio", (SUBR) init_send, NULL,
+  { "socksend", S(SOCKSEND), 0, 5, "", "aSiio", (SUBR) init_send, NULL,
     (SUBR) send_send },
-  { "socksends", S(SOCKSENDS), 5, "", "aaSiio", (SUBR) init_sendS, NULL,
+   { "socksend_k", S(SOCKSEND), 0, 3, "", "kSiio", (SUBR) init_send,
+     (SUBR) send_send_k, NULL },
+  { "socksends", S(SOCKSENDS), 0, 5, "", "aaSiio", (SUBR) init_sendS, NULL,
     (SUBR) send_sendS },
-  { "stsend", S(SOCKSEND), 5, "", "aSi", (SUBR) init_ssend, NULL,
+  { "stsend", S(SOCKSEND), 0, 5, "", "aSi", (SUBR) init_ssend, NULL,
     (SUBR) send_ssend }
 };
 
-LINKAGE1(socksend_localops)
+LINKAGE_BUILTIN(socksend_localops)
