@@ -27,7 +27,7 @@
 /* University of Washington, Seattle 1998 */
 /******************************************/
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include "space.h"
 #include <math.h>
 
@@ -45,15 +45,15 @@ static int spaceset(CSOUND *csound, SPACE *p)
     }
 
     if (p->auxch.auxp == NULL ||
-        p->auxch.size<sizeof(MYFLT)*(csound->ksmps * 4)) {
+        p->auxch.size<sizeof(MYFLT)*(CS_KSMPS * 4)) {
       MYFLT *fltp;
-      csound->AuxAlloc(csound, (size_t) (csound->ksmps * 4)
+      csound->AuxAlloc(csound, (size_t) (CS_KSMPS * 4)
                                * sizeof(MYFLT), &p->auxch);
       fltp = (MYFLT *) p->auxch.auxp;
-      p->rrev1 = fltp;   fltp += csound->ksmps;
-      p->rrev2 = fltp;   fltp += csound->ksmps;
-      p->rrev3 = fltp;   fltp += csound->ksmps;
-      p->rrev4 = fltp;   fltp += csound->ksmps;
+      p->rrev1 = fltp;   fltp += CS_KSMPS;
+      p->rrev2 = fltp;   fltp += CS_KSMPS;
+      p->rrev3 = fltp;   fltp += CS_KSMPS;
+      p->rrev4 = fltp;   fltp += CS_KSMPS;
     }
 
     pp = (STDOPCOD_GLOBALS*) csound->stdOp_Env;
@@ -71,11 +71,12 @@ static int space(CSOUND *csound, SPACE *p)
     MYFLT   half_pi = FL(0.5)*PI_F;
     MYFLT   sqrt2 = SQRT(FL(2.0));
     MYFLT   fabxndx, fabyndx;
-    int     n;
     FUNC    *ftp;
     int32    indx, length, halflen;
     MYFLT   v1, v2, fract, ndx;
-    int nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     if (*p->ifn > 0) { /* get xy vals from function table */
       if (UNLIKELY((ftp = p->ftp) == NULL)) goto err1;
@@ -146,7 +147,20 @@ static int space(CSOUND *csound, SPACE *p)
     rrev3 = p->rrev3;
     rrev4 = p->rrev4;
     sigp = p->asig;
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) {
+      memset(r1, '\0', offset*sizeof(MYFLT));
+      memset(r2, '\0', offset*sizeof(MYFLT));
+      memset(r3, '\0', offset*sizeof(MYFLT));
+      memset(r4, '\0', offset*sizeof(MYFLT));
+    }
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r1[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&r2[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&r3[nsmps], '\0', early*sizeof(MYFLT));
+      memset(&r4[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       direct = sigp[n] * distr;
       torev = sigp[n] * distrsq * *p->reverbamount;
       globalrev = torev * distr;
@@ -162,7 +176,8 @@ static int space(CSOUND *csound, SPACE *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("space: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("space: not initialised"));
 }
 
 static int spsendset(CSOUND *csound, SPSEND *p)
@@ -177,7 +192,7 @@ static int spsendset(CSOUND *csound, SPSEND *p)
 static int spsend(CSOUND *csound, SPSEND *p)
 {
     SPACE *q = p->space;
-    int nbytes = csound->ksmps*sizeof(MYFLT);
+    int nbytes = CS_KSMPS*sizeof(MYFLT);
 
     memmove(p->r1, q->rrev1, nbytes);
     memmove(p->r2, q->rrev2, nbytes);
@@ -244,15 +259,16 @@ static int spdist(CSOUND *csound, SPDIST *p)
     *r=distance;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("spdist: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("spdist: not initialised"));
 }
 
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "space",  S(SPACE),  TR|5, "aaaa", "aikkkk",(SUBR)spaceset, NULL, (SUBR)space },
-{ "spsend", S(SPSEND), 5, "aaaa", "",     (SUBR)spsendset, NULL, (SUBR)spsend },
-{ "spdist", S(SPDIST), 3,    "k", "ikkk", (SUBR)spdistset, (SUBR)spdist, NULL }
+  { "space",  S(SPACE), TR,5, "aaaa", "aikkkk",(SUBR)spaceset, NULL, (SUBR)space },
+  { "spsend", S(SPSEND), 0,5, "aaaa", "",     (SUBR)spsendset, NULL, (SUBR)spsend },
+  { "spdist", S(SPDIST), 0,3,    "k", "ikkk", (SUBR)spdistset, (SUBR)spdist, NULL }
 };
 
 int space_init_(CSOUND *csound)
