@@ -23,7 +23,7 @@
 
 /* ----- spat3d, spat3di, and spat3dt -- written by Istvan Varga, 2001 ----- */
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
@@ -45,7 +45,7 @@ static int    spat3d_init_window(CSOUND *csound, SPAT3D *p)
     o = p->oversamp << 5;           /* window size = 32 * oversample */
 
     i = ((o + 1) * (sizeof(int) + sizeof(MYFLT)));      /* allocate */
-    if ((p->fltr.auxp == NULL) || (p->fltr.size < i))   /* space    */
+    if ((p->fltr.auxp == NULL) || (p->fltr.size < (unsigned int)i)) /* space */
       csound->AuxAlloc(csound, i, &(p->fltr));
     p->sample = (int *) p->fltr.auxp;                 /* sample number */
     p->window = (MYFLT *) (p->sample + o + 1);        /* window value  */
@@ -207,8 +207,8 @@ spat3d_init_wall(SPAT3D *p,             /* opcode struct                    */
       /* extend delay buffer */
       if ((MYFLT) d0 > p->mdel) p->mdel = (MYFLT) d0;
       if ((MYFLT) d1 > p->mdel) p->mdel = (MYFLT) d1;
-      ws->D0 = d0 * (double) csound->esr + 0.5;
-      ws->D1 = d1 * (double) csound->esr + 0.5;
+      ws->D0 = d0 * (double) CS_ESR + 0.5;
+      ws->D1 = d1 * (double) CS_ESR + 0.5;
       ws->W0 = w; ws->X0 = x; ws->Y0 = y; ws->Z0 = z;
     }
 
@@ -232,12 +232,12 @@ static int spat3d_init_delay(CSOUND *csound, SPAT3D *p)
 {
     int32    i, j;
 
-    i = ((int32) (p->mdel * csound->esr) + (int32) csound->ksmps + 34L)
+    i = ((int32) (p->mdel * CS_ESR) + (int32) CS_KSMPS + 34L)
         * (int32) p->oversamp;
     p->mdel_s = i;
     if (p->o_num == 1) i += 4;      /* extra samples for spat3d */
     j = i * (int32) sizeof(MYFLT) * (int32) (p->zout > 3 ? 4 : p->zout + 1);
-    if ((p->del.auxp == NULL) || (p->del.size < j))         /* allocate */
+    if ((p->del.auxp == NULL) || (p->del.size < (unsigned int)j)) /* allocate */
       csound->AuxAlloc(csound, j, &(p->del));               /* space    */
     p->Wb = (MYFLT *) p->del.auxp;                  /* W */
     if (p->zout > 0) p->Yb = p->Wb + i;             /* Y */
@@ -292,7 +292,7 @@ static int spat3d_set_opcode_params(CSOUND *csound, SPAT3D *p)
     p->ftable = p->outft = NULL;    /* no ftables */
     p->zout = p->rseed = p->mindep = p->maxdep = p->outftlnth = wmask = 0;
     p->oversamp = 1;                /* oversample */
-    p->bs = (int) csound->ksmps;    /* block size */
+    p->bs = (int) CS_KSMPS;    /* block size */
     p->irlen = 2;                   /* IR length  */
     p->mdist = p->mdel = FL(0.001); /* unit circle dist., max. delay */
     p->mdel_s = p->del_p = 0L;
@@ -318,7 +318,7 @@ static int spat3d_set_opcode_params(CSOUND *csound, SPAT3D *p)
       p->mdist = *(p->args[xidist]);
     if (xift >= 0) {                                /* ftable */
       int fLen;
-      fLen = csound->GetTable(csound, &(p->ftable), (int) *(p->args[xift]));
+      fLen = csoundGetTable(csound, &(p->ftable), (int) *(p->args[xift]));
       if (fLen < 53)
         p->ftable = NULL;
     }
@@ -327,10 +327,10 @@ static int spat3d_set_opcode_params(CSOUND *csound, SPAT3D *p)
     if (xiovr >= 0)                                 /* oversample */
       p->oversamp = (int) MYFLT2LRND(*(p->args[xiovr]));
     if (xirlen >= 0)                                /* IR length */
-      p->irlen = (int) MYFLT2LRND(*(p->args[xirlen]) * csound->esr);
+      p->irlen = (int) MYFLT2LRND(*(p->args[xirlen]) * CS_ESR);
     if (xioutft >= 0) {                             /* output table */
       int fLen;
-      fLen = csound->GetTable(csound, &(p->outft), (int) *(p->args[xioutft]));
+      fLen = csoundGetTable(csound, &(p->outft), (int) *(p->args[xioutft]));
       if (fLen < 1) {
         p->outft = NULL; p->outftlnth = 0;
       }
@@ -352,7 +352,7 @@ static int spat3d_set_opcode_params(CSOUND *csound, SPAT3D *p)
       if (p->ftable[2] >= FL(0.0))            /* max. delay        */
         p->mdel = p->ftable[2];
       if (p->ftable[3] >= FL(0.0))            /* IR length         */
-        p->irlen = (int) MYFLT2LRND(p->ftable[3] * csound->esr);
+        p->irlen = (int) MYFLT2LRND(p->ftable[3] * CS_ESR);
       if (p->ftable[4] >= FL(0.0))            /* unit circle dist. */
         p->mdist = p->ftable[4];
       p->rseed = (int32) MYFLT2LRND(p->ftable[5]);     /* seed      */
@@ -381,11 +381,11 @@ static int spat3d_set_opcode_params(CSOUND *csound, SPAT3D *p)
     if (p->maxdep >= 0) {
       i = d = 0; spat3d_count_refl(&i, &d, 0, p->maxdep, 0, wmask);
       i *= (int32) sizeof(SPAT3D_WALL);
-      if ((p->ws.auxp == NULL) || (p->ws.size < i))
+      if ((p->ws.auxp == NULL) || (p->ws.size < (unsigned int)i))
         csound->AuxAlloc(csound, i, &(p->ws));
       i = (int32) p->bs * (int32) d;
       i *= (int32) sizeof(MYFLT);
-      if ((p->y.auxp == NULL) || (p->y.size < i))
+      if ((p->y.auxp == NULL) || (p->y.size < (unsigned int)i))
         csound->AuxAlloc(csound, i, &(p->y));
     }
     return OK;
@@ -486,9 +486,9 @@ static void spat3d_wall_perf(CSOUND     *csound, /* General environment       */
       a = SPAT3D_DIST2AMP(d);                   /* amp.      */
       x = SQRT(FL(1.0) - (x / (d + FL(0.0001))));
       x *= a; w = a - x;                                /* Lh, Ll   */
-      d1 *= (double) p->oversamp * (double) csound->esr;/* convert  */
+      d1 *= (double) p->oversamp * (double) CS_ESR;/* convert  */
     }                                                   /* delay to */
-    d0 *= (double) p->oversamp * (double) csound->esr;  /* samples  */
+    d0 *= (double) p->oversamp * (double) CS_ESR;  /* samples  */
 
     /* interpolate W, X, Y, Z, and delay */
 
@@ -661,7 +661,8 @@ static int    spat3d(CSOUND *csound, SPAT3D *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("spat3d: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("spat3d: not initialised"));
 }
 
 /* -------------------------- spat3di performance -------------------------- */
@@ -778,7 +779,8 @@ static int    spat3di(CSOUND *csound, SPAT3D *p)
     } while (--nn);
     return OK;
  err1:
-    return csound->PerfError(csound, Str("spat3di: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("spat3di: not initialised"));
 }
 
 /* ---------------------------- spat3dt opcode ----------------------------- */
@@ -848,7 +850,8 @@ static int    spat3dt(CSOUND *csound, SPAT3D *p)
     spat3d_init_wall(p, 0, 0, &wmax,
                      *(p->args[1]), *(p->args[2]), *(p->args[3]));
     p->outftlnth = ((p->outftlnth) >> 2) << 2;      /* table length  */
-    if (UNLIKELY((p->outft == NULL) || (p->outftlnth < 4))) return NOTOK; /* no table */
+    if (UNLIKELY((p->outft == NULL) || (p->outftlnth < 4)))
+      return NOTOK;                         /* no table */
 
     /* initialise IR */
 
@@ -872,11 +875,11 @@ static int    spat3dt(CSOUND *csound, SPAT3D *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-    { "spat3d", S(SPAT3D), 5, "aaaa", "akkkiiiiio",
+    { "spat3d", S(SPAT3D), 0, 5, "aaaa", "akkkiiiiio",
                     (SUBR) spat3dset,   NULL,   (SUBR) spat3d   },
-    { "spat3di",S(SPAT3D), 5, "aaaa", "aiiiiiio",
+    { "spat3di",S(SPAT3D), 0, 5, "aaaa", "aiiiiiio",
                     (SUBR) spat3diset,  NULL,   (SUBR) spat3di  },
-    { "spat3dt",S(SPAT3D), 1, "", "iiiiiiiio",
+    { "spat3dt",S(SPAT3D), 0, 1, "", "iiiiiiiio",
                     (SUBR) spat3dt,     NULL,   NULL            }
 };
 
