@@ -1255,10 +1255,10 @@ static OENTRY spectra_localops[] = {
 { "adsynt",S(HSBOSC), TR, 5,  "a", "kkiiiio", (SUBR)adsyntset, NULL, (SUBR)adsynt },
 { "mpulse", S(IMPULSE), 0, 5,  "a", "kko",
                                     (SUBR)impulse_set, NULL, (SUBR)impulse },
-{ "lpf18", S(LPF18),    0, 5,  "a", "akkko",  (SUBR)lpf18set, NULL, (SUBR)lpf18db },
-{ "waveset", S(BARRI),  0, 5,  "a", "ako",   (SUBR)wavesetset, NULL, (SUBR)waveset},
+{ "lpf18", S(LPF18), 0, 5,  "a", "axxxo",  (SUBR)lpf18set, NULL, (SUBR)lpf18db },
+{ "waveset", S(BARRI), 0, 5,  "a", "ako",  (SUBR)wavesetset, NULL, (SUBR)waveset},
 { "pinkish", S(PINKISH), 0, 5, "a", "xoooo", (SUBR)pinkset, NULL, (SUBR)pinkish },
-{ "noise",  S(VARI),  0, 5,    "a", "xk",   (SUBR)varicolset, NULL, (SUBR)varicol },
+{ "noise",  S(VARI), 0, 5,  "a", "xk",   (SUBR)varicolset, NULL, (SUBR)varicol },
 { "transeg", S(TRANSEG),0, 3,  "k", "iiim",
                                            (SUBR)trnset,(SUBR)ktrnseg, NULL},
 { "transeg.a", S(TRANSEG),0, 5,  "a", "iiim",
