@@ -72,20 +72,22 @@ typedef struct POP_OPCODE_ {
     int     initDone;
 } POP_OPCODE;
 
-/* fassign() was taken from pstream.c, written by Richard Dobson */
+/* fsg_assign() was taken from pstream.c, written by Richard Dobson */
 
-static CS_NOINLINE void fassign(CSOUND *csound,
+static CS_NOINLINE void fsg_assign(CSOUND *csound,
                                 PVSDAT *fdst, const PVSDAT *fsrc)
 {
-    if (UNLIKELY(fsrc->frame.auxp == NULL))
-      csound->Die(csound, Str("fsig = : source signal is not initialised"));
+  if (UNLIKELY(fsrc->frame.auxp == NULL)) {
+      csound->ErrorMsg(csound, Str("fsig = : source signal is not initialised"));
+      return;
+  }
     fdst->N = fsrc->N;
     fdst->overlap = fsrc->overlap;
     fdst->winsize = fsrc->winsize;
     fdst->wintype = fsrc->wintype;
     fdst->format = fsrc->format;
     if (fdst->frame.auxp == NULL ||
-        fdst->frame.size != (fdst->N + 2L) * (long) sizeof(float))
+        fdst->frame.size != (uint32_t)((fdst->N + 2L) * sizeof(float)))
       csound->AuxAlloc(csound,
                        (fdst->N + 2L) * (long) sizeof(float), &(fdst->frame));
     if (fdst->framecount != fsrc->framecount) {
@@ -105,17 +107,18 @@ static CS_NOINLINE int csoundStack_Error(void *p, const char *msg)
     CSOUND  *csound;
 
     csound = ((OPDS*) p)->insdshead->csound;
-    if (UNLIKELY(csound->ids != NULL && csound->pds == NULL)) {
+    if (UNLIKELY(csound->ids != NULL &&  ((OPDS*) p)->insdshead->pds == NULL)) {
       csound->InitError(csound, "%s: %s", csound->GetOpcodeName(p), msg);
       csound->LongJmp(csound, CSOUND_INITIALIZATION);
     }
-    else if (UNLIKELY(csound->ids == NULL && csound->pds != NULL)) {
-      csound->PerfError(csound, "%s: %s", csound->GetOpcodeName(p), msg);
+    else if (UNLIKELY(csound->ids == NULL && ((OPDS*) p)->insdshead->pds != NULL)) {
+      csound->PerfError(csound, ((OPDS*)p)->insdshead,
+                        "%s: %s", csound->GetOpcodeName(p), msg);
       csound->LongJmp(csound, CSOUND_PERFORMANCE);
     }
     else
-      csound->Die(csound, "%s: %s", csound->GetOpcodeName(p), msg);
-    /* this is actually never reached */
+      csound->ErrorMsg(csound, "%s: %s", csound->GetOpcodeName(p), msg);
+
     return NOTOK;
 }
 
@@ -137,11 +140,11 @@ static CS_NOINLINE int csoundStack_TypeError(void *p)
     return csoundStack_Error(p, Str("argument number or type mismatch"));
 }
 
-static CS_NOINLINE int csoundStack_LengthError(void *p)
-{
-    /* CSOUND  *csound = ((OPDS*) p)->insdshead->csound; */
-    return csoundStack_Error(p, Str("string argument is too long"));
-}
+/* static CS_NOINLINE int csoundStack_LengthError(void *p) */
+/* { */
+/*     /\* CSOUND  *csound = ((OPDS*) p)->insdshead->csound; *\/ */
+/*     return csoundStack_Error(p, Str("string argument is too long")); */
+/* } */
 
 static CS_NOINLINE CsoundArgStack_t *csoundStack_AllocGlobals(CSOUND *csound,
                                                               int stackSize)
@@ -155,9 +158,12 @@ static CS_NOINLINE CsoundArgStack_t *csoundStack_AllocGlobals(CSOUND *csound,
       stackSize = 16777200;
     nBytes = csoundStack_Align((int) sizeof(CsoundArgStack_t));
     nBytes += stackSize;
-    if (UNLIKELY(csound->CreateGlobalVariable(csound, "csArgStack", (size_t) nBytes)
-                 != 0))
-      csound->Die(csound, Str("Error allocating argument stack"));
+    if (UNLIKELY(csound->CreateGlobalVariable(csound,
+                                              "csArgStack", (size_t) nBytes)
+                 != 0)) {
+      csound->ErrorMsg(csound, Str("Error allocating argument stack"));
+      return NULL;
+    }
     pp = (CsoundArgStack_t*) csound->QueryGlobalVariable(csound, "csArgStack");
     pp->curBundle = (CsoundArgStack_t*) NULL;
     pp->dataSpace =
@@ -179,7 +185,7 @@ static CS_NOINLINE CsoundArgStack_t *csoundStack_GetGlobals(CSOUND *csound)
     return pp;
 }
 
-static CS_NOINLINE int csoundStack_CreateArgMap(void *p, int *argMap,
+static CS_NOINLINE int csoundStack_CreateArgMap(PUSH_OPCODE *p, int *argMap,
                                                 int isOutput)
 {
     CSOUND  *csound;
@@ -240,7 +246,7 @@ static CS_NOINLINE int csoundStack_CreateArgMap(void *p, int *argMap,
         /* performance time types */
         if (aMask & maskVal) {
           argMap[i + 3] = (curOffs_p | CS_STACK_A);
-          curOffs_p += ((int) sizeof(MYFLT) * csound->ksmps);
+          curOffs_p += ((int) sizeof(MYFLT) * CS_KSMPS);
         }
         else {
           argMap[i + 3] = (curOffs_p | CS_STACK_K);
@@ -251,8 +257,8 @@ static CS_NOINLINE int csoundStack_CreateArgMap(void *p, int *argMap,
         /* init time types */
         if (sMask & maskVal) {
           argMap[i + 3] = (curOffs_i | CS_STACK_S);
-          curOffs_i += csound->strVarMaxLen;
-          curOffs_i = csoundStack_Align(curOffs_i);
+          curOffs_i += (int) sizeof(STRINGDAT);
+            /* curOffs_i = csoundStack_Align(curOffs_i);*/
         }
         else {
           argMap[i + 3] = (curOffs_i | CS_STACK_I);
@@ -283,8 +289,9 @@ static int stack_opcode_init(CSOUND *csound, STACK_OPCODE *p)
 
 static int notinit_opcode_stub_perf(CSOUND *csound, void *p)
 {
-    return csound->PerfError(csound, Str("%s: not initialised"),
-                                     csound->GetOpcodeName(p));
+    return csound->PerfError(csound, ((OPDS*)p)->insdshead,
+                             Str("%s: not initialised"),
+                             csound->GetOpcodeName(p));
 }
 
 static int push_opcode_perf(CSOUND *csound, PUSH_OPCODE *p)
@@ -311,12 +318,19 @@ static int push_opcode_perf(CSOUND *csound, PUSH_OPCODE *p)
           case CS_STACK_A:
             {
               MYFLT *src, *dst;
-              int   j;
+              uint32_t offset = p->h.insdshead->ksmps_offset;
+              uint32_t early  = p->h.insdshead->ksmps_no_end;
+              uint32_t nsmps = CS_KSMPS;
               src = p->args[i];
               dst = (MYFLT*) ((char*) bp + (int) (curOffs & (int) 0x00FFFFFF));
-              //memcpy(dst, src, sizeof(MYFLT)*csound->ksmps;
-              for (j = 0; j < csound->ksmps; j++)
-                dst[j] = src[j];
+              if (UNLIKELY(offset)) memset(dst, '\0', offset*sizeof(MYFLT));
+              if (UNLIKELY(early)) {
+                nsmps -= early;
+                memset(&dst[nsmps], '\0', early*sizeof(MYFLT));
+              }
+              memcpy(&dst[offset], &src[offset], sizeof(MYFLT)*(nsmps-offset));
+              //for (j = 0; j < nsmps; j++)
+              //dst[j] = src[j];
             }
           }
         }
@@ -330,7 +344,7 @@ static int push_opcode_init(CSOUND *csound, PUSH_OPCODE *p)
 {
     if (!p->initDone) {
       p->pp = csoundStack_GetGlobals(csound);
-      if (UNLIKELY(csoundStack_CreateArgMap(p, &(p->argMap[0]), 0) != OK))
+      if (UNLIKELY(csoundStack_CreateArgMap(p, (int*)&(p->argMap[0]), 0) != OK))
         return NOTOK;
       p->h.opadr = (int (*)(CSOUND *, void *)) push_opcode_perf;
       p->initDone = 1;
@@ -338,7 +352,8 @@ static int push_opcode_init(CSOUND *csound, PUSH_OPCODE *p)
     if (p->argMap[1] != 0) {
       void  *bp;
       int   i, *ofsp;
-      if (UNLIKELY(p->pp->freeSpaceOffset + p->argMap[1] > p->pp->freeSpaceEndOffset))
+      if (UNLIKELY(p->pp->freeSpaceOffset + p->argMap[1] >
+                   p->pp->freeSpaceEndOffset))
         return csoundStack_OverflowError(p);
       bp = (void*) ((char*) p->pp->dataSpace + (int) p->pp->freeSpaceOffset);
       p->pp->freeSpaceOffset += p->argMap[1];
@@ -356,19 +371,26 @@ static int push_opcode_init(CSOUND *csound, PUSH_OPCODE *p)
             break;
           case CS_STACK_S:
             {
-              char  *src, *dst;
-              int   j, maxLen;
-              src = (char*) p->args[i];
-              dst = (char*) bp + (int) (curOffs & (int) 0x00FFFFFF);
-              maxLen = csound->strVarMaxLen - 1;
-              for (j = 0; src[j] != (char) 0; j++) {
-                dst[j] = src[j];
-                if (j >= maxLen) {
-                  dst[j] = (char) 0;
-                  csoundStack_LengthError(p);
-                }
+              char  *src;
+              STRINGDAT *dst;
+              /* int   j, maxLen; */
+              src = ((STRINGDAT*) p->args[i])->data;
+              dst = ((STRINGDAT*)(char*) bp + (int) (curOffs & (int) 0x00FFFFFF));
+              if(dst->size <= (int) strlen(src)){
+              dst->data = csound->Strdup(csound, src);
+              dst->size = strlen(src) + 1;
+              } else {
+                strcpy(dst->data, src);
               }
-              dst[j] = (char) 0;
+              /* maxLen = ((STRINGDAT*) p->args[i])->size; */
+              /* for (j = 0; src[j] != (char) 0; j++) { */
+              /*   dst[j] = src[j]; */
+              /*   if (j >= maxLen) { */
+              /*     dst[j] = (char) 0; */
+              /*     csoundStack_LengthError(p); */
+              /*   } */
+              /* } */
+              /* dst[j] = (char) 0; */
             }
           }
         }
@@ -401,11 +423,19 @@ static int pop_opcode_perf(CSOUND *csound, POP_OPCODE *p)
           case CS_STACK_A:
             {
               MYFLT *src, *dst;
-              int   j;
+              uint32_t offset = p->h.insdshead->ksmps_offset;
+              uint32_t early  = p->h.insdshead->ksmps_no_end;
+              uint32_t nsmps = CS_KSMPS;
               src = (MYFLT*) ((char*) bp + (int) (curOffs & (int) 0x00FFFFFF));
               dst = p->args[i];
-              for (j = 0; j < csound->ksmps; j++)
-                dst[j] = src[j];
+              if (UNLIKELY(offset)) memset(dst, '\0', offset*sizeof(MYFLT));
+              if (UNLIKELY(early)) {
+                nsmps -= early;
+                memset(&dst[nsmps], '\0', early*sizeof(MYFLT));
+              }
+              memcpy(&dst[offset], &src[offset], (nsmps-offset)*sizeof(MYFLT));
+              //for (j = 0; j < CS_KSMPS; j++)
+              //  dst[j] = src[j];
             }
             break;
           }
@@ -421,7 +451,8 @@ static int pop_opcode_init(CSOUND *csound, POP_OPCODE *p)
 {
     if (!p->initDone) {
       p->pp = csoundStack_GetGlobals(csound);
-      if (UNLIKELY(csoundStack_CreateArgMap(p, &(p->argMap[0]), 1) != OK))
+      if (UNLIKELY(csoundStack_CreateArgMap((PUSH_OPCODE*)p,
+                                            &(p->argMap[0]), 1) != OK))
         return NOTOK;
       p->h.opadr = (int (*)(CSOUND *, void *)) pop_opcode_perf;
       p->initDone = 1;
@@ -463,7 +494,8 @@ static int push_f_opcode_perf(CSOUND *csound, PUSH_OPCODE *p)
     int     *ofsp;
     int     offs;
 
-    if (UNLIKELY(p->pp->freeSpaceOffset + p->argMap[2] > p->pp->freeSpaceEndOffset))
+    if (UNLIKELY(p->pp->freeSpaceOffset + p->argMap[2] >
+                 p->pp->freeSpaceEndOffset))
       return csoundStack_OverflowError(p);
     bp = (void*) ((char*) p->pp->dataSpace + (int) p->pp->freeSpaceOffset);
     p->pp->freeSpaceOffset += p->argMap[2];
@@ -537,7 +569,7 @@ static int pop_f_opcode_perf(CSOUND *csound, POP_OPCODE *p)
     ofsp++;
     if (UNLIKELY(*ofsp != CS_STACK_END))
       csoundStack_TypeError(p);
-    fassign(csound,
+    fsg_assign(csound,
             (PVSDAT*) p->args[0],
             *((PVSDAT**) ((char*) bp + (int) (offs & (int) 0x00FFFFFF))));
     p->pp->curBundle = *((void**) bp);
@@ -582,7 +614,7 @@ static int pop_f_opcode_init(CSOUND *csound, POP_OPCODE *p)
     ofsp++;
     if (UNLIKELY(*ofsp != CS_STACK_END))
       csoundStack_TypeError(p);
-    fassign(csound,
+    fsg_assign(csound,
             (PVSDAT*) p->args[0],
             *((PVSDAT**) ((char*) bp + (int) (offs & (int) 0x00FFFFFF))));
     p->pp->curBundle = *((void**) bp);
@@ -591,64 +623,22 @@ static int pop_f_opcode_init(CSOUND *csound, POP_OPCODE *p)
     return OK;
 }
 
- /* ------------------------------------------------------------------------ */
 
-typedef struct MONITOR_OPCODE_ {
-    OPDS    h;
-    MYFLT   *ar[24];
-} MONITOR_OPCODE;
-
-static int monitor_opcode_perf(CSOUND *csound, MONITOR_OPCODE *p)
-{
-    int     i, j;
-
-    if (csound->spoutactive) {
-      int   k = 0;
-      i = 0;
-      do {
-        j = 0;
-        do {
-          p->ar[j][i] = csound->spout[k++];
-        } while (++j < csound->nchnls);
-      } while (++i < csound->ksmps);
-    }
-    else {
-      j = 0;
-      do {
-        i = 0;
-        do {
-          p->ar[j][i] = FL(0.0);
-        } while (++i < csound->ksmps);
-      } while (++j < csound->nchnls);
-    }
-    return OK;
-}
-
-static int monitor_opcode_init(CSOUND *csound, MONITOR_OPCODE *p)
-{
-    if (UNLIKELY(csound->GetOutputArgCnt(p) != csound->nchnls))
-      return csound->InitError(csound, Str("number of arguments != nchnls"));
-    p->h.opadr = (SUBR) monitor_opcode_perf;
-    return OK;
-}
 
  /* ------------------------------------------------------------------------ */
 
 static OENTRY stackops_localops[] = {
-  { "stack",  sizeof(STACK_OPCODE), SB|1,  "",                                "i",
+  { "stack",  sizeof(STACK_OPCODE), SK, 1,  "",                                "i",
       (SUBR) stack_opcode_init, (SUBR) NULL,                      (SUBR) NULL },
-  { "push",   sizeof(PUSH_OPCODE),  SB|3,  "",                                "N",
+  { "push",   sizeof(PUSH_OPCODE),  SK, 3,  "",                                "N",
       (SUBR) push_opcode_init,  (SUBR) notinit_opcode_stub_perf,  (SUBR) NULL },
-  { "pop",    sizeof(POP_OPCODE),   SB|3,  "NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN", "",
+  { "pop",    sizeof(POP_OPCODE),   SK, 3,  "NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN", "",
       (SUBR) pop_opcode_init,   (SUBR) notinit_opcode_stub_perf,  (SUBR) NULL },
-  { "push_f", sizeof(PUSH_OPCODE),  SB|3,  "",                                "f",
+  { "push_f", sizeof(PUSH_OPCODE),  SK, 3,  "",                                "f",
       (SUBR) push_f_opcode_init, (SUBR) notinit_opcode_stub_perf, (SUBR) NULL },
-  { "pop_f",  sizeof(POP_OPCODE),   SB|3,  "f",                               "",
-      (SUBR) pop_f_opcode_init,  (SUBR) notinit_opcode_stub_perf, (SUBR) NULL },
-  /* ----------------------------------------------------------------------- */
-  { "monitor",  sizeof(MONITOR_OPCODE), 3,  "mmmmmmmmmmmmmmmmmmmmmmmm", "",
-    (SUBR) monitor_opcode_init, (SUBR) notinit_opcode_stub_perf,  (SUBR) NULL }
+  { "pop_f",  sizeof(POP_OPCODE),   SK, 3,  "f",                               "",
+      (SUBR) pop_f_opcode_init,  (SUBR) notinit_opcode_stub_perf, (SUBR) NULL }
 };
 
-LINKAGE1(stackops_localops)
+LINKAGE_BUILTIN(stackops_localops)
 
