@@ -23,11 +23,11 @@
  * same terms as the Synthesis Tookit in C++ by Perry R. Cook and Gary P. Scavone.
  *
  * To compile these opcodes, copy the STK include, src, and rawwaves directories
- * to the csound5/Opcodes/stk directory as follows:
+ * to the csound6/Opcodes/stk directory as follows:
  *
- * csound5/Opcodes/stk/include
- * csound5/Opcodes/stk/src
- * csound5/Opcodes/stk/rawwaves
+ * csound6/Opcodes/stk/include
+ * csound6/Opcodes/stk/src
+ * csound6/Opcodes/stk/rawwaves
  *
  * Also, specify buildStkOpcodes=1 for SCons.
  *
@@ -80,6 +80,11 @@
 
 using namespace stk;
 
+#define __BUILDING_LIBCSOUND
+#include <csoundCore.h>
+#include <csGblMtx.h>
+#include <OpcodeBase.hpp>
+
 #include <cstdlib>
 #include <cstdio>
 #include <string>
@@ -88,10 +93,6 @@ using namespace stk;
 
 using namespace std;
 
-#include <OpcodeBase.hpp>
-
-#include "csGblMtx.h"
-
 static std::map<CSOUND *, std::vector<Instrmnt *> > &getStkInstances()
 {
     static std::map<CSOUND *, std::vector<Instrmnt *> > stkInstances;
@@ -148,11 +149,11 @@ public:
   {
       if(!instrument)
         {
-          Stk::setSampleRate(csound->esr);
+          Stk::setSampleRate(csound->GetSr(csound));
           instrument = new T();
           getStkInstances()[csound].push_back(instrument);
         }
-      ksmps = csound->ksmps;
+      ksmps = OpcodeBase< STKInstrumentAdapter<T> >::opds.insdshead->ksmps;
       instrument->noteOn(*ifrequency, *igain);
       released = false;
       oldkcontroller0 = -1.0;
@@ -175,6 +176,8 @@ public:
   }
   int kontrol(CSOUND *csound)
   {
+      uint32_t offset =
+        OpcodeBase< STKInstrumentAdapter<T> >::opds.insdshead->ksmps_offset;
       if(!released)
         {
           if(*kcontroller0 != oldkcontroller0 || *kvalue0 != oldkvalue0)
@@ -225,14 +228,15 @@ public:
               oldkcontroller7 = *kcontroller7;
               oldkvalue7 = *kvalue7;
             }
-          for(size_t i = 0; i < ksmps; i++)
+          memset(aoutput, '\0', offset*sizeof(MYFLT));
+          for(size_t i = offset; i < ksmps; i++)
             {
               aoutput[i] = instrument->tick();
             }
         }
       else
         {
-          // memset(aoutput, 0, ksmps*sizeof(MYFLT));
+          //memset(aoutput, 0, ksmps*sizeof(MYFLT));
           for(size_t i = 0; i < ksmps; i++)
             {
               aoutput[i] = 0;
@@ -291,11 +295,11 @@ public:
   int init(CSOUND *csound)
   {
       if(!instrument) {
-        Stk::setSampleRate(csound->esr);
+        Stk::setSampleRate(csound->GetSr(csound));
         instrument = new T((StkFloat) 10.0);
         getStkInstances()[csound].push_back(instrument);
       }
-      ksmps = csound->ksmps;
+      ksmps = OpcodeBase< STKInstrumentAdapter1<T> >::opds.insdshead->ksmps;
       instrument->noteOn(*ifrequency, *igain);
       released = false;
       oldkcontroller0 = -1.0;
@@ -318,6 +322,8 @@ public:
   }
   int kontrol(CSOUND *csound)
   {
+      uint32_t offset =
+        OpcodeBase< STKInstrumentAdapter1<T> >::opds.insdshead->ksmps_offset;
       if(!released)
         {
           if(*kcontroller0 != oldkcontroller0 || *kvalue0 != oldkvalue0)
@@ -368,7 +374,8 @@ public:
               oldkcontroller7 = *kcontroller7;
               oldkvalue7 = *kvalue7;
             }
-          for(size_t i = 0; i < ksmps; i++)
+          memset(aoutput, '\0', offset*sizeof(MYFLT));
+          for(size_t i = offset; i < ksmps; i++)
             {
               aoutput[i] = instrument->tick();
             }
@@ -392,6 +399,7 @@ extern "C"
       {
         (char*)"STKBandedWG",
         sizeof(STKInstrumentAdapter<BandedWG>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -402,6 +410,7 @@ extern "C"
       {
         (char*)"STKBeeThree",
         sizeof(STKInstrumentAdapter<BeeThree>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -412,6 +421,7 @@ extern "C"
       {
         (char*)"STKBlowBotl",
         sizeof(STKInstrumentAdapter<BlowBotl>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -422,6 +432,7 @@ extern "C"
       {
         (char*)"STKBlowHole",
         sizeof(STKInstrumentAdapter1<BlowHole>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -432,6 +443,7 @@ extern "C"
       {
         (char*)"STKBowed",
         sizeof(STKInstrumentAdapter1<Bowed>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -442,6 +454,7 @@ extern "C"
       {
         (char*)"STKBrass",
         sizeof(STKInstrumentAdapter1<Brass>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -452,6 +465,7 @@ extern "C"
       {
         (char*)"STKClarinet",
         sizeof(STKInstrumentAdapter1<Clarinet>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -462,6 +476,7 @@ extern "C"
       {
         (char*)"STKDrummer",
         sizeof(STKInstrumentAdapter<Drummer>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -472,6 +487,7 @@ extern "C"
       {
         (char*)"STKFlute",
         sizeof(STKInstrumentAdapter1<Flute>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -482,6 +498,7 @@ extern "C"
       {
         (char*)"STKFMVoices",
         sizeof(STKInstrumentAdapter<FMVoices>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -492,6 +509,7 @@ extern "C"
       {
         (char*)"STKHevyMetl",
         sizeof(STKInstrumentAdapter<HevyMetl>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -502,6 +520,7 @@ extern "C"
       {
         (char*)"STKMandolin",
         sizeof(STKInstrumentAdapter1<Mandolin>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -512,6 +531,7 @@ extern "C"
       {
         (char*)"STKModalBar",
         sizeof(STKInstrumentAdapter<ModalBar>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -522,6 +542,7 @@ extern "C"
       {
         (char*)"STKMoog",
         sizeof(STKInstrumentAdapter<Moog>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -532,6 +553,7 @@ extern "C"
       {
         (char*)"STKPercFlut",
         sizeof(STKInstrumentAdapter<PercFlut>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -542,6 +564,7 @@ extern "C"
       {
         (char*)"STKPlucked",
         sizeof(STKInstrumentAdapter1<Plucked>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -552,6 +575,7 @@ extern "C"
       {
         (char*)"STKResonate",
         sizeof(STKInstrumentAdapter<Resonate>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -562,6 +586,7 @@ extern "C"
       {
         (char*)"STKRhodey",
         sizeof(STKInstrumentAdapter<Rhodey>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -572,6 +597,7 @@ extern "C"
       {
         (char*)"STKSaxofony",
         sizeof(STKInstrumentAdapter1<Saxofony>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -582,6 +608,7 @@ extern "C"
       {
         (char*)"STKShakers",
         sizeof(STKInstrumentAdapter<Shakers>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -592,6 +619,7 @@ extern "C"
       {
         (char*)"STKSimple",
         sizeof(STKInstrumentAdapter<Simple>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -602,6 +630,7 @@ extern "C"
       {
         (char*)"STKSitar",
         sizeof(STKInstrumentAdapter<Sitar>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -612,6 +641,7 @@ extern "C"
       {
         (char*)"STKStifKarp",
         sizeof(STKInstrumentAdapter1<StifKarp>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -622,6 +652,7 @@ extern "C"
       {
         (char*)"STKTubeBell",
         sizeof(STKInstrumentAdapter<TubeBell>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -632,6 +663,7 @@ extern "C"
       {
         (char*)"STKVoicForm",
         sizeof(STKInstrumentAdapter<VoicForm>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -642,6 +674,7 @@ extern "C"
       {
         (char*)"STKWhistle",
         sizeof(STKInstrumentAdapter<Whistle>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -652,6 +685,7 @@ extern "C"
       {
         (char*)"STKWurley",
         sizeof(STKInstrumentAdapter<Wurley>),
+        0,
         3,
         (char*)"a",
         (char*)"iiJJJJJJJJJJJJJJJJ",
@@ -691,14 +725,15 @@ extern "C"
         csound_global_mutex_lock();
         Stk::setRawwavePath(path);
         csound_global_mutex_unlock();
-        csound->Message(csound,
-                        Str("RAWWAVE_PATH: %s\n"), Stk::rawwavePath().c_str());
+        csound->DebugMsg(csound,
+                         Str("RAWWAVE_PATH: %s\n"), Stk::rawwavePath().c_str());
       }
     int status = 0;
     for(OENTRY *oentry = &oentries[0]; oentry->opname; oentry++)
       {
         status |= csound->AppendOpcode(csound, oentry->opname,
-                                       oentry->dsblksiz, oentry->thread,
+                                       oentry->dsblksiz, oentry->flags,
+                                       oentry->thread,
                                        oentry->outypes, oentry->intypes,
                                        (int (*)(CSOUND*,void*)) oentry->iopadr,
                                        (int (*)(CSOUND*,void*)) oentry->kopadr,
@@ -720,4 +755,3 @@ extern "C"
   }
 
 }
-
