@@ -22,7 +22,7 @@
     02111-1307 USA
 */
 
-#include "csdl.h"               /*                      UGENS7.C        */
+#include "stdopcod.h"               /*                      UGENS7.C        */
 #include "ugens7.h"
 #include <math.h>
 
@@ -37,7 +37,7 @@ static int fofset0(CSOUND *csound, FOFS *p, int flag)
         (p->ftp2 = csound->FTFind(csound, p->ifnb)) != NULL) {
       OVRLAP *ovp, *nxtovp;
       int32  olaps;
-      p->durtogo = (int32)(*p->itotdur * csound->esr);
+      p->durtogo = (int32)(*p->itotdur * CS_ESR);
       if (!skip) { /* legato: skip all memory management */
         if (*p->iphs == FL(0.0))                /* if fundphs zero,  */
           p->fundphs = MAXLEN;                  /*   trigger new FOF */
@@ -89,7 +89,10 @@ static int fof(CSOUND *csound, FOFS *p)
     OVRLAP  *ovp;
     FUNC    *ftp1,  *ftp2;
     MYFLT   *ar, *amp, *fund, *form;
-    int32   n, nsmps = csound->ksmps, fund_inc, form_inc;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32   fund_inc, form_inc;
     MYFLT   v1, fract ,*ftab;
 
     if (UNLIKELY(p->auxch.auxp==NULL)) goto err1; /* RWD fix */
@@ -100,8 +103,13 @@ static int fof(CSOUND *csound, FOFS *p)
     ftp1 = p->ftp1;
     ftp2 = p->ftp2;
     fund_inc = (int32)(*fund * csound->sicvt);
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     form_inc = (int32)(*form * csound->sicvt);
-    for (n=0; n<nsmps; n++) {
+    for (n=offset; n<nsmps; n++) {
       if (p->fundphs & MAXLEN) {               /* if phs has wrapped */
         p->fundphs &= PHMASK;
         if ((ovp = p->basovrlap.nxtfree) == NULL) goto err2;
@@ -166,9 +174,11 @@ static int fof(CSOUND *csound, FOFS *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("fof: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("fof: not initialised"));
  err2:
-    return csound->PerfError(csound, Str("FOF needs more overlaps"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("FOF needs more overlaps"));
 }
 
 static int newpulse(CSOUND *csound,
@@ -177,7 +187,7 @@ static int newpulse(CSOUND *csound,
     MYFLT   octamp = *amp, oct;
     int32   rismps, newexp = 0;
 
-    if ((ovp->timrem = (int32)(*p->kdur * csound->esr)) > p->durtogo &&
+    if ((ovp->timrem = (int32)(*p->kdur * CS_ESR)) > p->durtogo &&
         (*p->iskip==FL(0.0))) /* ringtime */
       return(0);
     if ((oct = *p->koct) > FL(0.0)) {                   /* octaviation */
@@ -218,7 +228,7 @@ static int newpulse(CSOUND *csound,
     }
     ovp->curamp = octamp * p->preamp;                /* set startamp  */
     ovp->expamp = p->expamp;
-    if ((ovp->dectim = (int32)(*p->kdec * csound->esr)) > 0) /*  fnb dec  */
+    if ((ovp->dectim = (int32)(*p->kdec * CS_ESR)) > 0) /*  fnb dec  */
       ovp->decinc = (int32)(csound->sicvt / *p->kdec);
     ovp->decphs = PHMASK;
     if (!p->foftype) {
@@ -250,9 +260,9 @@ static int harmset(CSOUND *csound, HARMON *p)
       return csound->InitError(csound, Str("Minimum frequency too low"));
     }
     if (p->auxch.auxp == NULL || minfrq < p->minfrq) {
-      int32 nbufs = (int32)(csound->ekr * FL(3.0) / minfrq) + 1;
-      int32 nbufsmps = nbufs * csound->ksmps;
-      int32 maxprd = (int32)(csound->esr / minfrq);
+      int32 nbufs = (int32)(CS_EKR * FL(3.0) / minfrq) + 1;
+      int32 nbufsmps = nbufs * CS_KSMPS;
+      int32 maxprd = (int32)(CS_ESR / minfrq);
       int32 totalsiz = nbufsmps * 5 + maxprd; /* Surely 5! not 4 */
       csound->AuxAlloc(csound, (size_t)totalsiz * sizeof(MYFLT), &p->auxch);
       p->bufp = (MYFLT *) p->auxch.auxp;
@@ -265,7 +275,7 @@ static int harmset(CSOUND *csound, HARMON *p)
       p->lomaxdist = maxprd;
       p->minfrq = minfrq;
     }
-    if ((p->autoktim = MYFLT2LONG(*p->iptrkprd * csound->ekr)) < 1)
+    if ((p->autoktim = MYFLT2LONG(*p->iptrkprd * CS_EKR)) < 1)
       p->autoktim = 1;
     p->autokcnt = 1;              /* init for immediate autocorr attempt */
     p->lsicvt = FL(65536.0) * csound->onedsr;
@@ -295,7 +305,10 @@ static int harmon(CSOUND *csound, HARMON *p)
     MYFLT sum, minval, *minqp = NULL, *minq1, *minq2, *endp;
     MYFLT *pulstrt, lin1, lin2, lin3;
     int32  cnt1, cnt2, cnt3;
-    int32  nn, n, nsmps, phase1, phase2, phsinc1, phsinc2, period;
+    int32  nn, phase1, phase2, phsinc1, phsinc2, period;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
     inp1 = p->inp1;
     inp2 = p->inp2;
@@ -304,7 +317,7 @@ static int harmon(CSOUND *csound, HARMON *p)
     qval = p->prvq;
     if (*p->kest != p->prvest &&
         *p->kest != FL(0.0)) {    /* if new pitch estimate */
-      MYFLT estperiod = csound->esr / *p->kest;
+      MYFLT estperiod = CS_ESR / *p->kest;
       double b = 2.0 - cos((double)(*p->kest * csound->tpidsr));
       p->c2 = (MYFLT)(b - sqrt(b*b - 1.0)); /*   recalc lopass coefs */
       p->c1 = FL(1.0) - p->c2;
@@ -325,12 +338,12 @@ static int harmon(CSOUND *csound, HARMON *p)
     }
     c1 = p->c1;
     c2 = p->c2;
-    for (src1 = p->asig, nsmps = csound->ksmps; nsmps--; src1++) {
-      *inp1++ = *inp2++ = *src1;              /* dbl store the wavform */
-      if (*src1 > FL(0.0))
-        qval = c1 * *src1 + c2 * qval;        /*  & its half-wave rect */
+    for (src1 = p->asig, n = offset; n<nsmps; n++) {
+      inp1[n] = inp2[n] = src1[n];            /* dbl store the wavform */
+      if (src1[n] > FL(0.0))
+        qval = c1 * src1[n] + c2 * qval;      /*  & its half-wave rect */
       else qval = c2 * qval;
-      *inq1++ = *inq2++ = qval;
+      inq1[n] = inq2[n] = qval;
     }
     if (!(--p->autokcnt)) {                   /* if time for new autocorr  */
       MYFLT *mid1, *mid2, *src4;
@@ -398,10 +411,10 @@ static int harmon(CSOUND *csound, HARMON *p)
     if (period==0) {
       csound->Warning(csound, Str("Period zero\n"));
       outp = p->ar;
-      memset(outp, 0, sizeof(MYFLT)*csound->ksmps);
+      memset(outp, 0, sizeof(MYFLT)*CS_KSMPS);
       return OK;
     }
-    while (src1 + csound->ksmps > inp2)     /* if not enough smps presnt */
+    while (src1 + CS_KSMPS > inp2)     /* if not enough smps presnt */
       src1 -= period;                       /*      back up 1 prd        */
     pulstrt = src1;                         /* curr available pulse beg  */
 
@@ -419,8 +432,12 @@ static int harmon(CSOUND *csound, HARMON *p)
     phsinc1 = (int32)(*p->kfrq1 * p->lsicvt);
     phsinc2 = (int32)(*p->kfrq2 * p->lsicvt);
     outp = p->ar;
-    nsmps = csound->ksmps;
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(outp, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&outp[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       MYFLT sum;
       if (src1 != NULL) {
         if (++cnt1 < p->pnt11) {
@@ -581,9 +598,9 @@ static int harmon(CSOUND *csound, HARMON *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "fof",    S(FOFS),   TR|5, "a","xxxkkkkkiiiiooo",(SUBR)fofset,NULL,(SUBR)fof   },
-{ "fof2",   S(FOFS),   TR|5, "a","xxxkkkkkiiiikko",(SUBR)fofset2,NULL,(SUBR)fof  },
-{ "harmon", S(HARMON), 5, "a",  "akkkkiii",(SUBR)harmset,NULL,  (SUBR)harmon  }
+{ "fof",    S(FOFS),   TR, 5, "a","xxxkkkkkiiiiooo",(SUBR)fofset,NULL,(SUBR)fof   },
+{ "fof2",   S(FOFS),   TR, 5, "a","xxxkkkkkiiiikko",(SUBR)fofset2,NULL,(SUBR)fof  },
+{ "harmon", S(HARMON), 0,5, "a",  "akkkkiii",(SUBR)harmset,NULL,  (SUBR)harmon  }
 };
 
 int ugens7_init_(CSOUND *csound)
