@@ -21,30 +21,37 @@
     02111-1307 USA
 */
 
-#include "csdl.h"   /*                                      UGENS9.C        */
+#include "stdopcod.h"   /*                                      UGENS9.C        */
 #include <math.h>
 #include "convolve.h"
 #include "ugens9.h"
 #include "soundio.h"
-#include "oload.h"
 
-static int cvset(CSOUND *csound, CONVOLVE *p)
+static int cvset_(CSOUND *csound, CONVOLVE *p, int stringname)
 {
     char     cvfilnam[MAXNAME];
     MEMFIL   *mfp;
-    MYFLT *fltp;
+    MYFLT    *fltp;
     CVSTRUCT *cvh;
-    int siz;
+    int       siz;
     int32     Hlenpadded = 1, obufsiz, Hlen;
-    int      nchanls;
+    uint32_t  nchanls;
+    uint32_t  nsmps = CS_KSMPS;
 
     if (UNLIKELY(csound->oparms->odebug))
       csound->Message(csound, CONVOLVE_VERSION_STRING);
 
-    csound->strarg2name(csound, cvfilnam, p->ifilno, "convolve.", p->XSTRCODE);
+    if (stringname==0){
+      if (ISSTRCOD(*p->ifilno))
+        strncpy(cvfilnam,get_arg_string(csound, *p->ifilno), MAXNAME-1);
+      else csound->strarg2name(csound, cvfilnam,p->ifilno, "convolve.",0);
+    }
+    else strncpy(cvfilnam, ((STRINGDAT *)p->ifilno)->data, MAXNAME-1);
+
     if ((mfp = p->mfp) == NULL || strcmp(mfp->filename, cvfilnam) != 0) {
       /* if file not already readin */
-      if (UNLIKELY((mfp = csound->ldmemfile2(csound, cvfilnam, CSFTYPE_CVANAL))
+      if (UNLIKELY((mfp = csound->ldmemfile2withCB(csound, cvfilnam,
+                                                   CSFTYPE_CVANAL,NULL))
                    == NULL)) {
         return csound->InitError(csound,
                                  Str("CONVOLVE cannot load %s"), cvfilnam);
@@ -92,10 +99,10 @@ static int cvset(CSOUND *csound, CONVOLVE *p)
     if ((p->nchanls == 1) && (*p->channel > 0))
       p->H += (Hlenpadded + 2) * (int)(*p->channel - 1);
 
-    if (UNLIKELY(cvh->samplingRate != csound->esr)) {
+    if (UNLIKELY(cvh->samplingRate != CS_ESR)) {
       /* & chk the data */
       csound->Warning(csound, Str("%s's srate = %8.0f, orch's srate = %8.0f"),
-                              cvfilnam, cvh->samplingRate, csound->esr);
+                              cvfilnam, cvh->samplingRate, CS_ESR);
     }
     if (UNLIKELY(cvh->dataFormat != CVMYFLT)) {
       return csound->InitError(csound,
@@ -105,11 +112,10 @@ static int cvset(CSOUND *csound, CONVOLVE *p)
     }
 
     /* Determine size of circular output buffer */
-    if (Hlen >= csound->ksmps)
-      obufsiz = (int32) CEIL((MYFLT) Hlen / csound->ksmps)
-                * csound->ksmps;
+    if (Hlen >= (int32)nsmps)
+      obufsiz = (int32) CEIL((MYFLT) Hlen / nsmps) * nsmps;
     else
-      obufsiz = (int32) CEIL(csound->ksmps / (MYFLT) Hlen) * Hlen;
+      obufsiz = (int32) CEIL(CS_KSMPS / (MYFLT) Hlen) * Hlen;
 
     siz = ((Hlenpadded + 2) + p->nchanls * ((Hlen - 1) + obufsiz)
               + (p->nchanls > 1 ? (Hlenpadded + 2) : 0));
@@ -135,6 +141,15 @@ static int cvset(CSOUND *csound, CONVOLVE *p)
     return OK;
 }
 
+static int cvset(CSOUND *csound, CONVOLVE *p){
+  return cvset_(csound,p,0);
+
+}
+
+static int cvset_S(CSOUND *csound, CONVOLVE *p){
+  return cvset_(csound,p,1);
+
+}
 /* Write from a circular buffer into a linear output buffer without
    clearing data
    UPDATES SOURCE & DESTINATION POINTERS TO REFLECT NEW POSITIONS */
@@ -167,7 +182,7 @@ static void writeFromCircBuf(
 
 static int convolve(CSOUND *csound, CONVOLVE *p)
 {
-    int    nsmpso=csound->ksmps,nsmpsi=csound->ksmps,nsmpso_sav,outcnt_sav;
+    int    nsmpso=CS_KSMPS,nsmpsi=CS_KSMPS,outcnt_sav;
     int    nchm1 = p->nchanls - 1,chn;
     int32  i,j;
     MYFLT  *ar[4];
@@ -184,6 +199,9 @@ static int convolve(CSOUND *csound, CONVOLVE *p)
     MYFLT  *X;
     int32  Hlenpadded = p->Hlenpadded;
     MYFLT  scaleFac;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t nn,nsmpso_sav;
 
     scaleFac = csound->GetInverseRealFFTScale(csound, (int) Hlenpadded);
     ar[0] = p->ar1;
@@ -194,10 +212,10 @@ static int convolve(CSOUND *csound, CONVOLVE *p)
     if (p->auxch.auxp==NULL) goto err1;
   /* First dump as much pre-existing audio in output buffer as possible */
     if (outcnt > 0) {
-      if (outcnt <= csound->ksmps)
+      if (outcnt <= (int)CS_KSMPS)
         i = outcnt;
       else
-        i = csound->ksmps;
+        i = CS_KSMPS;
       nsmpso -= i; outcnt -= i;
       for (chn = nchm1;chn >= 0;chn--) {
         outhead = p->outhead + chn*obufsiz;
@@ -216,8 +234,13 @@ static int convolve(CSOUND *csound, CONVOLVE *p)
         i = Hlen - incount;
       nsmpsi -= i;
       incount += i;
-      while (i--)
-        *fftbufind++ = scaleFac * *ai++;
+      nsmpso_sav = CS_KSMPS-early;
+      for (nn=0; i>0; nn++, i--) {
+        if (nn<offset|| nn>(uint32_t)nsmpso_sav)
+          *fftbufind++ = FL(0.0);
+        else
+          *fftbufind++ = scaleFac * ai[nn];
+      }
       if (incount == Hlen) {
         /* We have enough audio for a convolution. */
         incount = 0;
@@ -338,7 +361,8 @@ static int convolve(CSOUND *csound, CONVOLVE *p)
     p->outail = outail;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("convolve: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("convolve: not initialised"));
 }
 
 /* partitioned (low latency) overlap-save convolution.
@@ -350,7 +374,7 @@ static int convolve(CSOUND *csound, CONVOLVE *p)
    allow this opcode to accept .con files.
    -ma++ april 2004 */
 
-static int pconvset(CSOUND *csound, PCONVOLVE *p)
+static int pconvset_(CSOUND *csound, PCONVOLVE *p, int stringname)
 {
     int     channel = (*(p->channel) <= 0 ? ALLCHNLS : (int) *(p->channel));
     SNDFILE *infd;
@@ -365,8 +389,14 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
     memset(&IRfile, 0, sizeof(SOUNDIN));
     /* open impulse response soundfile [code derived from SAsndgetset()] */
     IRfile.skiptime = FL(0.0);
-    csound->strarg2name(csound, IRfile.sfname, p->ifilno, "soundin.",
-                                p->XSTRCODE);
+
+     if (stringname==0){
+      if (ISSTRCOD(*p->ifilno))
+        strncpy(IRfile.sfname,get_arg_string(csound, *p->ifilno), 511);
+      else csound->strarg2name(csound, IRfile.sfname, p->ifilno, "soundin.",0);
+    }
+    else strncpy(IRfile.sfname, ((STRINGDAT *)p->ifilno)->data, 511);
+
     IRfile.sr = 0;
     if (UNLIKELY(channel < 1 || ((channel > 4) && (channel != ALLCHNLS)))) {
       return csound->InitError(csound, Str("channel request %d illegal"), channel);
@@ -391,19 +421,19 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
                             (long) IRfile.getframes, ainput_dur);
 
     p->nchanls = (channel != ALLCHNLS ? 1 : IRfile.nchanls);
-    if (UNLIKELY(p->nchanls != p->OUTOCOUNT)) {
+    if (UNLIKELY(p->nchanls != (int)p->OUTOCOUNT)) {
       return csound->InitError(csound, Str("PCONVOLVE: number of output channels "
                                            "not equal to input channels"));
     }
 
-    if (UNLIKELY(IRfile.sr != csound->esr)) {
+    if (UNLIKELY(IRfile.sr != CS_ESR)) {
       /* ## RWD suggests performing sr conversion here! */
-      csound->Warning(csound, "IR srate != orch's srate");
+      csound->Warning(csound, Str("IR srate != orch's srate"));
     }
 
     /* make sure the partition size is nonzero and a power of 2  */
     if (*p->partitionSize <= 0)
-      partitionSize = csound->oparms->outbufsamps / csound->nchnls;
+      partitionSize = csound->oparms->outbufsamps / csound->GetNchnls(csound);
     else
       partitionSize = *p->partitionSize;
 
@@ -429,7 +459,8 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
          handles finding the right channel */
       if (UNLIKELY((read_in = csound->getsndin(csound, infd, inbuf,
                                                p->Hlen*p->nchanls, &IRfile)) <= 0))
-        csound->Die(csound, Str("PCONVOLVE: less sound than expected!"));
+        return csound->InitError(csound,
+                                 Str("PCONVOLVE: less sound than expected!"));
 
       /* take FFT of each channel */
       scaleFac = csound->dbfs_to_float
@@ -466,7 +497,7 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
 
     /* allocate circular output sample buffer */
     p->outBufSiz = sizeof(MYFLT) * p->nchanls *
-      (p->Hlen >= csound->ksmps ? p->Hlenpadded : 2*csound->ksmps);
+      (p->Hlen >= (int)CS_KSMPS ? p->Hlenpadded : 2*(int)CS_KSMPS);
     csound->AuxAlloc(csound, p->outBufSiz, &p->output);
     p->outRead = (MYFLT *)p->output.auxp;
 
@@ -474,8 +505,8 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
        empty ksmps pass after a few initial generated buffers.  There is
        probably an equation to figure this out to reduce the delay, but
        I can't seem to figure it out */
-    if (p->Hlen > csound->ksmps) {
-      p->outCount = p->Hlen + csound->ksmps;
+    if (p->Hlen > (int)CS_KSMPS) {
+      p->outCount = p->Hlen + CS_KSMPS;
       p->outWrite = p->outRead + (p->nchanls * p->outCount);
     }
     else {
@@ -485,9 +516,19 @@ static int pconvset(CSOUND *csound, PCONVOLVE *p)
     return OK;
 }
 
+static int pconvset(CSOUND *csound, PCONVOLVE *p){
+  return pconvset_(csound,p,0);
+}
+
+static int pconvset_S(CSOUND *csound, PCONVOLVE *p){
+  return pconvset_(csound,p,1);
+}
+
 static int pconvolve(CSOUND *csound, PCONVOLVE *p)
 {
-    int    nsmpsi = csound->ksmps;
+    uint32_t nn, nsmps = CS_KSMPS;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = nsmps - p->h.insdshead->ksmps_no_end;
     MYFLT  *ai = p->ain;
     MYFLT  *buf;
     MYFLT  *input = (MYFLT*) p->savedInput.auxp, *workWrite = p->workWrite;
@@ -495,9 +536,9 @@ static int pconvolve(CSOUND *csound, PCONVOLVE *p)
     int32  i, j, count = p->inCount;
     int32  hlenpaddedplus2 = p->Hlenpadded+2;
 
-    while (nsmpsi-- > 0) {
+    for (nn=0; nn<nsmps; nn++) {
       /* Read input audio and place into buffer. */
-      input[count++] = *workWrite++ = *ai++;
+      input[count++] = *workWrite++ = (nn<offset||nn>early? FL(0.0) : ai[nn]);
 
       /* We have enough audio for a convolution. */
       if (count == p->Hlen) {
@@ -568,10 +609,10 @@ static int pconvolve(CSOUND *csound, PCONVOLVE *p)
 
     /* copy to output if we have enough samples [we always should
        except the first Hlen samples] */
-    if (p->outCount >= csound->ksmps) {
-      int n;
-      p->outCount -= csound->ksmps;
-      for (n=0; n < csound->ksmps; n++) {
+    if (p->outCount >= (int)CS_KSMPS) {
+      unsigned int n;
+      p->outCount -= CS_KSMPS;
+      for (n=0; n < CS_KSMPS; n++) {
         switch (p->nchanls) {
         case 1:
           *a1++ = *p->outRead++;
@@ -604,11 +645,17 @@ static int pconvolve(CSOUND *csound, PCONVOLVE *p)
 }
 
 static OENTRY localops[] = {
-    { "convolve", sizeof(CONVOLVE),   5, "mmmm", "aTo",
+    { "convolve", sizeof(CONVOLVE),   0, 5, "mmmm", "aSo",
+            (SUBR) cvset_S,     (SUBR) NULL,    (SUBR) convolve   },
+    { "convle",   sizeof(CONVOLVE),   0, 5, "mmmm", "aSo",
+            (SUBR) cvset_S,     (SUBR) NULL,    (SUBR) convolve   },
+    { "pconvolve",sizeof(PCONVOLVE),  0, 5, "mmmm", "aSoo",
+      (SUBR) pconvset_S,  (SUBR) NULL,    (SUBR) pconvolve  },
+     { "convolve.i", sizeof(CONVOLVE),   0, 5, "mmmm", "aio",
             (SUBR) cvset,     (SUBR) NULL,    (SUBR) convolve   },
-    { "convle",   sizeof(CONVOLVE),   5, "mmmm", "aTo",
+    { "convle.i",   sizeof(CONVOLVE),   0, 5, "mmmm", "aio",
             (SUBR) cvset,     (SUBR) NULL,    (SUBR) convolve   },
-    { "pconvolve",sizeof(PCONVOLVE),  5, "mmmm", "aToo",
+    { "pconvolve.i",sizeof(PCONVOLVE),  0, 5, "mmmm", "aioo",
             (SUBR) pconvset,  (SUBR) NULL,    (SUBR) pconvolve  }
 };
 
@@ -617,4 +664,3 @@ int ugens9_init_(CSOUND *csound)
     return csound->AppendOpcodes(csound, &(localops[0]),
                                  (int) (sizeof(localops) / sizeof(OENTRY)));
 }
-
