@@ -21,7 +21,7 @@
     02111-1307 USA
 */
 
-#include "csdl.h"                 /*                              UGENSA.C  */
+#include "stdopcod.h"                 /*                              UGENSA.C  */
 #include "ugensa.h"
 #include "ugens7.h"
 #include <math.h>
@@ -39,7 +39,7 @@ static int fogset(CSOUND *csound, FOGS *p)
       OVERLAP *ovp, *nxtovp;
       int32   olaps;
       p->fogcvt = FMAXLEN/(p->ftp1)->flen; /*JMC for FOG*/
-      p->durtogo = (int32)(*p->itotdur * csound->esr);
+      p->durtogo = (int32)(*p->itotdur * CS_ESR);
       if (!skip) { /* legato: skip all memory management */
         p->spdphs = 0L; /*JMC for FOG*/
         if (*p->iphs == FL(0.0))                  /* if fundphs zero,  */
@@ -82,7 +82,10 @@ static int fog(CSOUND *csound, FOGS *p)
     FUNC        *ftp1,  *ftp2;
     MYFLT       *ar, *amp, *fund, *ptch, *speed;
     MYFLT  v1, fract ,*ftab, fogcvt = p->fogcvt; /*JMC added for FOG*/
-    int32   n,nsmps = csound->ksmps, fund_inc, form_inc;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
+    int32   fund_inc, form_inc;
     /* long speed_inc; */ /*JMC added last--out for phs version*/
 
     ar = p->ar;
@@ -95,7 +98,12 @@ static int fog(CSOUND *csound, FOGS *p)
     fund_inc = (int32)(*fund * csound->sicvt);
     form_inc = (int32)(*ptch * fogcvt);  /*form_inc = *form * csound->sicvt;*/
 /*      speed_inc = *speed * fogcvt; */   /*JMC for FOG--out for phs version*/
-    for (n=0;n<nsmps;n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset;n<nsmps;n++) {
       if (p->fundphs & MAXLEN) {                       /* if phs has wrapped */
         p->fundphs &= PHMASK;
         if (UNLIKELY((ovp = p->basovrlap.nxtfree) == NULL)) goto err1;
@@ -154,7 +162,8 @@ static int fog(CSOUND *csound, FOGS *p)
     }
     return OK;
  err1:
-    return csound->PerfError(csound, Str("FOF needs more overlaps"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("FOF needs more overlaps"));
 }
 
 static int newpulse(CSOUND *csound, FOGS *p, OVERLAP *ovp, MYFLT   *amp,
@@ -163,7 +172,7 @@ static int newpulse(CSOUND *csound, FOGS *p, OVERLAP *ovp, MYFLT   *amp,
     MYFLT       octamp = *amp, oct;
     MYFLT       form = *ptch / csound->sicvt, fogcvt = p->fogcvt;
     int32   rismps, newexp = 0;
-    if ((ovp->timrem = (int32)(*p->kdur * csound->esr)) > p->durtogo &&
+    if ((ovp->timrem = (int32)(*p->kdur * CS_ESR)) > p->durtogo &&
         (*p->iskip==FL(0.0)))  /* ringtime    */
       return(0);
     if ((oct = *p->koct) > 0.0) {                   /* octaviation */
@@ -208,7 +217,7 @@ static int newpulse(CSOUND *csound, FOGS *p, OVERLAP *ovp, MYFLT   *amp,
     }
     ovp->curamp = octamp * p->preamp;                /* set startamp  */
     ovp->expamp = p->expamp;
-    if ((ovp->dectim = (int32)(*p->kdec * csound->esr)) > 0) /*      fnb dec  */
+    if ((ovp->dectim = (int32)(*p->kdec * CS_ESR)) > 0) /*      fnb dec  */
       ovp->decinc = (int32)(csound->sicvt / *p->kdec);
     ovp->decphs = PHMASK;
     return(1);
@@ -218,7 +227,7 @@ static int newpulse(CSOUND *csound, FOGS *p, OVERLAP *ovp, MYFLT   *amp,
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "fog",  S(FOGS), TR|5, "a","xxxakkkkkiiiiooo",(SUBR)fogset,NULL,(SUBR)fog}
+  { "fog",  S(FOGS), TR, 5, "a","xxxakkkkkiiiiooo",(SUBR)fogset,NULL,(SUBR)fog}
 };
 
 int ugensa_init_(CSOUND *csound)
