@@ -22,7 +22,7 @@
 */
 
                                                         /* ugmoss.c */
-#include "csdl.h"
+#include "stdopcod.h"
 #include "ugmoss.h"
 #include "aops.h"
 #include <math.h>
@@ -40,7 +40,8 @@ static int dconvset(CSOUND *csound, DCONV *p)
     FUNC *ftp;
 
     p->len = (int)*p->isize;
-    if (LIKELY((ftp = csound->FTFind(csound, p->ifn)) != NULL)) {   /* find table */
+    if (LIKELY((ftp = csound->FTnp2Find(csound,
+                                        p->ifn)) != NULL)) {   /* find table */
       p->ftp = ftp;
       if ((unsigned)ftp->flen < p->len)
         p->len = ftp->flen; /* correct len if flen shorter */
@@ -49,8 +50,10 @@ static int dconvset(CSOUND *csound, DCONV *p)
       return csound->InitError(csound, Str("No table for dconv"));
     }
     if (p->sigbuf.auxp == NULL ||
-        p->sigbuf.size < (int)(p->len*sizeof(MYFLT)))
+        p->sigbuf.size < (unsigned int)(p->len*sizeof(MYFLT)))
       csound->AuxAlloc(csound, p->len*sizeof(MYFLT), &p->sigbuf);
+    else
+      memset(p->sigbuf.auxp, '\0', p->len*sizeof(MYFLT));
     p->curp = (MYFLT *)p->sigbuf.auxp;
     return OK;
 }
@@ -58,7 +61,9 @@ static int dconvset(CSOUND *csound, DCONV *p)
 static int dconv(CSOUND *csound, DCONV *p)
 {
     int32 i = 0;
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32 len = p->len;
     MYFLT *ar, *ain, *ftp, *startp, *endp, *curp;
     MYFLT sum;
@@ -70,14 +75,19 @@ static int dconv(CSOUND *csound, DCONV *p)
     endp = startp + len;
     curp = p->curp;
 
-    for (n=0; n<nsmps; n++) {
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n=offset; n<nsmps; n++) {
       *curp = ain[n];                           /* get next input sample */
-      i = 1, sum = *curp++ * *ftp;
+      i = 1, sum = *curp++ * ftp[0];
       while (curp<endp)
-        sum += (*curp++ * *(ftp + i++));        /* start the convolution */
+        sum += (*curp++ * ftp[i++]);            /* start the convolution */
       curp = startp;                            /* correct the ptr */
       while (i<len)
-        sum += (*curp++ * *(ftp + i++));        /* finish the convolution */
+        sum += (*curp++ * ftp[i++]);            /* finish the convolution */
       if (--curp < startp)
         curp += len;                            /* correct for last curp++ */
       ar[n] = sum;
@@ -100,7 +110,7 @@ static int and_aa(CSOUND *csound, AOP *p)
     MYFLT *r    = p->r;
     MYFLT *in1  = p->a;
     MYFLT *in2  = p->b;
-    int   n, nsmps = csound->ksmps;
+    int   n, nsmps = CS_KSMPS;
     int32  input1, input2;
 
     for (n = 0; n < nsmps; n++) {
@@ -115,10 +125,17 @@ static int and_ak(CSOUND *csound, AOP *p)
 {
     MYFLT *r = p->r;
     MYFLT *in1 = p->a;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32  input2 = MYFLT2LRND(*p->b), input1;
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input1 = MYFLT2LRND(in1[n]);
       r[n] = (MYFLT)(input1 & input2);
     }
@@ -129,10 +146,17 @@ static int and_ka(CSOUND *csound, AOP *p)
 {
     MYFLT *r = p->r;
     MYFLT *in2 = p->b;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32  input2, input1 = MYFLT2LRND(*p->a);
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input2 = MYFLT2LRND(in2[n]);
       r[n] = (MYFLT)(input1 & input2);
     }
@@ -152,10 +176,17 @@ static int or_aa(CSOUND *csound, AOP *p)
     MYFLT *r = p->r;
     MYFLT *in1 = p->a;
     MYFLT *in2 = p->b;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32  input2, input1;
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input1 = MYFLT2LRND(in1[n]);
       input2 = MYFLT2LRND(in2[n]);
       r[n] = (MYFLT)(input1 | input2);
@@ -167,10 +198,17 @@ static int or_ak(CSOUND *csound, AOP *p)
 {
     MYFLT *r = p->r;
     MYFLT *in1 = p->a;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32  input2 = MYFLT2LRND(*p->b), input1;
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input1 = MYFLT2LRND(in1[n]);
       r[n] = (MYFLT)(input1 | input2);
     }
@@ -181,10 +219,17 @@ static int or_ka(CSOUND *csound, AOP *p)
 {
     MYFLT *r = p->r;
     MYFLT *in2 = p->b;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32  input2, input1 = MYFLT2LRND(*p->a);
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input2 = MYFLT2LRND(in2[n]);
       r[n] = (MYFLT)(input1 | input2);
     }
@@ -204,9 +249,16 @@ static int xor_aa(CSOUND *csound, AOP *p)
     MYFLT *r = p->r;
     MYFLT *in1 = p->a;
     MYFLT *in2 = p->b;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32  input2, input1;
 
+    if (UNLIKELY(offset)) memset(r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r[nsmps], '\0', early*sizeof(MYFLT));
+    }
     for (n = 0; n < nsmps; n++) {
       input1 = MYFLT2LRND(in1[n]);
       input2 = MYFLT2LRND(in2[n]);
@@ -219,10 +271,17 @@ static int xor_ak(CSOUND *csound, AOP *p)
 {
     MYFLT *r = p->r;
     MYFLT *in1 = p->a;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32  input2 = MYFLT2LRND(*p->b), input1;
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input1 = MYFLT2LRND(in1[n]);
       r[n] = (MYFLT)(input1 ^ input2);
     }
@@ -233,10 +292,17 @@ static int xor_ka(CSOUND *csound, AOP *p)
 {
     MYFLT *r = p->r;
     MYFLT *in2 = p->b;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32  input2, input1 = MYFLT2LRND(*p->a);
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input2 = MYFLT2LRND(in2[n]);
       r[n] = (MYFLT)(input1 ^ input2);
     }
@@ -253,12 +319,19 @@ static int shift_left_kk(CSOUND *csound, AOP *p)
 
 static int shift_left_aa(CSOUND *csound, AOP *p)
 {
-    int32  input1;
-    int   input2, n, nsmps = csound->ksmps;
+    int32  input1, input2;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(p->r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input1 = MYFLT2LRND(p->a[n]);
-      input2 = (int) MYFLT2LRND(p->b[n]);
+      input2 = (int32) MYFLT2LRND(p->b[n]);
       p->r[n] = (MYFLT) (input1 << input2);
     }
     return OK;
@@ -268,9 +341,16 @@ static int shift_left_ak(CSOUND *csound, AOP *p)
 {
     int32  input1;
     int   input2 = MYFLT2LRND(*p->b);
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(p->r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input1 = MYFLT2LRND(p->a[n]);
       p->r[n] = (MYFLT) (input1 << input2);
     }
@@ -279,10 +359,17 @@ static int shift_left_ak(CSOUND *csound, AOP *p)
 
 static int shift_left_ka(CSOUND *csound, AOP *p)
 {
-    int32  input1 = MYFLT2LRND(*p->a);
-    int   input2, n, nsmps = csound->ksmps;
+    int32  input1 = MYFLT2LRND(*p->a), input2;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(p->r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input2 = MYFLT2LRND(p->b[n]);
       p->r[n] = (MYFLT) (input1 << input2);
     }
@@ -299,10 +386,17 @@ static int shift_right_kk(CSOUND *csound, AOP *p)
 
 static int shift_right_aa(CSOUND *csound, AOP *p)
 {
-    int32  input1;
-    int   input2, n, nsmps = csound->ksmps;
+    int32  input1, input2;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(p->r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input1 = MYFLT2LRND(p->a[n]);
       input2 = (int) MYFLT2LRND(p->b[n]);
       p->r[n] = (MYFLT) (input1 >> input2);
@@ -313,10 +407,17 @@ static int shift_right_aa(CSOUND *csound, AOP *p)
 static int shift_right_ak(CSOUND *csound, AOP *p)
 {
     int32  input1;
-    int   input2 = MYFLT2LRND(*p->b);
-    int   n, nsmps = csound->ksmps;
+    int32   input2 = MYFLT2LRND(*p->b);
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(p->r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input1 = MYFLT2LRND(p->a[n]);
       p->r[n] = (MYFLT) (input1 >> input2);
     }
@@ -325,10 +426,17 @@ static int shift_right_ak(CSOUND *csound, AOP *p)
 
 static int shift_right_ka(CSOUND *csound, AOP *p)
 {
-    int32  input1 = MYFLT2LRND(*p->a);
-    int   input2, n, nsmps = csound->ksmps;
+    int32  input1 = MYFLT2LRND(*p->a), input2;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(p->r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&p->r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (n = offset; n < nsmps; n++) {
       input2 = MYFLT2LRND(p->b[n]);
       p->r[n] = (MYFLT) (input1 >> input2);
     }
@@ -346,10 +454,17 @@ static int not_a(CSOUND *csound, AOP *p)
 {
     MYFLT *r = p->r;
     MYFLT *in1 = p->a;
-    int   n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     int32  input1;
 
-    for (n = 0; n < nsmps; n++) {
+    if (UNLIKELY(offset)) memset(r, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&r[nsmps], '\0', early*sizeof(MYFLT));
+    }
+     for (n = offset; n < nsmps; n++) {
       input1 = MYFLT2LRND(in1[n]);
       r[n] = (MYFLT)(~input1);
     }
@@ -368,11 +483,11 @@ static int vcombset(CSOUND *csound, VCOMB *p)
         return csound->InitError(csound, Str("illegal loop time"));
       }
     }
-    else if (UNLIKELY((lpsiz = (int32)(*p->imaxlpt * csound->esr)) <= 0)) {
+    else if (UNLIKELY((lpsiz = (int32)(*p->imaxlpt * CS_ESR)) <= 0)) {
       return csound->InitError(csound, Str("illegal loop time"));
     }
     nbytes = lpsiz * sizeof(MYFLT);
-    if (p->auxch.auxp == NULL || nbytes != p->auxch.size) {
+    if (p->auxch.auxp == NULL || nbytes != (int)p->auxch.size) {
       csound->AuxAlloc(csound, (size_t)nbytes, &p->auxch);
       p->pntr = (MYFLT *) p->auxch.auxp;
       if (UNLIKELY(p->pntr==NULL)) {
@@ -391,14 +506,16 @@ static int vcombset(CSOUND *csound, VCOMB *p)
     p->lpt = FL(0.0);
     p->g   = FL(0.0);
     p->lpta = (XINARG3) ? 1 : 0;
-    if (*p->insmps == 0) p->maxlpt = *p->imaxlpt * csound->esr;
+    if (*p->insmps == 0) p->maxlpt = *p->imaxlpt * CS_ESR;
     else p->maxlpt = *p->imaxlpt;
     return OK;
 }
 
 static int vcomb(CSOUND *csound, VCOMB *p)
 {
-    int         n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t    n, nsmps = CS_KSMPS;
     uint32      xlpt, maxlpt = (uint32)p->maxlpt;
     MYFLT       *ar, *asig, *rp, *endp, *startp, *wp, *lpt;
     MYFLT       g = p->g;
@@ -409,10 +526,15 @@ static int vcomb(CSOUND *csound, VCOMB *p)
     endp = (MYFLT *) p->auxch.endp;
     startp = (MYFLT *) p->auxch.auxp;
     wp = p->pntr;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->lpta) {                               /* if xlpt is a-rate */
       lpt = p->xlpt;
-      for (n=0; n<nsmps; n++) {
-        xlpt = (uint32)((*p->insmps != 0) ? *lpt : *lpt * csound->esr);
+      for (n=offset; n<nsmps; n++) {
+        xlpt = (uint32)((*p->insmps != 0) ? *lpt : *lpt * CS_ESR);
         if (xlpt > maxlpt) xlpt = maxlpt;
         if ((rp = wp - xlpt) < startp) rp += maxlpt;
         if ((p->rvt != *p->krvt) || (p->lpt != *lpt)) {
@@ -428,14 +550,14 @@ static int vcomb(CSOUND *csound, VCOMB *p)
     }
     else {                                       /* if xlpt is k-rate */
       xlpt = (uint32) ((*p->insmps != 0) ? *p->xlpt
-                                                  : *p->xlpt * csound->esr);
+                                                  : *p->xlpt * CS_ESR);
       if (xlpt > maxlpt) xlpt = maxlpt;
       if ((rp = wp - xlpt) < startp) rp += maxlpt;
       if ((p->rvt != *p->krvt) || (p->lpt != *p->xlpt)) {
         p->rvt = *p->krvt, p->lpt = *p->xlpt;
         g = p->g = POWER(FL(0.001), (p->lpt / p->rvt));
       }
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         ar[n] = *rp++;
         *wp++ = (ar[n] * g) + asig[n];
         if (wp >= endp) wp = startp;
@@ -445,12 +567,15 @@ static int vcomb(CSOUND *csound, VCOMB *p)
     p->pntr = wp;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("vcomb: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("vcomb: not initialised"));
 }
 
 static int valpass(CSOUND *csound, VCOMB *p)
 {
-    int n, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t n, nsmps = CS_KSMPS;
     uint32 xlpt, maxlpt = (uint32)p->maxlpt;
     MYFLT       *ar, *asig, *rp, *startp, *endp, *wp, *lpt;
     MYFLT       y, z, g = p->g;
@@ -461,10 +586,15 @@ static int valpass(CSOUND *csound, VCOMB *p)
     endp = (MYFLT *) p->auxch.endp;
     startp = (MYFLT *) p->auxch.auxp;
     wp = p->pntr;
+    if (UNLIKELY(offset)) memset(ar, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&ar[nsmps], '\0', early*sizeof(MYFLT));
+    }
     if (p->lpta) {                                      /* if xlpt is a-rate */
       lpt = p->xlpt;
-      for (n=0; n<nsmps; n++) {
-        xlpt = (uint32)((*p->insmps != 0) ? lpt[n] : lpt[n] * csound->esr);
+      for (n=offset; n<nsmps; n++) {
+        xlpt = (uint32)((*p->insmps != 0) ? lpt[n] : lpt[n] * CS_ESR);
         if (xlpt > maxlpt) xlpt = maxlpt;
         if ((rp = wp - xlpt) < startp) rp += maxlpt;
         if ((p->rvt != *p->krvt) || (p->lpt != lpt[n])) {
@@ -480,14 +610,14 @@ static int valpass(CSOUND *csound, VCOMB *p)
     }
     else {                                              /* if xlpt is k-rate */
       xlpt = (uint32) ((*p->insmps != 0) ? *p->xlpt
-                                         : *p->xlpt * csound->esr);
+                                         : *p->xlpt * CS_ESR);
       if (xlpt > maxlpt) xlpt = maxlpt;
       if ((rp = wp - xlpt) < startp) rp += maxlpt;
       if ((p->rvt != *p->krvt) || (p->lpt != *p->xlpt)) {
         p->rvt = *p->krvt, p->lpt = *p->xlpt;
         g = p->g = POWER(FL(0.001), (p->lpt / p->rvt));
       }
-      for (n=0; n<nsmps; n++) {
+      for (n=offset; n<nsmps; n++) {
         y = *rp++;
         *wp++ = z = y * g + asig[n];
         ar[n] = y - g * z;
@@ -498,7 +628,8 @@ static int valpass(CSOUND *csound, VCOMB *p)
     p->pntr = wp;
     return OK;
  err1:
-    return csound->PerfError(csound, Str("valpass: not initialised"));
+    return csound->PerfError(csound, p->h.insdshead,
+                             Str("valpass: not initialised"));
 }
 
 static int ftmorfset(CSOUND *csound, FTMORF *p)
@@ -507,14 +638,14 @@ static int ftmorfset(CSOUND *csound, FTMORF *p)
     int j = 0;
     unsigned int len;
     /* make sure resfn exists and set it up */
-    if (LIKELY((ftp = csound->FTFind(csound, p->iresfn)) != NULL)) {
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->iresfn)) != NULL)) {
       p->resfn = ftp, len = p->resfn->flen;
     }
     else {
       return csound->InitError(csound, Str("iresfn for ftmorf does not exist"));
     }
     /* make sure ftfn exists and set it up */
-    if (LIKELY((ftp = csound->FTFind(csound, p->iftfn)) != NULL)) {
+    if (LIKELY((ftp = csound->FTnp2Find(csound, p->iftfn)) != NULL)) {
       p->ftfn = ftp;
     }
     else {
@@ -522,7 +653,7 @@ static int ftmorfset(CSOUND *csound, FTMORF *p)
     }
 
     do {                /* make sure tables in ftfn exist and are right size*/
-      if (LIKELY((ftp = csound->FTFind(csound, p->ftfn->ftable + j)) != NULL)) {
+      if (LIKELY((ftp = csound->FTnp2Find(csound, p->ftfn->ftable + j)) != NULL)) {
         if (UNLIKELY((unsigned int)ftp->flen != len)) {
           return csound->InitError(csound,
                                    Str("table in iftfn for ftmorf wrong size"));
@@ -532,7 +663,7 @@ static int ftmorfset(CSOUND *csound, FTMORF *p)
         return csound->InitError(csound, Str("table in iftfn for ftmorf "
                                              "does not exist"));
       }
-    } while (++j < p->ftfn->flen);
+    } while (++j < (int)p->ftfn->flen);
 
     p->len = len;
     p->ftndx = -FL(1.0);
@@ -551,8 +682,8 @@ static int ftmorf(CSOUND *csound, FTMORF *p)
     f = *p->kftndx - i;
     if (p->ftndx != *p->kftndx) {
       p->ftndx = *p->kftndx;
-      ftp1 = csound->FTFind(csound, p->ftfn->ftable + i++);
-      ftp2 = csound->FTFind(csound, p->ftfn->ftable + i--);
+      ftp1 = csound->FTnp2Find(csound, p->ftfn->ftable + i++);
+      ftp2 = csound->FTnp2Find(csound, p->ftfn->ftable + i--);
       do {
         *(p->resfn->ftable + j) = (*(ftp1->ftable + j) * (1-f)) +
           (*(ftp2->ftable + j) * f);
@@ -565,38 +696,38 @@ static int ftmorf(CSOUND *csound, FTMORF *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "dconv",  S(DCONV), TR|5, "a", "aii",   (SUBR)dconvset, NULL, (SUBR)dconv      },
-{ "vcomb", S(VCOMB),  5, "a", "akxioo", (SUBR)vcombset, NULL, (SUBR)vcomb     },
-{ "valpass", S(VCOMB),5, "a", "akxioo", (SUBR)vcombset, NULL, (SUBR)valpass   },
-{ "ftmorf", S(FTMORF),TR|3, "",  "kii",  (SUBR)ftmorfset,  (SUBR)ftmorf, NULL    },
-{ "and.ii",  S(AOP),  1, "i", "ii",   (SUBR)and_kk                  },
-{ "and.kk",  S(AOP),  2, "k", "kk",   NULL,   (SUBR)and_kk          },
-{ "and.ka",  S(AOP),  4, "a", "ka",   NULL,   NULL,   (SUBR)and_ka  },
-{ "and.ak",  S(AOP),  4, "a", "ak",   NULL,   NULL,   (SUBR)and_ak  },
-{ "and.aa",  S(AOP),  4, "a", "aa",   NULL,   NULL,   (SUBR)and_aa  },
-{ "or.ii",   S(AOP),  1, "i", "ii",   (SUBR)or_kk                   },
-{ "or.kk",   S(AOP),  2, "k", "kk",   NULL,   (SUBR)or_kk           },
-{ "or.ka",   S(AOP),  4, "a", "ka",   NULL,   NULL,   (SUBR)or_ka   },
-{ "or.ak",   S(AOP),  4, "a", "ak",   NULL,   NULL,   (SUBR)or_ak   },
-{ "or.aa",   S(AOP),  4, "a", "aa",   NULL,   NULL,   (SUBR)or_aa   },
-{ "xor.ii",  S(AOP),  1, "i", "ii",   (SUBR)xor_kk                  },
-{ "xor.kk",  S(AOP),  2, "k", "kk",   NULL,   (SUBR)xor_kk          },
-{ "xor.ka",  S(AOP),  4, "a", "ka",   NULL,   NULL,   (SUBR)xor_ka  },
-{ "xor.ak",  S(AOP),  4, "a", "ak",   NULL,   NULL,   (SUBR)xor_ak  },
-{ "xor.aa",  S(AOP),  4, "a", "aa",   NULL,   NULL,   (SUBR)xor_aa  },
-{ "not.i",   S(AOP),  1, "i", "i",    (SUBR)not_k                   },
-{ "not.k",   S(AOP),  2, "k", "k",    NULL,   (SUBR)not_k           },
-{ "not.a",   S(AOP),  4, "a", "a",    NULL,   NULL,   (SUBR)not_a   },
-{ "shl.ii",  S(AOP),  1, "i", "ii",   (SUBR) shift_left_kk          },
-{ "shl.kk",  S(AOP),  2, "k", "kk",   NULL, (SUBR) shift_left_kk    },
-{ "shl.ka", S(AOP), 4, "a", "ka", NULL, NULL, (SUBR) shift_left_ka  },
-{ "shl.ak", S(AOP), 4, "a", "ak", NULL, NULL, (SUBR) shift_left_ak  },
-{ "shl.aa", S(AOP), 4, "a", "aa", NULL, NULL, (SUBR) shift_left_aa  },
-{ "shr.ii",  S(AOP),  1, "i", "ii",   (SUBR) shift_right_kk         },
-{ "shr.kk",  S(AOP),  2, "k", "kk",   NULL, (SUBR) shift_right_kk   },
-{ "shr.ka", S(AOP), 4, "a", "ka", NULL, NULL, (SUBR) shift_right_ka },
-{ "shr.ak", S(AOP), 4, "a", "ak", NULL, NULL, (SUBR) shift_right_ak },
-{ "shr.aa", S(AOP), 4, "a", "aa", NULL, NULL, (SUBR) shift_right_aa }
+{ "dconv",  S(DCONV), TR, 5, "a", "aii",   (SUBR)dconvset, NULL, (SUBR)dconv },
+{ "vcomb", S(VCOMB),  0,5, "a", "akxioo", (SUBR)vcombset, NULL, (SUBR)vcomb   },
+{ "valpass", S(VCOMB),0,5, "a", "akxioo", (SUBR)vcombset, NULL, (SUBR)valpass },
+{ "ftmorf", S(FTMORF),TR, 3, "",  "kii",  (SUBR)ftmorfset,  (SUBR)ftmorf,    },
+{ "##and.ii",  S(AOP),  0,1, "i", "ii",   (SUBR)and_kk                  },
+{ "##and.kk",  S(AOP),  0,2, "k", "kk",   NULL,   (SUBR)and_kk          },
+{ "##and.ka",  S(AOP),  0,4, "a", "ka",   NULL,   NULL,   (SUBR)and_ka  },
+{ "##and.ak",  S(AOP),  0,4, "a", "ak",   NULL,   NULL,   (SUBR)and_ak  },
+{ "##and.aa",  S(AOP),  0,4, "a", "aa",   NULL,   NULL,   (SUBR)and_aa  },
+{ "##or.ii",   S(AOP),  0,1, "i", "ii",   (SUBR)or_kk                   },
+{ "##or.kk",   S(AOP),  0,2, "k", "kk",   NULL,   (SUBR)or_kk           },
+{ "##or.ka",   S(AOP),  0,4, "a", "ka",   NULL,   NULL,   (SUBR)or_ka   },
+{ "##or.ak",   S(AOP),  0,4, "a", "ak",   NULL,   NULL,   (SUBR)or_ak   },
+{ "##or.aa",   S(AOP),  0,4, "a", "aa",   NULL,   NULL,   (SUBR)or_aa   },
+{ "##xor.ii",  S(AOP),  0,1, "i", "ii",   (SUBR)xor_kk                  },
+{ "##xor.kk",  S(AOP),  0,2, "k", "kk",   NULL,   (SUBR)xor_kk          },
+{ "##xor.ka",  S(AOP),  0,4, "a", "ka",   NULL,   NULL,   (SUBR)xor_ka  },
+{ "##xor.ak",  S(AOP),  0,4, "a", "ak",   NULL,   NULL,   (SUBR)xor_ak  },
+{ "##xor.aa",  S(AOP),  0,4, "a", "aa",   NULL,   NULL,   (SUBR)xor_aa  },
+{ "##not.i",   S(AOP),  0,1, "i", "i",    (SUBR)not_k                   },
+{ "##not.k",   S(AOP),  0,2, "k", "k",    NULL,   (SUBR)not_k           },
+{ "##not.a",   S(AOP),  0,4, "a", "a",    NULL,   NULL,   (SUBR)not_a   },
+{ "##shl.ii",  S(AOP),  0,1, "i", "ii",   (SUBR) shift_left_kk          },
+{ "##shl.kk",  S(AOP),  0,2, "k", "kk",   NULL, (SUBR) shift_left_kk    },
+{ "##shl.ka", S(AOP), 0,4, "a", "ka", NULL, NULL, (SUBR) shift_left_ka  },
+{ "##shl.ak", S(AOP), 0,4, "a", "ak", NULL, NULL, (SUBR) shift_left_ak  },
+{ "##shl.aa", S(AOP), 0,4, "a", "aa", NULL, NULL, (SUBR) shift_left_aa  },
+{ "##shr.ii",  S(AOP),  0,1, "i", "ii",   (SUBR) shift_right_kk         },
+{ "##shr.kk",  S(AOP),  0,2, "k", "kk",   NULL, (SUBR) shift_right_kk   },
+{ "##shr.ka", S(AOP), 0,4, "a", "ka", NULL, NULL, (SUBR) shift_right_ka },
+{ "##shr.ak", S(AOP), 0,4, "a", "ak", NULL, NULL, (SUBR) shift_right_ak },
+{ "##shr.aa", S(AOP), 0,4, "a", "aa", NULL, NULL, (SUBR) shift_right_aa }
 };
 
 int ugmoss_init_(CSOUND *csound)
@@ -604,4 +735,3 @@ int ugmoss_init_(CSOUND *csound)
     return csound->AppendOpcodes(csound, &(localops[0]),
                                  (int) (sizeof(localops) / sizeof(OENTRY)));
 }
-
