@@ -24,7 +24,8 @@
 /* vbap.c
 
 assisting functions for VBAP
-functions for loudspeaker table initialization */
+functions for loudspeaker table initialization
+Re-written to take flexible number of outputs by JPff 2012 */
 
 
 #include "csoundCore.h"
@@ -44,20 +45,24 @@ static void add_ldsp_triplet(CSOUND *csound, int i, int j, int k,
                              ls *lss);
 static void calculate_3x3_matrixes(CSOUND *csound,
                                    ls_triplet_chain *ls_triplets,
-                                   ls lss[CHANNELS], int ls_amount);
+                                   ls lss[CHANNELS], int ls_amount, int ind);
 static void choose_ls_tuplets(CSOUND *csound, ls lss[CHANNELS],
                               ls_triplet_chain **ls_triplets,
-                              int ls_amount);
+                              int ls_amount, int ind);
 static void sort_2D_lss(ls lss[CHANNELS], int sorted_lss[CHANNELS],
                         int ls_amount);
 
-static MYFLT *create_ls_table(CSOUND *csound, size_t cnt)
+static MYFLT *create_ls_table(CSOUND *csound, size_t cnt, int ind)
 {
-    csound->DestroyGlobalVariable(csound, "vbap_ls_table");
-    if (UNLIKELY(csound->CreateGlobalVariable(csound, "vbap_ls_table",
-                                              cnt * sizeof(MYFLT)) != 0))
-      csound->Die(csound, Str("vbap: error allocating loudspeaker table"));
-    return get_ls_table(csound);
+    char name[24];
+    sprintf(name, "vbap_ls_table_%d", ind);
+    csound->DestroyGlobalVariable(csound, name);
+    if (UNLIKELY(csound->CreateGlobalVariable(csound, name,
+                                              cnt * sizeof(MYFLT)) != 0)) {
+      csound->ErrorMsg(csound, Str("vbap: error allocating loudspeaker table"));
+      return NULL;
+    }
+    return (MYFLT*) (csound->QueryGlobalVariableNoCheck(csound, name));
 }
 
 void calc_vbap_gns(int ls_set_am, int dim, LS_SET *sets,
@@ -123,9 +128,6 @@ void calc_vbap_gns(int ls_set_am, int dim, LS_SET *sets,
       sets[j].set_gains[2] = FL(1.0);
       }
 
-/*     for (i=0;i<ls_amount;i++) { */
-/*       gains[i]=FL(0.0); */
-/*     } */
     memset(gains, 0, ls_amount*sizeof(MYFLT));
 
     gains[sets[j].ls_nos[0]-1] = sets[j].set_gains[0];
@@ -263,7 +265,8 @@ static void choose_ls_triplets(CSOUND *csound, ls lss[CHANNELS],
     struct ls_triplet_chain *trip_ptr, *prev, *tmp_ptr;
 
     if (UNLIKELY(ls_amount == 0)) {
-      csound->Die(csound, Str("Number of loudspeakers is zero\nExiting"));
+      csound->ErrorMsg(csound, Str("Number of loudspeakers is zero\nExiting"));
+      return;
     }
 
     connections = calloc(CHANNELS * CHANNELS, sizeof(int));
@@ -555,33 +558,33 @@ int lines_intersect(int i,int j,int k,int l,ls  lss[CHANNELS])
     }
 }
 
-int vbap_ls_init (CSOUND *csound, VBAP_LS_INIT *p)
+static inline int vbap_ls_init_sr (CSOUND *csound, int dim, int count,
+                            MYFLT **f, int layout)
      /* Inits the loudspeaker data. Calls choose_ls_tuplets or _triplets
         according to current dimension. The inversion matrices are
         stored in transposed form to ease calculation at run time.*/
 {
-    int dim, count;
     struct ls_triplet_chain *ls_triplets = NULL;
     ls lss[CHANNELS];
 
     ANG_VEC a_vector;
     CART_VEC c_vector;
     int i=0,j;
-    int ls_amount;
 
-    dim = (int) *p->dim;
+    //dim = (int) *p->dim;
     csound->Warning(csound, "dim : %d\n",dim);
     if (UNLIKELY(!((dim==2) || (dim == 3)))) {
-      csound->Die(csound, Str("Error in loudspeaker dimension."));
+      csound->ErrorMsg(csound, Str("Error in loudspeaker dimension."));
+      return NOTOK;
     }
-    count = (int) *p->ls_amount;
+    //count = (int) *p->ls_amount;
     for (j=1;j<=count;j++) {
       if (dim == 3) {
-        a_vector.azi= (MYFLT) *p->f[2*j-2];
-        a_vector.ele= (MYFLT) *p->f[2*j-1];
+        a_vector.azi= (MYFLT) *f[2*j-2];
+        a_vector.ele= (MYFLT) *f[2*j-1];
       }
       else if (dim == 2) {
-        a_vector.azi= (MYFLT) *p->f[j-1];
+        a_vector.azi= (MYFLT) *f[j-1];
         a_vector.ele=FL(0.0);
       }
       angle_to_cart_II(&a_vector,&c_vector);
@@ -593,24 +596,32 @@ int vbap_ls_init (CSOUND *csound, VBAP_LS_INIT *p)
       lss[i].angles.length = FL(1.0);
       i++;
     }
-    ls_amount = (int)*p->ls_amount;
-    if (UNLIKELY(ls_amount < dim)) {
-      csound->Die(csound, Str("Too few loudspeakers"));
+    //ls_amount = (int)*p->ls_amount;
+    if (UNLIKELY(count < dim)) {
+      csound->ErrorMsg(csound, Str("Too few loudspeakers"));
+      return NOTOK;
     }
 
     if (dim == 3) {
-      choose_ls_triplets(csound, lss, &ls_triplets,ls_amount);
-      calculate_3x3_matrixes(csound, ls_triplets,lss,ls_amount);
+      choose_ls_triplets(csound, lss, &ls_triplets,count);
+      calculate_3x3_matrixes(csound, ls_triplets,lss,count, layout);
     }
     else if (dim ==2) {
-      choose_ls_tuplets(csound, lss, &ls_triplets,ls_amount);
+      choose_ls_tuplets(csound, lss, &ls_triplets,count, layout);
     }
     return OK;
 }
 
+int vbap_ls_init (CSOUND *csound, VBAP_LS_INIT *p)
+{
+    int dim = (int) *p->dim;
+    MYFLT  layout = (*p->dim-dim)*100;
+    return vbap_ls_init_sr(csound, dim, (int) *p->ls_amount, p->f, round(layout));
+}
+
 static void calculate_3x3_matrixes(CSOUND *csound,
                                    struct ls_triplet_chain *ls_triplets,
-                                   ls lss[CHANNELS], int ls_amount)
+                                   ls lss[CHANNELS], int ls_amount, int ind)
      /* Calculates the inverse matrices for 3D */
 {
     MYFLT invdet;
@@ -621,7 +632,8 @@ static void calculate_3x3_matrixes(CSOUND *csound,
     int triplet_amount = 0, i,j,k;
 
     if (UNLIKELY(tr_ptr == NULL)) {
-      csound->Die(csound, Str("Not valid 3-D configuration"));
+      csound->ErrorMsg(csound, Str("Not valid 3-D configuration"));
+      return;
     }
 
     /* counting triplet amount */
@@ -631,7 +643,7 @@ static void calculate_3x3_matrixes(CSOUND *csound,
     }
 
     /* calculations and data storage to a global array */
-    ls_table = create_ls_table(csound, triplet_amount * 12 + 3);
+    ls_table = create_ls_table(csound, triplet_amount * 12 + 3, ind);
     ls_table[0] = FL(3.0);  /* dimension */
     ls_table[1] = (MYFLT) ls_amount;
     ls_table[2] = (MYFLT) triplet_amount;
@@ -675,19 +687,19 @@ static void calculate_3x3_matrixes(CSOUND *csound,
       }
       csound->Warning(csound, "\n");
 
-   /* printf("\nMatrix "); */
-      for (j = 0; j < 9; j++) {
-     /* printf("%f ", ls_table[k]); */
-        k++;
-      }
-   /* printf("\n\n"); */
+    /* printf("\nMatrix ");  */
+    /*   for (j = 0; j < 9; j++) { */
+    /*     printf("%f ", ls_table[k]);  */
+    /*     k++; */
+    /*   } */
+    /* printf("\n\n"); */
     }
 }
 
 static void choose_ls_tuplets(CSOUND *csound,
                               ls lss[CHANNELS],
                               ls_triplet_chain **ls_triplets,
-                              int ls_amount)
+                              int ls_amount, int ind)
      /* selects the loudspeaker pairs, calculates the inversion
         matrices and stores the data to a global array */
 {
@@ -710,26 +722,36 @@ static void choose_ls_tuplets(CSOUND *csound,
       /*csound->Message(csound, "%d %d %f %f\n",sorted_lss[i],sorted_lss[i+1],
         lss[sorted_lss[i]].angles.azi,
         lss[sorted_lss[i+1]].angles.azi);*/
-      if ((lss[sorted_lss[i+1]].angles.azi -
-           lss[sorted_lss[i]].angles.azi) <= (PI - 0.175)) {
-        if (calc_2D_inv_tmatrix( lss[sorted_lss[i]].angles.azi,
-                                 lss[sorted_lss[i+1]].angles.azi,
-                                 inv_mat[i]) != 0) {
+      if (LIKELY((lss[sorted_lss[i+1]].angles.azi -
+                  lss[sorted_lss[i]].angles.azi) <= (PI - 0.0175))) {
+        if (LIKELY(calc_2D_inv_tmatrix( lss[sorted_lss[i]].angles.azi,
+                                        lss[sorted_lss[i+1]].angles.azi,
+                                        inv_mat[i]) != 0)) {
           exist[i]=1;
           amount++;
         }
       }
+      else  csound->Warning(csound, Str("Pair of speakers at %f and %f ignored\n"),
+                            lss[sorted_lss[i]].angles.azi*FL(180.0)/PI_F,
+                            lss[sorted_lss[i+1]].angles.azi*FL(180.0)/PI_F);
     }
 
-    if (((6.283 - lss[sorted_lss[ls_amount-1]].angles.azi)
-         +lss[sorted_lss[0]].angles.azi) <= (PI - 0.175)) {
-      if (calc_2D_inv_tmatrix(lss[sorted_lss[ls_amount-1]].angles.azi,
-                              lss[sorted_lss[0]].angles.azi,
-                              inv_mat[ls_amount-1]) != 0) {
+    if (LIKELY(((TWOPI_F - lss[sorted_lss[ls_amount-1]].angles.azi)
+                +lss[sorted_lss[0]].angles.azi) < (PI - 0.0175))) {
+      //printf("less than PI type 2- 0.175\n");
+      if (LIKELY(calc_2D_inv_tmatrix(lss[sorted_lss[ls_amount-1]].angles.azi,
+                                     lss[sorted_lss[0]].angles.azi,
+                                     inv_mat[ls_amount-1]) != 0)) {
         exist[ls_amount-1]=1;
         amount++;
       }
     }
+    else  csound->Warning(csound, Str("Pair of speakers at %f and %f ignored\n"),
+                          lss[sorted_lss[ls_amount-1]].angles.azi*FL(180.0)/PI_F,
+                          lss[sorted_lss[0]].angles.azi*FL(180.0)/PI_F);
+
+    if (UNLIKELY(amount==0))
+      csound->InitError(csound, Str("insufficient valid speakers"));
 
 #if 0
     if ( amount*6 + 6 <= 16) ftable_size = 16;
@@ -744,7 +766,7 @@ static void choose_ls_tuplets(CSOUND *csound,
       csound->Message(csound, "%.1f ", lss[i].angles.azi / atorad);
     csound->Message(csound, "\n");
 #endif
-    ls_table = create_ls_table(csound, amount * 6 + 3 + 100);
+    ls_table = create_ls_table(csound, amount * 6 + 3 + 100, ind);
     ls_table[0] = FL(2.0);  /* dimension */
     ls_table[1] = (MYFLT) ls_amount;
     ls_table[2] = (MYFLT) amount;
@@ -774,12 +796,12 @@ static void choose_ls_tuplets(CSOUND *csound,
       }
       csound->Warning(csound, "\n");
 
-   /* csound->Message(csound, "\nMatrix "); */
-      /* for (j=0; j < 4; j++) { */
-     /* csound->Message(csound, "%f ", ls_table[k]); */
-      /*   k++; */
-      /* } */
-   /* csound->Message(csound, "\n\n"); */
+    csound->Message(csound, "\nMatrix ");
+      for (j=0; j < 4; j++) {
+      csound->Message(csound, "%f ", ls_table[k]);
+        k++;
+      }
+   csound->Message(csound, "\n\n");
     }
 }
 
@@ -901,37 +923,53 @@ void new_spread_base(CART_VEC spreaddir, CART_VEC vscartdir,
 
 /* static */
 static OENTRY vbap_localops[] = {
-  { "vbap4",      S(VBAP_FOUR),             TR|5,  "aaaa",             "akOO",
-    (SUBR) vbap_FOUR_init,          (SUBR) NULL,    (SUBR) vbap_FOUR        },
-  { "vbap8",      S(VBAP_EIGHT),            TR|5,  "aaaaaaaa",         "akOO",
-    (SUBR) vbap_EIGHT_init,         (SUBR) NULL,    (SUBR) vbap_EIGHT       },
-  { "vbap16",     S(VBAP_SIXTEEN),          TR|5,  "aaaaaaaaaaaaaaaa", "akOO",
-    (SUBR) vbap_SIXTEEN_init,       (SUBR) NULL,    (SUBR) vbap_SIXTEEN     },
-  { "vbapz",      S(VBAP_ZAK),           ZW|TR|5,  "",                 "iiakOO",
+  { "vbap.a",      S(VBAP),
+    TR, 5,  "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm",
+    "akOOo",
+    (SUBR) vbap_init,          (SUBR) NULL,    (SUBR) vbap                   },
+  { "vbap.A",      S(VBAPA), TR, 5,  "a[]",    "akOOo",
+    (SUBR) vbap_init_a,          (SUBR) NULL,    (SUBR) vbap_a               },
+  { "vbap4",      S(VBAP),
+    TR|_QQ, 5,  "aaaa",  "akOOo", (SUBR) vbap_init, (SUBR) NULL, (SUBR) vbap },
+  { "vbap8",      S(VBAP),
+    TR|_QQ, 5,  "aaaaaaaa", "akOOo",
+    (SUBR) vbap_init,          (SUBR) NULL,    (SUBR) vbap                   },
+  { "vbap16",      S(VBAP),
+    TR|_QQ, 5,  "aaaaaaaaaaaaaaaa", "akOOo",
+    (SUBR) vbap_init,          (SUBR) NULL,    (SUBR) vbap                   },
+  { "vbapg.a",      S(VBAP1),             TR, 3,
+    "zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz", "kOOo",
+    (SUBR) vbap1_init,         (SUBR) vbap1                                  },
+  { "vbapg.A",      S(VBAPA1),            TR, 3,
+    "k[]",  "kOOo",
+    (SUBR) vbap1_init_a,         (SUBR) vbap1a                               },
+  { "vbapz",      S(VBAP_ZAK),     ZW|TR, 5,  "",                 "iiakOOo",
     (SUBR) vbap_zak_init,           (SUBR) NULL,    (SUBR) vbap_zak         },
-  { "vbaplsinit", S(VBAP_LS_INIT), TR|1, "", "iioooooooooooooooooooooooooooooooo",
+  { "vbaplsinit",S(VBAP_LS_INIT),TR,1, "", "iioooooooooooooooooooooooooooooooo",
     (SUBR) vbap_ls_init,            (SUBR) NULL,    (SUBR) NULL             },
-  { "vbap4move",  S(VBAP_FOUR_MOVING),      TR|5,  "aaaa",             "aiiim",
-    (SUBR) vbap_FOUR_moving_init,   (SUBR) NULL,    (SUBR) vbap_FOUR_moving },
-  { "vbap8move",  S(VBAP_EIGHT_MOVING),     TR|5,  "aaaaaaaa",         "aiiim",
-    (SUBR) vbap_EIGHT_moving_init,  (SUBR) NULL,    (SUBR) vbap_EIGHT_moving },
-  { "vbap16move", S(VBAP_SIXTEEN_MOVING),   TR|5,  "aaaaaaaaaaaaaaaa", "aiiim",
-    (SUBR) vbap_SIXTEEN_moving_init, (SUBR) NULL, (SUBR) vbap_SIXTEEN_moving },
-  { "vbapzmove",  S(VBAP_ZAK_MOVING),    ZW|TR|5,  "",                 "iiaiiim",
-    (SUBR) vbap_zak_moving_init,    (SUBR) NULL,    (SUBR) vbap_zak_moving  }
+  { "vbapmove.a", S(VBAP_MOVING),
+    TR, 5,  "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm",
+    "aiiim",
+    (SUBR) vbap_moving_init, (SUBR) NULL, (SUBR) vbap_moving },
+  { "vbapgmove.a",  S(VBAP1_MOVING),      TR, 5,
+    "zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz", "iiim",
+    (SUBR) vbap1_moving_init,   (SUBR) NULL,    (SUBR) vbap1_moving },
+  { "vbapmove.A", S(VBAPA_MOVING),
+    TR, 5,  "a[]",  "aiiim",
+    (SUBR) vbap_moving_init_a, (SUBR) NULL, (SUBR) vbap_moving_a },
+  { "vbapgmove.A",  S(VBAPA1_MOVING),      TR, 5,
+    "k[]", "iiim",
+    (SUBR) vbap1_moving_init_a,   (SUBR) NULL,    (SUBR) vbap1_moving_a },
+  { "vbapzmove",  S(VBAP_ZAK_MOVING),    ZW|TR, 5,  "",  "iiaiiim",
+    (SUBR) vbap_zak_moving_init,    (SUBR) NULL,    (SUBR) vbap_zak_moving  },
+  { "vbap4move", S(VBAP_MOVING),   TR|_QQ, 5,  "aaaa",
+   "aiiim",
+    (SUBR) vbap_moving_init, (SUBR) NULL, (SUBR) vbap_moving },
+  { "vbap8move", S(VBAP_MOVING),
+    TR|_QQ, 5,  "aaaaaaaa",
+    "aiiim",
+    (SUBR) vbap_moving_init, (SUBR) NULL, (SUBR) vbap_moving }
+
 };
 
-LINKAGE1(vbap_localops)
-
-/* PUBLIC long csound_opcode_init(CSOUND *csound, OENTRY **ep)
-{
-    create_ls_table(csound, 3);
-    *ep = localops;
-    return (long) sizeof(localops);
-}
-
-PUBLIC int csoundModuleInfo(void)
-{
-    return ((CS_APIVERSION << 16) + (CS_APISUBVER << 8) + (int) sizeof(MYFLT));
-}
-*/
+LINKAGE_BUILTIN(vbap_localops)
