@@ -29,7 +29,7 @@
 */
 
 
-#include "csdl.h"
+#include "csoundCore.h"
 #include "vbap.h"
 #include <math.h>
 #include <stdio.h>
@@ -43,28 +43,33 @@ int vbap_zak(CSOUND *csound, VBAP_ZAK *p)   /* during note performance: */
     MYFLT *outptr, *inptr;
     MYFLT ogain, ngain, gainsubstr;
     MYFLT invfloatn;
-    int i,j;
+    int j;
     int n = p->n;
-    int nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
 
     vbap_zak_control(csound,p);
-    for (i=0; i<n; i++) {
-      p->beg_gains[i] = p->end_gains[i];
-      p->end_gains[i] = p->updated_gains[i];
+    for (j=0; j<n; j++) {
+      p->beg_gains[j] = p->end_gains[j];
+      p->end_gains[j] = p->updated_gains[j];
     }
 
     /* write audio to result audio streams weighted
        with gain factors */
-    invfloatn =  csound->onedksmps;
     outptr = p->out_array;
+    if (UNLIKELY(early)) nsmps -= early;
+    invfloatn =  FL(1.0)/(nsmps-offset);
     for (j=0; j<n; j++) {
       inptr = p->audio;
       ogain = p->beg_gains[j];
       ngain = p->end_gains[j];
       gainsubstr = ngain - ogain;
+      if (UNLIKELY(offset)) memset(outptr, '\0', offset*sizeof(MYFLT));
+      if (UNLIKELY(early)) memset(&outptr[nsmps], '\0', early*sizeof(MYFLT));
       if (ngain != FL(0.0) || ogain != FL(0.0))
         if (ngain != ogain) {
-          for (i = 0; i < nsmps; i++) {
+          for (i = offset; i < nsmps; i++) {
             outptr[i] = inptr[i] *
               (ogain + (MYFLT) (i+1) * invfloatn * gainsubstr);
           }
@@ -72,7 +77,7 @@ int vbap_zak(CSOUND *csound, VBAP_ZAK *p)   /* during note performance: */
             (MYFLT) (i) * invfloatn * gainsubstr;
         }
         else {
-          for (i=0; i<nsmps; ++i)
+          for (i=offset; i<nsmps; ++i)
             outptr[i] = inptr[i] * ogain;
         }
       else
@@ -195,22 +200,27 @@ int vbap_zak_init(CSOUND *csound, VBAP_ZAK *p)
     MYFLT   *ls_table, *ptr; /* , *gains; */
     LS_SET  *ls_set_ptr;
     int n = p->n = (int)MYFLT2LONG(*p->numb); /* Set size */
+    char name[24];
     /* Check to see this index is within the limits of za space.    */
     indx = (int32) *p->ndx;
     if (UNLIKELY(indx > csound->zalast)) {
-      return csound->PerfError(csound, Str("outz index > isizea. No output"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("outz index > isizea. No output"));
     }
     else if (UNLIKELY(indx < 0)) {
-      return csound->PerfError(csound, Str("outz index < 0. No output."));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("outz index < 0. No output."));
     }
+    if ((int)*p->layout==0) strcpy(name, "vbap_ls_table");
+    else sprintf(name, "vbap_ls_table_%d", (int)*p->layout==0);
     /* Now read from the array in za space and write to the output. */
-    p->out_array     = csound->zastart + (indx * csound->ksmps);/* outputs */
+    p->out_array     = csound->zastart + (indx * CS_KSMPS);/* outputs */
     csound->AuxAlloc(csound, p->n*sizeof(MYFLT)*4, &p->auxch);
     p->curr_gains    = (MYFLT*)p->auxch.auxp;
     p->beg_gains     = p->curr_gains + p->n;
     p->end_gains     = p->beg_gains + p->n;
     p->updated_gains = p->end_gains + p->n;
-    ls_table = get_ls_table(csound);
+    ls_table = (MYFLT*) (csound->QueryGlobalVariableNoCheck(csound, name));
     p->dim           = (int) ls_table[0];   /* reading in loudspeaker info */
     p->ls_am         = (int) ls_table[1];
     p->ls_set_am     = (int) ls_table[2];
@@ -259,19 +269,26 @@ int vbap_zak_moving(CSOUND *csound, VBAP_ZAK_MOVING *p)
     MYFLT *outptr, *inptr;
     MYFLT ogain, ngain, gainsubstr;
     MYFLT invfloatn;
-    int i,j;
-    int nsmps = csound->ksmps;
+    int j;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
 
     vbap_zak_moving_control(csound,p);
-    for (i=0;i< p->n; i++) {
-      p->beg_gains[i] = p->end_gains[i];
-      p->end_gains[i] = p->updated_gains[i];
+    for (j=0;j< p->n; j++) {
+      p->beg_gains[j] = p->end_gains[j];
+      p->end_gains[j] = p->updated_gains[j];
     }
 
-    /* write audio to resulting audio streams wEIGHTed
+    /* write audio to resulting audio streams weighted
        with gain factors */
-    invfloatn =  csound->onedksmps;
+    invfloatn =  FL(1.0)/(nsmps-offset);
     outptr = p->out_array;
+    if (UNLIKELY(offset)) memset(outptr, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&outptr[nsmps], '\0', early*sizeof(MYFLT));
+    }
     for (j=0; j<p->n ;j++) {
       inptr = p->audio;
       ogain = p->beg_gains[j];
@@ -279,7 +296,7 @@ int vbap_zak_moving(CSOUND *csound, VBAP_ZAK_MOVING *p)
       gainsubstr = ngain - ogain;
       if (ngain != FL(0.0) || ogain != FL(0.0))
         if (ngain != ogain) {
-          for (i = 0; i < nsmps; i++) {
+          for (i = offset; i < nsmps; i++) {
             outptr[i] = inptr[i] *
               (ogain + (MYFLT) (i+1) * invfloatn * gainsubstr);
           }
@@ -287,7 +304,7 @@ int vbap_zak_moving(CSOUND *csound, VBAP_ZAK_MOVING *p)
             (MYFLT) (i) * invfloatn * gainsubstr;
         }
         else
-          for (i=0; i<nsmps; ++i)
+          for (i=offset; i<nsmps; ++i)
             outptr[i] = inptr[i] * ogain;
       else
         memset(outptr, 0, nsmps*sizeof(MYFLT));
@@ -334,7 +351,8 @@ int vbap_zak_moving_control(CSOUND *csound, VBAP_ZAK_MOVING *p)
         }
       }
       if (UNLIKELY((p->fld[abs(p->next_fld)]==NULL)))
-        csound->Die(csound, Str("Missing fields in vbapzmove\n"));
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("Missing fields in vbapzmove\n"));
       if (*p->field_am >= FL(0.0) && p->dim == 2) /* point-to-point */
         if (UNLIKELY(fabs(fabs(*p->fld[p->next_fld] - *p->fld[p->curr_fld])
                           - 180.0) < 1.0))
@@ -378,20 +396,21 @@ int vbap_zak_moving_control(CSOUND *csound, VBAP_ZAK_MOVING *p)
         p->ang_dir.ele = FL(0.0);
       }
       else {
-        csound->Die(csound, Str("Wrong dimension\n"));
+        return csound->PerfError(csound, p->h.insdshead,
+                                 Str("Missing fields in vbapzmove\n"));
       }
     }
     else { /* angular velocities */
       if (p->dim == 2) {
         p->ang_dir.azi = p->ang_dir.azi +
-          (*p->fld[p->next_fld] * csound->onedkr);
+          (*p->fld[p->next_fld] * CS_ONEDKR);
         scale_angles(&(p->ang_dir));
       }
       else { /* 3D angular */
         p->ang_dir.azi = p->ang_dir.azi +
-          (*p->fld[p->next_fld] * csound->onedkr);
+          (*p->fld[p->next_fld] * CS_ONEDKR);
         p->ang_dir.ele = p->ang_dir.ele +
-          p->ele_vel * (*p->fld[p->next_fld+1] * csound->onedkr);
+          p->ele_vel * (*p->fld[p->next_fld+1] * CS_ONEDKR);
         if (p->ang_dir.ele > FL(90.0)) {
           p->ang_dir.ele = FL(90.0);
           p->ele_vel = -p->ele_vel;
@@ -497,20 +516,23 @@ int vbap_zak_moving_init(CSOUND *csound, VBAP_ZAK_MOVING *p)
     /* Check to see this index is within the limits of za space.    */
     indx = (int32) *p->ndx;
     if (UNLIKELY(indx > csound->zalast)) {
-      return csound->PerfError(csound, Str("outz index > isizea. No output"));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("outz index > isizea. No output"));
     }
     else if (UNLIKELY(indx < 0)) {
-      return csound->PerfError(csound, Str("outz index < 0. No output."));
+      return csound->PerfError(csound, p->h.insdshead,
+                               Str("outz index < 0. No output."));
     }
     /* Now read from the array in za space and write to the output. */
-    p->out_array     = csound->zastart + (indx * csound->ksmps);/* outputs */
+    p->out_array     = csound->zastart + (indx * CS_KSMPS);/* outputs */
     csound->AuxAlloc(csound, p->n*sizeof(MYFLT)*4, &p->auxch);
     p->curr_gains    = (MYFLT*)p->auxch.auxp;
     p->beg_gains     = p->curr_gains + p->n;
     p->end_gains     = p->beg_gains + p->n;
     p->updated_gains = p->end_gains + p->n;
     /* reading in loudspeaker info */
-    ls_table = get_ls_table(csound);
+    ls_table = (MYFLT*) (csound->QueryGlobalVariableNoCheck(csound,
+                                                        "vbap_ls_table_0"));
     p->dim           = (int) ls_table[0];
     p->ls_am         = (int) ls_table[1];
     p->ls_set_am     = (int) ls_table[2];
@@ -536,18 +558,18 @@ int vbap_zak_moving_init(CSOUND *csound, VBAP_ZAK_MOVING *p)
     /* other initialization */
     p->ele_vel = FL(1.0);    /* functions specific to movement */
     if (UNLIKELY(fabs(*p->field_am) < (2+ (p->dim - 2)*2))) {
-      csound->Die(csound,
+      return csound->InitError(csound,
                   Str("Have to have at least %d directions in vbapzmove"),
                   2 + (p->dim - 2) * 2);
     }
     if (p->dim == 2)
-      p->point_change_interval = (int) (csound->ekr * *p->dur
+      p->point_change_interval = (int) (CS_EKR * *p->dur
                                         / (fabs(*p->field_am) - 1.0));
     else if (LIKELY(p->dim == 3))
-      p->point_change_interval = (int) (csound->ekr * *p->dur
+      p->point_change_interval = (int) (CS_EKR * *p->dur
                                         / (fabs(*p->field_am) * 0.5 - 1.0));
     else
-      csound->Die(csound, Str("Wrong dimension"));
+      return csound->InitError(csound, Str("Wrong dimension"));
     p->point_change_counter = 0;
     p->curr_fld = 0;
     p->next_fld = 1;
