@@ -35,7 +35,20 @@
 #include "vsthost.h"
 #include "fxbank.h"
 
-#include "InOut/widglobals.h" //gab
+// These two collections replace similar ones that used to be in widglobals.h.
+
+std::vector<VSTPlugin*> &vstPlugEditors()
+{
+    static std::vector<VSTPlugin*> vstPlugEditors_;
+    return vstPlugEditors_;
+}
+
+std::vector<VSTPlugin*> &vstPlugins()
+{
+    static std::vector<VSTPlugin*> vstPlugins_;
+    return vstPlugins_;
+}
+
 
 extern "C" {
   std::string version = "0.2beta";
@@ -43,14 +56,18 @@ extern "C" {
   static void path_convert(char *in);
 #endif
 
+    static MYFLT getCurrentTime(CSOUND *csound)
+    {
+        return csound->GetCurrentTimeSamples(csound) / csound->GetSr(csound);
+    }
+
   static int vstinit(CSOUND *csound, void *data)
   {
     VSTINIT *p = (VSTINIT *) data;
     VSTPlugin *plugin = new VSTPlugin(csound);
-
-    *p->iVSThandle = (MYFLT) ST(vstPlugins).size();
-    ST(vstPlugins).push_back(plugin);
-    if ((int) ST(vstPlugins).size() == 1) {
+    *p->iVSThandle = (MYFLT) vstPlugins().size();
+    vstPlugins().push_back(plugin);
+    if ((int) vstPlugins().size() == 1) {
       plugin->Log("============================================================\n");
       plugin->Log("vst4cs version %s by Andres Cabrera and Michael Gogins\n",
                   version.c_str());
@@ -59,8 +76,8 @@ extern "C" {
       plugin->Log("VST Plug-In Technology by Steinberg\n");
       plugin->Log("============================================================\n");
     }
-    char    vstplugname[0x100];
-    strcpy(vstplugname, (char *) p->iplugin);
+    char vstplugname[0x100];
+    strncpy(vstplugname,((STRINGDAT *)p->iplugin)->data, MAXNAME-1);
 #if WIN32
     path_convert(vstplugname);
 #endif
@@ -77,8 +94,7 @@ extern "C" {
   static int vstinfo(CSOUND *csound, void *data)
   {
     VSTINFO *p = (VSTINFO *) data;
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
-
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     plugin->Info();
     return OK;
   }
@@ -90,18 +106,12 @@ extern "C" {
     p->opcodeInChannels = (size_t) (csound->GetInputArgCnt(data) - 1);
     if (p->opcodeInChannels > 32)
       csound->InitError(csound, "vstaudio: too many input args");
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
-    //       if (!plugin) {
-    //         csound->InitError(csound, "vstaudio_init: invalid plugin pointer!");
-    //         return NOTOK;
-    //       }
-
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     plugin->Debug("vstaudio_init.\n");
-    p->framesPerBlock = csound->Getksmps(csound);
+    p->framesPerBlock = csound->GetKsmps(csound);
     p->pluginInChannels  = (size_t) plugin->getNumInputs();
     p->pluginOutChannels = (size_t) plugin->getNumOutputs();
     p->opcodeOutChannels = (size_t) csound->GetOutputArgCnt(data);
-
     return OK;
   }
 
@@ -109,7 +119,7 @@ extern "C" {
   {
     VSTAUDIO *p = (VSTAUDIO *) data;
     size_t  i, j;
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     uint32_t offset = p->h.insdshead->ksmps_offset;
     for(j=0; j < p->pluginOutChannels; j++)
        memset(p->aouts[j], '\0', offset*sizeof(MYFLT));
@@ -118,7 +128,7 @@ extern "C" {
       for (j = 0; j < p->pluginInChannels && j < p->opcodeInChannels; j++)
         for (i = offset; i < p->framesPerBlock; i++)
           plugin->inputs_[j][i] =
-            (float) (p->ains[j][i] * csound->dbfs_to_float);
+            (float) (p->ains[j][i] / csound->Get0dBFS(csound));
       for ( ; j < p->pluginInChannels; j++)
         for (i = 0; i < p->framesPerBlock; i++)
           plugin->inputs_[j][i] = 0.0f;
@@ -129,7 +139,7 @@ extern "C" {
                       p->framesPerBlock);
       for (j = 0; j < p->pluginOutChannels && j < p->opcodeOutChannels; j++)
         for (i = offset; i < p->framesPerBlock; i++)
-          p->aouts[j][i] = (MYFLT) plugin->outputs_[j][i] * csound->e0dbfs;
+          p->aouts[j][i] = (MYFLT) plugin->outputs_[j][i] * csound->Get0dBFS(csound);
       for ( ; j < p->opcodeOutChannels; j++)
         for (i = 0; i < p->framesPerBlock; i++)
           p->aouts[j][i] = FL(0.0);
@@ -149,15 +159,14 @@ extern "C" {
   {
     VSTAUDIO *p = (VSTAUDIO *) data;
     size_t  i, j;
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     uint32_t offset = p->h.insdshead->ksmps_offset;
     for(j=0; j < p->pluginOutChannels; j++)
        memset(p->aouts[j], '\0', offset*sizeof(MYFLT));
-    // plugin->Debug("vstaudio: plugin %x.\n", plugin);
     for (j = 0; j < p->pluginInChannels && j < p->opcodeInChannels; j++)
       for (i = offset; i < p->framesPerBlock; i++)
         plugin->inputs_[j][i] =
-          (float) (p->ains[j][i] * csound->dbfs_to_float);
+          (float) (p->ains[j][i] / csound->Get0dBFS(csound));
     for ( ; j < p->pluginInChannels; j++)
       for (i = 0; i < p->framesPerBlock; i++)
         plugin->inputs_[j][i] = 0.0f;
@@ -168,77 +177,22 @@ extern "C" {
                     p->framesPerBlock);
     for (j = 0; j < p->pluginOutChannels && j < p->opcodeOutChannels; j++)
       for (i = offset; i < p->framesPerBlock; i++)
-        p->aouts[j][i] = (MYFLT) plugin->outputs_[j][i] * csound->e0dbfs;
+        p->aouts[j][i] = (MYFLT) plugin->outputs_[j][i] * csound->Get0dBFS(csound);
     for ( ; j < p->opcodeOutChannels; j++)
       for (i = 0; i < p->framesPerBlock; i++)
         p->aouts[j][i] = FL(0.0);
-
     return OK;
   }
 
-  //~ static int vstnote_init(CSOUND *csound, void *data)
-  //~ {
-  //~ VSTNOTE *p = (VSTNOTE *) data;
-  //~ p->vstHandle = (size_t) *p->iVSThandle;
-  //~ VSTPlugin *vstPlugin = ST(vstPlugins)[p->vstHandle];
-
-  //~ int     rounded = (int) ((double) *p->knote + 0.5);
-  //~ int     cents = (int) (((double) *p->knote - (double) rounded) * 100.0
-  //~ + 100.5) - 100;
-  //~ int     veloc = (int) *p->kveloc;
-  //~ double  deltaTime =
-  //~ ((double) p->h.insdshead->p2 + csound->timeOffs) - csound->curTime;
-  //~ int     deltaFrames =
-  //~ (int) (deltaTime * (double) vstPlugin->framesPerSecond
-  //~ + (deltaTime < 0.0 ? -0.5 : 0.5));
-  //~ long    framesRemaining =
-  //~ (long) ((double) *p->kdur * (double) vstPlugin->framesPerSecond + 0.5)
-  //~ + (long) deltaFrames;
-
-  //~ p->chn = (int) *p->kchan & 15;
-  //~ p->note = (rounded >= 0 ? (rounded < 128 ? rounded : 127) : 0);
-  //~ veloc = (veloc >= 0 ? (veloc < 128 ? veloc : 127) : 0);
-  //~ if (deltaFrames < 0)
-  //~ deltaFrames = 0;
-  //~ else if (deltaFrames >= (int) vstPlugin->framesPerBlock)
-  //~ deltaFrames = (int) vstPlugin->framesPerBlock - 1;
-  //~ if (framesRemaining < 0L)
-  //~ framesRemaining = 0L;
-  //~ p->framesRemaining = (size_t) framesRemaining;
-
-  //~ vstPlugin->Debug("vstnote_init\n");
-  //~ vstPlugin->AddMIDI(144 | p->chn | (p->note << 8) | (veloc << 16),
-  //~ deltaFrames, cents);
-
-  //~ return OK;
-  //~ }
-
-  //~ static int vstnote(CSOUND *csound, void *data)
-  //~ {
-  //~ VSTNOTE *p = (VSTNOTE *) data;
-
-  //~ if (p->framesRemaining >= (size_t) csound->ksmps) {
-  //~ p->framesRemaining -= (size_t) csound->ksmps;
-  //~ return OK;
-  //~ }
-
-  //~ VSTPlugin *plugin = ST(vstPlugins)[p->vstHandle];
-  //~ plugin->Debug("vstnote.\n");
-  //~ plugin->AddMIDI(128 | p->chn | (p->note << 8), (int) p->framesRemaining, 0);
-  //~ p->framesRemaining = 0; /* ~((size_t) 0); */
-  //~ return OK;
-  //~ }
 
   static int vstmidiout_init(CSOUND *csound, void *data)
   {
     VSTMIDIOUT *p = (VSTMIDIOUT *) data;
     VSTPlugin *plugin;
-
     p->vstHandle = (size_t) *p->iVSThandle;
-    plugin = ST(vstPlugins)[p->vstHandle];
+    plugin = vstPlugins()[p->vstHandle];
     plugin->Debug("vstmidiout_init.\n");
     p->prvMidiData = 0;
-
     return OK;
   }
 
@@ -247,7 +201,6 @@ extern "C" {
     VSTMIDIOUT *p = (VSTMIDIOUT *) data;
     VSTPlugin *plugin;
     int     st, ch, d1, d2, midiData;
-
     st = (int) *(p->kstatus);
     if (st < 128 || st >= 240) {
       p->prvMidiData = 0;
@@ -270,12 +223,10 @@ extern "C" {
       d2 = 0;
     d1 = (d1 >= 0 ? (d1 < 128 ? d1 : 127) : 0);
     midiData = st | ch | (d1 << 8) | (d2 << 16);
-
     if (midiData == p->prvMidiData)
       return OK;
-
     p->prvMidiData = midiData;
-    plugin = ST(vstPlugins)[p->vstHandle];
+    plugin = vstPlugins()[p->vstHandle];
     plugin->Debug("vstmidiout. kstatus = %i kdata1 = %i kdata2 = %i"
                   "--- mididata = %i\n",
                   (int) *(p->kstatus),
@@ -283,7 +234,6 @@ extern "C" {
                   (int) *(p->kdata2),
                   midiData);
     plugin->AddMIDI(midiData, 0, 0);
-
     return OK;
   }
 
@@ -295,11 +245,9 @@ extern "C" {
   static int vstparamget(CSOUND *csound, void *data)
   {
     VSTPARAMGET *p = (VSTPARAMGET *) data;
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
-
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     plugin->Debug("vstparamset(%d).\n", int(*p->kparam));
     *p->kvalue = plugin->GetParamValue(int(*p->kparam));
-
     if (*(p->kvalue) == FL(-1.0))
       plugin->Log("Invalid parameter number %d.\n", int(*p->kparam));
     return OK;
@@ -308,8 +256,7 @@ extern "C" {
   static int vstparamset_init(CSOUND *csound, void *data)
   {
     VSTPARAMSET *p = (VSTPARAMSET *) data;
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
-
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     plugin->Debug("vstparamset_init.\n");
     p->oldkparam = 0;
     p->oldkvalue = 0;
@@ -319,25 +266,22 @@ extern "C" {
   static int vstparamset(CSOUND *csound, void *data)
   {
     VSTPARAMSET *p = (VSTPARAMSET *) data;
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
-
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     if (*p->kparam == p->oldkparam && *p->kvalue == p->oldkvalue)
       return OK;
     p->oldkparam = *p->kparam;
     p->oldkvalue = *p->kvalue;
     plugin->Debug("vstsend(%d, %f).\n", int(*p->kparam), *p->kvalue);
     plugin->SetParameter(int(*p->kparam), float(*p->kvalue));
-
     return OK;
   }
+
   static int vstbankload(CSOUND *csound, void *data)
   {
     VSTBANKLOAD *p = (VSTBANKLOAD *) data;
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     void    *dummyPointer = 0;
-
     CFxBank fxBank((char *) p->ibank);          /* load the bank    */
-
     plugin->Dispatch(effBeginLoadBank,
                      0, 0, (VstPatchChunkInfo *) fxBank.GetChunk(), 0);
     if (plugin->Dispatch(effBeginLoadBank,
@@ -345,7 +289,6 @@ extern "C" {
       csound->InitError(csound, "Error: BeginLoadBank.");
       return NOTOK;
     }
-    // csound->Message(csound, "EffBeginLoadBank\n");
     if (fxBank.IsLoaded()) {
       if (plugin->aeffect->uniqueID != fxBank.GetFxID()) {
         csound->InitError(csound, "Loaded bank ID doesn't match plug-in ID.");
@@ -362,13 +305,9 @@ extern "C" {
         plugin->Log("Chunks loaded OK.\n");
       }
       else {
-        //     int     cProg = plugin->EffGetProgram();
         int     cProg = plugin->Dispatch(effGetProgram, 0, 0, dummyPointer, 0);
-        //     csound->Message(csound, "Current Program= %i\n", cProg);
         int     i, j;
         int     nParms = fxBank.GetNumParams();
-
-        //     csound->Message(csound, "nParms= %i\n", nParms);
         for (i = 0; i < fxBank.GetNumPrograms(); i++) {
           plugin->Dispatch(effSetProgram, 0, i, dummyPointer, 0);
           plugin->Dispatch(effSetProgramName, 0, 0, fxBank.GetProgramName(i),
@@ -376,21 +315,14 @@ extern "C" {
           for (j = 0; j < nParms; j++)
             plugin->SetParameter(j, fxBank.GetProgParm(i, j));
         }
-        //     pEffect->EffSetProgram(cProg);
         plugin->Dispatch(effSetProgram, 0, cProg, dummyPointer, 0);
-        //     csound->Message(csound, "Programs OK\n");
       }
-      //   pEffect->SetChunkFile(dlg.GetPathName());
-      //   ShowDetails();
-      //   OnSetProgram();
     }
     else {
       csound->InitError(csound, "Problem loading bank.");
       return NOTOK;           /* check if error loading */
     }
     plugin->Log("Bank loaded OK.\n");
-    // if (fxBank.SetChunk()) {
-    // }
     return OK;
   }
 
@@ -399,7 +331,7 @@ extern "C" {
     // The changes here are part of an attempt to map 0 to 1 and others
     VSTPROGSET *p = (VSTPROGSET *) data;
     int program = (int)*p->iprogram;
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     if (program<=0) {
       csound->Message(csound, "VSTprogset: Program %d treated as 1\n", program);
       program = 1;
@@ -411,18 +343,17 @@ extern "C" {
   static int vstedit_init(CSOUND *csound, void *data)
   {
     VSTEDIT *p = (VSTEDIT *) data;
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
-
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     plugin->OpenEditor();
-    plugin->targetFLpanel = ST(fl_windows).size()-1; //gab
-    ST(VSTplugEditors).push_back(plugin); //gab
+    //plugin->targetFLpanel = vstPlugEditors().size()-1; //gab
+    vstPlugEditors().push_back(plugin); //gab
     return OK;
   }
 
   static int vstSetTempo(CSOUND *csound, void *data)
   {
     VSTTEMPO *p = (VSTTEMPO *)data;
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     plugin->vstTimeInfo.tempo = *p->tempo;
     return OK;
   }
@@ -430,17 +361,11 @@ extern "C" {
   int vstbanksave(CSOUND *csound, void *data)
   {
     VSTBANKLOAD *p = (VSTBANKLOAD *)data;
-    VSTPlugin *plugin = ST(vstPlugins)[(size_t) *p->iVSThandle];
+    VSTPlugin *plugin = vstPlugins()[(size_t) *p->iVSThandle];
     char bankname[512]; //gab
-    //  if (*p->ibank == SSTRCOD) {
     strcpy(bankname, (char *) p->ibank);          /*   use that         */
-    //  }
-
-    //------------
-    //CSmpEffect *pEffect = (CSmpEffect *)GetApp()->vstHost.GetAt(nEffect);
     if (!plugin)
       return NOTOK;
-
     CFxBank b;
 
     if (plugin->aeffect->flags & effFlagsProgramChunks)
@@ -484,9 +409,6 @@ extern "C" {
     if (b.SaveBank(bankname))
       {
         plugin->Log("%s Bank saved OK.\n",bankname);
-        //pEffect->SetChunkFile(bankname);
-        //      ShowDetails();
-        //              OnSetProgram();
       }
     else {
       plugin->Log("Error: Error saving file\n");
@@ -526,13 +448,10 @@ extern "C" {
   {
     VSTNOTEOUT *p = (VSTNOTEOUT *)data;
     size_t vstHandle = (size_t) *p->iVSThandle;
-    p->vstPlugin = ST(vstPlugins)[vstHandle];
-    p->startTime = CURTIME;
-    // The note may be scheduled to turn on some frames after the actual
-    // start of this kperiod.
-    // Use the warped p2 to compute this time.
-    double onTime = double(csound->timeOffs + p->h.insdshead->p2);
-    double deltaTime = onTime - CURTIME;
+    p->vstPlugin = vstPlugins()[vstHandle];
+    p->startTime = getCurrentTime(csound);
+    double onTime = double(p->h.insdshead->p2);
+    double deltaTime = onTime - getCurrentTime(csound);
     int deltaFrames = 0;
     if (deltaTime > 0) {
       deltaFrames = int(deltaTime / csound->GetSr(csound));
@@ -565,7 +484,7 @@ extern "C" {
     p->on = true;
     if (csound->GetDebug(csound)) {
       csound->Message(csound, "vstnote_init:      on time:      %f\n", onTime);
-      csound->Message(csound, "                   csound time:  %f\n", CURTIME);
+      csound->Message(csound, "                   csound time:  %f\n", getCurrentTime(csound));
       csound->Message(csound, "                   delta time:   %f\n", deltaTime);
       csound->Message(csound, "                   delta frames: %d\n", deltaFrames);
       csound->Message(csound, "                   off time:     %f\n", p->offTime);
@@ -581,10 +500,10 @@ extern "C" {
   {
     VSTNOTEOUT *p = (VSTNOTEOUT *)data;
     if (p->on) {
-      if (CURTIME >= p->offTime || p->h.insdshead->relesing) {
+      if (getCurrentTime(csound) >= p->offTime || p->h.insdshead->relesing) {
         // The note may be scheduled to turn off
         // some frames after the actual start of this kperiod.
-        double deltaTime = p->offTime - CURTIME;
+        double deltaTime = p->offTime - getCurrentTime(csound);
         int deltaFrames = 0;
         if (deltaTime > 0) {
           deltaFrames = int(deltaTime / csound->GetSr(csound));
@@ -594,7 +513,7 @@ extern "C" {
         p->on = false;
         if (csound->GetDebug(csound)) {
           csound->Message(csound, "vstnote_perf:      csound time:  %f\n",
-                          CURTIME);
+                          getCurrentTime(csound));
           csound->Message(csound, "                   off time:     %f\n",
                           p->offTime);
           csound->Message(csound, "                   delta time:   %f\n",
@@ -612,30 +531,26 @@ extern "C" {
   }
 
   static OENTRY localops[] = {
-    { "vstinit",      sizeof(VSTINIT),    1, "i", "So", &vstinit, 0, 0 },
-    { "vstinfo",      sizeof(VSTINFO),    1, "", "i", &vstinfo, 0, 0 },
-    { "vstaudio",     sizeof(VSTAUDIO),   5, "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm",
+    { "vstinit",      sizeof(VSTINIT),    0, 1, "i", "To", &vstinit, 0, 0 },
+    { "vstinfo",      sizeof(VSTINFO),    0, 1, "", "i", &vstinfo, 0, 0 },
+    { "vstaudio",     sizeof(VSTAUDIO),   0, 5, "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm",
                                              "iy", &vstaudio_init, 0, &vstaudio },
-    { "vstaudiog",    sizeof(VSTAUDIO),   5, "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm",
+    { "vstaudiog",    sizeof(VSTAUDIO),   0, 5, "mmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm",
                                              "iy", &vstaudio_init, 0, &vstaudiog },
-    { "vstmidiout",   sizeof(VSTMIDIOUT), 3, "", "ikkkk", &vstmidiout_init,
+    { "vstmidiout",   sizeof(VSTMIDIOUT), 0, 3, "", "ikkkk", &vstmidiout_init,
                                                                   &vstmidiout, 0 },
-    { "vstparamget",  sizeof(VSTPARAMGET),3, "k", "ik", &vstparamget_init,
+    { "vstparamget",  sizeof(VSTPARAMGET),0, 3, "k", "ik", &vstparamget_init,
                                                                  &vstparamget, 0 },
-    { "vstparamset",  sizeof(VSTPARAMSET),3, "", "ikk", &vstparamset_init,
+    { "vstparamset",  sizeof(VSTPARAMSET),0, 3, "", "ikk", &vstparamset_init,
                                                                  &vstparamset, 0 },
-    { "vstbankload",  sizeof(VSTBANKLOAD),1, "", "iS", &vstbankload, 0, 0 },
-    { "vstprogset",   sizeof(VSTPROGSET), 1, "", "ii", &vstprogset, 0, 0 },
-    { "vstedit",      sizeof(VSTEDIT),    1, "", "i", &vstedit_init, 0, 0 },
-    { "vsttempo",     sizeof(VSTTEMPO),   2, "" ,"ki", 0,
+    { "vstbankload",  sizeof(VSTBANKLOAD),0, 1, "", "iT", &vstbankload, 0, 0 },
+    { "vstprogset",   sizeof(VSTPROGSET), 0, 1, "", "ii", &vstprogset, 0, 0 },
+    { "vstedit",      sizeof(VSTEDIT),    0, 1, "", "i", &vstedit_init, 0, 0 },
+    { "vsttempo",     sizeof(VSTTEMPO),   0, 2, "", "ki", 0,
                                              &vstSetTempo, 0/*, &vstedit_deinit*/},
-    { "vstnote",      sizeof(VSTNOTEOUT), 3, "" ,"iiiii",  &vstnote_init,
-                                                                &vstnote_perf, 0 },
-    { "vstbanksave",  sizeof(VSTBANKLOAD),    1, "" ,"iS",    &vstbanksave,
-                                                    0,           0/*, 0        */},
-    //{ "vstnote",      sizeof(VSTNOTE),        3, "", "iiiii", &vstnote_init,
-                                                                     &vstnote, 0 },
-    { NULL, 0, 0, NULL, NULL, (SUBR) NULL, (SUBR) NULL, (SUBR) NULL }
+    { "vstnote",      sizeof(VSTNOTEOUT), 0, 3, "", "iiiii", &vstnote_init, &vstnote_perf, 0 },
+    { "vstbanksave",  sizeof(VSTBANKLOAD),0, 1, "", "iT",    &vstbanksave,  0,             0 },
+    { 0,              0,                   0, 0, 0,  0,      (SUBR) 0,     (SUBR) 0, (SUBR) 0 }
   };
 
   PUBLIC int csoundModuleCreate(CSOUND *csound)
@@ -649,8 +564,12 @@ extern "C" {
     int     err = 0;
     while (ep->opname != NULL) {
       err |= csound->AppendOpcode(csound,
-                                  ep->opname, ep->dsblksiz, ep->thread,
-                                  ep->outypes, ep->intypes,
+                                  ep->opname,
+                                  ep->dsblksiz,
+                                  ep->flags,
+                                  ep->thread,
+                                  ep->outypes,
+                                  ep->intypes,
                                   (int (*)(CSOUND *, void *)) ep->iopadr,
                                   (int (*)(CSOUND *, void *)) ep->kopadr,
                                   (int (*)(CSOUND *, void *)) ep->aopadr);
@@ -661,13 +580,11 @@ extern "C" {
 
   PUBLIC int csoundModuleDestroy(CSOUND *csound)
   {
-    if (csound->widgetGlobals) {
-      for (size_t i = 0, n = ST(vstPlugins).size(); i < n; ++i) {
-        if (ST(vstPlugins)[i]) {
-          delete ST(vstPlugins)[i];
+    for (size_t i = 0, n = vstPlugins().size(); i < n; ++i) {
+        if (vstPlugins()[i]) {
+            delete vstPlugins()[i];
         }
-      }
-      ST(vstPlugins).clear();
+        vstPlugins().clear();
     }
     return 0;
   }
