@@ -21,7 +21,7 @@
     02111-1307 USA
 */
 
-#include "csdl.h"
+#include "stdopcod.h"
 #include "wave-terrain.h"
 #include <math.h>
 
@@ -34,8 +34,8 @@
 static int wtinit(CSOUND *csound, WAVETER *p)
 {
     /* DECLARE */
-    FUNC *ftpx = csound->FTFind(csound, p->i_tabx);
-    FUNC *ftpy = csound->FTFind(csound, p->i_taby);
+    FUNC *ftpx = csound->FTnp2Find(csound, p->i_tabx);
+    FUNC *ftpy = csound->FTnp2Find(csound, p->i_taby);
 
     /* CHECK */
     if (UNLIKELY((ftpx == NULL)||(ftpy == NULL))) {
@@ -55,7 +55,9 @@ static int wtinit(CSOUND *csound, WAVETER *p)
 
 static int wtPerf(CSOUND *csound, WAVETER *p)
 {
-    int i, nsmps=csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     int xloc, yloc;
     MYFLT xc, yc;
     MYFLT amp = *p->kamp;
@@ -65,8 +67,14 @@ static int wtPerf(CSOUND *csound, WAVETER *p)
     MYFLT sizx = p->sizx, sizy = p->sizy;
     MYFLT theta = p->theta;
     MYFLT dtpidsr = csound->tpidsr;
+    MYFLT *aout = p->aout;
 
-    for (i=0; i<nsmps; i++) {
+    if (UNLIKELY(offset)) memset(aout, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&aout[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset; i<nsmps; i++) {
 
       /* COMPUTE LOCATION OF SCANNING POINT */
       xc = kcx + krx * SIN(theta);
@@ -81,7 +89,7 @@ static int wtPerf(CSOUND *csound, WAVETER *p)
       yloc = (int)(yc * sizy);
 
       /* OUTPUT AM OF TABLE VALUES * KAMP */
-      p->aout[i] = p->xarr[xloc] * p->yarr[yloc] * amp;
+      aout[i] = p->xarr[xloc] * p->yarr[yloc] * amp;
 
       /* MOVE SCANNING POINT ROUND THE ELLIPSE */
       theta += pch * dtpidsr;
@@ -103,11 +111,11 @@ static int wtPerf(CSOUND *csound, WAVETER *p)
 
 static int scanhinit(CSOUND *csound, SCANHAMMER *p)
 {
-  int srcpos = 0;
-  int dstpos = (int)MYFLT2LONG(*p->ipos);
+  unsigned int srcpos = 0;
+  unsigned int dstpos = (unsigned int)MYFLT2LONG(*p->ipos);
 
-  FUNC *fsrc = csound->FTFind(csound, p->isrc); /* Source table */
-  FUNC *fdst = csound->FTFind(csound, p->idst); /* Destination table */
+  FUNC *fsrc = csound->FTnp2Find(csound, p->isrc); /* Source table */
+  FUNC *fdst = csound->FTnp2Find(csound, p->idst); /* Destination table */
 
   if (UNLIKELY(fsrc->flen > fdst->flen)) {
     return csound->InitError(csound, Str(
@@ -138,11 +146,11 @@ static int scanhinit(CSOUND *csound, SCANHAMMER *p)
 static int scantinit(CSOUND *csound, SCANTABLE *p)
 {
     /* DECLARE */
-    FUNC *fpoint = csound->FTFind(csound, p->i_point);
-    FUNC *fmass  = csound->FTFind(csound, p->i_mass);
-    FUNC *fstiff = csound->FTFind(csound, p->i_stiff);
-    FUNC *fdamp  = csound->FTFind(csound, p->i_damp);
-    FUNC *fvel   = csound->FTFind(csound, p->i_vel);
+    FUNC *fpoint = csound->FTnp2Find(csound, p->i_point);
+    FUNC *fmass  = csound->FTnp2Find(csound, p->i_mass);
+    FUNC *fstiff = csound->FTnp2Find(csound, p->i_stiff);
+    FUNC *fdamp  = csound->FTnp2Find(csound, p->i_damp);
+    FUNC *fvel   = csound->FTnp2Find(csound, p->i_vel);
 
     /* CHECK */
     if (UNLIKELY(fpoint == NULL)) {
@@ -198,7 +206,9 @@ static int scantinit(CSOUND *csound, SCANTABLE *p)
 
 static int scantPerf(CSOUND *csound, SCANTABLE *p)
 {
-    int i, nsmps = csound->ksmps;
+    uint32_t offset = p->h.insdshead->ksmps_offset;
+    uint32_t early  = p->h.insdshead->ksmps_no_end;
+    uint32_t i, nsmps = CS_KSMPS;
     MYFLT force, fc1, fc2;
     int next, last;
 
@@ -211,6 +221,7 @@ static int scantPerf(CSOUND *csound, SCANTABLE *p)
     MYFLT inc    = p->size * *(p->kpch) * csound->onedsr;
     MYFLT amp    = *(p->kamp);
     MYFLT pos    = p->pos;
+    MYFLT *aout  = p->aout;
 
 /* CALCULATE NEW POSITIONS
  *
@@ -244,17 +255,22 @@ static int scantPerf(CSOUND *csound, SCANTABLE *p)
         fc2 = (fpoint->ftable[i] - fpoint->ftable[next]) * fstiff->ftable[i];
         force = fc1 + fc2;
         p->newvel[i] = (fvel->ftable[i]
-                        - force / (fmass->ftable[i] * csound->ekr))
+                        - force / (fmass->ftable[i] * CS_EKR))
                        * fdamp->ftable[i];
-        p->newloc[i] = fpoint->ftable[i] + p->newvel[i] * csound->onedkr;
+        p->newloc[i] = fpoint->ftable[i] + p->newvel[i] * CS_ONEDKR;
 
       }
     }
 
-    for (i=0; i<nsmps; i++) {
+    if (UNLIKELY(offset)) memset(aout, '\0', offset*sizeof(MYFLT));
+    if (UNLIKELY(early)) {
+      nsmps -= early;
+      memset(&aout[nsmps], '\0', early*sizeof(MYFLT));
+    }
+    for (i=offset; i<nsmps; i++) {
 
       /* NO INTERPOLATION */
-      p->aout[i] = fpoint->ftable[(int)pos] * amp;
+      aout[i] = fpoint->ftable[(int)pos] * amp;
 
       pos += inc /* p->size * *(p->kpch) * csound->onedsr */;
       if (pos > p->size) {
@@ -278,9 +294,11 @@ static int scantPerf(CSOUND *csound, SCANTABLE *p)
 #define S(x)    sizeof(x)
 
 static OENTRY localops[] = {
-{ "wterrain", S(WAVETER), TR|5,  "a", "kkkkkkii",(SUBR)wtinit, NULL, (SUBR)wtPerf },
-{ "scantable", S(SCANTABLE),TR|5,"a", "kkiiiii",(SUBR)scantinit,NULL,(SUBR)scantPerf},
-{ "scanhammer",S(SCANHAMMER),TB|1,"", "iiii", (SUBR)scanhinit, NULL, NULL    }
+{ "wterrain", S(WAVETER), TR, 5,  "a", "kkkkkkii",
+                                            (SUBR)wtinit, NULL, (SUBR)wtPerf },
+{ "scantable", S(SCANTABLE),TR, 5,"a", "kkiiiii",
+                                         (SUBR)scantinit,NULL,(SUBR)scantPerf},
+{ "scanhammer",S(SCANHAMMER),TB, 1,"", "iiii", (SUBR)scanhinit, NULL, NULL    }
 };
 
 int wave_terrain_init_(CSOUND *csound)
