@@ -270,9 +270,9 @@ PUBLIC EVENT * cscoreDefineEvent(CSOUND *csound, char *s)
     p = &evtmp->p[1];
     q = &evtmp->p[PMAX];
 #ifdef USE_DOUBLE
-    while (sscanf(s,"%lf",p++) > 0)         /* read pfields */
+    while (CS_SSCANF(s,"%lf",p++) > 0)         /* read pfields */
 #else
-    while (sscanf(s,"%f",p++) > 0)          /* read pfields */
+    while (CS_SSCANF(s,"%f",p++) > 0)          /* read pfields */
 #endif
     {
       while ((*s >= '0' && *s <= '9') || *s == '.' || *s == '-')
@@ -843,7 +843,7 @@ PUBLIC int csoundInitializeCscore(CSOUND *csound, FILE* insco, FILE* outsco)
     next = cscoreCreateEvent(csound, PMAX); /* creat EVENT blk receiving buf */
     next->op = '\0';
     savinfdata(csound, csound->scfp,
-               next, FL(0.0), 1, 0, 0);    /* curuntil 0, wasend, non-warp, not eof */
+               next, FL(0.0), 1, 0, 0);/* curuntil 0, wasend, non-warp, not eof */
     makecurrent(csound, csound->scfp);  /* make all this current         */
 
     return CSOUND_SUCCESS;
@@ -873,7 +873,7 @@ PUBLIC int csoundInitializeCscore(CSOUND *csound, FILE* insco, FILE* outsco)
     next->op = '\0';
 
     savinfdata(csound, csound->scfp,
-               next, FL(0.0), 1, 0, 0);    /* curuntil 0, wasend, non-warp, not eof */
+               next, FL(0.0), 1, 0, 0);/* curuntil 0, wasend, non-warp, not eof */
     makecurrent(csound, csound->scfp);  /* make all this current         */
 
     return CSOUND_SUCCESS;
