@@ -23,6 +23,7 @@
 
 #include "csoundCore.h"
 #include <setjmp.h>
+#include "corfile.h"
 
 typedef struct csUtility_s {
     char                *name;
@@ -35,7 +36,8 @@ int csoundAddUtility(CSOUND *csound, const char *name,
                                      int (*UtilFunc)(CSOUND*, int, char**))
 {
     csUtility_t *p;
-    /* csound->Message(csound, "csoundAddUtility: name: %s  function: 0x%p\n", name, UtilFunc); */
+    /* csound->Message(csound, "csoundAddUtility: name: %s  function: 0x%p\n",
+                       name, UtilFunc); */
     if (csound == NULL || name == NULL || name[0] == '\0' || UtilFunc == NULL)
       return -1;
     p = (csUtility_t*) csound->utility_db;
@@ -93,7 +95,7 @@ PUBLIC int csoundRunUtility(CSOUND *csound, const char *name,
         break;
       p = p->nxt;
     }
-    csound->engineState |= CS_STATE_UTIL;
+    csound->engineStatus |= CS_STATE_UTIL;
     csound->scorename = csound->orchname = (char*) name;    /* needed ? */
     csound->Message(csound, Str("util %s:\n"), name);
     n = p->UtilFunc(csound, argc, argv);
@@ -118,7 +120,7 @@ PUBLIC int csoundRunUtility(CSOUND *csound, const char *name,
           csound->Message(csound, "    %s\n", lst[i]);
       }
     }
-    csound->DeleteUtilityList(csound, lst);
+    csoundDeleteUtilityList(csound, lst);
     n = -1;
  err_return:
     memcpy((void*) &(csound->exitjmp), (void*) saved_exitjmp, sizeof(jmp_buf));
@@ -243,11 +245,19 @@ PUBLIC const char *csoundGetUtilityDescription(CSOUND *csound,
 PUBLIC int csoundScoreSort(CSOUND *csound, FILE *inFile, FILE *outFile)
 {
     int   err;
-
+    CORFIL *inf = corfile_create_w();
+    int c;
     if ((err = setjmp(csound->exitjmp)) != 0) {
       return ((err - CSOUND_EXITJMP_SUCCESS) | CSOUND_EXITJMP_SUCCESS);
     }
-    scsort(csound, inFile, outFile);
+    while ((c=getc(inFile))!=EOF) corfile_putc(c, inf);
+    corfile_rewind(inf);
+    /* scsortstr() ignores the second arg - Jan 5 2012 */
+    csound->scorestr = inf;
+    scsortstr(csound, inf);
+    while ((c=corfile_getc(csound->scstr))!=EOF)
+      putc(c, outFile);
+    corfile_rm(&csound->scstr);
     return 0;
 }
 
@@ -262,11 +272,16 @@ PUBLIC int csoundScoreExtract(CSOUND *csound,
                               FILE *inFile, FILE *outFile, FILE *extractFile)
 {
     int   err;
-
+    CORFIL *inf = corfile_create_w();
+    int c;
     if ((err = setjmp(csound->exitjmp)) != 0) {
       return ((err - CSOUND_EXITJMP_SUCCESS) | CSOUND_EXITJMP_SUCCESS);
     }
-    scxtract(csound, inFile, outFile, extractFile);
+    while ((c=getc(inFile))!=EOF) corfile_putc(c, inf);
+    corfile_rewind(inf);
+    scxtract(csound, inf, extractFile);
+    while ((c=corfile_getc(csound->scstr))!=EOF)
+      putc(c, outFile);
+    corfile_rm(&csound->scstr);
     return 0;
 }
-
