@@ -6,12 +6,12 @@ All rights reserved.
 This software is licensed under the terms of the
 GNU Lesser General Public License.
 
-This composition simulates some of the effects in 
+This composition simulates some of the effects in
 LaMonte Young's early electronic pieces, using Csound.
 The piece uses just intonation chords without thirds or sixths,
 tones with varying harmonic content,
 tones with waveshaping distortion,
-and varying amounts of reverberation 
+and varying amounts of reverberation
 to produce a slow progression of rich drones.
 
 The purpose of the GUI is to enable the user to interactively fine-tune
@@ -33,7 +33,7 @@ import 	traceback
 
 '''
 Psyco (http://www.psyco.org) is a just-in-time compiler for Python.
-One imagines that using Psyco here will make execution 
+One imagines that using Psyco here will make execution
 faster and therefore smoother.
 '''
 try:
@@ -44,13 +44,13 @@ except:
 
 from 	Tkinter import *
 import 	tkFileDialog
-import 	Tix 
+import 	Tix
 
 print __doc__
 '''
 A class that contains all control channel configuration values,
 in order to simplify saving and restoring configurations.
-Naming convention: 
+Naming convention:
 Csound global variable name equals Csound control channel name,
 equals Tkinter widget name, equals configuration variable name,
 and the widget command handler name is the same but prefixed with "on_".
@@ -104,11 +104,8 @@ nchnls                  =           2
 
 ; Bind named control channels to global variables.
 
-gkDistortFactor         chnexport   "gkDistortFactor", 3
 gkDistortFactor         init        1.0
-gkReverbscFeedback      chnexport   "gkReverbscFeedback", 3
 gkReverbscFeedback      init        0.9
-gkMasterLevel           chnexport   "gkMasterLevel", 3
 gkMasterLevel           init        1.0
 
 gareverb1               init        0
@@ -120,7 +117,7 @@ idecay                  init        40.0
 isustain        	init        p3 - iattack
 p3          		=           p3 + idecay
 			print       p1, p2, p3, p4, p5, p6
-ifundamental            =           p4  
+ifundamental            =           p4
 inumerator              =           p5
 idenominator            =           p6
 ivelocity               =           p7
@@ -135,8 +132,8 @@ aleft, aright           pan2        asignal , ipan
 gareverb1               =           gareverb1 + aleft
 gareverb2               =           gareverb1 + aright
 			endin
-            
-            
+
+
 			instr 30
 aleft, aright           reverbsc    gareverb1, gareverb2, gkReverbscFeedback, 15000.0
 aleft                   =           gkMasterLevel * (gareverb1 + aleft * 0.8)
@@ -152,30 +149,30 @@ gareverb2               =           0
 ; a good small "live" room sound, .8
 ; a small hall, .9 a large hall,
 ; .99 an enormous stone cavern.
-  
+
 ; p5 = amount of random pitch modulation
 ; for the delay lines. 1 is the "normal"
 ; amount, but this may be too high for
 ; held pitches such as piano tones.
 ; Adjust to taste.
-  
+
 ; p6 = cutoff frequency of lowpass filters
 ; in feedback loops of delay lines,
 ; in Hz. Lower cutoff frequencies results
 ; in a sound with more high-frequency
 ; damping.
-  
-; 8 delay line FDN reverb, with feedback matrix based upon 
+
+; 8 delay line FDN reverb, with feedback matrix based upon
 ; physical modeling scattering junction of 8 lossless waveguides
-; of equal characteristic impedance. Based on Julius O. Smith III, 
+; of equal characteristic impedance. Based on Julius O. Smith III,
 ; "A New Approach to Digital Reverberation using Closed Waveguide
-; Networks," Proceedings of the International Computer Music 
+; Networks," Proceedings of the International Computer Music
 ; Conference 1985, p. 47-53 (also available as a seperate
 ; publication from CCRMA), as well as some more recent papers by
 ; Smith and others.
 ; Coded by Sean Costello, October 1999
-ipitchmod 		= 			0.98 
-itone 			= 			16000     		
+ipitchmod 		= 			0.98
+itone 			= 			16000
 ain1 			=			gareverb1
 ain2 			=			gareverb2
 asignal 		= 			(ain1 + ain2) * 0.5
@@ -206,37 +203,37 @@ k5      		randi   		.001, 2.341, .63
 k6      		randi   		.0011, 1.897, .7
 k7      		randi   		.0017, 0.891, .9
 k8      		randi   		.0006, 3.221, .44
-; apj is used to calculate "resultant junction pressure" for 
+; apj is used to calculate "resultant junction pressure" for
 ; the scattering junction of 8 lossless waveguides
 ; of equal characteristic impedance. If you wish to
-; add more delay lines, simply add them to the following 
-; equation, and replace the .25 by 2/N, where N is the 
+; add more delay lines, simply add them to the following
+; equation, and replace the .25 by 2/N, where N is the
 ; number of delay lines.
 apj 			= 			.25 * (afilt1 + afilt2 + afilt3 + afilt4 + afilt5 + afilt6 + afilt7 + afilt8)
 adum1   		delayr  		1
 adel1   		deltapi 		idel1 + k1 * ipitchmod
-        		delayw  		asignal + apj - afilt1
+				delayw  		asignal + apj - afilt1
 adum2   		delayr  		1
 adel2   		deltapi 		idel2 + k2 * ipitchmod
-        		delayw  		asignal + apj - afilt2
+				delayw  		asignal + apj - afilt2
 adum3   		delayr  		1
 adel3   		deltapi 		idel3 + k3 * ipitchmod
-        		delayw  		asignal + apj - afilt3
+				delayw  		asignal + apj - afilt3
 adum4   		delayr  		1
 adel4   		deltapi 		idel4 + k4 * ipitchmod
-        		delayw  		asignal + apj - afilt4
+				delayw  		asignal + apj - afilt4
 adum5   		delayr  		1
 adel5   		deltapi 		idel5 + k5 * ipitchmod
-        		delayw  		asignal + apj - afilt5
+				delayw  		asignal + apj - afilt5
 adum6   		delayr  		1
 adel6   		deltapi 		idel6 + k6 * ipitchmod
-          		delayw  		asignal + apj - afilt6
+				delayw  		asignal + apj - afilt6
 adum7   		delayr  		1
 adel7   		deltapi 		idel7 + k7 * ipitchmod
-        		delayw  		asignal + apj - afilt7
+				delayw  		asignal + apj - afilt7
 adum8   		delayr  		1
 adel8   		deltapi 		idel8 + k8 * ipitchmod
-        		delayw  		asignal + apj - afilt8
+				delayw  		asignal + apj - afilt8
 ; 1st order lowpass filters in feedback
 ; loops of delay lines.
 afilt1  		tone    		adel1 * gkReverbscFeedback, itone
@@ -249,7 +246,7 @@ afilt7  		tone    		adel7 * gkReverbscFeedback, itone
 afilt8  		tone    		adel8 * gkReverbscFeedback, itone
 ; The outputs of the delay lines are summed
 ; and sent to the stereo outputs. This could
-; easily be modified for a 4 or 8-channel 
+; easily be modified for a 4 or 8-channel
 ; sound system.
 aout1 			= 			(afilt1 + afilt3 + afilt5 + afilt7)
 aout2 			= 			(afilt2 + afilt4 + afilt6 + afilt8)
@@ -261,13 +258,21 @@ gareverb1               =           0
 gareverb2               =           0
 			endin
 
+instr 50
+gkHarmonicTableFactor chnget "gkHarmonicTableFactor"
+gkDistortTableFactor chnget "gkDistortTableFactor"
+gkDistortFactor chnget "gkDistortFactor"
+gkReverbscFeedback chnget "gkReverbscFeedback"
+gkMasterLevel chnget "gkMasterLevel"
+endin
+
 '''
 
 csoundScore = '''\
 ;                       A few harmonics...
-f   1       0           65536       10      3   0   1   0   0   2 
+f   1       0           65536       10      3   0   1   0   0   2
 ;                       ...distorted by waveshaping.
-f   2       0           65536       13      1   1   0   3   0   2    
+f   2       0           65536       13      1   1   0   3   0   2
 ;                       Change the tempo, if you like.
 t   0      30
 
@@ -290,12 +295,14 @@ i   1       120     60          60         	    2          1            58
 
 i   30      0       -1
 
+i   50      0       -1
+
 s   10.0
 e   10.0
 '''
 
 csoundCommands = {
-	'Audio':    'csound --messagelevel=1 --noheader                         --nodisplays --sample-rate=44100 --control-rate=100   --midi-key=4 --midi-velocity=5                                                                  --output=%s' % (			   dacName),
+	'Audio':    'csound --messagelevel=3 --noheader                         --nodisplays --sample-rate=44100 --control-rate=100   --midi-key=4 --midi-velocity=5                                                                  --output=%s' % (			   dacName),
 	'Preview':  'csound --messagelevel=3 -W -f --rewrite --dither --nopeaks --nodisplays --sample-rate=44100 --control-rate=441   --midi-key=4 --midi-velocity=5 -+id_artist=%s -+id_copyright=Copyright_2007_by_%s -+id_title=%s --output=%s' % (author, author, title, soundfileName),
 	'CD':       'csound --messagelevel=3 -W -f --rewrite --dither --nopeaks --nodisplays --sample-rate=44100 --control-rate=44100 --midi-key=4 --midi-velocity=5 -+id_artist=%s -+id_copyright=Copyright_2007_by_%s -+id_title=%s --output=%s' % (author, author, title, soundfileName),
 	'Master':   'csound --messagelevel=3 -W -f --rewrite --dither --nopeaks --nodisplays --sample-rate=88200 --control-rate=88200 --midi-key=4 --midi-velocity=5 -+id_artist=%s -+id_copyright=Copyright_2007_by_%s -+id_title=%s --output=%s' % (author, author, title, soundfileName)
@@ -312,7 +319,7 @@ class Application(Frame):
 		self.master.title('D R O N E')
 		self.pack()
 		self.csound = csnd6.CppSound()
-		self.csound.setPythonMessageCallback()
+		#self.csound.setPythonMessageCallback()
 		self.playing = False
 		self.thread_ = thread_
 
@@ -326,7 +333,7 @@ class Application(Frame):
 
 		self.harmonicsFrame = Frame(
 		self.leftFrame,
-		bd = 2, 
+		bd = 2,
 		relief = 'groove'
 		)
 		self.harmonicsFrame.grid(row=r, column=c, sticky=N+E+W+S, padx=4, pady=4)
@@ -339,9 +346,9 @@ class Application(Frame):
 		r = r + 1
 
 		self.gkHarmonicTableFactor = Scale(
-		self.harmonicsFrame, 
-		from_ = 0.0, 
-		to = 2.0, 
+		self.harmonicsFrame,
+		from_ = 0.0,
+		to = 2.0,
 		resolution = 0.001,
 		length = 250,
 		orient = HORIZONTAL,
@@ -366,9 +373,9 @@ class Application(Frame):
 		r = r + 1
 
 		self.gkDistortTableFactor = Scale(
-		self.distortFrame, 
-		from_ = 0.0, 
-		to = 10.0, 
+		self.distortFrame,
+		from_ = 0.0,
+		to = 10.0,
 		resolution = 0.001,
 		length = 250,
 		orient = HORIZONTAL,
@@ -379,9 +386,9 @@ class Application(Frame):
 		r = r + 1
 
 		self.gkDistortFactor = Scale(
-		self.distortFrame, 
-		from_ = 0.0, 
-		to = 1.0, 
+		self.distortFrame,
+		from_ = 0.0,
+		to = 1.0,
 		resolution = 0.001,
 		length = 250,
 		orient = HORIZONTAL,
@@ -412,9 +419,9 @@ class Application(Frame):
 		r = r + 1
 
 		self.gkReverbscFeedback = Scale(
-		self.reverbFrame, 
-		from_ = 0.0, 
-		to = 0.9999, 
+		self.reverbFrame,
+		from_ = 0.0,
+		to = 0.9999,
 		resolution = 0.001,
 		length = 250,
 		orient = HORIZONTAL,
@@ -439,9 +446,9 @@ class Application(Frame):
 		r = r + 1
 
 		self.gkMasterLevel = Scale(
-		self.FactorFrame, 
-		from_ = 0.0, 
-		to = 2.0, 
+		self.FactorFrame,
+		from_ = 0.0,
+		to = 2.0,
 		resolution = 0.001,
 		length = 250,
 		orient = HORIZONTAL,
@@ -528,7 +535,7 @@ class Application(Frame):
 		r = r + 1
 
 		self.configure()
-    
+
 	'''
 	Set initial control channel values.
 	'''
@@ -565,15 +572,15 @@ class Application(Frame):
 				message = 'f  2  0  65536  13  1  %f  0  %f  0  %f\n' % (f * 1.0, f * 2.0, f * 3.0)
 				self.csound.inputMessage(message)
 				self.csound.SetChannel("gkDistortFactor",           float(self.gkDistortFactor.get()))
-				self.csound.SetChannel("gkReverbscFeedback",        float(self.gkReverbscFeedback.get()))        
+				self.csound.SetChannel("gkReverbscFeedback",        float(self.gkReverbscFeedback.get()))
 				self.csound.SetChannel("gkMasterLevel",             float(self.gkMasterLevel.get()))
 				# Tkinter only likes 1 thread per application.
-				# So, we hack the rules and switch back and forth between 
+				# So, we hack the rules and switch back and forth between
 				# computing sound and handling GUI events.
 				# When the user closes the application, self.update will raise
 				# an exception because the application has been destroyed.
 				kperiod = 1
-				while self.playing and not self.csound.performKsmps():
+				while self.playing and not self.csound.PerformKsmps():
 					kperiod = kperiod + 1
 					if kperiod % 10000 == 0:
 						scoreTime = self.csound.GetScoreTime()
@@ -582,8 +589,8 @@ class Application(Frame):
 						self.update()
 					except TclError:
 						traceback.print_exc()
-						self.csound.cleanup()
 						self.playing = False
+				self.csound.Reset()
 				self.playButton['text'] = 'Play'
 				self.playing = False
 				if output != 'Audio':
@@ -591,7 +598,7 @@ class Application(Frame):
 					print
 					print 'Soundfile:             "%s"' % soundfileName
 					os.spawnl(os.P_NOWAIT, soundfilePlayer, soundfilePlayer, soundfileName)
-					print									
+					print
 			except:
 				traceback.print_exc()
 		else:
@@ -601,7 +608,7 @@ class Application(Frame):
 				self.playing = False
 			except:
 				print traceback.print_exc()
-        
+
 	def on_load(self):
 		try:
 			filename =  tkFileDialog.askopenfilename(filetypes=[('Drone files', "*.pickled"), ("All files", "*")])
@@ -614,7 +621,7 @@ class Application(Frame):
 
 		except:
 			traceback.print_exc()
-    
+
 	def on_save(self):
 		try:
 			self.configuration.output = self.outputStringVar.get()
@@ -626,13 +633,13 @@ class Application(Frame):
 			self.master.title('D R O N E   %s' % filename)
 		except:
 			traceback.print_exc()
-        
+
 	def on_gkHarmonicTableFactor(self, value):
 		f = float(value)
 		self.configuration.gkHarmonicTableFactor = f
 		message = 'f  1  0  65536  10  3  0  %f  0  0 %f\n' % (f * 1.0, f * 2.0)
 		self.csound.inputMessage(message)
-    
+
 	def on_gkDistortTableFactor(self, value):
 		f = float(value)
 		self.configuration.gkDistortTableFactor = f
@@ -655,7 +662,7 @@ try:
 	tk = Tix.Tk()
 	application = Application(tk)
 	application.mainloop()
-	# Before the application exits, its reference to Csound 
+	# Before the application exits, its reference to Csound
 	# must be nulled, so that Csound is not deleted twice
 	# (once by Tkinter and once by the Python garbage collector).
 	application.csound = None
