@@ -28,15 +28,19 @@
 #include <boost/format.hpp>
 #include <sstream>
 #if defined(HAVE_IO_H)
+#ifdef LINUX
+#include <sys/io.h>
+#else
 #include <io.h>
 #endif
+#endif
 #include <stdio.h>
 using boost::format;
 using boost::io::group;
 
 namespace csound
 {
-  ChordLindenmayer::ChordLindenmayer() : angle(1.0)
+  ChordLindenmayer::ChordLindenmayer() : iterationCount(0), angle(1.0)
   {
   }
 
@@ -620,7 +624,7 @@ namespace csound
         break;
       case 'R':
         {
-	  Eigen::MatrixXd rotation = createRotation(dimension, dimension1, scalar);
+          Eigen::MatrixXd rotation = createRotation(dimension, dimension1, scalar);
           turtle.orientation = rotation * turtle.orientation;
         }
         break;
