@@ -30,6 +30,7 @@
 #include "System.hpp"
 #include "CsoundFile.hpp"
 #include "Conversions.hpp"
+#include <csound.h>
 
 namespace csound
 {
@@ -44,7 +45,7 @@ namespace csound
 
   void Logger::write(const char *text)
   {
-    fprintf(stderr, text);
+    fprintf(stderr, "%s", text);
   }
 
   ThreadLock::ThreadLock() : lock(0)
