@@ -84,6 +84,7 @@ namespace csound
     }
     return transposed;
   }
+
   double round(double x)
   {
     return std::floor(x + 0.5);
@@ -99,7 +100,7 @@ namespace csound
   double Voicelead::pc(double pitchSemitones, size_t divisionsPerOctave)
   {
     double absolutePitchSemitones = std::abs(pitchSemitones);
-    double pitchClass = std::fmod(absolutePitchSemitones, semitonesPerOctave);
+    double pitchClass = std::fmod(absolutePitchSemitones, 12.0);
     return pitchClass;
   }
 
@@ -147,7 +148,6 @@ namespace csound
     return result;
   }
 
-
   std::vector<double> Voicelead::voiceleading(const std::vector<double> &a,
                                               const std::vector<double> &b)
   {
@@ -294,21 +294,15 @@ namespace csound
 
   std::vector<double> Voicelead::uniquePcs(const std::vector<double> &chord, size_t divisionsPerOctave)
   {
-    std::vector<double> pcs_;
-    std::set<double> pcsSet;
-    for (size_t i = 0, n = chord.size(); i < n; i++) {
-      double pc_ = pc(chord[i], divisionsPerOctave);
-      if (pcsSet.find(pc_) == pcsSet.end()) {
-        pcsSet.insert(pc_);
-        pcs_.push_back(pc_);
+    std::vector<double> uniquepcs;
+    for (size_t i = 0, n = chord.size(); i < n; ++i) {
+      double pc_ = pc(chord[i]);
+      if (std::find(uniquepcs.begin(), uniquepcs.end(), pc_) == uniquepcs.end()) {
+        uniquepcs.push_back(pc_);
       }
     }
-    std::sort(pcs_.begin(), pcs_.end());
-    if (debug > 2) {
-      std::cerr << "chord: " << chord << std::endl;
-      std::cerr << "pcs: " << pcs_ << std::endl;
-    }
-    return pcs_;
+    sort(uniquepcs);
+    return uniquepcs;
   }
 
   const std::vector<double> Voicelead::closest(const std::vector<double> &source,
@@ -776,10 +770,14 @@ namespace csound
     return result;
   }
 
-  bool Voicelead::addOctave(const std::vector<double> &lowestVoicing, std::vector<double> &newVoicing, size_t maximumPitch, size_t divisionsPerOctave)
+  bool Voicelead::addOctave(const std::vector<double>
+                            &lowestVoicing,
+                            std::vector<double> &newVoicing,
+                            size_t maximumPitch,
+                            size_t divisionsPerOctave)
   {
     for (size_t voice = 0, voices = lowestVoicing.size(); voice < voices; voice++) {
-      double newPitch = newVoicing[voice] + double(divisionsPerOctave);
+      double newPitch = newVoicing[voice] + 12.0;
       if (newPitch >= maximumPitch) {
         newVoicing[voice] = lowestVoicing[voice];
       } else {
@@ -819,22 +817,22 @@ namespace csound
     std::vector<double> zeroVoicing = normalChord(pAndTtoPitchClassSet(P, T, divisionsPerOctave));
     while (zeroVoicing[0] < lowestPitch) {
       for (size_t i = 0, n = zeroVoicing.size(); i < n; i++) {
-        zeroVoicing[i] += double(divisionsPerOctave);
+        zeroVoicing[i] += 12.0;
       }
     }
     while (zeroVoicing[0] >= (lowestPitch + double(divisionsPerOctave))) {
       for (size_t i = 0, n = zeroVoicing.size(); i < n; i++) {
-        zeroVoicing[i] -= double(divisionsPerOctave);
+        zeroVoicing[i] -= 12.0;
       }
     }
     std::vector<double> zeroVoicing_ = sort(zeroVoicing);
     std::vector<double> zeroIterator = pcs(zeroVoicing, divisionsPerOctave);
     for(size_t i = 0, n = zeroIterator.size(); i < n; i++) {
       while (zeroIterator[i] < lowestPitch) {
-        zeroIterator[i] += double(divisionsPerOctave);
+        zeroIterator[i] += 12.0;
       }
       while (zeroIterator[i] >= (lowestPitch + double(divisionsPerOctave))) {
-        zeroIterator[i] -= double(divisionsPerOctave);
+        zeroIterator[i] -= 12.0;
       }
     }
     size_t zeroVoicingEnumeration = 0;
@@ -876,22 +874,22 @@ namespace csound
     std::vector<double> zeroVoicing = normalChord(chord_);
     while (zeroVoicing[0] < lowestPitch) {
       for (size_t i = 0, n = zeroVoicing.size(); i < n; i++) {
-        zeroVoicing[i] += double(divisionsPerOctave);
+        zeroVoicing[i] += 12.0;
       }
     }
     while (zeroVoicing[0] >= (lowestPitch + double(divisionsPerOctave))) {
       for (size_t i = 0, n = zeroVoicing.size(); i < n; i++) {
-        zeroVoicing[i] -= double(divisionsPerOctave);
+        zeroVoicing[i] -= 12.0;
       }
     }
     std::vector<double> zeroVoicing_ = sort(zeroVoicing);
     std::vector<double> zeroIterator = pcs(zeroVoicing, divisionsPerOctave);
     for(size_t i = 0, n = zeroIterator.size(); i < n; i++) {
       while (zeroIterator[i] < lowestPitch) {
-        zeroIterator[i] += double(divisionsPerOctave);
+        zeroIterator[i] += 12.0;
       }
       while (zeroIterator[i] >= (lowestPitch + double(divisionsPerOctave))) {
-        zeroIterator[i] -= double(divisionsPerOctave);
+        zeroIterator[i] -= 12.0;
       }
     }
     size_t zeroVoicingEnumeration = 0;
@@ -944,4 +942,86 @@ namespace csound
     }
     return voicings;
   }
+
+  double Voicelead::T(double p, double n)
+  {
+    return pc(p + n);
+  }
+
+  std::vector<double> Voicelead::T(const std::vector<double> &c, double t)
+  {
+    std::vector<double> returnValue(c.size());
+    for (size_t i = 0, n = c.size(); i < n; ++i) {
+      returnValue[i] = pc(c[i] + t);
+    }
+    sort(returnValue);
+    return returnValue;
+  }
+
+  double Voicelead::I(double p, double n)
+  {
+    return pc((12.0 - pc(p)) + n);
+  }
+
+  std::vector<double> Voicelead::I(const std::vector<double> &c, double ii)
+  {
+    std::vector<double> returnValue(c.size());
+    for (size_t i = 0, n = c.size(); i < n; ++i) {
+      returnValue[i] = I(c[i], ii);
+    }
+    sort(returnValue);
+    return returnValue;
+  }
+
+  std::vector<double> Voicelead::K(const std::vector<double> &c)
+  {
+    if (c.size() < 2) {
+      return c;
+    }
+    double n = c[0] + c[1];
+    return I(c, n);
+  }
+
+  bool Voicelead::Tform(const std::vector<double> &X, const std::vector<double> &Y, double g)
+  {
+    double i = 0.0;
+    std::vector<double> pcsx = pcs(X);
+    while (i < 12.0) {
+      std::vector<double> ty = T(Y, i);
+      std::vector<double> pcsty = pcs(ty);
+      if (pcsx == pcsty) {
+        return true;
+      }
+      i = i + g;
+    }
+    return false;
+  }
+
+  bool Voicelead::Iform(const std::vector<double> &X, const std::vector<double> &Y, double g)
+  {
+    double i = 0.0;
+    std::vector<double> pcsx = pcs(X);
+    while (i < 12.0) {
+      std::vector<double> iy = I(Y, i);
+      std::vector<double> pcsiy = pcs(iy);
+      if (pcsx == pcsiy) {
+        return true;
+      }
+      i = i + g;
+    }
+    return false;
+  }
+
+  std::vector<double> Voicelead::Q(const std::vector<double> &c, double n, const std::vector<double> &s, double g)
+  {
+    if (Tform(c, s, g)) {
+      return T(c, n);
+    }
+    if (Iform(c, s, g)) {
+      return T(c, -n);
+    } else {
+      return c;
+    }
+  }
+
 }
