@@ -120,24 +120,19 @@ int main(int argc, char **argv)
         return -1;
       }
     }
-
-    /* initialise Csound library */
-    csoundInitialize(&argc, &argv, 0);
+    /* if logging to file, set message callback */
+    if (logFile != NULL)
+      csoundSetDefaultMessageCallback(msg_callback);
+    else if (nomessages)
+      csoundSetDefaultMessageCallback(nomsg_callback);
 
     /*  Create Csound. */
     csound = csoundCreate(NULL);
 
-    /* if logging to file, set message callback */
-    if (logFile != NULL)
-      csoundSetMessageCallback(csound, msg_callback);
-    else if (nomessages)
-      csoundSetMessageCallback(csound, nomsg_callback);
-
     /*  One complete performance cycle. */
     result = csoundCompile(csound, argc, argv);
-    if (!result)
-      result = csoundPerform(csound);
 
+     if(!result) csoundPerform(csound);
     /* delete Csound instance */
      csoundDestroy(csound);
     /* close log file */
@@ -149,4 +144,3 @@ int main(int argc, char **argv)
 #endif
     return (result >= 0 ? 0 : result);
 }
-
