@@ -180,12 +180,12 @@ void csound_assist(t_csound *x, void *b, long msg, long arg, char *s)
 		switch(arg)
 		{
 		case 0:  sprintf(s, "(signal) Audio In 0 / (int) MIDI Input"); break;
-		default: sprintf(s, "(signal) Audio In %l", arg); break;
+		default: sprintf(s, "(signal) Audio In %ld", arg); break;
 		}
 	}
 	else if(msg == ASSIST_OUTLET)
 	{
-		if(arg < x->numOutSignals) sprintf(s, "(signal) Audio Out %l", arg);
+		if(arg < x->numOutSignals) sprintf(s, "(signal) Audio Out %ld", arg);
 		else
 		{
 			i = arg - x->numOutSignals;
@@ -366,7 +366,8 @@ void csound_dsp(t_csound *x, t_signal **sp, short *count)
 void csound_dsp64(t_csound *x, t_object *dsp64, short *count, double samplerate, long maxvectorsize, long flags)
 {
 	CsoundObject *cso = x->cso;
-	int i=0, totalVectors=0;
+//	int i=0, totalVectors=0;
+    int totalVectors=0;
 //	void **perform_args;
 	
 	totalVectors = x->numInSignals + x->numOutSignals;
@@ -429,8 +430,8 @@ void csound_control(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 	name = argv[0].a_w.w_sym->s_name;
 	switch(argv[1].a_type)
 	{
-	case A_FLOAT: f_val = (MYFLT) argv[1].a_w.w_float; break;
-	case A_LONG:  f_val = (MYFLT) argv[1].a_w.w_long; break;
+	case A_FLOAT: f_val = (MYFLT) atom_getfloat(&argv[1]); break;
+	case A_LONG:  f_val = (MYFLT) atom_getlong(&argv[1]); break;
 	case A_SYM:   s_val = argv[1].a_w.w_sym->s_name; break;
 	}
 
@@ -476,7 +477,7 @@ void csound_midi(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 		switch(argv[i].a_type)
 		{
 		case A_LONG:
-			buffer[i] = (byte) argv[i].a_w.w_long;
+			buffer[i] = (byte) atom_getlong(&argv[i]);
 			break;
 		default: 
 			object_error(x->m_obj, "Only integers are allowed in midi messages.");
@@ -543,12 +544,12 @@ void csound_event(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 		switch(argv[i].a_type)
 		{
 		case A_LONG:
-			sprintf(tmp, " %lld", argv[i].a_w.w_long);
+			sprintf(tmp, " %lld", (long long)atom_getlong(&argv[i]));
 			totalSize += strlen(tmp);
 			strncat(buffer, tmp, MAX_EVENT_MESSAGE_SIZE - strlen(buffer) - 1);
 			break;
 		case A_FLOAT:
-			sprintf(tmp, " %f", argv[i].a_w.w_float);
+			sprintf(tmp, " %f", atom_getfloat(&argv[i]));
 			totalSize += strlen(tmp);
 			strncat(buffer, tmp, MAX_EVENT_MESSAGE_SIZE - strlen(buffer) - 1);
 			break;
@@ -649,7 +650,7 @@ void *csound_new(t_symbol *s, short argc, t_atom *argv)
 			switch(argv[i].a_type)
 			{
 				case A_LONG:
-					iarg = argv[i].a_w.w_long;
+					iarg = atom_getlong(&argv[i]);
 
 					if(lastAttr != NULL)
 					{
@@ -664,9 +665,9 @@ void *csound_new(t_symbol *s, short argc, t_atom *argv)
 					else
 					{
 						if(numArgCount == 0)
-							x->numInSignals = x->numOutSignals = argv[i].a_w.w_long;
+							x->numInSignals = x->numOutSignals = atom_getlong(&argv[i]);
 						else if(numArgCount == 1)
-							x->numOutSignals = argv[i].a_w.w_long;
+							x->numOutSignals = atom_getlong(&argv[i]);
 						else
 							object_error(x->m_obj, "Too many integer arguments.");
 						++numArgCount;
@@ -1019,8 +1020,8 @@ void csound_loadsamp(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 	float sizeSeconds = 0.0f; 
 	
 	if(argc < 3 || argv[0].a_type != A_LONG || argv[1].a_type != A_LONG || argv[2].a_type != A_SYM) return;
-	tableNum = argv[0].a_w.w_long;
-	channel = argv[1].a_w.w_long;
+	tableNum = atom_getlong(&argv[0]);
+	channel = atom_getlong(&argv[1]);
 	strncpy(filename, argv[2].a_w.w_sym->s_name, MAX_STRING_LENGTH-1);
 	
 	// If 4'th argument exists, it specifies offset time in seconds.
@@ -1028,8 +1029,8 @@ void csound_loadsamp(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 	{
 		switch(argv[3].a_type) 
 		{
-		case A_FLOAT: offsetSeconds = argv[3].a_w.w_float; break;
-		case A_LONG:  offsetSeconds = (float) argv[3].a_w.w_long; break;
+		case A_FLOAT: offsetSeconds = (float) atom_getfloat(&argv[3]); break;
+		case A_LONG:  offsetSeconds = (float) atom_getlong(&argv[3]); break;
 		}
 	}
 	
@@ -1038,8 +1039,8 @@ void csound_loadsamp(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 	{
 		switch(argv[4].a_type) 
 		{
-		case A_FLOAT: sizeSeconds = argv[4].a_w.w_float; break;
-		case A_LONG:  sizeSeconds = (float) argv[4].a_w.w_long; break;
+		case A_FLOAT: sizeSeconds = atom_getfloat(&argv[4]); break;
+		case A_LONG:  sizeSeconds = (float) atom_getlong(&argv[4]); break;
 		}
 	}
 	
@@ -1072,16 +1073,16 @@ void csound_readbuf(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 	float sizeSeconds = 0.0f;
 	
 	if(argc < 3 || argv[0].a_type != A_LONG || argv[1].a_type != A_LONG || argv[2].a_type != A_SYM) return;
-	tableNum = argv[0].a_w.w_long;
-	channel = argv[1].a_w.w_long;
+	tableNum = atom_getlong(&argv[0]);
+	channel = atom_getlong(&argv[1]);
 	
 	// If 4'th argument exists, it specifies offset time in seconds.
 	if(argc > 3)
 	{
 		switch(argv[3].a_type) 
 		{
-		case A_FLOAT: offsetSeconds = argv[3].a_w.w_float;  break;
-		case A_LONG:  offsetSeconds = (float) argv[3].a_w.w_long;  break;
+		case A_FLOAT: offsetSeconds = (float) atom_getfloat(&argv[3]);  break;
+		case A_LONG:  offsetSeconds = (float) atom_getlong(&argv[3]);  break;
 		}
 	}
 	
@@ -1090,8 +1091,8 @@ void csound_readbuf(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 	{
 		switch(argv[4].a_type) 
 		{
-		case A_FLOAT: sizeSeconds = argv[4].a_w.w_float;  break;
-		case A_LONG:  sizeSeconds = (float) argv[4].a_w.w_long; break;
+		case A_FLOAT: sizeSeconds = (float) atom_getfloat(&argv[4]);  break;
+		case A_LONG:  sizeSeconds = (float) atom_getlong(&argv[4]); break;
 		}
 	}
 
@@ -1112,16 +1113,16 @@ void csound_writebuf(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 	float sizeSeconds = 0.0f; 
 	
 	if(argc < 3 || argv[0].a_type != A_LONG || argv[1].a_type != A_LONG || argv[2].a_type != A_SYM) return;
-	tableNum = argv[0].a_w.w_long;
-	channel = argv[1].a_w.w_long;
+	tableNum = atom_getlong(&argv[0]);
+	channel = atom_getlong(&argv[1]);
 	
 	// If 4'th argument exists, it specifies offset time in seconds.
 	if(argc > 3)
 	{
 		switch(argv[3].a_type) 
 		{
-		case A_FLOAT: offsetSeconds = argv[3].a_w.w_float;  break;
-		case A_LONG:  offsetSeconds = (float) argv[3].a_w.w_long;  break;
+		case A_FLOAT: offsetSeconds = (float) atom_getfloat(&argv[3]);  break;
+		case A_LONG:  offsetSeconds = (float) atom_getlong(&argv[3]);  break;
 		}
 	}
 	
@@ -1130,8 +1131,8 @@ void csound_writebuf(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 	{
 		switch(argv[4].a_type) 
 		{
-		case A_FLOAT: sizeSeconds = argv[4].a_w.w_float;  break;
-		case A_LONG:  sizeSeconds = (float) argv[4].a_w.w_long;  break;
+		case A_FLOAT: sizeSeconds = (float) atom_getfloat(&argv[4]);  break;
+		case A_LONG:  sizeSeconds = (float) atom_getlong(&argv[4]);  break;
 		}
 	}
 
@@ -1151,8 +1152,8 @@ void csound_rsidx(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 	MYFLT val;
 	
 	if(argc != 2 || argv[0].a_type != A_LONG || argv[1].a_type != A_LONG) return;
-	tableNum = argv[0].a_w.w_long;
-	index = argv[1].a_w.w_long;
+	tableNum = atom_getlong(&argv[0]);
+	index = atom_getlong(&argv[1]);
 	if(index < 0) return;
 	
 	{
@@ -1167,9 +1168,9 @@ void csound_rsidx(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 	switch(result)
 	{
 	case 0:
-		x->atomList[1].a_w.w_long = tableNum;
-		x->atomList[2].a_w.w_long = index;
-		x->atomList[3].a_w.w_float = (float) val;
+        atom_setlong(&x->atomList[1], (t_atom_long)tableNum);
+        atom_setlong(&x->atomList[2], (t_atom_long)index);
+        atom_setfloat(&x->atomList[3], (t_atom_float)val);
 		outlet_list(x->message_outlet, 0L, 4, x->atomList);
 		break;
 	case 1:
@@ -1188,9 +1189,9 @@ void csound_wsidx(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 	
 	if(argc != 3 || argv[0].a_type != A_LONG || argv[1].a_type != A_LONG || 
 	   (argv[2].a_type != A_FLOAT && argv[2].a_type != A_LONG)) return;
-	tableNum = argv[0].a_w.w_long;
-	index = argv[1].a_w.w_long;
-	val = (float)(argv[2].a_type == A_FLOAT ? argv[2].a_w.w_float : argv[2].a_w.w_long);
+	tableNum = atom_getlong(&argv[0]);
+	index = atom_getlong(&argv[1]);
+	val = (float)(argv[2].a_type == A_FLOAT ? atom_getfloat(&argv[2]) : atom_getlong(&argv[2]));
 	if(index < 0) return;
 	
 	{
@@ -1232,11 +1233,11 @@ void csound_run(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 		switch(argv[i].a_type)
 		{
 		case A_FLOAT:
-			snprintf(tmp, MAX_STRING_LENGTH-1, "%l", argv[i].a_w.w_long);
+			snprintf(tmp, MAX_STRING_LENGTH-1, "%f", (float)atom_getfloat(&argv[i]));
 			args[i] = strdup(tmp);
 			break;
 		case A_LONG:
-			snprintf(tmp, MAX_STRING_LENGTH-1, "%f", argv[i].a_w.w_float);
+			snprintf(tmp, MAX_STRING_LENGTH-1, "%lld", (long long)atom_getlong(&argv[i]));
 			args[i] = strdup(tmp);
 			break;
 		case A_SYM:
@@ -1276,10 +1277,10 @@ void csound_run(t_csound *x, t_symbol *s, short argc, t_atom *argv)
 		switch(argv[i].a_type)
 		{
 		case A_FLOAT:
-			snprintf(tmp, MAX_STRING_LENGTH-1, "%d", argv[i].a_w.w_long);
+			snprintf(tmp, MAX_STRING_LENGTH-1, "%f", (float)atom_getfloat(&argv[i]));
 			break;
 		case A_LONG:
-			snprintf(tmp, MAX_STRING_LENGTH-1, "%f", argv[i].a_w.w_float);
+			snprintf(tmp, MAX_STRING_LENGTH-1, "%lld", (long long)atom_getlong(argv[i]));
 			break;
 		case A_SYM:
 			strncpy(tmp, argv[i].a_w.w_sym->s_name, MAX_STRING_LENGTH-1);
