@@ -7,7 +7,8 @@
 #include "winsound.h"
 #include <errno.h>
 #include <stdarg.h>
-#if defined(MSVC)
+#include <unistd.h>
+#if defined(WIN32)
 #include <direct.h>
 #endif
 #include <FL/Fl_Preferences.H>
@@ -65,10 +66,9 @@ int main(int argc, char **argv)
     uw = make_utils();
     xw = make_extras();
     textw = make_textwindow();
-    csoundInitialize(&argc, &argv, CSOUNDINIT_NO_SIGNAL_HANDLER);
+    csoundInitialize(CSOUNDINIT_NO_SIGNAL_HANDLER);
     csound = csoundCreate(NULL);
     csoundSetMessageCallback(csound, mytextNull);
-    csoundPreCompile(csound);
     csoundSetMessageCallback(csound, mytextOutput);
     csoundSetYieldCallback(csound, yieldCallback);
     mw->show();
@@ -112,7 +112,7 @@ void cs_compile_run(void)
       prof.set("f",size_f->value());
       prof.set("3",size_24->value());
       prof.set("K",mK->value());
-      argv[0] = (char*)"csound5";
+      argv[0] = (char*)"csound6";
       argv[nxt++] = (char *)orchname->value();
       if (strstr(argv[nxt-1], ".csd")==NULL)
         argv[nxt++] = (char *)scorename->value();
@@ -205,28 +205,27 @@ void cs_compile_run(void)
           Fl::wait(0);
         }
       };
-      res = csoundPreCompile(csound);
 //       {
 //         int n;
 //         printf("nxt=%d\n", nxt);
 //         for (n=0; n<nxt; n++) printf("%d: \"%s\"\n", n, argv[n]);
 //       }
-      if (res == 0) {
+
         // set default, but allow to be overridden by .csoundrc or <CsOptions>
 #if defined(WIN32)
-        csoundParseConfigurationVariable(csound, "rtaudio", "pa_cb");
+      csoundParseConfigurationVariable(csound, "rtaudio", "pa_cb");
 #elif defined(LINUX)
-        csoundParseConfigurationVariable(csound, "rtaudio", "alsa");
+      csoundParseConfigurationVariable(csound, "rtaudio", "alsa");
 #elif defined(OSX)
-        csoundParseConfigurationVariable(csound, "rtaudio", "CoreAudio");
+      csoundParseConfigurationVariable(csound, "rtaudio", "CoreAudio");
 #else
 #endif
-        // disable threading in widgets plugin, and also graphs for safety
-        // (the latter is only needed with a static FLTK library)
-        csoundCreateGlobalVariable(csound, "FLTK_Flags", sizeof(int));
-        *((int*) csoundQueryGlobalVariable(csound, "FLTK_Flags")) = 30;
-        res = csoundCompile(csound, nxt, argv);
-      }
+      // disable threading in widgets plugin, and also graphs for safety
+      // (the latter is only needed with a static FLTK library)
+      csoundCreateGlobalVariable(csound, "FLTK_Flags", sizeof(int));
+      *((int*) csoundQueryGlobalVariable(csound, "FLTK_Flags")) = 30;
+      res = csoundCompile(csound, nxt, argv);
+
       if (chdir(olddir)) {
           text->insert(strerror(errno));
           text->show_insert_position();
@@ -260,7 +259,6 @@ void cs_util_sndinfo(void)
       textw->show();
       argv[0] = (char*)"sndinfo";
       argv[1] = (char *)sndinfo_file->value();
-      csoundPreCompile(csound);
       csoundRunUtility(csound, "sndinfo", 2, argv);
       csoundReset(csound);
     }
@@ -364,7 +362,6 @@ void cs_util_het(void)
       else if (het_c4->value()) argv[nxt++] = (char*)"-c4";
       argv[nxt++] = (char *)het_analin->value();
       argv[nxt++] = (char *)het_analout->value();
-      csoundPreCompile(csound);
       csoundRunUtility(csound, "hetro", nxt, argv);
       csoundReset(csound);
     }
@@ -440,7 +437,6 @@ void cs_util_lpc(void)
       else if (lpc_c4->value()) argv[nxt++] = (char*)"-c4";
       argv[nxt++] = (char *)lpc_analin->value();
       argv[nxt++] = (char *)lpc_analout->value();
-      csoundPreCompile(csound);
       csoundRunUtility(csound, "lpanal", nxt, argv);
       csoundReset(csound);
     }
@@ -513,7 +509,6 @@ void cs_util_pvc(void)
       else if (pvc_c4->value()) argv[nxt++] = (char*)"-c4";
       argv[nxt++] = (char *)pvc_analin->value();
       argv[nxt++] = (char *)pvc_analout->value();
-      csoundPreCompile(csound);
       csoundRunUtility(csound, "pvanal", nxt, argv);
       csoundReset(csound);
     }
@@ -565,7 +560,6 @@ void cs_util_cvl(void)
       else if (cvl_c4->value()) argv[nxt++] = (char*)"-c4";
       argv[nxt++] = (char *)cvl_analin->value();
       argv[nxt++] = (char *)cvl_analout->value();
-      csoundPreCompile(csound);
       csoundRunUtility(csound, "cvanal", nxt, argv);
       csoundReset(csound);
     }
@@ -615,7 +609,6 @@ void cs_util_pinfo(void)
       }
       if (plk_i->value()) argv[nxt++] = (char*)"-i";
       argv[nxt++] = (char *)plk_analin->value();
-      csoundPreCompile(csound);
       csoundRunUtility(csound, "pvlook", nxt, argv);
       csoundReset(csound);
     }
@@ -713,7 +706,6 @@ void cs_util_dnoise(void)
       argv[nxt++] = (char*)"-o"; argv[nxt++] = (char *)dns_analout->value();
       argv[nxt++] = (char*)"-i"; argv[nxt++] = (char *)dns_noise->value();
       argv[nxt++] = (char *)dns_analin->value();
-      csoundPreCompile(csound);
       csoundRunUtility(csound, "dnoise", nxt, argv);
       csoundReset(csound);
     }
@@ -738,4 +730,3 @@ void savetext(Fl_Text_Buffer *b, int type)
   }
   delete hw;
 }
-
