@@ -394,6 +394,7 @@ extern "C" {
     int     nchnls_override;  /* overriding number of out channels */
     int     nchnls_i_override;  /* overriding number of in channels */
     MYFLT   e0dbfs_override;   /* overriding 0dbfs */
+    int     daemon;  /* daemon mode */
   } CSOUND_PARAMS;
 
     /**
@@ -2176,7 +2177,7 @@ extern "C" {
   * void *out - preallocated buffer with at least items number of elements, where
   *              buffer contents will be read into
   * int items - number of samples to be read
-  * returns the actual number of samples read (0 <= n <= items)
+  * returns the actual number of items read (0 <= n <= items)
   */
   PUBLIC int csoundReadCircularBuffer(CSOUND *csound, void *circular_buffer,
                                       void *out, int items);
@@ -2187,7 +2188,7 @@ extern "C" {
    * void *out - preallocated buffer with at least items number of elements, where
    *              buffer contents will be read into
    * int items - number of samples to be read
-   * returns the actual number of samples read (0 <= n <= items)
+   * returns the actual number of items read (0 <= n <= items)
    */
    PUBLIC int csoundPeekCircularBuffer(CSOUND *csound, void *circular_buffer,
                                        void *out, int items);
