@@ -47,7 +47,7 @@ extern "C" {
 #include <xlocale.h>
 #endif
 
-#if (defined(__MACH__) || defined(ANDROID))
+#if (defined(__MACH__) || defined(ANDROID) || defined(NACL))
 #define BARRIER_SERIAL_THREAD (-1)
 typedef struct {
   pthread_mutex_t mut;
@@ -93,7 +93,7 @@ typedef struct {
 #define CURTIME (((double)csound->icurTime)/((double)csound->esr))
 #define CURTIME_inc (((double)csound->ksmps)/((double)csound->esr))
 
-#ifdef USE_DOUBLE
+#ifdef  B64BIT
 #define MAXLEN     0x40000000
 #define FMAXLEN    ((MYFLT)(MAXLEN))
 #define PHMASK     0x3fffffff
@@ -213,6 +213,7 @@ typedef struct {
     int     sampleAccurate;  /* switch for score events sample accuracy */
     int     realtime; /* realtime priority mode  */
     MYFLT   e0dbfs_override;
+    int     daemon;
   } OPARMS;
 
   typedef struct arglst {
@@ -425,6 +426,8 @@ typedef struct {
     /** String argument(s) (NULL if none) */
     int     scnt;
     char    *strarg;
+    /* instance pointer */
+    void  *pinstance;
     /** Event type */
     char    opcod;
     /** Number of p-fields */
@@ -514,6 +517,7 @@ typedef struct {
     int    tieflag;
     int    reinitflag;
     MYFLT  retval;
+    MYFLT  *lclbas;  /* base for variable memory pool */ 
     char   *strarg;       /* string argument */
     /* Copy of required p-field values for quick access */
     MYFLT   p0;
@@ -621,7 +625,7 @@ typedef struct {
     /** GEN01 parameters */
     GEN01ARGS gen01args;
     /** table data (flen + 1 MYFLT values) */
-    MYFLT   ftable[1];
+    MYFLT   *ftable;
   } FUNC;
 
   typedef struct {
@@ -903,19 +907,24 @@ typedef struct NAME__ {
     MYFLT (*GetSr)(CSOUND *);
     MYFLT (*GetKr)(CSOUND *);
     uint32_t (*GetKsmps)(CSOUND *);
+     /** Get number of output channels */
     uint32_t (*GetNchnls)(CSOUND *);
+    /** Get number of input channels */
     uint32_t (*GetNchnls_i)(CSOUND *);
     MYFLT (*Get0dBFS) (CSOUND *);
+    /** Get number of control blocks elapsed */
     long (*GetKcounter)(CSOUND *);
     int64_t (*GetCurrentTimeSamples)(CSOUND *);
     long (*GetInputBufferSize)(CSOUND *);
     long (*GetOutputBufferSize)(CSOUND *);
     MYFLT *(*GetInputBuffer)(CSOUND *);
     MYFLT *(*GetOutputBuffer)(CSOUND *);
+    /** Set internal debug mode */
     void (*SetDebug)(CSOUND *, int d);
     int (*GetDebug)(CSOUND *);
     int (*GetSizeOfMYFLT)(void);
     void (*GetOParms)(CSOUND *, OPARMS *);
+    /** Get environment variable */
     const char *(*GetEnv)(CSOUND *, const char *name);
     /**@}*/
     /** @name Message printout */
@@ -945,10 +954,15 @@ typedef struct NAME__ {
     int (*GetZakBounds)(CSOUND *, MYFLT **);
     int (*GetTieFlag)(CSOUND *);
     int (*GetReinitFlag)(CSOUND *);
+    /** Current maximum number of strings, accessible through the strset 
+        and strget opcodes */
     int (*GetStrsmax)(CSOUND *);
     char *(*GetStrsets)(CSOUND *, long);
+    /* Fast power of two function from a precomputed table */
     MYFLT (*Pow2)(CSOUND *, MYFLT a);
+    /* Fast power function for positive integers */
     MYFLT (*intpow)(MYFLT, int32);
+    /* Returns a string name for the file type */
     char *(*type2string)(int type);
     /**@}*/
     /** @name Arguments to opcodes */
@@ -981,8 +995,12 @@ typedef struct NAME__ {
     int (*hfgens)(CSOUND *, FUNC **, const EVTBLK *, int);
     int (*FTAlloc)(CSOUND *, int tableNum, int len);
     int (*FTDelete)(CSOUND *, int tableNum);
+    /** Find tables with power of two size. If table exists but is 
+        not a power of 2, NULL is returned. */
     FUNC *(*FTFind)(CSOUND *, MYFLT *argp);
+    /** Find any table, except deferred load tables. */
     FUNC *(*FTFindP)(CSOUND *, MYFLT *argp);
+    /** Find any table. */
     FUNC *(*FTnp2Find)(CSOUND *, MYFLT *argp);
     int (*GetTable)(CSOUND *, MYFLT **tablePtr, int tableNum);
     int (*TableLength)(CSOUND *, int table);
@@ -1239,18 +1257,20 @@ typedef struct NAME__ {
     int (*sprintf)(char *str, const char *format, ...);
     int (*sscanf)(char *str, const char *format, ...);
       /**@}*/
-    /** @name Placeholders */
+    /** @name Placeholders
+        To allow the API to grow while maintining backward binary compatibility. */
     /**@{ */
     SUBR dummyfn_2[48];
     /**@}*/
-    /*  NO MORE PUBLIC VARIABLES IN CSOUND struct
-
-      NB: if a new variable member is needed by the library, add it below, as a
-      private data member.
-
-      If access is required solely by plugins (and not by internally by the
-      library), use the CreateGlobalVariable() etc. interface, instead of adding
-      to CSOUND.
+#ifdef __BUILDING_LIBCSOUND
+    /* ------- private data (not to be used by hosts or externals) ------- */
+    /** @name Private Data
+      Private Data in the CSOUND struct to be used internally by the Csound 
+      library and should be hidden from plugins.
+      If a new variable member is needed by the library, add it below, as a
+      private data member. If access is required solely by plugins (and not
+      internally by the library), use the CreateGlobalVariable() etc. interface,
+      instead of adding to CSOUND.
 
       If you find that a plugin needs to access existing private data,
       first check above for an existing interface; if none is available,
@@ -1259,17 +1279,19 @@ typedef struct NAME__ {
       below:
 
       1) To get the data member value:
+      \code
          returnType (*GetVar)(CSOUND *)
-
+      \endcode
       2) in case of pointers, data should be copied out to a supplied memory
          slot, rather than the pointer being obtained:
+      \code
          void (*GetData)(CSOUND *, dataType *)
 
          dataType var;
          csound->GetData(csound, &var);
+      \endcode
     */
-#ifdef __BUILDING_LIBCSOUND
-    /* ------- private data (not to be used by hosts or externals) ------- */
+    /**@{ */
     SUBR          first_callback_;
     channelCallback_t InputChannelCallback_;
     channelCallback_t OutputChannelCallback_;
@@ -1477,9 +1499,9 @@ typedef struct NAME__ {
     } sreadStatics;
     struct onefileStatics__ {
       NAMELST *toremove;
-      char    orcname[L_tmpnam + 4];
-      char    sconame[L_tmpnam + 4];
-      char    midname[L_tmpnam + 4];
+      char    *orcname;
+      char    *sconame;
+      char    *midname;
       int     midiSet;
       int     csdlinecount;
     } onefileStatics;
@@ -1636,7 +1658,8 @@ typedef struct NAME__ {
     int           jumpset;
     int           info_message_request;
     int           modules_loaded;
-    struct CSOUND_ **self;
+    /*struct CSOUND_ **self;*/
+    /**@}*/
 #endif  /* __BUILDING_LIBCSOUND */
   };
 
