@@ -30,7 +30,7 @@
 #endif
 
 #ifdef SWIG
-%module csnd
+%module csnd6
 %include "std_string.i"
 %include "std_vector.i"
 %apply std::vector<double> *INPUT { std::vector<double> const & };
@@ -56,7 +56,17 @@ class PUBLIC CppSound : public Csound, public CsoundFile
   size_t spoutSize;
   std::string renderedSoundfile;
 public:
-  CppSound();
+  CppSound() : Csound(),
+                       go(false),
+                       isCompiled(false),
+                       isPerforming(false),
+                       spoutSize(0)
+{
+
+  //SetHostData((CSOUND *)0);
+
+}
+
   virtual ~CppSound();
   virtual CSOUND *getCsound();
   virtual long getThis();
@@ -67,7 +77,7 @@ public:
   virtual std::string getOutputSoundfileName() const;
   virtual int perform(int argc, char **argv);
   virtual int perform();
-  virtual int performKsmps(bool absolute);
+  virtual int performKsmps();
   virtual void cleanup();
   virtual void inputMessage(const char *istatement);
   virtual void write(const char *text);
