@@ -136,19 +136,20 @@ CsoundOpcodeList::~CsoundOpcodeList()
 
 void CsoundChannelList::ResetVariables()
 {
-    lst = (CsoundChannelListEntry*) 0;
+    lst = (controlChannelInfo_t*) 0;
     cnt = -1;
     csound = (CSOUND*) 0;
 }
 
 int CsoundChannelList::GetChannelMetaData(int ndx,
-                                          MYFLT &dflt, MYFLT &min, MYFLT &max)
+                                          controlChannelHints_t *hints)
 {
     const char  *name;
     if (!lst || (unsigned int) ndx >= (unsigned int) cnt)
       return -1;
     name = lst[ndx].name;
-    return csoundGetControlChannelParams(csound, name, &dflt, &min, &max);
+    int ret = csoundGetControlChannelHints(csound, name, hints);
+    return ret;
 }
 
 /**
@@ -255,10 +256,13 @@ int CsoundChannelList::IsOutputChannel(int ndx)
 
 int CsoundChannelList::SubType(int ndx)
 {
-    MYFLT dflt, min, max;
     int   tmp;
-    tmp = this->GetChannelMetaData(ndx, dflt, min, max);
-    return (tmp >= 0 ? tmp : -1);
+    controlChannelHints_t hints;
+    tmp = this->GetChannelMetaData(ndx, &hints);
+    if (tmp >= 0) {
+        tmp = hints.behav;
+    }
+    return tmp;
 }
 
 /**
@@ -269,9 +273,9 @@ int CsoundChannelList::SubType(int ndx)
 
 double CsoundChannelList::DefaultValue(int ndx)
 {
-    MYFLT dflt, min, max;
-    if (this->GetChannelMetaData(ndx, dflt, min, max) > 0)
-      return dflt;
+    controlChannelHints_t hints;
+    if (this->GetChannelMetaData(ndx, &hints) > 0)
+      return hints.dflt;
     return 0.0;
 }
 
@@ -283,9 +287,9 @@ double CsoundChannelList::DefaultValue(int ndx)
 
 double CsoundChannelList::MinValue(int ndx)
 {
-    MYFLT dflt, min, max;
-    if (this->GetChannelMetaData(ndx, dflt, min, max) > 0)
-      return min;
+    controlChannelHints_t hints;
+    if (this->GetChannelMetaData(ndx, &hints) > 0)
+      return hints.min;
     return 0.0;
 }
 
@@ -297,9 +301,9 @@ double CsoundChannelList::MinValue(int ndx)
 
 double CsoundChannelList::MaxValue(int ndx)
 {
-    MYFLT dflt, min, max;
-    if (this->GetChannelMetaData(ndx, dflt, min, max) > 0)
-      return max;
+    controlChannelHints_t hints;
+    if (this->GetChannelMetaData(ndx, &hints) > 0)
+      return hints.max;
     return 0.0;
 }
 
@@ -319,7 +323,7 @@ void CsoundChannelList::Clear()
 
 CsoundChannelList::CsoundChannelList(CSOUND *csound)
 {
-    lst = (CsoundChannelListEntry*) 0;
+    lst = (controlChannelInfo_t*) 0;
     cnt = csoundListChannels(csound, &lst);
     this->csound = csound;
     if (cnt < 0 || !lst)
@@ -328,7 +332,7 @@ CsoundChannelList::CsoundChannelList(CSOUND *csound)
 
 CsoundChannelList::CsoundChannelList(Csound *csound)
 {
-    lst = (CsoundChannelListEntry*) 0;
+    lst = (controlChannelInfo_t*) 0;
     cnt = csound->ListChannels(lst);
     this->csound = csound->GetCsound();
     if (cnt < 0 || !lst)
@@ -692,42 +696,6 @@ extern "C" {
     return p->YieldCallback();
   }
 
-  static CS_NOINLINE void ChannelIOCallback_wrapper(CSOUND *csound, const char *chnName,
-                                        MYFLT *value, int chnType)
-  {
-    CsoundCallbackWrapper *p;
-    p = (CsoundCallbackWrapper*) csoundGetHostData(csound);
-    switch (chnType & (CSOUND_CHANNEL_TYPE_MASK
-                       | CSOUND_INPUT_CHANNEL | CSOUND_OUTPUT_CHANNEL)) {
-    case CSOUND_CONTROL_CHANNEL | CSOUND_INPUT_CHANNEL:
-      *value = (MYFLT) p->ControlChannelInputCallback(chnName);
-      break;
-    case CSOUND_CONTROL_CHANNEL | CSOUND_OUTPUT_CHANNEL:
-      p->ControlChannelOutputCallback(chnName, (double) *value);
-      break;
-#if 0
-    case CSOUND_AUDIO_CHANNEL | CSOUND_INPUT_CHANNEL:
-      p->AudioChannelInputCallback(chnName, value);
-      break;
-    case CSOUND_AUDIO_CHANNEL | CSOUND_OUTPUT_CHANNEL:
-      p->AudioChannelOutputCallback(chnName, value);
-      break;
-#endif
-    case CSOUND_STRING_CHANNEL | CSOUND_INPUT_CHANNEL:
-      {
-        const char  *s = p->StringChannelInputCallback(chnName);
-        int         i, maxLen = csoundGetStrVarMaxLen(csound) - 1;
-        for (i = 0; s[i] != (char) 0 && i < maxLen; i++)
-          ((char*) value)[i] = s[i];
-        ((char*) value)[i] = (char) 0;
-      }
-      break;
-    case CSOUND_STRING_CHANNEL | CSOUND_OUTPUT_CHANNEL:
-      p->StringChannelOutputCallback(chnName, (char*) value);
-      break;
-    }
-  }
-
 }       // extern "C"
 
 void CsoundCallbackWrapper::SetMessageCallback()
@@ -735,7 +703,7 @@ void CsoundCallbackWrapper::SetMessageCallback()
     csoundSetMessageCallback(csound_, MessageCallback_wrapper);
 }
 
-void CsoundCallbackWrapper::SetInputValueCallback()
+  /*void CsoundCallbackWrapper::SetInputValueCallback()
 {
     csoundSetInputValueCallback(csound_, InputValueCallback_wrapper);
 }
@@ -744,7 +712,7 @@ void CsoundCallbackWrapper::SetOutputValueCallback()
 {
     csoundSetOutputValueCallback(csound_, OutputValueCallback_wrapper);
 }
-
+  */
 void CsoundCallbackWrapper::SetYieldCallback()
 {
     csoundSetYieldCallback(csound_, YieldCallback_wrapper);
@@ -845,14 +813,14 @@ int CsoundCallbackWrapper::midiOutCloseCallback(CSOUND *csound, void *userData)
     (void) userData;
     return 0;
 }
-
+  /*
 void CsoundCallbackWrapper::SetChannelIOCallbacks()
 {
     csoundSetChannelIOCallback(csound_,
                                (CsoundChannelIOCallback_t)
                                    ChannelIOCallback_wrapper);
 }
-
+  */
 // ----------------------------------------------------------------------------
 
 static const unsigned char midiMessageByteCnt[32] = {
@@ -1096,9 +1064,7 @@ int CsoundMidiInputStream::midiInCloseCallback(CSOUND *csound, void *userData)
 
 void CsoundMidiInputStream::EnableMidiInput(CsoundArgVList *argv)
 {
-    int     err;
-
-    err = csoundCreateGlobalVariable(csound, "__csnd_MidiInObject",
+    csoundCreateGlobalVariable(csound, "__csnd_MidiInObject",
                                              sizeof(void*));
     *((void**) csoundQueryGlobalVariable(csound, "__csnd_MidiInObject")) =
         (void*) this;
@@ -1358,9 +1324,7 @@ int CsoundMidiOutputStream::midiOutCloseCallback(CSOUND *csound,
 
 void CsoundMidiOutputStream::EnableMidiOutput(CsoundArgVList *argv)
 {
-    int     err;
-
-    err = csoundCreateGlobalVariable(csound, "__csnd_MidiOutObject",
+    csoundCreateGlobalVariable(csound, "__csnd_MidiOutObject",
                                              sizeof(void*));
     *((void**) csoundQueryGlobalVariable(csound, "__csnd_MidiOutObject")) =
         (void*) this;
