@@ -87,55 +87,73 @@ extern "C" {
     files[csound].score.push_back(line);
   }
 
-  PUBLIC void csoundCsdAddEvent11(CSOUND *csound, double p1, double p2, double p3, double p4, double p5, double p6, double p7, double p8, double p9, double p10, double p11)
+  PUBLIC void csoundCsdAddEvent11(CSOUND *csound, double p1, double p2, double p3,
+                                  double p4, double p5, double p6, double p7,
+                                  double p8, double p9, double p10, double p11)
   {
     char note[0x100];
-    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11);
+    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g "
+            "%-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11);
     files[csound].score.push_back(note);
   }
 
-  PUBLIC void csoundCsdAddEvent10(CSOUND *csound, double p1, double p2, double p3, double p4, double p5, double p6, double p7, double p8, double p9, double p10)
+  PUBLIC void csoundCsdAddEvent10(CSOUND *csound, double p1, double p2, double p3,
+                                  double p4, double p5, double p6, double p7,
+                                  double p8, double p9, double p10)
   {
     char note[0x100];
-    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8, p9, p10);
+    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g "
+            "%-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8, p9, p10);
     files[csound].score.push_back(note);
   }
 
-  PUBLIC void csoundCsdAddEvent9(CSOUND *csound, double p1, double p2, double p3, double p4, double p5, double p6, double p7, double p8, double p9)
+  PUBLIC void csoundCsdAddEvent9(CSOUND *csound, double p1, double p2, double p3,
+                                 double p4, double p5, double p6, double p7,
+                                 double p8, double p9)
   {
     char note[0x100];
-    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8, p9);
+    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g "
+            "%-.10g", p1, p2, p3, p4, p5, p6, p7, p8, p9);
     files[csound].score.push_back(note);
   }
 
-  PUBLIC void csoundCsdAddEvent8(CSOUND *csound, double p1, double p2, double p3, double p4, double p5, double p6, double p7, double p8)
+  PUBLIC void csoundCsdAddEvent8(CSOUND *csound, double p1, double p2, double p3,
+                                 double p4, double p5, double p6, double p7,
+                                 double p8)
   {
     char note[0x100];
-    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7, p8);
+    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g",
+            p1, p2, p3, p4, p5, p6, p7, p8);
     files[csound].score.push_back(note);
   }
-  PUBLIC void csoundCsdAddEvent7(CSOUND *csound, double p1, double p2, double p3, double p4, double p5, double p6, double p7)
+  PUBLIC void csoundCsdAddEvent7(CSOUND *csound, double p1, double p2, double p3,
+                                 double p4, double p5, double p6, double p7)
   {
     char note[0x100];
-    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6, p7);
+    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g",
+            p1, p2, p3, p4, p5, p6, p7);
     files[csound].score.push_back(note);
   }
 
-  PUBLIC void csoundCsdAddEvent6(CSOUND *csound, double p1, double p2, double p3, double p4, double p5, double p6)
+  PUBLIC void csoundCsdAddEvent6(CSOUND *csound, double p1, double p2, double p3,
+                                 double p4, double p5, double p6)
   {
     char note[0x100];
-    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5, p6);
+    sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g %-.10g",
+            p1, p2, p3, p4, p5, p6);
     files[csound].score.push_back(note);
   }
 
-  PUBLIC void csoundCsdAddEvent5(CSOUND *csound, double p1, double p2, double p3, double p4, double p5)
+  PUBLIC void csoundCsdAddEvent5(CSOUND *csound, double p1, double p2, double p3,
+                                 double p4, double p5)
   {
     char note[0x100];
     sprintf(note, "i %-.10g %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4, p5);
     files[csound].score.push_back(note);
   }
 
-  PUBLIC void csoundCsdAddEvent4(CSOUND *csound, double p1, double p2, double p3, double p4)
+  PUBLIC void csoundCsdAddEvent4(CSOUND *csound, double p1, double p2, double p3,
+                                 double p4)
   {
     char note[0x100];
     sprintf(note, "i %-.10g %-.10g %-.10g %-.10g", p1, p2, p3, p4);
@@ -161,7 +179,8 @@ extern "C" {
     fprintf(file, "%s", csoundFile.orchestra.c_str());
     fprintf(file, "</CsInstruments>");
     fprintf(file, "<CsScore>");
-    for (std::vector<std::string>::iterator it = csoundFile.score.begin(); it != csoundFile.score.end(); ++it) {
+    for (std::vector<std::string>::iterator it = csoundFile.score.begin();
+         it != csoundFile.score.end(); ++it) {
       fprintf(file, it->c_str());
     }
     fprintf(file, "</CsScore>");
