@@ -5,7 +5,7 @@
 #include <string.h>
 #include <dlfcn.h>
 #include <stdarg.h>
-#include "H/sysdep.h"
+#include "include/sysdep.h"
 
 typedef struct CSOUND_  CSOUND;
 #ifndef MYFLT
@@ -72,8 +72,8 @@ struct CSOUND_ {
     MYFLT (*GetKr)(CSOUND *);
     int (*GetKsmps)(CSOUND *);
     int (*GetNchnls)(CSOUND *);
-    int (*GetSampleFormat)(CSOUND *);
-    int (*GetSampleSize)(CSOUND *);
+//    int (*GetSampleFormat)(CSOUND *);
+//    int (*GetSampleSize)(CSOUND *);
     long (*GetInputBufferSize)(CSOUND *);
     long (*GetOutputBufferSize)(CSOUND *);
     MYFLT *(*GetInputBuffer)(CSOUND *);
@@ -81,9 +81,6 @@ struct CSOUND_ {
     MYFLT *(*GetSpin)(CSOUND *);
     MYFLT *(*GetSpout)(CSOUND *);
     double (*GetScoreTime)(CSOUND *);
-    void (*SetMakeXYinCallback)(CSOUND *);
-    void (*SetReadXYinCallback)(CSOUND *);
-    void (*SetKillXYinCallback)(CSOUND * );
     int (*IsScorePending)(CSOUND *);
     void (*SetScorePending)(CSOUND *, int pending);
     MYFLT (*GetScoreOffsetSeconds)(CSOUND *);
@@ -93,7 +90,7 @@ struct CSOUND_ {
     void (*MessageS)(CSOUND *, int attr, const char *fmt, ...);
     void (*MessageV)(CSOUND *, int attr, const char *format, va_list args);
     void (*DeleteUtilityList)(CSOUND *, char **lst);
-  void (*DeleteChannelList)(CSOUND * /* , CsoundChannelListEntry *lst*/);
+    void (*DeleteChannelList)(CSOUND * /* , controlChannelInfo_t *lst*/);
     void (*SetMessageCallback)(CSOUND *,
                 void (*csoundMessageCallback)(CSOUND *,
                                               int attr, const char *format,
@@ -170,7 +167,7 @@ struct CSOUND_ {
   void (*display)(CSOUND * /* , WINDAT * */);
     int (*dispexit)(CSOUND *);
     MYFLT (*intpow)(MYFLT, int32);
-    void *(*ldmemfile)(CSOUND *, const char *);  /* use ldmemfile2 instead */
+  //void *(*ldmemfile)(CSOUND *, const char *);  /* use ldmemfile2 instead */
     int32 (*strarg2insno)(CSOUND *, void *p, int is_string);
     char *(*strarg2name)(CSOUND *, char *, void *, const char *, int);
   int (*hfgens)(CSOUND * /* , FUNC **, const EVTBLK *, int */);
@@ -305,10 +302,10 @@ struct CSOUND_ {
     void (*ErrMsgV)(CSOUND *, const char *hdr, const char *fmt, va_list);
     int (*GetChannelPtr)(CSOUND *, MYFLT **p, const char *name, int type);
     int (*ListChannels)(CSOUND *, void **lst);
-    int (*SetControlChannelParams)(CSOUND *, const char *name,
-                                   int type, MYFLT dflt, MYFLT min, MYFLT max);
-    int (*GetControlChannelParams)(CSOUND *, const char *name,
-                                   MYFLT *dflt, MYFLT *min, MYFLT *max);
+    int (*SetControlChannelHints)(CSOUND *, const char *name,
+                                  const controlChannelHints_t hints);
+    int (*GetControlChannelHints)(CSOUND *, const char *name,
+                                   controlChannelHints_t *hints);
     int (*ChanIKSet)(CSOUND *, MYFLT value, int n);
     int (*ChanOKGet)(CSOUND *, MYFLT *value, int n);
     int (*ChanIASet)(CSOUND *, const MYFLT *value, int n);
@@ -326,8 +323,8 @@ struct CSOUND_ {
                                   void *userData, unsigned int typeMask);
     void (*Remove_Callback)(CSOUND *,
                             int (*func)(void *, void *, unsigned int));
-    int (*PvsinSet)(CSOUND *, void *value, int n);
-    int (*PvsoutGet)(CSOUND *, void *value, int n);
+    int (*PvsinSet)(CSOUND *, void *value, const char *name);
+    int (*PvsoutGet)(CSOUND *, void *value, const char *name);
     void (*SetInternalYieldCallback)(CSOUND *,
                        int (*yieldCallback)(CSOUND *));
     void *(*CreateBarrier)(unsigned int max);
@@ -336,7 +333,7 @@ struct CSOUND_ {
     void *(*FileOpen2)(CSOUND *, void *, int, const char *, void *,
                       const char *, int, int);
     int (*type2csfiletype)(int type, int encoding);
-    void *(*ldmemfile2)(CSOUND *, const char *, int);
+  //    void *(*ldmemfile2)(CSOUND *, const char *, int);
     void (*NotifyFileOpened)(CSOUND*, const char*, int, int, int);
     int (*sftype2csfiletype)(int type);
     int (*insert_score_event_at_sample)(CSOUND *, void *, long);
