@@ -89,6 +89,7 @@ typedef struct {
   MYFLT  *auxp;                 /* pointer to input file */
   MYFLT  *adp;                  /* pointer to front of sample file */
   double *c_p,*s_p;             /* pointers to space for sine and cos terms */
+  int    newformat;             /* flag for m/c independent format */
 } HET;
 
 #if INCSDIF
@@ -129,6 +130,7 @@ static void init_het(HET *thishet)
     thishet->amp_min   = 64;            /* amplitude cutout threshold */
     thishet->bufsiz    = 1;             /* circular buffer size */
     thishet->skip      = 0;             /* JPff: this was missing */
+    thishet->newformat = 1;
 }
 
 static int hetro(CSOUND *csound, int argc, char **argv)
@@ -155,9 +157,9 @@ static int hetro(CSOUND *csound, int argc, char **argv)
         case 's':
           FIND(Str("no sampling rate"))
 #if defined(USE_DOUBLE)
-          sscanf(s,"%lf",&thishet->sr);
+          csound->sscanf(s,"%lf",&thishet->sr);
 #else
-          sscanf(s,"%f",&thishet->sr);
+          csound->sscanf(s,"%f",&thishet->sr);
 #endif
           break;
         case 'c':
@@ -167,25 +169,25 @@ static int hetro(CSOUND *csound, int argc, char **argv)
         case 'b':
           FIND(Str("no begin time"))
 #if defined(USE_DOUBLE)
-          sscanf(s,"%lf",&thishet->beg_time);
+          csound->sscanf(s,"%lf",&thishet->beg_time);
 #else
-          sscanf(s,"%f",&thishet->beg_time);
+          csound->sscanf(s,"%f",&thishet->beg_time);
 #endif
           break;
         case 'd':
           FIND(Str("no duration time"))
 #if defined(USE_DOUBLE)
-          sscanf(s,"%lf",&thishet->input_dur);
+          csound->sscanf(s,"%lf",&thishet->input_dur);
 #else
-          sscanf(s,"%f",&thishet->input_dur);
+          csound->sscanf(s,"%f",&thishet->input_dur);
 #endif
           break;
         case 'f':
           FIND(Str("no fundamental estimate"))
 #if defined(USE_DOUBLE)
-          sscanf(s,"%lf",&thishet->fund_est);
+          csound->sscanf(s,"%lf",&thishet->fund_est);
 #else
-          sscanf(s,"%f",&thishet->fund_est);
+          csound->sscanf(s,"%f",&thishet->fund_est);
 #endif
           break;
         case 'h':
@@ -202,7 +204,7 @@ static int hetro(CSOUND *csound, int argc, char **argv)
           break;
         case 'M':
           FIND(Str("no amplitude maximum"))
-          sscanf(s,"%lf",&thishet->m_ampsum);
+          csound->sscanf(s,"%lf",&thishet->m_ampsum);
           break;
         case 'm':
           FIND(Str("no amplitude minimum"))
@@ -215,11 +217,17 @@ static int hetro(CSOUND *csound, int argc, char **argv)
         case 'l':
           FIND(Str("no filter cutoff"))
 #if defined(USE_DOUBLE)
-          sscanf(s,"%lf",&thishet->freq_c);
+          csound->sscanf(s,"%lf",&thishet->freq_c);
 #else
-          sscanf(s,"%f",&thishet->freq_c);
+          csound->sscanf(s,"%f",&thishet->freq_c);
 #endif
           break;
+        case 'X':
+          het.newformat = 1;
+          break;
+        case 'x':
+          het.newformat = 0;
+          break;
         case '-':
           FIND(Str("no log file"));
           while (*s++); s--;
@@ -576,13 +584,13 @@ static int quit(CSOUND *csound, char *msg)
 
 static int filedump(HET *thishet, CSOUND *csound)
 {
-    int     h, pnt, ofd, nbytes;
-    double  scale,x,y;
+    int     h, pnt, ofd, nbytes;    double  scale,x,y;
     int16   **mags, **freqs, *magout, *frqout;
     double  ampsum, maxampsum = 0.0;
     int32   lenfil = 0;
     int16   *TIME;
     MYFLT   timesiz;
+    FILE    *ff;
 
     mags = (int16 **) csound->Malloc(csound, thishet->hmax * sizeof(int16*));
     freqs = (int16 **) csound->Malloc(csound, thishet->hmax * sizeof(int16*));
@@ -599,13 +607,21 @@ static int filedump(HET *thishet, CSOUND *csound)
       TIME[pnt] = (int16)(pnt * timesiz);
 
     /* fullpath else cur dir */
-    if (csound->FileOpen2(csound, &ofd, CSFILE_FD_W, thishet->outfilnam,
-                                 NULL, "", CSFTYPE_HETRO, 0) == NULL)
+    if (thishet->newformat) {
+      if (csound->FileOpen2(csound, &ff, CSFILE_STD, thishet->outfilnam,
+                              "w", "", CSFTYPE_HETROT, 0) == NULL)
       return quit(csound, Str("cannot create output file\n"));
+    } else
+      if (csound->FileOpen2(csound, &ofd, CSFILE_FD_W, thishet->outfilnam,
+                            NULL, "", CSFTYPE_HETRO, 0) == NULL)
+        return quit(csound, Str("cannot create output file\n"));
 
-    if (UNLIKELY(write(ofd, (char*)&thishet->hmax, sizeof(thishet->hmax))<0))
-      csound->Message(csound,Str("Write failure\n")); /* Write header */
-
+    if (thishet->newformat)
+      fprintf(ff,"HETRO %d\n", thishet->hmax);        /* Header */
+    else {
+      if (UNLIKELY(write(ofd, (char*)&thishet->hmax, sizeof(thishet->hmax))<0))
+        csound->Message(csound,Str("Write failure\n")); /* Write header */
+    }
     for (pnt=0; pnt < thishet->num_pts; pnt++) {
       ampsum = 0.0;
       for (h = 0; h < thishet->hmax; h++)
@@ -686,8 +702,15 @@ static int filedump(HET *thishet, CSOUND *csound)
       *fp++ = END;
       mpoints = ((mp - magout) / 2) - 1;
       nbytes = (mp - magout) * sizeof(int16);
-      if (UNLIKELY(write(ofd, (char *)magout, nbytes)<0))
-        csound->Message(csound, Str("Write failure\n"));
+      if (thishet->newformat) {
+        int i;
+        for (i=0; i<(mp - magout); i++)
+          fprintf(ff,"%hd%c", magout[i], i==(mp-magout-1)?'\n':',');
+      }
+      else {
+        if (UNLIKELY(write(ofd, (char *)magout, nbytes)<0))
+          csound->Message(csound, Str("Write failure\n"));
+      }
 #ifdef DEBUG
       {
         int i;
@@ -699,8 +722,16 @@ static int filedump(HET *thishet, CSOUND *csound)
       lenfil += nbytes;
       fpoints = ((fp - frqout) / 2) - 1;
       nbytes = (fp - frqout) * sizeof(int16);
-      if (UNLIKELY(write(ofd, (char *)frqout, nbytes)<0))
-        csound->Message(csound, Str("Write failure\n"));
+      if (thishet->newformat) {
+        int i;
+        for (i=0; i<fp - frqout; i++)
+          fprintf(ff,"%hd%c", frqout[i], i==(fp-frqout-1)?'\n':',');
+        fprintf(ff,"\n");
+      }
+      else {
+        if (UNLIKELY(write(ofd, (char *)frqout, nbytes)<0))
+          csound->Message(csound, Str("Write failure\n"));
+      }
 #ifdef DEBUG
       {
         int i;
@@ -762,7 +793,7 @@ static int writesdif(CSOUND *csound, HET *thishet)
     scale = thishet->m_ampsum / maxampsum;
     /* SDIF does not specify a range, 'cos it's too clever for that sort
      * of thing, but this seems consistent with existing examples! */
-    scale *= (double) csound->dbfs_to_float;
+    scale *= (double) (1.0/csound->Get0dBFS(csound));
 
     for (h = 0; h < thishet->hmax; h++) {
       for (pnt = 0; pnt < thishet->num_pts; pnt++) {
@@ -793,7 +824,7 @@ static int writesdif(CSOUND *csound, HET *thishet)
     /* no padding bytes */
     /*timesiz = 1000.0f * thishet->input_dur / thishet->num_pts;*/
     timesiz = thishet->input_dur / thishet->num_pts;
-    /* don't even need a TIME array */
+    /* do not even need a TIME array */
 
     /* main loop to write 1TRC frames */
     for (i=0; i < thishet->num_pts; i++) {
@@ -862,4 +893,3 @@ int hetro_init_(CSOUND *csound)
     }
     return retval;
 }
-
