@@ -346,7 +346,8 @@ static int lpanal(CSOUND *csound, int argc, char **argv)
     double  errn, rms1, rms2, filterCoef[MAXPOLES+1];
     MYFLT   *sigbuf, *sigbuf2;      /* changed from short */
     long    n;
-    int     nb, osiz, hsize;
+    unsigned int     osiz, nb;
+    int     hsize;
     LPHEADER    *lph;
     char    *lpbuf, *tp;
     MYFLT   pchlow, pchhigh;
@@ -364,6 +365,8 @@ static int lpanal(CSOUND *csound, int argc, char **argv)
     double  polyReal[MAXPOLES], polyImag[MAXPOLES];
 #endif
     LPANAL_GLOBALS *lpg;
+    int     new_format=0;
+    FILE    *oFd;
 
     lpc.debug   = 0;
     lpc.verbose = 0;
@@ -402,23 +405,23 @@ static int lpanal(CSOUND *csound, int argc, char **argv)
         switch (*s++) {
         case 's':       FIND(Str("no sampling rate"))
 #if defined(USE_DOUBLE)
-                        sscanf(s,"%lf",&sr); break;
+                        csound->sscanf(s,"%lf",&sr); break;
 #else
-                        sscanf(s,"%f",&sr); break;
+                        csound->sscanf(s,"%f",&sr); break;
 #endif
         case 'c':       FIND(Str("no channel"))
                         sscanf(s,"%d",&channel); break;
         case 'b':       FIND(Str("no begin time"))
 #if defined(USE_DOUBLE)
-                        sscanf(s,"%lf",&beg_time); break;
+                        csound->sscanf(s,"%lf",&beg_time); break;
 #else
-                        sscanf(s,"%f",&beg_time); break;
+                        csound->sscanf(s,"%f",&beg_time); break;
 #endif
         case 'd':       FIND(Str("no duration time"))
 #if defined(USE_DOUBLE)
-                        sscanf(s,"%lf",&input_dur); break;
+                        csound->sscanf(s,"%lf",&input_dur); break;
 #else
-                        sscanf(s,"%f",&input_dur); break;
+                        csound->sscanf(s,"%f",&input_dur); break;
 #endif
         case 'p':       FIND(Str("no poles"))
                         sscanf(s,"%d",&lpc.poleCount); break;
@@ -430,18 +433,18 @@ static int lpanal(CSOUND *csound, int argc, char **argv)
                         break;
         case 'P':       FIND(Str("no low frequency"))
 #if defined(USE_DOUBLE)
-                        sscanf(s,"%lf",&pchlow);
+                        csound->sscanf(s,"%lf",&pchlow);
 #else
-                        sscanf(s,"%f",&pchlow);
+                        csound->sscanf(s,"%f",&pchlow);
 #endif
                         if (pchlow == 0.0)
                           lpc.doPitch = 0; /* -P0 inhibits ptrack */
                         break;
         case 'Q':       FIND(Str("no high frequency"))
 #if defined(USE_DOUBLE)
-                        sscanf(s,"%lf",&pchhigh); break;
+                        csound->sscanf(s,"%lf",&pchhigh); break;
 #else
-                        sscanf(s,"%f",&pchhigh); break;
+                        csound->sscanf(s,"%f",&pchhigh); break;
 #endif
         case 'v':       FIND(Str("no verbose level"))
                         sscanf(s,"%d",&lpc.verbose);
@@ -454,6 +457,9 @@ static int lpanal(CSOUND *csound, int argc, char **argv)
         case 'a':
                         storePoles = TRUE;
                         break;
+        case 'X':
+                        new_format = 1;
+                        break;
         default:
           {
             char errmsg[256];
@@ -520,7 +526,12 @@ static int lpanal(CSOUND *csound, int argc, char **argv)
     }
 
     /* Try to open output file */
-    if (csound->FileOpen2(csound, &ofd, CSFILE_FD_W,
+    if (new_format) {
+      if (csound->FileOpen2(csound, &oFd, CSFILE_STD,
+                            outfilnam, "w", "", CSFTYPE_LPC, 0) == NULL)
+      quit(csound, Str("cannot create output file"));
+    }
+    else if (csound->FileOpen2(csound, &ofd, CSFILE_FD_W,
                           outfilnam, NULL, "", CSFTYPE_LPC, 0) == NULL)
       quit(csound, Str("cannot create output file"));
 
@@ -542,7 +553,12 @@ static int lpanal(CSOUND *csound, int argc, char **argv)
       lph->headersize = LPBUFSIZ;
 
     /* Write header to disk */
-    if ((nb = write(ofd,(char *)lph,(int)lph->headersize)) <
+    if (new_format) {
+      fprintf(oFd, "LPANAL\n%d %d %d %d\n%f %f %f\n",
+              lph->headersize, lph->lpmagic, lph->npoles, lph->nvals,
+              lph->framrate, lph->srate, lph->duration);
+    }
+    else if ((nb = write(ofd,(char *)lph,(int)lph->headersize)) <
         lph->headersize)
       quit(csound,Str("cannot write header"));
 
@@ -575,8 +591,8 @@ static int lpanal(CSOUND *csound, int argc, char **argv)
     /* Space for a array */
     lpc.a = (double (*)[MAXPOLES])
       csound->Malloc(csound, MAXPOLES * MAXPOLES * sizeof(double));
-    lpc.x = (double *)
-      csound->Malloc(csound, lpc.WINDIN * sizeof(double));  /* alloc a double array */
+    lpc.x = (double *) csound->Malloc(csound,   /* alloc a double array */
+                                      lpc.WINDIN * sizeof(double));
 #ifdef TRACE
     csound->FileOpen2(csound, &trace, CSFILE_STD, "lpanal.trace", "w", NULL,
                       CSFTYPE_OTHER_TEXT, 0);
@@ -609,7 +625,7 @@ static int lpanal(CSOUND *csound, int argc, char **argv)
   /*  for (fp1=coef+NDATA, dfp=cc+poleCount, n=poleCount; n--; ) */
   /*    *fp1++ = - (MYFLT) *--dfp; */  /* rev coefs & chng sgn */
 #if 0
-      sprintf(lpc.pwindow.caption, "pitch: %8.2f", coef[3]);
+      CS_SPRINTF(lpc.pwindow.caption, "pitch: %8.2f", coef[3]);
       display(csound, &lpc.pwindow);
 #endif
 
@@ -697,8 +713,14 @@ static int lpanal(CSOUND *csound, int argc, char **argv)
       }
 
       /* Write frame to disk */
-      if ((nb = write(ofd, (char *)coef, osiz)) != osiz)
-        quit(csound, Str("write error"));
+      if (new_format) {
+        unsigned int i, j;
+        for (i=0, j=0; i<osiz; i+=sizeof(MYFLT), j++)
+          fprintf(oFd, "%a\n", coef[j]);
+      }
+      else
+        if ((nb = write(ofd, (char *)coef, osiz)) != osiz)
+          quit(csound, Str("write error"));
       memcpy(sigbuf, sigbuf2, sizeof(MYFLT)*slice);
 
       /* Some unused stuff. I think from when all snd was in mem */
@@ -761,21 +783,23 @@ static void alpol(LPC *thislp, MYFLT *sig, double *errn,
     int i, j, k, limit;
 
    /* Transfer signal in x array */
-    for (xp=thislp->x; xp-thislp->x < thislp->WINDIN;++xp,++sig){ /*   &  copy sigin into */
+    for (xp=thislp->x; xp-thislp->x < thislp->WINDIN;++xp,++sig) {
       *xp = (double) *sig;
     }
 
    /* Build system to be solved */
-    for (i=0; i < thislp->poleCount;++i)  {
+    for (i=0; i < thislp->poleCount;++i) {
       sum = (double) 0.0;
       for (k=thislp->poleCount; k < thislp->WINDIN;++k)
         sum += thislp->x[k-(i+1)] * thislp->x[k];
       v[i] = -sum;
-      if (i != thislp->poleCount - 1)  {
+      if (i != thislp->poleCount - 1) {
         limit = thislp->poleCount - (i+1);
-        for (j=0; j < limit; j++)  {
-          sum += thislp->x[thislp->poleCount-(i+1)-(j+1)] * thislp->x[thislp->poleCount-(j+1)]
-            - thislp->x[thislp->WINDIN-(i+1)-(j+1)] * thislp->x[thislp->WINDIN-(j+1)];
+        for (j=0; j < limit; j++) {
+          sum += thislp->x[thislp->poleCount-(i+1)-(j+1)] *
+            thislp->x[thislp->poleCount-(j+1)]
+            - thislp->x[thislp->WINDIN-(i+1)-(j+1)] *
+            thislp->x[thislp->WINDIN-(j+1)];
           thislp->a[(i+1)+j][j] = thislp->a[j][(i+1)+j] = sum;
         }
       }
@@ -838,13 +862,14 @@ static void gauss(LPC* thislp,
           amax = npq;
         }
       }
-       if (amax < 1.0e-20) {
+      if (amax < 1.0e-20) {
    /* csound->Message(csound,Str("Row %d or %d have maximum of %g\n"),
                       i, thislp->poleCount, amax);
       csound->Die(csound, Str("gauss: ill-conditioned"));
    */
-         for (ii=i; ii < thislp->poleCount;++ii) a[ii][i] = 1.0e-20; /* VL: fix for very low values */
-       }
+        for (ii=i; ii < thislp->poleCount;++ii)
+          a[ii][i] = 1.0e-20; /* VL: fix for very low values */
+      }
       if (i != istar) {
         for (j=0; j < thislp->poleCount;++j)  {    /* switch rows */
           dum = a[istar][j];
@@ -872,8 +897,9 @@ static void gauss(LPC* thislp,
           a[j][k] = a[j][k] - pivot * a[i][k];
       }
     }                               /* return if last pivot is too small */
+ /* VL: fix for very low values */
     if (fabs(a[thislp->poleCount-1][thislp->poleCount-1]) < 1e-20) {
-      a[thislp->poleCount-1][thislp->poleCount-1] = 1.0e-20; /* VL: fix for very low values */
+      a[thislp->poleCount-1][thislp->poleCount-1] = 1.0e-20;
 
       /*
             csound->Message(csound,"Row %d or %d have maximum of %g\n",
