@@ -48,7 +48,7 @@ static int lpc_export(CSOUND *csound, int argc, char **argv)
     FILE *inf;
     FILE *outf;
     LPHEADER hdr;
-    int i, j;
+    unsigned int i, j;
     char *str;
     MYFLT *coef;
 
@@ -81,14 +81,14 @@ static int lpc_export(CSOUND *csound, int argc, char **argv)
     if (UNLIKELY(fread(&hdr, sizeof(char),
                        hdr.headersize-sizeof(LPHEADER)+4, inf)!=
                  hdr.headersize-sizeof(LPHEADER)+4))
-      csound->Message(csound, Str("Read faailure\n"));
-    for (i=0; i<(signed int)hdr.headersize-sizeof(LPHEADER)+4; i++)
+      csound->Message(csound, Str("Read failure\n"));
+    for (i=0; i<hdr.headersize-sizeof(LPHEADER)+4; i++)
       putc(str[i],outf);
     putc('\n', outf);
     coef = (MYFLT *)csound->Malloc(csound,(hdr.npoles+hdr.nvals)*sizeof(MYFLT));
-    for (i = 0; i<floor(hdr.framrate*hdr.duration); i++) {
+    for (i = 0; i<(unsigned int)floor(hdr.framrate*hdr.duration); i++) {
       if (UNLIKELY(fread(&coef[0], sizeof(MYFLT), hdr.npoles, inf)!=hdr.npoles))
-        csound->Message(csound, Str("Read faailure\n"));
+        csound->Message(csound, Str("Read failure\n"));
       for (j=0; j<hdr.npoles; j++)
         fprintf(outf, "%f%c", coef[j], (j==hdr.npoles-1 ? '\n' : ','));
     }
