@@ -47,7 +47,7 @@ int main(int argc, char **argv)
     FILE *inf;
     FILE *outf;
     LPHEADER hdr;
-    int i, j;
+    unsigned int i, j;
     char *str;
     MYFLT *coef;
 
