@@ -116,8 +116,8 @@ static  int     pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd,
                                         const char *fname,
                                         long srate, long chans, long fftsize,
                                         long overlap, long winsize,
-                                        pv_wtype wintype, /*int verbose,*/
-                                        double beta);
+                                        pv_wtype wintype,
+                                        double beta, int displays);
 static  long    generate_frame(CSOUND*, PVX *pvx, MYFLT *fbuf, float *outanal,
                                         long samps, int frametype);
 static  void    chan_split(CSOUND*, const MYFLT *inbuf, MYFLT **chbuf,
@@ -163,9 +163,9 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
     pv_wtype  WindowType = PVOC_HANN;
     char    err_msg[512];
     double  beta = 6.8;
-    int verbose = 0;
+    int displays = 0;
+
 
-    csound->oparms->displays = 0;
     if (!(--argc))
       return quit(csound, Str("insufficient arguments"));
       do {
@@ -174,9 +174,9 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
           switch (*s++) {
           case 's': FIND(Str("no sampling rate"));
 #if defined(USE_DOUBLE)
-            sscanf(s, "%lf", &sr);
+            csound->sscanf(s, "%lf", &sr);
 #else
-            sscanf(s, "%f", &sr);
+            csound->sscanf(s, "%f", &sr);
 #endif
             break;
           case 'c':  FIND(Str("no channel"));
@@ -184,16 +184,16 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
             break;
           case 'b':  FIND(Str("no begin time"));
 #if defined(USE_DOUBLE)
-            sscanf(s, "%lf", &beg_time);
+            csound->sscanf(s, "%lf", &beg_time);
 #else
-            sscanf(s, "%f", &beg_time);
+            csound->sscanf(s, "%f", &beg_time);
 #endif
             break;
           case 'd':  FIND(Str("no duration time"));
 #if defined(USE_DOUBLE)
-            sscanf(s, "%lf", &input_dur);
+            csound->sscanf(s, "%lf", &input_dur);
 #else
-            sscanf(s, "%f", &input_dur);
+            csound->sscanf(s, "%f", &input_dur);
 #endif
             break;
           case 'H':
@@ -204,7 +204,7 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
             break;
           case 'B':
             FIND(Str("no beta given"));
-            sscanf(s, "%lf", &beta);
+            csound->sscanf(s, "%lf", &beta);
             break;
           case 'n':  FIND(Str("no framesize"));
             sscanf(s, "%ld", &frameSize);
@@ -222,11 +222,11 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
           case 'h':  FIND(Str("no hopsize"));
             sscanf(s, "%ld", &frameIncr);
             break;
-          case 'g':  csound->oparms->displays = 1;
+          case 'g':  displays = 1;
             break;
           case 'G':  FIND(Str("no latch"));
             sscanf(s, "%d", &latch);
-            csound->oparms->displays = 1;
+            displays = 1;
             break;
           case 'V':  FIND(Str("no output file for trace"));
             {
@@ -236,9 +236,6 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
                 return quit(csound, Str("Failed to open text file"));
               csound->Message(csound, Str("Writing text form to file %s\n"), s);
             }
-          case 'v':
-            verbose = 1;
-            break;
           default:
             return quit(csound, Str("unrecognised switch option"));
           }
@@ -268,6 +265,7 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
         frameSize >>= 1;        /* divide down until just larger */
     }
     if (ovlp == 0 && frameIncr == 0) {
+      csound->Message(csound, "frameSize=%d\n", frameSize);
       ovlp = OVLP_DEF;          /* default overlap */
       frameIncr = frameSize / ovlp;
     }
@@ -276,9 +274,12 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
     else frameIncr = frameSize/ovlp;
 
     if (ovlp < 2 || ovlp > 64) {
-      csound->Message(csound, Str("WARNING: pvanal: %d might be a bad window overlap index\n"),
-                              (int) ovlp);
-      /* return -1; */ /* VL: removed this restriction, which sounds a bit drastic */
+      csound->Message(csound,
+                      Str("WARNING: pvanal: %d might be a bad window "
+                          "overlap index\n"),
+                      (int) ovlp);
+      /* return -1; */
+      /* VL: removed this restriction, which sounds a bit drastic */
     }
     oframeEst = (p->getframes - frameSize/2) / frameIncr;
     csound->Message(csound, Str("%ld infrsize, %ld infrInc\n"),
@@ -294,16 +295,16 @@ static int pvanal(CSOUND *csound, int argc, char **argv)
     }
     csound->Message(csound, Str("pvanal: creating pvocex file\n"));
     /* handle all messages in here, for now */
-    if (csound->oparms->displays)
-      csound->dispinit(csound);
+    if (displays)
+        csound->dispinit(csound);
     if (pvxanal(csound, p, infd, outfilnam, p->sr,
                         ((!channel || channel == ALLCHNLS) ? p->nchanls : 1),
                         frameSize, frameIncr, frameSize * 2,
-                WindowType, /*verbose,*/ beta) != 0) {
+                WindowType, beta, displays) != 0) {
       csound->Message(csound, Str("error generating pvocex file.\n"));
       return -1;
     }
-    if (csound->oparms->displays)
+    if (displays)
       csound->dispexit(csound);
 
     return 0;
@@ -361,8 +362,6 @@ static void PVDisplay_Init(CSOUND *csound, PVDISPLAY *p,
 
     memset(p, 0, sizeof(PVDISPLAY));
     p->csound = csound;
-    if (!csound->oparms->displays)
-      return;
     p->npts = (fftSize / 2) + 1;
     p->dispCntMax = dispCntMax;
     for (i = 0; i < DISPFRAMES; i++)
@@ -373,7 +372,7 @@ static void PVDisplay_Update(PVDISPLAY *p, const float *buf)
 {
     int     i;
 
-    if (!p->csound->oparms->displays || p->dispFrame >= DISPFRAMES)
+    if (p->dispFrame >= DISPFRAMES)
       return;
     for (i = 0; i < p->npts; i++)
       p->dispBufs[p->dispFrame][i] += ((MYFLT) buf[i * 2] * (MYFLT) buf[i * 2]);
@@ -384,7 +383,7 @@ static void PVDisplay_Display(PVDISPLAY *p, int frame)
 {
     int     i;
 
-    if (!p->csound->oparms->displays || p->dispFrame >= DISPFRAMES ||
+    if (p->dispFrame >= DISPFRAMES ||
         p->dispCnt < p->dispCntMax)
       return;
     for (i = 0; i < p->npts; i++)
@@ -401,12 +400,11 @@ static void PVDisplay_Display(PVDISPLAY *p, int frame)
 
 /* Only supports PVOC_AMP_FREQ format for now */
 
-/* not sure how to use 'verbose' yet; but it's here...*/
 /* cannot add display code, as we may have 8 channels here...*/
 
 static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
                    long srate, long chans, long fftsize, long overlap,
-                   long winsize, pv_wtype wintype, /*int verbose,*/ double beta)
+                   long winsize, pv_wtype wintype, double beta, int displays)
 {
     int         i, k, pvfile = -1, rc = 0;
     pv_stype    stype = STYPE_16;
@@ -469,7 +467,7 @@ static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
       rc = 1;
       goto error;
     }
-
+    if(displays)
     PVDisplay_Init(csound, &disp, (int) fftsize,
                    (int) (((long) p->getframes * chans / overlap)
                           / DISPFRAMES));
@@ -501,11 +499,11 @@ static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
             goto error;
           }
           blocks_written++;
-          PVDisplay_Update(&disp, frame);
+          if(displays) PVDisplay_Update(&disp, frame);
           if ((blocks_written/chans) % 20 == 0) {
             csound->Message(csound, "%ld\n", blocks_written/chans);
         }
-        PVDisplay_Display(&disp, (int) (blocks_written / chans));
+          if(displays) PVDisplay_Display(&disp, (int) (blocks_written / chans));
       }
       }
       if (total_sampsread >= p->getframes*chans)
@@ -533,9 +531,9 @@ static int pvxanal(CSOUND *csound, SOUNDIN *p, SNDFILE *fd, const char *fname,
           goto error;
         }
         blocks_written++;
-        PVDisplay_Update(&disp, frame);
+        if(displays) PVDisplay_Update(&disp, frame);
       }
-      PVDisplay_Display(&disp, (int) (blocks_written / chans));
+      if(displays) PVDisplay_Display(&disp, (int) (blocks_written / chans));
     }
     csound->Message(csound, Str("\n%ld %d-chan blocks written to %s\n"),
                     (long) blocks_written / (long) chans, (int) chans, fname);
@@ -829,7 +827,7 @@ static void chan_split(CSOUND *csound, const MYFLT *inbuf, MYFLT **chbuf,
     const MYFLT *p_inbuf = inbuf;
 
     len = insize/chans;
-    ampfac = csound->dbfs_to_float;
+    ampfac = (1.0/csound->Get0dBFS(csound));
 
     for (i=0;i < chans;i++)
       buf_c[i] = chbuf[i];
@@ -923,4 +921,3 @@ static void vonhann(MYFLT *win, int winLen, int even)
         win[i] = (MYFLT)(0.5 + 0.5 * cos(ftmp*(double)i));
     }
 }
-
