@@ -57,27 +57,26 @@ typedef struct {
 /* Static function prototypes */
 
 static SNDFILE*  EXsndgetset(CSOUND *, XTRC*,char *);
-static void ExtractSound(CSOUND *, XTRC *, SNDFILE*, SNDFILE*);
+static void ExtractSound(CSOUND *, XTRC *, SNDFILE*, SNDFILE*, OPARMS *);
 
 static void usage(CSOUND *csound, char *mesg, ...)
 {
     va_list args;
 
     csound->Message(csound,Str("Usage:\textractor [-flags] soundfile\n"));
-    csound->Message(csound, "Legal flags are:\n");
+    csound->Message(csound,Str("Legal flags are:\n"));
     csound->Message(csound,Str("-o fname\tsound output filename\n"));
     csound->Message(csound,Str("-N\t\tnotify (ring the bell) when done\n"));
-    csound->Message(csound,Str("-S integer\tsample number at which to start file\n"));
+    csound->Message(csound,Str("-S integer\tsample number at which"
+                               " to start file\n"));
     csound->Message(csound,Str("-Z integer\tsample number at which to end file\n"));
     csound->Message(csound,Str("-Q integer\tnumber of samples to read\n"));
 
     csound->Message(csound,Str("-T fpnum\ttime in secs at which to start file\n"));
     csound->Message(csound,Str("-E fpnum\ttime in secs at which to end file\n"));
     csound->Message(csound,Str("-D fpnum\tduration in secs of extract\n"));
-#ifndef OLPC
     csound->Message(csound,Str("-R\tRewrite header\n"));
     csound->Message(csound,Str("-H\t\tHeartbeat\n"));
-#endif
     csound->Message(csound,Str("-v\t\tverbose mode for debugging\n"));
     csound->Message(csound,Str("-- fname\tLog output to file\n"));
     csound->Message(csound,Str("flag defaults: extractor -otest -S 0\n"));
@@ -90,7 +89,7 @@ static void usage(CSOUND *csound, char *mesg, ...)
 
 static int xtrct(CSOUND *csound, int argc, char **argv)
 {
-    OPARMS      *O = csound->oparms;
+    OPARMS      O;
     char        *inputfile = NULL;
     SNDFILE*    infd;
     SNDFILE*    outfd;
@@ -98,9 +97,12 @@ static int xtrct(CSOUND *csound, int argc, char **argv)
     char        c, *s;
     SF_INFO     sfinfo;
     int         debug   = 0;
-    int         Omsg = csound->oparms->msglevel;
+    int         Omsg;
     XTRC        xtrc;
 
+    csound->GetOParms(csound, &O);
+    Omsg = O.msglevel;
+
     /* Check arguments */
     xtrc.sample = -1; xtrc.stime = -FL(1.0);
     xtrc.stop  = -1; xtrc.endtime = -FL(1.0);
@@ -116,15 +118,10 @@ static int xtrct(CSOUND *csound, int argc, char **argv)
           switch(c) {
           case 'o':
             FIND(Str("no outfilename"))
-            O->outfilename = s;         /* soundout name */
+            O.outfilename = s;         /* soundout name */
             for ( ; *s != '\0'; s++) ;
-            if (strcmp(O->outfilename, "stdin") == 0)
+            if (strcmp(O.outfilename, "stdin") == 0)
               csound->Die(csound, Str("-o cannot be stdin"));
-#ifdef macintosh
-            if (strcmp(O->outfilename, "stdout") == 0) {
-              csound->Die(csound, Str("stdout audio not supported"));
-            }
-#endif
             break;
           case 'S':
             FIND(Str("no start sample"));
@@ -226,16 +223,14 @@ static int xtrct(CSOUND *csound, int argc, char **argv)
               xtrc.stop = -1;
             }
             break;
-#ifndef OLPC
           case 'H':
-            O->heartbeat = 1;
+            O.heartbeat = 1;
             break;
           case 'R':
-            O->rewrt_hdr = 1;
+            O.rewrt_hdr = 1;
             break;
-#endif
           case 'N':
-            O->ringbell = 1;            /* notify on completion */
+            O.ringbell = 1;            /* notify on completion */
             break;
           case 'v':                     /* Verbose mode */
             debug = 1;
@@ -270,28 +265,30 @@ static int xtrct(CSOUND *csound, int argc, char **argv)
     else if (xtrc.numsamps < 0) xtrc.numsamps =xtrc. p->getframes - xtrc.sample;
 
     if (xtrc.sample<0) xtrc.sample = 0;
-    csound->Message(csound,Str("Extracting from sample %ld for %ld samples (%.5f secs)\n"),
+    csound->Message(csound,
+                    Str("Extracting from sample %ld for %ld samples (%.5f secs)\n"),
                     xtrc.sample, xtrc.numsamps, (MYFLT)xtrc.numsamps/xtrc.p->sr);
 
     xtrc.outputs = xtrc.p->nchanls;
 
-    O->outformat = xtrc.p->format; /* Copy from input file */
-    O->sfsampsize = csound->sfsampsize(FORMAT2SF(O->outformat));
-    O->filetyp = xtrc.p->filetyp; /* Copy from input file */
-    O->sfheader = 1;
-    if (O->outfilename == NULL)
-      O->outfilename = "test";
-    csound->esr = (MYFLT)xtrc.p->sr;
-    csound->nchnls = xtrc.outputs;
+    O.outformat = xtrc.p->format; /* Copy from input file */
+    O.sfsampsize = csound->sfsampsize(FORMAT2SF(O.outformat));
+    O.filetyp = xtrc.p->filetyp; /* Copy from input file */
+    O.sfheader = 1;
+    if (O.outfilename == NULL)
+      O.outfilename = "test";
+
+    csound->SetUtilSr(csound, (MYFLT)xtrc.p->sr);
+    csound->SetUtilNchnls(csound, xtrc.outputs);
     memset(&sfinfo, 0, sizeof(SF_INFO));
     sfinfo.frames = -1;
-    sfinfo.samplerate = (int) (csound->esr + FL(0.5));
-    sfinfo.channels = csound->nchnls;
-    sfinfo.format = TYPE2SF(O->filetyp) | FORMAT2SF(O->outformat);
+    sfinfo.samplerate = (int) ((MYFLT)xtrc.p->sr + FL(0.5));
+    sfinfo.channels = xtrc.outputs;
+    sfinfo.format = TYPE2SF(O.filetyp) | FORMAT2SF(O.outformat);
     /* open file for write */
     fd = NULL;
-    if (strcmp(O->outfilename, "stdout") == 0 ||
-        strcmp(O->outfilename, "-") == 0) {
+    if (strcmp(O.outfilename, "stdout") == 0 ||
+        strcmp(O.outfilename, "-") == 0) {
       outfd = sf_open_fd(1, SFM_WRITE, &sfinfo, 0);
       if (outfd != NULL) {
         fd = csound->CreateFileHandle(csound, &outfd, CSFILE_SND_W, "stdout");
@@ -303,13 +300,13 @@ static int xtrct(CSOUND *csound, int argc, char **argv)
     }
     else
       fd = csound->FileOpen2(csound, &outfd, CSFILE_SND_W,
-                       O->outfilename, &sfinfo, "SFDIR",
-                       csound->type2csfiletype(O->filetyp, O->outformat), 0);
+                       O.outfilename, &sfinfo, "SFDIR",
+                       csound->type2csfiletype(O.filetyp, O.outformat), 0);
     if (fd == NULL)
       csound->Die(csound, Str("Failed to open output file %s"),
-                          O->outfilename);
-    ExtractSound(csound, &xtrc, infd, outfd);
-    if (O->ringbell)
+                          O.outfilename);
+    ExtractSound(csound, &xtrc, infd, outfd, &O);
+    if (O.ringbell)
       csound->MessageS(csound, CSOUNDMSG_REALTIME, "%c", '\007');
     return 0;
 }
@@ -320,7 +317,7 @@ EXsndgetset(CSOUND *csound, XTRC *x, char *name)
     SNDFILE*    infd;
     MYFLT       dur;
 
-    csound->esr = FL(0.0);      /* set esr 0. with no orchestra   */
+    csound->SetUtilSr(csound,FL(0.0));      /* set esr 0. with no orchestra   */
     x->p = (SOUNDIN *) csound->Calloc(csound, sizeof(SOUNDIN));
     x->p->channel = ALLCHNLS;
     x->p->skiptime = FL(0.0);
@@ -335,7 +332,7 @@ EXsndgetset(CSOUND *csound, XTRC *x, char *name)
 }
 
 static void
-ExtractSound(CSOUND *csound, XTRC *x, SNDFILE* infd, SNDFILE* outfd)
+ExtractSound(CSOUND *csound, XTRC *x, SNDFILE* infd, SNDFILE* outfd, OPARMS *oparms)
 {
     double buffer[NUMBER_OF_SAMPLES];
     long  read_in;
@@ -352,11 +349,11 @@ ExtractSound(CSOUND *csound, XTRC *x, SNDFILE* infd, SNDFILE* outfd)
       sf_writef_double(outfd, buffer, read_in);
       block++;
       frames += read_in;
-      if (csound->oparms->rewrt_hdr) {
+      if (oparms->rewrt_hdr) {
         sf_command(outfd, SFC_UPDATE_HEADER_NOW, NULL, 0);
         sf_seek(outfd, 0L, SEEK_END); /* Place at end again */
       }
-      if (csound->oparms->heartbeat) {
+      if (oparms->heartbeat) {
         csound->MessageS(csound, CSOUNDMSG_REALTIME, "%c\b", "|/-\\"[block&3]);
       }
       if (read_in < num) break;
@@ -376,4 +373,3 @@ int xtrct_init_(CSOUND *csound)
     }
     return retval;
 }
-
